/* Generated by Cython 0.29.30 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include",
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/ms_peak_picker/_c",
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/brainpy/_c"
        ],
        "name": "ms_deisotope._c.feature_map.profile_transform",
        "sources": [
            "ms_deisotope/_c/feature_map/profile_transform.pyx"
        ]
    },
    "module_name": "ms_deisotope._c.feature_map.profile_transform"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_30"
#define CYTHON_HEX_VERSION 0x001D1EF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ms_deisotope___c__feature_map__profile_transform
#define __PYX_HAVE_API__ms_deisotope___c__feature_map__profile_transform
/* Early includes */
#include <math.h>
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include "pythread.h"
#include <stdlib.h>
#include "pystate.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "ms_deisotope/_c/feature_map/profile_transform.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
  "peak_set.pxd",
  "ms_deisotope/_c/feature_map/lcms_feature.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

/* MemviewSliceStruct.proto */
struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;
#define __Pyx_MemoryView_Len(m)  (m.shape[0])

/* Atomics.proto */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __pyx_atomic_int_type int
#if CYTHON_ATOMICS && __GNUC__ >= 4 && (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL >= 2)) &&\
                    !defined(__i386__)
    #define __pyx_atomic_incr_aligned(value, lock) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_aligned(value, lock) __sync_fetch_and_sub(value, 1)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER) && 0
    #include <Windows.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type LONG
    #define __pyx_atomic_incr_aligned(value, lock) InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#elif CYTHON_ATOMICS && (defined(__ICC) || defined(__INTEL_COMPILER)) && 0
    #define __pyx_atomic_incr_aligned(value, lock) _InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) _InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using Intel atomics"
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()


/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":689
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */

#line 689 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */

#line 690 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */

#line 691 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */

#line 692 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":696
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */

#line 696 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */

#line 697 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */

#line 698 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */

#line 699 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":703
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */

#line 703 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */

#line 704 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":713
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */

#line 713 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */

#line 714 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */

#line 715 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":717
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */

#line 717 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */

#line 718 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */

#line 719 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":721
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */

#line 721 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */

#line 722 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":724
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */

#line 724 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */

#line 725 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */

#line 726 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys;
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr;
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":728
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */

#line 728 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */

#line 729 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */

#line 730 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":732
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cdouble __pyx_t_5numpy_complex_t;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted;

/* "ms_peak_picker/_c/peak_set.pxd":42
 *     cdef FittedPeak _get_nearest_peak(self, double mz, double* errout)
 * 
 *     cdef FittedPeak _has_peak(self, double mz, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakSet clone(self)
 */

#line 42 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak {

#line 42 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 42 "ms_peak_picker/_c/peak_set.pxd"
  double tolerance;

#line 42 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":50
 *     cdef int _between_bounds(self, double m1, double m2, size_t* startp, size_t* endp)
 * 
 *     cpdef tuple all_peaks_for(self, double mz, double error_tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef FittedPeak getitem(self, size_t i)
 */

#line 50 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for {

#line 50 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 50 "ms_peak_picker/_c/peak_set.pxd"
  double error_tolerance;

#line 50 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":64
 * 
 * 
 * cdef struct index_cell:             # <<<<<<<<<<<<<<
 *     double center_value
 *     size_t start
 */

#line 64 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell {

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  double center_value;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  size_t start;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  size_t end;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":70
 * 
 * 
 * cdef struct index_list:             # <<<<<<<<<<<<<<
 *     index_cell* index
 *     size_t size
 */

#line 70 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list {

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell *index;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  size_t size;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  double low;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  double high;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":90
 *     cpdef _allocate_index(self, size_t interval_index_size)
 * 
 *     cpdef FittedPeak has_peak_hinted(self, double mz, double tolerance=*, size_t hint_start=*, size_t hint_end=*)             # <<<<<<<<<<<<<<
 * 
 * cdef size_t double_binary_search_ppm(double* array, double value, double tolerance, size_t n)
 */

#line 90 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted {

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  double tolerance;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  size_t hint_start;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  size_t hint_end;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
};

#line 90 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature__invalidate;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_split_sparse;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_clone;

/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":98
 *     cpdef insert_node(self, LCMSFeatureTreeNode node)
 *     cpdef insert(self, PeakBase peak, double time)
 *     cpdef _invalidate(self, bint reaverage=*)             # <<<<<<<<<<<<<<
 *     cpdef list split_sparse(self, double delta_rt=*)
 *     cpdef LCMSFeature clone(self, deep=*, cls=*)
 */

#line 98 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature__invalidate {

#line 98 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int __pyx_n;

#line 98 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int reaverage;

#line 98 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":99
 *     cpdef insert(self, PeakBase peak, double time)
 *     cpdef _invalidate(self, bint reaverage=*)
 *     cpdef list split_sparse(self, double delta_rt=*)             # <<<<<<<<<<<<<<
 *     cpdef LCMSFeature clone(self, deep=*, cls=*)
 *     cpdef double max_intensity(self)
 */

#line 99 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_split_sparse {

#line 99 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int __pyx_n;

#line 99 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double delta_rt;

#line 99 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":100
 *     cpdef _invalidate(self, bint reaverage=*)
 *     cpdef list split_sparse(self, double delta_rt=*)
 *     cpdef LCMSFeature clone(self, deep=*, cls=*)             # <<<<<<<<<<<<<<
 *     cpdef double max_intensity(self)
 * 
 */

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_clone {

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int __pyx_n;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *deep;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *cls;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate;

#line 100 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys;

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":132
 * 
 * 
 * cdef np.ndarray[double, ndim=1, mode='c'] gauss(np.ndarray[double, ndim=1, mode='c'] x,             # <<<<<<<<<<<<<<
 *                                                 double mean, double variance, double scale=2.0):
 *     return 1. / np.sqrt(2 * np.pi * variance) * np.exp(-(x - mean) ** 2 / (scale * variance))
 */

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss {

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_n;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double scale;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":137
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] gaussian_smooth(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] x,
 *         np.ndarray[double, ndim=1, mode='c'] y, double width=0.05):
 */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_n;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double width;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":163
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] smooth_leveled(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] xs,
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_n;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double level;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":272
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 * 
 *     cpdef object interpolate(self, xs, ys, int n=200):             # <<<<<<<<<<<<<<
 *         new_xs = np.linspace(xs.min(), xs.max(), n)
 *         new_ys = np.interp(new_xs, xs, ys)
 */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_n;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int n;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":318
 * 
 *     @cython.boundscheck(False)
 *     cpdef locate_valleys(self, double scale=0.3, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1, mode='c'] xs
 */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_n;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double scale;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double smooth;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int interpolate_past;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 2 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase {

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  PyObject_HEAD

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtab;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double mz;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double intensity;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double area;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
};

#line 2 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */

#line 12 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak {

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double signal_to_noise;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double full_width_at_half_max;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double left_width;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double right_width;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  long peak_count;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  long index;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
};

#line 12 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

#line 29 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet {

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject_HEAD

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtab;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *peaks;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int indexed;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
};

#line 29 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */

#line 83 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  double *mz_index;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list *interval_index;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
};

#line 83 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":4
 * 
 * 
 * cdef class LCMSFeatureTreeList(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list roots
 */

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList {

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject_HEAD

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *__pyx_vtab;

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *roots;

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *_node_id_hash;

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":23
 * 
 * 
 * cdef class LCMSFeatureTreeNodeBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double time
 */

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase {

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject_HEAD

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *__pyx_vtab;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double time;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *members;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *_most_abundant_member;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *node_id;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":44
 * 
 * 
 * cdef class LCMSFeatureTreeNode(LCMSFeatureTreeNodeBase):             # <<<<<<<<<<<<<<
 *     cdef:
 * 
 */

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode {

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase __pyx_base;

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _mz;

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":56
 * 
 * 
 * cdef class FeatureBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public LCMSFeatureTreeList nodes
 */

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase {

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject_HEAD

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *__pyx_vtab;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *nodes;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _mz;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _start_time;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _end_time;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":73
 * 
 * 
 * cdef class LCMSFeature(FeatureBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double _total_intensity
 */

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature {

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase __pyx_base;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _total_intensity;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double _last_mz;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *_times;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *_peaks;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *adducts;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *used_as_adduct;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *created_at;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *feature_id;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *_peak_averager;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":104
 * 
 * 
 * cdef class EmptyFeature(FeatureBase):             # <<<<<<<<<<<<<<
 *     @staticmethod
 *     cdef EmptyFeature _create(double mz)
 */

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature {

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase __pyx_base;

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":109
 * 
 * 
 * cdef class FeatureSetIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list features
 */

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator {

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject_HEAD

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *__pyx_vtab;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *features;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *real_features;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double start_time;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double end_time;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double last_time_seen;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t *index_list;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":137
 * 
 * 
 * cdef class RunningWeightedAverage(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double current_mean
 */

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage {

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject_HEAD

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *__pyx_vtab;

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double current_mean;

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t current_count;

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double total_weight;

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":154
 * 
 * 
 * cdef class RunningWeightedAverageNeutralMass(RunningWeightedAverage):             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass {

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage __pyx_base;

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":178
 * 
 * 
 * cdef class PeakBoundary(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double first_minimum
 */

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary {

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject_HEAD

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_vtab;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double first_minimum;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double maximum;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double second_minimum;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double first_minimum_index;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double maximum_index;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double second_minimum_index;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double total_distance;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":218
 * 
 * 
 * cdef class ValleyPoint(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double first_maximum
 */

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint {

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject_HEAD

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_vtab;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double first_maximum;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double minimum;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double second_maximum;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double first_maximum_index;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double minimum_index;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double second_maximum_index;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double total_distance;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":252
 * 
 * 
 * cdef class ProfileSplitter(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object profile
 */

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter {

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject_HEAD

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_vtab;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *profile;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *partition_sites;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *xs;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *ys;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject_HEAD

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_ps;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_v_splitter;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":404
 *             if vallies:
 *                 splitter = vallies[0]
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)             # <<<<<<<<<<<<<<
 *             else:
 *                 out.append(case)
 */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject_HEAD

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *__pyx_outer_scope;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_v_p;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_1;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_2)(PyObject *);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

#line 105 "View.MemoryView"
struct __pyx_array_obj {

#line 105 "View.MemoryView"
  PyObject_HEAD

#line 105 "View.MemoryView"
  struct __pyx_vtabstruct_array *__pyx_vtab;

#line 105 "View.MemoryView"
  char *data;

#line 105 "View.MemoryView"
  Py_ssize_t len;

#line 105 "View.MemoryView"
  char *format;

#line 105 "View.MemoryView"
  int ndim;

#line 105 "View.MemoryView"
  Py_ssize_t *_shape;

#line 105 "View.MemoryView"
  Py_ssize_t *_strides;

#line 105 "View.MemoryView"
  Py_ssize_t itemsize;

#line 105 "View.MemoryView"
  PyObject *mode;

#line 105 "View.MemoryView"
  PyObject *_format;

#line 105 "View.MemoryView"
  void (*callback_free_data)(void *);

#line 105 "View.MemoryView"
  int free_data;

#line 105 "View.MemoryView"
  int dtype_is_object;

#line 105 "View.MemoryView"
};

#line 105 "View.MemoryView"


/* "View.MemoryView":279
 * 
 * @cname('__pyx_MemviewEnum')
 * cdef class Enum(object):             # <<<<<<<<<<<<<<
 *     cdef object name
 *     def __init__(self, name):
 */

#line 279 "View.MemoryView"
struct __pyx_MemviewEnum_obj {

#line 279 "View.MemoryView"
  PyObject_HEAD

#line 279 "View.MemoryView"
  PyObject *name;

#line 279 "View.MemoryView"
};

#line 279 "View.MemoryView"


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

#line 330 "View.MemoryView"
struct __pyx_memoryview_obj {

#line 330 "View.MemoryView"
  PyObject_HEAD

#line 330 "View.MemoryView"
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;

#line 330 "View.MemoryView"
  PyObject *obj;

#line 330 "View.MemoryView"
  PyObject *_size;

#line 330 "View.MemoryView"
  PyObject *_array_interface;

#line 330 "View.MemoryView"
  PyThread_type_lock lock;

#line 330 "View.MemoryView"
  __pyx_atomic_int acquisition_count[2];

#line 330 "View.MemoryView"
  __pyx_atomic_int *acquisition_count_aligned_p;

#line 330 "View.MemoryView"
  Py_buffer view;

#line 330 "View.MemoryView"
  int flags;

#line 330 "View.MemoryView"
  int dtype_is_object;

#line 330 "View.MemoryView"
  __Pyx_TypeInfo *typeinfo;

#line 330 "View.MemoryView"
};

#line 330 "View.MemoryView"


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

#line 965 "View.MemoryView"
struct __pyx_memoryviewslice_obj {

#line 965 "View.MemoryView"
  struct __pyx_memoryview_obj __pyx_base;

#line 965 "View.MemoryView"
  __Pyx_memviewslice from_slice;

#line 965 "View.MemoryView"
  PyObject *from_object;

#line 965 "View.MemoryView"
  PyObject *(*to_object_func)(char *);

#line 965 "View.MemoryView"
  int (*to_dtype_func)(char *, PyObject *);

#line 965 "View.MemoryView"
};

#line 965 "View.MemoryView"


#line 965 "View.MemoryView"


/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 2 "ms_peak_picker/_c/peak_set.pxd"


#line 2 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase {

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);

#line 2 "ms_peak_picker/_c/peak_set.pxd"
};

#line 2 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase;

#line 2 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */

#line 12 "ms_peak_picker/_c/peak_set.pxd"


#line 12 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak {

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int __pyx_skip_dispatch);

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_create)(double, double, double, double, double, double, long, long, double);

#line 12 "ms_peak_picker/_c/peak_set.pxd"
};

#line 12 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak;

#line 12 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

#line 29 "ms_peak_picker/_c/peak_set.pxd"


#line 29 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet {

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_create)(PyObject *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*reindex)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  size_t (*_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_get_nearest_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak *__pyx_optional_args);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_between)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int (*_between_bounds)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double, size_t *, size_t *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*all_peaks_for)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for *__pyx_optional_args);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*getitem)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, size_t);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  size_t (*get_size)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
};

#line 29 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet;

#line 29 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */

#line 83 "ms_peak_picker/_c/peak_set.pxd"


#line 83 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*_allocate_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, size_t, int __pyx_skip_dispatch);

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*has_peak_hinted)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted *__pyx_optional_args);

#line 83 "ms_peak_picker/_c/peak_set.pxd"
};

#line 83 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;

#line 83 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":4
 * 
 * 
 * cdef class LCMSFeatureTreeList(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list roots
 */

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList {

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *(*_create)(PyObject *);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  void (*_invalidate)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *(*_make_node)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *, double, PyObject *, int __pyx_skip_dispatch);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*find_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *, double, int __pyx_skip_dispatch);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *(*_find_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *, double, size_t *);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *, size_t);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *);

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList;

#line 4 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":23
 * 
 * 
 * cdef class LCMSFeatureTreeNodeBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double time
 */

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase {

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*_total_intensity_members)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*max_intensity)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*total_intensity)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*_find_most_abundant_member)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_calculate_most_abundant_member)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_recalculate)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *, size_t);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t (*get_members_size)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *);

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase;

#line 23 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":44
 * 
 * 
 * cdef class LCMSFeatureTreeNode(LCMSFeatureTreeNodeBase):             # <<<<<<<<<<<<<<
 *     cdef:
 * 
 */

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode {

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase __pyx_base;

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, int __pyx_skip_dispatch);

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*_ne)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, int __pyx_skip_dispatch);

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*getpeak)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, size_t);

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_mz)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *);

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode;

#line 44 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":56
 * 
 * 
 * cdef class FeatureBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public LCMSFeatureTreeList nodes
 */

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase {

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_mz)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_neutral_mass)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_start_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_end_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*find_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *, double, int __pyx_skip_dispatch);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *(*_find_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *, double, size_t *);

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase;

#line 56 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":73
 * 
 * 
 * cdef class LCMSFeature(FeatureBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double _total_intensity
 */

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature {

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase __pyx_base;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, PyObject *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*_ne)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, PyObject *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*overlaps_in_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*spans_in_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, double, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, size_t);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_initialize_averager)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_update_from_averager)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_reaverage_and_update)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*insert_node)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*insert)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, double, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_invalidate)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature__invalidate *__pyx_optional_args);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*split_sparse)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_split_sparse *__pyx_optional_args);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_12lcms_feature_11LCMSFeature_clone *__pyx_optional_args);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*max_intensity)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch);

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature;

#line 73 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":104
 * 
 * 
 * cdef class EmptyFeature(FeatureBase):             # <<<<<<<<<<<<<<
 *     @staticmethod
 *     cdef EmptyFeature _create(double mz)
 */

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature {

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase __pyx_base;

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature *(*_create)(double);

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature;

#line 104 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":109
 * 
 * 
 * cdef class FeatureSetIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list features
 */

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator {

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *(*_create)(PyObject *);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *(*_create_with_threshold)(PyObject *, PyObject *, double);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*_initialize)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, PyObject *);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*init_indices)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_next_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  double (*get_current_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*has_more)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*get_peaks_for_time)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, double, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*get_next_value)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, int __pyx_skip_dispatch);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *, size_t);

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator;

#line 109 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":137
 * 
 * 
 * cdef class RunningWeightedAverage(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double current_mean
 */

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage {

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*reset)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, int __pyx_skip_dispatch);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*_initialize)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, int __pyx_skip_dispatch);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  PyObject *(*add)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *(*update)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, PyObject *, int __pyx_skip_dispatch);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  int (*feed_from_feature)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *, int __pyx_skip_dispatch);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *(*_create)(PyObject *);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  void (*_update)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *, PyObject *);

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage;

#line 137 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/lcms_feature.pxd":154
 * 
 * 
 * cdef class RunningWeightedAverageNeutralMass(RunningWeightedAverage):             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass {

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage __pyx_base;

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass *(*_create)(PyObject *);

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
};

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass;

#line 154 "ms_deisotope/_c/feature_map/lcms_feature.pxd"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":178
 * 
 * 
 * cdef class PeakBoundary(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double first_minimum
 */

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary {

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double (*compute_distance)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *);

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;

#line 178 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":218
 * 
 * 
 * cdef class ValleyPoint(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double first_maximum
 */

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint {

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double (*compute_distance)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *);

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;

#line 218 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "ms_deisotope/_c/feature_map/profile_transform.pyx":252
 * 
 * 
 * cdef class ProfileSplitter(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object profile
 */

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter {

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*_extreme_indices)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, PyObject *, int __pyx_skip_dispatch);

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*interpolate)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate *__pyx_optional_args);

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*locate_valleys)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys *__pyx_optional_args);

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
};

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;

#line 252 "ms_deisotope/_c/feature_map/profile_transform.pyx"


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

#line 105 "View.MemoryView"


#line 105 "View.MemoryView"
struct __pyx_vtabstruct_array {

#line 105 "View.MemoryView"
  PyObject *(*get_memview)(struct __pyx_array_obj *);

#line 105 "View.MemoryView"
};

#line 105 "View.MemoryView"
static struct __pyx_vtabstruct_array *__pyx_vtabptr_array;

#line 105 "View.MemoryView"


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

#line 330 "View.MemoryView"


#line 330 "View.MemoryView"
struct __pyx_vtabstruct_memoryview {

#line 330 "View.MemoryView"
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);

#line 330 "View.MemoryView"
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);

#line 330 "View.MemoryView"
};

#line 330 "View.MemoryView"
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;

#line 330 "View.MemoryView"


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

#line 965 "View.MemoryView"


#line 965 "View.MemoryView"
struct __pyx_vtabstruct__memoryviewslice {

#line 965 "View.MemoryView"
  struct __pyx_vtabstruct_memoryview __pyx_base;

#line 965 "View.MemoryView"
};

#line 965 "View.MemoryView"
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* BufferIndexError.proto */
static void __Pyx_RaiseBufferIndexError(int axis);

#define __Pyx_BufPtrStrided1d(type, buf, i0, s0) (type)((char*)buf + i0 * s0)
/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* BufferFallbackError.proto */
static void __Pyx_RaiseBufferFallbackError(void);

#define __Pyx_BufPtrCContig1d(type, buf, i0, s0) ((type)buf + i0)
/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_DivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_DivideCObj(op1, op2, floatval, inplace, zerodivision_check)\
    ((inplace ? __Pyx_PyNumber_InPlaceDivide(op1, op2) : __Pyx_PyNumber_Divide(op1, op2)))
    #endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* MemviewSliceInit.proto */
#define __Pyx_BUF_MAX_NDIMS %(BUF_MAX_NDIMS)d
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (memview->acquisition_count_aligned_p)
#define __pyx_get_slice_count(memview) (*__pyx_get_slice_count_pointer(memview))
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XDEC_MEMVIEW(slice, have_gil) __Pyx_XDEC_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *, int, int);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* DivInt[Py_ssize_t].proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *); /*proto*/
/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* DivInt[long].proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* FusedFunction.proto */
typedef struct {
    __pyx_CyFunctionObject func;
    PyObject *__signatures__;
    PyObject *type;
    PyObject *self;
} __pyx_FusedFunctionObject;
static PyObject *__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                                         PyObject *qualname, PyObject *closure,
                                         PyObject *module, PyObject *globals,
                                         PyObject *code);
static int __pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self);
static PyTypeObject *__pyx_FusedFunctionType = NULL;
static int __pyx_FusedFunction_init(void);
#define __Pyx_FusedFunction_USED

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

/* MemviewSliceIsContig.proto */
static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim);

/* OverlappingSlices.proto */
static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

/* Capsule.proto */
static CYTHON_INLINE PyObject *__pyx_capsule_create(void *p, const char *sig);

/* TypeInfoCompare.proto */
static int __pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b);

/* MemviewSliceValidateAndInit.proto */
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_float(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_double(PyObject *, int writable_flag);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* MemviewSliceCopyTemplate.proto */
static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int64(npy_int64 value);

/* BytesContains.proto */
static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character);

/* ImportNumPyArray.proto */
static PyObject *__pyx_numpy_ndarray = NULL;
static PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void);

/* CIntFromPy.proto */
static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_compute_distance(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto*/
static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_compute_distance(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter__extreme_indices(CYTHON_UNUSED struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_ys, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate(CYTHON_UNUSED struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys *__pyx_optional_args); /* proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self); /* proto*/
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/

/* Module declarations from 'libc.math' */

/* Module declarations from 'cython.view' */

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'ms_peak_picker._c.peak_set' */
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = 0;
static size_t *__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE = 0;
#define __pyx_v_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE (*__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE)

/* Module declarations from 'ms_deisotope._c.feature_map.lcms_feature' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass = 0;

/* Module declarations from 'ms_deisotope._c.feature_map.shape_fitter' */
static PyObject *(*__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_gaussian_shape)(PyArrayObject *, double, double, double, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_bigaussian_shape)(PyArrayObject *, double, double, double, double, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_skewed_gaussian_shape)(PyArrayObject *, double, double, double, double, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'ms_deisotope._c.feature_map.profile_transform' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr = 0;
static PyTypeObject *__pyx_array_type = 0;
static PyTypeObject *__pyx_MemviewEnum_type = 0;
static PyTypeObject *__pyx_memoryview_type = 0;
static PyTypeObject *__pyx_memoryviewslice_type = 0;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static int __pyx_memoryview_thread_locks_used;
static PyThread_type_lock __pyx_memoryview_thread_locks[8];
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(PyArrayObject *, double, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_locate_extrema(PyArrayObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_median(PyArrayObject *); /*proto*/
static size_t __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch(PyArrayObject *, double, int __pyx_skip_dispatch); /*proto*/
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss(PyArrayObject *, double, double, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss *__pyx_optional_args); /*proto*/
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth(PyArrayObject *, PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth *__pyx_optional_args); /*proto*/
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled(PyArrayObject *, PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled *__pyx_optional_args); /*proto*/
static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__get_total_distance(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__create_ValleyPoint(double, double, double, double, double, double); /*proto*/
static PyObject *__pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(PyArrayObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(PyArrayObject *, int __pyx_skip_dispatch); /*proto*/
static size_t __pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__Pyx_memviewslice, float); /*proto*/
static size_t __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__Pyx_memviewslice, double); /*proto*/
static double __pyx_fuse_0_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice, __Pyx_memviewslice, double, int __pyx_skip_dispatch); /*proto*/
static double __pyx_fuse_0_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice, __Pyx_memviewslice, double, int __pyx_skip_dispatch); /*proto*/
static double __pyx_fuse_1_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice, __Pyx_memviewslice, double, int __pyx_skip_dispatch); /*proto*/
static double __pyx_fuse_1_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice, __Pyx_memviewslice, double, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_PeakBoundary__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *, PyObject *); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ValleyPoint__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *, PyObject *); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ProfileSplitter__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, PyObject *); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/
static void *__pyx_align_pointer(void *, size_t); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static PyObject *assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int, int); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, char *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, char *); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *, PyObject *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_double = { "double", NULL, sizeof(double), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_float = { "float", NULL, sizeof(float), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t = { "int64_t", NULL, sizeof(__pyx_t_5numpy_int64_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_int64_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_int64_t), 0 };
#define __Pyx_MODULE_NAME "ms_deisotope._c.feature_map.profile_transform"
extern int __pyx_module_is_main_ms_deisotope___c__feature_map__profile_transform;
int __pyx_module_is_main_ms_deisotope___c__feature_map__profile_transform = 0;

/* Implementation of 'ms_deisotope._c.feature_map.profile_transform' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_sorted;
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_IndexError;
static const char __pyx_k_[] = "";
static const char __pyx_k_O[] = "O";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k__2[] = "()";
static const char __pyx_k__3[] = "|";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_pi[] = "pi";
static const char __pyx_k_ps[] = "ps";
static const char __pyx_k_xs[] = "xs";
static const char __pyx_k_ys[] = "ys";
static const char __pyx_k_exp[] = "exp";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_max[] = "max";
static const char __pyx_k_min[] = "min";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_obj[] = "obj";
static const char __pyx_k_out[] = "out";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_var[] = "var";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_case[] = "case";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_kind[] = "kind";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mean[] = "mean";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_ndim[] = "ndim";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_sqrt[] = "sqrt";
static const char __pyx_k_step[] = "step";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_ASCII[] = "ASCII";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_int64[] = "int64";
static const char __pyx_k_level[] = "level";
static const char __pyx_k_nodes[] = "nodes";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_scale[] = "scale";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_width[] = "width";
static const char __pyx_k_astype[] = "astype";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_interp[] = "interp";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_median[] = "median";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_new_xs[] = "new_xs";
static const char __pyx_k_new_ys[] = "new_ys";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_smooth[] = "smooth";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_unpack[] = "unpack";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_fortran[] = "fortran";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_maximum[] = "maximum";
static const char __pyx_k_memview[] = "memview";
static const char __pyx_k_minimum[] = "minimum";
static const char __pyx_k_profile[] = "profile";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_vallies[] = "vallies";
static const char __pyx_k_Ellipsis[] = "Ellipsis";
static const char __pyx_k_defaults[] = "defaults";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_linspace[] = "linspace";
static const char __pyx_k_n_levels[] = "n_levels";
static const char __pyx_k_profiles[] = "profiles";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_splitter[] = "splitter";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_as_arrays[] = "as_arrays";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_find_time[] = "find_time";
static const char __pyx_k_intensity[] = "intensity";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_signatures[] = "signatures";
static const char __pyx_k_zeros_like[] = "zeros_like";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_ValleyPoint[] = "ValleyPoint";
static const char __pyx_k_concatenate[] = "concatenate";
static const char __pyx_k_float_float[] = "float|float";
static const char __pyx_k_interpolate[] = "interpolate";
static const char __pyx_k_PeakBoundary[] = "PeakBoundary";
static const char __pyx_k_double_float[] = "double|float";
static const char __pyx_k_float_double[] = "float|double";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_sliding_mean[] = "sliding_mean";
static const char __pyx_k_split_valley[] = "split_valley";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_double_double[] = "double|double";
static const char __pyx_k_first_maximum[] = "first_maximum";
static const char __pyx_k_first_minimum[] = "first_minimum";
static const char __pyx_k_maximum_index[] = "maximum_index";
static const char __pyx_k_minimum_index[] = "minimum_index";
static const char __pyx_k_pyx_getbuffer[] = "__pyx_getbuffer";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_split_valleys[] = "split_valleys";
static const char __pyx_k_locate_valleys[] = "locate_valleys";
static const char __pyx_k_second_maximum[] = "second_maximum";
static const char __pyx_k_second_minimum[] = "second_minimum";
static const char __pyx_k_total_distance[] = "total_distance";
static const char __pyx_k_ProfileSplitter[] = "ProfileSplitter";
static const char __pyx_k_View_MemoryView[] = "View.MemoryView";
static const char __pyx_k_allocate_buffer[] = "allocate_buffer";
static const char __pyx_k_dtype_is_object[] = "dtype_is_object";
static const char __pyx_k_extreme_indices[] = "_extreme_indices";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_interpolate_past[] = "interpolate_past";
static const char __pyx_k_interpolate_point[] = "interpolate_point";
static const char __pyx_k_pyx_unpickle_Enum[] = "__pyx_unpickle_Enum";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_get_total_distance[] = "_get_total_distance";
static const char __pyx_k_strided_and_direct[] = "<strided and direct>";
static const char __pyx_k_first_maximum_index[] = "first_maximum_index";
static const char __pyx_k_first_minimum_index[] = "first_minimum_index";
static const char __pyx_k_second_maximum_index[] = "second_maximum_index";
static const char __pyx_k_second_minimum_index[] = "second_minimum_index";
static const char __pyx_k_strided_and_indirect[] = "<strided and indirect>";
static const char __pyx_k_contiguous_and_direct[] = "<contiguous and direct>";
static const char __pyx_k_MemoryView_of_r_object[] = "<MemoryView of %r object>";
static const char __pyx_k_pyx_fuse_0sliding_mean[] = "__pyx_fuse_0sliding_mean";
static const char __pyx_k_pyx_fuse_1sliding_mean[] = "__pyx_fuse_1sliding_mean";
static const char __pyx_k_MemoryView_of_r_at_0x_x[] = "<MemoryView of %r at 0x%x>";
static const char __pyx_k_contiguous_and_indirect[] = "<contiguous and indirect>";
static const char __pyx_k_Cannot_index_with_type_s[] = "Cannot index with type '%s'";
static const char __pyx_k_pyx_unpickle_ValleyPoint[] = "__pyx_unpickle_ValleyPoint";
static const char __pyx_k_Invalid_shape_in_axis_d_d[] = "Invalid shape in axis %d: %d.";
static const char __pyx_k_pyx_unpickle_PeakBoundary[] = "__pyx_unpickle_PeakBoundary";
static const char __pyx_k_No_matching_signature_found[] = "No matching signature found";
static const char __pyx_k_itemsize_0_for_cython_array[] = "itemsize <= 0 for cython.array";
static const char __pyx_k_pyx_unpickle_ProfileSplitter[] = "__pyx_unpickle_ProfileSplitter";
static const char __pyx_k_split_valleys_locals_genexpr[] = "split_valleys.<locals>.genexpr";
static const char __pyx_k_pyx_fuse_0_0interpolate_point[] = "__pyx_fuse_0_0interpolate_point";
static const char __pyx_k_pyx_fuse_0_1interpolate_point[] = "__pyx_fuse_0_1interpolate_point";
static const char __pyx_k_pyx_fuse_1_0interpolate_point[] = "__pyx_fuse_1_0interpolate_point";
static const char __pyx_k_pyx_fuse_1_1interpolate_point[] = "__pyx_fuse_1_1interpolate_point";
static const char __pyx_k_unable_to_allocate_array_data[] = "unable to allocate array data.";
static const char __pyx_k_strided_and_direct_or_indirect[] = "<strided and direct or indirect>";
static const char __pyx_k_ValleyPoint_0_2f_0_2f_0_2f_0_3e[] = "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_Buffer_view_does_not_expose_stri[] = "Buffer view does not expose strides";
static const char __pyx_k_Can_only_create_a_buffer_that_is[] = "Can only create a buffer that is contiguous in memory.";
static const char __pyx_k_Cannot_assign_to_read_only_memor[] = "Cannot assign to read-only memoryview";
static const char __pyx_k_Cannot_create_writable_memory_vi[] = "Cannot create writable memory view from read-only memoryview";
static const char __pyx_k_Empty_shape_tuple_for_cython_arr[] = "Empty shape tuple for cython.array";
static const char __pyx_k_Expected_at_least_d_argument_s_g[] = "Expected at least %d argument%s, got %d";
static const char __pyx_k_Function_call_with_ambiguous_arg[] = "Function call with ambiguous argument types";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))";
static const char __pyx_k_Indirect_dimensions_not_supporte[] = "Indirect dimensions not supported";
static const char __pyx_k_Invalid_mode_expected_c_or_fortr[] = "Invalid mode, expected 'c' or 'fortran', got %s";
static const char __pyx_k_Out_of_bounds_on_buffer_access_a[] = "Out of bounds on buffer access (axis %d)";
static const char __pyx_k_PeakBoundary_0_2f_0_2f_0_2f_0_3e[] = "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)";
static const char __pyx_k_Unable_to_convert_item_to_object[] = "Unable to convert item to object";
static const char __pyx_k_got_differing_extents_in_dimensi[] = "got differing extents in dimension %d (got %d and %d)";
static const char __pyx_k_locate_peak_boundaries_locals_la[] = "locate_peak_boundaries.<locals>.<lambda>";
static const char __pyx_k_ms_deisotope__c_feature_map_prof[] = "ms_deisotope._c.feature_map.profile_transform";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_unable_to_allocate_shape_and_str[] = "unable to allocate shape and strides.";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_2[] = "Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_3[] = "Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_4[] = "Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))";
static const char __pyx_k_ms_deisotope__c_feature_map_prof_2[] = "ms_deisotope/_c/feature_map/profile_transform.pyx";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_ASCII;
static PyObject *__pyx_kp_s_Buffer_view_does_not_expose_stri;
static PyObject *__pyx_kp_s_Can_only_create_a_buffer_that_is;
static PyObject *__pyx_kp_s_Cannot_assign_to_read_only_memor;
static PyObject *__pyx_kp_s_Cannot_create_writable_memory_vi;
static PyObject *__pyx_kp_s_Cannot_index_with_type_s;
static PyObject *__pyx_n_s_Ellipsis;
static PyObject *__pyx_kp_s_Empty_shape_tuple_for_cython_arr;
static PyObject *__pyx_kp_s_Expected_at_least_d_argument_s_g;
static PyObject *__pyx_kp_s_Function_call_with_ambiguous_arg;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_3;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_4;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_kp_s_Indirect_dimensions_not_supporte;
static PyObject *__pyx_kp_s_Invalid_mode_expected_c_or_fortr;
static PyObject *__pyx_kp_s_Invalid_shape_in_axis_d_d;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_MemoryView_of_r_at_0x_x;
static PyObject *__pyx_kp_s_MemoryView_of_r_object;
static PyObject *__pyx_kp_s_No_matching_signature_found;
static PyObject *__pyx_n_b_O;
static PyObject *__pyx_kp_s_Out_of_bounds_on_buffer_access_a;
static PyObject *__pyx_n_s_PeakBoundary;
static PyObject *__pyx_kp_s_PeakBoundary_0_2f_0_2f_0_2f_0_3e;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_ProfileSplitter;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_convert_item_to_object;
static PyObject *__pyx_n_s_ValleyPoint;
static PyObject *__pyx_kp_s_ValleyPoint_0_2f_0_2f_0_2f_0_3e;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_View_MemoryView;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_n_s_allocate_buffer;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_as_arrays;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_n_s_case;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_concatenate;
static PyObject *__pyx_kp_s_contiguous_and_direct;
static PyObject *__pyx_kp_s_contiguous_and_indirect;
static PyObject *__pyx_n_s_defaults;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_kp_s_double_double;
static PyObject *__pyx_kp_s_double_float;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dtype_is_object;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_exp;
static PyObject *__pyx_n_s_extreme_indices;
static PyObject *__pyx_n_s_find_time;
static PyObject *__pyx_n_s_first_maximum;
static PyObject *__pyx_n_s_first_maximum_index;
static PyObject *__pyx_n_s_first_minimum;
static PyObject *__pyx_n_s_first_minimum_index;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_kp_s_float_double;
static PyObject *__pyx_kp_s_float_float;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_fortran;
static PyObject *__pyx_n_u_fortran;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get_total_distance;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_kp_s_got_differing_extents_in_dimensi;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_int64;
static PyObject *__pyx_n_s_intensity;
static PyObject *__pyx_n_s_interp;
static PyObject *__pyx_n_s_interpolate;
static PyObject *__pyx_n_s_interpolate_past;
static PyObject *__pyx_n_s_interpolate_point;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_kp_s_itemsize_0_for_cython_array;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_level;
static PyObject *__pyx_n_s_linspace;
static PyObject *__pyx_n_s_locate_peak_boundaries_locals_la;
static PyObject *__pyx_n_s_locate_valleys;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max;
static PyObject *__pyx_n_s_maximum;
static PyObject *__pyx_n_s_maximum_index;
static PyObject *__pyx_n_s_mean;
static PyObject *__pyx_n_s_median;
static PyObject *__pyx_n_s_memview;
static PyObject *__pyx_n_s_min;
static PyObject *__pyx_n_s_minimum;
static PyObject *__pyx_n_s_minimum_index;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_ms_deisotope__c_feature_map_prof;
static PyObject *__pyx_kp_s_ms_deisotope__c_feature_map_prof_2;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_n_levels;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_new_xs;
static PyObject *__pyx_n_s_new_ys;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_nodes;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_out;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_pi;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_profile;
static PyObject *__pyx_n_s_profiles;
static PyObject *__pyx_n_s_ps;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_fuse_0_0interpolate_point;
static PyObject *__pyx_n_s_pyx_fuse_0_1interpolate_point;
static PyObject *__pyx_n_s_pyx_fuse_0sliding_mean;
static PyObject *__pyx_n_s_pyx_fuse_1_0interpolate_point;
static PyObject *__pyx_n_s_pyx_fuse_1_1interpolate_point;
static PyObject *__pyx_n_s_pyx_fuse_1sliding_mean;
static PyObject *__pyx_n_s_pyx_getbuffer;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Enum;
static PyObject *__pyx_n_s_pyx_unpickle_PeakBoundary;
static PyObject *__pyx_n_s_pyx_unpickle_ProfileSplitter;
static PyObject *__pyx_n_s_pyx_unpickle_ValleyPoint;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_scale;
static PyObject *__pyx_n_s_second_maximum;
static PyObject *__pyx_n_s_second_maximum_index;
static PyObject *__pyx_n_s_second_minimum;
static PyObject *__pyx_n_s_second_minimum_index;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_signatures;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_sliding_mean;
static PyObject *__pyx_n_s_smooth;
static PyObject *__pyx_n_s_sorted;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_split_valley;
static PyObject *__pyx_n_s_split_valleys;
static PyObject *__pyx_n_s_split_valleys_locals_genexpr;
static PyObject *__pyx_n_s_splitter;
static PyObject *__pyx_n_s_sqrt;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_strided_and_direct;
static PyObject *__pyx_kp_s_strided_and_direct_or_indirect;
static PyObject *__pyx_kp_s_strided_and_indirect;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_total_distance;
static PyObject *__pyx_kp_s_unable_to_allocate_array_data;
static PyObject *__pyx_kp_s_unable_to_allocate_shape_and_str;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_vallies;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_var;
static PyObject *__pyx_n_s_width;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_xs;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_ys;
static PyObject *__pyx_n_s_zeros_like;
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, double __pyx_v_scale); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_2locate_extrema(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_4interpolate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, PyObject *__pyx_v_n); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_20__pyx_fuse_0sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_ys); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_22__pyx_fuse_1sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_ys); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_8binsearch(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_10gaussian_smooth(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, PyArrayObject *__pyx_v_y, double __pyx_v_width); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12smooth_leveled(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_xs, PyArrayObject *__pyx_v_ys, double __pyx_v_level); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_first_minimum, PyObject *__pyx_v_maximum, PyObject *__pyx_v_second_minimum, PyObject *__pyx_v_first_minimum_index, PyObject *__pyx_v_maximum_index, PyObject *__pyx_v_second_minimum_index); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_4split(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_profile); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_6__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_8__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_first_maximum, PyObject *__pyx_v_minimum, PyObject *__pyx_v_second_maximum, PyObject *__pyx_v_first_maximum_index, PyObject *__pyx_v_minimum_index, PyObject *__pyx_v_second_maximum_index); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_4split(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_profile); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_6__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_8__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_profile); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2_extreme_indices(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_ys); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_4interpolate(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, int __pyx_v_n); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_6locate_peak_boundaries(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, double __pyx_v_smooth, int __pyx_v_interpolate_past); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_8locate_valleys(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, double __pyx_v_scale, double __pyx_v_smooth, int __pyx_v_interpolate_past); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_10extract_peak(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_peak_boundary); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_12split_valley(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_valley); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_14__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_16__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_14_get_total_distance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_obj); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_16split_valleys(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_profiles, PyObject *__pyx_v_scale, PyObject *__pyx_v_n_levels); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_32__pyx_fuse_0_0interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_34__pyx_fuse_0_1interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_36__pyx_fuse_1_0interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_38__pyx_fuse_1_1interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_26__pyx_unpickle_PeakBoundary(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_28__pyx_unpickle_ValleyPoint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_30__pyx_unpickle_ProfileSplitter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1_;
static PyObject *__pyx_float_0_3;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_200;
static PyObject *__pyx_int_19759364;
static PyObject *__pyx_int_57438986;
static PyObject *__pyx_int_107829254;
static PyObject *__pyx_int_112105877;
static PyObject *__pyx_int_136983863;
static PyObject *__pyx_int_164464950;
static PyObject *__pyx_int_184977713;
static PyObject *__pyx_int_210030367;
static PyObject *__pyx_int_237309992;
static PyObject *__pyx_int_256207781;
static PyObject *__pyx_int_261271048;
static PyObject *__pyx_int_261528200;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_slice__6;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__26;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__51;
/* Late includes */

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":22
 * 
 * 
 * cpdef object find_immediate_minima(np.ndarray[double, ndim=1] x, double scale):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_1find_immediate_minima(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(PyArrayObject *__pyx_v_x, double __pyx_v_scale, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_maxima = 0
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_minima = 0
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_val
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_prev
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_nxt
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_2;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_3;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_4;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_5;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_6;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_7;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_8;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_9;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("find_immediate_minima", 0);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 22, __pyx_L1_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":28
 *         double val, prev, nxt
 * 
 *     maxima = []             # <<<<<<<<<<<<<<
 *     minima = []
 *     n = x.shape[0]
 */

#line 28 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)

#line 28 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 28 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_maxima = ((PyObject*)__pyx_t_1);

#line 28 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":29
 * 
 *     maxima = []
 *     minima = []             # <<<<<<<<<<<<<<
 *     n = x.shape[0]
 *     for i in range(1, n - 1):
 */

#line 29 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)

#line 29 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 29 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_minima = ((PyObject*)__pyx_t_1);

#line 29 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":30
 *     maxima = []
 *     minima = []
 *     n = x.shape[0]             # <<<<<<<<<<<<<<
 *     for i in range(1, n - 1):
 *         val = x[i]
 */

#line 30 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = (__pyx_v_x->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":31
 *     minima = []
 *     n = x.shape[0]
 *     for i in range(1, n - 1):             # <<<<<<<<<<<<<<
 *         val = x[i]
 *         prev = x[i - 1]
 */

#line 31 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_v_n - 1);

#line 31 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_t_2;

#line 31 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_4 = 1; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 31 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":32
 *     n = x.shape[0]
 *     for i in range(1, n - 1):
 *         val = x[i]             # <<<<<<<<<<<<<<
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 */

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = __pyx_v_i;

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 32, __pyx_L1_error)

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 32 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_val = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":33
 *     for i in range(1, n - 1):
 *         val = x[i]
 *         prev = x[i - 1]             # <<<<<<<<<<<<<<
 *         nxt = x[i + 1]
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):
 */

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = (__pyx_v_i - 1);

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 33, __pyx_L1_error)

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 33 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_prev = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":34
 *         val = x[i]
 *         prev = x[i - 1]
 *         nxt = x[i + 1]             # <<<<<<<<<<<<<<
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):
 *             maxima.append(i)
 */

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = (__pyx_v_i + 1);

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 34, __pyx_L1_error)

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 34 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_nxt = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":35
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):             # <<<<<<<<<<<<<<
 *             maxima.append(i)
 *         elif val < prev * scale and val < nxt * scale:
 */

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val > (__pyx_v_prev * (1.0 + __pyx_v_scale))) != 0);

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_8) {

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = __pyx_t_8;

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L6_bool_binop_done;

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val > (__pyx_v_nxt * (1.0 + __pyx_v_scale))) != 0);

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = __pyx_t_8;

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L6_bool_binop_done:;

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":36
 *         nxt = x[i + 1]
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):
 *             maxima.append(i)             # <<<<<<<<<<<<<<
 *         elif val < prev * scale and val < nxt * scale:
 *             minima.append(i)
 */

#line 36 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)

#line 36 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 36 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_maxima, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 36, __pyx_L1_error)

#line 36 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":35
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):             # <<<<<<<<<<<<<<
 *             maxima.append(i)
 *         elif val < prev * scale and val < nxt * scale:
 */

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 35 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":37
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):
 *             maxima.append(i)
 *         elif val < prev * scale and val < nxt * scale:             # <<<<<<<<<<<<<<
 *             minima.append(i)
 * 
 */

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val < (__pyx_v_prev * __pyx_v_scale)) != 0);

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_8) {

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = __pyx_t_8;

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L8_bool_binop_done;

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val < (__pyx_v_nxt * __pyx_v_scale)) != 0);

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = __pyx_t_8;

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L8_bool_binop_done:;

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":38
 *             maxima.append(i)
 *         elif val < prev * scale and val < nxt * scale:
 *             minima.append(i)             # <<<<<<<<<<<<<<
 * 
 *     return np.array(maxima, dtype=np.int64), np.array(minima, dtype=np.int64)
 */

#line 38 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)

#line 38 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 38 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_minima, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 38, __pyx_L1_error)

#line 38 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":37
 *         if val > prev * (1 + scale) and val > nxt * (1 + scale):
 *             maxima.append(i)
 *         elif val < prev * scale and val < nxt * scale:             # <<<<<<<<<<<<<<
 *             minima.append(i)
 * 
 */

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

#line 37 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":40
 *             minima.append(i)
 * 
 *     return np.array(maxima, dtype=np.int64), np.array(minima, dtype=np.int64)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_array); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_maxima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_maxima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_maxima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_int64); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_dtype, __pyx_t_13) < 0) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_minima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_minima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_minima);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_int64); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_dtype, __pyx_t_14) < 0) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 40, __pyx_L1_error)

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_13);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_14);

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_10;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = 0;

#line 40 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":22
 * 
 * 
 * cpdef object find_immediate_minima(np.ndarray[double, ndim=1] x, double scale):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.find_immediate_minima", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_maxima);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_minima);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_1find_immediate_minima(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima[] = "find_immediate_minima(ndarray x, double scale)";

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_1find_immediate_minima(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_x = 0
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_scale
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("find_immediate_minima (wrapper)", 0);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_scale,0};

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[2] = {0,0};

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  2: 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_scale)) != 0)) kw_args--;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("find_immediate_minima", 1, 2, 2, 1); 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 22, __pyx_L3_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "find_immediate_minima") < 0)) __PYX_ERR(0, 22, __pyx_L3_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = ((PyArrayObject *)values[0]);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_scale = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_scale == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L3_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_immediate_minima", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 22, __pyx_L3_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.find_immediate_minima", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_5numpy_ndarray, 1, "x", 0))) __PYX_ERR(0, 22, __pyx_L1_error)
  __pyx_r = 
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(__pyx_self, __pyx_v_x, __pyx_v_scale);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, double __pyx_v_scale) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("find_immediate_minima", 0);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 22, __pyx_L1_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima(__pyx_v_x, __pyx_v_scale, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.find_immediate_minima", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 22 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":43
 * 
 * 
 * cpdef object locate_extrema(np.ndarray[double, ndim=1] x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_3locate_extrema(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_locate_extrema(PyArrayObject *__pyx_v_x, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_maxima = 0
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_minima = 0
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_val
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_prev
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_nxt
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_2;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_3;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_4;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_5;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_6;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_7;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_8;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_9;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_extrema", 0);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 43, __pyx_L1_error)

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":49
 *         double val, prev, nxt
 * 
 *     maxima = []             # <<<<<<<<<<<<<<
 *     minima = []
 *     n = x.shape[0]
 */

#line 49 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)

#line 49 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 49 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_maxima = ((PyObject*)__pyx_t_1);

#line 49 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":50
 * 
 *     maxima = []
 *     minima = []             # <<<<<<<<<<<<<<
 *     n = x.shape[0]
 *     for i in range(1, n - 1):
 */

#line 50 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 50, __pyx_L1_error)

#line 50 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 50 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_minima = ((PyObject*)__pyx_t_1);

#line 50 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":51
 *     maxima = []
 *     minima = []
 *     n = x.shape[0]             # <<<<<<<<<<<<<<
 *     for i in range(1, n - 1):
 *         val = x[i]
 */

#line 51 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = (__pyx_v_x->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":52
 *     minima = []
 *     n = x.shape[0]
 *     for i in range(1, n - 1):             # <<<<<<<<<<<<<<
 *         val = x[i]
 *         prev = x[i - 1]
 */

#line 52 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_v_n - 1);

#line 52 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_t_2;

#line 52 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_4 = 1; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 52 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":53
 *     n = x.shape[0]
 *     for i in range(1, n - 1):
 *         val = x[i]             # <<<<<<<<<<<<<<
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 */

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = __pyx_v_i;

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 53, __pyx_L1_error)

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 53 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_val = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":54
 *     for i in range(1, n - 1):
 *         val = x[i]
 *         prev = x[i - 1]             # <<<<<<<<<<<<<<
 *         nxt = x[i + 1]
 *         if val > prev and val > nxt:
 */

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = (__pyx_v_i - 1);

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 54, __pyx_L1_error)

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 54 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_prev = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":55
 *         val = x[i]
 *         prev = x[i - 1]
 *         nxt = x[i + 1]             # <<<<<<<<<<<<<<
 *         if val > prev and val > nxt:
 *             maxima.append(i)
 */

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = (__pyx_v_i + 1);

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = -1;

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_6 = 0;

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_6 != -1)) {

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_6);

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 55, __pyx_L1_error)

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 55 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_nxt = (*__Pyx_BufPtrStrided1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_5, __pyx_pybuffernd_x.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":56
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 *         if val > prev and val > nxt:             # <<<<<<<<<<<<<<
 *             maxima.append(i)
 *         elif val < prev and val < nxt:
 */

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val > __pyx_v_prev) != 0);

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_8) {

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = __pyx_t_8;

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L6_bool_binop_done;

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val > __pyx_v_nxt) != 0);

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = __pyx_t_8;

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L6_bool_binop_done:;

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":57
 *         nxt = x[i + 1]
 *         if val > prev and val > nxt:
 *             maxima.append(i)             # <<<<<<<<<<<<<<
 *         elif val < prev and val < nxt:
 *             minima.append(i)
 */

#line 57 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)

#line 57 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 57 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_maxima, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 57, __pyx_L1_error)

#line 57 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":56
 *         prev = x[i - 1]
 *         nxt = x[i + 1]
 *         if val > prev and val > nxt:             # <<<<<<<<<<<<<<
 *             maxima.append(i)
 *         elif val < prev and val < nxt:
 */

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 56 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":58
 *         if val > prev and val > nxt:
 *             maxima.append(i)
 *         elif val < prev and val < nxt:             # <<<<<<<<<<<<<<
 *             minima.append(i)
 * 
 */

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val < __pyx_v_prev) != 0);

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_8) {

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = __pyx_t_8;

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L8_bool_binop_done;

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = ((__pyx_v_val < __pyx_v_nxt) != 0);

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = __pyx_t_8;

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L8_bool_binop_done:;

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":59
 *             maxima.append(i)
 *         elif val < prev and val < nxt:
 *             minima.append(i)             # <<<<<<<<<<<<<<
 * 
 *     return np.array(maxima, dtype=np.int64), np.array(minima, dtype=np.int64)
 */

#line 59 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)

#line 59 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 59 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_minima, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 59, __pyx_L1_error)

#line 59 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":58
 *         if val > prev and val > nxt:
 *             maxima.append(i)
 *         elif val < prev and val < nxt:             # <<<<<<<<<<<<<<
 *             minima.append(i)
 * 
 */

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

#line 58 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":61
 *             minima.append(i)
 * 
 *     return np.array(maxima, dtype=np.int64), np.array(minima, dtype=np.int64)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_array); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_maxima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_maxima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_maxima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_int64); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_dtype, __pyx_t_13) < 0) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_minima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_minima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_minima);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_int64); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_dtype, __pyx_t_14) < 0) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 61, __pyx_L1_error)

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_13);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_14);

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_10;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = 0;

#line 61 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":43
 * 
 * 
 * cpdef object locate_extrema(np.ndarray[double, ndim=1] x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.locate_extrema", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_maxima);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_minima);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_3locate_extrema(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_2locate_extrema[] = "locate_extrema(ndarray x)";

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_3locate_extrema(PyObject *__pyx_self, PyObject *__pyx_v_x) {

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_extrema (wrapper)", 0);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_5numpy_ndarray, 1, "x", 0))) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_r = 
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_2locate_extrema(__pyx_self, ((PyArrayObject *)__pyx_v_x));

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_2locate_extrema(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_extrema", 0);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 43, __pyx_L1_error)

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_locate_extrema(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.locate_extrema", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 43 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":64
 * 
 * 
 * def interpolate(xs, ys, n=200):             # <<<<<<<<<<<<<<
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)
 */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_5interpolate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_4interpolate[] = "interpolate(xs, ys, n=200)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_5interpolate = 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"interpolate", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_5interpolate, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_4interpolate};

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_5interpolate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_xs = 0
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_ys = 0
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_n = 0
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate (wrapper)", 0);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xs,&__pyx_n_s_ys,&__pyx_n_s_n,0};

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    values[2] = ((PyObject *)__pyx_int_200);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_xs)) != 0)) kw_args--;
        else 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ys)) != 0)) kw_args--;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate", 0, 2, 3, 1); 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 64, __pyx_L3_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate") < 0)) __PYX_ERR(0, 64, __pyx_L3_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;
        default: 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_xs = values[0];

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_ys = values[1];

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_n = values[2];

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 64, __pyx_L3_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.interpolate", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_4interpolate(__pyx_self, __pyx_v_xs, __pyx_v_ys, __pyx_v_n);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_4interpolate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, PyObject *__pyx_v_n) {
  PyObject *__pyx_v_new_xs = NULL
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_new_ys = NULL
#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_7;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":65
 * 
 * def interpolate(xs, ys, n=200):
 *     new_xs = np.linspace(xs.min(), xs.max(), n)             # <<<<<<<<<<<<<<
 *     new_ys = np.interp(new_xs, xs, ys)
 *     return new_xs, new_ys
 */

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_linspace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_xs, __pyx_n_s_min); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = NULL;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_5)) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_4, function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_xs, __pyx_n_s_max); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = NULL;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_6)) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_6);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_5, function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = NULL;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_5)) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = 1;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCALL

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyFunction_Check(__pyx_t_3)) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_t_2, __pyx_t_4, __pyx_v_n};

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCCALL

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_t_2, __pyx_t_4, __pyx_v_n};

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_5) {

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_2);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_7, __pyx_t_2);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_7, __pyx_t_4);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_n);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_n);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_7, __pyx_v_n);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_new_xs = __pyx_t_1;

#line 65 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":66
 * def interpolate(xs, ys, n=200):
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)             # <<<<<<<<<<<<<<
 *     return new_xs, new_ys
 * 
 */

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_interp); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = NULL;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_3)) {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_6, function);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = 1;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCALL

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyFunction_Check(__pyx_t_6)) {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_new_xs, __pyx_v_xs, __pyx_v_ys};

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCCALL

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_new_xs, __pyx_v_xs, __pyx_v_ys};

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_new_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_new_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, __pyx_v_new_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, __pyx_v_xs);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_ys);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_ys);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_7, __pyx_v_ys);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_new_ys = __pyx_t_1;

#line 66 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":67
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)
 *     return new_xs, new_ys             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_new_xs);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_new_xs);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_new_xs);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_new_ys);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_new_ys);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_new_ys);

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 67 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":64
 * 
 * 
 * def interpolate(xs, ys, n=200):             # <<<<<<<<<<<<<<
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)
 */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.interpolate", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_new_xs);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_new_ys);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean[] = "sliding_mean(signatures, args, kwargs, defaults)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"sliding_mean", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean};

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_args = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_kwargs = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[4] = {0,0,0,0};

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  4: 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 70, __pyx_L3_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 70, __pyx_L3_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  3:

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 70, __pyx_L3_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) __PYX_ERR(0, 70, __pyx_L3_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_signatures = values[0];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_args = values[1];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_kwargs = values[2];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_defaults = values[3];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 70, __pyx_L3_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  Py_ssize_t __pyx_v_i
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyTypeObject *__pyx_v_ndarray = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_memslice
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  Py_ssize_t __pyx_v_itemsize
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  CYTHON_UNUSED int __pyx_v_dtype_signed
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  char __pyx_v_kind
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_arg = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_dtype = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_arg_base = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_candidates = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_sig = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  int __pyx_v_match_found
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_src_sig = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_dst_type = NULL
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_4;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_5;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  long __pyx_t_7;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_memviewslice __pyx_t_8;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_9;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_10;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_11;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_15;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_16;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_17;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_18;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("sliding_mean", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_kwargs);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_4) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((!__pyx_t_4) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__Pyx_ImportNumPyArrayTypeIfAvailable()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_itemsize = -1L;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_args == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((0 < __pyx_t_5) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_GetItemInt_Tuple(((PyObject*)__pyx_v_args), 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_4) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L7_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_ys, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_t_4 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L7_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_ys); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_int_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_int_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_int_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_kp_s_);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_kp_s_);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_kp_s_);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_2 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype = __pyx_t_6;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L12;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __pyx_memoryview_check(__pyx_v_arg); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_arg_base = __pyx_t_6;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = (__pyx_t_3 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_dtype = __pyx_t_6;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L13;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_dtype = Py_None;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L13:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L12;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype = Py_None;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L12:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_itemsize = -1L;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_v_dtype != Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_itemsize = __pyx_t_5;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __Pyx_PyObject_Ord(__pyx_t_6); if (unlikely(__pyx_t_7 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_kind = __pyx_t_7;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        switch (__pyx_v_kind) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'i':

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'u':

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'f':

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L16_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L16_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L10_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L19_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L19_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L10_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'c':

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'O':

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          default: break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L22_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L22_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_v_arg, 0); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L10_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L26_bool_binop_done;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L26_bool_binop_done:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_v_arg, 0); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L10_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L10_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L10_break:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_candidates = ((PyObject*)__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_signatures == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_9), (&__pyx_t_10)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_9, &__pyx_t_5, &__pyx_t_1, NULL, NULL, __pyx_t_10);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 == 0)) break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_match_found = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_14)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_14);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_kp_s__2) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__2);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_12);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_split); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_12)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_12);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_12, __pyx_kp_s__3) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_src_sig, __pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_15 = PyList_GET_SIZE(__pyx_v_dest_sig); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = __pyx_t_15;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_i = __pyx_t_17;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_dest_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_dst_type != Py_None);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_src_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_13 = PyObject_RichCompare(__pyx_t_1, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_match_found = 1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L34;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_match_found = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L32_break;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L34:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L32_break:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_v_match_found != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_2) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_18 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_18 == ((int)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((!__pyx_t_2) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_9 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_t_9 > 1) != 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_signatures == Py_None)) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_candidates, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), __pyx_t_6); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = __pyx_t_13;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dest_sig);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_ndarray);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_arg);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dtype);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_arg_base);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_candidates);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_sig);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_src_sig);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dst_type);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_kwargs);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys); /*proto*/

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(PyArrayObject *__pyx_v_ys, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  npy_intp __pyx_v_knd
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_result = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_vlo
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_v
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_vhi
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_result
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_result
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_2 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_7;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_8;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_9;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_10;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_11;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0sliding_mean", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_result.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_result.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_result.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_result.rcbuffer = &__pyx_pybuffer_result;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_float, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":77
 *         double vlo, v, vhi
 * 
 *     n = ys.shape[0]             # <<<<<<<<<<<<<<
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 */

#line 77 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = (__pyx_v_ys->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":78
 * 
 *     n = ys.shape[0]
 *     knd = n             # <<<<<<<<<<<<<<
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 */

#line 78 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_knd = __pyx_v_n;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":79
 *     n = ys.shape[0]
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         if i == 0:
 */

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyArray_ZEROS(1, (&__pyx_v_knd), NPY_FLOAT64, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((PyArrayObject *)__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_result.rcbuffer->pybuffer, (PyObject*)__pyx_t_2, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 < 0)) {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_result.rcbuffer->pybuffer, (PyObject*)__pyx_v_result, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_4, __pyx_t_5, __pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_4 = __pyx_t_5 = __pyx_t_6 = 0;

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_result.diminfo[0].strides = __pyx_pybuffernd_result.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_result.diminfo[0].shape = __pyx_pybuffernd_result.rcbuffer->pybuffer.shape[0];

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_result = ((PyArrayObject *)__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":80
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             vlo = 0
 */

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = __pyx_v_n;

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = __pyx_t_7;

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_9;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":81
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             vlo = 0
 *         else:
 */

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = ((__pyx_v_i == 0) != 0);

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_10) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":82
 *     for i in range(n):
 *         if i == 0:
 *             vlo = 0             # <<<<<<<<<<<<<<
 *         else:
 *             vlo = ys[i - 1]
 */

#line 82 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vlo = 0.0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":81
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             vlo = 0
 *         else:
 */

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":84
 *             vlo = 0
 *         else:
 *             vlo = ys[i - 1]             # <<<<<<<<<<<<<<
 *         v = ys[i]
 *         if i == (n - 1):
 */

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = (__pyx_v_i - 1);

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = -1;

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_3 != -1)) {

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 84, __pyx_L1_error)

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vlo = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":85
 *         else:
 *             vlo = ys[i - 1]
 *         v = ys[i]             # <<<<<<<<<<<<<<
 *         if i == (n - 1):
 *             vhi = 0
 */

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __pyx_v_i;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = -1;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 != -1)) {

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 85, __pyx_L1_error)

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_v = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":86
 *             vlo = ys[i - 1]
 *         v = ys[i]
 *         if i == (n - 1):             # <<<<<<<<<<<<<<
 *             vhi = 0
 *         else:
 */

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = ((__pyx_v_i == (__pyx_v_n - 1)) != 0);

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_10) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":87
 *         v = ys[i]
 *         if i == (n - 1):
 *             vhi = 0             # <<<<<<<<<<<<<<
 *         else:
 *             vhi = ys[i + 1]
 */

#line 87 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vhi = 0.0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":86
 *             vlo = ys[i - 1]
 *         v = ys[i]
 *         if i == (n - 1):             # <<<<<<<<<<<<<<
 *             vhi = 0
 *         else:
 */

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L6;

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":89
 *             vhi = 0
 *         else:
 *             vhi = ys[i + 1]             # <<<<<<<<<<<<<<
 *         result[i] = (vlo + v + vhi) / 3
 *     return result
 */

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = (__pyx_v_i + 1);

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = -1;

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_3 != -1)) {

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 89, __pyx_L1_error)

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vhi = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L6:;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":90
 *         else:
 *             vhi = ys[i + 1]
 *         result[i] = (vlo + v + vhi) / 3             # <<<<<<<<<<<<<<
 *     return result
 * 
 */

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __pyx_v_i;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = -1;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_result.diminfo[0].shape)) __pyx_t_3 = 0;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 != -1)) {

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 90, __pyx_L1_error)

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    *__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_result.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_result.diminfo[0].strides) = (((__pyx_v_vlo + __pyx_v_v) + __pyx_v_vhi) / 3.0);

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":91
 *             vhi = ys[i + 1]
 *         result[i] = (vlo + v + vhi) / 3
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.sliding_mean", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_0sliding_mean", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean};

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0sliding_mean (wrapper)", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_ys), __pyx_ptype_5numpy_ndarray, 1, "ys", 0))) __PYX_ERR(0, 70, __pyx_L1_error)
  __pyx_r = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_20__pyx_fuse_0sliding_mean(__pyx_self, ((PyArrayObject *)__pyx_v_ys));

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_20__pyx_fuse_0sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_ys) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0sliding_mean", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_float, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(((PyArrayObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_0sliding_mean", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys); /*proto*/

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(PyArrayObject *__pyx_v_ys, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  npy_intp __pyx_v_knd
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_result = 0
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_vlo
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_v
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_vhi
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_result
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_result
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_2 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_7;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_8;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_9;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_10;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_11;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1sliding_mean", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_result.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_result.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_result.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_result.rcbuffer = &__pyx_pybuffer_result;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":77
 *         double vlo, v, vhi
 * 
 *     n = ys.shape[0]             # <<<<<<<<<<<<<<
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 */

#line 77 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = (__pyx_v_ys->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":78
 * 
 *     n = ys.shape[0]
 *     knd = n             # <<<<<<<<<<<<<<
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 */

#line 78 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_knd = __pyx_v_n;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":79
 *     n = ys.shape[0]
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         if i == 0:
 */

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyArray_ZEROS(1, (&__pyx_v_knd), NPY_FLOAT64, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((PyArrayObject *)__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_result.rcbuffer->pybuffer, (PyObject*)__pyx_t_2, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 < 0)) {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_result.rcbuffer->pybuffer, (PyObject*)__pyx_v_result, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_4, __pyx_t_5, __pyx_t_6);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_4 = __pyx_t_5 = __pyx_t_6 = 0;

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_result.diminfo[0].strides = __pyx_pybuffernd_result.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_result.diminfo[0].shape = __pyx_pybuffernd_result.rcbuffer->pybuffer.shape[0];

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_result = ((PyArrayObject *)__pyx_t_1);

#line 79 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":80
 *     knd = n
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             vlo = 0
 */

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = __pyx_v_n;

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = __pyx_t_7;

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {

#line 80 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_9;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":81
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             vlo = 0
 *         else:
 */

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = ((__pyx_v_i == 0) != 0);

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_10) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":82
 *     for i in range(n):
 *         if i == 0:
 *             vlo = 0             # <<<<<<<<<<<<<<
 *         else:
 *             vlo = ys[i - 1]
 */

#line 82 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vlo = 0.0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":81
 *     result = np.PyArray_ZEROS(1, &knd, np.NPY_FLOAT64, 0)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             vlo = 0
 *         else:
 */

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 81 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":84
 *             vlo = 0
 *         else:
 *             vlo = ys[i - 1]             # <<<<<<<<<<<<<<
 *         v = ys[i]
 *         if i == (n - 1):
 */

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = (__pyx_v_i - 1);

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = -1;

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_3 != -1)) {

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 84, __pyx_L1_error)

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vlo = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 84 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":85
 *         else:
 *             vlo = ys[i - 1]
 *         v = ys[i]             # <<<<<<<<<<<<<<
 *         if i == (n - 1):
 *             vhi = 0
 */

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __pyx_v_i;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = -1;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 != -1)) {

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 85, __pyx_L1_error)

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 85 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_v = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":86
 *             vlo = ys[i - 1]
 *         v = ys[i]
 *         if i == (n - 1):             # <<<<<<<<<<<<<<
 *             vhi = 0
 *         else:
 */

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = ((__pyx_v_i == (__pyx_v_n - 1)) != 0);

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_10) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":87
 *         v = ys[i]
 *         if i == (n - 1):
 *             vhi = 0             # <<<<<<<<<<<<<<
 *         else:
 *             vhi = ys[i + 1]
 */

#line 87 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vhi = 0.0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":86
 *             vlo = ys[i - 1]
 *         v = ys[i]
 *         if i == (n - 1):             # <<<<<<<<<<<<<<
 *             vhi = 0
 *         else:
 */

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L6;

#line 86 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":89
 *             vhi = 0
 *         else:
 *             vhi = ys[i + 1]             # <<<<<<<<<<<<<<
 *         result[i] = (vlo + v + vhi) / 3
 *     return result
 */

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = (__pyx_v_i + 1);

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = -1;

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_ys.diminfo[0].shape)) __pyx_t_3 = 0;

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_3 != -1)) {

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 89, __pyx_L1_error)

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_vhi = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_ys.diminfo[0].strides));

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 89 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L6:;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":90
 *         else:
 *             vhi = ys[i + 1]
 *         result[i] = (vlo + v + vhi) / 3             # <<<<<<<<<<<<<<
 *     return result
 * 
 */

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __pyx_v_i;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = -1;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 >= (size_t)__pyx_pybuffernd_result.diminfo[0].shape)) __pyx_t_3 = 0;

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 != -1)) {

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 90, __pyx_L1_error)

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    *__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_result.rcbuffer->pybuffer.buf, __pyx_t_11, __pyx_pybuffernd_result.diminfo[0].strides) = (((__pyx_v_vlo + __pyx_v_v) + __pyx_v_vhi) / 3.0);

#line 90 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":91
 *             vhi = ys[i + 1]
 *         result[i] = (vlo + v + vhi) / 3
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 91 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.sliding_mean", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_result.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_1sliding_mean", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_6sliding_mean};

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean(PyObject *__pyx_self, PyObject *__pyx_v_ys) {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1sliding_mean (wrapper)", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_ys), __pyx_ptype_5numpy_ndarray, 1, "ys", 0))) __PYX_ERR(0, 70, __pyx_L1_error)
  __pyx_r = 
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_22__pyx_fuse_1sliding_mean(__pyx_self, ((PyArrayObject *)__pyx_v_ys));

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_22__pyx_fuse_1sliding_mean(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_ys) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1sliding_mean", 0);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(((PyArrayObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_1sliding_mean", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":95
 * 
 * 
 * cdef object sliding_median(np.ndarray[double, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.ndarray[double, ndim=1, mode='c'] arr
 */

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_median(PyArrayObject *__pyx_v_ys) {
  PyArrayObject *__pyx_v_arr = 0
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_i
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_value
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_arr
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_arr
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_4 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_5;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_9;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_10;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_11;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_12;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_13;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_16 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_17 = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_18;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_19;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("sliding_median", 0);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_arr.pybuffer.buf = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_arr.refcount = 0;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_arr.data = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_arr.rcbuffer = &__pyx_pybuffer_arr;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 95, __pyx_L1_error)

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":100
 *         size_t i, n
 *         double value
 *     arr = np.zeros_like(ys)             # <<<<<<<<<<<<<<
 *     n = len(ys)
 *     for i in range(n):
 */

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 100, __pyx_L1_error)

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = NULL;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_2)) {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_2);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, ((PyObject *)__pyx_v_ys)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_ys));

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 100, __pyx_L1_error)

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = ((PyArrayObject *)__pyx_t_1);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_arr.rcbuffer->pybuffer, (PyObject*)__pyx_t_4, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 < 0)) {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_arr.rcbuffer->pybuffer, (PyObject*)__pyx_v_arr, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_6); Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_8);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_6, __pyx_t_7, __pyx_t_8);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = __pyx_t_7 = __pyx_t_8 = 0;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_arr.diminfo[0].strides = __pyx_pybuffernd_arr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_arr.diminfo[0].shape = __pyx_pybuffernd_arr.rcbuffer->pybuffer.shape[0];

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 100, __pyx_L1_error)

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_arr = ((PyArrayObject *)__pyx_t_1);

#line 100 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":101
 *         double value
 *     arr = np.zeros_like(ys)
 *     n = len(ys)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         if i == 0:
 */

#line 101 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_9 = PyObject_Length(((PyObject *)__pyx_v_ys)); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 101, __pyx_L1_error)

#line 101 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = __pyx_t_9;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":102
 *     arr = np.zeros_like(ys)
 *     n = len(ys)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 */

#line 102 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __pyx_v_n;

#line 102 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = __pyx_t_10;

#line 102 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {

#line 102 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_12;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":103
 *     n = len(ys)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 *         elif i == n - 1:
 */

#line 103 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = ((__pyx_v_i == 0) != 0);

#line 103 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_13) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":104
 *     for i in range(n):
 *         if i == 0:
 *             value = np.median(np.concatenate(([0], ys[:2]),))             # <<<<<<<<<<<<<<
 *         elif i == n - 1:
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))
 */

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_median); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_n_s_np); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_14);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_concatenate); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_15);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PyList_New(1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_14);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_int_0);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_int_0);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyList_SET_ITEM(__pyx_t_14, 0, __pyx_int_0);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_slice__6); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_16);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = PyTuple_New(2); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_17);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_14);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_14);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_16);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyTuple_SET_ITEM(__pyx_t_17, 1, __pyx_t_16);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = NULL;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_15))) {

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_15);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(__pyx_t_16)) {

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_t_16);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(function);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF_SET(__pyx_t_15, function);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_15, __pyx_t_16, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_t_17);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_15 = NULL;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_2);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(__pyx_t_15)) {

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_t_15);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(function);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF_SET(__pyx_t_2, function);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_15, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_18 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_18 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_value = __pyx_t_18;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":103
 *     n = len(ys)
 *     for i in range(n):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 *         elif i == n - 1:
 */

#line 103 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 103 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":105
 *         if i == 0:
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 *         elif i == n - 1:             # <<<<<<<<<<<<<<
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))
 *         else:
 */

#line 105 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = ((__pyx_v_i == (__pyx_v_n - 1)) != 0);

#line 105 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_13) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":106
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 *         elif i == n - 1:
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))             # <<<<<<<<<<<<<<
 *         else:
 *             value = np.median(ys[i - 1:i + 2])
 */

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_median); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_n_s_np); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_15);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_concatenate); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_17);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_15 = PyList_New(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_15);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_int_0);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_int_0);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyList_SET_ITEM(__pyx_t_15, 0, __pyx_int_0);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = __Pyx_PyInt_FromSize_t((__pyx_v_n - 2)); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_16);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PySlice_New(__pyx_t_16, Py_None, Py_None); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_14);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_t_14); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_16);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_14);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_15);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_15);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_16);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_16);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_15 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = NULL;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_17))) {

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_17);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(__pyx_t_16)) {

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_t_16);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(function);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF_SET(__pyx_t_17, function);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_17, __pyx_t_16, __pyx_t_14) : __Pyx_PyObject_CallOneArg(__pyx_t_17, __pyx_t_14);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = NULL;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_3);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(__pyx_t_17)) {

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_t_17);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(function);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF_SET(__pyx_t_3, function);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_17, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_18 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_18 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 106 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_value = __pyx_t_18;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":105
 *         if i == 0:
 *             value = np.median(np.concatenate(([0], ys[:2]),))
 *         elif i == n - 1:             # <<<<<<<<<<<<<<
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))
 *         else:
 */

#line 105 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 105 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":108
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))
 *         else:
 *             value = np.median(ys[i - 1:i + 2])             # <<<<<<<<<<<<<<
 *         arr[i] = value
 *     return arr
 */

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_median); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __Pyx_PyInt_FromSize_t((__pyx_v_i - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = __Pyx_PyInt_FromSize_t((__pyx_v_i + 2)); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_17);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PySlice_New(__pyx_t_3, __pyx_t_17, Py_None); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_14);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_t_14); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_17);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = NULL;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_2);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(__pyx_t_14)) {

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_t_14);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(function);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF_SET(__pyx_t_2, function);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_14, __pyx_t_17) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_17);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_18 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_18 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 108, __pyx_L1_error)

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_value = __pyx_t_18;

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 108 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":109
 *         else:
 *             value = np.median(ys[i - 1:i + 2])
 *         arr[i] = value             # <<<<<<<<<<<<<<
 *     return arr
 * 
 */

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_19 = __pyx_v_i;

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = -1;

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_19 >= (size_t)__pyx_pybuffernd_arr.diminfo[0].shape)) __pyx_t_5 = 0;

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 != -1)) {

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 109, __pyx_L1_error)

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    *__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_arr.rcbuffer->pybuffer.buf, __pyx_t_19, __pyx_pybuffernd_arr.diminfo[0].strides) = __pyx_v_value;

#line 109 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":110
 *             value = np.median(ys[i - 1:i + 2])
 *         arr[i] = value
 *     return arr             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 110 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 110 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_arr));

#line 110 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyObject *)__pyx_v_arr);

#line 110 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":95
 * 
 * 
 * cdef object sliding_median(np.ndarray[double, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.ndarray[double, ndim=1, mode='c'] arr
 */

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_16);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_17);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.sliding_median", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_arr.rcbuffer->pybuffer);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 95 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":113
 * 
 * 
 * cpdef size_t binsearch(np.ndarray[double, ndim=1, mode='c'] array, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid
 */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_9binsearch(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static size_t __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch(PyArrayObject *__pyx_v_array, double __pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_lo
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_hi
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_mid
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_point
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_array
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_r;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_2;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("binsearch", 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_array.pybuffer.buf = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_array.refcount = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.data = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 113, __pyx_L1_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":117
 *         size_t lo, hi, mid
 *         double point
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = array.shape[0]
 *     while hi != lo:
 */

#line 117 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":118
 *         double point
 *     lo = 0
 *     hi = array.shape[0]             # <<<<<<<<<<<<<<
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 */

#line 118 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_hi = (__pyx_v_array->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":119
 *     lo = 0
 *     hi = array.shape[0]
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         point = array[mid]
 */

#line 119 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 119 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 119 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":120
 *     hi = array.shape[0]
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         point = array[mid]
 *         if value == point:
 */

#line 120 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":121
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         point = array[mid]             # <<<<<<<<<<<<<<
 *         if value == point:
 *             return mid
 */

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_v_mid;

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = -1;

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 >= (size_t)__pyx_pybuffernd_array.diminfo[0].shape)) __pyx_t_3 = 0;

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_3 != -1)) {

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_3);

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 121, __pyx_L1_error)

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 121 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_point = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_2, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":122
 *         mid = (hi + lo) // 2
 *         point = array[mid]
 *         if value == point:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif hi - lo == 1:
 */

#line 122 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_value == __pyx_v_point) != 0);

#line 122 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":123
 *         point = array[mid]
 *         if value == point:
 *             return mid             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             return mid
 */

#line 123 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_r = __pyx_v_mid;

#line 123 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":122
 *         mid = (hi + lo) // 2
 *         point = array[mid]
 *         if value == point:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif hi - lo == 1:
 */

#line 122 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":124
 *         if value == point:
 *             return mid
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif point > value:
 */

#line 124 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 124 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":125
 *             return mid
 *         elif hi - lo == 1:
 *             return mid             # <<<<<<<<<<<<<<
 *         elif point > value:
 *             hi = mid
 */

#line 125 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_r = __pyx_v_mid;

#line 125 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":124
 *         if value == point:
 *             return mid
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif point > value:
 */

#line 124 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":126
 *         elif hi - lo == 1:
 *             return mid
 *         elif point > value:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 126 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_point > __pyx_v_value) != 0);

#line 126 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":127
 *             return mid
 *         elif point > value:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 127 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":126
 *         elif hi - lo == 1:
 *             return mid
 *         elif point > value:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 126 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 126 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":129
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 129 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 129 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 129 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 129 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

#line 129 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":113
 * 
 * 
 * cpdef size_t binsearch(np.ndarray[double, ndim=1, mode='c'] array, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid
 */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.feature_map.profile_transform.binsearch", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_9binsearch(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_8binsearch[] = "binsearch(ndarray array, double value) -> size_t";

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_9binsearch(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_array = 0
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_value
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("binsearch (wrapper)", 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_array,&__pyx_n_s_value,0};

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[2] = {0,0};

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  2: 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_array)) != 0)) kw_args--;
        else 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("binsearch", 1, 2, 2, 1); 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 113, __pyx_L3_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "binsearch") < 0)) __PYX_ERR(0, 113, __pyx_L3_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_array = ((PyArrayObject *)values[0]);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_value = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 113, __pyx_L3_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("binsearch", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 113, __pyx_L3_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.binsearch", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array), __pyx_ptype_5numpy_ndarray, 1, "array", 0))) __PYX_ERR(0, 113, __pyx_L1_error)
  __pyx_r = 
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_8binsearch(__pyx_self, __pyx_v_array, __pyx_v_value);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_8binsearch(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array, double __pyx_v_value) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_array
#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("binsearch", 0);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_array.pybuffer.buf = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_array.refcount = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.data = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 113, __pyx_L1_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch(__pyx_v_array, __pyx_v_value, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.binsearch", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 113 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":132
 * 
 * 
 * cdef np.ndarray[double, ndim=1, mode='c'] gauss(np.ndarray[double, ndim=1, mode='c'] x,             # <<<<<<<<<<<<<<
 *                                                 double mean, double variance, double scale=2.0):
 *     return 1. / np.sqrt(2 * np.pi * variance) * np.exp(-(x - mean) ** 2 / (scale * variance))
 */

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss(PyArrayObject *__pyx_v_x, double __pyx_v_mean, double __pyx_v_variance, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss *__pyx_optional_args) {

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_v_scale = ((double)2.0);
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_r = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("gauss", 0);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_optional_args) {

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_scale = __pyx_optional_args->scale;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 132, __pyx_L1_error)

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":134
 * cdef np.ndarray[double, ndim=1, mode='c'] gauss(np.ndarray[double, ndim=1, mode='c'] x,
 *                                                 double mean, double variance, double scale=2.0):
 *     return 1. / np.sqrt(2 * np.pi * variance) * np.exp(-(x - mean) ** 2 / (scale * variance))             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_pi); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyNumber_Multiply(__pyx_int_2, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_variance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyNumber_Multiply(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = NULL;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_4)) {

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_, __pyx_t_1, 1., 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_exp); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_mean); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_x), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyNumber_Power(__pyx_t_2, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyNumber_Negative(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyFloat_FromDouble((__pyx_v_scale * __pyx_v_variance)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = NULL;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_5)) {

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_4, function);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 134, __pyx_L1_error)

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyArrayObject *)__pyx_t_4);

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 134 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":132
 * 
 * 
 * cdef np.ndarray[double, ndim=1, mode='c'] gauss(np.ndarray[double, ndim=1, mode='c'] x,             # <<<<<<<<<<<<<<
 *                                                 double mean, double variance, double scale=2.0):
 *     return 1. / np.sqrt(2 * np.pi * variance) * np.exp(-(x - mean) ** 2 / (scale * variance))
 */

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.gauss", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 132 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":137
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] gaussian_smooth(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] x,
 *         np.ndarray[double, ndim=1, mode='c'] y, double width=0.05):
 */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11gaussian_smooth(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth(PyArrayObject *__pyx_v_x, PyArrayObject *__pyx_v_y, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth *__pyx_optional_args) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_v_width = ((double)0.05);
  PyArrayObject *__pyx_v_smoothed = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_x_slice = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_y_slice = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_weights = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_i
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_low_edge
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_high_edge
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_center
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_spread
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_smoothed
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_smoothed
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_weights
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_weights
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x_slice
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x_slice
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_y
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y_slice
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_y_slice
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_r = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_4 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_5;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_9;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_10;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_11;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_12;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_13;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_14;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_15;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_16;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_17 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("gaussian_smooth", 0);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_optional_args) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_width = __pyx_optional_args->width;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_smoothed.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_smoothed.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_smoothed.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_smoothed.rcbuffer = &__pyx_pybuffer_smoothed;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x_slice.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x_slice.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x_slice.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x_slice.rcbuffer = &__pyx_pybuffer_x_slice;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y_slice.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y_slice.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y_slice.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y_slice.rcbuffer = &__pyx_pybuffer_y_slice;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_weights.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_weights.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_weights.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_weights.rcbuffer = &__pyx_pybuffer_weights;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.rcbuffer = &__pyx_pybuffer_y;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y.rcbuffer->pybuffer, (PyObject*)__pyx_v_y, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.diminfo[0].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y.diminfo[0].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":145
 *         double center, spread
 * 
 *     smoothed = np.zeros_like(y)             # <<<<<<<<<<<<<<
 * 
 *     n = x.shape[0]
 */

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = NULL;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_2)) {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_2);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, ((PyObject *)__pyx_v_y)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_y));

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = ((PyArrayObject *)__pyx_t_1);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_smoothed.rcbuffer->pybuffer);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_smoothed.rcbuffer->pybuffer, (PyObject*)__pyx_t_4, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 < 0)) {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_smoothed.rcbuffer->pybuffer, (PyObject*)__pyx_v_smoothed, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS| PyBUF_WRITABLE, 1, 0, __pyx_stack) == -1)) {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_6); Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_8);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_6, __pyx_t_7, __pyx_t_8);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = __pyx_t_7 = __pyx_t_8 = 0;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_smoothed.diminfo[0].strides = __pyx_pybuffernd_smoothed.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_smoothed.diminfo[0].shape = __pyx_pybuffernd_smoothed.rcbuffer->pybuffer.shape[0];

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_smoothed = ((PyArrayObject *)__pyx_t_1);

#line 145 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":147
 *     smoothed = np.zeros_like(y)
 * 
 *     n = x.shape[0]             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         low_edge = binsearch(x, x[i] - width)
 */

#line 147 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n = (__pyx_v_x->dimensions[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":148
 * 
 *     n = x.shape[0]
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         low_edge = binsearch(x, x[i] - width)
 *         high_edge = binsearch(x, x[i] + width)
 */

#line 148 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_9 = __pyx_v_n;

#line 148 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = __pyx_t_9;

#line 148 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {

#line 148 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_11;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":149
 *     n = x.shape[0]
 *     for i in range(n):
 *         low_edge = binsearch(x, x[i] - width)             # <<<<<<<<<<<<<<
 *         high_edge = binsearch(x, x[i] + width)
 *         if high_edge - 1 == low_edge or high_edge == low_edge:
 */

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = __pyx_v_i;

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = -1;

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_5 = 0;

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 != -1)) {

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 149, __pyx_L1_error)

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 149 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_low_edge = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch(((PyArrayObject *)__pyx_v_x), ((*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_x.diminfo[0].strides)) - __pyx_v_width), 0);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":150
 *     for i in range(n):
 *         low_edge = binsearch(x, x[i] - width)
 *         high_edge = binsearch(x, x[i] + width)             # <<<<<<<<<<<<<<
 *         if high_edge - 1 == low_edge or high_edge == low_edge:
 *             smoothed[i] = y[i]
 */

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = __pyx_v_i;

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = -1;

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_x.diminfo[0].shape)) __pyx_t_5 = 0;

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 != -1)) {

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 150, __pyx_L1_error)

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 150 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_high_edge = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch(((PyArrayObject *)__pyx_v_x), ((*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_x.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_x.diminfo[0].strides)) + __pyx_v_width), 0);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":151
 *         low_edge = binsearch(x, x[i] - width)
 *         high_edge = binsearch(x, x[i] + width)
 *         if high_edge - 1 == low_edge or high_edge == low_edge:             # <<<<<<<<<<<<<<
 *             smoothed[i] = y[i]
 *             continue
 */

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14 = (((__pyx_v_high_edge - 1) == __pyx_v_low_edge) != 0);

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_14) {

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_13 = __pyx_t_14;

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L6_bool_binop_done;

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14 = ((__pyx_v_high_edge == __pyx_v_low_edge) != 0);

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __pyx_t_14;

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L6_bool_binop_done:;

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_13) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":152
 *         high_edge = binsearch(x, x[i] + width)
 *         if high_edge - 1 == low_edge or high_edge == low_edge:
 *             smoothed[i] = y[i]             # <<<<<<<<<<<<<<
 *             continue
 *         x_slice = x[low_edge:high_edge]
 */

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = __pyx_v_i;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = -1;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_y.diminfo[0].shape)) __pyx_t_5 = 0;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 != -1)) {

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 152, __pyx_L1_error)

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_15 = __pyx_v_i;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = -1;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_15 >= (size_t)__pyx_pybuffernd_smoothed.diminfo[0].shape)) __pyx_t_5 = 0;

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 != -1)) {

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 152, __pyx_L1_error)

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 152 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      *__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_smoothed.rcbuffer->pybuffer.buf, __pyx_t_15, __pyx_pybuffernd_smoothed.diminfo[0].strides) = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_y.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_y.diminfo[0].strides));

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":153
 *         if high_edge - 1 == low_edge or high_edge == low_edge:
 *             smoothed[i] = y[i]
 *             continue             # <<<<<<<<<<<<<<
 *         x_slice = x[low_edge:high_edge]
 *         y_slice = y[low_edge:high_edge]
 */

#line 153 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L3_continue;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":151
 *         low_edge = binsearch(x, x[i] - width)
 *         high_edge = binsearch(x, x[i] + width)
 *         if high_edge - 1 == low_edge or high_edge == low_edge:             # <<<<<<<<<<<<<<
 *             smoothed[i] = y[i]
 *             continue
 */

#line 151 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":154
 *             smoothed[i] = y[i]
 *             continue
 *         x_slice = x[low_edge:high_edge]             # <<<<<<<<<<<<<<
 *         y_slice = y[low_edge:high_edge]
 *         center = np.mean(x_slice)
 */

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_low_edge); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_high_edge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PySlice_New(__pyx_t_1, __pyx_t_3, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_x), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = ((PyArrayObject *)__pyx_t_3);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x_slice.rcbuffer->pybuffer);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x_slice.rcbuffer->pybuffer, (PyObject*)__pyx_t_4, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) {

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x_slice.rcbuffer->pybuffer, (PyObject*)__pyx_v_x_slice, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_6);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_8, __pyx_t_7, __pyx_t_6);

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_8 = __pyx_t_7 = __pyx_t_6 = 0;

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_x_slice.diminfo[0].strides = __pyx_pybuffernd_x_slice.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x_slice.diminfo[0].shape = __pyx_pybuffernd_x_slice.rcbuffer->pybuffer.shape[0];

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 154, __pyx_L1_error)

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = 0;

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_x_slice, ((PyArrayObject *)__pyx_t_3));

#line 154 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":155
 *             continue
 *         x_slice = x[low_edge:high_edge]
 *         y_slice = y[low_edge:high_edge]             # <<<<<<<<<<<<<<
 *         center = np.mean(x_slice)
 *         spread = np.var(x_slice)
 */

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_low_edge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_high_edge); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PySlice_New(__pyx_t_3, __pyx_t_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_y), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = ((PyArrayObject *)__pyx_t_2);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y_slice.rcbuffer->pybuffer);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y_slice.rcbuffer->pybuffer, (PyObject*)__pyx_t_4, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) {

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y_slice.rcbuffer->pybuffer, (PyObject*)__pyx_v_y_slice, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_6); Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_8);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_6, __pyx_t_7, __pyx_t_8);

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __pyx_t_7 = __pyx_t_8 = 0;

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_y_slice.diminfo[0].strides = __pyx_pybuffernd_y_slice.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y_slice.diminfo[0].shape = __pyx_pybuffernd_y_slice.rcbuffer->pybuffer.shape[0];

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 155, __pyx_L1_error)

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = 0;

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_y_slice, ((PyArrayObject *)__pyx_t_2));

#line 155 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":156
 *         x_slice = x[low_edge:high_edge]
 *         y_slice = y[low_edge:high_edge]
 *         center = np.mean(x_slice)             # <<<<<<<<<<<<<<
 *         spread = np.var(x_slice)
 *         weights = gauss(x_slice, center, spread)
 */

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_mean); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = NULL;

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_1)) {

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_1);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_3, function);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, ((PyObject *)__pyx_v_x_slice)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_x_slice));

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 156 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_center = __pyx_t_16;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":157
 *         y_slice = y[low_edge:high_edge]
 *         center = np.mean(x_slice)
 *         spread = np.var(x_slice)             # <<<<<<<<<<<<<<
 *         weights = gauss(x_slice, center, spread)
 *         smoothed[i] = ((y_slice * weights).sum() / weights.sum())
 */

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 157, __pyx_L1_error)

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_var); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = NULL;

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_3)) {

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_3);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, ((PyObject *)__pyx_v_x_slice)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_x_slice));

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 157, __pyx_L1_error)

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 157, __pyx_L1_error)

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 157 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_spread = __pyx_t_16;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":158
 *         center = np.mean(x_slice)
 *         spread = np.var(x_slice)
 *         weights = gauss(x_slice, center, spread)             # <<<<<<<<<<<<<<
 *         smoothed[i] = ((y_slice * weights).sum() / weights.sum())
 *     return smoothed
 */

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gauss(((PyArrayObject *)__pyx_v_x_slice), __pyx_v_center, __pyx_v_spread, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 158, __pyx_L1_error)

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_weights.rcbuffer->pybuffer);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_weights.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_2), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) {

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_weights.rcbuffer->pybuffer, (PyObject*)__pyx_v_weights, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_8); Py_XDECREF(__pyx_t_7); Py_XDECREF(__pyx_t_6);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_8, __pyx_t_7, __pyx_t_6);

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_8 = __pyx_t_7 = __pyx_t_6 = 0;

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_weights.diminfo[0].strides = __pyx_pybuffernd_weights.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_weights.diminfo[0].shape = __pyx_pybuffernd_weights.rcbuffer->pybuffer.shape[0];

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 158, __pyx_L1_error)

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_weights, ((PyArrayObject *)__pyx_t_2));

#line 158 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":159
 *         spread = np.var(x_slice)
 *         weights = gauss(x_slice, center, spread)
 *         smoothed[i] = ((y_slice * weights).sum() / weights.sum())             # <<<<<<<<<<<<<<
 *     return smoothed
 * 
 */

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyNumber_Multiply(((PyObject *)__pyx_v_y_slice), ((PyObject *)__pyx_v_weights)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sum); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = NULL;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_1)) {

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_3, function);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_weights), __pyx_n_s_sum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_17 = NULL;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_17)) {

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_17);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_17) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_17) : __Pyx_PyObject_CallNoArg(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = __pyx_v_i;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = -1;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_12 >= (size_t)__pyx_pybuffernd_smoothed.diminfo[0].shape)) __pyx_t_5 = 0;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_5 != -1)) {

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RaiseBufferIndexError(__pyx_t_5);

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 159, __pyx_L1_error)

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    *__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_smoothed.rcbuffer->pybuffer.buf, __pyx_t_12, __pyx_pybuffernd_smoothed.diminfo[0].strides) = __pyx_t_16;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L3_continue:;

#line 159 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":160
 *         weights = gauss(x_slice, center, spread)
 *         smoothed[i] = ((y_slice * weights).sum() / weights.sum())
 *     return smoothed             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 160 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 160 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_smoothed));

#line 160 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyArrayObject *)__pyx_v_smoothed);

#line 160 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":137
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] gaussian_smooth(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] x,
 *         np.ndarray[double, ndim=1, mode='c'] y, double width=0.05):
 */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_17);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_smoothed.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_weights.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x_slice.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y_slice.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.gaussian_smooth", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_smoothed.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_weights.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x_slice.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y_slice.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_smoothed);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_x_slice);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_y_slice);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_weights);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11gaussian_smooth(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_10gaussian_smooth[] = "gaussian_smooth(ndarray x, ndarray y, double width=0.05) -> ndarray";

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11gaussian_smooth(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_x = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_y = 0
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_width
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("gaussian_smooth (wrapper)", 0);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,0};

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("gaussian_smooth", 0, 2, 3, 1); 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 137, __pyx_L3_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_width);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gaussian_smooth") < 0)) __PYX_ERR(0, 137, __pyx_L3_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;
        default: 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = ((PyArrayObject *)values[0]);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_y = ((PyArrayObject *)values[1]);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[2]) {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_width = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_width == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 139, __pyx_L3_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_width = ((double)0.05);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gaussian_smooth", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 137, __pyx_L3_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.gaussian_smooth", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_5numpy_ndarray, 1, "x", 0))) __PYX_ERR(0, 138, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_5numpy_ndarray, 1, "y", 0))) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_r = 
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_10gaussian_smooth(__pyx_self, __pyx_v_x, __pyx_v_y, __pyx_v_width);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_10gaussian_smooth(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, PyArrayObject *__pyx_v_y, double __pyx_v_width) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_x
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_y
#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth __pyx_t_2;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("gaussian_smooth", 0);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_x.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y.pybuffer.buf = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_y.refcount = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.data = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.rcbuffer = &__pyx_pybuffer_y;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y.rcbuffer->pybuffer, (PyObject*)__pyx_v_y, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_y.diminfo[0].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y.diminfo[0].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[0];

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.width = __pyx_v_width;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth(__pyx_v_x, __pyx_v_y, 0, &__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.gaussian_smooth", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 137 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":163
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] smooth_leveled(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] xs,
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_13smooth_leveled(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyArrayObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled(PyArrayObject *__pyx_v_xs, PyArrayObject *__pyx_v_ys, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled *__pyx_optional_args) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_v_level = ((double)0.0);
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xs
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_xs
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_r = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth __pyx_t_4;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("smooth_leveled", 0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_optional_args) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_level = __pyx_optional_args->level;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.pybuffer.buf = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.refcount = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.data = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.rcbuffer = &__pyx_pybuffer_xs;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 163, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 163, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":166
 *         np.ndarray[double, ndim=1, mode='c'] xs,
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 *     if level == 0:             # <<<<<<<<<<<<<<
 *         return ys
 *     elif level == 1:
 */

#line 166 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((__pyx_v_level == 0.0) != 0);

#line 166 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":167
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 *     if level == 0:
 *         return ys             # <<<<<<<<<<<<<<
 *     elif level == 1:
 *         return sliding_mean[double](ys)
 */

#line 167 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 167 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(((PyObject *)__pyx_v_ys));

#line 167 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = ((PyArrayObject *)__pyx_v_ys);

#line 167 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":166
 *         np.ndarray[double, ndim=1, mode='c'] xs,
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 *     if level == 0:             # <<<<<<<<<<<<<<
 *         return ys
 *     elif level == 1:
 */

#line 166 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":168
 *     if level == 0:
 *         return ys
 *     elif level == 1:             # <<<<<<<<<<<<<<
 *         return sliding_mean[double](ys)
 *     elif level == 2:
 */

#line 168 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((__pyx_v_level == 1.0) != 0);

#line 168 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":169
 *         return ys
 *     elif level == 1:
 *         return sliding_mean[double](ys)             # <<<<<<<<<<<<<<
 *     elif level == 2:
 *         return sliding_mean[double](sliding_median(ys))
 */

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(((PyArrayObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 169, __pyx_L1_error)

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = ((PyArrayObject *)__pyx_t_2);

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

#line 169 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":168
 *     if level == 0:
 *         return ys
 *     elif level == 1:             # <<<<<<<<<<<<<<
 *         return sliding_mean[double](ys)
 *     elif level == 2:
 */

#line 168 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":170
 *     elif level == 1:
 *         return sliding_mean[double](ys)
 *     elif level == 2:             # <<<<<<<<<<<<<<
 *         return sliding_mean[double](sliding_median(ys))
 *     elif level == 3:
 */

#line 170 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((__pyx_v_level == 2.0) != 0);

#line 170 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":171
 *         return sliding_mean[double](ys)
 *     elif level == 2:
 *         return sliding_mean[double](sliding_median(ys))             # <<<<<<<<<<<<<<
 *     elif level == 3:
 *         return gaussian_smooth(xs, ys, 0.05)
 */

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_median(((PyArrayObject *)__pyx_v_ys)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(((PyArrayObject *)__pyx_t_2), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = ((PyArrayObject *)__pyx_t_3);

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 171 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":170
 *     elif level == 1:
 *         return sliding_mean[double](ys)
 *     elif level == 2:             # <<<<<<<<<<<<<<
 *         return sliding_mean[double](sliding_median(ys))
 *     elif level == 3:
 */

#line 170 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":172
 *     elif level == 2:
 *         return sliding_mean[double](sliding_median(ys))
 *     elif level == 3:             # <<<<<<<<<<<<<<
 *         return gaussian_smooth(xs, ys, 0.05)
 *     else:
 */

#line 172 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((__pyx_v_level == 3.0) != 0);

#line 172 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":173
 *         return sliding_mean[double](sliding_median(ys))
 *     elif level == 3:
 *         return gaussian_smooth(xs, ys, 0.05)             # <<<<<<<<<<<<<<
 *     else:
 *         return gaussian_smooth(xs, ys, level)
 */

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4.__pyx_n = 1;

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4.width = 0.05;

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth(((PyArrayObject *)__pyx_v_xs), ((PyArrayObject *)__pyx_v_ys), 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L1_error)

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = ((PyArrayObject *)__pyx_t_3);

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 173 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":172
 *     elif level == 2:
 *         return sliding_mean[double](sliding_median(ys))
 *     elif level == 3:             # <<<<<<<<<<<<<<
 *         return gaussian_smooth(xs, ys, 0.05)
 *     else:
 */

#line 172 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":175
 *         return gaussian_smooth(xs, ys, 0.05)
 *     else:
 *         return gaussian_smooth(xs, ys, level)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4.__pyx_n = 1;

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4.width = __pyx_v_level;

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_gaussian_smooth(((PyArrayObject *)__pyx_v_xs), ((PyArrayObject *)__pyx_v_ys), 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 175, __pyx_L1_error)

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = ((PyArrayObject *)__pyx_t_3);

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 175 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":163
 * 
 * 
 * cpdef np.ndarray[double, ndim=1, mode='c'] smooth_leveled(             # <<<<<<<<<<<<<<
 *         np.ndarray[double, ndim=1, mode='c'] xs,
 *         np.ndarray[double, ndim=1, mode='c'] ys, double level=0):
 */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.smooth_leveled", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_13smooth_leveled(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12smooth_leveled[] = "smooth_leveled(ndarray xs, ndarray ys, double level=0) -> ndarray";

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_13smooth_leveled(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_xs = 0
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_ys = 0
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_level
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("smooth_leveled (wrapper)", 0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xs,&__pyx_n_s_ys,&__pyx_n_s_level,0};

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_xs)) != 0)) kw_args--;
        else 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ys)) != 0)) kw_args--;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("smooth_leveled", 0, 2, 3, 1); 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 163, __pyx_L3_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_level);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "smooth_leveled") < 0)) __PYX_ERR(0, 163, __pyx_L3_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;
        default: 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_xs = ((PyArrayObject *)values[0]);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_ys = ((PyArrayObject *)values[1]);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[2]) {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_level = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_level == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 165, __pyx_L3_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_level = ((double)0.0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("smooth_leveled", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 163, __pyx_L3_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.smooth_leveled", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_xs), __pyx_ptype_5numpy_ndarray, 1, "xs", 0))) __PYX_ERR(0, 164, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_ys), __pyx_ptype_5numpy_ndarray, 1, "ys", 0))) __PYX_ERR(0, 165, __pyx_L1_error)
  __pyx_r = 
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12smooth_leveled(__pyx_self, __pyx_v_xs, __pyx_v_ys, __pyx_v_level);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12smooth_leveled(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_xs, PyArrayObject *__pyx_v_ys, double __pyx_v_level) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xs
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_xs
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled __pyx_t_2;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("smooth_leveled", 0);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.pybuffer.buf = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.refcount = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.data = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.rcbuffer = &__pyx_pybuffer_xs;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 163, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 163, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.level = __pyx_v_level;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled(__pyx_v_xs, __pyx_v_ys, 0, &__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.smooth_leveled", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 163 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":188
 *         public double total_distance
 * 
 *     def __init__(self, first_minimum, maximum, second_minimum,             # <<<<<<<<<<<<<<
 *                  first_minimum_index, maximum_index, second_minimum_index):
 *         self.first_minimum = first_minimum
 */

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_first_minimum = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_maximum = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_second_minimum = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_first_minimum_index = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_maximum_index = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_second_minimum_index = 0
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_first_minimum,&__pyx_n_s_maximum,&__pyx_n_s_second_minimum,&__pyx_n_s_first_minimum_index,&__pyx_n_s_maximum_index,&__pyx_n_s_second_minimum_index,0};

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[6] = {0,0,0,0,0,0};

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  6: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  5: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_first_minimum)) != 0)) kw_args--;
        else 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_maximum)) != 0)) kw_args--;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 1); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_second_minimum)) != 0)) kw_args--;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 2); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  3:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_first_minimum_index)) != 0)) kw_args--;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 3); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  4:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_maximum_index)) != 0)) kw_args--;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 4); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  5:

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_second_minimum_index)) != 0)) kw_args--;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 5); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_first_minimum = values[0];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_maximum = values[1];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_second_minimum = values[2];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_first_minimum_index = values[3];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_maximum_index = values[4];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_second_minimum_index = values[5];

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 188, __pyx_L3_error)

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return -1;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary___init__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), __pyx_v_first_minimum, __pyx_v_maximum, __pyx_v_second_minimum, __pyx_v_first_minimum_index, __pyx_v_maximum_index, __pyx_v_second_minimum_index);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_first_minimum, PyObject *__pyx_v_maximum, PyObject *__pyx_v_second_minimum, PyObject *__pyx_v_first_minimum_index, PyObject *__pyx_v_maximum_index, PyObject *__pyx_v_second_minimum_index) {

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":190
 *     def __init__(self, first_minimum, maximum, second_minimum,
 *                  first_minimum_index, maximum_index, second_minimum_index):
 *         self.first_minimum = first_minimum             # <<<<<<<<<<<<<<
 *         self.maximum = maximum
 *         self.second_minimum = second_minimum
 */

#line 190 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_first_minimum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 190, __pyx_L1_error)

#line 190 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_minimum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":191
 *                  first_minimum_index, maximum_index, second_minimum_index):
 *         self.first_minimum = first_minimum
 *         self.maximum = maximum             # <<<<<<<<<<<<<<
 *         self.second_minimum = second_minimum
 *         self.first_minimum_index = first_minimum_index
 */

#line 191 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_maximum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 191, __pyx_L1_error)

#line 191 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->maximum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":192
 *         self.first_minimum = first_minimum
 *         self.maximum = maximum
 *         self.second_minimum = second_minimum             # <<<<<<<<<<<<<<
 *         self.first_minimum_index = first_minimum_index
 *         self.maximum_index = maximum_index
 */

#line 192 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_second_minimum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 192, __pyx_L1_error)

#line 192 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_minimum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":193
 *         self.maximum = maximum
 *         self.second_minimum = second_minimum
 *         self.first_minimum_index = first_minimum_index             # <<<<<<<<<<<<<<
 *         self.maximum_index = maximum_index
 *         self.second_minimum_index = second_minimum_index
 */

#line 193 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_first_minimum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 193, __pyx_L1_error)

#line 193 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_minimum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":194
 *         self.second_minimum = second_minimum
 *         self.first_minimum_index = first_minimum_index
 *         self.maximum_index = maximum_index             # <<<<<<<<<<<<<<
 *         self.second_minimum_index = second_minimum_index
 *         self.total_distance = self.compute_distance()
 */

#line 194 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_maximum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 194, __pyx_L1_error)

#line 194 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->maximum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":195
 *         self.first_minimum_index = first_minimum_index
 *         self.maximum_index = maximum_index
 *         self.second_minimum_index = second_minimum_index             # <<<<<<<<<<<<<<
 *         self.total_distance = self.compute_distance()
 * 
 */

#line 195 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_second_minimum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 195, __pyx_L1_error)

#line 195 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_minimum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":196
 *         self.maximum_index = maximum_index
 *         self.second_minimum_index = second_minimum_index
 *         self.total_distance = self.compute_distance()             # <<<<<<<<<<<<<<
 * 
 *     cdef double compute_distance(self):
 */

#line 196 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->total_distance = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self->__pyx_vtab)->compute_distance(__pyx_v_self);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":188
 *         public double total_distance
 * 
 *     def __init__(self, first_minimum, maximum, second_minimum,             # <<<<<<<<<<<<<<
 *                  first_minimum_index, maximum_index, second_minimum_index):
 *         self.first_minimum = first_minimum
 */

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 188 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":198
 *         self.total_distance = self.compute_distance()
 * 
 *     cdef double compute_distance(self):             # <<<<<<<<<<<<<<
 *         return (self.maximum - self.first_minimum) + (self.maximum - self.second_minimum)
 * 
 */

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_compute_distance(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("compute_distance", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":199
 * 
 *     cdef double compute_distance(self):
 *         return (self.maximum - self.first_minimum) + (self.maximum - self.second_minimum)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 199 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((__pyx_v_self->maximum - __pyx_v_self->first_minimum) + (__pyx_v_self->maximum - __pyx_v_self->second_minimum));

#line 199 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":198
 *         self.total_distance = self.compute_distance()
 * 
 *     cdef double compute_distance(self):             # <<<<<<<<<<<<<<
 *         return (self.maximum - self.first_minimum) + (self.maximum - self.second_minimum)
 * 
 */

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 198 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":201
 *         return (self.maximum - self.first_minimum) + (self.maximum - self.second_minimum)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,
 */

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_3__repr__(PyObject *__pyx_v_self); /*proto*/

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_3__repr__(PyObject *__pyx_v_self) {

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_2__repr__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":202
 * 
 *     def __repr__(self):
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (             # <<<<<<<<<<<<<<
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,
 *             self.total_distance)
 */

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":203
 *     def __repr__(self):
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,             # <<<<<<<<<<<<<<
 *             self.total_distance)
 * 
 */

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_minimum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->maximum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->second_minimum_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 203, __pyx_L1_error)

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":204
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,
 *             self.total_distance)             # <<<<<<<<<<<<<<
 * 
 *     def split(self, profile):
 */

#line 204 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)

#line 204 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":203
 *     def __repr__(self):
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,             # <<<<<<<<<<<<<<
 *             self.total_distance)
 * 
 */

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 203, __pyx_L1_error)

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 203 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":202
 * 
 *     def __repr__(self):
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (             # <<<<<<<<<<<<<<
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,
 *             self.total_distance)
 */

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_PeakBoundary_0_2f_0_2f_0_2f_0_3e, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_4;

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 202 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":201
 *         return (self.maximum - self.first_minimum) + (self.maximum - self.second_minimum)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "PeakBoundary(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_minimum_index, self.maximum_index, self.second_minimum_index,
 */

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 201 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":206
 *             self.total_distance)
 * 
 *     def split(self, profile):             # <<<<<<<<<<<<<<
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 */

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_5split(PyObject *__pyx_v_self, PyObject *__pyx_v_profile); /*proto*/

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_4split[] = "PeakBoundary.split(self, profile)";

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_5split(PyObject *__pyx_v_self, PyObject *__pyx_v_profile) {

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split (wrapper)", 0);
  __pyx_r = 
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_4split(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_profile));

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_4split(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_profile) {
  PyObject *__pyx_v_first_minimum = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_second_minimum = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_prefix = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_peak = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_suffix = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_cases = NULL
#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_5;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":207
 * 
 *     def split(self, profile):
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]             # <<<<<<<<<<<<<<
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 *         if first_minimum == 0:
 */

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_profile, __pyx_n_s_nodes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_find_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->first_minimum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = NULL;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_4)) {

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_first_minimum = __pyx_t_3;

#line 207 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":208
 *     def split(self, profile):
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]             # <<<<<<<<<<<<<<
 *         if first_minimum == 0:
 *             first_minimum += 1
 */

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_profile, __pyx_n_s_nodes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_find_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->second_minimum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = NULL;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_4)) {

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_3, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_second_minimum = __pyx_t_2;

#line 208 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":209
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 *         if first_minimum == 0:             # <<<<<<<<<<<<<<
 *             first_minimum += 1
 *         prefix = profile[:first_minimum]
 */

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_first_minimum, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 209, __pyx_L1_error)

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 209, __pyx_L1_error)

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_5) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":210
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 *         if first_minimum == 0:
 *             first_minimum += 1             # <<<<<<<<<<<<<<
 *         prefix = profile[:first_minimum]
 *         peak = profile[(first_minimum - 1):second_minimum + 1]
 */

#line 210 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_first_minimum, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)

#line 210 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 210 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_first_minimum, __pyx_t_2);

#line 210 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":209
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 *         if first_minimum == 0:             # <<<<<<<<<<<<<<
 *             first_minimum += 1
 *         prefix = profile[:first_minimum]
 */

#line 209 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":211
 *         if first_minimum == 0:
 *             first_minimum += 1
 *         prefix = profile[:first_minimum]             # <<<<<<<<<<<<<<
 *         peak = profile[(first_minimum - 1):second_minimum + 1]
 *         suffix = profile[second_minimum + 1:]
 */

#line 211 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_profile, 0, 0, NULL, &__pyx_v_first_minimum, NULL, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)

#line 211 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 211 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_prefix = __pyx_t_2;

#line 211 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":212
 *             first_minimum += 1
 *         prefix = profile[:first_minimum]
 *         peak = profile[(first_minimum - 1):second_minimum + 1]             # <<<<<<<<<<<<<<
 *         suffix = profile[second_minimum + 1:]
 *         cases = LCMSFeature(prefix), LCMSFeature(peak), LCMSFeature(suffix)
 */

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyInt_SubtractObjC(__pyx_v_first_minimum, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 212, __pyx_L1_error)

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_v_second_minimum, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 212, __pyx_L1_error)

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_profile, 0, 0, &__pyx_t_2, &__pyx_t_3, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_peak = __pyx_t_1;

#line 212 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":213
 *         prefix = profile[:first_minimum]
 *         peak = profile[(first_minimum - 1):second_minimum + 1]
 *         suffix = profile[second_minimum + 1:]             # <<<<<<<<<<<<<<
 *         cases = LCMSFeature(prefix), LCMSFeature(peak), LCMSFeature(suffix)
 *         return cases
 */

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_second_minimum, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_profile, 0, 0, &__pyx_t_1, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_suffix = __pyx_t_3;

#line 213 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":214
 *         peak = profile[(first_minimum - 1):second_minimum + 1]
 *         suffix = profile[second_minimum + 1:]
 *         cases = LCMSFeature(prefix), LCMSFeature(peak), LCMSFeature(suffix)             # <<<<<<<<<<<<<<
 *         return cases
 * 
 */

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __pyx_v_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __pyx_v_peak); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __pyx_v_suffix); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 214, __pyx_L1_error)

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_cases = ((PyObject*)__pyx_t_4);

#line 214 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":215
 *         suffix = profile[second_minimum + 1:]
 *         cases = LCMSFeature(prefix), LCMSFeature(peak), LCMSFeature(suffix)
 *         return cases             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 215 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 215 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_cases);

#line 215 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_cases;

#line 215 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":206
 *             self.total_distance)
 * 
 *     def split(self, profile):             # <<<<<<<<<<<<<<
 *         first_minimum = profile.nodes.find_time(self.first_minimum_index)[1]
 *         second_minimum = profile.nodes.find_time(self.second_minimum_index)[1]
 */

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.split", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_first_minimum);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_second_minimum);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_prefix);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_peak);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_suffix);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_cases);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 206 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":180
 * cdef class PeakBoundary(object):
 *     cdef:
 *         public double first_minimum             # <<<<<<<<<<<<<<
 *         public double maximum
 *         public double second_minimum
 */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_1__get__(PyObject *__pyx_v_self) {

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_minimum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.first_minimum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 180, __pyx_L1_error)

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_minimum = __pyx_t_1;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.first_minimum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 180 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":181
 *     cdef:
 *         public double first_minimum
 *         public double maximum             # <<<<<<<<<<<<<<
 *         public double second_minimum
 *         public double first_minimum_index
 */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_1__get__(PyObject *__pyx_v_self) {

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->maximum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.maximum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 181, __pyx_L1_error)

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->maximum = __pyx_t_1;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.maximum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 181 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":182
 *         public double first_minimum
 *         public double maximum
 *         public double second_minimum             # <<<<<<<<<<<<<<
 *         public double first_minimum_index
 *         public double maximum_index
 */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_1__get__(PyObject *__pyx_v_self) {

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->second_minimum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.second_minimum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 182, __pyx_L1_error)

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_minimum = __pyx_t_1;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.second_minimum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 182 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":183
 *         public double maximum
 *         public double second_minimum
 *         public double first_minimum_index             # <<<<<<<<<<<<<<
 *         public double maximum_index
 *         public double second_minimum_index
 */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_1__get__(PyObject *__pyx_v_self) {

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_minimum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.first_minimum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L1_error)

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_minimum_index = __pyx_t_1;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.first_minimum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 183 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":184
 *         public double second_minimum
 *         public double first_minimum_index
 *         public double maximum_index             # <<<<<<<<<<<<<<
 *         public double second_minimum_index
 *         public double total_distance
 */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_1__get__(PyObject *__pyx_v_self) {

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->maximum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.maximum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 184, __pyx_L1_error)

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->maximum_index = __pyx_t_1;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.maximum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 184 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":185
 *         public double first_minimum_index
 *         public double maximum_index
 *         public double second_minimum_index             # <<<<<<<<<<<<<<
 *         public double total_distance
 * 
 */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_1__get__(PyObject *__pyx_v_self) {

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->second_minimum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.second_minimum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 185, __pyx_L1_error)

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_minimum_index = __pyx_t_1;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.second_minimum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 185 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":186
 *         public double maximum_index
 *         public double second_minimum_index
 *         public double total_distance             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, first_minimum, maximum, second_minimum,
 */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_1__get__(PyObject *__pyx_v_self) {

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.total_distance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v_value) {

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L1_error)

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->total_distance = __pyx_t_1;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.total_distance.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 186 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_6__reduce_cython__[] = "PeakBoundary.__reduce_cython__(self)";

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_6__reduce_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_6__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self) {
  PyObject *__pyx_v_state = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v__dict = 0
#line 1 "(tree fragment)"
;
  int __pyx_v_use_setstate
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_2 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_3 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_9;

#line 1 "(tree fragment)"
  int __pyx_t_10;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.first_minimum, self.first_minimum_index, self.maximum, self.maximum_index, self.second_minimum, self.second_minimum_index, self.total_distance)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */

#line 5 "(tree fragment)"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_minimum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->first_minimum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_2);

#line 5 "(tree fragment)"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->maximum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_3);

#line 5 "(tree fragment)"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->maximum_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->second_minimum); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_5);

#line 5 "(tree fragment)"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->second_minimum_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_6);

#line 5 "(tree fragment)"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_7);

#line 5 "(tree fragment)"
  __pyx_t_8 = PyTuple_New(7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_8);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_2);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_3);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_3);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_4);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_t_4);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_5);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_5);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_6);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 5, __pyx_t_6);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_7);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 6, __pyx_t_7);

#line 5 "(tree fragment)"
  __pyx_t_1 = 0;

#line 5 "(tree fragment)"
  __pyx_t_2 = 0;

#line 5 "(tree fragment)"
  __pyx_t_3 = 0;

#line 5 "(tree fragment)"
  __pyx_t_4 = 0;

#line 5 "(tree fragment)"
  __pyx_t_5 = 0;

#line 5 "(tree fragment)"
  __pyx_t_6 = 0;

#line 5 "(tree fragment)"
  __pyx_t_7 = 0;

#line 5 "(tree fragment)"
  __pyx_v_state = ((PyObject*)__pyx_t_8);

#line 5 "(tree fragment)"
  __pyx_t_8 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.first_minimum, self.first_minimum_index, self.maximum, self.maximum_index, self.second_minimum, self.second_minimum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */

#line 6 "(tree fragment)"
  __pyx_t_8 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_8);

#line 6 "(tree fragment)"
  __pyx_v__dict = __pyx_t_8;

#line 6 "(tree fragment)"
  __pyx_t_8 = 0;

  /* "(tree fragment)":7
 *     state = (self.first_minimum, self.first_minimum_index, self.maximum, self.maximum_index, self.second_minimum, self.second_minimum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
  __pyx_t_9 = (__pyx_v__dict != Py_None);

#line 7 "(tree fragment)"
  __pyx_t_10 = (__pyx_t_9 != 0);

#line 7 "(tree fragment)"
  if (__pyx_t_10) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */

#line 8 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 8 "(tree fragment)"
    __Pyx_INCREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v__dict);

#line 8 "(tree fragment)"
    __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 8 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 8 "(tree fragment)"
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_7));

#line 8 "(tree fragment)"
    __pyx_t_7 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */

#line 9 "(tree fragment)"
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.first_minimum, self.first_minimum_index, self.maximum, self.maximum_index, self.second_minimum, self.second_minimum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
    goto __pyx_L3;

#line 7 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, None), state
 */

#line 11 "(tree fragment)"
  /*else*/ {

#line 11 "(tree fragment)"
    __pyx_v_use_setstate = 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  __pyx_t_10 = (__pyx_v_use_setstate != 0);

#line 12 "(tree fragment)"
  if (__pyx_t_10) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, state)
 */

#line 13 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 13 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_pyx_unpickle_PeakBoundary); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 13 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 13 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_256207781);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_256207781);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_256207781);

#line 13 "(tree fragment)"
    __Pyx_INCREF(Py_None);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(Py_None);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 2, Py_None);

#line 13 "(tree fragment)"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_7);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_8);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_8);

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);

#line 13 "(tree fragment)"
    __pyx_t_7 = 0;

#line 13 "(tree fragment)"
    __pyx_t_8 = 0;

#line 13 "(tree fragment)"
    __pyx_r = __pyx_t_6;

#line 13 "(tree fragment)"
    __pyx_t_6 = 0;

#line 13 "(tree fragment)"
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, None), state
 *     else:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PeakBoundary__set_state(self, __pyx_state)
 */

#line 15 "(tree fragment)"
  /*else*/ {

#line 15 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 15 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_PeakBoundary); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 15 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 15 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_256207781);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_256207781);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_256207781);

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_state);

#line 15 "(tree fragment)"
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_6);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_8);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);

#line 15 "(tree fragment)"
    __pyx_t_6 = 0;

#line 15 "(tree fragment)"
    __pyx_t_8 = 0;

#line 15 "(tree fragment)"
    __pyx_r = __pyx_t_7;

#line 15 "(tree fragment)"
    __pyx_t_7 = 0;

#line 15 "(tree fragment)"
    goto __pyx_L0;

#line 15 "(tree fragment)"
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_2);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_3);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v_state);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v__dict);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PeakBoundary__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
/* Python wrapper */

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 16 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_8__setstate_cython__[] = "PeakBoundary.__setstate_cython__(self, __pyx_state)";

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 16 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_8__setstate_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_8__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 16 "(tree fragment)"
  int __pyx_lineno = 0;

#line 16 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 16 "(tree fragment)"
  int __pyx_clineno = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PeakBoundary__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */

#line 17 "(tree fragment)"
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_PeakBoundary__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PeakBoundary, (type(self), 0xf456ba5, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PeakBoundary__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 16 "(tree fragment)"
  goto __pyx_L0;

#line 16 "(tree fragment)"
  __pyx_L1_error:;

#line 16 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.PeakBoundary.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "(tree fragment)"
  __pyx_r = NULL;

#line 16 "(tree fragment)"
  __pyx_L0:;

#line 16 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":228
 *         public double total_distance
 * 
 *     def __init__(self, first_maximum, minimum, second_maximum, first_maximum_index,             # <<<<<<<<<<<<<<
 *                  minimum_index, second_maximum_index):
 *         self.first_maximum = first_maximum
 */

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_first_maximum = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_minimum = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_second_maximum = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_first_maximum_index = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_minimum_index = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_second_maximum_index = 0
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_first_maximum,&__pyx_n_s_minimum,&__pyx_n_s_second_maximum,&__pyx_n_s_first_maximum_index,&__pyx_n_s_minimum_index,&__pyx_n_s_second_maximum_index,0};

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[6] = {0,0,0,0,0,0};

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  6: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  5: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_first_maximum)) != 0)) kw_args--;
        else 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_minimum)) != 0)) kw_args--;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 1); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_second_maximum)) != 0)) kw_args--;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 2); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  3:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_first_maximum_index)) != 0)) kw_args--;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 3); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  4:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_minimum_index)) != 0)) kw_args--;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 4); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  5:

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_second_maximum_index)) != 0)) kw_args--;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, 5); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_first_maximum = values[0];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_minimum = values[1];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_second_maximum = values[2];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_first_maximum_index = values[3];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_minimum_index = values[4];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_second_maximum_index = values[5];

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 228, __pyx_L3_error)

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return -1;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint___init__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), __pyx_v_first_maximum, __pyx_v_minimum, __pyx_v_second_maximum, __pyx_v_first_maximum_index, __pyx_v_minimum_index, __pyx_v_second_maximum_index);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_first_maximum, PyObject *__pyx_v_minimum, PyObject *__pyx_v_second_maximum, PyObject *__pyx_v_first_maximum_index, PyObject *__pyx_v_minimum_index, PyObject *__pyx_v_second_maximum_index) {

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":230
 *     def __init__(self, first_maximum, minimum, second_maximum, first_maximum_index,
 *                  minimum_index, second_maximum_index):
 *         self.first_maximum = first_maximum             # <<<<<<<<<<<<<<
 *         self.minimum = minimum
 *         self.second_maximum = second_maximum
 */

#line 230 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_first_maximum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 230, __pyx_L1_error)

#line 230 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":231
 *                  minimum_index, second_maximum_index):
 *         self.first_maximum = first_maximum
 *         self.minimum = minimum             # <<<<<<<<<<<<<<
 *         self.second_maximum = second_maximum
 *         self.first_maximum_index = first_maximum_index
 */

#line 231 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_minimum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 231, __pyx_L1_error)

#line 231 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":232
 *         self.first_maximum = first_maximum
 *         self.minimum = minimum
 *         self.second_maximum = second_maximum             # <<<<<<<<<<<<<<
 *         self.first_maximum_index = first_maximum_index
 *         self.minimum_index = minimum_index
 */

#line 232 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_second_maximum); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 232, __pyx_L1_error)

#line 232 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_maximum = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":233
 *         self.minimum = minimum
 *         self.second_maximum = second_maximum
 *         self.first_maximum_index = first_maximum_index             # <<<<<<<<<<<<<<
 *         self.minimum_index = minimum_index
 *         self.second_maximum_index = second_maximum_index
 */

#line 233 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_first_maximum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 233, __pyx_L1_error)

#line 233 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":234
 *         self.second_maximum = second_maximum
 *         self.first_maximum_index = first_maximum_index
 *         self.minimum_index = minimum_index             # <<<<<<<<<<<<<<
 *         self.second_maximum_index = second_maximum_index
 *         self.total_distance = self.compute_distance()
 */

#line 234 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_minimum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 234, __pyx_L1_error)

#line 234 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":235
 *         self.first_maximum_index = first_maximum_index
 *         self.minimum_index = minimum_index
 *         self.second_maximum_index = second_maximum_index             # <<<<<<<<<<<<<<
 *         self.total_distance = self.compute_distance()
 * 
 */

#line 235 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_second_maximum_index); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 235, __pyx_L1_error)

#line 235 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_maximum_index = __pyx_t_1;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":236
 *         self.minimum_index = minimum_index
 *         self.second_maximum_index = second_maximum_index
 *         self.total_distance = self.compute_distance()             # <<<<<<<<<<<<<<
 * 
 *     cdef double compute_distance(self):
 */

#line 236 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->total_distance = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self->__pyx_vtab)->compute_distance(__pyx_v_self);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":228
 *         public double total_distance
 * 
 *     def __init__(self, first_maximum, minimum, second_maximum, first_maximum_index,             # <<<<<<<<<<<<<<
 *                  minimum_index, second_maximum_index):
 *         self.first_maximum = first_maximum
 */

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 228 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":238
 *         self.total_distance = self.compute_distance()
 * 
 *     cdef double compute_distance(self):             # <<<<<<<<<<<<<<
 *         return (self.first_maximum - self.minimum) + (self.second_maximum - self.minimum)
 * 
 */

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_compute_distance(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("compute_distance", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":239
 * 
 *     cdef double compute_distance(self):
 *         return (self.first_maximum - self.minimum) + (self.second_maximum - self.minimum)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 239 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((__pyx_v_self->first_maximum - __pyx_v_self->minimum) + (__pyx_v_self->second_maximum - __pyx_v_self->minimum));

#line 239 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":238
 *         self.total_distance = self.compute_distance()
 * 
 *     cdef double compute_distance(self):             # <<<<<<<<<<<<<<
 *         return (self.first_maximum - self.minimum) + (self.second_maximum - self.minimum)
 * 
 */

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 238 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":241
 *         return (self.first_maximum - self.minimum) + (self.second_maximum - self.minimum)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 */

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_3__repr__(PyObject *__pyx_v_self); /*proto*/

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_3__repr__(PyObject *__pyx_v_self) {

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_2__repr__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":242
 * 
 *     def __repr__(self):
 *         return "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)" % (             # <<<<<<<<<<<<<<
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 * 
 */

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":243
 *     def __repr__(self):
 *         return "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)             # <<<<<<<<<<<<<<
 * 
 *     def split(self, profile):
 */

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_maximum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->minimum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->second_maximum_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 243, __pyx_L1_error)

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 243, __pyx_L1_error)

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 243, __pyx_L1_error)

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 243 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":242
 * 
 *     def __repr__(self):
 *         return "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)" % (             # <<<<<<<<<<<<<<
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 * 
 */

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_ValleyPoint_0_2f_0_2f_0_2f_0_3e, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L1_error)

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_4;

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 242 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":241
 *         return (self.first_maximum - self.minimum) + (self.second_maximum - self.minimum)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "ValleyPoint(%0.2f, %0.2f, %0.2f, %0.3e)" % (
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 */

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 241 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":245
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 * 
 *     def split(self, profile):             # <<<<<<<<<<<<<<
 *         index = profile.nodes.find_time(self.minimum_index)[1]
 *         before = LCMSFeature(profile[:index])
 */

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_5split(PyObject *__pyx_v_self, PyObject *__pyx_v_profile); /*proto*/

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_4split[] = "ValleyPoint.split(self, profile)";

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_5split(PyObject *__pyx_v_self, PyObject *__pyx_v_profile) {

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split (wrapper)", 0);
  __pyx_r = 
#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_4split(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_profile));

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_4split(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_profile) {
  PyObject *__pyx_v_index = NULL
#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *__pyx_v_before = NULL
#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *__pyx_v_after = NULL
#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":246
 * 
 *     def split(self, profile):
 *         index = profile.nodes.find_time(self.minimum_index)[1]             # <<<<<<<<<<<<<<
 *         before = LCMSFeature(profile[:index])
 *         after = LCMSFeature(profile[index:])
 */

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_profile, __pyx_n_s_nodes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_find_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->minimum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = NULL;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_4)) {

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_index = __pyx_t_3;

#line 246 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":247
 *     def split(self, profile):
 *         index = profile.nodes.find_time(self.minimum_index)[1]
 *         before = LCMSFeature(profile[:index])             # <<<<<<<<<<<<<<
 *         after = LCMSFeature(profile[index:])
 *         return before, after
 */

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_profile, 0, 0, NULL, &__pyx_v_index, NULL, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_before = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *)__pyx_t_1);

#line 247 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":248
 *         index = profile.nodes.find_time(self.minimum_index)[1]
 *         before = LCMSFeature(profile[:index])
 *         after = LCMSFeature(profile[index:])             # <<<<<<<<<<<<<<
 *         return before, after
 * 
 */

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_profile, 0, 0, &__pyx_v_index, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_after = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature *)__pyx_t_3);

#line 248 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":249
 *         before = LCMSFeature(profile[:index])
 *         after = LCMSFeature(profile[index:])
 *         return before, after             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_before));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_before));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_before));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_after));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_after));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_after));

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_3;

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 249 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":245
 *             self.first_maximum_index, self.minimum_index, self.second_maximum_index, self.total_distance)
 * 
 *     def split(self, profile):             # <<<<<<<<<<<<<<
 *         index = profile.nodes.find_time(self.minimum_index)[1]
 *         before = LCMSFeature(profile[:index])
 */

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.split", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_index);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_before);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_after);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 245 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":220
 * cdef class ValleyPoint(object):
 *     cdef:
 *         public double first_maximum             # <<<<<<<<<<<<<<
 *         public double minimum
 *         public double second_maximum
 */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_1__get__(PyObject *__pyx_v_self) {

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_maximum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.first_maximum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 220, __pyx_L1_error)

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum = __pyx_t_1;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.first_maximum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 220 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":221
 *     cdef:
 *         public double first_maximum
 *         public double minimum             # <<<<<<<<<<<<<<
 *         public double second_maximum
 *         public double first_maximum_index
 */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_1__get__(PyObject *__pyx_v_self) {

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->minimum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.minimum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 221, __pyx_L1_error)

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum = __pyx_t_1;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.minimum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 221 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":222
 *         public double first_maximum
 *         public double minimum
 *         public double second_maximum             # <<<<<<<<<<<<<<
 *         public double first_maximum_index
 *         public double minimum_index
 */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_1__get__(PyObject *__pyx_v_self) {

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->second_maximum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.second_maximum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 222, __pyx_L1_error)

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_maximum = __pyx_t_1;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.second_maximum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 222 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":223
 *         public double minimum
 *         public double second_maximum
 *         public double first_maximum_index             # <<<<<<<<<<<<<<
 *         public double minimum_index
 *         public double second_maximum_index
 */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_1__get__(PyObject *__pyx_v_self) {

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_maximum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 223, __pyx_L1_error)

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.first_maximum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 223, __pyx_L1_error)

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum_index = __pyx_t_1;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.first_maximum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 223 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":224
 *         public double second_maximum
 *         public double first_maximum_index
 *         public double minimum_index             # <<<<<<<<<<<<<<
 *         public double second_maximum_index
 *         public double total_distance
 */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_1__get__(PyObject *__pyx_v_self) {

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->minimum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.minimum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 224, __pyx_L1_error)

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum_index = __pyx_t_1;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.minimum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 224 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":225
 *         public double first_maximum_index
 *         public double minimum_index
 *         public double second_maximum_index             # <<<<<<<<<<<<<<
 *         public double total_distance
 * 
 */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_1__get__(PyObject *__pyx_v_self) {

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->second_maximum_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 225, __pyx_L1_error)

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.second_maximum_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 225, __pyx_L1_error)

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_maximum_index = __pyx_t_1;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.second_maximum_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 225 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":226
 *         public double minimum_index
 *         public double second_maximum_index
 *         public double total_distance             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, first_maximum, minimum, second_maximum, first_maximum_index,
 */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_1__get__(PyObject *__pyx_v_self) {

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 226, __pyx_L1_error)

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.total_distance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v_value) {

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_1;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 226, __pyx_L1_error)

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->total_distance = __pyx_t_1;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.total_distance.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 226 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_6__reduce_cython__[] = "ValleyPoint.__reduce_cython__(self)";

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_6__reduce_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_6__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self) {
  PyObject *__pyx_v_state = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v__dict = 0
#line 1 "(tree fragment)"
;
  int __pyx_v_use_setstate
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_2 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_3 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_9;

#line 1 "(tree fragment)"
  int __pyx_t_10;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.first_maximum, self.first_maximum_index, self.minimum, self.minimum_index, self.second_maximum, self.second_maximum_index, self.total_distance)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */

#line 5 "(tree fragment)"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->first_maximum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->first_maximum_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_2);

#line 5 "(tree fragment)"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->minimum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_3);

#line 5 "(tree fragment)"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->minimum_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->second_maximum); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_5);

#line 5 "(tree fragment)"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->second_maximum_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_6);

#line 5 "(tree fragment)"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->total_distance); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_7);

#line 5 "(tree fragment)"
  __pyx_t_8 = PyTuple_New(7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_8);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_2);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_3);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_3);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_4);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_t_4);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_5);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_5);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_6);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 5, __pyx_t_6);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_7);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_8, 6, __pyx_t_7);

#line 5 "(tree fragment)"
  __pyx_t_1 = 0;

#line 5 "(tree fragment)"
  __pyx_t_2 = 0;

#line 5 "(tree fragment)"
  __pyx_t_3 = 0;

#line 5 "(tree fragment)"
  __pyx_t_4 = 0;

#line 5 "(tree fragment)"
  __pyx_t_5 = 0;

#line 5 "(tree fragment)"
  __pyx_t_6 = 0;

#line 5 "(tree fragment)"
  __pyx_t_7 = 0;

#line 5 "(tree fragment)"
  __pyx_v_state = ((PyObject*)__pyx_t_8);

#line 5 "(tree fragment)"
  __pyx_t_8 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.first_maximum, self.first_maximum_index, self.minimum, self.minimum_index, self.second_maximum, self.second_maximum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */

#line 6 "(tree fragment)"
  __pyx_t_8 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_8);

#line 6 "(tree fragment)"
  __pyx_v__dict = __pyx_t_8;

#line 6 "(tree fragment)"
  __pyx_t_8 = 0;

  /* "(tree fragment)":7
 *     state = (self.first_maximum, self.first_maximum_index, self.minimum, self.minimum_index, self.second_maximum, self.second_maximum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
  __pyx_t_9 = (__pyx_v__dict != Py_None);

#line 7 "(tree fragment)"
  __pyx_t_10 = (__pyx_t_9 != 0);

#line 7 "(tree fragment)"
  if (__pyx_t_10) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */

#line 8 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 8 "(tree fragment)"
    __Pyx_INCREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v__dict);

#line 8 "(tree fragment)"
    __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 8 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 8 "(tree fragment)"
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_7));

#line 8 "(tree fragment)"
    __pyx_t_7 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */

#line 9 "(tree fragment)"
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.first_maximum, self.first_maximum_index, self.minimum, self.minimum_index, self.second_maximum, self.second_maximum_index, self.total_distance)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
    goto __pyx_L3;

#line 7 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, None), state
 */

#line 11 "(tree fragment)"
  /*else*/ {

#line 11 "(tree fragment)"
    __pyx_v_use_setstate = 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  __pyx_t_10 = (__pyx_v_use_setstate != 0);

#line 12 "(tree fragment)"
  if (__pyx_t_10) {

    /* "(tree fragment)":13
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, state)
 */

#line 13 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 13 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_pyx_unpickle_ValleyPoint); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 13 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 13 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_19759364);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_19759364);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_19759364);

#line 13 "(tree fragment)"
    __Pyx_INCREF(Py_None);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(Py_None);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 2, Py_None);

#line 13 "(tree fragment)"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_7);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_8);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_8);

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);

#line 13 "(tree fragment)"
    __pyx_t_7 = 0;

#line 13 "(tree fragment)"
    __pyx_t_8 = 0;

#line 13 "(tree fragment)"
    __pyx_r = __pyx_t_6;

#line 13 "(tree fragment)"
    __pyx_t_6 = 0;

#line 13 "(tree fragment)"
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, None), state
 *     else:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ValleyPoint__set_state(self, __pyx_state)
 */

#line 15 "(tree fragment)"
  /*else*/ {

#line 15 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 15 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ValleyPoint); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 15 "(tree fragment)"
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 15 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_19759364);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_19759364);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_19759364);

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_state);

#line 15 "(tree fragment)"
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_6);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_8);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);

#line 15 "(tree fragment)"
    __pyx_t_6 = 0;

#line 15 "(tree fragment)"
    __pyx_t_8 = 0;

#line 15 "(tree fragment)"
    __pyx_r = __pyx_t_7;

#line 15 "(tree fragment)"
    __pyx_t_7 = 0;

#line 15 "(tree fragment)"
    goto __pyx_L0;

#line 15 "(tree fragment)"
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_2);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_3);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v_state);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v__dict);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ValleyPoint__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
/* Python wrapper */

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 16 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_8__setstate_cython__[] = "ValleyPoint.__setstate_cython__(self, __pyx_state)";

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 16 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_8__setstate_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_8__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 16 "(tree fragment)"
  int __pyx_lineno = 0;

#line 16 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 16 "(tree fragment)"
  int __pyx_clineno = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ValleyPoint__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */

#line 17 "(tree fragment)"
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ValleyPoint__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ValleyPoint, (type(self), 0x12d8104, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ValleyPoint__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 16 "(tree fragment)"
  goto __pyx_L0;

#line 16 "(tree fragment)"
  __pyx_L1_error:;

#line 16 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ValleyPoint.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "(tree fragment)"
  __pyx_r = NULL;

#line 16 "(tree fragment)"
  __pyx_L0:;

#line 16 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":259
 *         public np.ndarray ys
 * 
 *     def __init__(self, profile):             # <<<<<<<<<<<<<<
 *         if isinstance(profile, LCMSFeature):
 *             self.profile = profile
 */

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_profile = 0
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_profile,0};

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[1] = {0};

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  1: 
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_profile)) != 0)) kw_args--;
        else 
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 259, __pyx_L3_error)

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_profile = values[0];

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 259, __pyx_L3_error)

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return -1;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter___init__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), __pyx_v_profile);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter___init__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_profile) {

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_7)(PyObject *);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":260
 * 
 *     def __init__(self, profile):
 *         if isinstance(profile, LCMSFeature):             # <<<<<<<<<<<<<<
 *             self.profile = profile
 *             self.xs, self.ys = profile.as_arrays()
 */

#line 260 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_profile, __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature); 

#line 260 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 260 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":261
 *     def __init__(self, profile):
 *         if isinstance(profile, LCMSFeature):
 *             self.profile = profile             # <<<<<<<<<<<<<<
 *             self.xs, self.ys = profile.as_arrays()
 *         else:
 */

#line 261 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_profile);

#line 261 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_profile);

#line 261 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->profile);

#line 261 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_v_self->profile);

#line 261 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->profile = __pyx_v_profile;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":262
 *         if isinstance(profile, LCMSFeature):
 *             self.profile = profile
 *             self.xs, self.ys = profile.as_arrays()             # <<<<<<<<<<<<<<
 *         else:
 *             self.xs, self.ys = profile
 */

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_profile, __pyx_n_s_as_arrays); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_5)) {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* sequence = __pyx_t_3;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(size != 2)) {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyTuple_CheckExact(sequence))) {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #else

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t index = -1;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_6);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_4)) 
#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) 
#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 2) < 0) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = NULL;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_unpacking_done;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L4_unpacking_failed:;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = NULL;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L5_unpacking_done:;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 262, __pyx_L1_error)

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->xs);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->xs));

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->xs = ((PyArrayObject *)__pyx_t_4);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = 0;

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->ys);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->ys));

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->ys = ((PyArrayObject *)__pyx_t_5);

#line 262 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":260
 * 
 *     def __init__(self, profile):
 *         if isinstance(profile, LCMSFeature):             # <<<<<<<<<<<<<<
 *             self.profile = profile
 *             self.xs, self.ys = profile.as_arrays()
 */

#line 260 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 260 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":264
 *             self.xs, self.ys = profile.as_arrays()
 *         else:
 *             self.xs, self.ys = profile             # <<<<<<<<<<<<<<
 *             self.profile = None
 *         self.partition_sites = []
 */

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if ((likely(PyTuple_CheckExact(__pyx_v_profile))) || (PyList_CheckExact(__pyx_v_profile))) {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* sequence = __pyx_v_profile;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(size != 2)) {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyTuple_CheckExact(sequence))) {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #else

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t index = -1;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_4 = PyObject_GetIter(__pyx_v_profile); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = Py_TYPE(__pyx_t_4)->tp_iternext;
      index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_3)) 
#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L6_unpacking_failed;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_5 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_5)) 
#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L6_unpacking_failed;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_4), 2) < 0) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = NULL;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L7_unpacking_done;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L6_unpacking_failed:;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = NULL;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L7_unpacking_done:;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 264, __pyx_L1_error)

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->xs);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->xs));

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->xs = ((PyArrayObject *)__pyx_t_3);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_5);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->ys);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->ys));

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->ys = ((PyArrayObject *)__pyx_t_5);

#line 264 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":265
 *         else:
 *             self.xs, self.ys = profile
 *             self.profile = None             # <<<<<<<<<<<<<<
 *         self.partition_sites = []
 * 
 */

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(Py_None);

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_v_self->profile);

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_v_self->profile);

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_self->profile = Py_None;

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 265 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":266
 *             self.xs, self.ys = profile
 *             self.profile = None
 *         self.partition_sites = []             # <<<<<<<<<<<<<<
 * 
 *     cpdef _extreme_indices(self, ys):
 */

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 266, __pyx_L1_error)

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->partition_sites);

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_v_self->partition_sites);

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->partition_sites = ((PyObject*)__pyx_t_5);

#line 266 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":259
 *         public np.ndarray ys
 * 
 *     def __init__(self, profile):             # <<<<<<<<<<<<<<
 *         if isinstance(profile, LCMSFeature):
 *             self.profile = profile
 */

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 259 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":268
 *         self.partition_sites = []
 * 
 *     cpdef _extreme_indices(self, ys):             # <<<<<<<<<<<<<<
 *         maxima_indices, minima_indices = locate_extrema(ys)
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_3_extreme_indices(PyObject *__pyx_v_self, PyObject *__pyx_v_ys); /*proto*/

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter__extreme_indices(CYTHON_UNUSED struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_ys, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_maxima_indices = NULL
#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_minima_indices = NULL
#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_5)(PyObject *);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_extreme_indices", 0);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if called by wrapper */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if overridden in Python */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_extreme_indices); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_3_extreme_indices)) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_4 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (likely(__pyx_t_4)) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(function);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_ys) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_ys);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 268, __pyx_L1_error)

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_r = __pyx_t_2;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":269
 * 
 *     cpdef _extreme_indices(self, ys):
 *         maxima_indices, minima_indices = locate_extrema(ys)             # <<<<<<<<<<<<<<
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 * 
 */

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_v_ys) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_ys, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_locate_extrema(((PyArrayObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* sequence = __pyx_t_1;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(size != 2)) {

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(PyTuple_CheckExact(sequence))) {

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_2);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #else

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t index = -1;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) 
#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L3_unpacking_failed;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) 
#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L3_unpacking_failed;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_unpacking_done;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L3_unpacking_failed:;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L4_unpacking_done:;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_maxima_indices = __pyx_t_2;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_minima_indices = __pyx_t_3;

#line 269 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":270
 *     cpdef _extreme_indices(self, ys):
 *         maxima_indices, minima_indices = locate_extrema(ys)
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)             # <<<<<<<<<<<<<<
 * 
 *     cpdef object interpolate(self, xs, ys, int n=200):
 */

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_maxima_indices, __pyx_n_s_astype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = NULL;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_2)) {

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_2);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_minima_indices, __pyx_n_s_astype); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int64); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = NULL;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_2)) {

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_2);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_4, function);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_4;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = 0;

#line 270 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":268
 *         self.partition_sites = []
 * 
 *     cpdef _extreme_indices(self, ys):             # <<<<<<<<<<<<<<
 *         maxima_indices, minima_indices = locate_extrema(ys)
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter._extreme_indices", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_maxima_indices);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_minima_indices);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_3_extreme_indices(PyObject *__pyx_v_self, PyObject *__pyx_v_ys); /*proto*/

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2_extreme_indices[] = "ProfileSplitter._extreme_indices(self, ys)";

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_3_extreme_indices(PyObject *__pyx_v_self, PyObject *__pyx_v_ys) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_extreme_indices (wrapper)", 0);
  __pyx_r = 
#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2_extreme_indices(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_ys));

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2_extreme_indices(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_ys) {

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_extreme_indices", 0);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter__extreme_indices(__pyx_v_self, __pyx_v_ys, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter._extreme_indices", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 268 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":272
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 * 
 *     cpdef object interpolate(self, xs, ys, int n=200):             # <<<<<<<<<<<<<<
 *         new_xs = np.linspace(xs.min(), xs.max(), n)
 *         new_ys = np.interp(new_xs, xs, ys)
 */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_5interpolate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate(CYTHON_UNUSED struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate *__pyx_optional_args) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_v_n = ((int)0xC8);
  PyObject *__pyx_v_new_xs = NULL
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_new_ys = NULL
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_6;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate", 0);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_optional_args) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_n = __pyx_optional_args->n;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if called by wrapper */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if overridden in Python */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_interpolate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_5interpolate)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_3);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_5 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (likely(__pyx_t_5)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(__pyx_t_5);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(function);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_DECREF_SET(__pyx_t_4, function);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_6 = 1;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_FAST_PYCALL

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (PyFunction_Check(__pyx_t_4)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_xs, __pyx_v_ys, __pyx_t_3};

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_FAST_PYCCALL

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_xs, __pyx_v_ys, __pyx_t_3};

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_7);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_5) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_v_xs);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_v_xs);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_xs);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(__pyx_v_ys);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_v_ys);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_ys);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_3);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_3);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_r = __pyx_t_2;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":273
 * 
 *     cpdef object interpolate(self, xs, ys, int n=200):
 *         new_xs = np.linspace(xs.min(), xs.max(), n)             # <<<<<<<<<<<<<<
 *         new_ys = np.interp(new_xs, xs, ys)
 *         return new_xs, new_ys
 */

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_linspace); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_xs, __pyx_n_s_min); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = NULL;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_3)) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_7, function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_xs, __pyx_n_s_max); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = NULL;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_5)) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = NULL;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_5)) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_4, function);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = 1;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCALL

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyFunction_Check(__pyx_t_4)) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_t_2, __pyx_t_7, __pyx_t_3};

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCCALL

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_t_2, __pyx_t_7, __pyx_t_3};

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_8);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_5) {

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_2);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_6, __pyx_t_2);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_6, __pyx_t_7);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_6, __pyx_t_3);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_new_xs = __pyx_t_1;

#line 273 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":274
 *     cpdef object interpolate(self, xs, ys, int n=200):
 *         new_xs = np.linspace(xs.min(), xs.max(), n)
 *         new_ys = np.interp(new_xs, xs, ys)             # <<<<<<<<<<<<<<
 *         return new_xs, new_ys
 * 
 */

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_interp); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = NULL;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_4)) {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_8, function);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = 1;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCALL

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyFunction_Check(__pyx_t_8)) {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_new_xs, __pyx_v_xs, __pyx_v_ys};

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if CYTHON_FAST_PYCCALL

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_new_xs, __pyx_v_xs, __pyx_v_ys};

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_4) {

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_new_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_new_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_v_new_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_v_xs);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_ys);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_ys);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, __pyx_v_ys);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_new_ys = __pyx_t_1;

#line 274 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":275
 *         new_xs = np.linspace(xs.min(), xs.max(), n)
 *         new_ys = np.interp(new_xs, xs, ys)
 *         return new_xs, new_ys             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
 */

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_new_xs);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_new_xs);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_new_xs);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_new_ys);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_new_ys);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_new_ys);

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 275 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":272
 *         return maxima_indices.astype(np.int64), minima_indices.astype(np.int64)
 * 
 *     cpdef object interpolate(self, xs, ys, int n=200):             # <<<<<<<<<<<<<<
 *         new_xs = np.linspace(xs.min(), xs.max(), n)
 *         new_ys = np.interp(new_xs, xs, ys)
 */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.interpolate", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_new_xs);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_new_ys);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_5interpolate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_4interpolate[] = "ProfileSplitter.interpolate(self, xs, ys, int n=200)";

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_5interpolate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_xs = 0
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_ys = 0
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  int __pyx_v_n
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate (wrapper)", 0);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xs,&__pyx_n_s_ys,&__pyx_n_s_n,0};

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_xs)) != 0)) kw_args--;
        else 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ys)) != 0)) kw_args--;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate", 0, 2, 3, 1); 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 272, __pyx_L3_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate") < 0)) __PYX_ERR(0, 272, __pyx_L3_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;
        default: 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_xs = values[0];

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_ys = values[1];

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[2]) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_n = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_n == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 272, __pyx_L3_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_n = ((int)0xC8);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 272, __pyx_L3_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.interpolate", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_4interpolate(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), __pyx_v_xs, __pyx_v_ys, __pyx_v_n);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_4interpolate(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_xs, PyObject *__pyx_v_ys, int __pyx_v_n) {

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate __pyx_t_2;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate", 0);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.n = __pyx_v_n;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter->interpolate(__pyx_v_self, __pyx_v_xs, __pyx_v_ys, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.interpolate", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 272 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":278
 * 
 *     @cython.boundscheck(False)
 *     def locate_peak_boundaries(self, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1] xs, ys
 */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7locate_peak_boundaries(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_6locate_peak_boundaries[] = "ProfileSplitter.locate_peak_boundaries(self, double smooth=1, int interpolate_past=200)";

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7locate_peak_boundaries(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_smooth
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  int __pyx_v_interpolate_past
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_peak_boundaries (wrapper)", 0);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_smooth,&__pyx_n_s_interpolate_past,0};

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[2] = {0,0};

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  2: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_smooth);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[0] = value; kw_args--; }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_interpolate_past);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "locate_peak_boundaries") < 0)) __PYX_ERR(0, 278, __pyx_L3_error)

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  0: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
break;
        default: 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[0]) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_smooth = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_smooth == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 278, __pyx_L3_error)

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_smooth = ((double)1.0);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[1]) {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_interpolate_past = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_interpolate_past == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 278, __pyx_L3_error)

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_interpolate_past = ((int)0xC8);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("locate_peak_boundaries", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 278, __pyx_L3_error)

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_peak_boundaries", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_6locate_peak_boundaries(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), __pyx_v_smooth, __pyx_v_interpolate_past);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":311
 *                         candidates.append(point)
 *         if candidates:
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)             # <<<<<<<<<<<<<<
 *             best_point = candidates[0]
 *             self.partition_sites.append(best_point)
 */

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_22locate_peak_boundaries_lambda(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_22locate_peak_boundaries_lambda = 
#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"lambda", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_22locate_peak_boundaries_lambda, METH_O, 0};

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_22locate_peak_boundaries_lambda(PyObject *__pyx_self, PyObject *__pyx_v_x) {

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = 
#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_x));

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("lambda", 0);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_total_distance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_peak_boundaries.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":278
 * 
 *     @cython.boundscheck(False)
 *     def locate_peak_boundaries(self, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1] xs, ys
 */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_6locate_peak_boundaries(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, double __pyx_v_smooth, int __pyx_v_interpolate_past) {
  PyArrayObject *__pyx_v_xs = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_ys = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_maxima_indices = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_minima_indices = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_candidates = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_i
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_j
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_k
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_i
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_j
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_k
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v_point = 0
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_min_i = NULL
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_min_j = NULL
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_max_k = NULL
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_best_point = NULL
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_maxima_indices
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_maxima_indices
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_minima_indices
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_minima_indices
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xs
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_xs
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_6 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_7;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_8;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate __pyx_t_9;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_13)(PyObject *);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled __pyx_t_14;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_15 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_16;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_17;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_18;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_19;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_20;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_21;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_22;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_23;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_24;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_25;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_26;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_27 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_28 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_29 = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_30;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_peak_boundaries", 0);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.pybuffer.buf = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.refcount = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.data = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.rcbuffer = &__pyx_pybuffer_xs;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_maxima_indices.pybuffer.buf = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_maxima_indices.refcount = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_maxima_indices.data = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_maxima_indices.rcbuffer = &__pyx_pybuffer_maxima_indices;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_minima_indices.pybuffer.buf = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_minima_indices.refcount = 0;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_minima_indices.data = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_minima_indices.rcbuffer = &__pyx_pybuffer_minima_indices;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":287
 *             PeakBoundary point
 * 
 *         xs = self.xs             # <<<<<<<<<<<<<<
 *         ys = self.ys
 *         ys = sliding_mean[double](ys)
 */

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_v_self->xs);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) {

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_3); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_4 = __pyx_t_5 = 0;

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_xs = ((PyArrayObject *)__pyx_t_1);

#line 287 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":288
 * 
 *         xs = self.xs
 *         ys = self.ys             # <<<<<<<<<<<<<<
 *         ys = sliding_mean[double](ys)
 *         if len(xs) > interpolate_past:
 */

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_v_self->ys);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) {

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_3);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_5, __pyx_t_4, __pyx_t_3);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __pyx_t_4 = __pyx_t_3 = 0;

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 288, __pyx_L1_error)

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_ys = ((PyArrayObject *)__pyx_t_1);

#line 288 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":289
 *         xs = self.xs
 *         ys = self.ys
 *         ys = sliding_mean[double](ys)             # <<<<<<<<<<<<<<
 *         if len(xs) > interpolate_past:
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 */

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_sliding_mean(((PyArrayObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 289, __pyx_L1_error)

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = ((PyArrayObject *)__pyx_t_1);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) {

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_3); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_4 = __pyx_t_5 = 0;

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF_SET(__pyx_v_ys, ((PyArrayObject *)__pyx_t_1));

#line 289 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":290
 *         ys = self.ys
 *         ys = sliding_mean[double](ys)
 *         if len(xs) > interpolate_past:             # <<<<<<<<<<<<<<
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 *         if smooth:
 */

#line 290 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = PyObject_Length(((PyObject *)__pyx_v_xs)); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 290, __pyx_L1_error)

#line 290 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = ((__pyx_t_7 > __pyx_v_interpolate_past) != 0);

#line 290 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_8) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":291
 *         ys = sliding_mean[double](ys)
 *         if len(xs) > interpolate_past:
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)             # <<<<<<<<<<<<<<
 *         if smooth:
 *             ys = smooth_leveled(xs, ys, smooth)
 */

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_9.__pyx_n = 1;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_9.n = __pyx_v_interpolate_past;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self->__pyx_vtab)->interpolate(__pyx_v_self, ((PyObject *)__pyx_v_xs), ((PyObject *)__pyx_v_ys), 0, &__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* sequence = __pyx_t_1;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(size != 2)) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyTuple_CheckExact(sequence))) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_10);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_11);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #else

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_10);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_11);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t index = -1;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_12);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_10 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_10)) 
#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_10);
      index = 1; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) 
#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_11);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < 0) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_13 = NULL;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_unpacking_done;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L4_unpacking_failed:;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_13 = NULL;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L5_unpacking_done:;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = ((PyArrayObject *)__pyx_t_10);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_3);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_5, __pyx_t_4, __pyx_t_3);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __pyx_t_4 = __pyx_t_3 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_xs, ((PyArrayObject *)__pyx_t_10));

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = ((PyArrayObject *)__pyx_t_11);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_3); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = __pyx_t_4 = __pyx_t_5 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 291, __pyx_L1_error)

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_ys, ((PyArrayObject *)__pyx_t_11));

#line 291 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":290
 *         ys = self.ys
 *         ys = sliding_mean[double](ys)
 *         if len(xs) > interpolate_past:             # <<<<<<<<<<<<<<
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 *         if smooth:
 */

#line 290 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":292
 *         if len(xs) > interpolate_past:
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 *         if smooth:             # <<<<<<<<<<<<<<
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 */

#line 292 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = (__pyx_v_smooth != 0);

#line 292 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_8) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":293
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 *         if smooth:
 *             ys = smooth_leveled(xs, ys, smooth)             # <<<<<<<<<<<<<<
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 */

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14.__pyx_n = 1;

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14.level = __pyx_v_smooth;

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled(((PyArrayObject *)__pyx_v_xs), ((PyArrayObject *)__pyx_v_ys), 0, &__pyx_t_14)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) {

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_3);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_5, __pyx_t_4, __pyx_t_3);

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __pyx_t_4 = __pyx_t_3 = 0;

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 293, __pyx_L1_error)

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_ys, ((PyArrayObject *)__pyx_t_1));

#line 293 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":292
 *         if len(xs) > interpolate_past:
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 *         if smooth:             # <<<<<<<<<<<<<<
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 */

#line 292 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":295
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)             # <<<<<<<<<<<<<<
 *         candidates = []
 *         for i in range(len(minima_indices)):
 */

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self->__pyx_vtab)->_extreme_indices(__pyx_v_self, ((PyObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* sequence = __pyx_t_1;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(size != 2)) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(PyTuple_CheckExact(sequence))) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_11);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_10);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #else

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_11);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_10);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t index = -1;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_12);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
    index = 0; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) 
#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L7_unpacking_failed;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_11);
    index = 1; __pyx_t_10 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_10)) 
#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L7_unpacking_failed;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_10);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < 0) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = NULL;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8_unpacking_done;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L7_unpacking_failed:;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = NULL;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L8_unpacking_done:;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_15 = ((PyArrayObject *)__pyx_t_11);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_t_15, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_maxima_indices, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_3); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_5);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_4 = __pyx_t_5 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_maxima_indices.diminfo[0].strides = __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_maxima_indices.diminfo[0].shape = __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.shape[0];

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_15 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_maxima_indices = ((PyArrayObject *)__pyx_t_11);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_11 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_15 = ((PyArrayObject *)__pyx_t_10);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_t_15, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_minima_indices, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_5); Py_XDECREF(__pyx_t_4); Py_XDECREF(__pyx_t_3);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_5, __pyx_t_4, __pyx_t_3);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __pyx_t_4 = __pyx_t_3 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_minima_indices.diminfo[0].strides = __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_minima_indices.diminfo[0].shape = __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.shape[0];

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 295, __pyx_L1_error)

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_15 = 0;

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_minima_indices = ((PyArrayObject *)__pyx_t_10);

#line 295 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_10 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":296
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 *         candidates = []             # <<<<<<<<<<<<<<
 *         for i in range(len(minima_indices)):
 *             min_i = minima_indices[i]
 */

#line 296 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)

#line 296 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 296 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_candidates = ((PyObject*)__pyx_t_1);

#line 296 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":297
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 *         candidates = []
 *         for i in range(len(minima_indices)):             # <<<<<<<<<<<<<<
 *             min_i = minima_indices[i]
 *             for j in range(i + 1, len(minima_indices)):
 */

#line 297 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_7 = PyObject_Length(((PyObject *)__pyx_v_minima_indices)); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 297, __pyx_L1_error)

#line 297 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_16 = __pyx_t_7;

#line 297 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {

#line 297 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_17;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":298
 *         candidates = []
 *         for i in range(len(minima_indices)):
 *             min_i = minima_indices[i]             # <<<<<<<<<<<<<<
 *             for j in range(i + 1, len(minima_indices)):
 *                 min_j = minima_indices[j]
 */

#line 298 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_18 = __pyx_v_i;

#line 298 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyInt_From_npy_int64((*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.buf, __pyx_t_18, __pyx_pybuffernd_minima_indices.diminfo[0].strides))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)

#line 298 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 298 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_min_i, __pyx_t_1);

#line 298 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":299
 *         for i in range(len(minima_indices)):
 *             min_i = minima_indices[i]
 *             for j in range(i + 1, len(minima_indices)):             # <<<<<<<<<<<<<<
 *                 min_j = minima_indices[j]
 *                 for k in range(len(maxima_indices)):
 */

#line 299 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_19 = PyObject_Length(((PyObject *)__pyx_v_minima_indices)); if (unlikely(__pyx_t_19 == ((Py_ssize_t)-1))) __PYX_ERR(0, 299, __pyx_L1_error)

#line 299 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_20 = __pyx_t_19;

#line 299 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (__pyx_t_18 = (__pyx_v_i + 1); __pyx_t_18 < __pyx_t_20; __pyx_t_18+=1) {

#line 299 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_j = __pyx_t_18;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":300
 *             min_i = minima_indices[i]
 *             for j in range(i + 1, len(minima_indices)):
 *                 min_j = minima_indices[j]             # <<<<<<<<<<<<<<
 *                 for k in range(len(maxima_indices)):
 *                     max_k = maxima_indices[k]
 */

#line 300 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_21 = __pyx_v_j;

#line 300 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyInt_From_npy_int64((*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.buf, __pyx_t_21, __pyx_pybuffernd_minima_indices.diminfo[0].strides))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)

#line 300 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 300 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_v_min_j, __pyx_t_1);

#line 300 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":301
 *             for j in range(i + 1, len(minima_indices)):
 *                 min_j = minima_indices[j]
 *                 for k in range(len(maxima_indices)):             # <<<<<<<<<<<<<<
 *                     max_k = maxima_indices[k]
 *                     y_i = ys[min_i]
 */

#line 301 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_22 = PyObject_Length(((PyObject *)__pyx_v_maxima_indices)); if (unlikely(__pyx_t_22 == ((Py_ssize_t)-1))) __PYX_ERR(0, 301, __pyx_L1_error)

#line 301 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_23 = __pyx_t_22;

#line 301 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_23; __pyx_t_21+=1) {

#line 301 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_k = __pyx_t_21;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":302
 *                 min_j = minima_indices[j]
 *                 for k in range(len(maxima_indices)):
 *                     max_k = maxima_indices[k]             # <<<<<<<<<<<<<<
 *                     y_i = ys[min_i]
 *                     y_j = ys[min_j]
 */

#line 302 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_24 = __pyx_v_k;

#line 302 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyInt_From_npy_int64((*__Pyx_BufPtrStrided1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.buf, __pyx_t_24, __pyx_pybuffernd_maxima_indices.diminfo[0].strides))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)

#line 302 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 302 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF_SET(__pyx_v_max_k, __pyx_t_1);

#line 302 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = 0;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":303
 *                 for k in range(len(maxima_indices)):
 *                     max_k = maxima_indices[k]
 *                     y_i = ys[min_i]             # <<<<<<<<<<<<<<
 *                     y_j = ys[min_j]
 *                     y_k = ys[max_k]
 */

#line 303 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_v_min_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)

#line 303 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 303 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_25 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_25 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 303, __pyx_L1_error)

#line 303 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 303 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_i = __pyx_t_25;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":304
 *                     max_k = maxima_indices[k]
 *                     y_i = ys[min_i]
 *                     y_j = ys[min_j]             # <<<<<<<<<<<<<<
 *                     y_k = ys[max_k]
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (
 */

#line 304 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_v_min_j); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)

#line 304 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 304 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_25 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_25 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 304, __pyx_L1_error)

#line 304 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 304 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_j = __pyx_t_25;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":305
 *                     y_i = ys[min_i]
 *                     y_j = ys[min_j]
 *                     y_k = ys[max_k]             # <<<<<<<<<<<<<<
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (
 *                             y_k - y_j) > (y_k * 0.01):
 */

#line 305 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ys), __pyx_v_max_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)

#line 305 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 305 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_25 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_25 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)

#line 305 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 305 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_k = __pyx_t_25;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":306
 *                     y_j = ys[min_j]
 *                     y_k = ys[max_k]
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (             # <<<<<<<<<<<<<<
 *                             y_k - y_j) > (y_k * 0.01):
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 */

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_min_i, __pyx_v_max_k, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__Pyx_PyObject_IsTrue(__pyx_t_1)) {

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_1);

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = PyObject_RichCompare(__pyx_v_max_k, __pyx_v_min_j, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 306, __pyx_L1_error)

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_26) {

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_8 = __pyx_t_26;

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L16_bool_binop_done;

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_26 = (((__pyx_v_y_k - __pyx_v_y_i) > (__pyx_v_y_k * 0.01)) != 0);

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_26) {

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_8 = __pyx_t_26;

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L16_bool_binop_done;

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":307
 *                     y_k = ys[max_k]
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (
 *                             y_k - y_j) > (y_k * 0.01):             # <<<<<<<<<<<<<<
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 *                         candidates.append(point)
 */

#line 307 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_26 = (((__pyx_v_y_k - __pyx_v_y_j) > (__pyx_v_y_k * 0.01)) != 0);

#line 307 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_8 = __pyx_t_26;

#line 307 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L16_bool_binop_done:;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":306
 *                     y_j = ys[min_j]
 *                     y_k = ys[max_k]
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (             # <<<<<<<<<<<<<<
 *                             y_k - y_j) > (y_k * 0.01):
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 */

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_8) {

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":308
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (
 *                             y_k - y_j) > (y_k * 0.01):
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])             # <<<<<<<<<<<<<<
 *                         candidates.append(point)
 *         if candidates:
 */

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = PyFloat_FromDouble(__pyx_v_y_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_10 = PyFloat_FromDouble(__pyx_v_y_k); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_10);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_11 = PyFloat_FromDouble(__pyx_v_y_j); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_11);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_12 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_xs), __pyx_v_min_i); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_12);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_27 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_xs), __pyx_v_max_k); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_27);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_28 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_xs), __pyx_v_min_j); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_28);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_29 = PyTuple_New(6); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_29);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_1);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 0, __pyx_t_1);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_10);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 1, __pyx_t_10);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_11);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 2, __pyx_t_11);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_12);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 3, __pyx_t_12);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_27);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 4, __pyx_t_27);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_28);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_29, 5, __pyx_t_28);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_10 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_11 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_12 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_27 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_28 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_28 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary), __pyx_t_29, NULL); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 308, __pyx_L1_error)

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_28);

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF_SET(__pyx_v_point, ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_t_28));

#line 308 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_28 = 0;

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":309
 *                             y_k - y_j) > (y_k * 0.01):
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 *                         candidates.append(point)             # <<<<<<<<<<<<<<
 *         if candidates:
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)
 */

#line 309 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_candidates, ((PyObject *)__pyx_v_point)); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 309, __pyx_L1_error)

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":306
 *                     y_j = ys[min_j]
 *                     y_k = ys[max_k]
 *                     if min_i < max_k < min_j and (y_k - y_i) > (y_k * 0.01) and (             # <<<<<<<<<<<<<<
 *                             y_k - y_j) > (y_k * 0.01):
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 */

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 306 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":310
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 *                         candidates.append(point)
 *         if candidates:             # <<<<<<<<<<<<<<
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)
 *             best_point = candidates[0]
 */

#line 310 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_8 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);

#line 310 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_8) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":311
 *                         candidates.append(point)
 *         if candidates:
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)             # <<<<<<<<<<<<<<
 *             best_point = candidates[0]
 *             self.partition_sites.append(best_point)
 */

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_28 = PyTuple_New(1); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_28);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_candidates);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_candidates);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_v_candidates);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_29 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_29);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_27 = __Pyx_CyFunction_New(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_22locate_peak_boundaries_lambda, 0, __pyx_n_s_locate_peak_boundaries_locals_la, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, NULL); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_27);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (PyDict_SetItem(__pyx_t_29, __pyx_n_s_key, __pyx_t_27) < 0) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_27); __pyx_t_27 = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (PyDict_SetItem(__pyx_t_29, __pyx_n_s_reverse, Py_True) < 0) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_27 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_28, __pyx_t_29); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_27);

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_29); __pyx_t_29 = 0;

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(PyList_CheckExact(__pyx_t_27))||((__pyx_t_27) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_27)->tp_name), 0))) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_candidates, ((PyObject*)__pyx_t_27));

#line 311 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_27 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":312
 *         if candidates:
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)
 *             best_point = candidates[0]             # <<<<<<<<<<<<<<
 *             self.partition_sites.append(best_point)
 * 
 */

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_candidates == Py_None)) {

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 312, __pyx_L1_error)

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_27 = PyList_GET_ITEM(__pyx_v_candidates, 0);

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_27);

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_best_point = __pyx_t_27;

#line 312 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_27 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":313
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)
 *             best_point = candidates[0]
 *             self.partition_sites.append(best_point)             # <<<<<<<<<<<<<<
 * 
 *         return candidates
 */

#line 313 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_self->partition_sites == Py_None)) {

#line 313 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");

#line 313 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 313, __pyx_L1_error)

#line 313 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 313 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_30 = __Pyx_PyList_Append(__pyx_v_self->partition_sites, __pyx_v_best_point); if (unlikely(__pyx_t_30 == ((int)-1))) __PYX_ERR(0, 313, __pyx_L1_error)

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":310
 *                         point = PeakBoundary(y_i, y_k, y_j, xs[min_i], xs[max_k], xs[min_j])
 *                         candidates.append(point)
 *         if candidates:             # <<<<<<<<<<<<<<
 *             candidates = sorted(candidates, key=lambda x: x.total_distance, reverse=True)
 *             best_point = candidates[0]
 */

#line 310 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":315
 *             self.partition_sites.append(best_point)
 * 
 *         return candidates             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
 */

#line 315 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 315 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_candidates);

#line 315 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_candidates;

#line 315 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":278
 * 
 *     @cython.boundscheck(False)
 *     def locate_peak_boundaries(self, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1] xs, ys
 */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_27);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_28);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_29);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_peak_boundaries", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_xs);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_ys);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_maxima_indices);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_minima_indices);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_candidates);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_point);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_min_i);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_min_j);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_max_k);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_best_point);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 278 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":318
 * 
 *     @cython.boundscheck(False)
 *     cpdef locate_valleys(self, double scale=0.3, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1, mode='c'] xs
 */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_9locate_valleys(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys *__pyx_optional_args) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_v_scale = ((double)0.3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_v_smooth = ((double)1.0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_v_interpolate_past = ((int)0xC8);
  PyArrayObject *__pyx_v_xs = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_ys = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_maxima_indices = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyArrayObject *__pyx_v_minima_indices = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_candidates = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_i
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_j
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_k
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_max_i
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_max_j
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_min_k
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n_maxima
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_n_minima
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_i
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_j
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y_k
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_point = 0
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_best_point = NULL
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_maxima_indices
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_maxima_indices
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_minima_indices
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_minima_indices
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_xs
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_xs
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_ys
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_ys
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_8;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_13;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_14;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate __pyx_t_15;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_16)(PyObject *);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_17 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_18 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled __pyx_t_19;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_20 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyArrayObject *__pyx_t_21 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_22;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_23;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_24;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_25;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_26;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_27;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_28;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_29;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_30;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_31;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_32;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_33;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_34;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_35;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_36;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_valleys", 0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_optional_args) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_scale = __pyx_optional_args->scale;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_optional_args->__pyx_n > 1) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_smooth = __pyx_optional_args->smooth;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_optional_args->__pyx_n > 2) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_interpolate_past = __pyx_optional_args->interpolate_past;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.pybuffer.buf = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_xs.refcount = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.data = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_xs.rcbuffer = &__pyx_pybuffer_xs;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.pybuffer.buf = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_ys.refcount = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.data = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_ys.rcbuffer = &__pyx_pybuffer_ys;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_maxima_indices.pybuffer.buf = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_maxima_indices.refcount = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_maxima_indices.data = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_maxima_indices.rcbuffer = &__pyx_pybuffer_maxima_indices;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_minima_indices.pybuffer.buf = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffer_minima_indices.refcount = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_minima_indices.data = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_pybuffernd_minima_indices.rcbuffer = &__pyx_pybuffer_minima_indices;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if called by wrapper */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* Check if overridden in Python */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_locate_valleys); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_9locate_valleys)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_scale); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_smooth); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_interpolate_past); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_5);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_6 = __pyx_t_1; 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_7 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_8 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (likely(__pyx_t_7)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(__pyx_t_7);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_INCREF(function);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_DECREF_SET(__pyx_t_6, function);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_8 = 1;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_FAST_PYCALL

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (PyFunction_Check(__pyx_t_6)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_FAST_PYCCALL

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_9);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_7) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_4);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GIVEREF(__pyx_t_5);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_4 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_r = __pyx_t_2;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":329
 *             ValleyPoint point
 * 
 *         xs = self.xs             # <<<<<<<<<<<<<<
 *         ys = self.ys
 * 
 */

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_v_self->xs);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) {

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = __pyx_t_11 = __pyx_t_12 = 0;

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 329, __pyx_L1_error)

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_xs = ((PyArrayObject *)__pyx_t_1);

#line 329 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":330
 * 
 *         xs = self.xs
 *         ys = self.ys             # <<<<<<<<<<<<<<
 * 
 *         if len(xs) > interpolate_past:
 */

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_v_self->ys);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) {

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_12, __pyx_t_11, __pyx_t_10);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = __pyx_t_11 = __pyx_t_10 = 0;

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 330, __pyx_L1_error)

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_ys = ((PyArrayObject *)__pyx_t_1);

#line 330 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":332
 *         ys = self.ys
 * 
 *         if len(xs) > interpolate_past:             # <<<<<<<<<<<<<<
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 * 
 */

#line 332 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = PyObject_Length(((PyObject *)__pyx_v_xs)); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 332, __pyx_L1_error)

#line 332 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = ((__pyx_t_13 > __pyx_v_interpolate_past) != 0);

#line 332 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_14) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":333
 * 
 *         if len(xs) > interpolate_past:
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)             # <<<<<<<<<<<<<<
 * 
 *         if smooth:
 */

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_15.__pyx_n = 1;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_15.n = __pyx_v_interpolate_past;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self->__pyx_vtab)->interpolate(__pyx_v_self, ((PyObject *)__pyx_v_xs), ((PyObject *)__pyx_v_ys), 0, &__pyx_t_15); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* sequence = __pyx_t_1;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(size != 2)) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyTuple_CheckExact(sequence))) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_2);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_6);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #else

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_6);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      #endif

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t index = -1;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_9);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = Py_TYPE(__pyx_t_9)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_16(__pyx_t_9); if (unlikely(!__pyx_t_2)) 
#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_6 = __pyx_t_16(__pyx_t_9); if (unlikely(!__pyx_t_6)) 
#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L4_unpacking_failed;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_6);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_9), 2) < 0) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = NULL;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_unpacking_done;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L4_unpacking_failed:;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_16 = NULL;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L5_unpacking_done:;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_17 = ((PyArrayObject *)__pyx_t_2);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_t_17, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_xs.rcbuffer->pybuffer, (PyObject*)__pyx_v_xs, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_10 = __pyx_t_11 = __pyx_t_12 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_xs.diminfo[0].strides = __pyx_pybuffernd_xs.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_xs.diminfo[0].shape = __pyx_pybuffernd_xs.rcbuffer->pybuffer.shape[0];

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_17 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_xs, ((PyArrayObject *)__pyx_t_2));

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_18 = ((PyArrayObject *)__pyx_t_6);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_t_18, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_12, __pyx_t_11, __pyx_t_10);

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_12 = __pyx_t_11 = __pyx_t_10 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 333, __pyx_L1_error)

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_18 = 0;

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_ys, ((PyArrayObject *)__pyx_t_6));

#line 333 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":332
 *         ys = self.ys
 * 
 *         if len(xs) > interpolate_past:             # <<<<<<<<<<<<<<
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 * 
 */

#line 332 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":335
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 * 
 *         if smooth:             # <<<<<<<<<<<<<<
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 */

#line 335 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = (__pyx_v_smooth != 0);

#line 335 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_14) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":336
 * 
 *         if smooth:
 *             ys = smooth_leveled(xs, ys, smooth)             # <<<<<<<<<<<<<<
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 */

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_19.__pyx_n = 1;

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_19.level = __pyx_v_smooth;

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_smooth_leveled(((PyArrayObject *)__pyx_v_xs), ((PyArrayObject *)__pyx_v_ys), 0, &__pyx_t_19)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    {

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)((PyArrayObject *)__pyx_t_1), &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) {

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Fetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_ys.rcbuffer->pybuffer, (PyObject*)__pyx_v_ys, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_RaiseBufferFallbackError();

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyErr_Restore(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_10 = __pyx_t_11 = __pyx_t_12 = 0;

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_pybuffernd_ys.diminfo[0].strides = __pyx_pybuffernd_ys.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_ys.diminfo[0].shape = __pyx_pybuffernd_ys.rcbuffer->pybuffer.shape[0];

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 336, __pyx_L1_error)

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_ys, ((PyArrayObject *)__pyx_t_1));

#line 336 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":335
 *             xs, ys = self.interpolate(xs, ys, interpolate_past)
 * 
 *         if smooth:             # <<<<<<<<<<<<<<
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 */

#line 335 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":338
 *             ys = smooth_leveled(xs, ys, smooth)
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)             # <<<<<<<<<<<<<<
 *         candidates = []
 *         n_minima = len(minima_indices)
 */

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self->__pyx_vtab)->_extreme_indices(__pyx_v_self, ((PyObject *)__pyx_v_ys), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* sequence = __pyx_t_1;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(size != 2)) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(PyTuple_CheckExact(sequence))) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_6);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #else

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    #endif

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    Py_ssize_t index = -1;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_9);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = Py_TYPE(__pyx_t_9)->tp_iternext;
    index = 0; __pyx_t_6 = __pyx_t_16(__pyx_t_9); if (unlikely(!__pyx_t_6)) 
#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L7_unpacking_failed;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);
    index = 1; __pyx_t_2 = __pyx_t_16(__pyx_t_9); if (unlikely(!__pyx_t_2)) 
#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L7_unpacking_failed;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_9), 2) < 0) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = NULL;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8_unpacking_done;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L7_unpacking_failed:;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = NULL;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L8_unpacking_done:;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_20 = ((PyArrayObject *)__pyx_t_6);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_t_20, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_maxima_indices, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_12, __pyx_t_11, __pyx_t_10);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = __pyx_t_11 = __pyx_t_10 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_maxima_indices.diminfo[0].strides = __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_maxima_indices.diminfo[0].shape = __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.shape[0];

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_20 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_maxima_indices = ((PyArrayObject *)__pyx_t_6);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_21 = ((PyArrayObject *)__pyx_t_2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_8 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_t_21, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Fetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer, (PyObject*)__pyx_v_minima_indices, &__Pyx_TypeInfo_nn___pyx_t_5numpy_int64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_RaiseBufferFallbackError();

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Restore(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = __pyx_t_11 = __pyx_t_12 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_pybuffernd_minima_indices.diminfo[0].strides = __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_minima_indices.diminfo[0].shape = __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.shape[0];

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 338, __pyx_L1_error)

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_21 = 0;

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_minima_indices = ((PyArrayObject *)__pyx_t_2);

#line 338 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":339
 * 
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 *         candidates = []             # <<<<<<<<<<<<<<
 *         n_minima = len(minima_indices)
 *         n_maxima = len(maxima_indices)
 */

#line 339 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)

#line 339 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 339 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_candidates = ((PyObject*)__pyx_t_1);

#line 339 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":340
 *         maxima_indices, minima_indices = self._extreme_indices(ys)
 *         candidates = []
 *         n_minima = len(minima_indices)             # <<<<<<<<<<<<<<
 *         n_maxima = len(maxima_indices)
 * 
 */

#line 340 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = PyObject_Length(((PyObject *)__pyx_v_minima_indices)); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 340, __pyx_L1_error)

#line 340 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n_minima = __pyx_t_13;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":341
 *         candidates = []
 *         n_minima = len(minima_indices)
 *         n_maxima = len(maxima_indices)             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n_maxima):
 */

#line 341 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_13 = PyObject_Length(((PyObject *)__pyx_v_maxima_indices)); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 341, __pyx_L1_error)

#line 341 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_n_maxima = __pyx_t_13;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":343
 *         n_maxima = len(maxima_indices)
 * 
 *         for i in range(n_maxima):             # <<<<<<<<<<<<<<
 *             max_i = maxima_indices[i]
 *             for j in range(i + 1, n_maxima):
 */

#line 343 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_22 = __pyx_v_n_maxima;

#line 343 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_23 = __pyx_t_22;

#line 343 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (__pyx_t_24 = 0; __pyx_t_24 < __pyx_t_23; __pyx_t_24+=1) {

#line 343 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_i = __pyx_t_24;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":344
 * 
 *         for i in range(n_maxima):
 *             max_i = maxima_indices[i]             # <<<<<<<<<<<<<<
 *             for j in range(i + 1, n_maxima):
 *                 max_j = maxima_indices[j]
 */

#line 344 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_25 = __pyx_v_i;

#line 344 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_max_i = (*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.buf, __pyx_t_25, __pyx_pybuffernd_maxima_indices.diminfo[0].strides));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":345
 *         for i in range(n_maxima):
 *             max_i = maxima_indices[i]
 *             for j in range(i + 1, n_maxima):             # <<<<<<<<<<<<<<
 *                 max_j = maxima_indices[j]
 *                 for k in range(n_minima):
 */

#line 345 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_25 = __pyx_v_n_maxima;

#line 345 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_26 = __pyx_t_25;

#line 345 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (__pyx_t_27 = (__pyx_v_i + 1); __pyx_t_27 < __pyx_t_26; __pyx_t_27+=1) {

#line 345 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_j = __pyx_t_27;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":346
 *             max_i = maxima_indices[i]
 *             for j in range(i + 1, n_maxima):
 *                 max_j = maxima_indices[j]             # <<<<<<<<<<<<<<
 *                 for k in range(n_minima):
 *                     min_k = minima_indices[k]
 */

#line 346 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_28 = __pyx_v_j;

#line 346 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_max_j = (*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer.buf, __pyx_t_28, __pyx_pybuffernd_maxima_indices.diminfo[0].strides));

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":347
 *             for j in range(i + 1, n_maxima):
 *                 max_j = maxima_indices[j]
 *                 for k in range(n_minima):             # <<<<<<<<<<<<<<
 *                     min_k = minima_indices[k]
 *                     y_i = ys[max_i]
 */

#line 347 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_28 = __pyx_v_n_minima;

#line 347 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_29 = __pyx_t_28;

#line 347 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      for (__pyx_t_30 = 0; __pyx_t_30 < __pyx_t_29; __pyx_t_30+=1) {

#line 347 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_k = __pyx_t_30;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":348
 *                 max_j = maxima_indices[j]
 *                 for k in range(n_minima):
 *                     min_k = minima_indices[k]             # <<<<<<<<<<<<<<
 *                     y_i = ys[max_i]
 *                     y_j = ys[max_j]
 */

#line 348 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_31 = __pyx_v_k;

#line 348 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_min_k = (*__Pyx_BufPtrCContig1d(__pyx_t_5numpy_int64_t *, __pyx_pybuffernd_minima_indices.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_minima_indices.diminfo[0].strides));

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":349
 *                 for k in range(n_minima):
 *                     min_k = minima_indices[k]
 *                     y_i = ys[max_i]             # <<<<<<<<<<<<<<
 *                     y_j = ys[max_j]
 *                     y_k = ys[min_k]
 */

#line 349 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_31 = __pyx_v_max_i;

#line 349 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_i = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_ys.diminfo[0].strides));

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":350
 *                     min_k = minima_indices[k]
 *                     y_i = ys[max_i]
 *                     y_j = ys[max_j]             # <<<<<<<<<<<<<<
 *                     y_k = ys[min_k]
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (
 */

#line 350 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_31 = __pyx_v_max_j;

#line 350 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_j = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_ys.diminfo[0].strides));

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":351
 *                     y_i = ys[max_i]
 *                     y_j = ys[max_j]
 *                     y_k = ys[min_k]             # <<<<<<<<<<<<<<
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (
 *                             y_j - y_k) > (y_j * scale):
 */

#line 351 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_31 = __pyx_v_min_k;

#line 351 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_y_k = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_ys.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_ys.diminfo[0].strides));

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":352
 *                     y_j = ys[max_j]
 *                     y_k = ys[min_k]
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (             # <<<<<<<<<<<<<<
 *                             y_j - y_k) > (y_j * scale):
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 */

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_32 = (__pyx_v_max_i < __pyx_v_min_k);

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_32) {

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_32 = (__pyx_v_min_k < __pyx_v_max_j);

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_33 = (__pyx_t_32 != 0);

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_33) {

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_14 = __pyx_t_33;

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L16_bool_binop_done;

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_33 = (((__pyx_v_y_i - __pyx_v_y_k) > (__pyx_v_y_i * __pyx_v_scale)) != 0);

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_33) {

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_14 = __pyx_t_33;

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L16_bool_binop_done;

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":353
 *                     y_k = ys[min_k]
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (
 *                             y_j - y_k) > (y_j * scale):             # <<<<<<<<<<<<<<
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 *                         candidates.append(point)
 */

#line 353 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_33 = (((__pyx_v_y_j - __pyx_v_y_k) > (__pyx_v_y_j * __pyx_v_scale)) != 0);

#line 353 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_14 = __pyx_t_33;

#line 353 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L16_bool_binop_done:;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":352
 *                     y_j = ys[max_j]
 *                     y_k = ys[min_k]
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (             # <<<<<<<<<<<<<<
 *                             y_j - y_k) > (y_j * scale):
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 */

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_14) {

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":354
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (
 *                             y_j - y_k) > (y_j * scale):
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])             # <<<<<<<<<<<<<<
 *                         candidates.append(point)
 *         if candidates:
 */

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_31 = __pyx_v_max_i;

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_34 = __pyx_v_min_k;

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_35 = __pyx_v_max_j;

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__create_ValleyPoint(__pyx_v_y_i, __pyx_v_y_k, __pyx_v_y_j, (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_xs.rcbuffer->pybuffer.buf, __pyx_t_31, __pyx_pybuffernd_xs.diminfo[0].strides)), (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_xs.rcbuffer->pybuffer.buf, __pyx_t_34, __pyx_pybuffernd_xs.diminfo[0].strides)), (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_xs.rcbuffer->pybuffer.buf, __pyx_t_35, __pyx_pybuffernd_xs.diminfo[0].strides)))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 354, __pyx_L1_error)

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF_SET(__pyx_v_point, ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_t_1));

#line 354 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":355
 *                             y_j - y_k) > (y_j * scale):
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 *                         candidates.append(point)             # <<<<<<<<<<<<<<
 *         if candidates:
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)
 */

#line 355 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_36 = __Pyx_PyList_Append(__pyx_v_candidates, ((PyObject *)__pyx_v_point)); if (unlikely(__pyx_t_36 == ((int)-1))) __PYX_ERR(0, 355, __pyx_L1_error)

          /* "ms_deisotope/_c/feature_map/profile_transform.pyx":352
 *                     y_j = ys[max_j]
 *                     y_k = ys[min_k]
 *                     if max_i < min_k < max_j and (y_i - y_k) > (y_i * scale) and (             # <<<<<<<<<<<<<<
 *                             y_j - y_k) > (y_j * scale):
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 */

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 352 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":356
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 *                         candidates.append(point)
 *         if candidates:             # <<<<<<<<<<<<<<
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)
 *             best_point = candidates[0]
 */

#line 356 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_14 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);

#line 356 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_14) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":357
 *                         candidates.append(point)
 *         if candidates:
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)             # <<<<<<<<<<<<<<
 *             best_point = candidates[0]
 *             self.partition_sites.append(best_point)
 */

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_candidates);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_v_candidates);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_candidates);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_get_total_distance); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_t_6) < 0) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_reverse, Py_True) < 0) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!(likely(PyList_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(0, 357, __pyx_L1_error)

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_candidates, ((PyObject*)__pyx_t_6));

#line 357 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":358
 *         if candidates:
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)
 *             best_point = candidates[0]             # <<<<<<<<<<<<<<
 *             self.partition_sites.append(best_point)
 * 
 */

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_candidates == Py_None)) {

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 358, __pyx_L1_error)

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyList_GET_ITEM(__pyx_v_candidates, 0);

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_6);

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_best_point = __pyx_t_6;

#line 358 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":359
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)
 *             best_point = candidates[0]
 *             self.partition_sites.append(best_point)             # <<<<<<<<<<<<<<
 * 
 *         return candidates
 */

#line 359 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_self->partition_sites == Py_None)) {

#line 359 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");

#line 359 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 359, __pyx_L1_error)

#line 359 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 359 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_36 = __Pyx_PyList_Append(__pyx_v_self->partition_sites, __pyx_v_best_point); if (unlikely(__pyx_t_36 == ((int)-1))) __PYX_ERR(0, 359, __pyx_L1_error)

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":356
 *                         point = _create_ValleyPoint(y_i, y_k, y_j, xs[max_i], xs[min_k], xs[max_j])
 *                         candidates.append(point)
 *         if candidates:             # <<<<<<<<<<<<<<
 *             candidates = sorted(candidates, key=_get_total_distance, reverse=True)
 *             best_point = candidates[0]
 */

#line 356 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":361
 *             self.partition_sites.append(best_point)
 * 
 *         return candidates             # <<<<<<<<<<<<<<
 * 
 *     def extract_peak(self, peak_boundary):
 */

#line 361 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 361 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_candidates);

#line 361 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_candidates;

#line 361 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":318
 * 
 *     @cython.boundscheck(False)
 *     cpdef locate_valleys(self, double scale=0.3, double smooth=1, int interpolate_past=200):             # <<<<<<<<<<<<<<
 *         cdef:
 *             np.ndarray[double, ndim=1, mode='c'] xs
 */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_declare

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_PyThreadState_assign

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_valleys", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L2;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_maxima_indices.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_minima_indices.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_xs.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_ys.rcbuffer->pybuffer);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L2:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_xs);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_ys);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_maxima_indices);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_minima_indices);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_candidates);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_point);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_best_point);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_9locate_valleys(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_8locate_valleys[] = "ProfileSplitter.locate_valleys(self, double scale=0.3, double smooth=1, int interpolate_past=200)";

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_9locate_valleys(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_scale
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_smooth
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  int __pyx_v_interpolate_past
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_valleys (wrapper)", 0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_scale,&__pyx_n_s_smooth,&__pyx_n_s_interpolate_past,0};

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_scale);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[0] = value; kw_args--; }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_smooth);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_interpolate_past);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "locate_valleys") < 0)) __PYX_ERR(0, 318, __pyx_L3_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  0: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
break;
        default: 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[0]) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_scale = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_scale == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 318, __pyx_L3_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_scale = ((double)0.3);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[1]) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_smooth = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_smooth == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 318, __pyx_L3_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_smooth = ((double)1.0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (values[2]) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_interpolate_past = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_interpolate_past == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 318, __pyx_L3_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_interpolate_past = ((int)0xC8);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("locate_valleys", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 318, __pyx_L3_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_valleys", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_8locate_valleys(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), __pyx_v_scale, __pyx_v_smooth, __pyx_v_interpolate_past);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_8locate_valleys(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, double __pyx_v_scale, double __pyx_v_smooth, int __pyx_v_interpolate_past) {

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys __pyx_t_2;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("locate_valleys", 0);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.__pyx_n = 3;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.scale = __pyx_v_scale;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.smooth = __pyx_v_smooth;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2.interpolate_past = __pyx_v_interpolate_past;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter->locate_valleys(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.locate_valleys", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 318 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":363
 *         return candidates
 * 
 *     def extract_peak(self, peak_boundary):             # <<<<<<<<<<<<<<
 *         if self.profile is not None:
 *             cases = peak_boundary.split(self.profile)
 */

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_11extract_peak(PyObject *__pyx_v_self, PyObject *__pyx_v_peak_boundary); /*proto*/

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_10extract_peak[] = "ProfileSplitter.extract_peak(self, peak_boundary)";

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_11extract_peak(PyObject *__pyx_v_self, PyObject *__pyx_v_peak_boundary) {

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("extract_peak (wrapper)", 0);
  __pyx_r = 
#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_10extract_peak(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_peak_boundary));

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_10extract_peak(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_peak_boundary) {
  PyObject *__pyx_v_cases = NULL
#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("extract_peak", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":364
 * 
 *     def extract_peak(self, peak_boundary):
 *         if self.profile is not None:             # <<<<<<<<<<<<<<
 *             cases = peak_boundary.split(self.profile)
 *         else:
 */

#line 364 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_v_self->profile != Py_None);

#line 364 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 364 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":365
 *     def extract_peak(self, peak_boundary):
 *         if self.profile is not None:
 *             cases = peak_boundary.split(self.profile)             # <<<<<<<<<<<<<<
 *         else:
 *             cases = None
 */

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak_boundary, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_5)) {

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_5);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_v_self->profile) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_self->profile);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_cases = __pyx_t_3;

#line 365 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":364
 * 
 *     def extract_peak(self, peak_boundary):
 *         if self.profile is not None:             # <<<<<<<<<<<<<<
 *             cases = peak_boundary.split(self.profile)
 *         else:
 */

#line 364 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 364 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":367
 *             cases = peak_boundary.split(self.profile)
 *         else:
 *             cases = None             # <<<<<<<<<<<<<<
 *         return cases
 * 
 */

#line 367 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 367 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 367 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_cases = Py_None;

#line 367 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 367 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":368
 *         else:
 *             cases = None
 *         return cases             # <<<<<<<<<<<<<<
 * 
 *     def split_valley(self, valley):
 */

#line 368 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 368 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_cases);

#line 368 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_cases;

#line 368 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":363
 *         return candidates
 * 
 *     def extract_peak(self, peak_boundary):             # <<<<<<<<<<<<<<
 *         if self.profile is not None:
 *             cases = peak_boundary.split(self.profile)
 */

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.extract_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_cases);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 363 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":370
 *         return cases
 * 
 *     def split_valley(self, valley):             # <<<<<<<<<<<<<<
 *         if self.profile is not None:
 *             return valley.split(self.profile)
 */

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_13split_valley(PyObject *__pyx_v_self, PyObject *__pyx_v_valley); /*proto*/

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_12split_valley[] = "ProfileSplitter.split_valley(self, valley)";

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_13split_valley(PyObject *__pyx_v_self, PyObject *__pyx_v_valley) {

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split_valley (wrapper)", 0);
  __pyx_r = 
#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_12split_valley(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_valley));

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_12split_valley(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_valley) {

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split_valley", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":371
 * 
 *     def split_valley(self, valley):
 *         if self.profile is not None:             # <<<<<<<<<<<<<<
 *             return valley.split(self.profile)
 *         else:
 */

#line 371 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_v_self->profile != Py_None);

#line 371 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 371 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":372
 *     def split_valley(self, valley):
 *         if self.profile is not None:
 *             return valley.split(self.profile)             # <<<<<<<<<<<<<<
 *         else:
 *             return None
 */

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_valley, __pyx_n_s_split); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 372, __pyx_L1_error)

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_5)) {

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_5);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_v_self->profile) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_self->profile);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 372, __pyx_L1_error)

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = __pyx_t_3;

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = 0;

#line 372 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":371
 * 
 *     def split_valley(self, valley):
 *         if self.profile is not None:             # <<<<<<<<<<<<<<
 *             return valley.split(self.profile)
 *         else:
 */

#line 371 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":374
 *             return valley.split(self.profile)
 *         else:
 *             return None             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 374 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 374 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 374 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 374 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 374 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":370
 *         return cases
 * 
 *     def split_valley(self, valley):             # <<<<<<<<<<<<<<
 *         if self.profile is not None:
 *             return valley.split(self.profile)
 */

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.split_valley", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 370 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":254
 * cdef class ProfileSplitter(object):
 *     cdef:
 *         public object profile             # <<<<<<<<<<<<<<
 *         public list partition_sites
 *         public np.ndarray xs
 */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_1__get__(PyObject *__pyx_v_self) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_self->profile);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_self->profile;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_value);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_v_value);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->profile);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_v_self->profile);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->profile = __pyx_v_value;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_5__del__(PyObject *__pyx_v_self) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(Py_None);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(Py_None);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->profile);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_v_self->profile);

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->profile = Py_None;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 254 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":255
 *     cdef:
 *         public object profile
 *         public list partition_sites             # <<<<<<<<<<<<<<
 *         public np.ndarray xs
 *         public np.ndarray ys
 */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_1__get__(PyObject *__pyx_v_self) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_self->partition_sites);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_self->partition_sites;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 255, __pyx_L1_error)

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_value;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->partition_sites);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_v_self->partition_sites);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->partition_sites = ((PyObject*)__pyx_t_1);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.partition_sites.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_5__del__(PyObject *__pyx_v_self) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(Py_None);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(Py_None);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->partition_sites);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_v_self->partition_sites);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->partition_sites = ((PyObject*)Py_None);

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 255 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":256
 *         public object profile
 *         public list partition_sites
 *         public np.ndarray xs             # <<<<<<<<<<<<<<
 *         public np.ndarray ys
 * 
 */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_1__get__(PyObject *__pyx_v_self) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->xs));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyObject *)__pyx_v_self->xs);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 256, __pyx_L1_error)

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_value;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->xs);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->xs));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->xs = ((PyArrayObject *)__pyx_t_1);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.xs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_5__del__(PyObject *__pyx_v_self) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(Py_None);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(Py_None);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->xs);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->xs));

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->xs = ((PyArrayObject *)Py_None);

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":257
 *         public list partition_sites
 *         public np.ndarray xs
 *         public np.ndarray ys             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, profile):
 */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_1__get__(PyObject *__pyx_v_self) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys___get__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys___get__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->ys));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((PyObject *)__pyx_v_self->ys);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_2__set__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v_value) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 257, __pyx_L1_error)

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_value;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->ys);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->ys));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->ys = ((PyArrayObject *)__pyx_t_1);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.ys.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = -1;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_5__del__(PyObject *__pyx_v_self) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static int __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_4__del__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(Py_None);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(Py_None);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_v_self->ys);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->ys));

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->ys = ((PyArrayObject *)Py_None);

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 257 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_14__reduce_cython__[] = "ProfileSplitter.__reduce_cython__(self)";

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_14__reduce_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_14__reduce_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self) {
  PyObject *__pyx_v_state = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v__dict = 0
#line 1 "(tree fragment)"
;
  int __pyx_v_use_setstate
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_5;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.partition_sites, self.profile, self.xs, self.ys)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */

#line 5 "(tree fragment)"
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
  __Pyx_INCREF(__pyx_v_self->partition_sites);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_v_self->partition_sites);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->partition_sites);

#line 5 "(tree fragment)"
  __Pyx_INCREF(__pyx_v_self->profile);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_v_self->profile);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->profile);

#line 5 "(tree fragment)"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->xs));

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->xs));

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_self->xs));

#line 5 "(tree fragment)"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->ys));

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->ys));

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 3, ((PyObject *)__pyx_v_self->ys));

#line 5 "(tree fragment)"
  __pyx_v_state = ((PyObject*)__pyx_t_1);

#line 5 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.partition_sites, self.profile, self.xs, self.ys)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */

#line 6 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
  __pyx_v__dict = __pyx_t_1;

#line 6 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.partition_sites, self.profile, self.xs, self.ys)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
  __pyx_t_2 = (__pyx_v__dict != Py_None);

#line 7 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 7 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */

#line 8 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 8 "(tree fragment)"
    __Pyx_INCREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);

#line 8 "(tree fragment)"
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 8 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 8 "(tree fragment)"
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));

#line 8 "(tree fragment)"
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.partition_sites is not None or self.profile is not None or self.xs is not None or self.ys is not None
 */

#line 9 "(tree fragment)"
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.partition_sites, self.profile, self.xs, self.ys)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
    goto __pyx_L3;

#line 7 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.partition_sites is not None or self.profile is not None or self.xs is not None or self.ys is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, None), state
 */

#line 11 "(tree fragment)"
  /*else*/ {

#line 11 "(tree fragment)"
    __pyx_t_2 = (__pyx_v_self->partition_sites != ((PyObject*)Py_None));

#line 11 "(tree fragment)"
    __pyx_t_5 = (__pyx_t_2 != 0);

#line 11 "(tree fragment)"
    if (!__pyx_t_5) {

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      __pyx_t_3 = __pyx_t_5;

#line 11 "(tree fragment)"
      goto __pyx_L4_bool_binop_done;

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    __pyx_t_5 = (__pyx_v_self->profile != Py_None);

#line 11 "(tree fragment)"
    __pyx_t_2 = (__pyx_t_5 != 0);

#line 11 "(tree fragment)"
    if (!__pyx_t_2) {

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      __pyx_t_3 = __pyx_t_2;

#line 11 "(tree fragment)"
      goto __pyx_L4_bool_binop_done;

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    __pyx_t_2 = (((PyObject *)__pyx_v_self->xs) != Py_None);

#line 11 "(tree fragment)"
    __pyx_t_5 = (__pyx_t_2 != 0);

#line 11 "(tree fragment)"
    if (!__pyx_t_5) {

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      __pyx_t_3 = __pyx_t_5;

#line 11 "(tree fragment)"
      goto __pyx_L4_bool_binop_done;

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    __pyx_t_5 = (((PyObject *)__pyx_v_self->ys) != Py_None);

#line 11 "(tree fragment)"
    __pyx_t_2 = (__pyx_t_5 != 0);

#line 11 "(tree fragment)"
    __pyx_t_3 = __pyx_t_2;

#line 11 "(tree fragment)"
    __pyx_L4_bool_binop_done:;

#line 11 "(tree fragment)"
    __pyx_v_use_setstate = __pyx_t_3;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.partition_sites is not None or self.profile is not None or self.xs is not None or self.ys is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  __pyx_t_3 = (__pyx_v_use_setstate != 0);

#line 12 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.partition_sites is not None or self.profile is not None or self.xs is not None or self.ys is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, state)
 */

#line 13 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 13 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ProfileSplitter); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 13 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_210030367);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_210030367);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_210030367);

#line 13 "(tree fragment)"
    __Pyx_INCREF(Py_None);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(Py_None);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);

#line 13 "(tree fragment)"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_4);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);

#line 13 "(tree fragment)"
    __pyx_t_4 = 0;

#line 13 "(tree fragment)"
    __pyx_t_1 = 0;

#line 13 "(tree fragment)"
    __pyx_r = __pyx_t_6;

#line 13 "(tree fragment)"
    __pyx_t_6 = 0;

#line 13 "(tree fragment)"
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.partition_sites is not None or self.profile is not None or self.xs is not None or self.ys is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, None), state
 *     else:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ProfileSplitter__set_state(self, __pyx_state)
 */

#line 15 "(tree fragment)"
  /*else*/ {

#line 15 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 15 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ProfileSplitter); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 15 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 15 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_210030367);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_210030367);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_210030367);

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);

#line 15 "(tree fragment)"
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_6);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);

#line 15 "(tree fragment)"
    __pyx_t_6 = 0;

#line 15 "(tree fragment)"
    __pyx_t_1 = 0;

#line 15 "(tree fragment)"
    __pyx_r = __pyx_t_4;

#line 15 "(tree fragment)"
    __pyx_t_4 = 0;

#line 15 "(tree fragment)"
    goto __pyx_L0;

#line 15 "(tree fragment)"
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v_state);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v__dict);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ProfileSplitter__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
/* Python wrapper */

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 16 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_16__setstate_cython__[] = "ProfileSplitter.__setstate_cython__(self, __pyx_state)";

#line 16 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 16 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_16__setstate_cython__(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_16__setstate_cython__(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 16 "(tree fragment)"
  int __pyx_lineno = 0;

#line 16 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 16 "(tree fragment)"
  int __pyx_clineno = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ProfileSplitter__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */

#line 17 "(tree fragment)"
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ProfileSplitter__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ProfileSplitter, (type(self), 0xc84cf1f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ProfileSplitter__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 16 "(tree fragment)"
  goto __pyx_L0;

#line 16 "(tree fragment)"
  __pyx_L1_error:;

#line 16 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.ProfileSplitter.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "(tree fragment)"
  __pyx_r = NULL;

#line 16 "(tree fragment)"
  __pyx_L0:;

#line 16 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":377
 * 
 * 
 * cpdef double _get_total_distance(object obj):             # <<<<<<<<<<<<<<
 *     return (<ValleyPoint>obj).total_distance
 * 
 */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15_get_total_distance(PyObject *__pyx_self, PyObject *__pyx_v_obj); /*proto*/

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__get_total_distance(PyObject *__pyx_v_obj, CYTHON_UNUSED int __pyx_skip_dispatch) {

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_get_total_distance", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":378
 * 
 * cpdef double _get_total_distance(object obj):
 *     return (<ValleyPoint>obj).total_distance             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 378 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v_obj)->total_distance;

#line 378 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":377
 * 
 * 
 * cpdef double _get_total_distance(object obj):             # <<<<<<<<<<<<<<
 *     return (<ValleyPoint>obj).total_distance
 * 
 */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15_get_total_distance(PyObject *__pyx_self, PyObject *__pyx_v_obj); /*proto*/

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_14_get_total_distance[] = "_get_total_distance(obj) -> double";

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15_get_total_distance(PyObject *__pyx_self, PyObject *__pyx_v_obj) {

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_get_total_distance (wrapper)", 0);
  __pyx_r = 
#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_14_get_total_distance(__pyx_self, ((PyObject *)__pyx_v_obj));

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_14_get_total_distance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_obj) {

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_get_total_distance", 0);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__get_total_distance(__pyx_v_obj, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform._get_total_distance", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 377 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":381
 * 
 * 
 * cdef ValleyPoint _create_ValleyPoint(double first_maximum, double minimum, double second_maximum,             # <<<<<<<<<<<<<<
 *                                      double first_maximum_index, double minimum_index,
 *                                      double second_maximum_index):
 */

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform__create_ValleyPoint(double __pyx_v_first_maximum, double __pyx_v_minimum, double __pyx_v_second_maximum, double __pyx_v_first_maximum_index, double __pyx_v_minimum_index, CYTHON_UNUSED double __pyx_v_second_maximum_index) {
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v_self = 0
#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_r = NULL;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("_create_ValleyPoint", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":386
 *     cdef:
 *         ValleyPoint self
 *     self = ValleyPoint.__new__(ValleyPoint)             # <<<<<<<<<<<<<<
 *     self.first_maximum = first_maximum
 *     self.minimum = minimum
 */

#line 386 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)

#line 386 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));

#line 386 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_t_1);

#line 386 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":387
 *         ValleyPoint self
 *     self = ValleyPoint.__new__(ValleyPoint)
 *     self.first_maximum = first_maximum             # <<<<<<<<<<<<<<
 *     self.minimum = minimum
 *     self.second_maximum = second_maximum
 */

#line 387 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum = __pyx_v_first_maximum;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":388
 *     self = ValleyPoint.__new__(ValleyPoint)
 *     self.first_maximum = first_maximum
 *     self.minimum = minimum             # <<<<<<<<<<<<<<
 *     self.second_maximum = second_maximum
 *     self.first_maximum_index = first_maximum_index
 */

#line 388 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum = __pyx_v_minimum;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":389
 *     self.first_maximum = first_maximum
 *     self.minimum = minimum
 *     self.second_maximum = second_maximum             # <<<<<<<<<<<<<<
 *     self.first_maximum_index = first_maximum_index
 *     self.minimum_index = minimum_index
 */

#line 389 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->second_maximum = __pyx_v_second_maximum;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":390
 *     self.minimum = minimum
 *     self.second_maximum = second_maximum
 *     self.first_maximum_index = first_maximum_index             # <<<<<<<<<<<<<<
 *     self.minimum_index = minimum_index
 *     self.second_maximum_index
 */

#line 390 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->first_maximum_index = __pyx_v_first_maximum_index;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":391
 *     self.second_maximum = second_maximum
 *     self.first_maximum_index = first_maximum_index
 *     self.minimum_index = minimum_index             # <<<<<<<<<<<<<<
 *     self.second_maximum_index
 *     return self
 */

#line 391 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_self->minimum_index = __pyx_v_minimum_index;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":392
 *     self.first_maximum_index = first_maximum_index
 *     self.minimum_index = minimum_index
 *     self.second_maximum_index             # <<<<<<<<<<<<<<
 *     return self
 * 
 */

#line 392 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  (void)(__pyx_v_self->second_maximum_index);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":393
 *     self.minimum_index = minimum_index
 *     self.second_maximum_index
 *     return self             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 393 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 393 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 393 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_self;

#line 393 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":381
 * 
 * 
 * cdef ValleyPoint _create_ValleyPoint(double first_maximum, double minimum, double second_maximum,             # <<<<<<<<<<<<<<
 *                                      double first_maximum_index, double minimum_index,
 *                                      double second_maximum_index):
 */

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform._create_ValleyPoint", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_self);

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 381 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_17split_valleys(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_16split_valleys[] = "split_valleys(profiles, scale=0.3, n_levels=2)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_17split_valleys = 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"split_valleys", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_17split_valleys, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_16split_valleys};

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_17split_valleys(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_profiles = 0
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_scale = 0
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_n_levels = 0
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split_valleys (wrapper)", 0);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_profiles,&__pyx_n_s_scale,&__pyx_n_s_n_levels,0};

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    values[1] = ((PyObject *)__pyx_float_0_3);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    values[2] = ((PyObject *)__pyx_int_2);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_profiles)) != 0)) kw_args--;
        else 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_scale);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (kw_args > 0) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n_levels);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "split_valleys") < 0)) __PYX_ERR(0, 396, __pyx_L3_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;
        default: 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_profiles = values[0];

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_scale = values[1];

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_n_levels = values[2];

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("split_valleys", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 396, __pyx_L3_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.split_valleys", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_16split_valleys(__pyx_self, __pyx_v_profiles, __pyx_v_scale, __pyx_v_n_levels);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":404
 *             if vallies:
 *                 splitter = vallies[0]
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)             # <<<<<<<<<<<<<<
 *             else:
 *                 out.append(case)
 */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *__pyx_cur_scope
#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr(__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)Py_None);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *) __pyx_self;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_2generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_split_valleys_locals_genexpr, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!gen)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_cur_scope);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_RefNannyFinishContext();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    return (PyObject *) gen;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.split_valleys.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_4;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_5)(PyObject *);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_6;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_7;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  switch (__pyx_generator->resume_label) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    case 0: goto __pyx_L3_first_run;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    case 1: goto __pyx_L7_resume_from_yield;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    default: /* CPython raises the right error here */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_RefNannyFinishContext();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    return NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_first_run:;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_ps)) { __Pyx_RaiseClosureNameError("ps"); __PYX_ERR(0, 404, __pyx_L1_error) }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_ps), __pyx_n_s_split_valley); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_splitter)) { __Pyx_RaiseClosureNameError("splitter"); __PYX_ERR(0, 404, __pyx_L1_error) }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(__pyx_t_3)) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(function);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_splitter) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_splitter);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = NULL;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {
    __pyx_t_4 = -1; 
#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (;;) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(!__pyx_t_5)) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyList_CheckExact(__pyx_t_2))) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #else

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #else

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }
    } else 
#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_1)) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* exc_type = PyErr_Occurred();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (exc_type) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          else __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_p);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_p, __pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyObject_Length(__pyx_cur_scope->__pyx_v_p); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_7 = ((__pyx_t_6 > 0) != 0);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_7) {

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_p);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_r = __pyx_cur_scope->__pyx_v_p;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XGIVEREF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XGIVEREF(__pyx_r);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_RefNannyFinishContext();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /* return from generator, yielding value */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_generator->resume_label = 1;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      return __pyx_r;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L7_resume_from_yield:;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_cur_scope->__pyx_t_0 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XGOTREF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyErr_SetNone(PyExc_StopIteration);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #if !CYTHON_USE_EXC_INFO_STACK

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  #endif

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_generator->resume_label = -1;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_16split_valleys(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_profiles, PyObject *__pyx_v_scale, PyObject *__pyx_v_n_levels) {
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *__pyx_cur_scope
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  CYTHON_UNUSED PyObject *__pyx_v_i = NULL
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_out = NULL
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_case = NULL
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_vallies = NULL
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_gb_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_2generator = 0
#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_3;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_4)(PyObject *);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_5;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *(*__pyx_t_6)(PyObject *);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_t_8;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys __pyx_t_9;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_10;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_11;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("split_valleys", 0);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys, __pyx_empty_tuple, NULL);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)Py_None);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 396, __pyx_L1_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_profiles);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":397
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):
 *     for i in range(n_levels):             # <<<<<<<<<<<<<<
 *         out = []
 *         for case in profiles:
 */

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_n_levels); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = NULL;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {
    __pyx_t_3 = -1; 
#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  for (;;) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(!__pyx_t_4)) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(PyList_CheckExact(__pyx_t_2))) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #else

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      } else {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #else

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        #endif

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }
    } else 
#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(!__pyx_t_1)) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* exc_type = PyErr_Occurred();

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (exc_type) {

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          else __PYX_ERR(0, 397, __pyx_L1_error)

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        break;

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":398
 * def split_valleys(profiles, scale=0.3, n_levels=2):
 *     for i in range(n_levels):
 *         out = []             # <<<<<<<<<<<<<<
 *         for case in profiles:
 *             ps = ProfileSplitter(case)
 */

#line 398 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 398 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_out, ((PyObject*)__pyx_t_1));

#line 398 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":399
 *     for i in range(n_levels):
 *         out = []
 *         for case in profiles:             # <<<<<<<<<<<<<<
 *             ps = ProfileSplitter(case)
 *             vallies = ps.locate_valleys(scale)
 */

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (likely(PyList_CheckExact(__pyx_v_profiles)) || PyTuple_CheckExact(__pyx_v_profiles)) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_1 = __pyx_v_profiles; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = NULL;

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {
      __pyx_t_5 = -1; 
#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_t_1 = PyObject_GetIter(__pyx_v_profiles); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (;;) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(!__pyx_t_6)) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely(PyList_CheckExact(__pyx_t_1))) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #else

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_7);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #endif

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        } else {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_1)) break;

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #else

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_7);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          #endif

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }
      } else 
#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __pyx_t_6(__pyx_t_1);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(!__pyx_t_7)) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          PyObject* exc_type = PyErr_Occurred();

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (exc_type) {

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            else __PYX_ERR(0, 399, __pyx_L1_error)

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_7);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_v_case, __pyx_t_7);

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":400
 *         out = []
 *         for case in profiles:
 *             ps = ProfileSplitter(case)             # <<<<<<<<<<<<<<
 *             vallies = ps.locate_valleys(scale)
 *             if vallies:
 */

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter), __pyx_v_case); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 400, __pyx_L1_error)

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_7);

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_ps));

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_ps, ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_t_7));

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(__pyx_t_7);

#line 400 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":401
 *         for case in profiles:
 *             ps = ProfileSplitter(case)
 *             vallies = ps.locate_valleys(scale)             # <<<<<<<<<<<<<<
 *             if vallies:
 *                 splitter = vallies[0]
 */

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_v_scale); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9.__pyx_n = 1;

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_9.scale = __pyx_t_8;

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_cur_scope->__pyx_v_ps->__pyx_vtab)->locate_valleys(__pyx_cur_scope->__pyx_v_ps, 0, &__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_7);

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_v_vallies, __pyx_t_7);

#line 401 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_7 = 0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":402
 *             ps = ProfileSplitter(case)
 *             vallies = ps.locate_valleys(scale)
 *             if vallies:             # <<<<<<<<<<<<<<
 *                 splitter = vallies[0]
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)
 */

#line 402 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_vallies); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 402, __pyx_L1_error)

#line 402 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_10) {

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":403
 *             vallies = ps.locate_valleys(scale)
 *             if vallies:
 *                 splitter = vallies[0]             # <<<<<<<<<<<<<<
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)
 *             else:
 */

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_vallies, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 403, __pyx_L1_error)

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_7);

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_splitter);

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_splitter, __pyx_t_7);

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GIVEREF(__pyx_t_7);

#line 403 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = 0;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":404
 *             if vallies:
 *                 splitter = vallies[0]
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)             # <<<<<<<<<<<<<<
 *             else:
 *                 out.append(case)
 */

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_7);

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_11 = __Pyx_PyList_Extend(__pyx_v_out, __pyx_t_7); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "ms_deisotope/_c/feature_map/profile_transform.pyx":402
 *             ps = ProfileSplitter(case)
 *             vallies = ps.locate_valleys(scale)
 *             if vallies:             # <<<<<<<<<<<<<<
 *                 splitter = vallies[0]
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)
 */

#line 402 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L7;

#line 402 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":406
 *                 out.extend(p for p in ps.split_valley(splitter) if len(p) > 0)
 *             else:
 *                 out.append(case)             # <<<<<<<<<<<<<<
 *         profiles = out
 *     return profiles
 */

#line 406 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 406 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_out, __pyx_v_case); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 406, __pyx_L1_error)

#line 406 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 406 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L7:;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":399
 *     for i in range(n_levels):
 *         out = []
 *         for case in profiles:             # <<<<<<<<<<<<<<
 *             ps = ProfileSplitter(case)
 *             vallies = ps.locate_valleys(scale)
 */

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 399 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":407
 *             else:
 *                 out.append(case)
 *         profiles = out             # <<<<<<<<<<<<<<
 *     return profiles
 * 
 */

#line 407 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_v_out);

#line 407 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_profiles, __pyx_v_out);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":397
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):
 *     for i in range(n_levels):             # <<<<<<<<<<<<<<
 *         out = []
 *         for case in profiles:
 */

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 397 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":408
 *                 out.append(case)
 *         profiles = out
 *     return profiles             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 408 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 408 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_profiles);

#line 408 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_profiles;

#line 408 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.split_valleys", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_i);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_out);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_case);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_vallies);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_gb_12ms_deisotope_2_c_11feature_map_17profile_transform_13split_valleys_2generator);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_profiles);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":414
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cdef size_t binsearch_view(cython.floating[::1] array, cython.floating x) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid
 */

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static size_t __pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__Pyx_memviewslice __pyx_v_array, float __pyx_v_x) {
  size_t __pyx_v_lo
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_hi
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_mid
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  float __pyx_v_y
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  float __pyx_v_err
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_r;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_2;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":419
 *         cython.floating y, err
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = array.shape[0]
 * 
 */

#line 419 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":420
 * 
 *     lo = 0
 *     hi = array.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 420 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_hi = (__pyx_v_array.shape[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":422
 *     hi = array.shape[0]
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         y = array[mid]
 */

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":423
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         y = array[mid]
 *         err = y - x
 */

#line 423 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":424
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         y = array[mid]             # <<<<<<<<<<<<<<
 *         err = y - x
 *         if hi - lo == 1:
 */

#line 424 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_v_mid;

#line 424 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_y = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_array.data) + __pyx_t_2)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":425
 *         mid = (hi + lo) / 2
 *         y = array[mid]
 *         err = y - x             # <<<<<<<<<<<<<<
 *         if hi - lo == 1:
 *             return mid
 */

#line 425 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_err = (__pyx_v_y - __pyx_v_x);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":426
 *         y = array[mid]
 *         err = y - x
 *         if hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif err > 0:
 */

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":427
 *         err = y - x
 *         if hi - lo == 1:
 *             return mid             # <<<<<<<<<<<<<<
 *         elif err > 0:
 *             hi = mid
 */

#line 427 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_r = __pyx_v_mid;

#line 427 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":426
 *         y = array[mid]
 *         err = y - x
 *         if hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif err > 0:
 */

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":428
 *         if hi - lo == 1:
 *             return mid
 *         elif err > 0:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_err > 0.0) != 0);

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":429
 *             return mid
 *         elif err > 0:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 429 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":428
 *         if hi - lo == 1:
 *             return mid
 *         elif err > 0:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":431
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":432
 *         else:
 *             lo = mid
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 432 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 432 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":414
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cdef size_t binsearch_view(cython.floating[::1] array, cython.floating x) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid
 */

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static size_t __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__Pyx_memviewslice __pyx_v_array, double __pyx_v_x) {
  size_t __pyx_v_lo
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_hi
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  size_t __pyx_v_mid
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_y
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_err
#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_r;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_1;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_2;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":419
 *         cython.floating y, err
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = array.shape[0]
 * 
 */

#line 419 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":420
 * 
 *     lo = 0
 *     hi = array.shape[0]             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 420 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_hi = (__pyx_v_array.shape[0]);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":422
 *     hi = array.shape[0]
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) / 2
 *         y = array[mid]
 */

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 422 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":423
 * 
 *     while hi != lo:
 *         mid = (hi + lo) / 2             # <<<<<<<<<<<<<<
 *         y = array[mid]
 *         err = y - x
 */

#line 423 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":424
 *     while hi != lo:
 *         mid = (hi + lo) / 2
 *         y = array[mid]             # <<<<<<<<<<<<<<
 *         err = y - x
 *         if hi - lo == 1:
 */

#line 424 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_v_mid;

#line 424 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_y = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_array.data) + __pyx_t_2)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":425
 *         mid = (hi + lo) / 2
 *         y = array[mid]
 *         err = y - x             # <<<<<<<<<<<<<<
 *         if hi - lo == 1:
 *             return mid
 */

#line 425 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_err = (__pyx_v_y - __pyx_v_x);

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":426
 *         y = array[mid]
 *         err = y - x
 *         if hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif err > 0:
 */

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":427
 *         err = y - x
 *         if hi - lo == 1:
 *             return mid             # <<<<<<<<<<<<<<
 *         elif err > 0:
 *             hi = mid
 */

#line 427 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_r = __pyx_v_mid;

#line 427 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":426
 *         y = array[mid]
 *         err = y - x
 *         if hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return mid
 *         elif err > 0:
 */

#line 426 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":428
 *         if hi - lo == 1:
 *             return mid
 *         elif err > 0:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = ((__pyx_v_err > 0.0) != 0);

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":429
 *             return mid
 *         elif err > 0:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 429 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/feature_map/profile_transform.pyx":428
 *         if hi - lo == 1:
 *             return mid
 *         elif err > 0:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5;

#line 428 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":431
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    /*else*/ {

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L5:;

#line 431 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":432
 *         else:
 *             lo = mid
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 432 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = 0;

#line 432 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":414
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cdef size_t binsearch_view(cython.floating[::1] array, cython.floating x) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid
 */

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 414 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point[] = "interpolate_point(signatures, args, kwargs, defaults)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"interpolate_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_args = 0
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_kwargs = 0
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[4] = {0,0,0,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  4: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  3:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_signatures = values[0];

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_args = values[1];

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_kwargs = values[2];

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_defaults = values[3];

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  Py_ssize_t __pyx_v_i
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyTypeObject *__pyx_v_ndarray = 0
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_memslice
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  Py_ssize_t __pyx_v_itemsize
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  CYTHON_UNUSED int __pyx_v_dtype_signed
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  char __pyx_v_kind
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_arg = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_dtype = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_arg_base = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_candidates = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_sig = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  int __pyx_v_match_found
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_src_sig = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  PyObject *__pyx_v_dst_type = NULL
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_4;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_5;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  long __pyx_t_7;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_memviewslice __pyx_t_8;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_9;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_10;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_11;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_15;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_16;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  Py_ssize_t __pyx_t_17;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_18;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("interpolate_point", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_INCREF(__pyx_v_kwargs);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(1 * 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  { Py_ssize_t __pyx_temp;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (__pyx_temp=0; __pyx_temp < 2; __pyx_temp++) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GIVEREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyList_SET_ITEM(__pyx_t_1, __pyx_temp, Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_4) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((!__pyx_t_4) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = ((PyObject *)__Pyx_ImportNumPyArrayTypeIfAvailable()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_itemsize = -1L;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((0 < __pyx_t_5) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_4) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L7_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_time, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (__pyx_t_4 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L7_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype = __pyx_t_6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L12;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __pyx_memoryview_check(__pyx_v_arg); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_arg_base = __pyx_t_6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_dtype = __pyx_t_6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L13;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_dtype = Py_None;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L13:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L12;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype = Py_None;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L12:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_itemsize = -1L;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_v_dtype != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_itemsize = __pyx_t_5;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __Pyx_PyObject_Ord(__pyx_t_6); if (unlikely(__pyx_t_7 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_kind = __pyx_t_7;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        switch (__pyx_v_kind) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'i':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'u':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'f':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L16_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L16_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L10_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L19_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L19_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L10_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'c':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'O':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          default: break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L22_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L22_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dc_float(__pyx_v_arg, 0); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L10_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L26_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L26_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dc_double(__pyx_v_arg, 0); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L10_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L10_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L10_break:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((1 < __pyx_t_5) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L29;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_v_kwargs != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_4) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = __pyx_t_4;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L30_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_intensity, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = (__pyx_t_4 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L30_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_intensity); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L29;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_INCREF(__pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_s);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GIVEREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L29:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L35;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = __pyx_memoryview_check(__pyx_v_arg); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF_SET(__pyx_v_arg_base, __pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L36;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L36:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L35;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_L35:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_itemsize = -1L;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_v_dtype != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_itemsize = __pyx_t_5;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_7 = __Pyx_PyObject_Ord(__pyx_t_1); if (unlikely(__pyx_t_7 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_kind = __pyx_t_7;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        switch (__pyx_v_kind) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'i':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'u':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'f':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L39_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L39_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L33_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L42_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_L42_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
            goto __pyx_L33_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'c':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          case 'O':

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          default: break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L45_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L45_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dc_float(__pyx_v_arg, 0); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L33_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (!__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L49_bool_binop_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L49_bool_binop_done:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dc_double(__pyx_v_arg, 0); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        goto __pyx_L33_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyErr_Clear(); 

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L33_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L33_break:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_candidates = ((PyObject*)__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_5 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(__pyx_v_signatures == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_9), (&__pyx_t_10)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_t_6;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  while (1) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_9, &__pyx_t_5, &__pyx_t_6, NULL, NULL, __pyx_t_10);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 == 0)) break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_match_found = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_14 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_14)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_14);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_kp_s__2) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_12);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_split); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_12 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (likely(__pyx_t_12)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(__pyx_t_12);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_INCREF(function);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_12, __pyx_kp_s__3) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF_SET(__pyx_v_src_sig, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_15 = PyList_GET_SIZE(__pyx_v_dest_sig); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_16 = __pyx_t_15;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_v_i = __pyx_t_17;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_dest_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_src_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_13 = PyObject_RichCompare(__pyx_t_6, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_match_found = 1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L57;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          __pyx_v_match_found = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
          goto __pyx_L55_break;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        __pyx_L57:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_L55_break:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_3 = (__pyx_v_match_found != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (__pyx_t_3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __pyx_t_18 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_18 == ((int)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((!__pyx_t_3) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_9 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((__pyx_t_9 > 1) != 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_v_signatures == Py_None)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dest_sig);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_ndarray);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_arg);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dtype);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_arg_base);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_candidates);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_sig);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_src_sig);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_dst_type);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_v_kwargs);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_fuse_0_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_contrib
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_tmp
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_0interpolate_point", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":453
 *         double inten_j, inten_j1
 * 
 *     j = binsearch_view(time, x)             # <<<<<<<<<<<<<<
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 */

#line 453 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_j = __pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__pyx_v_time, __pyx_v_x);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":454
 * 
 *     j = binsearch_view(time, x)
 *     time_j = time[j]             # <<<<<<<<<<<<<<
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 */

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_j;

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_time_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j <= __pyx_v_x) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (((__pyx_v_j + 1) < (__pyx_v_time.shape[0])) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":456
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]             # <<<<<<<<<<<<<<
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 */

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":457
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 */

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":458
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]             # <<<<<<<<<<<<<<
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 */

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j > __pyx_v_x) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6_bool_binop_done;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_j > 0) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6_bool_binop_done:;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":460
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 */

#line 460 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = __pyx_v_time_j;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":461
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]             # <<<<<<<<<<<<<<
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]
 */

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":462
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]             # <<<<<<<<<<<<<<
 *         inten_j = time[j - 1]
 *     else:
 */

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":463
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]             # <<<<<<<<<<<<<<
 *     else:
 *         return 0.0
 */

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":465
 *         inten_j = time[j - 1]
 *     else:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 */

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = 0.0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":466
 *     else:
 *         return 0.0
 *     tmp = time_j1 - time_j             # <<<<<<<<<<<<<<
 *     if tmp == 0:
 *         contrib = 0.0
 */

#line 466 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_tmp = (__pyx_v_time_j1 - __pyx_v_time_j);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((__pyx_v_tmp == 0.0) != 0);

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":468
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 *         contrib = 0.0             # <<<<<<<<<<<<<<
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 */

#line 468 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = 0.0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8;

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":470
 *         contrib = 0.0
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)             # <<<<<<<<<<<<<<
 *     return contrib
 */

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = (((__pyx_v_inten_j * (__pyx_v_time_j1 - __pyx_v_x)) + (__pyx_v_inten_j1 * (__pyx_v_x - __pyx_v_time_j))) / (__pyx_v_time_j1 - __pyx_v_time_j));

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L8:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":471
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 *     return contrib             # <<<<<<<<<<<<<<
 */

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_contrib;

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_0_0interpolate_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_time = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_intensity = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_x
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_0interpolate_point (wrapper)", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_time,&__pyx_n_s_intensity,&__pyx_n_s_x,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time)) != 0)) kw_args--;
        else 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_0interpolate_point", 1, 3, 3, 1); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_0interpolate_point", 1, 3, 3, 2); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fuse_0_0interpolate_point") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time = __Pyx_PyObject_to_MemoryviewSlice_dc_float(values[0], PyBUF_WRITABLE); if (unlikely(!__pyx_v_time.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_intensity = __Pyx_PyObject_to_MemoryviewSlice_dc_float(values[1], PyBUF_WRITABLE); if (unlikely(!__pyx_v_intensity.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_0interpolate_point", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_0_0interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_32__pyx_fuse_0_0interpolate_point(__pyx_self, __pyx_v_time, __pyx_v_intensity, __pyx_v_x);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_32__pyx_fuse_0_0interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_0interpolate_point", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_time.memview)) { __Pyx_RaiseUnboundLocalError("time"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_intensity.memview)) { __Pyx_RaiseUnboundLocalError("intensity"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_fuse_0_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__pyx_v_time, __pyx_v_intensity, __pyx_v_x, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_0_0interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_time, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_intensity, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_fuse_0_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_contrib
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_tmp
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_1interpolate_point", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":453
 *         double inten_j, inten_j1
 * 
 *     j = binsearch_view(time, x)             # <<<<<<<<<<<<<<
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 */

#line 453 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_j = __pyx_fuse_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__pyx_v_time, __pyx_v_x);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":454
 * 
 *     j = binsearch_view(time, x)
 *     time_j = time[j]             # <<<<<<<<<<<<<<
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 */

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_j;

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_time_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j <= __pyx_v_x) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (((__pyx_v_j + 1) < (__pyx_v_time.shape[0])) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":456
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]             # <<<<<<<<<<<<<<
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 */

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":457
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 */

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":458
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]             # <<<<<<<<<<<<<<
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 */

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j > __pyx_v_x) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6_bool_binop_done;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_j > 0) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6_bool_binop_done:;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":460
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 */

#line 460 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = __pyx_v_time_j;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":461
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]             # <<<<<<<<<<<<<<
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]
 */

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":462
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]             # <<<<<<<<<<<<<<
 *         inten_j = time[j - 1]
 *     else:
 */

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":463
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]             # <<<<<<<<<<<<<<
 *     else:
 *         return 0.0
 */

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":465
 *         inten_j = time[j - 1]
 *     else:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 */

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = 0.0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":466
 *     else:
 *         return 0.0
 *     tmp = time_j1 - time_j             # <<<<<<<<<<<<<<
 *     if tmp == 0:
 *         contrib = 0.0
 */

#line 466 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_tmp = (__pyx_v_time_j1 - __pyx_v_time_j);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((__pyx_v_tmp == 0.0) != 0);

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":468
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 *         contrib = 0.0             # <<<<<<<<<<<<<<
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 */

#line 468 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = 0.0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8;

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":470
 *         contrib = 0.0
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)             # <<<<<<<<<<<<<<
 *     return contrib
 */

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = (((__pyx_v_inten_j * (__pyx_v_time_j1 - __pyx_v_x)) + (__pyx_v_inten_j1 * (__pyx_v_x - __pyx_v_time_j))) / (__pyx_v_time_j1 - __pyx_v_time_j));

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L8:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":471
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 *     return contrib             # <<<<<<<<<<<<<<
 */

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_contrib;

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_0_1interpolate_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_time = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_intensity = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_x
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_1interpolate_point (wrapper)", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_time,&__pyx_n_s_intensity,&__pyx_n_s_x,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time)) != 0)) kw_args--;
        else 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_1interpolate_point", 1, 3, 3, 1); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_1interpolate_point", 1, 3, 3, 2); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fuse_0_1interpolate_point") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time = __Pyx_PyObject_to_MemoryviewSlice_dc_float(values[0], PyBUF_WRITABLE); if (unlikely(!__pyx_v_time.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_intensity = __Pyx_PyObject_to_MemoryviewSlice_dc_double(values[1], PyBUF_WRITABLE); if (unlikely(!__pyx_v_intensity.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fuse_0_1interpolate_point", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_0_1interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_34__pyx_fuse_0_1interpolate_point(__pyx_self, __pyx_v_time, __pyx_v_intensity, __pyx_v_x);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_34__pyx_fuse_0_1interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0_1interpolate_point", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_time.memview)) { __Pyx_RaiseUnboundLocalError("time"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_intensity.memview)) { __Pyx_RaiseUnboundLocalError("intensity"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_fuse_0_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__pyx_v_time, __pyx_v_intensity, __pyx_v_x, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_0_1interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_time, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_intensity, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_fuse_1_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_contrib
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_tmp
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_0interpolate_point", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":453
 *         double inten_j, inten_j1
 * 
 *     j = binsearch_view(time, x)             # <<<<<<<<<<<<<<
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 */

#line 453 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_j = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__pyx_v_time, __pyx_v_x);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":454
 * 
 *     j = binsearch_view(time, x)
 *     time_j = time[j]             # <<<<<<<<<<<<<<
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 */

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_j;

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_time_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j <= __pyx_v_x) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (((__pyx_v_j + 1) < (__pyx_v_time.shape[0])) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":456
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]             # <<<<<<<<<<<<<<
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 */

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":457
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 */

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":458
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]             # <<<<<<<<<<<<<<
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 */

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j > __pyx_v_x) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6_bool_binop_done;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_j > 0) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6_bool_binop_done:;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":460
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 */

#line 460 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = __pyx_v_time_j;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":461
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]             # <<<<<<<<<<<<<<
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]
 */

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((float *) ( /* dim=0 */ ((char *) (((float *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":462
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]             # <<<<<<<<<<<<<<
 *         inten_j = time[j - 1]
 *     else:
 */

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":463
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]             # <<<<<<<<<<<<<<
 *     else:
 *         return 0.0
 */

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":465
 *         inten_j = time[j - 1]
 *     else:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 */

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = 0.0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":466
 *     else:
 *         return 0.0
 *     tmp = time_j1 - time_j             # <<<<<<<<<<<<<<
 *     if tmp == 0:
 *         contrib = 0.0
 */

#line 466 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_tmp = (__pyx_v_time_j1 - __pyx_v_time_j);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((__pyx_v_tmp == 0.0) != 0);

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":468
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 *         contrib = 0.0             # <<<<<<<<<<<<<<
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 */

#line 468 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = 0.0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8;

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":470
 *         contrib = 0.0
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)             # <<<<<<<<<<<<<<
 *     return contrib
 */

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = (((__pyx_v_inten_j * (__pyx_v_time_j1 - __pyx_v_x)) + (__pyx_v_inten_j1 * (__pyx_v_x - __pyx_v_time_j))) / (__pyx_v_time_j1 - __pyx_v_time_j));

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L8:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":471
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 *     return contrib             # <<<<<<<<<<<<<<
 */

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_contrib;

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_1_0interpolate_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_time = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_intensity = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_x
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_0interpolate_point (wrapper)", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_time,&__pyx_n_s_intensity,&__pyx_n_s_x,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time)) != 0)) kw_args--;
        else 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_0interpolate_point", 1, 3, 3, 1); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_0interpolate_point", 1, 3, 3, 2); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fuse_1_0interpolate_point") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time = __Pyx_PyObject_to_MemoryviewSlice_dc_double(values[0], PyBUF_WRITABLE); if (unlikely(!__pyx_v_time.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_intensity = __Pyx_PyObject_to_MemoryviewSlice_dc_float(values[1], PyBUF_WRITABLE); if (unlikely(!__pyx_v_intensity.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_0interpolate_point", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_1_0interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_36__pyx_fuse_1_0interpolate_point(__pyx_self, __pyx_v_time, __pyx_v_intensity, __pyx_v_x);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_36__pyx_fuse_1_0interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_0interpolate_point", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_time.memview)) { __Pyx_RaiseUnboundLocalError("time"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_intensity.memview)) { __Pyx_RaiseUnboundLocalError("intensity"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_fuse_1_0__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__pyx_v_time, __pyx_v_intensity, __pyx_v_x, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_1_0interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_time, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_intensity, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static double __pyx_fuse_1_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_contrib
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_tmp
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_time_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_inten_j1
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  double __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  size_t __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_2;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_1interpolate_point", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":453
 *         double inten_j, inten_j1
 * 
 *     j = binsearch_view(time, x)             # <<<<<<<<<<<<<<
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 */

#line 453 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_j = __pyx_fuse_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_binsearch_view(__pyx_v_time, __pyx_v_x);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":454
 * 
 *     j = binsearch_view(time, x)
 *     time_j = time[j]             # <<<<<<<<<<<<<<
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 */

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_v_j;

#line 454 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_time_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j <= __pyx_v_x) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L4_bool_binop_done;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = (((__pyx_v_j + 1) < (__pyx_v_time.shape[0])) != 0);

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_bool_binop_done:;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":456
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]             # <<<<<<<<<<<<<<
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 */

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 456 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":457
 *     if time_j <= x and j + 1 < time.shape[0]:
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 */

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 457 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":458
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]             # <<<<<<<<<<<<<<
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 */

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j + 1);

#line 458 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":455
 *     j = binsearch_view(time, x)
 *     time_j = time[j]
 *     if time_j <= x and j + 1 < time.shape[0]:             # <<<<<<<<<<<<<<
 *         time_j1 = time[j + 1]
 *         inten_j = intensity[j]
 */

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 455 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_time_j > __pyx_v_x) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_3) {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  } else {

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L6_bool_binop_done;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = ((__pyx_v_j > 0) != 0);

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L6_bool_binop_done:;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":460
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j             # <<<<<<<<<<<<<<
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 */

#line 460 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j1 = __pyx_v_time_j;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":461
 *     elif time_j > x and j > 0:
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]             # <<<<<<<<<<<<<<
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]
 */

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = __pyx_v_j;

#line 461 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j1 = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_intensity.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":462
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]             # <<<<<<<<<<<<<<
 *         inten_j = time[j - 1]
 *     else:
 */

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 462 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":463
 *         inten_j1 = intensity[j]
 *         time_j = time[j - 1]
 *         inten_j = time[j - 1]             # <<<<<<<<<<<<<<
 *     else:
 *         return 0.0
 */

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_t_1 = (__pyx_v_j - 1);

#line 463 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_inten_j = (*((double *) ( /* dim=0 */ ((char *) (((double *) __pyx_v_time.data) + __pyx_t_1)) )));

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":459
 *         inten_j = intensity[j]
 *         inten_j1 = intensity[j + 1]
 *     elif time_j > x and j > 0:             # <<<<<<<<<<<<<<
 *         time_j1 = time_j
 *         inten_j1 = intensity[j]
 */

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L3;

#line 459 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":465
 *         inten_j = time[j - 1]
 *     else:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 */

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_r = 0.0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L0;

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 465 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":466
 *     else:
 *         return 0.0
 *     tmp = time_j1 - time_j             # <<<<<<<<<<<<<<
 *     if tmp == 0:
 *         contrib = 0.0
 */

#line 466 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_v_tmp = (__pyx_v_time_j1 - __pyx_v_time_j);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = ((__pyx_v_tmp == 0.0) != 0);

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":468
 *     tmp = time_j1 - time_j
 *     if tmp == 0:
 *         contrib = 0.0             # <<<<<<<<<<<<<<
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 */

#line 468 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = 0.0;

    /* "ms_deisotope/_c/feature_map/profile_transform.pyx":467
 *         return 0.0
 *     tmp = time_j1 - time_j
 *     if tmp == 0:             # <<<<<<<<<<<<<<
 *         contrib = 0.0
 *     else:
 */

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    goto __pyx_L8;

#line 467 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":470
 *         contrib = 0.0
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)             # <<<<<<<<<<<<<<
 *     return contrib
 */

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /*else*/ {

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_contrib = (((__pyx_v_inten_j * (__pyx_v_time_j1 - __pyx_v_x)) + (__pyx_v_inten_j1 * (__pyx_v_x - __pyx_v_time_j))) / (__pyx_v_time_j1 - __pyx_v_time_j));

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 470 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L8:;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":471
 *     else:
 *         contrib = ((inten_j * (time_j1 - x)) + (inten_j1 * (x - time_j))) / (time_j1 - time_j)
 *     return contrib             # <<<<<<<<<<<<<<
 */

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_v_contrib;

#line 471 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
/* Python wrapper */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
{"__pyx_fuse_1_1interpolate_point", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_18interpolate_point};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_time = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  __Pyx_memviewslice __pyx_v_intensity = { 0, 0, { 0 }, { 0 }, { 0 } }
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;
  double __pyx_v_x
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_1interpolate_point (wrapper)", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_time,&__pyx_n_s_intensity,&__pyx_n_s_x,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    PyObject* values[3] = {0,0,0};

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    if (unlikely(__pyx_kwds)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      Py_ssize_t kw_args;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {
        case  3: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0: break;
        default: 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      switch (pos_args) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  0:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time)) != 0)) kw_args--;
        else 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  1:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_1interpolate_point", 1, 3, 3, 1); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        CYTHON_FALLTHROUGH;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        case  2:

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_1interpolate_point", 1, 3, 3, 2); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      if (unlikely(kw_args > 0)) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fuse_1_1interpolate_point") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      goto __pyx_L5_argtuple_error;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    } else {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_time = __Pyx_PyObject_to_MemoryviewSlice_dc_double(values[0], PyBUF_WRITABLE); if (unlikely(!__pyx_v_time.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_intensity = __Pyx_PyObject_to_MemoryviewSlice_dc_double(values[1], PyBUF_WRITABLE); if (unlikely(!__pyx_v_intensity.memview)) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fuse_1_1interpolate_point", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__PYX_ERR(0, 446, __pyx_L3_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L3_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_1_1interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_38__pyx_fuse_1_1interpolate_point(__pyx_self, __pyx_v_time, __pyx_v_intensity, __pyx_v_x);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_38__pyx_fuse_1_1interpolate_point(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_time, __Pyx_memviewslice __pyx_v_intensity, double __pyx_v_x) {

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyDeclarations

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_lineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  const char *__pyx_filename = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  int __pyx_clineno = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1_1interpolate_point", 0);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_time.memview)) { __Pyx_RaiseUnboundLocalError("time"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (unlikely(!__pyx_v_intensity.memview)) { __Pyx_RaiseUnboundLocalError("intensity"); __PYX_ERR(0, 446, __pyx_L1_error) }

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_fuse_1_1__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_interpolate_point(__pyx_v_time, __pyx_v_intensity, __pyx_v_x, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = __pyx_t_1;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  goto __pyx_L0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"


#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  /* function exit code */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L1_error:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_fuse_1_1interpolate_point", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_r = NULL;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_L0:;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_time, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __PYX_XDEC_MEMVIEW(&__pyx_v_intensity, 1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_RefNannyFinishContext();

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  return __pyx_r;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PeakBoundary(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_27__pyx_unpickle_PeakBoundary(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_26__pyx_unpickle_PeakBoundary[] = "__pyx_unpickle_PeakBoundary(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_27__pyx_unpickle_PeakBoundary = 
#line 1 "(tree fragment)"
{"__pyx_unpickle_PeakBoundary", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_27__pyx_unpickle_PeakBoundary, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_26__pyx_unpickle_PeakBoundary};

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_27__pyx_unpickle_PeakBoundary(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0
#line 1 "(tree fragment)"
;
  long __pyx_v___pyx_checksum
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_state = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeakBoundary (wrapper)", 0);

#line 1 "(tree fragment)"
  {

#line 1 "(tree fragment)"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};

#line 1 "(tree fragment)"
    PyObject* values[3] = {0,0,0};

#line 1 "(tree fragment)"
    if (unlikely(__pyx_kwds)) {

#line 1 "(tree fragment)"
      Py_ssize_t kw_args;

#line 1 "(tree fragment)"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1 "(tree fragment)"
      switch (pos_args) {
        case  3: 
#line 1 "(tree fragment)"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1 "(tree fragment)"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1 "(tree fragment)"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  0: break;
        default: 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1 "(tree fragment)"
      switch (pos_args) {

#line 1 "(tree fragment)"
        case  0:

#line 1 "(tree fragment)"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  1:

#line 1 "(tree fragment)"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeakBoundary", 1, 3, 3, 1); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  2:

#line 1 "(tree fragment)"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeakBoundary", 1, 3, 3, 2); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      if (unlikely(kw_args > 0)) {

#line 1 "(tree fragment)"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_PeakBoundary") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 1 "(tree fragment)"
      goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
    } else {

#line 1 "(tree fragment)"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
    }

#line 1 "(tree fragment)"
    __pyx_v___pyx_type = values[0];

#line 1 "(tree fragment)"
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
    __pyx_v___pyx_state = values[2];

#line 1 "(tree fragment)"
  }

#line 1 "(tree fragment)"
  goto __pyx_L4_argument_unpacking_done;

#line 1 "(tree fragment)"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeakBoundary", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
  __pyx_L3_error:;

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_PeakBoundary", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return NULL;

#line 1 "(tree fragment)"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_26__pyx_unpickle_PeakBoundary(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_26__pyx_unpickle_PeakBoundary(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_result = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeakBoundary", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xf456ba5, 0xf92ae08, 0x36c730a):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__7, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 4 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xf456ba5, 0xf92ae08, 0x36c730a):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)
 */

#line 5 "(tree fragment)"
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __pyx_v___pyx_PickleError = __pyx_t_1;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xf456ba5, 0xf92ae08, 0x36c730a):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */

#line 6 "(tree fragment)"
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; 
#line 6 "(tree fragment)"
__pyx_t_6 = NULL;

#line 6 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 6 "(tree fragment)"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);

#line 6 "(tree fragment)"
      if (likely(__pyx_t_6)) {

#line 6 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 6 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_6);

#line 6 "(tree fragment)"
        __Pyx_INCREF(function);

#line 6 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 6 "(tree fragment)"
      }

#line 6 "(tree fragment)"
    }

#line 6 "(tree fragment)"
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 6 "(tree fragment)"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 6 "(tree fragment)"
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xf456ba5, 0xf92ae08, 0x36c730a):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 */

#line 7 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "(tree fragment)"
  __pyx_t_5 = NULL;

#line 7 "(tree fragment)"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 7 "(tree fragment)"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);

#line 7 "(tree fragment)"
    if (likely(__pyx_t_5)) {

#line 7 "(tree fragment)"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 7 "(tree fragment)"
      __Pyx_INCREF(__pyx_t_5);

#line 7 "(tree fragment)"
      __Pyx_INCREF(function);

#line 7 "(tree fragment)"
      __Pyx_DECREF_SET(__pyx_t_1, function);

#line 7 "(tree fragment)"
    }

#line 7 "(tree fragment)"
  }

#line 7 "(tree fragment)"
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);

#line 7 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 7 "(tree fragment)"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 7 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 7 "(tree fragment)"
  __pyx_v___pyx_result = __pyx_t_4;

#line 7 "(tree fragment)"
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);

#line 8 "(tree fragment)"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 8 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):
 */

#line 9 "(tree fragment)"
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __pyx_t_4 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_PeakBoundary__set_state(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 9 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = PeakBoundary.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 */

#line 10 "(tree fragment)"
  __Pyx_XDECREF(__pyx_r);

#line 10 "(tree fragment)"
  __Pyx_INCREF(__pyx_v___pyx_result);

#line 10 "(tree fragment)"
  __pyx_r = __pyx_v___pyx_result;

#line 10 "(tree fragment)"
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeakBoundary(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_PeakBoundary", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_result);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_PeakBoundary__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {

#line 11 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 11 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 11 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 11 "(tree fragment)"
  double __pyx_t_2;

#line 11 "(tree fragment)"
  int __pyx_t_3;

#line 11 "(tree fragment)"
  Py_ssize_t __pyx_t_4;

#line 11 "(tree fragment)"
  int __pyx_t_5;

#line 11 "(tree fragment)"
  int __pyx_t_6;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_9 = NULL;

#line 11 "(tree fragment)"
  int __pyx_lineno = 0;

#line 11 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 11 "(tree fragment)"
  int __pyx_clineno = 0;

#line 11 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeakBoundary__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->first_minimum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->first_minimum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->maximum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->maximum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->second_minimum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->second_minimum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->total_distance = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 13 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 13 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 13 "(tree fragment)"
    __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);

#line 13 "(tree fragment)"
  if (__pyx_t_5) {

#line 13 "(tree fragment)"
  } else {

#line 13 "(tree fragment)"
    __pyx_t_3 = __pyx_t_5;

#line 13 "(tree fragment)"
    goto __pyx_L4_bool_binop_done;

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_6 = (__pyx_t_5 != 0);

#line 13 "(tree fragment)"
  __pyx_t_3 = __pyx_t_6;

#line 13 "(tree fragment)"
  __pyx_L4_bool_binop_done:;

#line 13 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 14 "(tree fragment)"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 14 "(tree fragment)"
      __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __pyx_t_9 = NULL;

#line 14 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {

#line 14 "(tree fragment)"
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);

#line 14 "(tree fragment)"
      if (likely(__pyx_t_9)) {

#line 14 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);

#line 14 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_9);

#line 14 "(tree fragment)"
        __Pyx_INCREF(function);

#line 14 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_8, function);

#line 14 "(tree fragment)"
      }

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);

#line 14 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 13 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /* function exit code */

#line 11 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_9);

#line 11 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_PeakBoundary__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
  __pyx_r = 0;

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  return __pyx_r;

#line 11 "(tree fragment)"
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ValleyPoint(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_29__pyx_unpickle_ValleyPoint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_28__pyx_unpickle_ValleyPoint[] = "__pyx_unpickle_ValleyPoint(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_29__pyx_unpickle_ValleyPoint = 
#line 1 "(tree fragment)"
{"__pyx_unpickle_ValleyPoint", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_29__pyx_unpickle_ValleyPoint, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_28__pyx_unpickle_ValleyPoint};

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_29__pyx_unpickle_ValleyPoint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0
#line 1 "(tree fragment)"
;
  long __pyx_v___pyx_checksum
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_state = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ValleyPoint (wrapper)", 0);

#line 1 "(tree fragment)"
  {

#line 1 "(tree fragment)"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};

#line 1 "(tree fragment)"
    PyObject* values[3] = {0,0,0};

#line 1 "(tree fragment)"
    if (unlikely(__pyx_kwds)) {

#line 1 "(tree fragment)"
      Py_ssize_t kw_args;

#line 1 "(tree fragment)"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1 "(tree fragment)"
      switch (pos_args) {
        case  3: 
#line 1 "(tree fragment)"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1 "(tree fragment)"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1 "(tree fragment)"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  0: break;
        default: 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1 "(tree fragment)"
      switch (pos_args) {

#line 1 "(tree fragment)"
        case  0:

#line 1 "(tree fragment)"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  1:

#line 1 "(tree fragment)"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ValleyPoint", 1, 3, 3, 1); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  2:

#line 1 "(tree fragment)"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ValleyPoint", 1, 3, 3, 2); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      if (unlikely(kw_args > 0)) {

#line 1 "(tree fragment)"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ValleyPoint") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 1 "(tree fragment)"
      goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
    } else {

#line 1 "(tree fragment)"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
    }

#line 1 "(tree fragment)"
    __pyx_v___pyx_type = values[0];

#line 1 "(tree fragment)"
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
    __pyx_v___pyx_state = values[2];

#line 1 "(tree fragment)"
  }

#line 1 "(tree fragment)"
  goto __pyx_L4_argument_unpacking_done;

#line 1 "(tree fragment)"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ValleyPoint", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
  __pyx_L3_error:;

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ValleyPoint", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return NULL;

#line 1 "(tree fragment)"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_28__pyx_unpickle_ValleyPoint(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_28__pyx_unpickle_ValleyPoint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_result = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ValleyPoint", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x12d8104, 0x66d5806, 0xe251028):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__8, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 4 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x12d8104, 0x66d5806, 0xe251028):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)
 */

#line 5 "(tree fragment)"
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __pyx_v___pyx_PickleError = __pyx_t_1;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x12d8104, 0x66d5806, 0xe251028):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */

#line 6 "(tree fragment)"
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; 
#line 6 "(tree fragment)"
__pyx_t_6 = NULL;

#line 6 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 6 "(tree fragment)"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);

#line 6 "(tree fragment)"
      if (likely(__pyx_t_6)) {

#line 6 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 6 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_6);

#line 6 "(tree fragment)"
        __Pyx_INCREF(function);

#line 6 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 6 "(tree fragment)"
      }

#line 6 "(tree fragment)"
    }

#line 6 "(tree fragment)"
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 6 "(tree fragment)"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 6 "(tree fragment)"
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x12d8104, 0x66d5806, 0xe251028):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 */

#line 7 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "(tree fragment)"
  __pyx_t_5 = NULL;

#line 7 "(tree fragment)"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 7 "(tree fragment)"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);

#line 7 "(tree fragment)"
    if (likely(__pyx_t_5)) {

#line 7 "(tree fragment)"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 7 "(tree fragment)"
      __Pyx_INCREF(__pyx_t_5);

#line 7 "(tree fragment)"
      __Pyx_INCREF(function);

#line 7 "(tree fragment)"
      __Pyx_DECREF_SET(__pyx_t_1, function);

#line 7 "(tree fragment)"
    }

#line 7 "(tree fragment)"
  }

#line 7 "(tree fragment)"
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);

#line 7 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 7 "(tree fragment)"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 7 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 7 "(tree fragment)"
  __pyx_v___pyx_result = __pyx_t_4;

#line 7 "(tree fragment)"
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);

#line 8 "(tree fragment)"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 8 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):
 */

#line 9 "(tree fragment)"
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __pyx_t_4 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ValleyPoint__set_state(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 9 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x12d8104, 0x66d5806, 0xe251028) = (first_maximum, first_maximum_index, minimum, minimum_index, second_maximum, second_maximum_index, total_distance))" % __pyx_checksum)
 *     __pyx_result = ValleyPoint.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 */

#line 10 "(tree fragment)"
  __Pyx_XDECREF(__pyx_r);

#line 10 "(tree fragment)"
  __Pyx_INCREF(__pyx_v___pyx_result);

#line 10 "(tree fragment)"
  __pyx_r = __pyx_v___pyx_result;

#line 10 "(tree fragment)"
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ValleyPoint(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ValleyPoint", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_result);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ValleyPoint__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {

#line 11 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 11 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 11 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 11 "(tree fragment)"
  double __pyx_t_2;

#line 11 "(tree fragment)"
  int __pyx_t_3;

#line 11 "(tree fragment)"
  Py_ssize_t __pyx_t_4;

#line 11 "(tree fragment)"
  int __pyx_t_5;

#line 11 "(tree fragment)"
  int __pyx_t_6;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_9 = NULL;

#line 11 "(tree fragment)"
  int __pyx_lineno = 0;

#line 11 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 11 "(tree fragment)"
  int __pyx_clineno = 0;

#line 11 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ValleyPoint__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->first_maximum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->first_maximum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->minimum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->minimum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->second_maximum = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->second_maximum_index = __pyx_t_2;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->total_distance = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 13 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 13 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 13 "(tree fragment)"
    __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);

#line 13 "(tree fragment)"
  if (__pyx_t_5) {

#line 13 "(tree fragment)"
  } else {

#line 13 "(tree fragment)"
    __pyx_t_3 = __pyx_t_5;

#line 13 "(tree fragment)"
    goto __pyx_L4_bool_binop_done;

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_6 = (__pyx_t_5 != 0);

#line 13 "(tree fragment)"
  __pyx_t_3 = __pyx_t_6;

#line 13 "(tree fragment)"
  __pyx_L4_bool_binop_done:;

#line 13 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_8);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 14 "(tree fragment)"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 14 "(tree fragment)"
      __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __pyx_t_9 = NULL;

#line 14 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {

#line 14 "(tree fragment)"
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);

#line 14 "(tree fragment)"
      if (likely(__pyx_t_9)) {

#line 14 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);

#line 14 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_9);

#line 14 "(tree fragment)"
        __Pyx_INCREF(function);

#line 14 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_8, function);

#line 14 "(tree fragment)"
      }

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);

#line 14 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */

#line 13 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ValleyPoint__set_state(<ValleyPoint> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ValleyPoint__set_state(ValleyPoint __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.first_maximum = __pyx_state[0]; __pyx_result.first_maximum_index = __pyx_state[1]; __pyx_result.minimum = __pyx_state[2]; __pyx_result.minimum_index = __pyx_state[3]; __pyx_result.second_maximum = __pyx_state[4]; __pyx_result.second_maximum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /* function exit code */

#line 11 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_9);

#line 11 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ValleyPoint__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
  __pyx_r = 0;

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  return __pyx_r;

#line 11 "(tree fragment)"
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ProfileSplitter(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_31__pyx_unpickle_ProfileSplitter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1 "(tree fragment)"
static char __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_30__pyx_unpickle_ProfileSplitter[] = "__pyx_unpickle_ProfileSplitter(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_31__pyx_unpickle_ProfileSplitter = 
#line 1 "(tree fragment)"
{"__pyx_unpickle_ProfileSplitter", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_31__pyx_unpickle_ProfileSplitter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_30__pyx_unpickle_ProfileSplitter};

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_31__pyx_unpickle_ProfileSplitter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0
#line 1 "(tree fragment)"
;
  long __pyx_v___pyx_checksum
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_state = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProfileSplitter (wrapper)", 0);

#line 1 "(tree fragment)"
  {

#line 1 "(tree fragment)"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};

#line 1 "(tree fragment)"
    PyObject* values[3] = {0,0,0};

#line 1 "(tree fragment)"
    if (unlikely(__pyx_kwds)) {

#line 1 "(tree fragment)"
      Py_ssize_t kw_args;

#line 1 "(tree fragment)"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1 "(tree fragment)"
      switch (pos_args) {
        case  3: 
#line 1 "(tree fragment)"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1 "(tree fragment)"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1 "(tree fragment)"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  0: break;
        default: 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1 "(tree fragment)"
      switch (pos_args) {

#line 1 "(tree fragment)"
        case  0:

#line 1 "(tree fragment)"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  1:

#line 1 "(tree fragment)"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProfileSplitter", 1, 3, 3, 1); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  2:

#line 1 "(tree fragment)"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProfileSplitter", 1, 3, 3, 2); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      if (unlikely(kw_args > 0)) {

#line 1 "(tree fragment)"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ProfileSplitter") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 1 "(tree fragment)"
      goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
    } else {

#line 1 "(tree fragment)"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
    }

#line 1 "(tree fragment)"
    __pyx_v___pyx_type = values[0];

#line 1 "(tree fragment)"
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
    __pyx_v___pyx_state = values[2];

#line 1 "(tree fragment)"
  }

#line 1 "(tree fragment)"
  goto __pyx_L4_argument_unpacking_done;

#line 1 "(tree fragment)"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ProfileSplitter", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
  __pyx_L3_error:;

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ProfileSplitter", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return NULL;

#line 1 "(tree fragment)"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_30__pyx_unpickle_ProfileSplitter(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_12ms_deisotope_2_c_11feature_map_17profile_transform_30__pyx_unpickle_ProfileSplitter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_result = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProfileSplitter", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc84cf1f, 0x9cd8936, 0xf969a88):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__9, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 4 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc84cf1f, 0x9cd8936, 0xf969a88):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)
 */

#line 5 "(tree fragment)"
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __pyx_v___pyx_PickleError = __pyx_t_1;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xc84cf1f, 0x9cd8936, 0xf969a88):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */

#line 6 "(tree fragment)"
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_3, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; 
#line 6 "(tree fragment)"
__pyx_t_6 = NULL;

#line 6 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 6 "(tree fragment)"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);

#line 6 "(tree fragment)"
      if (likely(__pyx_t_6)) {

#line 6 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 6 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_6);

#line 6 "(tree fragment)"
        __Pyx_INCREF(function);

#line 6 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 6 "(tree fragment)"
      }

#line 6 "(tree fragment)"
    }

#line 6 "(tree fragment)"
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 6 "(tree fragment)"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 6 "(tree fragment)"
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xc84cf1f, 0x9cd8936, 0xf969a88):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 */

#line 7 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "(tree fragment)"
  __pyx_t_5 = NULL;

#line 7 "(tree fragment)"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 7 "(tree fragment)"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);

#line 7 "(tree fragment)"
    if (likely(__pyx_t_5)) {

#line 7 "(tree fragment)"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 7 "(tree fragment)"
      __Pyx_INCREF(__pyx_t_5);

#line 7 "(tree fragment)"
      __Pyx_INCREF(function);

#line 7 "(tree fragment)"
      __Pyx_DECREF_SET(__pyx_t_1, function);

#line 7 "(tree fragment)"
    }

#line 7 "(tree fragment)"
  }

#line 7 "(tree fragment)"
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);

#line 7 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 7 "(tree fragment)"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 7 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 7 "(tree fragment)"
  __pyx_v___pyx_result = __pyx_t_4;

#line 7 "(tree fragment)"
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);

#line 8 "(tree fragment)"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 8 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):
 */

#line 9 "(tree fragment)"
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __pyx_t_4 = __pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ProfileSplitter__set_state(((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 9 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xc84cf1f, 0x9cd8936, 0xf969a88) = (partition_sites, profile, xs, ys))" % __pyx_checksum)
 *     __pyx_result = ProfileSplitter.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 */

#line 10 "(tree fragment)"
  __Pyx_XDECREF(__pyx_r);

#line 10 "(tree fragment)"
  __Pyx_INCREF(__pyx_v___pyx_result);

#line 10 "(tree fragment)"
  __pyx_r = __pyx_v___pyx_result;

#line 10 "(tree fragment)"
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ProfileSplitter(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ProfileSplitter", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_result);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_unpickle_ProfileSplitter__set_state(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {

#line 11 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 11 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 11 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 11 "(tree fragment)"
  int __pyx_t_2;

#line 11 "(tree fragment)"
  Py_ssize_t __pyx_t_3;

#line 11 "(tree fragment)"
  int __pyx_t_4;

#line 11 "(tree fragment)"
  int __pyx_t_5;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 11 "(tree fragment)"
  int __pyx_lineno = 0;

#line 11 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 11 "(tree fragment)"
  int __pyx_clineno = 0;

#line 11 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_ProfileSplitter__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->partition_sites);

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_v___pyx_result->partition_sites);

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->partition_sites = ((PyObject*)__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->profile);

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_v___pyx_result->profile);

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->profile = __pyx_t_1;

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->xs);

#line 12 "(tree fragment)"
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->xs));

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->xs = ((PyArrayObject *)__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->ys);

#line 12 "(tree fragment)"
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->ys));

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->ys = ((PyArrayObject *)__pyx_t_1);

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */

#line 13 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 13 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 13 "(tree fragment)"
    __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_4 = ((__pyx_t_3 > 4) != 0);

#line 13 "(tree fragment)"
  if (__pyx_t_4) {

#line 13 "(tree fragment)"
  } else {

#line 13 "(tree fragment)"
    __pyx_t_2 = __pyx_t_4;

#line 13 "(tree fragment)"
    goto __pyx_L4_bool_binop_done;

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_5 = (__pyx_t_4 != 0);

#line 13 "(tree fragment)"
  __pyx_t_2 = __pyx_t_5;

#line 13 "(tree fragment)"
  __pyx_L4_bool_binop_done:;

#line 13 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])             # <<<<<<<<<<<<<<
 */

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 14 "(tree fragment)"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 14 "(tree fragment)"
      __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_8 = NULL;

#line 14 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {

#line 14 "(tree fragment)"
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);

#line 14 "(tree fragment)"
      if (likely(__pyx_t_8)) {

#line 14 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);

#line 14 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_8);

#line 14 "(tree fragment)"
        __Pyx_INCREF(function);

#line 14 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_7, function);

#line 14 "(tree fragment)"
      }

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);

#line 14 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */

#line 13 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ProfileSplitter__set_state(<ProfileSplitter> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ProfileSplitter__set_state(ProfileSplitter __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.partition_sites = __pyx_state[0]; __pyx_result.profile = __pyx_state[1]; __pyx_result.xs = __pyx_state[2]; __pyx_result.ys = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /* function exit code */

#line 11 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 11 "(tree fragment)"
  __Pyx_AddTraceback("ms_deisotope._c.feature_map.profile_transform.__pyx_unpickle_ProfileSplitter__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
  __pyx_r = 0;

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  return __pyx_r;

#line 11 "(tree fragment)"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 735, __pyx_L1_error)

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":738
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 738, __pyx_L1_error)

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":741
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 741, __pyx_L1_error)

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":744
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 744, __pyx_L1_error)

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":747
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 747, __pyx_L1_error)

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_1;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":753
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /*else*/ {

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_INCREF(__pyx_empty_tuple);

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = __pyx_empty_tuple;

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":869
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */

#line 869 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  Py_INCREF(__pyx_v_base);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":870
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */

#line 870 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base
#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_1;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":873
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */

#line 873 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":875
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":876
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_INCREF(((PyObject *)__pyx_v_base));

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = ((PyObject *)__pyx_v_base);

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":882
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */

#line 882 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 882, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":883
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 883, __pyx_L5_except_error)

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 884, __pyx_L5_except_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 884, __pyx_L5_except_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":888
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */

#line 888 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 888, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":889
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 889, __pyx_L5_except_error)

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 890, __pyx_L5_except_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 890, __pyx_L5_except_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":894
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */

#line 894 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 894, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":895
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 895, __pyx_L5_except_error)

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":896
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 896, __pyx_L5_except_error)

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 896, __pyx_L5_except_error)

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
/* Python wrapper */

#line 122 "View.MemoryView"
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 122 "View.MemoryView"
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_itemsize
#line 122 "View.MemoryView"
;
  PyObject *__pyx_v_format = 0
#line 122 "View.MemoryView"
;
  PyObject *__pyx_v_mode = 0
#line 122 "View.MemoryView"
;
  int __pyx_v_allocate_buffer
#line 122 "View.MemoryView"
;

#line 122 "View.MemoryView"
  int __pyx_lineno = 0;

#line 122 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 122 "View.MemoryView"
  int __pyx_clineno = 0;

#line 122 "View.MemoryView"
  int __pyx_r;

#line 122 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 122 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);

#line 122 "View.MemoryView"
  {

#line 122 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,&__pyx_n_s_itemsize,&__pyx_n_s_format,&__pyx_n_s_mode,&__pyx_n_s_allocate_buffer,0};

#line 122 "View.MemoryView"
    PyObject* values[5] = {0,0,0,0,0};

#line 122 "View.MemoryView"
    values[3] = ((PyObject *)__pyx_n_s_c);

#line 122 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 122 "View.MemoryView"
      Py_ssize_t kw_args;

#line 122 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 122 "View.MemoryView"
      switch (pos_args) {
        case  5: 
#line 122 "View.MemoryView"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 122 "View.MemoryView"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 122 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 122 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 122 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  0: break;
        default: 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 122 "View.MemoryView"
      switch (pos_args) {

#line 122 "View.MemoryView"
        case  0:

#line 122 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shape)) != 0)) kw_args--;
        else 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  1:

#line 122 "View.MemoryView"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_itemsize)) != 0)) kw_args--;

#line 122 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 1); 
#line 122 "View.MemoryView"
__PYX_ERR(1, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  2:

#line 122 "View.MemoryView"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;

#line 122 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 2); 
#line 122 "View.MemoryView"
__PYX_ERR(1, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  3:

#line 122 "View.MemoryView"
        if (kw_args > 0) {

#line 122 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode);

#line 122 "View.MemoryView"
          if (value) { values[3] = value; kw_args--; }

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  4:

#line 122 "View.MemoryView"
        if (kw_args > 0) {

#line 122 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_allocate_buffer);

#line 122 "View.MemoryView"
          if (value) { values[4] = value; kw_args--; }

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 122 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(1, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
    } else {

#line 122 "View.MemoryView"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: 
#line 122 "View.MemoryView"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 122 "View.MemoryView"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 122 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 122 "View.MemoryView"
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 122 "View.MemoryView"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 122 "View.MemoryView"
        break;
        default: 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
    }

#line 122 "View.MemoryView"
    __pyx_v_shape = ((PyObject*)values[0]);

#line 122 "View.MemoryView"
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
    __pyx_v_format = values[2];

#line 122 "View.MemoryView"
    __pyx_v_mode = values[3];

#line 122 "View.MemoryView"
    if (values[4]) {

#line 122 "View.MemoryView"
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 123, __pyx_L3_error)

#line 122 "View.MemoryView"
    } else {

      /* "View.MemoryView":123
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
 */

#line 123 "View.MemoryView"
      __pyx_v_allocate_buffer = ((int)1);

#line 123 "View.MemoryView"
    }

#line 123 "View.MemoryView"
  }

#line 123 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 123 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); 
#line 123 "View.MemoryView"
__PYX_ERR(1, 122, __pyx_L3_error)

#line 123 "View.MemoryView"
  __pyx_L3_error:;

#line 123 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 123 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 123 "View.MemoryView"
  return -1;

#line 123 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;

#line 123 "View.MemoryView"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) __PYX_ERR(1, 122, __pyx_L1_error)

#line 123 "View.MemoryView"
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {

#line 123 "View.MemoryView"
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); __PYX_ERR(1, 122, __pyx_L1_error)

#line 123 "View.MemoryView"
  }
  __pyx_r = 
#line 123 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
  /* function exit code */

#line 122 "View.MemoryView"
  goto __pyx_L0;

#line 122 "View.MemoryView"
  __pyx_L1_error:;

#line 122 "View.MemoryView"
  __pyx_r = -1;

#line 122 "View.MemoryView"
  __pyx_L0:;

#line 122 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 122 "View.MemoryView"
  return __pyx_r;

#line 122 "View.MemoryView"
}

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_i
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dim
#line 122 "View.MemoryView"
;
  PyObject **__pyx_v_p
#line 122 "View.MemoryView"
;
  char __pyx_v_order
#line 122 "View.MemoryView"
;

#line 122 "View.MemoryView"
  int __pyx_r;

#line 122 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 122 "View.MemoryView"
  int __pyx_t_2;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 122 "View.MemoryView"
  int __pyx_t_4;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 122 "View.MemoryView"
  char *__pyx_t_7;

#line 122 "View.MemoryView"
  int __pyx_t_8;

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_11;

#line 122 "View.MemoryView"
  int __pyx_lineno = 0;

#line 122 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 122 "View.MemoryView"
  int __pyx_clineno = 0;

#line 122 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__", 0);

#line 122 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":129
 *         cdef PyObject **p
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
 */

#line 129 "View.MemoryView"
  if (unlikely(__pyx_v_shape == Py_None)) {

#line 129 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 129 "View.MemoryView"
    __PYX_ERR(1, 129, __pyx_L1_error)

#line 129 "View.MemoryView"
  }

#line 129 "View.MemoryView"
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(1, 129, __pyx_L1_error)

#line 129 "View.MemoryView"
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":130
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
 */

#line 130 "View.MemoryView"
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */

#line 132 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_self->ndim != 0)) != 0);

#line 132 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */

#line 133 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 133, __pyx_L1_error)

#line 133 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 133 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 133 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 133 "View.MemoryView"
    __PYX_ERR(1, 133, __pyx_L1_error)

    /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */

#line 132 "View.MemoryView"
  }

  /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */

#line 135 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_itemsize <= 0) != 0);

#line 135 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */

#line 136 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 136, __pyx_L1_error)

#line 136 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 136 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 136 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 136 "View.MemoryView"
    __PYX_ERR(1, 136, __pyx_L1_error)

    /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */

#line 135 "View.MemoryView"
  }

  /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */

#line 138 "View.MemoryView"
  __pyx_t_2 = PyBytes_Check(__pyx_v_format); 

#line 138 "View.MemoryView"
  __pyx_t_4 = ((!(__pyx_t_2 != 0)) != 0);

#line 138 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":139
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */

#line 139 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_format, __pyx_n_s_encode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 139, __pyx_L1_error)

#line 139 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 139 "View.MemoryView"
    __pyx_t_6 = NULL;

#line 139 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 139 "View.MemoryView"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 139 "View.MemoryView"
      if (likely(__pyx_t_6)) {

#line 139 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 139 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_6);

#line 139 "View.MemoryView"
        __Pyx_INCREF(function);

#line 139 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_5, function);

#line 139 "View.MemoryView"
      }

#line 139 "View.MemoryView"
    }

#line 139 "View.MemoryView"
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_n_s_ASCII) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_ASCII);

#line 139 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 139 "View.MemoryView"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 139, __pyx_L1_error)

#line 139 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 139 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 139 "View.MemoryView"
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_3);

#line 139 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */

#line 138 "View.MemoryView"
  }

  /* "View.MemoryView":140
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
 */

#line 140 "View.MemoryView"
  if (!(likely(PyBytes_CheckExact(__pyx_v_format))||((__pyx_v_format) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_format)->tp_name), 0))) __PYX_ERR(1, 140, __pyx_L1_error)

#line 140 "View.MemoryView"
  __pyx_t_3 = __pyx_v_format;

#line 140 "View.MemoryView"
  __Pyx_INCREF(__pyx_t_3);

#line 140 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 140 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->_format);

#line 140 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->_format);

#line 140 "View.MemoryView"
  __pyx_v_self->_format = ((PyObject*)__pyx_t_3);

#line 140 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":141
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 141 "View.MemoryView"
  if (unlikely(__pyx_v_self->_format == Py_None)) {

#line 141 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");

#line 141 "View.MemoryView"
    __PYX_ERR(1, 141, __pyx_L1_error)

#line 141 "View.MemoryView"
  }

#line 141 "View.MemoryView"
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_self->_format); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(1, 141, __pyx_L1_error)

#line 141 "View.MemoryView"
  __pyx_v_self->format = __pyx_t_7;

  /* "View.MemoryView":144
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
 */

#line 144 "View.MemoryView"
  __pyx_v_self->_shape = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":145
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
 */

#line 145 "View.MemoryView"
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */

#line 147 "View.MemoryView"
  __pyx_t_4 = ((!(__pyx_v_self->_shape != 0)) != 0);

#line 147 "View.MemoryView"
  if (unlikely(__pyx_t_4)) {

    /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 148 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 148, __pyx_L1_error)

#line 148 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 148 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 148 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 148 "View.MemoryView"
    __PYX_ERR(1, 148, __pyx_L1_error)

    /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */

#line 147 "View.MemoryView"
  }

  /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */

#line 151 "View.MemoryView"
  __pyx_t_8 = 0;

#line 151 "View.MemoryView"
  __pyx_t_3 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_3); __pyx_t_1 = 0;

#line 151 "View.MemoryView"
  for (;;) {

#line 151 "View.MemoryView"
    if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_3)) break;

#line 151 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 151 "View.MemoryView"
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_5); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(1, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    #else

#line 151 "View.MemoryView"
    __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 151 "View.MemoryView"
    #endif

#line 151 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 151 "View.MemoryView"
    __pyx_v_dim = __pyx_t_9;

#line 151 "View.MemoryView"
    __pyx_v_idx = __pyx_t_8;

#line 151 "View.MemoryView"
    __pyx_t_8 = (__pyx_t_8 + 1);

    /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */

#line 152 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_dim <= 0) != 0);

#line 152 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":153
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
 */

#line 153 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 153 "View.MemoryView"
      __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 153 "View.MemoryView"
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 153 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_5);

#line 153 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5);

#line 153 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_6);

#line 153 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_6);

#line 153 "View.MemoryView"
      __pyx_t_5 = 0;

#line 153 "View.MemoryView"
      __pyx_t_6 = 0;

#line 153 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 153 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 153 "View.MemoryView"
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 153 "View.MemoryView"
      __PYX_ERR(1, 153, __pyx_L1_error)

      /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */

#line 152 "View.MemoryView"
    }

    /* "View.MemoryView":154
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
 */

#line 154 "View.MemoryView"
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */

#line 151 "View.MemoryView"
  }

#line 151 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */

#line 157 "View.MemoryView"
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_fortran, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(1, 157, __pyx_L1_error)

#line 157 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":158
 *         cdef char order
 *         if mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 */

#line 158 "View.MemoryView"
    __pyx_v_order = 'F';

    /* "View.MemoryView":159
 *         if mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         elif mode == 'c':
 *             order = b'C'
 */

#line 159 "View.MemoryView"
    __Pyx_INCREF(__pyx_n_u_fortran);

#line 159 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_n_u_fortran);

#line 159 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->mode);

#line 159 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->mode);

#line 159 "View.MemoryView"
    __pyx_v_self->mode = __pyx_n_u_fortran;

    /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */

#line 157 "View.MemoryView"
    goto __pyx_L10;

#line 157 "View.MemoryView"
  }

  /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */

#line 160 "View.MemoryView"
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_c, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(1, 160, __pyx_L1_error)

#line 160 "View.MemoryView"
  if (likely(__pyx_t_4)) {

    /* "View.MemoryView":161
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         else:
 */

#line 161 "View.MemoryView"
    __pyx_v_order = 'C';

    /* "View.MemoryView":162
 *         elif mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 */

#line 162 "View.MemoryView"
    __Pyx_INCREF(__pyx_n_u_c);

#line 162 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_n_u_c);

#line 162 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->mode);

#line 162 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->mode);

#line 162 "View.MemoryView"
    __pyx_v_self->mode = __pyx_n_u_c;

    /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */

#line 160 "View.MemoryView"
    goto __pyx_L10;

#line 160 "View.MemoryView"
  }

  /* "View.MemoryView":164
 *             self.mode = u'c'
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,
 */

#line 164 "View.MemoryView"
  /*else*/ {

#line 164 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_v_mode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 164 "View.MemoryView"
    __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_10);

#line 164 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 164 "View.MemoryView"
    __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 164 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 164 "View.MemoryView"
    __PYX_ERR(1, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
  }

#line 164 "View.MemoryView"
  __pyx_L10:;

  /* "View.MemoryView":166
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,             # <<<<<<<<<<<<<<
 *                                              itemsize, self.ndim, order)
 * 
 */

#line 166 "View.MemoryView"
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":169
 *                                              itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:
 */

#line 169 "View.MemoryView"
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":170
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 *         if allocate_buffer:
 * 
 */

#line 170 "View.MemoryView"
  __pyx_t_10 = PyObject_RichCompare(__pyx_v_format, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 170, __pyx_L1_error)

#line 170 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 170, __pyx_L1_error)

#line 170 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 170 "View.MemoryView"
  __pyx_v_self->dtype_is_object = __pyx_t_4;

  /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 171 "View.MemoryView"
  __pyx_t_4 = (__pyx_v_allocate_buffer != 0);

#line 171 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":174
 * 
 * 
 *             self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")
 */

#line 174 "View.MemoryView"
    __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

    /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */

#line 175 "View.MemoryView"
    __pyx_t_4 = ((!(__pyx_v_self->data != 0)) != 0);

#line 175 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */

#line 176 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 176, __pyx_L1_error)

#line 176 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 176 "View.MemoryView"
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 176 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 176 "View.MemoryView"
      __PYX_ERR(1, 176, __pyx_L1_error)

      /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */

#line 175 "View.MemoryView"
    }

    /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */

#line 178 "View.MemoryView"
    __pyx_t_4 = (__pyx_v_self->dtype_is_object != 0);

#line 178 "View.MemoryView"
    if (__pyx_t_4) {

      /* "View.MemoryView":179
 * 
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 */

#line 179 "View.MemoryView"
      __pyx_v_p = ((PyObject **)__pyx_v_self->data);

      /* "View.MemoryView":180
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):             # <<<<<<<<<<<<<<
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)
 */

#line 180 "View.MemoryView"
      if (unlikely(__pyx_v_itemsize == 0)) {

#line 180 "View.MemoryView"
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");

#line 180 "View.MemoryView"
        __PYX_ERR(1, 180, __pyx_L1_error)

#line 180 "View.MemoryView"
      }

#line 180 "View.MemoryView"
      else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {

#line 180 "View.MemoryView"
        PyErr_SetString(PyExc_OverflowError, "value too large to perform division");

#line 180 "View.MemoryView"
        __PYX_ERR(1, 180, __pyx_L1_error)

#line 180 "View.MemoryView"
      }

#line 180 "View.MemoryView"
      __pyx_t_1 = __Pyx_div_Py_ssize_t(__pyx_v_self->len, __pyx_v_itemsize);

#line 180 "View.MemoryView"
      __pyx_t_9 = __pyx_t_1;

#line 180 "View.MemoryView"
      for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_9; __pyx_t_11+=1) {

#line 180 "View.MemoryView"
        __pyx_v_i = __pyx_t_11;

        /* "View.MemoryView":181
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None             # <<<<<<<<<<<<<<
 *                     Py_INCREF(Py_None)
 * 
 */

#line 181 "View.MemoryView"
        (__pyx_v_p[__pyx_v_i]) = Py_None;

        /* "View.MemoryView":182
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 182 "View.MemoryView"
        Py_INCREF(Py_None);

#line 182 "View.MemoryView"
      }

      /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */

#line 178 "View.MemoryView"
    }

    /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 171 "View.MemoryView"
  }

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
  /* function exit code */

#line 122 "View.MemoryView"
  __pyx_r = 0;

#line 122 "View.MemoryView"
  goto __pyx_L0;

#line 122 "View.MemoryView"
  __pyx_L1_error:;

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_10);

#line 122 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 122 "View.MemoryView"
  __pyx_r = -1;

#line 122 "View.MemoryView"
  __pyx_L0:;

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_format);

#line 122 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 122 "View.MemoryView"
  return __pyx_r;

#line 122 "View.MemoryView"
}

/* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
/* Python wrapper */

#line 185 "View.MemoryView"
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/

#line 185 "View.MemoryView"
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 185 "View.MemoryView"
  int __pyx_r;

#line 185 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 185 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = 
#line 185 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
  /* function exit code */

#line 185 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 185 "View.MemoryView"
  return __pyx_r;

#line 185 "View.MemoryView"
}

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode
#line 185 "View.MemoryView"
;

#line 185 "View.MemoryView"
  int __pyx_r;

#line 185 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 185 "View.MemoryView"
  int __pyx_t_1;

#line 185 "View.MemoryView"
  int __pyx_t_2;

#line 185 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 185 "View.MemoryView"
  char *__pyx_t_4;

#line 185 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 185 "View.MemoryView"
  int __pyx_t_6;

#line 185 "View.MemoryView"
  Py_ssize_t *__pyx_t_7;

#line 185 "View.MemoryView"
  int __pyx_lineno = 0;

#line 185 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 185 "View.MemoryView"
  int __pyx_clineno = 0;

#line 185 "View.MemoryView"
  if (__pyx_v_info == NULL) {

#line 185 "View.MemoryView"
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");

#line 185 "View.MemoryView"
    return -1;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__", 0);

#line 185 "View.MemoryView"
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);

#line 185 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":186
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */

#line 186 "View.MemoryView"
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */

#line 187 "View.MemoryView"
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_c, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 187, __pyx_L1_error)

#line 187 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 187 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":188
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */

#line 188 "View.MemoryView"
    __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */

#line 187 "View.MemoryView"
    goto __pyx_L3;

#line 187 "View.MemoryView"
  }

  /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */

#line 189 "View.MemoryView"
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_fortran, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 189, __pyx_L1_error)

#line 189 "View.MemoryView"
  __pyx_t_1 = (__pyx_t_2 != 0);

#line 189 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":190
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 */

#line 190 "View.MemoryView"
    __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */

#line 189 "View.MemoryView"
  }

#line 189 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */

#line 191 "View.MemoryView"
  __pyx_t_1 = ((!((__pyx_v_flags & __pyx_v_bufmode) != 0)) != 0);

#line 191 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */

#line 192 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 192, __pyx_L1_error)

#line 192 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 192 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 192 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 192 "View.MemoryView"
    __PYX_ERR(1, 192, __pyx_L1_error)

    /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */

#line 191 "View.MemoryView"
  }

  /* "View.MemoryView":193
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 *         info.ndim = self.ndim
 */

#line 193 "View.MemoryView"
  __pyx_t_4 = __pyx_v_self->data;

#line 193 "View.MemoryView"
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":194
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 */

#line 194 "View.MemoryView"
  __pyx_t_5 = __pyx_v_self->len;

#line 194 "View.MemoryView"
  __pyx_v_info->len = __pyx_t_5;

  /* "View.MemoryView":195
 *         info.buf = self.data
 *         info.len = self.len
 *         info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *         info.shape = self._shape
 *         info.strides = self._strides
 */

#line 195 "View.MemoryView"
  __pyx_t_6 = __pyx_v_self->ndim;

#line 195 "View.MemoryView"
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":196
 *         info.len = self.len
 *         info.ndim = self.ndim
 *         info.shape = self._shape             # <<<<<<<<<<<<<<
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 */

#line 196 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->_shape;

#line 196 "View.MemoryView"
  __pyx_v_info->shape = __pyx_t_7;

  /* "View.MemoryView":197
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 *         info.strides = self._strides             # <<<<<<<<<<<<<<
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 */

#line 197 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->_strides;

#line 197 "View.MemoryView"
  __pyx_v_info->strides = __pyx_t_7;

  /* "View.MemoryView":198
 *         info.shape = self._shape
 *         info.strides = self._strides
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 */

#line 198 "View.MemoryView"
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":199
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 * 
 */

#line 199 "View.MemoryView"
  __pyx_t_5 = __pyx_v_self->itemsize;

#line 199 "View.MemoryView"
  __pyx_v_info->itemsize = __pyx_t_5;

  /* "View.MemoryView":200
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 200 "View.MemoryView"
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */

#line 202 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 202 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":203
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */

#line 203 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->format;

#line 203 "View.MemoryView"
    __pyx_v_info->format = __pyx_t_4;

    /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */

#line 202 "View.MemoryView"
    goto __pyx_L5;

#line 202 "View.MemoryView"
  }

  /* "View.MemoryView":205
 *             info.format = self.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.obj = self
 */

#line 205 "View.MemoryView"
  /*else*/ {

#line 205 "View.MemoryView"
    __pyx_v_info->format = NULL;

#line 205 "View.MemoryView"
  }

#line 205 "View.MemoryView"
  __pyx_L5:;

  /* "View.MemoryView":207
 *             info.format = NULL
 * 
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 207 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 207 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 207 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_info->obj);

#line 207 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_info->obj);

#line 207 "View.MemoryView"
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
  /* function exit code */

#line 185 "View.MemoryView"
  __pyx_r = 0;

#line 185 "View.MemoryView"
  goto __pyx_L0;

#line 185 "View.MemoryView"
  __pyx_L1_error:;

#line 185 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 185 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 185 "View.MemoryView"
  __pyx_r = -1;

#line 185 "View.MemoryView"
  if (__pyx_v_info->obj != NULL) {

#line 185 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 185 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  goto __pyx_L2;

#line 185 "View.MemoryView"
  __pyx_L0:;

#line 185 "View.MemoryView"
  if (__pyx_v_info->obj == Py_None) {

#line 185 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 185 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  __pyx_L2:;

#line 185 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 185 "View.MemoryView"
  return __pyx_r;

#line 185 "View.MemoryView"
}

/* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
/* Python wrapper */

#line 211 "View.MemoryView"
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 211 "View.MemoryView"
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {

#line 211 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 211 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 211 "View.MemoryView"
  __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
  /* function exit code */

#line 211 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 211 "View.MemoryView"
}

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {

#line 211 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 211 "View.MemoryView"
  int __pyx_t_1;

#line 211 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */

#line 212 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->callback_free_data != NULL) != 0);

#line 212 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":213
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data:
 *             if self.dtype_is_object:
 */

#line 213 "View.MemoryView"
    __pyx_v_self->callback_free_data(__pyx_v_self->data);

    /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */

#line 212 "View.MemoryView"
    goto __pyx_L3;

#line 212 "View.MemoryView"
  }

  /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */

#line 214 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->free_data != 0);

#line 214 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */

#line 215 "View.MemoryView"
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);

#line 215 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":216
 *         elif self.free_data:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,             # <<<<<<<<<<<<<<
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 */

#line 216 "View.MemoryView"
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);

      /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */

#line 215 "View.MemoryView"
    }

    /* "View.MemoryView":218
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyObject_Free(self._shape)
 * 
 */

#line 218 "View.MemoryView"
    free(__pyx_v_self->data);

    /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */

#line 214 "View.MemoryView"
  }

#line 214 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":219
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 *         PyObject_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 219 "View.MemoryView"
  PyObject_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
  /* function exit code */

#line 211 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 211 "View.MemoryView"
}

/* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
/* Python wrapper */

#line 222 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 222 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self) {

#line 222 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 222 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 222 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 222 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
  /* function exit code */

#line 222 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 222 "View.MemoryView"
  return __pyx_r;

#line 222 "View.MemoryView"
}

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {

#line 222 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 222 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 222 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 222 "View.MemoryView"
  int __pyx_lineno = 0;

#line 222 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 222 "View.MemoryView"
  int __pyx_clineno = 0;

#line 222 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":223
 *     @property
 *     def memview(self):
 *         return self.get_memview()             # <<<<<<<<<<<<<<
 * 
 *     @cname('get_memview')
 */

#line 223 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 223 "View.MemoryView"
  __pyx_t_1 = ((struct __pyx_vtabstruct_array *)__pyx_v_self->__pyx_vtab)->get_memview(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 223, __pyx_L1_error)

#line 223 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 223 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 223 "View.MemoryView"
  __pyx_t_1 = 0;

#line 223 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
  /* function exit code */

#line 222 "View.MemoryView"
  __pyx_L1_error:;

#line 222 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 222 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 222 "View.MemoryView"
  __pyx_r = NULL;

#line 222 "View.MemoryView"
  __pyx_L0:;

#line 222 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 222 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 222 "View.MemoryView"
  return __pyx_r;

#line 222 "View.MemoryView"
}

/* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

#line 226 "View.MemoryView"


#line 226 "View.MemoryView"
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags
#line 226 "View.MemoryView"
;

#line 226 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 226 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 226 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 226 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 226 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 226 "View.MemoryView"
  int __pyx_lineno = 0;

#line 226 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 226 "View.MemoryView"
  int __pyx_clineno = 0;

#line 226 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_memview", 0);

  /* "View.MemoryView":227
 *     @cname('get_memview')
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 */

#line 227 "View.MemoryView"
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":228
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 228 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 228 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 228 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 228 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 228 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 228 "View.MemoryView"
  __pyx_t_1 = 0;

#line 228 "View.MemoryView"
  __pyx_t_2 = 0;

#line 228 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 228 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 228 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 228 "View.MemoryView"
  __pyx_t_2 = 0;

#line 228 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

#line 226 "View.MemoryView"


#line 226 "View.MemoryView"
  /* function exit code */

#line 226 "View.MemoryView"
  __pyx_L1_error:;

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 226 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.get_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 226 "View.MemoryView"
  __pyx_r = 0;

#line 226 "View.MemoryView"
  __pyx_L0:;

#line 226 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 226 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 226 "View.MemoryView"
  return __pyx_r;

#line 226 "View.MemoryView"
}

/* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
/* Python wrapper */

#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self); /*proto*/

#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self) {

#line 230 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 230 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 230 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 230 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(((struct __pyx_array_obj *)__pyx_v_self));

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
  /* function exit code */

#line 230 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 230 "View.MemoryView"
  return __pyx_r;

#line 230 "View.MemoryView"
}

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self) {

#line 230 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 230 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 230 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":231
 * 
 *     def __len__(self):
 *         return self._shape[0]             # <<<<<<<<<<<<<<
 * 
 *     def __getattr__(self, attr):
 */

#line 231 "View.MemoryView"
  __pyx_r = (__pyx_v_self->_shape[0]);

#line 231 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
  /* function exit code */

#line 230 "View.MemoryView"
  __pyx_L0:;

#line 230 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 230 "View.MemoryView"
  return __pyx_r;

#line 230 "View.MemoryView"
}

/* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
/* Python wrapper */

#line 233 "View.MemoryView"
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/

#line 233 "View.MemoryView"
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {

#line 233 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 233 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 233 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_r = 
#line 233 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
  /* function exit code */

#line 233 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 233 "View.MemoryView"
  return __pyx_r;

#line 233 "View.MemoryView"
}

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {

#line 233 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 233 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 233 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 233 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 233 "View.MemoryView"
  int __pyx_lineno = 0;

#line 233 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 233 "View.MemoryView"
  int __pyx_clineno = 0;

#line 233 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":234
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */

#line 234 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 234 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 234, __pyx_L1_error)

#line 234 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 234 "View.MemoryView"
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 234, __pyx_L1_error)

#line 234 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 234 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 234 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 234 "View.MemoryView"
  __pyx_t_2 = 0;

#line 234 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
  /* function exit code */

#line 233 "View.MemoryView"
  __pyx_L1_error:;

#line 233 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 233 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 233 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 233 "View.MemoryView"
  __pyx_r = NULL;

#line 233 "View.MemoryView"
  __pyx_L0:;

#line 233 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 233 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 233 "View.MemoryView"
  return __pyx_r;

#line 233 "View.MemoryView"
}

/* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
/* Python wrapper */

#line 236 "View.MemoryView"
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/

#line 236 "View.MemoryView"
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {

#line 236 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 236 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 236 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 236 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
  /* function exit code */

#line 236 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 236 "View.MemoryView"
  return __pyx_r;

#line 236 "View.MemoryView"
}

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {

#line 236 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 236 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 236 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 236 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 236 "View.MemoryView"
  int __pyx_lineno = 0;

#line 236 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 236 "View.MemoryView"
  int __pyx_clineno = 0;

#line 236 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":237
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
 */

#line 237 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 237 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 237, __pyx_L1_error)

#line 237 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 237 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 237, __pyx_L1_error)

#line 237 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 237 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 237 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 237 "View.MemoryView"
  __pyx_t_2 = 0;

#line 237 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
  /* function exit code */

#line 236 "View.MemoryView"
  __pyx_L1_error:;

#line 236 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 236 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 236 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 236 "View.MemoryView"
  __pyx_r = NULL;

#line 236 "View.MemoryView"
  __pyx_L0:;

#line 236 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 236 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 236 "View.MemoryView"
  return __pyx_r;

#line 236 "View.MemoryView"
}

/* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
/* Python wrapper */

#line 239 "View.MemoryView"
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/

#line 239 "View.MemoryView"
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {

#line 239 "View.MemoryView"
  int __pyx_r;

#line 239 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 239 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = 
#line 239 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
  /* function exit code */

#line 239 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 239 "View.MemoryView"
  return __pyx_r;

#line 239 "View.MemoryView"
}

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {

#line 239 "View.MemoryView"
  int __pyx_r;

#line 239 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 239 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 239 "View.MemoryView"
  int __pyx_lineno = 0;

#line 239 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 239 "View.MemoryView"
  int __pyx_clineno = 0;

#line 239 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":240
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 240 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 240, __pyx_L1_error)

#line 240 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 240 "View.MemoryView"
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0)) __PYX_ERR(1, 240, __pyx_L1_error)

#line 240 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
  /* function exit code */

#line 239 "View.MemoryView"
  __pyx_r = 0;

#line 239 "View.MemoryView"
  goto __pyx_L0;

#line 239 "View.MemoryView"
  __pyx_L1_error:;

#line 239 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 239 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 239 "View.MemoryView"
  __pyx_r = -1;

#line 239 "View.MemoryView"
  __pyx_L0:;

#line 239 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 239 "View.MemoryView"
  return __pyx_r;

#line 239 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_array___reduce_cython__(((struct __pyx_array_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.array.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_array_2__setstate_cython__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

#line 244 "View.MemoryView"


#line 244 "View.MemoryView"
static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char *__pyx_v_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0
#line 244 "View.MemoryView"
;

#line 244 "View.MemoryView"
  struct __pyx_array_obj *__pyx_r = NULL;

#line 244 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 244 "View.MemoryView"
  int __pyx_t_1;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 244 "View.MemoryView"
  int __pyx_lineno = 0;

#line 244 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 244 "View.MemoryView"
  int __pyx_clineno = 0;

#line 244 "View.MemoryView"
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */

#line 248 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_buf == NULL) != 0);

#line 248 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":249
 * 
 *     if buf == NULL:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))             # <<<<<<<<<<<<<<
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 */

#line 249 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 249 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 249 "View.MemoryView"
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 249 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_shape);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_shape);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_2);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_3);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_2 = 0;

#line 249 "View.MemoryView"
    __pyx_t_3 = 0;

#line 249 "View.MemoryView"
    __pyx_t_4 = 0;

#line 249 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 249 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 249 "View.MemoryView"
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_4 = 0;

    /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */

#line 248 "View.MemoryView"
    goto __pyx_L3;

#line 248 "View.MemoryView"
  }

  /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */

#line 251 "View.MemoryView"
  /*else*/ {

#line 251 "View.MemoryView"
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 251 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 251 "View.MemoryView"
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 251 "View.MemoryView"
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 251 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_shape);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_shape);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_5);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_3);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);

#line 251 "View.MemoryView"
    __pyx_t_4 = 0;

#line 251 "View.MemoryView"
    __pyx_t_5 = 0;

#line 251 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":252
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
 */

#line 252 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 252, __pyx_L1_error)

#line 252 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 252 "View.MemoryView"
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_allocate_buffer, Py_False) < 0) __PYX_ERR(1, 252, __pyx_L1_error)

    /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */

#line 251 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 251 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 251 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 251 "View.MemoryView"
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_5);

#line 251 "View.MemoryView"
    __pyx_t_5 = 0;

    /* "View.MemoryView":253
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 253 "View.MemoryView"
    __pyx_v_result->data = __pyx_v_buf;

#line 253 "View.MemoryView"
  }

#line 253 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":255
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 255 "View.MemoryView"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 255 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 255 "View.MemoryView"
  __pyx_r = __pyx_v_result;

#line 255 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

#line 244 "View.MemoryView"


#line 244 "View.MemoryView"
  /* function exit code */

#line 244 "View.MemoryView"
  __pyx_L1_error:;

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 244 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 244 "View.MemoryView"
  __pyx_r = 0;

#line 244 "View.MemoryView"
  __pyx_L0:;

#line 244 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 244 "View.MemoryView"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 244 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 244 "View.MemoryView"
  return __pyx_r;

#line 244 "View.MemoryView"
}

/* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
/* Python wrapper */

#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0
#line 281 "View.MemoryView"
;

#line 281 "View.MemoryView"
  int __pyx_lineno = 0;

#line 281 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 281 "View.MemoryView"
  int __pyx_clineno = 0;

#line 281 "View.MemoryView"
  int __pyx_r;

#line 281 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 281 "View.MemoryView"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 281 "View.MemoryView"
  {

#line 281 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};

#line 281 "View.MemoryView"
    PyObject* values[1] = {0};

#line 281 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 281 "View.MemoryView"
      Py_ssize_t kw_args;

#line 281 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 281 "View.MemoryView"
      switch (pos_args) {
        case  1: 
#line 281 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 281 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 281 "View.MemoryView"
        case  0: break;
        default: 
#line 281 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 281 "View.MemoryView"
      switch (pos_args) {

#line 281 "View.MemoryView"
        case  0:

#line 281 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else 
#line 281 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 281 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 281, __pyx_L3_error)

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 281 "View.MemoryView"
      goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
    } else {

#line 281 "View.MemoryView"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 281 "View.MemoryView"
    }

#line 281 "View.MemoryView"
    __pyx_v_name = values[0];

#line 281 "View.MemoryView"
  }

#line 281 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 281 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 281 "View.MemoryView"
__PYX_ERR(1, 281, __pyx_L3_error)

#line 281 "View.MemoryView"
  __pyx_L3_error:;

#line 281 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return -1;

#line 281 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 281 "View.MemoryView"
__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
  /* function exit code */

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return __pyx_r;

#line 281 "View.MemoryView"
}

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {

#line 281 "View.MemoryView"
  int __pyx_r;

#line 281 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 281 "View.MemoryView"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":282
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
 */

#line 282 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_name);

#line 282 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_name);

#line 282 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->name);

#line 282 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->name);

#line 282 "View.MemoryView"
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
  /* function exit code */

#line 281 "View.MemoryView"
  __pyx_r = 0;

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return __pyx_r;

#line 281 "View.MemoryView"
}

/* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
/* Python wrapper */

#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/

#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {

#line 283 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 283 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 283 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 283 "View.MemoryView"
__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
  /* function exit code */

#line 283 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 283 "View.MemoryView"
  return __pyx_r;

#line 283 "View.MemoryView"
}

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {

#line 283 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 283 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 283 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":284
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 */

#line 284 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 284 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->name);

#line 284 "View.MemoryView"
  __pyx_r = __pyx_v_self->name;

#line 284 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
  /* function exit code */

#line 283 "View.MemoryView"
  __pyx_L0:;

#line 283 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 283 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 283 "View.MemoryView"
  return __pyx_r;

#line 283 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_MemviewEnum___reduce_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_v_state = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v__dict = 0
#line 1 "(tree fragment)"
;
  int __pyx_v_use_setstate
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */

#line 5 "(tree fragment)"
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
  __Pyx_INCREF(__pyx_v_self->name);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_v_self->name);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name);

#line 5 "(tree fragment)"
  __pyx_v_state = ((PyObject*)__pyx_t_1);

#line 5 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */

#line 6 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
  __pyx_v__dict = __pyx_t_1;

#line 6 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
  __pyx_t_2 = (__pyx_v__dict != Py_None);

#line 7 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 7 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */

#line 8 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 8 "(tree fragment)"
    __Pyx_INCREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);

#line 8 "(tree fragment)"
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 8 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 8 "(tree fragment)"
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));

#line 8 "(tree fragment)"
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
 */

#line 9 "(tree fragment)"
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
    goto __pyx_L3;

#line 7 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 */

#line 11 "(tree fragment)"
  /*else*/ {

#line 11 "(tree fragment)"
    __pyx_t_3 = (__pyx_v_self->name != Py_None);

#line 11 "(tree fragment)"
    __pyx_v_use_setstate = __pyx_t_3;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  __pyx_t_3 = (__pyx_v_use_setstate != 0);

#line 12 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 */

#line 13 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 13 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 13 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_184977713);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_184977713);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);

#line 13 "(tree fragment)"
    __Pyx_INCREF(Py_None);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(Py_None);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);

#line 13 "(tree fragment)"
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_4);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);

#line 13 "(tree fragment)"
    __pyx_t_4 = 0;

#line 13 "(tree fragment)"
    __pyx_t_1 = 0;

#line 13 "(tree fragment)"
    __pyx_r = __pyx_t_5;

#line 13 "(tree fragment)"
    __pyx_t_5 = 0;

#line 13 "(tree fragment)"
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 15 "(tree fragment)"
  /*else*/ {

#line 15 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 15 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 15 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 15 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_184977713);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_184977713);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);

#line 15 "(tree fragment)"
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_5);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);

#line 15 "(tree fragment)"
    __pyx_t_5 = 0;

#line 15 "(tree fragment)"
    __pyx_t_1 = 0;

#line 15 "(tree fragment)"
    __pyx_r = __pyx_t_4;

#line 15 "(tree fragment)"
    __pyx_t_4 = 0;

#line 15 "(tree fragment)"
    goto __pyx_L0;

#line 15 "(tree fragment)"
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.Enum.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v_state);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v__dict);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
/* Python wrapper */

#line 16 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 16 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 16 "(tree fragment)"
__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 16 "(tree fragment)"
  int __pyx_lineno = 0;

#line 16 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 16 "(tree fragment)"
  int __pyx_clineno = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */

#line 17 "(tree fragment)"
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __pyx_t_1 = __pyx_unpickle_Enum__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 16 "(tree fragment)"
  goto __pyx_L0;

#line 16 "(tree fragment)"
  __pyx_L1_error:;

#line 16 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.Enum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "(tree fragment)"
  __pyx_r = NULL;

#line 16 "(tree fragment)"
  __pyx_L0:;

#line 16 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

/* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

#line 298 "View.MemoryView"


#line 298 "View.MemoryView"
static void *__pyx_align_pointer(void *__pyx_v_memory, size_t __pyx_v_alignment) {
  Py_intptr_t __pyx_v_aligned_p
#line 298 "View.MemoryView"
;
  size_t __pyx_v_offset
#line 298 "View.MemoryView"
;

#line 298 "View.MemoryView"
  void *__pyx_r;

#line 298 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":300
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory             # <<<<<<<<<<<<<<
 *     cdef size_t offset
 * 
 */

#line 300 "View.MemoryView"
  __pyx_v_aligned_p = ((Py_intptr_t)__pyx_v_memory);

  /* "View.MemoryView":304
 * 
 *     with cython.cdivision(True):
 *         offset = aligned_p % alignment             # <<<<<<<<<<<<<<
 * 
 *     if offset > 0:
 */

#line 304 "View.MemoryView"
  __pyx_v_offset = (__pyx_v_aligned_p % __pyx_v_alignment);

  /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */

#line 306 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_offset > 0) != 0);

#line 306 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":307
 * 
 *     if offset > 0:
 *         aligned_p += alignment - offset             # <<<<<<<<<<<<<<
 * 
 *     return <void *> aligned_p
 */

#line 307 "View.MemoryView"
    __pyx_v_aligned_p = (__pyx_v_aligned_p + (__pyx_v_alignment - __pyx_v_offset));

    /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */

#line 306 "View.MemoryView"
  }

  /* "View.MemoryView":309
 *         aligned_p += alignment - offset
 * 
 *     return <void *> aligned_p             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 309 "View.MemoryView"
  __pyx_r = ((void *)__pyx_v_aligned_p);

#line 309 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

#line 298 "View.MemoryView"


#line 298 "View.MemoryView"
  /* function exit code */

#line 298 "View.MemoryView"
  __pyx_L0:;

#line 298 "View.MemoryView"
  return __pyx_r;

#line 298 "View.MemoryView"
}

/* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
/* Python wrapper */

#line 345 "View.MemoryView"
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 345 "View.MemoryView"
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0
#line 345 "View.MemoryView"
;
  int __pyx_v_flags
#line 345 "View.MemoryView"
;
  int __pyx_v_dtype_is_object
#line 345 "View.MemoryView"
;

#line 345 "View.MemoryView"
  int __pyx_lineno = 0;

#line 345 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 345 "View.MemoryView"
  int __pyx_clineno = 0;

#line 345 "View.MemoryView"
  int __pyx_r;

#line 345 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 345 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);

#line 345 "View.MemoryView"
  {

#line 345 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_obj,&__pyx_n_s_flags,&__pyx_n_s_dtype_is_object,0};

#line 345 "View.MemoryView"
    PyObject* values[3] = {0,0,0};

#line 345 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 345 "View.MemoryView"
      Py_ssize_t kw_args;

#line 345 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 345 "View.MemoryView"
      switch (pos_args) {
        case  3: 
#line 345 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 345 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 345 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  0: break;
        default: 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 345 "View.MemoryView"
      switch (pos_args) {

#line 345 "View.MemoryView"
        case  0:

#line 345 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_obj)) != 0)) kw_args--;
        else 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  1:

#line 345 "View.MemoryView"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;

#line 345 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); 
#line 345 "View.MemoryView"
__PYX_ERR(1, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
        }

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  2:

#line 345 "View.MemoryView"
        if (kw_args > 0) {

#line 345 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype_is_object);

#line 345 "View.MemoryView"
          if (value) { values[2] = value; kw_args--; }

#line 345 "View.MemoryView"
        }

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 345 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(1, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
    } else {

#line 345 "View.MemoryView"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 345 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 345 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 345 "View.MemoryView"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 345 "View.MemoryView"
        break;
        default: 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
    }

#line 345 "View.MemoryView"
    __pyx_v_obj = values[0];

#line 345 "View.MemoryView"
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
    if (values[2]) {

#line 345 "View.MemoryView"
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
    } else {

#line 345 "View.MemoryView"
      __pyx_v_dtype_is_object = ((int)0);

#line 345 "View.MemoryView"
    }

#line 345 "View.MemoryView"
  }

#line 345 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 345 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 345 "View.MemoryView"
__PYX_ERR(1, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
  __pyx_L3_error:;

#line 345 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return -1;

#line 345 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 345 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
  /* function exit code */

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return __pyx_r;

#line 345 "View.MemoryView"
}

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {

#line 345 "View.MemoryView"
  int __pyx_r;

#line 345 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 345 "View.MemoryView"
  int __pyx_t_1;

#line 345 "View.MemoryView"
  int __pyx_t_2;

#line 345 "View.MemoryView"
  int __pyx_t_3;

#line 345 "View.MemoryView"
  int __pyx_t_4;

#line 345 "View.MemoryView"
  int __pyx_lineno = 0;

#line 345 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 345 "View.MemoryView"
  int __pyx_clineno = 0;

#line 345 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":346
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 */

#line 346 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

#line 346 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_obj);

#line 346 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->obj);

#line 346 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->obj);

#line 346 "View.MemoryView"
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":347
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 */

#line 347 "View.MemoryView"
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */

#line 348 "View.MemoryView"
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)__pyx_memoryview_type));

#line 348 "View.MemoryView"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 348 "View.MemoryView"
  if (!__pyx_t_3) {

#line 348 "View.MemoryView"
  } else {

#line 348 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 348 "View.MemoryView"
    goto __pyx_L4_bool_binop_done;

#line 348 "View.MemoryView"
  }

#line 348 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_obj != Py_None);

#line 348 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 348 "View.MemoryView"
  __pyx_t_1 = __pyx_t_2;

#line 348 "View.MemoryView"
  __pyx_L4_bool_binop_done:;

#line 348 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":349
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 */

#line 349 "View.MemoryView"
    __pyx_t_4 = __Pyx_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 349, __pyx_L1_error)

    /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */

#line 350 "View.MemoryView"
    __pyx_t_1 = ((((PyObject *)__pyx_v_self->view.obj) == NULL) != 0);

#line 350 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":351
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
 */

#line 351 "View.MemoryView"
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":352
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         global __pyx_memoryview_thread_locks_used
 */

#line 352 "View.MemoryView"
      Py_INCREF(Py_None);

      /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */

#line 350 "View.MemoryView"
    }

    /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */

#line 348 "View.MemoryView"
  }

  /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */

#line 355 "View.MemoryView"
  __pyx_t_1 = ((__pyx_memoryview_thread_locks_used < 8) != 0);

#line 355 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":356
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]             # <<<<<<<<<<<<<<
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 */

#line 356 "View.MemoryView"
    __pyx_v_self->lock = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

    /* "View.MemoryView":357
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1             # <<<<<<<<<<<<<<
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 */

#line 357 "View.MemoryView"
    __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used + 1);

    /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */

#line 355 "View.MemoryView"
  }

  /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */

#line 358 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);

#line 358 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":359
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *             if self.lock is NULL:
 *                 raise MemoryError
 */

#line 359 "View.MemoryView"
    __pyx_v_self->lock = PyThread_allocate_lock();

    /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */

#line 360 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);

#line 360 "View.MemoryView"
    if (unlikely(__pyx_t_1)) {

      /* "View.MemoryView":361
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 361 "View.MemoryView"
      PyErr_NoMemory(); __PYX_ERR(1, 361, __pyx_L1_error)

      /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */

#line 360 "View.MemoryView"
    }

    /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */

#line 358 "View.MemoryView"
  }

  /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */

#line 363 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 363 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":364
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
 */

#line 364 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_self->view.format[0]) == 'O') != 0);

#line 364 "View.MemoryView"
    if (__pyx_t_2) {

#line 364 "View.MemoryView"
    } else {

#line 364 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 364 "View.MemoryView"
      goto __pyx_L11_bool_binop_done;

#line 364 "View.MemoryView"
    }

#line 364 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_self->view.format[1]) == '\x00') != 0);

#line 364 "View.MemoryView"
    __pyx_t_1 = __pyx_t_2;

#line 364 "View.MemoryView"
    __pyx_L11_bool_binop_done:;

#line 364 "View.MemoryView"
    __pyx_v_self->dtype_is_object = __pyx_t_1;

    /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */

#line 363 "View.MemoryView"
    goto __pyx_L10;

#line 363 "View.MemoryView"
  }

  /* "View.MemoryView":366
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 */

#line 366 "View.MemoryView"
  /*else*/ {

#line 366 "View.MemoryView"
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;

#line 366 "View.MemoryView"
  }

#line 366 "View.MemoryView"
  __pyx_L10:;

  /* "View.MemoryView":368
 *             self.dtype_is_object = dtype_is_object
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(             # <<<<<<<<<<<<<<
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL
 */

#line 368 "View.MemoryView"
  __pyx_v_self->acquisition_count_aligned_p = ((__pyx_atomic_int *)__pyx_align_pointer(((void *)(&(__pyx_v_self->acquisition_count[0]))), (sizeof(__pyx_atomic_int))));

  /* "View.MemoryView":370
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
 */

#line 370 "View.MemoryView"
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
  /* function exit code */

#line 345 "View.MemoryView"
  __pyx_r = 0;

#line 345 "View.MemoryView"
  goto __pyx_L0;

#line 345 "View.MemoryView"
  __pyx_L1_error:;

#line 345 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 345 "View.MemoryView"
  __pyx_r = -1;

#line 345 "View.MemoryView"
  __pyx_L0:;

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return __pyx_r;

#line 345 "View.MemoryView"
}

/* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
/* Python wrapper */

#line 372 "View.MemoryView"
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 372 "View.MemoryView"
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {

#line 372 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 372 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 372 "View.MemoryView"
  __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
  /* function exit code */

#line 372 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 372 "View.MemoryView"
}

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i
#line 372 "View.MemoryView"
;

#line 372 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 372 "View.MemoryView"
  int __pyx_t_1;

#line 372 "View.MemoryView"
  int __pyx_t_2;

#line 372 "View.MemoryView"
  int __pyx_t_3;

#line 372 "View.MemoryView"
  int __pyx_t_4;

#line 372 "View.MemoryView"
  int __pyx_t_5;

#line 372 "View.MemoryView"
  PyThread_type_lock __pyx_t_6;

#line 372 "View.MemoryView"
  PyThread_type_lock __pyx_t_7;

#line 372 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */

#line 373 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);

#line 373 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 373 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":374
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)             # <<<<<<<<<<<<<<
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 */

#line 374 "View.MemoryView"
    __Pyx_ReleaseBuffer((&__pyx_v_self->view));

    /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */

#line 373 "View.MemoryView"
    goto __pyx_L3;

#line 373 "View.MemoryView"
  }

  /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */

#line 375 "View.MemoryView"
  __pyx_t_2 = ((((Py_buffer *)(&__pyx_v_self->view))->obj == Py_None) != 0);

#line 375 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":377
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL             # <<<<<<<<<<<<<<
 *             Py_DECREF(Py_None)
 * 
 */

#line 377 "View.MemoryView"
    ((Py_buffer *)(&__pyx_v_self->view))->obj = NULL;

    /* "View.MemoryView":378
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 *             Py_DECREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */

#line 378 "View.MemoryView"
    Py_DECREF(Py_None);

    /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */

#line 375 "View.MemoryView"
  }

#line 375 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */

#line 382 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_self->lock != NULL) != 0);

#line 382 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":383
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):             # <<<<<<<<<<<<<<
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 */

#line 383 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_thread_locks_used;

#line 383 "View.MemoryView"
    __pyx_t_4 = __pyx_t_3;

#line 383 "View.MemoryView"
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {

#line 383 "View.MemoryView"
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */

#line 384 "View.MemoryView"
      __pyx_t_2 = (((__pyx_memoryview_thread_locks[__pyx_v_i]) == __pyx_v_self->lock) != 0);

#line 384 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":385
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1             # <<<<<<<<<<<<<<
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 */

#line 385 "View.MemoryView"
        __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used - 1);

        /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */

#line 386 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_i != __pyx_memoryview_thread_locks_used) != 0);

#line 386 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":388
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])             # <<<<<<<<<<<<<<
 *                     break
 *             else:
 */

#line 388 "View.MemoryView"
          __pyx_t_6 = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

#line 388 "View.MemoryView"
          __pyx_t_7 = (__pyx_memoryview_thread_locks[__pyx_v_i]);

          /* "View.MemoryView":387
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (             # <<<<<<<<<<<<<<
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break
 */

#line 387 "View.MemoryView"
          (__pyx_memoryview_thread_locks[__pyx_v_i]) = __pyx_t_6;

#line 387 "View.MemoryView"
          (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]) = __pyx_t_7;

          /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */

#line 386 "View.MemoryView"
        }

        /* "View.MemoryView":389
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break             # <<<<<<<<<<<<<<
 *             else:
 *                 PyThread_free_lock(self.lock)
 */

#line 389 "View.MemoryView"
        goto __pyx_L6_break;

        /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */

#line 384 "View.MemoryView"
      }

#line 384 "View.MemoryView"
    }

#line 384 "View.MemoryView"
    /*else*/ {

      /* "View.MemoryView":391
 *                     break
 *             else:
 *                 PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 */

#line 391 "View.MemoryView"
      PyThread_free_lock(__pyx_v_self->lock);

#line 391 "View.MemoryView"
    }

#line 391 "View.MemoryView"
    __pyx_L6_break:;

    /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */

#line 382 "View.MemoryView"
  }

  /* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
  /* function exit code */

#line 372 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 372 "View.MemoryView"
}

/* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

#line 393 "View.MemoryView"


#line 393 "View.MemoryView"
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim
#line 393 "View.MemoryView"
;
  char *__pyx_v_itemp
#line 393 "View.MemoryView"
;
  PyObject *__pyx_v_idx = NULL
#line 393 "View.MemoryView"
;

#line 393 "View.MemoryView"
  char *__pyx_r;

#line 393 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 393 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 393 "View.MemoryView"
  PyObject *(*__pyx_t_4)(PyObject *);

#line 393 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

#line 393 "View.MemoryView"
  char *__pyx_t_7;

#line 393 "View.MemoryView"
  int __pyx_lineno = 0;

#line 393 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 393 "View.MemoryView"
  int __pyx_clineno = 0;

#line 393 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":395
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
 */

#line 395 "View.MemoryView"
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */

#line 397 "View.MemoryView"
  __pyx_t_1 = 0;

#line 397 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {

#line 397 "View.MemoryView"
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;

#line 397 "View.MemoryView"
    __pyx_t_4 = NULL;

#line 397 "View.MemoryView"
  } else {
    __pyx_t_3 = -1; 
#line 397 "View.MemoryView"
__pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 397 "View.MemoryView"
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
  }

#line 397 "View.MemoryView"
  for (;;) {

#line 397 "View.MemoryView"
    if (likely(!__pyx_t_4)) {

#line 397 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_2))) {

#line 397 "View.MemoryView"
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;

#line 397 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "View.MemoryView"
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        #else

#line 397 "View.MemoryView"
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
        #endif

#line 397 "View.MemoryView"
      } else {

#line 397 "View.MemoryView"
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;

#line 397 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "View.MemoryView"
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        #else

#line 397 "View.MemoryView"
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
        #endif

#line 397 "View.MemoryView"
      }
    } else 
#line 397 "View.MemoryView"
{

#line 397 "View.MemoryView"
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);

#line 397 "View.MemoryView"
      if (unlikely(!__pyx_t_5)) {

#line 397 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 397 "View.MemoryView"
        if (exc_type) {

#line 397 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 397 "View.MemoryView"
          else __PYX_ERR(1, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        }

#line 397 "View.MemoryView"
        break;

#line 397 "View.MemoryView"
      }

#line 397 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
    }

#line 397 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);

#line 397 "View.MemoryView"
    __pyx_t_5 = 0;

#line 397 "View.MemoryView"
    __pyx_v_dim = __pyx_t_1;

#line 397 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":398
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
 */

#line 398 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 398, __pyx_L1_error)

#line 398 "View.MemoryView"
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == ((char *)NULL))) __PYX_ERR(1, 398, __pyx_L1_error)

#line 398 "View.MemoryView"
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */

#line 397 "View.MemoryView"
  }

#line 397 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":400
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 400 "View.MemoryView"
  __pyx_r = __pyx_v_itemp;

#line 400 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

#line 393 "View.MemoryView"


#line 393 "View.MemoryView"
  /* function exit code */

#line 393 "View.MemoryView"
  __pyx_L1_error:;

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 393 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 393 "View.MemoryView"
  __pyx_r = NULL;

#line 393 "View.MemoryView"
  __pyx_L0:;

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_idx);

#line 393 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 393 "View.MemoryView"
  return __pyx_r;

#line 393 "View.MemoryView"
}

/* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
/* Python wrapper */

#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/

#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {

#line 403 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 403 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 403 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 403 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
  /* function exit code */

#line 403 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 403 "View.MemoryView"
  return __pyx_r;

#line 403 "View.MemoryView"
}

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL
#line 403 "View.MemoryView"
;
  PyObject *__pyx_v_indices = NULL
#line 403 "View.MemoryView"
;
  char *__pyx_v_itemp
#line 403 "View.MemoryView"
;

#line 403 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 403 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 403 "View.MemoryView"
  int __pyx_t_1;

#line 403 "View.MemoryView"
  int __pyx_t_2;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 403 "View.MemoryView"
  char *__pyx_t_6;

#line 403 "View.MemoryView"
  int __pyx_lineno = 0;

#line 403 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 403 "View.MemoryView"
  int __pyx_clineno = 0;

#line 403 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */

#line 404 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);

#line 404 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 404 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":405
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
 */

#line 405 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 405 "View.MemoryView"
    __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 405 "View.MemoryView"
    __pyx_r = ((PyObject *)__pyx_v_self);

#line 405 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */

#line 404 "View.MemoryView"
  }

  /* "View.MemoryView":407
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
 */

#line 407 "View.MemoryView"
  __pyx_t_3 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 407 "View.MemoryView"
  if (likely(__pyx_t_3 != Py_None)) {

#line 407 "View.MemoryView"
    PyObject* sequence = __pyx_t_3;

#line 407 "View.MemoryView"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 407 "View.MemoryView"
    if (unlikely(size != 2)) {

#line 407 "View.MemoryView"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 407 "View.MemoryView"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 407 "View.MemoryView"
      __PYX_ERR(1, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    }

#line 407 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 407 "View.MemoryView"
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 

#line 407 "View.MemoryView"
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 

#line 407 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_4);

#line 407 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_5);

#line 407 "View.MemoryView"
    #else

#line 407 "View.MemoryView"
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 407 "View.MemoryView"
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 407 "View.MemoryView"
    #endif

#line 407 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 407 "View.MemoryView"
  } else {

#line 407 "View.MemoryView"
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
  }

#line 407 "View.MemoryView"
  __pyx_v_have_slices = __pyx_t_4;

#line 407 "View.MemoryView"
  __pyx_t_4 = 0;

#line 407 "View.MemoryView"
  __pyx_v_indices = __pyx_t_5;

#line 407 "View.MemoryView"
  __pyx_t_5 = 0;

  /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */

#line 410 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 410, __pyx_L1_error)

#line 410 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":411
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
 */

#line 411 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 411 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 411, __pyx_L1_error)

#line 411 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 411 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 411 "View.MemoryView"
    __pyx_t_3 = 0;

#line 411 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */

#line 410 "View.MemoryView"
  }

  /* "View.MemoryView":413
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
 */

#line 413 "View.MemoryView"
  /*else*/ {

#line 413 "View.MemoryView"
    __pyx_t_6 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_6 == ((char *)NULL))) __PYX_ERR(1, 413, __pyx_L1_error)

#line 413 "View.MemoryView"
    __pyx_v_itemp = __pyx_t_6;

    /* "View.MemoryView":414
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
 */

#line 414 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 414 "View.MemoryView"
    __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 414, __pyx_L1_error)

#line 414 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 414 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 414 "View.MemoryView"
    __pyx_t_3 = 0;

#line 414 "View.MemoryView"
    goto __pyx_L0;

#line 414 "View.MemoryView"
  }

  /* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
  /* function exit code */

#line 403 "View.MemoryView"
  __pyx_L1_error:;

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 403 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 403 "View.MemoryView"
  __pyx_r = NULL;

#line 403 "View.MemoryView"
  __pyx_L0:;

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_have_slices);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_indices);

#line 403 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 403 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 403 "View.MemoryView"
  return __pyx_r;

#line 403 "View.MemoryView"
}

/* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
/* Python wrapper */

#line 416 "View.MemoryView"
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/

#line 416 "View.MemoryView"
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {

#line 416 "View.MemoryView"
  int __pyx_r;

#line 416 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 416 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = 
#line 416 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
  /* function exit code */

#line 416 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 416 "View.MemoryView"
  return __pyx_r;

#line 416 "View.MemoryView"
}

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL
#line 416 "View.MemoryView"
;
  PyObject *__pyx_v_obj = NULL
#line 416 "View.MemoryView"
;

#line 416 "View.MemoryView"
  int __pyx_r;

#line 416 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 416 "View.MemoryView"
  int __pyx_t_1;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 416 "View.MemoryView"
  int __pyx_lineno = 0;

#line 416 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 416 "View.MemoryView"
  int __pyx_clineno = 0;

#line 416 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__", 0);

#line 416 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */

#line 417 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->view.readonly != 0);

#line 417 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */

#line 418 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 418, __pyx_L1_error)

#line 418 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 418 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 418 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 418 "View.MemoryView"
    __PYX_ERR(1, 418, __pyx_L1_error)

    /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */

#line 417 "View.MemoryView"
  }

  /* "View.MemoryView":420
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
 */

#line 420 "View.MemoryView"
  __pyx_t_2 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 420 "View.MemoryView"
  if (likely(__pyx_t_2 != Py_None)) {

#line 420 "View.MemoryView"
    PyObject* sequence = __pyx_t_2;

#line 420 "View.MemoryView"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 420 "View.MemoryView"
    if (unlikely(size != 2)) {

#line 420 "View.MemoryView"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 420 "View.MemoryView"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 420 "View.MemoryView"
      __PYX_ERR(1, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    }

#line 420 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 420 "View.MemoryView"
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 

#line 420 "View.MemoryView"
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 

#line 420 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 420 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_4);

#line 420 "View.MemoryView"
    #else

#line 420 "View.MemoryView"
    __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 420 "View.MemoryView"
    __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 420 "View.MemoryView"
    #endif

#line 420 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 420 "View.MemoryView"
  } else {

#line 420 "View.MemoryView"
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
  }

#line 420 "View.MemoryView"
  __pyx_v_have_slices = __pyx_t_3;

#line 420 "View.MemoryView"
  __pyx_t_3 = 0;

#line 420 "View.MemoryView"
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_4);

#line 420 "View.MemoryView"
  __pyx_t_4 = 0;

  /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */

#line 422 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 422, __pyx_L1_error)

#line 422 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":423
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)
 */

#line 423 "View.MemoryView"
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 423, __pyx_L1_error)

#line 423 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 423 "View.MemoryView"
    __pyx_v_obj = __pyx_t_2;

#line 423 "View.MemoryView"
    __pyx_t_2 = 0;

    /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */

#line 424 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_obj); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 424, __pyx_L1_error)

#line 424 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":425
 *             obj = self.is_slice(value)
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
 */

#line 425 "View.MemoryView"
      __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 425, __pyx_L1_error)

#line 425 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_2);

#line 425 "View.MemoryView"
      __pyx_t_4 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_2, __pyx_v_obj); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 425, __pyx_L1_error)

#line 425 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 425 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 425 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */

#line 424 "View.MemoryView"
      goto __pyx_L5;

#line 424 "View.MemoryView"
    }

    /* "View.MemoryView":427
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
 */

#line 427 "View.MemoryView"
    /*else*/ {

#line 427 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 427 "View.MemoryView"
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_memoryview_type))))) __PYX_ERR(1, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_4), __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_2);

#line 427 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 427 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 427 "View.MemoryView"
    }

#line 427 "View.MemoryView"
    __pyx_L5:;

    /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */

#line 422 "View.MemoryView"
    goto __pyx_L4;

#line 422 "View.MemoryView"
  }

  /* "View.MemoryView":429
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
 */

#line 429 "View.MemoryView"
  /*else*/ {

#line 429 "View.MemoryView"
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 429, __pyx_L1_error)

#line 429 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 429 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 429 "View.MemoryView"
  }

#line 429 "View.MemoryView"
  __pyx_L4:;

  /* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
  /* function exit code */

#line 416 "View.MemoryView"
  __pyx_r = 0;

#line 416 "View.MemoryView"
  goto __pyx_L0;

#line 416 "View.MemoryView"
  __pyx_L1_error:;

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 416 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 416 "View.MemoryView"
  __pyx_r = -1;

#line 416 "View.MemoryView"
  __pyx_L0:;

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_have_slices);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_obj);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_index);

#line 416 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 416 "View.MemoryView"
  return __pyx_r;

#line 416 "View.MemoryView"
}

/* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

#line 431 "View.MemoryView"


#line 431 "View.MemoryView"
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {

#line 431 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 431 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 431 "View.MemoryView"
  int __pyx_t_1;

#line 431 "View.MemoryView"
  int __pyx_t_2;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 431 "View.MemoryView"
  int __pyx_t_9;

#line 431 "View.MemoryView"
  int __pyx_lineno = 0;

#line 431 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 431 "View.MemoryView"
  int __pyx_clineno = 0;

#line 431 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_slice", 0);

#line 431 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */

#line 432 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_memoryview_type); 

#line 432 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 432 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
    {

#line 433 "View.MemoryView"
      __Pyx_PyThreadState_declare

#line 433 "View.MemoryView"
      __Pyx_PyThreadState_assign

#line 433 "View.MemoryView"
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_5);

#line 433 "View.MemoryView"
      /*try:*/ {

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */

#line 434 "View.MemoryView"
        __pyx_t_6 = __Pyx_PyInt_From_int(((__pyx_v_self->flags & (~PyBUF_WRITABLE)) | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":435
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
 */

#line 435 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 435, __pyx_L4_error)

#line 435 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */

#line 434 "View.MemoryView"
        __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_8);

#line 434 "View.MemoryView"
        __Pyx_INCREF(__pyx_v_obj);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_v_obj);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_obj);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_6);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_7);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);

#line 434 "View.MemoryView"
        __pyx_t_6 = 0;

#line 434 "View.MemoryView"
        __pyx_t_7 = 0;

#line 434 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 434 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 434 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_v_obj, __pyx_t_7);

#line 434 "View.MemoryView"
        __pyx_t_7 = 0;

        /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
      }

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 433 "View.MemoryView"
      goto __pyx_L9_try_end;

#line 433 "View.MemoryView"
      __pyx_L4_error:;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "View.MemoryView":436
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */

#line 436 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);

#line 436 "View.MemoryView"
      if (__pyx_t_9) {

#line 436 "View.MemoryView"
        __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 436 "View.MemoryView"
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_6) < 0) __PYX_ERR(1, 436, __pyx_L6_except_error)

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_8);

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":437
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
 */

#line 437 "View.MemoryView"
        __Pyx_XDECREF(__pyx_r);

#line 437 "View.MemoryView"
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 437 "View.MemoryView"
        goto __pyx_L7_except_return;

#line 437 "View.MemoryView"
      }

#line 437 "View.MemoryView"
      goto __pyx_L6_except_error;

#line 437 "View.MemoryView"
      __pyx_L6_except_error:;

      /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 433 "View.MemoryView"
      goto __pyx_L1_error;

#line 433 "View.MemoryView"
      __pyx_L7_except_return:;

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 433 "View.MemoryView"
      goto __pyx_L0;

#line 433 "View.MemoryView"
      __pyx_L9_try_end:;

#line 433 "View.MemoryView"
    }

    /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */

#line 432 "View.MemoryView"
  }

  /* "View.MemoryView":439
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
 */

#line 439 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 439 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

#line 439 "View.MemoryView"
  __pyx_r = __pyx_v_obj;

#line 439 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

#line 431 "View.MemoryView"


#line 431 "View.MemoryView"
  /* function exit code */

#line 431 "View.MemoryView"
  __pyx_L1_error:;

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_8);

#line 431 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 431 "View.MemoryView"
  __pyx_r = 0;

#line 431 "View.MemoryView"
  __pyx_L0:;

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_obj);

#line 431 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 431 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 431 "View.MemoryView"
  return __pyx_r;

#line 431 "View.MemoryView"
}

/* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

#line 441 "View.MemoryView"


#line 441 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice
#line 441 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_src_slice
#line 441 "View.MemoryView"
;

#line 441 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 441 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 441 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 441 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_2;

#line 441 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 441 "View.MemoryView"
  int __pyx_t_4;

#line 441 "View.MemoryView"
  int __pyx_t_5;

#line 441 "View.MemoryView"
  int __pyx_t_6;

#line 441 "View.MemoryView"
  int __pyx_lineno = 0;

#line 441 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 441 "View.MemoryView"
  int __pyx_clineno = 0;

#line 441 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */

#line 445 "View.MemoryView"
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_memoryview_type))))) __PYX_ERR(1, 445, __pyx_L1_error)

#line 445 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(1, 445, __pyx_L1_error)

  /* "View.MemoryView":446
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],             # <<<<<<<<<<<<<<
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 */

#line 446 "View.MemoryView"
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_memoryview_type))))) __PYX_ERR(1, 446, __pyx_L1_error)

#line 446 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice)); if (unlikely(__pyx_t_2 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(1, 446, __pyx_L1_error)

  /* "View.MemoryView":447
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 */

#line 447 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 447 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 447 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 447 "View.MemoryView"
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */

#line 445 "View.MemoryView"
  __pyx_t_6 = __pyx_memoryview_copy_contents((__pyx_t_1[0]), (__pyx_t_2[0]), __pyx_t_4, __pyx_t_5, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 445, __pyx_L1_error)

  /* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

#line 441 "View.MemoryView"


#line 441 "View.MemoryView"
  /* function exit code */

#line 441 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 441 "View.MemoryView"
  goto __pyx_L0;

#line 441 "View.MemoryView"
  __pyx_L1_error:;

#line 441 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 441 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 441 "View.MemoryView"
  __pyx_r = 0;

#line 441 "View.MemoryView"
  __pyx_L0:;

#line 441 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 441 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 441 "View.MemoryView"
  return __pyx_r;

#line 441 "View.MemoryView"
}

/* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

#line 449 "View.MemoryView"


#line 449 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[0x80]
#line 449 "View.MemoryView"
;
  void *__pyx_v_tmp
#line 449 "View.MemoryView"
;
  void *__pyx_v_item
#line 449 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_dst_slice
#line 449 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp_slice
#line 449 "View.MemoryView"
;

#line 449 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 449 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 449 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 449 "View.MemoryView"
  int __pyx_t_2;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 449 "View.MemoryView"
  int __pyx_t_4;

#line 449 "View.MemoryView"
  int __pyx_t_5;

#line 449 "View.MemoryView"
  char const *__pyx_t_6;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_11 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_12 = NULL;

#line 449 "View.MemoryView"
  int __pyx_lineno = 0;

#line 449 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 449 "View.MemoryView"
  int __pyx_clineno = 0;

#line 449 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":451
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
 */

#line 451 "View.MemoryView"
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":456
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 */

#line 456 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(1, 456, __pyx_L1_error)

#line 456 "View.MemoryView"
  __pyx_v_dst_slice = __pyx_t_1;

  /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */

#line 458 "View.MemoryView"
  __pyx_t_2 = ((((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array))) != 0);

#line 458 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":459
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
 */

#line 459 "View.MemoryView"
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */

#line 460 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_tmp == NULL) != 0);

#line 460 "View.MemoryView"
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":461
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
 */

#line 461 "View.MemoryView"
      PyErr_NoMemory(); __PYX_ERR(1, 461, __pyx_L1_error)

      /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */

#line 460 "View.MemoryView"
    }

    /* "View.MemoryView":462
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
 */

#line 462 "View.MemoryView"
    __pyx_v_item = __pyx_v_tmp;

    /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */

#line 458 "View.MemoryView"
    goto __pyx_L3;

#line 458 "View.MemoryView"
  }

  /* "View.MemoryView":464
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
 */

#line 464 "View.MemoryView"
  /*else*/ {

#line 464 "View.MemoryView"
    __pyx_v_item = ((void *)__pyx_v_array);

#line 464 "View.MemoryView"
  }

#line 464 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":466
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
 */

#line 466 "View.MemoryView"
  /*try:*/ {

    /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */

#line 467 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_self->dtype_is_object != 0);

#line 467 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":468
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
 */

#line 468 "View.MemoryView"
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);

      /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */

#line 467 "View.MemoryView"
      goto __pyx_L8;

#line 467 "View.MemoryView"
    }

    /* "View.MemoryView":470
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 470 "View.MemoryView"
    /*else*/ {

#line 470 "View.MemoryView"
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 470, __pyx_L6_error)

#line 470 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 470 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 470 "View.MemoryView"
    }

#line 470 "View.MemoryView"
    __pyx_L8:;

    /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */

#line 474 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_self->view.suboffsets != NULL) != 0);

#line 474 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":475
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
 */

#line 475 "View.MemoryView"
      __pyx_t_3 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 475, __pyx_L6_error)

#line 475 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 475 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */

#line 474 "View.MemoryView"
    }

    /* "View.MemoryView":476
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
 */

#line 476 "View.MemoryView"
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);

#line 476 "View.MemoryView"
  }

  /* "View.MemoryView":479
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
 */

#line 479 "View.MemoryView"
  /*finally:*/ {

#line 479 "View.MemoryView"
    /*normal exit:*/{

#line 479 "View.MemoryView"
      PyMem_Free(__pyx_v_tmp);

#line 479 "View.MemoryView"
      goto __pyx_L7;

#line 479 "View.MemoryView"
    }

#line 479 "View.MemoryView"
    __pyx_L6_error:;

#line 479 "View.MemoryView"
    /*exception exit:*/{

#line 479 "View.MemoryView"
      __Pyx_PyThreadState_declare

#line 479 "View.MemoryView"
      __Pyx_PyThreadState_assign

#line 479 "View.MemoryView"
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;

#line 479 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 479 "View.MemoryView"
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 479 "View.MemoryView"
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_7);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_8);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_10);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_11);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_12);

#line 479 "View.MemoryView"
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;

#line 479 "View.MemoryView"
      {

#line 479 "View.MemoryView"
        PyMem_Free(__pyx_v_tmp);

#line 479 "View.MemoryView"
      }

#line 479 "View.MemoryView"
      if (PY_MAJOR_VERSION >= 3) {

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_10);

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_11);

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_12);

#line 479 "View.MemoryView"
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 479 "View.MemoryView"
      }

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_7);

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_8);

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);

#line 479 "View.MemoryView"
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;

#line 479 "View.MemoryView"
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;

#line 479 "View.MemoryView"
      goto __pyx_L1_error;

#line 479 "View.MemoryView"
    }

#line 479 "View.MemoryView"
    __pyx_L7:;

#line 479 "View.MemoryView"
  }

  /* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

#line 449 "View.MemoryView"


#line 449 "View.MemoryView"
  /* function exit code */

#line 449 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 449 "View.MemoryView"
  goto __pyx_L0;

#line 449 "View.MemoryView"
  __pyx_L1_error:;

#line 449 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 449 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 449 "View.MemoryView"
  __pyx_r = 0;

#line 449 "View.MemoryView"
  __pyx_L0:;

#line 449 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 449 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 449 "View.MemoryView"
  return __pyx_r;

#line 449 "View.MemoryView"
}

/* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

#line 481 "View.MemoryView"


#line 481 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp
#line 481 "View.MemoryView"
;

#line 481 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 481 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 481 "View.MemoryView"
  char *__pyx_t_1;

#line 481 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 481 "View.MemoryView"
  int __pyx_lineno = 0;

#line 481 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 481 "View.MemoryView"
  int __pyx_clineno = 0;

#line 481 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":482
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
 */

#line 482 "View.MemoryView"
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(1, 482, __pyx_L1_error)

#line 482 "View.MemoryView"
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":483
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */

#line 483 "View.MemoryView"
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 483, __pyx_L1_error)

#line 483 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 483 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

#line 481 "View.MemoryView"


#line 481 "View.MemoryView"
  /* function exit code */

#line 481 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 481 "View.MemoryView"
  goto __pyx_L0;

#line 481 "View.MemoryView"
  __pyx_L1_error:;

#line 481 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 481 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 481 "View.MemoryView"
  __pyx_r = 0;

#line 481 "View.MemoryView"
  __pyx_L0:;

#line 481 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 481 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 481 "View.MemoryView"
  return __pyx_r;

#line 481 "View.MemoryView"
}

/* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 485 "View.MemoryView"


#line 485 "View.MemoryView"
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL
#line 485 "View.MemoryView"
;
  PyObject *__pyx_v_bytesitem = 0
#line 485 "View.MemoryView"
;
  PyObject *__pyx_v_result = NULL
#line 485 "View.MemoryView"
;

#line 485 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 485 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 485 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 485 "View.MemoryView"
  int __pyx_t_8;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 485 "View.MemoryView"
  size_t __pyx_t_10;

#line 485 "View.MemoryView"
  int __pyx_t_11;

#line 485 "View.MemoryView"
  int __pyx_lineno = 0;

#line 485 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 485 "View.MemoryView"
  int __pyx_clineno = 0;

#line 485 "View.MemoryView"
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":488
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
 */

#line 488 "View.MemoryView"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 488, __pyx_L1_error)

#line 488 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 488 "View.MemoryView"
  __pyx_v_struct = __pyx_t_1;

#line 488 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":491
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 */

#line 491 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 491, __pyx_L1_error)

#line 491 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 491 "View.MemoryView"
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);

#line 491 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
  {

#line 492 "View.MemoryView"
    __Pyx_PyThreadState_declare

#line 492 "View.MemoryView"
    __Pyx_PyThreadState_assign

#line 492 "View.MemoryView"
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_4);

#line 492 "View.MemoryView"
    /*try:*/ {

      /* "View.MemoryView":493
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")
 */

#line 493 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 493 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 493 "View.MemoryView"
      __pyx_t_7 = NULL;

#line 493 "View.MemoryView"
      __pyx_t_8 = 0;

#line 493 "View.MemoryView"
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 493 "View.MemoryView"
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);

#line 493 "View.MemoryView"
        if (likely(__pyx_t_7)) {

#line 493 "View.MemoryView"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 493 "View.MemoryView"
          __Pyx_INCREF(__pyx_t_7);

#line 493 "View.MemoryView"
          __Pyx_INCREF(function);

#line 493 "View.MemoryView"
          __Pyx_DECREF_SET(__pyx_t_5, function);

#line 493 "View.MemoryView"
          __pyx_t_8 = 1;

#line 493 "View.MemoryView"
        }

#line 493 "View.MemoryView"
      }

#line 493 "View.MemoryView"
      #if CYTHON_FAST_PYCALL

#line 493 "View.MemoryView"
      if (PyFunction_Check(__pyx_t_5)) {

#line 493 "View.MemoryView"
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 493 "View.MemoryView"
      } else

#line 493 "View.MemoryView"
      #endif

#line 493 "View.MemoryView"
      #if CYTHON_FAST_PYCCALL

#line 493 "View.MemoryView"
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {

#line 493 "View.MemoryView"
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 493 "View.MemoryView"
      } else

#line 493 "View.MemoryView"
      #endif

#line 493 "View.MemoryView"
      {

#line 493 "View.MemoryView"
        __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 493 "View.MemoryView"
        if (__pyx_t_7) {

#line 493 "View.MemoryView"
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;

#line 493 "View.MemoryView"
        }

#line 493 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_6);

#line 493 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);

#line 493 "View.MemoryView"
        __Pyx_INCREF(__pyx_v_bytesitem);

#line 493 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_v_bytesitem);

#line 493 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_bytesitem);

#line 493 "View.MemoryView"
        __pyx_t_6 = 0;

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 493 "View.MemoryView"
      }

#line 493 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 493 "View.MemoryView"
      __pyx_v_result = __pyx_t_1;

#line 493 "View.MemoryView"
      __pyx_t_1 = 0;

      /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
    }

    /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */

#line 497 "View.MemoryView"
    /*else:*/ {

#line 497 "View.MemoryView"
      __pyx_t_10 = strlen(__pyx_v_self->view.format); 

#line 497 "View.MemoryView"
      __pyx_t_11 = ((__pyx_t_10 == 1) != 0);

#line 497 "View.MemoryView"
      if (__pyx_t_11) {

        /* "View.MemoryView":498
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
 */

#line 498 "View.MemoryView"
        __Pyx_XDECREF(__pyx_r);

#line 498 "View.MemoryView"
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 498, __pyx_L5_except_error)

#line 498 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 498 "View.MemoryView"
        __pyx_r = __pyx_t_1;

#line 498 "View.MemoryView"
        __pyx_t_1 = 0;

#line 498 "View.MemoryView"
        goto __pyx_L6_except_return;

        /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */

#line 497 "View.MemoryView"
      }

      /* "View.MemoryView":499
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */

#line 499 "View.MemoryView"
      __Pyx_XDECREF(__pyx_r);

#line 499 "View.MemoryView"
      __Pyx_INCREF(__pyx_v_result);

#line 499 "View.MemoryView"
      __pyx_r = __pyx_v_result;

#line 499 "View.MemoryView"
      goto __pyx_L6_except_return;

#line 499 "View.MemoryView"
    }

#line 499 "View.MemoryView"
    __pyx_L3_error:;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "View.MemoryView":494
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError("Unable to convert item to object")
 *         else:
 */

#line 494 "View.MemoryView"
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_5, &__pyx_t_9);

#line 494 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 494, __pyx_L5_except_error)

#line 494 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 494 "View.MemoryView"
    __pyx_t_8 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_6);

#line 494 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 494 "View.MemoryView"
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_5, __pyx_t_9);

#line 494 "View.MemoryView"
    __pyx_t_1 = 0; __pyx_t_5 = 0; __pyx_t_9 = 0;

#line 494 "View.MemoryView"
    if (__pyx_t_8) {

#line 494 "View.MemoryView"
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 494 "View.MemoryView"
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_5, &__pyx_t_1) < 0) __PYX_ERR(1, 494, __pyx_L5_except_error)

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_1);

      /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */

#line 495 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 495, __pyx_L5_except_error)

#line 495 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 495 "View.MemoryView"
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 495 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 495 "View.MemoryView"
      __PYX_ERR(1, 495, __pyx_L5_except_error)

#line 495 "View.MemoryView"
    }

#line 495 "View.MemoryView"
    goto __pyx_L5_except_error;

#line 495 "View.MemoryView"
    __pyx_L5_except_error:;

    /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);

#line 492 "View.MemoryView"
    goto __pyx_L1_error;

#line 492 "View.MemoryView"
    __pyx_L6_except_return:;

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);

#line 492 "View.MemoryView"
    goto __pyx_L0;

#line 492 "View.MemoryView"
  }

  /* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 485 "View.MemoryView"


#line 485 "View.MemoryView"
  /* function exit code */

#line 485 "View.MemoryView"
  __pyx_L1_error:;

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_9);

#line 485 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 485 "View.MemoryView"
  __pyx_r = 0;

#line 485 "View.MemoryView"
  __pyx_L0:;

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_struct);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_bytesitem);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 485 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 485 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 485 "View.MemoryView"
  return __pyx_r;

#line 485 "View.MemoryView"
}

/* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 501 "View.MemoryView"


#line 501 "View.MemoryView"
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL
#line 501 "View.MemoryView"
;
  char __pyx_v_c
#line 501 "View.MemoryView"
;
  PyObject *__pyx_v_bytesvalue = 0
#line 501 "View.MemoryView"
;
  Py_ssize_t __pyx_v_i
#line 501 "View.MemoryView"
;

#line 501 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 501 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 501 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 501 "View.MemoryView"
  int __pyx_t_2;

#line 501 "View.MemoryView"
  int __pyx_t_3;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 501 "View.MemoryView"
  int __pyx_t_7;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 501 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 501 "View.MemoryView"
  char *__pyx_t_11;

#line 501 "View.MemoryView"
  char *__pyx_t_12;

#line 501 "View.MemoryView"
  char *__pyx_t_13;

#line 501 "View.MemoryView"
  char *__pyx_t_14;

#line 501 "View.MemoryView"
  int __pyx_lineno = 0;

#line 501 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 501 "View.MemoryView"
  int __pyx_clineno = 0;

#line 501 "View.MemoryView"
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":504
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
 */

#line 504 "View.MemoryView"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 504, __pyx_L1_error)

#line 504 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 504 "View.MemoryView"
  __pyx_v_struct = __pyx_t_1;

#line 504 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */

#line 509 "View.MemoryView"
  __pyx_t_2 = PyTuple_Check(__pyx_v_value); 

#line 509 "View.MemoryView"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 509 "View.MemoryView"
  if (__pyx_t_3) {

    /* "View.MemoryView":510
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
 */

#line 510 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_1);

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 510 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 510 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_4 = 0;

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 510 "View.MemoryView"
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(1, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_4 = 0;

    /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */

#line 509 "View.MemoryView"
    goto __pyx_L3;

#line 509 "View.MemoryView"
  }

  /* "View.MemoryView":512
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
 */

#line 512 "View.MemoryView"
  /*else*/ {

#line 512 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 512 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_1);

#line 512 "View.MemoryView"
    __pyx_t_5 = NULL;

#line 512 "View.MemoryView"
    __pyx_t_7 = 0;

#line 512 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {

#line 512 "View.MemoryView"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);

#line 512 "View.MemoryView"
      if (likely(__pyx_t_5)) {

#line 512 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);

#line 512 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_5);

#line 512 "View.MemoryView"
        __Pyx_INCREF(function);

#line 512 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_6, function);

#line 512 "View.MemoryView"
        __pyx_t_7 = 1;

#line 512 "View.MemoryView"
      }

#line 512 "View.MemoryView"
    }

#line 512 "View.MemoryView"
    #if CYTHON_FAST_PYCALL

#line 512 "View.MemoryView"
    if (PyFunction_Check(__pyx_t_6)) {

#line 512 "View.MemoryView"
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "View.MemoryView"
    } else

#line 512 "View.MemoryView"
    #endif

#line 512 "View.MemoryView"
    #if CYTHON_FAST_PYCCALL

#line 512 "View.MemoryView"
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {

#line 512 "View.MemoryView"
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "View.MemoryView"
    } else

#line 512 "View.MemoryView"
    #endif

#line 512 "View.MemoryView"
    {

#line 512 "View.MemoryView"
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_8);

#line 512 "View.MemoryView"
      if (__pyx_t_5) {

#line 512 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;

#line 512 "View.MemoryView"
      }

#line 512 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_1);

#line 512 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_1);

#line 512 "View.MemoryView"
      __Pyx_INCREF(__pyx_v_value);

#line 512 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_v_value);

#line 512 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_value);

#line 512 "View.MemoryView"
      __pyx_t_1 = 0;

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 512 "View.MemoryView"
    }

#line 512 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 512 "View.MemoryView"
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(1, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);

#line 512 "View.MemoryView"
    __pyx_t_4 = 0;

#line 512 "View.MemoryView"
  }

#line 512 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */

#line 514 "View.MemoryView"
  __pyx_t_9 = 0;

#line 514 "View.MemoryView"
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {

#line 514 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");

#line 514 "View.MemoryView"
    __PYX_ERR(1, 514, __pyx_L1_error)

#line 514 "View.MemoryView"
  }

#line 514 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_bytesvalue);

#line 514 "View.MemoryView"
  __pyx_t_10 = __pyx_v_bytesvalue;

#line 514 "View.MemoryView"
  __pyx_t_12 = PyBytes_AS_STRING(__pyx_t_10);

#line 514 "View.MemoryView"
  __pyx_t_13 = (__pyx_t_12 + PyBytes_GET_SIZE(__pyx_t_10));

#line 514 "View.MemoryView"
  for (__pyx_t_14 = __pyx_t_12; __pyx_t_14 < __pyx_t_13; __pyx_t_14++) {

#line 514 "View.MemoryView"
    __pyx_t_11 = __pyx_t_14;

#line 514 "View.MemoryView"
    __pyx_v_c = (__pyx_t_11[0]);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 515 "View.MemoryView"
    __pyx_v_i = __pyx_t_9;

    /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */

#line 514 "View.MemoryView"
    __pyx_t_9 = (__pyx_t_9 + 1);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 515 "View.MemoryView"
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;

#line 515 "View.MemoryView"
  }

#line 515 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 501 "View.MemoryView"


#line 501 "View.MemoryView"
  /* function exit code */

#line 501 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 501 "View.MemoryView"
  goto __pyx_L0;

#line 501 "View.MemoryView"
  __pyx_L1_error:;

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_8);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_10);

#line 501 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 501 "View.MemoryView"
  __pyx_r = 0;

#line 501 "View.MemoryView"
  __pyx_L0:;

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_struct);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_bytesvalue);

#line 501 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 501 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 501 "View.MemoryView"
  return __pyx_r;

#line 501 "View.MemoryView"
}

/* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
/* Python wrapper */

#line 518 "View.MemoryView"
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/

#line 518 "View.MemoryView"
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 518 "View.MemoryView"
  int __pyx_r;

#line 518 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 518 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = 
#line 518 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
  /* function exit code */

#line 518 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 518 "View.MemoryView"
  return __pyx_r;

#line 518 "View.MemoryView"
}

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 518 "View.MemoryView"
  int __pyx_r;

#line 518 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 518 "View.MemoryView"
  int __pyx_t_1;

#line 518 "View.MemoryView"
  int __pyx_t_2;

#line 518 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 518 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 518 "View.MemoryView"
  char *__pyx_t_5;

#line 518 "View.MemoryView"
  void *__pyx_t_6;

#line 518 "View.MemoryView"
  int __pyx_t_7;

#line 518 "View.MemoryView"
  Py_ssize_t __pyx_t_8;

#line 518 "View.MemoryView"
  int __pyx_lineno = 0;

#line 518 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 518 "View.MemoryView"
  int __pyx_clineno = 0;

#line 518 "View.MemoryView"
  if (__pyx_v_info == NULL) {

#line 518 "View.MemoryView"
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");

#line 518 "View.MemoryView"
    return -1;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__", 0);

#line 518 "View.MemoryView"
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);

#line 518 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */

#line 519 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_flags & PyBUF_WRITABLE) != 0);

#line 519 "View.MemoryView"
  if (__pyx_t_2) {

#line 519 "View.MemoryView"
  } else {

#line 519 "View.MemoryView"
    __pyx_t_1 = __pyx_t_2;

#line 519 "View.MemoryView"
    goto __pyx_L4_bool_binop_done;

#line 519 "View.MemoryView"
  }

#line 519 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_self->view.readonly != 0);

#line 519 "View.MemoryView"
  __pyx_t_1 = __pyx_t_2;

#line 519 "View.MemoryView"
  __pyx_L4_bool_binop_done:;

#line 519 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */

#line 520 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 520, __pyx_L1_error)

#line 520 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 520 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 520 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 520 "View.MemoryView"
    __PYX_ERR(1, 520, __pyx_L1_error)

    /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */

#line 519 "View.MemoryView"
  }

  /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */

#line 522 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_ND) != 0);

#line 522 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":523
 * 
 *         if flags & PyBUF_ND:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
 */

#line 523 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.shape;

#line 523 "View.MemoryView"
    __pyx_v_info->shape = __pyx_t_4;

    /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */

#line 522 "View.MemoryView"
    goto __pyx_L6;

#line 522 "View.MemoryView"
  }

  /* "View.MemoryView":525
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
 */

#line 525 "View.MemoryView"
  /*else*/ {

#line 525 "View.MemoryView"
    __pyx_v_info->shape = NULL;

#line 525 "View.MemoryView"
  }

#line 525 "View.MemoryView"
  __pyx_L6:;

  /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */

#line 527 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);

#line 527 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":528
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
 */

#line 528 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.strides;

#line 528 "View.MemoryView"
    __pyx_v_info->strides = __pyx_t_4;

    /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */

#line 527 "View.MemoryView"
    goto __pyx_L7;

#line 527 "View.MemoryView"
  }

  /* "View.MemoryView":530
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
 */

#line 530 "View.MemoryView"
  /*else*/ {

#line 530 "View.MemoryView"
    __pyx_v_info->strides = NULL;

#line 530 "View.MemoryView"
  }

#line 530 "View.MemoryView"
  __pyx_L7:;

  /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */

#line 532 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);

#line 532 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":533
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
 */

#line 533 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.suboffsets;

#line 533 "View.MemoryView"
    __pyx_v_info->suboffsets = __pyx_t_4;

    /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */

#line 532 "View.MemoryView"
    goto __pyx_L8;

#line 532 "View.MemoryView"
  }

  /* "View.MemoryView":535
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 535 "View.MemoryView"
  /*else*/ {

#line 535 "View.MemoryView"
    __pyx_v_info->suboffsets = NULL;

#line 535 "View.MemoryView"
  }

#line 535 "View.MemoryView"
  __pyx_L8:;

  /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */

#line 537 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 537 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":538
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */

#line 538 "View.MemoryView"
    __pyx_t_5 = __pyx_v_self->view.format;

#line 538 "View.MemoryView"
    __pyx_v_info->format = __pyx_t_5;

    /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */

#line 537 "View.MemoryView"
    goto __pyx_L9;

#line 537 "View.MemoryView"
  }

  /* "View.MemoryView":540
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
 */

#line 540 "View.MemoryView"
  /*else*/ {

#line 540 "View.MemoryView"
    __pyx_v_info->format = NULL;

#line 540 "View.MemoryView"
  }

#line 540 "View.MemoryView"
  __pyx_L9:;

  /* "View.MemoryView":542
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 */

#line 542 "View.MemoryView"
  __pyx_t_6 = __pyx_v_self->view.buf;

#line 542 "View.MemoryView"
  __pyx_v_info->buf = __pyx_t_6;

  /* "View.MemoryView":543
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 */

#line 543 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->view.ndim;

#line 543 "View.MemoryView"
  __pyx_v_info->ndim = __pyx_t_7;

  /* "View.MemoryView":544
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 */

#line 544 "View.MemoryView"
  __pyx_t_8 = __pyx_v_self->view.itemsize;

#line 544 "View.MemoryView"
  __pyx_v_info->itemsize = __pyx_t_8;

  /* "View.MemoryView":545
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = self.view.readonly
 *         info.obj = self
 */

#line 545 "View.MemoryView"
  __pyx_t_8 = __pyx_v_self->view.len;

#line 545 "View.MemoryView"
  __pyx_v_info->len = __pyx_t_8;

  /* "View.MemoryView":546
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
 */

#line 546 "View.MemoryView"
  __pyx_t_1 = __pyx_v_self->view.readonly;

#line 546 "View.MemoryView"
  __pyx_v_info->readonly = __pyx_t_1;

  /* "View.MemoryView":547
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 547 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 547 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 547 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_info->obj);

#line 547 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_info->obj);

#line 547 "View.MemoryView"
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
  /* function exit code */

#line 518 "View.MemoryView"
  __pyx_r = 0;

#line 518 "View.MemoryView"
  goto __pyx_L0;

#line 518 "View.MemoryView"
  __pyx_L1_error:;

#line 518 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 518 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 518 "View.MemoryView"
  __pyx_r = -1;

#line 518 "View.MemoryView"
  if (__pyx_v_info->obj != NULL) {

#line 518 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 518 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  goto __pyx_L2;

#line 518 "View.MemoryView"
  __pyx_L0:;

#line 518 "View.MemoryView"
  if (__pyx_v_info->obj == Py_None) {

#line 518 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 518 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  __pyx_L2:;

#line 518 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 518 "View.MemoryView"
  return __pyx_r;

#line 518 "View.MemoryView"
}

/* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
/* Python wrapper */

#line 553 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 553 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self) {

#line 553 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 553 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 553 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 553 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
  /* function exit code */

#line 553 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 553 "View.MemoryView"
  return __pyx_r;

#line 553 "View.MemoryView"
}

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0
#line 553 "View.MemoryView"
;

#line 553 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 553 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 553 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 553 "View.MemoryView"
  int __pyx_t_2;

#line 553 "View.MemoryView"
  int __pyx_lineno = 0;

#line 553 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 553 "View.MemoryView"
  int __pyx_clineno = 0;

#line 553 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":554
 *     @property
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&result.from_slice)
 *         return result
 */

#line 554 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 554, __pyx_L1_error)

#line 554 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 554 "View.MemoryView"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_memoryviewslice_type))))) __PYX_ERR(1, 554, __pyx_L1_error)

#line 554 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);

#line 554 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":555
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */

#line 555 "View.MemoryView"
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(1, 555, __pyx_L1_error)

  /* "View.MemoryView":556
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 556 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 556 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 556 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 556 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
  /* function exit code */

#line 553 "View.MemoryView"
  __pyx_L1_error:;

#line 553 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 553 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 553 "View.MemoryView"
  __pyx_r = NULL;

#line 553 "View.MemoryView"
  __pyx_L0:;

#line 553 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 553 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 553 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 553 "View.MemoryView"
  return __pyx_r;

#line 553 "View.MemoryView"
}

/* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
/* Python wrapper */

#line 559 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 559 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self) {

#line 559 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 559 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 559 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 559 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
  /* function exit code */

#line 559 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 559 "View.MemoryView"
  return __pyx_r;

#line 559 "View.MemoryView"
}

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 559 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 559 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 559 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":560
 *     @property
 *     def base(self):
 *         return self.obj             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 560 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 560 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->obj);

#line 560 "View.MemoryView"
  __pyx_r = __pyx_v_self->obj;

#line 560 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
  /* function exit code */

#line 559 "View.MemoryView"
  __pyx_L0:;

#line 559 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 559 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 559 "View.MemoryView"
  return __pyx_r;

#line 559 "View.MemoryView"
}

/* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
/* Python wrapper */

#line 563 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 563 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self) {

#line 563 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 563 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 563 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 563 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
  /* function exit code */

#line 563 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 563 "View.MemoryView"
  return __pyx_r;

#line 563 "View.MemoryView"
}

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_length
#line 563 "View.MemoryView"
;

#line 563 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 563 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 563 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 563 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 563 "View.MemoryView"
  int __pyx_lineno = 0;

#line 563 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 563 "View.MemoryView"
  int __pyx_clineno = 0;

#line 563 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":564
 *     @property
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 564 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 564 "View.MemoryView"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 564 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);

#line 564 "View.MemoryView"
  for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {

#line 564 "View.MemoryView"
    __pyx_t_2 = __pyx_t_4;

#line 564 "View.MemoryView"
    __pyx_v_length = (__pyx_t_2[0]);

#line 564 "View.MemoryView"
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 564 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(1, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 564 "View.MemoryView"
  }

#line 564 "View.MemoryView"
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_5);

#line 564 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 564 "View.MemoryView"
  __pyx_r = __pyx_t_5;

#line 564 "View.MemoryView"
  __pyx_t_5 = 0;

#line 564 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
  /* function exit code */

#line 563 "View.MemoryView"
  __pyx_L1_error:;

#line 563 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 563 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 563 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 563 "View.MemoryView"
  __pyx_r = NULL;

#line 563 "View.MemoryView"
  __pyx_L0:;

#line 563 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 563 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 563 "View.MemoryView"
  return __pyx_r;

#line 563 "View.MemoryView"
}

/* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
/* Python wrapper */

#line 567 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 567 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self) {

#line 567 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 567 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 567 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 567 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
  /* function exit code */

#line 567 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 567 "View.MemoryView"
  return __pyx_r;

#line 567 "View.MemoryView"
}

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_stride
#line 567 "View.MemoryView"
;

#line 567 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 567 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 567 "View.MemoryView"
  int __pyx_t_1;

#line 567 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 567 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 567 "View.MemoryView"
  int __pyx_lineno = 0;

#line 567 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 567 "View.MemoryView"
  int __pyx_clineno = 0;

#line 567 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */

#line 568 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.strides == NULL) != 0);

#line 568 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */

#line 570 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 570, __pyx_L1_error)

#line 570 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 570 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 570 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 570 "View.MemoryView"
    __PYX_ERR(1, 570, __pyx_L1_error)

    /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */

#line 568 "View.MemoryView"
  }

  /* "View.MemoryView":572
 *             raise ValueError("Buffer view does not expose strides")
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 572 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 572 "View.MemoryView"
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 572 "View.MemoryView"
  __pyx_t_4 = (__pyx_v_self->view.strides + __pyx_v_self->view.ndim);

#line 572 "View.MemoryView"
  for (__pyx_t_5 = __pyx_v_self->view.strides; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {

#line 572 "View.MemoryView"
    __pyx_t_3 = __pyx_t_5;

#line 572 "View.MemoryView"
    __pyx_v_stride = (__pyx_t_3[0]);

#line 572 "View.MemoryView"
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_stride); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 572 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 572 "View.MemoryView"
  }

#line 572 "View.MemoryView"
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_6);

#line 572 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 572 "View.MemoryView"
  __pyx_r = __pyx_t_6;

#line 572 "View.MemoryView"
  __pyx_t_6 = 0;

#line 572 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
  /* function exit code */

#line 567 "View.MemoryView"
  __pyx_L1_error:;

#line 567 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 567 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 567 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 567 "View.MemoryView"
  __pyx_r = NULL;

#line 567 "View.MemoryView"
  __pyx_L0:;

#line 567 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 567 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 567 "View.MemoryView"
  return __pyx_r;

#line 567 "View.MemoryView"
}

/* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
/* Python wrapper */

#line 575 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 575 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self) {

#line 575 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 575 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 575 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 575 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
  /* function exit code */

#line 575 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 575 "View.MemoryView"
  return __pyx_r;

#line 575 "View.MemoryView"
}

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_suboffset
#line 575 "View.MemoryView"
;

#line 575 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 575 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 575 "View.MemoryView"
  int __pyx_t_1;

#line 575 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 575 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_6;

#line 575 "View.MemoryView"
  int __pyx_lineno = 0;

#line 575 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 575 "View.MemoryView"
  int __pyx_clineno = 0;

#line 575 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */

#line 576 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.suboffsets == NULL) != 0);

#line 576 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */

#line 577 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 577 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 577 "View.MemoryView"
    __pyx_t_3 = PyNumber_Multiply(__pyx_tuple__23, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 577 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 577 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 577 "View.MemoryView"
    __pyx_t_3 = 0;

#line 577 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */

#line 576 "View.MemoryView"
  }

  /* "View.MemoryView":579
 *             return (-1,) * self.view.ndim
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 579 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 579 "View.MemoryView"
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 579 "View.MemoryView"
  __pyx_t_5 = (__pyx_v_self->view.suboffsets + __pyx_v_self->view.ndim);

#line 579 "View.MemoryView"
  for (__pyx_t_6 = __pyx_v_self->view.suboffsets; __pyx_t_6 < __pyx_t_5; __pyx_t_6++) {

#line 579 "View.MemoryView"
    __pyx_t_4 = __pyx_t_6;

#line 579 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_4[0]);

#line 579 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_suboffset); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 579 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_2))) __PYX_ERR(1, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 579 "View.MemoryView"
  }

#line 579 "View.MemoryView"
  __pyx_t_2 = PyList_AsTuple(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 579 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 579 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 579 "View.MemoryView"
  __pyx_t_2 = 0;

#line 579 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
  /* function exit code */

#line 575 "View.MemoryView"
  __pyx_L1_error:;

#line 575 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 575 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 575 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 575 "View.MemoryView"
  __pyx_r = NULL;

#line 575 "View.MemoryView"
  __pyx_L0:;

#line 575 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 575 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 575 "View.MemoryView"
  return __pyx_r;

#line 575 "View.MemoryView"
}

/* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
/* Python wrapper */

#line 582 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 582 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self) {

#line 582 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 582 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 582 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 582 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
  /* function exit code */

#line 582 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 582 "View.MemoryView"
  return __pyx_r;

#line 582 "View.MemoryView"
}

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 582 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 582 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 582 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 582 "View.MemoryView"
  int __pyx_lineno = 0;

#line 582 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 582 "View.MemoryView"
  int __pyx_clineno = 0;

#line 582 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":583
 *     @property
 *     def ndim(self):
 *         return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 583 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 583 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 583, __pyx_L1_error)

#line 583 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 583 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 583 "View.MemoryView"
  __pyx_t_1 = 0;

#line 583 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
  /* function exit code */

#line 582 "View.MemoryView"
  __pyx_L1_error:;

#line 582 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 582 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 582 "View.MemoryView"
  __pyx_r = NULL;

#line 582 "View.MemoryView"
  __pyx_L0:;

#line 582 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 582 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 582 "View.MemoryView"
  return __pyx_r;

#line 582 "View.MemoryView"
}

/* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
/* Python wrapper */

#line 586 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 586 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self) {

#line 586 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 586 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 586 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 586 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
  /* function exit code */

#line 586 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 586 "View.MemoryView"
  return __pyx_r;

#line 586 "View.MemoryView"
}

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 586 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 586 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 586 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 586 "View.MemoryView"
  int __pyx_lineno = 0;

#line 586 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 586 "View.MemoryView"
  int __pyx_clineno = 0;

#line 586 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":587
 *     @property
 *     def itemsize(self):
 *         return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 587 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 587 "View.MemoryView"
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 587, __pyx_L1_error)

#line 587 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 587 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 587 "View.MemoryView"
  __pyx_t_1 = 0;

#line 587 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
  /* function exit code */

#line 586 "View.MemoryView"
  __pyx_L1_error:;

#line 586 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 586 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 586 "View.MemoryView"
  __pyx_r = NULL;

#line 586 "View.MemoryView"
  __pyx_L0:;

#line 586 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 586 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 586 "View.MemoryView"
  return __pyx_r;

#line 586 "View.MemoryView"
}

/* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
/* Python wrapper */

#line 590 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 590 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self) {

#line 590 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 590 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 590 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 590 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
  /* function exit code */

#line 590 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 590 "View.MemoryView"
  return __pyx_r;

#line 590 "View.MemoryView"
}

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 590 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 590 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 590 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 590 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 590 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 590 "View.MemoryView"
  int __pyx_lineno = 0;

#line 590 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 590 "View.MemoryView"
  int __pyx_clineno = 0;

#line 590 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":591
 *     @property
 *     def nbytes(self):
 *         return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 591 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 591 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 591 "View.MemoryView"
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 591 "View.MemoryView"
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 591 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 591 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 591 "View.MemoryView"
  __pyx_r = __pyx_t_3;

#line 591 "View.MemoryView"
  __pyx_t_3 = 0;

#line 591 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
  /* function exit code */

#line 590 "View.MemoryView"
  __pyx_L1_error:;

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 590 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 590 "View.MemoryView"
  __pyx_r = NULL;

#line 590 "View.MemoryView"
  __pyx_L0:;

#line 590 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 590 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 590 "View.MemoryView"
  return __pyx_r;

#line 590 "View.MemoryView"
}

/* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
/* Python wrapper */

#line 594 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 594 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self) {

#line 594 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 594 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 594 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 594 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
  /* function exit code */

#line 594 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 594 "View.MemoryView"
  return __pyx_r;

#line 594 "View.MemoryView"
}

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL
#line 594 "View.MemoryView"
;
  PyObject *__pyx_v_length = NULL
#line 594 "View.MemoryView"
;

#line 594 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 594 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 594 "View.MemoryView"
  int __pyx_t_1;

#line 594 "View.MemoryView"
  int __pyx_t_2;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 594 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 594 "View.MemoryView"
  int __pyx_lineno = 0;

#line 594 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 594 "View.MemoryView"
  int __pyx_clineno = 0;

#line 594 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */

#line 595 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);

#line 595 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 595 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":596
 *     def size(self):
 *         if self._size is None:
 *             result = 1             # <<<<<<<<<<<<<<
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 */

#line 596 "View.MemoryView"
    __Pyx_INCREF(__pyx_int_1);

#line 596 "View.MemoryView"
    __pyx_v_result = __pyx_int_1;

    /* "View.MemoryView":598
 *             result = 1
 * 
 *             for length in self.view.shape[:self.view.ndim]:             # <<<<<<<<<<<<<<
 *                 result *= length
 * 
 */

#line 598 "View.MemoryView"
    __pyx_t_4 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);

#line 598 "View.MemoryView"
    for (__pyx_t_5 = __pyx_v_self->view.shape; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {

#line 598 "View.MemoryView"
      __pyx_t_3 = __pyx_t_5;

#line 598 "View.MemoryView"
      __pyx_t_6 = PyInt_FromSsize_t((__pyx_t_3[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 598, __pyx_L1_error)

#line 598 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 598 "View.MemoryView"
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_6);

#line 598 "View.MemoryView"
      __pyx_t_6 = 0;

      /* "View.MemoryView":599
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 *                 result *= length             # <<<<<<<<<<<<<<
 * 
 *             self._size = result
 */

#line 599 "View.MemoryView"
      __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 599, __pyx_L1_error)

#line 599 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 599 "View.MemoryView"
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_6);

#line 599 "View.MemoryView"
      __pyx_t_6 = 0;

#line 599 "View.MemoryView"
    }

    /* "View.MemoryView":601
 *                 result *= length
 * 
 *             self._size = result             # <<<<<<<<<<<<<<
 * 
 *         return self._size
 */

#line 601 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_result);

#line 601 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_result);

#line 601 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->_size);

#line 601 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->_size);

#line 601 "View.MemoryView"
    __pyx_v_self->_size = __pyx_v_result;

    /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */

#line 595 "View.MemoryView"
  }

  /* "View.MemoryView":603
 *             self._size = result
 * 
 *         return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 603 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 603 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->_size);

#line 603 "View.MemoryView"
  __pyx_r = __pyx_v_self->_size;

#line 603 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
  /* function exit code */

#line 594 "View.MemoryView"
  __pyx_L1_error:;

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 594 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 594 "View.MemoryView"
  __pyx_r = NULL;

#line 594 "View.MemoryView"
  __pyx_L0:;

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_length);

#line 594 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 594 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 594 "View.MemoryView"
  return __pyx_r;

#line 594 "View.MemoryView"
}

/* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
/* Python wrapper */

#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/

#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {

#line 605 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 605 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 605 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 605 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
  /* function exit code */

#line 605 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 605 "View.MemoryView"
  return __pyx_r;

#line 605 "View.MemoryView"
}

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 605 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 605 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 605 "View.MemoryView"
  int __pyx_t_1;

#line 605 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */

#line 606 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.ndim >= 1) != 0);

#line 606 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":607
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */

#line 607 "View.MemoryView"
    __pyx_r = (__pyx_v_self->view.shape[0]);

#line 607 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */

#line 606 "View.MemoryView"
  }

  /* "View.MemoryView":609
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 609 "View.MemoryView"
  __pyx_r = 0;

#line 609 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
  /* function exit code */

#line 605 "View.MemoryView"
  __pyx_L0:;

#line 605 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 605 "View.MemoryView"
  return __pyx_r;

#line 605 "View.MemoryView"
}

/* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
/* Python wrapper */

#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/

#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {

#line 611 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 611 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 611 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 611 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
  /* function exit code */

#line 611 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 611 "View.MemoryView"
  return __pyx_r;

#line 611 "View.MemoryView"
}

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 611 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 611 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 611 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 611 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 611 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 611 "View.MemoryView"
  int __pyx_lineno = 0;

#line 611 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 611 "View.MemoryView"
  int __pyx_clineno = 0;

#line 611 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */

#line 612 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 612 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 612 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 612 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":613
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */

#line 613 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 613, __pyx_L1_error)

#line 613 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */

#line 612 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 612 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 612 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 612 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 612 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 612 "View.MemoryView"
  __pyx_t_1 = 0;

#line 612 "View.MemoryView"
  __pyx_t_2 = 0;

#line 612 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 612 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 612 "View.MemoryView"
  __pyx_t_2 = 0;

#line 612 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
  /* function exit code */

#line 611 "View.MemoryView"
  __pyx_L1_error:;

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 611 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 611 "View.MemoryView"
  __pyx_r = NULL;

#line 611 "View.MemoryView"
  __pyx_L0:;

#line 611 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 611 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 611 "View.MemoryView"
  return __pyx_r;

#line 611 "View.MemoryView"
}

/* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
/* Python wrapper */

#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/

#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {

#line 615 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 615 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 615 "View.MemoryView"
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = 
#line 615 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
  /* function exit code */

#line 615 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 615 "View.MemoryView"
  return __pyx_r;

#line 615 "View.MemoryView"
}

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 615 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 615 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 615 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 615 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 615 "View.MemoryView"
  int __pyx_lineno = 0;

#line 615 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 615 "View.MemoryView"
  int __pyx_clineno = 0;

#line 615 "View.MemoryView"
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":616
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 616 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 616 "View.MemoryView"
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 616 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 616 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);

#line 616 "View.MemoryView"
  __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 616 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 616 "View.MemoryView"
  __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
  /* function exit code */

#line 615 "View.MemoryView"
  __pyx_L1_error:;

#line 615 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 615 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 615 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 615 "View.MemoryView"
  __pyx_r = NULL;

#line 615 "View.MemoryView"
  __pyx_L0:;

#line 615 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 615 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 615 "View.MemoryView"
  return __pyx_r;

#line 615 "View.MemoryView"
}

/* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
/* Python wrapper */

#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 619 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 619 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 619 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  __pyx_r = 
#line 619 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
  /* function exit code */

#line 619 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 619 "View.MemoryView"
  return __pyx_r;

#line 619 "View.MemoryView"
}

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice
#line 619 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 619 "View.MemoryView"
;

#line 619 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 619 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 619 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 619 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 619 "View.MemoryView"
  int __pyx_lineno = 0;

#line 619 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 619 "View.MemoryView"
  int __pyx_clineno = 0;

#line 619 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":622
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 */

#line 622 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(1, 622, __pyx_L1_error)

#line 622 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":623
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
 */

#line 623 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 623 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 623, __pyx_L1_error)

#line 623 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 623 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 623 "View.MemoryView"
  __pyx_t_2 = 0;

#line 623 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
  /* function exit code */

#line 619 "View.MemoryView"
  __pyx_L1_error:;

#line 619 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 619 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 619 "View.MemoryView"
  __pyx_r = NULL;

#line 619 "View.MemoryView"
  __pyx_L0:;

#line 619 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 619 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 619 "View.MemoryView"
  return __pyx_r;

#line 619 "View.MemoryView"
}

/* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
/* Python wrapper */

#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 625 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 625 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 625 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  __pyx_r = 
#line 625 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
  /* function exit code */

#line 625 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 625 "View.MemoryView"
  return __pyx_r;

#line 625 "View.MemoryView"
}

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice
#line 625 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 625 "View.MemoryView"
;

#line 625 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 625 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 625 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 625 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 625 "View.MemoryView"
  int __pyx_lineno = 0;

#line 625 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 625 "View.MemoryView"
  int __pyx_clineno = 0;

#line 625 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":628
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 */

#line 628 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(1, 628, __pyx_L1_error)

#line 628 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":629
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
 */

#line 629 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 629 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 629, __pyx_L1_error)

#line 629 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 629 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 629 "View.MemoryView"
  __pyx_t_2 = 0;

#line 629 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
  /* function exit code */

#line 625 "View.MemoryView"
  __pyx_L1_error:;

#line 625 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 625 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 625 "View.MemoryView"
  __pyx_r = NULL;

#line 625 "View.MemoryView"
  __pyx_L0:;

#line 625 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 625 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 625 "View.MemoryView"
  return __pyx_r;

#line 625 "View.MemoryView"
}

/* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
/* Python wrapper */

#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 631 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 631 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 631 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = 
#line 631 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
  /* function exit code */

#line 631 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 631 "View.MemoryView"
  return __pyx_r;

#line 631 "View.MemoryView"
}

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice
#line 631 "View.MemoryView"
;
  int __pyx_v_flags
#line 631 "View.MemoryView"
;

#line 631 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 631 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 631 "View.MemoryView"
  __Pyx_memviewslice __pyx_t_1;

#line 631 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 631 "View.MemoryView"
  int __pyx_lineno = 0;

#line 631 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 631 "View.MemoryView"
  int __pyx_clineno = 0;

#line 631 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":633
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
 */

#line 633 "View.MemoryView"
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":635
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
 */

#line 635 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":636
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
 */

#line 636 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), ((char *)"c"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 636, __pyx_L1_error)

#line 636 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":641
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
 */

#line 641 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 641 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 641, __pyx_L1_error)

#line 641 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 641 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 641 "View.MemoryView"
  __pyx_t_2 = 0;

#line 641 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
  /* function exit code */

#line 631 "View.MemoryView"
  __pyx_L1_error:;

#line 631 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 631 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 631 "View.MemoryView"
  __pyx_r = NULL;

#line 631 "View.MemoryView"
  __pyx_L0:;

#line 631 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 631 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 631 "View.MemoryView"
  return __pyx_r;

#line 631 "View.MemoryView"
}

/* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
/* Python wrapper */

#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 643 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 643 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 643 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  __pyx_r = 
#line 643 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
  /* function exit code */

#line 643 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 643 "View.MemoryView"
  return __pyx_r;

#line 643 "View.MemoryView"
}

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src
#line 643 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_dst
#line 643 "View.MemoryView"
;
  int __pyx_v_flags
#line 643 "View.MemoryView"
;

#line 643 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 643 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 643 "View.MemoryView"
  __Pyx_memviewslice __pyx_t_1;

#line 643 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 643 "View.MemoryView"
  int __pyx_lineno = 0;

#line 643 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 643 "View.MemoryView"
  int __pyx_clineno = 0;

#line 643 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":645
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
 */

#line 645 "View.MemoryView"
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":647
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
 */

#line 647 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":648
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
 */

#line 648 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), ((char *)"fortran"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 648, __pyx_L1_error)

#line 648 "View.MemoryView"
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":653
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 653 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 653 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 653, __pyx_L1_error)

#line 653 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 653 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 653 "View.MemoryView"
  __pyx_t_2 = 0;

#line 653 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
  /* function exit code */

#line 643 "View.MemoryView"
  __pyx_L1_error:;

#line 643 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 643 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 643 "View.MemoryView"
  __pyx_r = NULL;

#line 643 "View.MemoryView"
  __pyx_L0:;

#line 643 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 643 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 643 "View.MemoryView"
  return __pyx_r;

#line 643 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_memoryview___reduce_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_memoryview_2__setstate_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

#line 657 "View.MemoryView"


#line 657 "View.MemoryView"
static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0
#line 657 "View.MemoryView"
;

#line 657 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 657 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 657 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 657 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 657 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 657 "View.MemoryView"
  int __pyx_lineno = 0;

#line 657 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 657 "View.MemoryView"
  int __pyx_clineno = 0;

#line 657 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":658
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
 */

#line 658 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 658 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 658 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_o);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_o);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_1 = 0;

#line 658 "View.MemoryView"
  __pyx_t_2 = 0;

#line 658 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 658 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 658 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":659
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
 */

#line 659 "View.MemoryView"
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":660
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
 */

#line 660 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 660 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 660 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 660 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

#line 657 "View.MemoryView"


#line 657 "View.MemoryView"
  /* function exit code */

#line 657 "View.MemoryView"
  __pyx_L1_error:;

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 657 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 657 "View.MemoryView"
  __pyx_r = 0;

#line 657 "View.MemoryView"
  __pyx_L0:;

#line 657 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 657 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 657 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 657 "View.MemoryView"
  return __pyx_r;

#line 657 "View.MemoryView"
}

/* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

#line 663 "View.MemoryView"


#line 663 "View.MemoryView"
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {

#line 663 "View.MemoryView"
  int __pyx_r;

#line 663 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 663 "View.MemoryView"
  int __pyx_t_1;

#line 663 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_check", 0);

  /* "View.MemoryView":664
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
 */

#line 664 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_memoryview_type); 

#line 664 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 664 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

#line 663 "View.MemoryView"


#line 663 "View.MemoryView"
  /* function exit code */

#line 663 "View.MemoryView"
  __pyx_L0:;

#line 663 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 663 "View.MemoryView"
  return __pyx_r;

#line 663 "View.MemoryView"
}

/* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

#line 666 "View.MemoryView"


#line 666 "View.MemoryView"
static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  PyObject *__pyx_v_tup = NULL
#line 666 "View.MemoryView"
;
  PyObject *__pyx_v_result = NULL
#line 666 "View.MemoryView"
;
  int __pyx_v_have_slices
#line 666 "View.MemoryView"
;
  int __pyx_v_seen_ellipsis
#line 666 "View.MemoryView"
;
  CYTHON_UNUSED PyObject *__pyx_v_idx = NULL
#line 666 "View.MemoryView"
;
  PyObject *__pyx_v_item = NULL
#line 666 "View.MemoryView"
;
  Py_ssize_t __pyx_v_nslices
#line 666 "View.MemoryView"
;

#line 666 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 666 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 666 "View.MemoryView"
  int __pyx_t_1;

#line 666 "View.MemoryView"
  int __pyx_t_2;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 666 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 666 "View.MemoryView"
  PyObject *(*__pyx_t_6)(PyObject *);

#line 666 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 666 "View.MemoryView"
  Py_ssize_t __pyx_t_8;

#line 666 "View.MemoryView"
  int __pyx_t_9;

#line 666 "View.MemoryView"
  int __pyx_t_10;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_11 = NULL;

#line 666 "View.MemoryView"
  int __pyx_lineno = 0;

#line 666 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 666 "View.MemoryView"
  int __pyx_clineno = 0;

#line 666 "View.MemoryView"
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */

#line 671 "View.MemoryView"
  __pyx_t_1 = PyTuple_Check(__pyx_v_index); 

#line 671 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 671 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":672
 *     """
 *     if not isinstance(index, tuple):
 *         tup = (index,)             # <<<<<<<<<<<<<<
 *     else:
 *         tup = index
 */

#line 672 "View.MemoryView"
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 672, __pyx_L1_error)

#line 672 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 672 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_index);

#line 672 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_index);

#line 672 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index);

#line 672 "View.MemoryView"
    __pyx_v_tup = __pyx_t_3;

#line 672 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */

#line 671 "View.MemoryView"
    goto __pyx_L3;

#line 671 "View.MemoryView"
  }

  /* "View.MemoryView":674
 *         tup = (index,)
 *     else:
 *         tup = index             # <<<<<<<<<<<<<<
 * 
 *     result = []
 */

#line 674 "View.MemoryView"
  /*else*/ {

#line 674 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_index);

#line 674 "View.MemoryView"
    __pyx_v_tup = __pyx_v_index;

#line 674 "View.MemoryView"
  }

#line 674 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":676
 *         tup = index
 * 
 *     result = []             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
 */

#line 676 "View.MemoryView"
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 676, __pyx_L1_error)

#line 676 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 676 "View.MemoryView"
  __pyx_v_result = ((PyObject*)__pyx_t_3);

#line 676 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":677
 * 
 *     result = []
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 */

#line 677 "View.MemoryView"
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":678
 *     result = []
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 */

#line 678 "View.MemoryView"
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */

#line 679 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_0);

#line 679 "View.MemoryView"
  __pyx_t_3 = __pyx_int_0;

#line 679 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_tup)) || PyTuple_CheckExact(__pyx_v_tup)) {

#line 679 "View.MemoryView"
    __pyx_t_4 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;

#line 679 "View.MemoryView"
    __pyx_t_6 = NULL;

#line 679 "View.MemoryView"
  } else {
    __pyx_t_5 = -1; 
#line 679 "View.MemoryView"
__pyx_t_4 = PyObject_GetIter(__pyx_v_tup); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 679 "View.MemoryView"
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
  }

#line 679 "View.MemoryView"
  for (;;) {

#line 679 "View.MemoryView"
    if (likely(!__pyx_t_6)) {

#line 679 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_4))) {

#line 679 "View.MemoryView"
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;

#line 679 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 679 "View.MemoryView"
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        #else

#line 679 "View.MemoryView"
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
        #endif

#line 679 "View.MemoryView"
      } else {

#line 679 "View.MemoryView"
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;

#line 679 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 679 "View.MemoryView"
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        #else

#line 679 "View.MemoryView"
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
        #endif

#line 679 "View.MemoryView"
      }
    } else 
#line 679 "View.MemoryView"
{

#line 679 "View.MemoryView"
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);

#line 679 "View.MemoryView"
      if (unlikely(!__pyx_t_7)) {

#line 679 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 679 "View.MemoryView"
        if (exc_type) {

#line 679 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 679 "View.MemoryView"
          else __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        }

#line 679 "View.MemoryView"
        break;

#line 679 "View.MemoryView"
      }

#line 679 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
    }

#line 679 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_7);

#line 679 "View.MemoryView"
    __pyx_t_7 = 0;

#line 679 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 679 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_3);

#line 679 "View.MemoryView"
    __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3);

#line 679 "View.MemoryView"
    __pyx_t_3 = __pyx_t_7;

#line 679 "View.MemoryView"
    __pyx_t_7 = 0;

    /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */

#line 680 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);

#line 680 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_2 != 0);

#line 680 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */

#line 681 "View.MemoryView"
      __pyx_t_1 = ((!(__pyx_v_seen_ellipsis != 0)) != 0);

#line 681 "View.MemoryView"
      if (__pyx_t_1) {

        /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */

#line 682 "View.MemoryView"
        __pyx_t_8 = PyObject_Length(__pyx_v_tup); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(1, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __pyx_t_7 = PyList_New(1 * ((((__pyx_v_ndim - __pyx_t_8) + 1)<0) ? 0:((__pyx_v_ndim - __pyx_t_8) + 1))); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 682 "View.MemoryView"
        { Py_ssize_t __pyx_temp;

#line 682 "View.MemoryView"
          for (__pyx_temp=0; __pyx_temp < ((__pyx_v_ndim - __pyx_t_8) + 1); __pyx_temp++) {

#line 682 "View.MemoryView"
            __Pyx_INCREF(__pyx_slice__26);

#line 682 "View.MemoryView"
            __Pyx_GIVEREF(__pyx_slice__26);

#line 682 "View.MemoryView"
            PyList_SET_ITEM(__pyx_t_7, __pyx_temp, __pyx_slice__26);

#line 682 "View.MemoryView"
          }

#line 682 "View.MemoryView"
        }

#line 682 "View.MemoryView"
        __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "View.MemoryView":683
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(slice(None))
 */

#line 683 "View.MemoryView"
        __pyx_v_seen_ellipsis = 1;

        /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */

#line 681 "View.MemoryView"
        goto __pyx_L7;

#line 681 "View.MemoryView"
      }

      /* "View.MemoryView":685
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */

#line 685 "View.MemoryView"
      /*else*/ {

#line 685 "View.MemoryView"
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_slice__26); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 685, __pyx_L1_error)

#line 685 "View.MemoryView"
      }

#line 685 "View.MemoryView"
      __pyx_L7:;

      /* "View.MemoryView":686
 *             else:
 *                 result.append(slice(None))
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 */

#line 686 "View.MemoryView"
      __pyx_v_have_slices = 1;

      /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */

#line 680 "View.MemoryView"
      goto __pyx_L6;

#line 680 "View.MemoryView"
    }

    /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */

#line 688 "View.MemoryView"
    /*else*/ {

#line 688 "View.MemoryView"
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 

#line 688 "View.MemoryView"
      __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);

#line 688 "View.MemoryView"
      if (__pyx_t_10) {

#line 688 "View.MemoryView"
      } else {

#line 688 "View.MemoryView"
        __pyx_t_1 = __pyx_t_10;

#line 688 "View.MemoryView"
        goto __pyx_L9_bool_binop_done;

#line 688 "View.MemoryView"
      }

#line 688 "View.MemoryView"
      __pyx_t_10 = ((!(PyIndex_Check(__pyx_v_item) != 0)) != 0);

#line 688 "View.MemoryView"
      __pyx_t_1 = __pyx_t_10;

#line 688 "View.MemoryView"
      __pyx_L9_bool_binop_done:;

#line 688 "View.MemoryView"
      if (unlikely(__pyx_t_1)) {

        /* "View.MemoryView":689
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 *                 raise TypeError("Cannot index with type '%s'" % type(item))             # <<<<<<<<<<<<<<
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 */

#line 689 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Cannot_index_with_type_s, ((PyObject *)Py_TYPE(__pyx_v_item))); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 689, __pyx_L1_error)

#line 689 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 689 "View.MemoryView"
        __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 689, __pyx_L1_error)

#line 689 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_11);

#line 689 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 689 "View.MemoryView"
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);

#line 689 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 689 "View.MemoryView"
        __PYX_ERR(1, 689, __pyx_L1_error)

        /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */

#line 688 "View.MemoryView"
      }

      /* "View.MemoryView":691
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 *             have_slices = have_slices or isinstance(item, slice)             # <<<<<<<<<<<<<<
 *             result.append(item)
 * 
 */

#line 691 "View.MemoryView"
      __pyx_t_10 = (__pyx_v_have_slices != 0);

#line 691 "View.MemoryView"
      if (!__pyx_t_10) {

#line 691 "View.MemoryView"
      } else {

#line 691 "View.MemoryView"
        __pyx_t_1 = __pyx_t_10;

#line 691 "View.MemoryView"
        goto __pyx_L11_bool_binop_done;

#line 691 "View.MemoryView"
      }

#line 691 "View.MemoryView"
      __pyx_t_10 = PySlice_Check(__pyx_v_item); 

#line 691 "View.MemoryView"
      __pyx_t_2 = (__pyx_t_10 != 0);

#line 691 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 691 "View.MemoryView"
      __pyx_L11_bool_binop_done:;

#line 691 "View.MemoryView"
      __pyx_v_have_slices = __pyx_t_1;

      /* "View.MemoryView":692
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 *             result.append(item)             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - len(result)
 */

#line 692 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_item); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 692, __pyx_L1_error)

#line 692 "View.MemoryView"
    }

#line 692 "View.MemoryView"
    __pyx_L6:;

    /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */

#line 679 "View.MemoryView"
  }

#line 679 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 679 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":694
 *             result.append(item)
 * 
 *     nslices = ndim - len(result)             # <<<<<<<<<<<<<<
 *     if nslices:
 *         result.extend([slice(None)] * nslices)
 */

#line 694 "View.MemoryView"
  __pyx_t_5 = PyList_GET_SIZE(__pyx_v_result); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 694, __pyx_L1_error)

#line 694 "View.MemoryView"
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_t_5);

  /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */

#line 695 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_nslices != 0);

#line 695 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":696
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */

#line 696 "View.MemoryView"
    __pyx_t_3 = PyList_New(1 * ((__pyx_v_nslices<0) ? 0:__pyx_v_nslices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 696, __pyx_L1_error)

#line 696 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 696 "View.MemoryView"
    { Py_ssize_t __pyx_temp;

#line 696 "View.MemoryView"
      for (__pyx_temp=0; __pyx_temp < __pyx_v_nslices; __pyx_temp++) {

#line 696 "View.MemoryView"
        __Pyx_INCREF(__pyx_slice__26);

#line 696 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_slice__26);

#line 696 "View.MemoryView"
        PyList_SET_ITEM(__pyx_t_3, __pyx_temp, __pyx_slice__26);

#line 696 "View.MemoryView"
      }

#line 696 "View.MemoryView"
    }

#line 696 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 696, __pyx_L1_error)

#line 696 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */

#line 695 "View.MemoryView"
  }

  /* "View.MemoryView":698
 *         result.extend([slice(None)] * nslices)
 * 
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 */

#line 698 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 698 "View.MemoryView"
  if (!__pyx_v_have_slices) {

#line 698 "View.MemoryView"
  } else {

#line 698 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
    __pyx_t_3 = __pyx_t_4;

#line 698 "View.MemoryView"
    __pyx_t_4 = 0;

#line 698 "View.MemoryView"
    goto __pyx_L14_bool_binop_done;

#line 698 "View.MemoryView"
  }

#line 698 "View.MemoryView"
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_3 = __pyx_t_4;

#line 698 "View.MemoryView"
  __pyx_t_4 = 0;

#line 698 "View.MemoryView"
  __pyx_L14_bool_binop_done:;

#line 698 "View.MemoryView"
  __pyx_t_4 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_11);

#line 698 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 698 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_3);

#line 698 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_4);

#line 698 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_3 = 0;

#line 698 "View.MemoryView"
  __pyx_t_4 = 0;

#line 698 "View.MemoryView"
  __pyx_r = ((PyObject*)__pyx_t_11);

#line 698 "View.MemoryView"
  __pyx_t_11 = 0;

#line 698 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

#line 666 "View.MemoryView"


#line 666 "View.MemoryView"
  /* function exit code */

#line 666 "View.MemoryView"
  __pyx_L1_error:;

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_11);

#line 666 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 666 "View.MemoryView"
  __pyx_r = 0;

#line 666 "View.MemoryView"
  __pyx_L0:;

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_tup);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_idx);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_item);

#line 666 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 666 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 666 "View.MemoryView"
  return __pyx_r;

#line 666 "View.MemoryView"
}

/* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 700 "View.MemoryView"


#line 700 "View.MemoryView"
static PyObject *assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_suboffset
#line 700 "View.MemoryView"
;

#line 700 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 700 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_1;

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 700 "View.MemoryView"
  int __pyx_t_4;

#line 700 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 700 "View.MemoryView"
  int __pyx_lineno = 0;

#line 700 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 700 "View.MemoryView"
  int __pyx_clineno = 0;

#line 700 "View.MemoryView"
  __Pyx_RefNannySetupContext("assert_direct_dimensions", 0);

  /* "View.MemoryView":701
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")
 */

#line 701 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_suboffsets + __pyx_v_ndim);

#line 701 "View.MemoryView"
  for (__pyx_t_3 = __pyx_v_suboffsets; __pyx_t_3 < __pyx_t_2; __pyx_t_3++) {

#line 701 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 701 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_1[0]);

    /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */

#line 702 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_suboffset >= 0) != 0);

#line 702 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 703 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 703, __pyx_L1_error)

#line 703 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 703 "View.MemoryView"
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);

#line 703 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 703 "View.MemoryView"
      __PYX_ERR(1, 703, __pyx_L1_error)

      /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */

#line 702 "View.MemoryView"
    }

#line 702 "View.MemoryView"
  }

  /* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 700 "View.MemoryView"


#line 700 "View.MemoryView"
  /* function exit code */

#line 700 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 700 "View.MemoryView"
  goto __pyx_L0;

#line 700 "View.MemoryView"
  __pyx_L1_error:;

#line 700 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 700 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 700 "View.MemoryView"
  __pyx_r = 0;

#line 700 "View.MemoryView"
  __pyx_L0:;

#line 700 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 700 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 700 "View.MemoryView"
  return __pyx_r;

#line 700 "View.MemoryView"
}

/* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

#line 710 "View.MemoryView"


#line 710 "View.MemoryView"
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim
#line 710 "View.MemoryView"
;
  int __pyx_v_suboffset_dim
#line 710 "View.MemoryView"
;
  int __pyx_v_dim
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_src
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_dst
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_p_src
#line 710 "View.MemoryView"
;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_p_dst
#line 710 "View.MemoryView"
;
  int *__pyx_v_p_suboffset_dim
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_start
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stop
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_step
#line 710 "View.MemoryView"
;
  int __pyx_v_have_start
#line 710 "View.MemoryView"
;
  int __pyx_v_have_stop
#line 710 "View.MemoryView"
;
  int __pyx_v_have_step
#line 710 "View.MemoryView"
;
  PyObject *__pyx_v_index = NULL
#line 710 "View.MemoryView"
;

#line 710 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_r = NULL;

#line 710 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 710 "View.MemoryView"
  int __pyx_t_1;

#line 710 "View.MemoryView"
  int __pyx_t_2;

#line 710 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 710 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_t_4;

#line 710 "View.MemoryView"
  char *__pyx_t_5;

#line 710 "View.MemoryView"
  int __pyx_t_6;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_7;

#line 710 "View.MemoryView"
  PyObject *(*__pyx_t_8)(PyObject *);

#line 710 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_10;

#line 710 "View.MemoryView"
  int __pyx_t_11;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_12;

#line 710 "View.MemoryView"
  int __pyx_lineno = 0;

#line 710 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 710 "View.MemoryView"
  int __pyx_clineno = 0;

#line 710 "View.MemoryView"
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":711
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
 */

#line 711 "View.MemoryView"
  __pyx_v_new_ndim = 0;

#line 711 "View.MemoryView"
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":718
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
 */

#line 718 "View.MemoryView"
  (void)(memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst))));

  /* "View.MemoryView":722
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */

#line 722 "View.MemoryView"
  #ifndef CYTHON_WITHOUT_ASSERTIONS

#line 722 "View.MemoryView"
  if (unlikely(!Py_OptimizeFlag)) {

#line 722 "View.MemoryView"
    if (unlikely(!((__pyx_v_memview->view.ndim > 0) != 0))) {

#line 722 "View.MemoryView"
      PyErr_SetNone(PyExc_AssertionError);

#line 722 "View.MemoryView"
      __PYX_ERR(1, 722, __pyx_L1_error)

#line 722 "View.MemoryView"
    }

#line 722 "View.MemoryView"
  }

#line 722 "View.MemoryView"
  #endif

  /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */

#line 724 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 724 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 724 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":725
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 */

#line 725 "View.MemoryView"
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(1, 725, __pyx_L1_error)

#line 725 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);

#line 725 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 725 "View.MemoryView"
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);

#line 725 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":726
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
 */

#line 726 "View.MemoryView"
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);

    /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */

#line 724 "View.MemoryView"
    goto __pyx_L3;

#line 724 "View.MemoryView"
  }

  /* "View.MemoryView":728
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
 */

#line 728 "View.MemoryView"
  /*else*/ {

#line 728 "View.MemoryView"
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":729
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 729 "View.MemoryView"
    __pyx_v_p_src = (&__pyx_v_src);

#line 729 "View.MemoryView"
  }

#line 729 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":735
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
 */

#line 735 "View.MemoryView"
  __pyx_t_4 = __pyx_v_p_src->memview;

#line 735 "View.MemoryView"
  __pyx_v_dst.memview = __pyx_t_4;

  /* "View.MemoryView":736
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 736 "View.MemoryView"
  __pyx_t_5 = __pyx_v_p_src->data;

#line 736 "View.MemoryView"
  __pyx_v_dst.data = __pyx_t_5;

  /* "View.MemoryView":741
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step
 */

#line 741 "View.MemoryView"
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":742
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step
 *     cdef bint have_start, have_stop, have_step
 */

#line 742 "View.MemoryView"
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */

#line 746 "View.MemoryView"
  __pyx_t_6 = 0;

#line 746 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {

#line 746 "View.MemoryView"
    __pyx_t_3 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;

#line 746 "View.MemoryView"
    __pyx_t_8 = NULL;

#line 746 "View.MemoryView"
  } else {
    __pyx_t_7 = -1; 
#line 746 "View.MemoryView"
__pyx_t_3 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 746 "View.MemoryView"
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
  }

#line 746 "View.MemoryView"
  for (;;) {

#line 746 "View.MemoryView"
    if (likely(!__pyx_t_8)) {

#line 746 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_3))) {

#line 746 "View.MemoryView"
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;

#line 746 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 746 "View.MemoryView"
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        #else

#line 746 "View.MemoryView"
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
        #endif

#line 746 "View.MemoryView"
      } else {

#line 746 "View.MemoryView"
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;

#line 746 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 746 "View.MemoryView"
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        #else

#line 746 "View.MemoryView"
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
        #endif

#line 746 "View.MemoryView"
      }
    } else 
#line 746 "View.MemoryView"
{

#line 746 "View.MemoryView"
      __pyx_t_9 = __pyx_t_8(__pyx_t_3);

#line 746 "View.MemoryView"
      if (unlikely(!__pyx_t_9)) {

#line 746 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 746 "View.MemoryView"
        if (exc_type) {

#line 746 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 746 "View.MemoryView"
          else __PYX_ERR(1, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        }

#line 746 "View.MemoryView"
        break;

#line 746 "View.MemoryView"
      }

#line 746 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
    }

#line 746 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_9);

#line 746 "View.MemoryView"
    __pyx_t_9 = 0;

#line 746 "View.MemoryView"
    __pyx_v_dim = __pyx_t_6;

#line 746 "View.MemoryView"
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */

#line 747 "View.MemoryView"
    __pyx_t_2 = (PyIndex_Check(__pyx_v_index) != 0);

#line 747 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":751
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 *                 index, 0, 0, # start, stop, step             # <<<<<<<<<<<<<<
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 */

#line 751 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 751, __pyx_L1_error)

      /* "View.MemoryView":748
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */

#line 748 "View.MemoryView"
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_t_10, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(1, 748, __pyx_L1_error)

      /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */

#line 747 "View.MemoryView"
      goto __pyx_L6;

#line 747 "View.MemoryView"
    }

    /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */

#line 754 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_index == Py_None);

#line 754 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_2 != 0);

#line 754 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":755
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 */

#line 755 "View.MemoryView"
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":756
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
 */

#line 756 "View.MemoryView"
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":757
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
 */

#line 757 "View.MemoryView"
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1L;

      /* "View.MemoryView":758
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
 */

#line 758 "View.MemoryView"
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

      /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */

#line 754 "View.MemoryView"
      goto __pyx_L6;

#line 754 "View.MemoryView"
    }

    /* "View.MemoryView":760
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
 */

#line 760 "View.MemoryView"
    /*else*/ {

#line 760 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 760 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
      if (!__pyx_t_1) {

#line 760 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 760 "View.MemoryView"
      } else {

#line 760 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 760 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 760 "View.MemoryView"
        goto __pyx_L7_bool_binop_done;

#line 760 "View.MemoryView"
      }

#line 760 "View.MemoryView"
      __pyx_t_10 = 0;

#line 760 "View.MemoryView"
      __pyx_L7_bool_binop_done:;

#line 760 "View.MemoryView"
      __pyx_v_start = __pyx_t_10;

      /* "View.MemoryView":761
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
 */

#line 761 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 761 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
      if (!__pyx_t_1) {

#line 761 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 761 "View.MemoryView"
      } else {

#line 761 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 761 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 761 "View.MemoryView"
        goto __pyx_L9_bool_binop_done;

#line 761 "View.MemoryView"
      }

#line 761 "View.MemoryView"
      __pyx_t_10 = 0;

#line 761 "View.MemoryView"
      __pyx_L9_bool_binop_done:;

#line 761 "View.MemoryView"
      __pyx_v_stop = __pyx_t_10;

      /* "View.MemoryView":762
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
 */

#line 762 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 762 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
      if (!__pyx_t_1) {

#line 762 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 762 "View.MemoryView"
      } else {

#line 762 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 762 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 762 "View.MemoryView"
        goto __pyx_L11_bool_binop_done;

#line 762 "View.MemoryView"
      }

#line 762 "View.MemoryView"
      __pyx_t_10 = 0;

#line 762 "View.MemoryView"
      __pyx_L11_bool_binop_done:;

#line 762 "View.MemoryView"
      __pyx_v_step = __pyx_t_10;

      /* "View.MemoryView":764
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
 */

#line 764 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 764, __pyx_L1_error)

#line 764 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 764 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 764 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 764 "View.MemoryView"
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":765
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
 */

#line 765 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 765, __pyx_L1_error)

#line 765 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 765 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 765 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 765 "View.MemoryView"
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":766
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
 */

#line 766 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 766, __pyx_L1_error)

#line 766 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 766 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 766 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 766 "View.MemoryView"
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":768
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */

#line 768 "View.MemoryView"
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(1, 768, __pyx_L1_error)

      /* "View.MemoryView":774
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */

#line 774 "View.MemoryView"
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

#line 774 "View.MemoryView"
    }

#line 774 "View.MemoryView"
    __pyx_L6:;

    /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */

#line 746 "View.MemoryView"
  }

#line 746 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */

#line 776 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 776 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 776 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */

#line 777 "View.MemoryView"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":778
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
 */

#line 778 "View.MemoryView"
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(1, 778, __pyx_L1_error) }

    /* "View.MemoryView":779
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
 */

#line 779 "View.MemoryView"
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(1, 779, __pyx_L1_error) }

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */

#line 777 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 777, __pyx_L1_error)

#line 777 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 777 "View.MemoryView"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(1, 777, __pyx_L1_error)

#line 777 "View.MemoryView"
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);

#line 777 "View.MemoryView"
    __pyx_t_3 = 0;

#line 777 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */

#line 776 "View.MemoryView"
  }

  /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */

#line 782 "View.MemoryView"
  /*else*/ {

#line 782 "View.MemoryView"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":783
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 783 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 782, __pyx_L1_error)

#line 783 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */

#line 782 "View.MemoryView"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(1, 782, __pyx_L1_error)

#line 782 "View.MemoryView"
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);

#line 782 "View.MemoryView"
    __pyx_t_3 = 0;

#line 782 "View.MemoryView"
    goto __pyx_L0;

#line 782 "View.MemoryView"
  }

  /* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

#line 710 "View.MemoryView"


#line 710 "View.MemoryView"
  /* function exit code */

#line 710 "View.MemoryView"
  __pyx_L1_error:;

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_9);

#line 710 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 710 "View.MemoryView"
  __pyx_r = 0;

#line 710 "View.MemoryView"
  __pyx_L0:;

#line 710 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_index);

#line 710 "View.MemoryView"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 710 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 710 "View.MemoryView"
  return __pyx_r;

#line 710 "View.MemoryView"
}

/* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

#line 807 "View.MemoryView"


#line 807 "View.MemoryView"
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *__pyx_v_dst, Py_ssize_t __pyx_v_shape, Py_ssize_t __pyx_v_stride, Py_ssize_t __pyx_v_suboffset, int __pyx_v_dim, int __pyx_v_new_ndim, int *__pyx_v_suboffset_dim, Py_ssize_t __pyx_v_start, Py_ssize_t __pyx_v_stop, Py_ssize_t __pyx_v_step, int __pyx_v_have_start, int __pyx_v_have_stop, int __pyx_v_have_step, int __pyx_v_is_slice) {
  Py_ssize_t __pyx_v_new_shape
#line 807 "View.MemoryView"
;
  int __pyx_v_negative_step
#line 807 "View.MemoryView"
;

#line 807 "View.MemoryView"
  int __pyx_r;

#line 807 "View.MemoryView"
  int __pyx_t_1;

#line 807 "View.MemoryView"
  int __pyx_t_2;

#line 807 "View.MemoryView"
  int __pyx_t_3;

#line 807 "View.MemoryView"
  int __pyx_lineno = 0;

#line 807 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 807 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */

#line 827 "View.MemoryView"
  __pyx_t_1 = ((!(__pyx_v_is_slice != 0)) != 0);

#line 827 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */

#line 829 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_start < 0) != 0);

#line 829 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":830
 * 
 *         if start < 0:
 *             start += shape             # <<<<<<<<<<<<<<
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 */

#line 830 "View.MemoryView"
      __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

      /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */

#line 829 "View.MemoryView"
    }

    /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */

#line 831 "View.MemoryView"
    __pyx_t_1 = (0 <= __pyx_v_start);

#line 831 "View.MemoryView"
    if (__pyx_t_1) {

#line 831 "View.MemoryView"
      __pyx_t_1 = (__pyx_v_start < __pyx_v_shape);

#line 831 "View.MemoryView"
    }

#line 831 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 831 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":832
 *             start += shape
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)             # <<<<<<<<<<<<<<
 *     else:
 * 
 */

#line 832 "View.MemoryView"
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"Index out of bounds (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 832, __pyx_L1_error)

      /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */

#line 831 "View.MemoryView"
    }

    /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */

#line 827 "View.MemoryView"
    goto __pyx_L3;

#line 827 "View.MemoryView"
  }

  /* "View.MemoryView":835
 *     else:
 * 
 *         negative_step = have_step != 0 and step < 0             # <<<<<<<<<<<<<<
 * 
 *         if have_step and step == 0:
 */

#line 835 "View.MemoryView"
  /*else*/ {

#line 835 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_have_step != 0) != 0);

#line 835 "View.MemoryView"
    if (__pyx_t_1) {

#line 835 "View.MemoryView"
    } else {

#line 835 "View.MemoryView"
      __pyx_t_2 = __pyx_t_1;

#line 835 "View.MemoryView"
      goto __pyx_L6_bool_binop_done;

#line 835 "View.MemoryView"
    }

#line 835 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_step < 0) != 0);

#line 835 "View.MemoryView"
    __pyx_t_2 = __pyx_t_1;

#line 835 "View.MemoryView"
    __pyx_L6_bool_binop_done:;

#line 835 "View.MemoryView"
    __pyx_v_negative_step = __pyx_t_2;

    /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */

#line 837 "View.MemoryView"
    __pyx_t_1 = (__pyx_v_have_step != 0);

#line 837 "View.MemoryView"
    if (__pyx_t_1) {

#line 837 "View.MemoryView"
    } else {

#line 837 "View.MemoryView"
      __pyx_t_2 = __pyx_t_1;

#line 837 "View.MemoryView"
      goto __pyx_L9_bool_binop_done;

#line 837 "View.MemoryView"
    }

#line 837 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_step == 0) != 0);

#line 837 "View.MemoryView"
    __pyx_t_2 = __pyx_t_1;

#line 837 "View.MemoryView"
    __pyx_L9_bool_binop_done:;

#line 837 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":838
 * 
 *         if have_step and step == 0:
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 838 "View.MemoryView"
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Step may not be zero (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 838, __pyx_L1_error)

      /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */

#line 837 "View.MemoryView"
    }

    /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */

#line 841 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_have_start != 0);

#line 841 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */

#line 842 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_start < 0) != 0);

#line 842 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":843
 *         if have_start:
 *             if start < 0:
 *                 start += shape             # <<<<<<<<<<<<<<
 *                 if start < 0:
 *                     start = 0
 */

#line 843 "View.MemoryView"
        __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

        /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */

#line 844 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_start < 0) != 0);

#line 844 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":845
 *                 start += shape
 *                 if start < 0:
 *                     start = 0             # <<<<<<<<<<<<<<
 *             elif start >= shape:
 *                 if negative_step:
 */

#line 845 "View.MemoryView"
          __pyx_v_start = 0;

          /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */

#line 844 "View.MemoryView"
        }

        /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */

#line 842 "View.MemoryView"
        goto __pyx_L12;

#line 842 "View.MemoryView"
      }

      /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */

#line 846 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_start >= __pyx_v_shape) != 0);

#line 846 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */

#line 847 "View.MemoryView"
        __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 847 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":848
 *             elif start >= shape:
 *                 if negative_step:
 *                     start = shape - 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = shape
 */

#line 848 "View.MemoryView"
          __pyx_v_start = (__pyx_v_shape - 1);

          /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */

#line 847 "View.MemoryView"
          goto __pyx_L14;

#line 847 "View.MemoryView"
        }

        /* "View.MemoryView":850
 *                     start = shape - 1
 *                 else:
 *                     start = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */

#line 850 "View.MemoryView"
        /*else*/ {

#line 850 "View.MemoryView"
          __pyx_v_start = __pyx_v_shape;

#line 850 "View.MemoryView"
        }

#line 850 "View.MemoryView"
        __pyx_L14:;

        /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */

#line 846 "View.MemoryView"
      }

#line 846 "View.MemoryView"
      __pyx_L12:;

      /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */

#line 841 "View.MemoryView"
      goto __pyx_L11;

#line 841 "View.MemoryView"
    }

    /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */

#line 852 "View.MemoryView"
    /*else*/ {

#line 852 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 852 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":853
 *         else:
 *             if negative_step:
 *                 start = shape - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 start = 0
 */

#line 853 "View.MemoryView"
        __pyx_v_start = (__pyx_v_shape - 1);

        /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */

#line 852 "View.MemoryView"
        goto __pyx_L15;

#line 852 "View.MemoryView"
      }

      /* "View.MemoryView":855
 *                 start = shape - 1
 *             else:
 *                 start = 0             # <<<<<<<<<<<<<<
 * 
 *         if have_stop:
 */

#line 855 "View.MemoryView"
      /*else*/ {

#line 855 "View.MemoryView"
        __pyx_v_start = 0;

#line 855 "View.MemoryView"
      }

#line 855 "View.MemoryView"
      __pyx_L15:;

#line 855 "View.MemoryView"
    }

#line 855 "View.MemoryView"
    __pyx_L11:;

    /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */

#line 857 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_have_stop != 0);

#line 857 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */

#line 858 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_stop < 0) != 0);

#line 858 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":859
 *         if have_stop:
 *             if stop < 0:
 *                 stop += shape             # <<<<<<<<<<<<<<
 *                 if stop < 0:
 *                     stop = 0
 */

#line 859 "View.MemoryView"
        __pyx_v_stop = (__pyx_v_stop + __pyx_v_shape);

        /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */

#line 860 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_stop < 0) != 0);

#line 860 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":861
 *                 stop += shape
 *                 if stop < 0:
 *                     stop = 0             # <<<<<<<<<<<<<<
 *             elif stop > shape:
 *                 stop = shape
 */

#line 861 "View.MemoryView"
          __pyx_v_stop = 0;

          /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */

#line 860 "View.MemoryView"
        }

        /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */

#line 858 "View.MemoryView"
        goto __pyx_L17;

#line 858 "View.MemoryView"
      }

      /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */

#line 862 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_stop > __pyx_v_shape) != 0);

#line 862 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":863
 *                     stop = 0
 *             elif stop > shape:
 *                 stop = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */

#line 863 "View.MemoryView"
        __pyx_v_stop = __pyx_v_shape;

        /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */

#line 862 "View.MemoryView"
      }

#line 862 "View.MemoryView"
      __pyx_L17:;

      /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */

#line 857 "View.MemoryView"
      goto __pyx_L16;

#line 857 "View.MemoryView"
    }

    /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */

#line 865 "View.MemoryView"
    /*else*/ {

#line 865 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 865 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":866
 *         else:
 *             if negative_step:
 *                 stop = -1             # <<<<<<<<<<<<<<
 *             else:
 *                 stop = shape
 */

#line 866 "View.MemoryView"
        __pyx_v_stop = -1L;

        /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */

#line 865 "View.MemoryView"
        goto __pyx_L19;

#line 865 "View.MemoryView"
      }

      /* "View.MemoryView":868
 *                 stop = -1
 *             else:
 *                 stop = shape             # <<<<<<<<<<<<<<
 * 
 *         if not have_step:
 */

#line 868 "View.MemoryView"
      /*else*/ {

#line 868 "View.MemoryView"
        __pyx_v_stop = __pyx_v_shape;

#line 868 "View.MemoryView"
      }

#line 868 "View.MemoryView"
      __pyx_L19:;

#line 868 "View.MemoryView"
    }

#line 868 "View.MemoryView"
    __pyx_L16:;

    /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */

#line 870 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_v_have_step != 0)) != 0);

#line 870 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":871
 * 
 *         if not have_step:
 *             step = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 871 "View.MemoryView"
      __pyx_v_step = 1;

      /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */

#line 870 "View.MemoryView"
    }

    /* "View.MemoryView":875
 * 
 *         with cython.cdivision(True):
 *             new_shape = (stop - start) // step             # <<<<<<<<<<<<<<
 * 
 *             if (stop - start) - step * new_shape:
 */

#line 875 "View.MemoryView"
    __pyx_v_new_shape = ((__pyx_v_stop - __pyx_v_start) / __pyx_v_step);

    /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */

#line 877 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_stop - __pyx_v_start) - (__pyx_v_step * __pyx_v_new_shape)) != 0);

#line 877 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":878
 * 
 *             if (stop - start) - step * new_shape:
 *                 new_shape += 1             # <<<<<<<<<<<<<<
 * 
 *         if new_shape < 0:
 */

#line 878 "View.MemoryView"
      __pyx_v_new_shape = (__pyx_v_new_shape + 1);

      /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */

#line 877 "View.MemoryView"
    }

    /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */

#line 880 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_new_shape < 0) != 0);

#line 880 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":881
 * 
 *         if new_shape < 0:
 *             new_shape = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 881 "View.MemoryView"
      __pyx_v_new_shape = 0;

      /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */

#line 880 "View.MemoryView"
    }

    /* "View.MemoryView":884
 * 
 * 
 *         dst.strides[new_ndim] = stride * step             # <<<<<<<<<<<<<<
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset
 */

#line 884 "View.MemoryView"
    (__pyx_v_dst->strides[__pyx_v_new_ndim]) = (__pyx_v_stride * __pyx_v_step);

    /* "View.MemoryView":885
 * 
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape             # <<<<<<<<<<<<<<
 *         dst.suboffsets[new_ndim] = suboffset
 * 
 */

#line 885 "View.MemoryView"
    (__pyx_v_dst->shape[__pyx_v_new_ndim]) = __pyx_v_new_shape;

    /* "View.MemoryView":886
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 886 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_v_new_ndim]) = __pyx_v_suboffset;

#line 886 "View.MemoryView"
  }

#line 886 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */

#line 889 "View.MemoryView"
  __pyx_t_2 = (((__pyx_v_suboffset_dim[0]) < 0) != 0);

#line 889 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":890
 * 
 *     if suboffset_dim[0] < 0:
 *         dst.data += start * stride             # <<<<<<<<<<<<<<
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 */

#line 890 "View.MemoryView"
    __pyx_v_dst->data = (__pyx_v_dst->data + (__pyx_v_start * __pyx_v_stride));

    /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */

#line 889 "View.MemoryView"
    goto __pyx_L23;

#line 889 "View.MemoryView"
  }

  /* "View.MemoryView":892
 *         dst.data += start * stride
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride             # <<<<<<<<<<<<<<
 * 
 *     if suboffset >= 0:
 */

#line 892 "View.MemoryView"
  /*else*/ {

#line 892 "View.MemoryView"
    __pyx_t_3 = (__pyx_v_suboffset_dim[0]);

#line 892 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_t_3]) = ((__pyx_v_dst->suboffsets[__pyx_t_3]) + (__pyx_v_start * __pyx_v_stride));

#line 892 "View.MemoryView"
  }

#line 892 "View.MemoryView"
  __pyx_L23:;

  /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */

#line 894 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);

#line 894 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */

#line 895 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_v_is_slice != 0)) != 0);

#line 895 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */

#line 896 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_new_ndim == 0) != 0);

#line 896 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":897
 *         if not is_slice:
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 */

#line 897 "View.MemoryView"
        __pyx_v_dst->data = ((((char **)__pyx_v_dst->data)[0]) + __pyx_v_suboffset);

        /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */

#line 896 "View.MemoryView"
        goto __pyx_L26;

#line 896 "View.MemoryView"
      }

      /* "View.MemoryView":899
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "             # <<<<<<<<<<<<<<
 *                                      "must be indexed and not sliced", dim)
 *         else:
 */

#line 899 "View.MemoryView"
      /*else*/ {

        /* "View.MemoryView":900
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 *                                      "must be indexed and not sliced", dim)             # <<<<<<<<<<<<<<
 *         else:
 *             suboffset_dim[0] = new_ndim
 */

#line 900 "View.MemoryView"
        __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"All dimensions preceding dimension %d must be indexed and not sliced"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 899, __pyx_L1_error)

#line 900 "View.MemoryView"
      }

#line 900 "View.MemoryView"
      __pyx_L26:;

      /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */

#line 895 "View.MemoryView"
      goto __pyx_L25;

#line 895 "View.MemoryView"
    }

    /* "View.MemoryView":902
 *                                      "must be indexed and not sliced", dim)
 *         else:
 *             suboffset_dim[0] = new_ndim             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */

#line 902 "View.MemoryView"
    /*else*/ {

#line 902 "View.MemoryView"
      (__pyx_v_suboffset_dim[0]) = __pyx_v_new_ndim;

#line 902 "View.MemoryView"
    }

#line 902 "View.MemoryView"
    __pyx_L25:;

    /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */

#line 894 "View.MemoryView"
  }

  /* "View.MemoryView":904
 *             suboffset_dim[0] = new_ndim
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 904 "View.MemoryView"
  __pyx_r = 0;

#line 904 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

#line 807 "View.MemoryView"


#line 807 "View.MemoryView"
  /* function exit code */

#line 807 "View.MemoryView"
  __pyx_L1_error:;

#line 807 "View.MemoryView"
  {

#line 807 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 807 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 807 "View.MemoryView"
    #endif

#line 807 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.slice_memviewslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 807 "View.MemoryView"
    #ifdef WITH_THREAD

#line 807 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 807 "View.MemoryView"
    #endif

#line 807 "View.MemoryView"
  }

#line 807 "View.MemoryView"
  __pyx_r = -1;

#line 807 "View.MemoryView"
  __pyx_L0:;

#line 807 "View.MemoryView"
  return __pyx_r;

#line 807 "View.MemoryView"
}

/* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

#line 910 "View.MemoryView"


#line 910 "View.MemoryView"
static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stride
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_suboffset
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_itemsize
#line 910 "View.MemoryView"
;
  char *__pyx_v_resultp
#line 910 "View.MemoryView"
;

#line 910 "View.MemoryView"
  char *__pyx_r;

#line 910 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 910 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 910 "View.MemoryView"
  int __pyx_t_2;

#line 910 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 910 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 910 "View.MemoryView"
  int __pyx_lineno = 0;

#line 910 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 910 "View.MemoryView"
  int __pyx_clineno = 0;

#line 910 "View.MemoryView"
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":912
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
 */

#line 912 "View.MemoryView"
  __pyx_v_suboffset = -1L;

  /* "View.MemoryView":913
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
 */

#line 913 "View.MemoryView"
  __pyx_t_1 = __pyx_v_view->itemsize;

#line 913 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */

#line 916 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_view->ndim == 0) != 0);

#line 916 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":917
 * 
 *     if view.ndim == 0:
 *         shape = view.len / itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
 */

#line 917 "View.MemoryView"
    if (unlikely(__pyx_v_itemsize == 0)) {

#line 917 "View.MemoryView"
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");

#line 917 "View.MemoryView"
      __PYX_ERR(1, 917, __pyx_L1_error)

#line 917 "View.MemoryView"
    }

#line 917 "View.MemoryView"
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {

#line 917 "View.MemoryView"
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");

#line 917 "View.MemoryView"
      __PYX_ERR(1, 917, __pyx_L1_error)

#line 917 "View.MemoryView"
    }

#line 917 "View.MemoryView"
    __pyx_v_shape = __Pyx_div_Py_ssize_t(__pyx_v_view->len, __pyx_v_itemsize);

    /* "View.MemoryView":918
 *     if view.ndim == 0:
 *         shape = view.len / itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
 */

#line 918 "View.MemoryView"
    __pyx_v_stride = __pyx_v_itemsize;

    /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */

#line 916 "View.MemoryView"
    goto __pyx_L3;

#line 916 "View.MemoryView"
  }

  /* "View.MemoryView":920
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 */

#line 920 "View.MemoryView"
  /*else*/ {

#line 920 "View.MemoryView"
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":921
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
 */

#line 921 "View.MemoryView"
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */

#line 922 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_view->suboffsets != NULL) != 0);

#line 922 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":923
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
 */

#line 923 "View.MemoryView"
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);

      /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */

#line 922 "View.MemoryView"
    }

#line 922 "View.MemoryView"
  }

#line 922 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */

#line 925 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_index < 0) != 0);

#line 925 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":926
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 */

#line 926 "View.MemoryView"
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 927 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_index < 0) != 0);

#line 927 "View.MemoryView"
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":928
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
 */

#line 928 "View.MemoryView"
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 928 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 928 "View.MemoryView"
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 928 "View.MemoryView"
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 928 "View.MemoryView"
      __PYX_ERR(1, 928, __pyx_L1_error)

      /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 927 "View.MemoryView"
    }

    /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */

#line 925 "View.MemoryView"
  }

  /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 930 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_index >= __pyx_v_shape) != 0);

#line 930 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":931
 * 
 *     if index >= shape:
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
 */

#line 931 "View.MemoryView"
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 931 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 931 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 931 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 931 "View.MemoryView"
    __PYX_ERR(1, 931, __pyx_L1_error)

    /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 930 "View.MemoryView"
  }

  /* "View.MemoryView":933
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
 */

#line 933 "View.MemoryView"
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */

#line 934 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);

#line 934 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":935
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
 */

#line 935 "View.MemoryView"
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);

    /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */

#line 934 "View.MemoryView"
  }

  /* "View.MemoryView":937
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 937 "View.MemoryView"
  __pyx_r = __pyx_v_resultp;

#line 937 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

#line 910 "View.MemoryView"


#line 910 "View.MemoryView"
  /* function exit code */

#line 910 "View.MemoryView"
  __pyx_L1_error:;

#line 910 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 910 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 910 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 910 "View.MemoryView"
  __pyx_r = NULL;

#line 910 "View.MemoryView"
  __pyx_L0:;

#line 910 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 910 "View.MemoryView"
  return __pyx_r;

#line 910 "View.MemoryView"
}

/* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

#line 943 "View.MemoryView"


#line 943 "View.MemoryView"
static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim
#line 943 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_shape
#line 943 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_strides
#line 943 "View.MemoryView"
;
  int __pyx_v_i
#line 943 "View.MemoryView"
;
  int __pyx_v_j
#line 943 "View.MemoryView"
;

#line 943 "View.MemoryView"
  int __pyx_r;

#line 943 "View.MemoryView"
  int __pyx_t_1;

#line 943 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 943 "View.MemoryView"
  long __pyx_t_3;

#line 943 "View.MemoryView"
  long __pyx_t_4;

#line 943 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 943 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

#line 943 "View.MemoryView"
  int __pyx_t_7;

#line 943 "View.MemoryView"
  int __pyx_t_8;

#line 943 "View.MemoryView"
  int __pyx_t_9;

#line 943 "View.MemoryView"
  int __pyx_lineno = 0;

#line 943 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 943 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":944
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 */

#line 944 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;

#line 944 "View.MemoryView"
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":946
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
 */

#line 946 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memslice->shape;

#line 946 "View.MemoryView"
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":947
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 947 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memslice->strides;

#line 947 "View.MemoryView"
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":951
 * 
 *     cdef int i, j
 *     for i in range(ndim / 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 */

#line 951 "View.MemoryView"
  __pyx_t_3 = __Pyx_div_long(__pyx_v_ndim, 2);

#line 951 "View.MemoryView"
  __pyx_t_4 = __pyx_t_3;

#line 951 "View.MemoryView"
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_4; __pyx_t_1+=1) {

#line 951 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":952
 *     cdef int i, j
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
 */

#line 952 "View.MemoryView"
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":953
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 */

#line 953 "View.MemoryView"
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_j]);

#line 953 "View.MemoryView"
    __pyx_t_6 = (__pyx_v_strides[__pyx_v_i]);

#line 953 "View.MemoryView"
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_5;

#line 953 "View.MemoryView"
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_6;

    /* "View.MemoryView":954
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 */

#line 954 "View.MemoryView"
    __pyx_t_6 = (__pyx_v_shape[__pyx_v_j]);

#line 954 "View.MemoryView"
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_i]);

#line 954 "View.MemoryView"
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_6;

#line 954 "View.MemoryView"
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */

#line 956 "View.MemoryView"
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0) != 0);

#line 956 "View.MemoryView"
    if (!__pyx_t_8) {

#line 956 "View.MemoryView"
    } else {

#line 956 "View.MemoryView"
      __pyx_t_7 = __pyx_t_8;

#line 956 "View.MemoryView"
      goto __pyx_L6_bool_binop_done;

#line 956 "View.MemoryView"
    }

#line 956 "View.MemoryView"
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0) != 0);

#line 956 "View.MemoryView"
    __pyx_t_7 = __pyx_t_8;

#line 956 "View.MemoryView"
    __pyx_L6_bool_binop_done:;

#line 956 "View.MemoryView"
    if (__pyx_t_7) {

      /* "View.MemoryView":957
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 1
 */

#line 957 "View.MemoryView"
      __pyx_t_9 = __pyx_memoryview_err(__pyx_builtin_ValueError, ((char *)"Cannot transpose memoryview with indirect dimensions")); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 957, __pyx_L1_error)

      /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */

#line 956 "View.MemoryView"
    }

#line 956 "View.MemoryView"
  }

  /* "View.MemoryView":959
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 959 "View.MemoryView"
  __pyx_r = 1;

#line 959 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

#line 943 "View.MemoryView"


#line 943 "View.MemoryView"
  /* function exit code */

#line 943 "View.MemoryView"
  __pyx_L1_error:;

#line 943 "View.MemoryView"
  {

#line 943 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 943 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 943 "View.MemoryView"
    #endif

#line 943 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 943 "View.MemoryView"
    #ifdef WITH_THREAD

#line 943 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 943 "View.MemoryView"
    #endif

#line 943 "View.MemoryView"
  }

#line 943 "View.MemoryView"
  __pyx_r = 0;

#line 943 "View.MemoryView"
  __pyx_L0:;

#line 943 "View.MemoryView"
  return __pyx_r;

#line 943 "View.MemoryView"
}

/* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
/* Python wrapper */

#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {

#line 976 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 976 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 976 "View.MemoryView"
  __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
  /* function exit code */

#line 976 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 976 "View.MemoryView"
}

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 976 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 976 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":977
 * 
 *     def __dealloc__(self):
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */

#line 977 "View.MemoryView"
  __PYX_XDEC_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
  /* function exit code */

#line 976 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 976 "View.MemoryView"
}

/* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

#line 979 "View.MemoryView"


#line 979 "View.MemoryView"
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {

#line 979 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 979 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 979 "View.MemoryView"
  int __pyx_t_1;

#line 979 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 979 "View.MemoryView"
  int __pyx_lineno = 0;

#line 979 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 979 "View.MemoryView"
  int __pyx_clineno = 0;

#line 979 "View.MemoryView"
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */

#line 980 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->to_object_func != NULL) != 0);

#line 980 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":981
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
 */

#line 981 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 981 "View.MemoryView"
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 981, __pyx_L1_error)

#line 981 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 981 "View.MemoryView"
    __pyx_r = __pyx_t_2;

#line 981 "View.MemoryView"
    __pyx_t_2 = 0;

#line 981 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */

#line 980 "View.MemoryView"
  }

  /* "View.MemoryView":983
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */

#line 983 "View.MemoryView"
  /*else*/ {

#line 983 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 983 "View.MemoryView"
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 983, __pyx_L1_error)

#line 983 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 983 "View.MemoryView"
    __pyx_r = __pyx_t_2;

#line 983 "View.MemoryView"
    __pyx_t_2 = 0;

#line 983 "View.MemoryView"
    goto __pyx_L0;

#line 983 "View.MemoryView"
  }

  /* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

#line 979 "View.MemoryView"


#line 979 "View.MemoryView"
  /* function exit code */

#line 979 "View.MemoryView"
  __pyx_L1_error:;

#line 979 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 979 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 979 "View.MemoryView"
  __pyx_r = 0;

#line 979 "View.MemoryView"
  __pyx_L0:;

#line 979 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 979 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 979 "View.MemoryView"
  return __pyx_r;

#line 979 "View.MemoryView"
}

/* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

#line 985 "View.MemoryView"


#line 985 "View.MemoryView"
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {

#line 985 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 985 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 985 "View.MemoryView"
  int __pyx_t_1;

#line 985 "View.MemoryView"
  int __pyx_t_2;

#line 985 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 985 "View.MemoryView"
  int __pyx_lineno = 0;

#line 985 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 985 "View.MemoryView"
  int __pyx_clineno = 0;

#line 985 "View.MemoryView"
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */

#line 986 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->to_dtype_func != NULL) != 0);

#line 986 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":987
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
 */

#line 987 "View.MemoryView"
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(1, 987, __pyx_L1_error)

    /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */

#line 986 "View.MemoryView"
    goto __pyx_L3;

#line 986 "View.MemoryView"
  }

  /* "View.MemoryView":989
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 989 "View.MemoryView"
  /*else*/ {

#line 989 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 989, __pyx_L1_error)

#line 989 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 989 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 989 "View.MemoryView"
  }

#line 989 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

#line 985 "View.MemoryView"


#line 985 "View.MemoryView"
  /* function exit code */

#line 985 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 985 "View.MemoryView"
  goto __pyx_L0;

#line 985 "View.MemoryView"
  __pyx_L1_error:;

#line 985 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 985 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 985 "View.MemoryView"
  __pyx_r = 0;

#line 985 "View.MemoryView"
  __pyx_L0:;

#line 985 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 985 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 985 "View.MemoryView"
  return __pyx_r;

#line 985 "View.MemoryView"
}

/* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
/* Python wrapper */

#line 992 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 992 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self) {

#line 992 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 992 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 992 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 992 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
  /* function exit code */

#line 992 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 992 "View.MemoryView"
  return __pyx_r;

#line 992 "View.MemoryView"
}

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 992 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 992 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 992 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":993
 *     @property
 *     def base(self):
 *         return self.from_object             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 993 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 993 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->from_object);

#line 993 "View.MemoryView"
  __pyx_r = __pyx_v_self->from_object;

#line 993 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
  /* function exit code */

#line 992 "View.MemoryView"
  __pyx_L0:;

#line 992 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 992 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 992 "View.MemoryView"
  return __pyx_r;

#line 992 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_memoryviewslice___reduce_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

#line 999 "View.MemoryView"


#line 999 "View.MemoryView"
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0
#line 999 "View.MemoryView"
;
  Py_ssize_t __pyx_v_suboffset
#line 999 "View.MemoryView"
;
  PyObject *__pyx_v_length = NULL
#line 999 "View.MemoryView"
;

#line 999 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 999 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 999 "View.MemoryView"
  int __pyx_t_1;

#line 999 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 999 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 999 "View.MemoryView"
  __Pyx_TypeInfo *__pyx_t_4;

#line 999 "View.MemoryView"
  Py_buffer __pyx_t_5;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_6;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_7;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_8;

#line 999 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 999 "View.MemoryView"
  int __pyx_lineno = 0;

#line 999 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 999 "View.MemoryView"
  int __pyx_clineno = 0;

#line 999 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */

#line 1007 "View.MemoryView"
  __pyx_t_1 = ((((PyObject *)__pyx_v_memviewslice.memview) == Py_None) != 0);

#line 1007 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1008
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1008 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 1008 "View.MemoryView"
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 1008 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */

#line 1007 "View.MemoryView"
  }

  /* "View.MemoryView":1013
 * 
 * 
 *     result = _memoryviewslice(None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
 */

#line 1013 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1013 "View.MemoryView"
  __Pyx_INCREF(Py_None);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(Py_None);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None);

#line 1013 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_0);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_int_0);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_0);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_2 = 0;

#line 1013 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryviewslice_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1013 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":1015
 *     result = _memoryviewslice(None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 */

#line 1015 "View.MemoryView"
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":1016
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 */

#line 1016 "View.MemoryView"
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":1018
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 */

#line 1018 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_memviewslice.memview), __pyx_n_s_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1018, __pyx_L1_error)

#line 1018 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1018 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1018 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_result->from_object);

#line 1018 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_result->from_object);

#line 1018 "View.MemoryView"
  __pyx_v_result->from_object = __pyx_t_2;

#line 1018 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":1019
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
 */

#line 1019 "View.MemoryView"
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;

#line 1019 "View.MemoryView"
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":1021
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 */

#line 1021 "View.MemoryView"
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;

#line 1021 "View.MemoryView"
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":1022
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 */

#line 1022 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":1023
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
 */

#line 1023 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":1024
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
 */

#line 1024 "View.MemoryView"
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":1025
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 */

#line 1025 "View.MemoryView"
  Py_INCREF(Py_None);

  /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */

#line 1027 "View.MemoryView"
  __pyx_t_1 = ((((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)->flags & PyBUF_WRITABLE) != 0);

#line 1027 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1028
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 *         result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 *     else:
 *         result.flags = PyBUF_RECORDS_RO
 */

#line 1028 "View.MemoryView"
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

    /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */

#line 1027 "View.MemoryView"
    goto __pyx_L4;

#line 1027 "View.MemoryView"
  }

  /* "View.MemoryView":1030
 *         result.flags = PyBUF_RECORDS
 *     else:
 *         result.flags = PyBUF_RECORDS_RO             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 */

#line 1030 "View.MemoryView"
  /*else*/ {

#line 1030 "View.MemoryView"
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS_RO;

#line 1030 "View.MemoryView"
  }

#line 1030 "View.MemoryView"
  __pyx_L4:;

  /* "View.MemoryView":1032
 *         result.flags = PyBUF_RECORDS_RO
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 * 
 */

#line 1032 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":1033
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1033 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":1036
 * 
 * 
 *     result.view.suboffsets = NULL             # <<<<<<<<<<<<<<
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 1036 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.suboffsets = NULL;

  /* "View.MemoryView":1037
 * 
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 */

#line 1037 "View.MemoryView"
  __pyx_t_7 = (__pyx_v_result->from_slice.suboffsets + __pyx_v_ndim);

#line 1037 "View.MemoryView"
  for (__pyx_t_8 = __pyx_v_result->from_slice.suboffsets; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {

#line 1037 "View.MemoryView"
    __pyx_t_6 = __pyx_t_8;

#line 1037 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_6[0]);

    /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */

#line 1038 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_suboffset >= 0) != 0);

#line 1038 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":1039
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1039 "View.MemoryView"
      __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

      /* "View.MemoryView":1040
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
 */

#line 1040 "View.MemoryView"
      goto __pyx_L6_break;

      /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */

#line 1038 "View.MemoryView"
    }

#line 1038 "View.MemoryView"
  }

#line 1038 "View.MemoryView"
  __pyx_L6_break:;

  /* "View.MemoryView":1042
 *             break
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length
 */

#line 1042 "View.MemoryView"
  __pyx_t_9 = __pyx_v_result->__pyx_base.view.itemsize;

#line 1042 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

  /* "View.MemoryView":1043
 * 
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         result.view.len *= length
 * 
 */

#line 1043 "View.MemoryView"
  __pyx_t_7 = (__pyx_v_result->__pyx_base.view.shape + __pyx_v_ndim);

#line 1043 "View.MemoryView"
  for (__pyx_t_8 = __pyx_v_result->__pyx_base.view.shape; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {

#line 1043 "View.MemoryView"
    __pyx_t_6 = __pyx_t_8;

#line 1043 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t((__pyx_t_6[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1043, __pyx_L1_error)

#line 1043 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1043 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_2);

#line 1043 "View.MemoryView"
    __pyx_t_2 = 0;

    /* "View.MemoryView":1044
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
 */

#line 1044 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_result->__pyx_base.view.len); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1044 "View.MemoryView"
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_2, __pyx_v_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 1044 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1044 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1044 "View.MemoryView"
    __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

#line 1044 "View.MemoryView"
  }

  /* "View.MemoryView":1046
 *         result.view.len *= length
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
 */

#line 1046 "View.MemoryView"
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":1047
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 1047 "View.MemoryView"
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":1049
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 */

#line 1049 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 1049 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 1049 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 1049 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

#line 999 "View.MemoryView"


#line 999 "View.MemoryView"
  /* function exit code */

#line 999 "View.MemoryView"
  __pyx_L1_error:;

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 999 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 999 "View.MemoryView"
  __pyx_r = 0;

#line 999 "View.MemoryView"
  __pyx_L0:;

#line 999 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_length);

#line 999 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 999 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 999 "View.MemoryView"
  return __pyx_r;

#line 999 "View.MemoryView"
}

/* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

#line 1052 "View.MemoryView"


#line 1052 "View.MemoryView"
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0
#line 1052 "View.MemoryView"
;

#line 1052 "View.MemoryView"
  __Pyx_memviewslice *__pyx_r;

#line 1052 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1052 "View.MemoryView"
  int __pyx_t_1;

#line 1052 "View.MemoryView"
  int __pyx_t_2;

#line 1052 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1052 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1052 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1052 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1052 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */

#line 1055 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 1055 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 1055 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1056
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
 */

#line 1056 "View.MemoryView"
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(1, 1056, __pyx_L1_error)

#line 1056 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);

#line 1056 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 1056 "View.MemoryView"
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);

#line 1056 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":1057
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
 */

#line 1057 "View.MemoryView"
    __pyx_r = (&__pyx_v_obj->from_slice);

#line 1057 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */

#line 1055 "View.MemoryView"
  }

  /* "View.MemoryView":1059
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
 */

#line 1059 "View.MemoryView"
  /*else*/ {

#line 1059 "View.MemoryView"
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":1060
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
 */

#line 1060 "View.MemoryView"
    __pyx_r = __pyx_v_mslice;

#line 1060 "View.MemoryView"
    goto __pyx_L0;

#line 1060 "View.MemoryView"
  }

  /* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

#line 1052 "View.MemoryView"


#line 1052 "View.MemoryView"
  /* function exit code */

#line 1052 "View.MemoryView"
  __pyx_L1_error:;

#line 1052 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1052 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1052 "View.MemoryView"
  __pyx_r = NULL;

#line 1052 "View.MemoryView"
  __pyx_L0:;

#line 1052 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);

#line 1052 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1052 "View.MemoryView"
  return __pyx_r;

#line 1052 "View.MemoryView"
}

/* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

#line 1063 "View.MemoryView"


#line 1063 "View.MemoryView"
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_shape
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_strides
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_suboffsets
#line 1063 "View.MemoryView"
;

#line 1063 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1063 "View.MemoryView"
  Py_ssize_t *__pyx_t_1;

#line 1063 "View.MemoryView"
  int __pyx_t_2;

#line 1063 "View.MemoryView"
  int __pyx_t_3;

#line 1063 "View.MemoryView"
  int __pyx_t_4;

#line 1063 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 1063 "View.MemoryView"
  __Pyx_RefNannySetupContext("slice_copy", 0);

  /* "View.MemoryView":1067
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
 */

#line 1067 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.shape;

#line 1067 "View.MemoryView"
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":1068
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
 */

#line 1068 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.strides;

#line 1068 "View.MemoryView"
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":1069
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 */

#line 1069 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;

#line 1069 "View.MemoryView"
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":1071
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
 */

#line 1071 "View.MemoryView"
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":1072
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
 */

#line 1072 "View.MemoryView"
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":1074
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 */

#line 1074 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memview->view.ndim;

#line 1074 "View.MemoryView"
  __pyx_t_3 = __pyx_t_2;

#line 1074 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1074 "View.MemoryView"
    __pyx_v_dim = __pyx_t_4;

    /* "View.MemoryView":1075
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 */

#line 1075 "View.MemoryView"
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":1076
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 */

#line 1076 "View.MemoryView"
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":1077
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
 */

#line 1077 "View.MemoryView"
    if ((__pyx_v_suboffsets != 0)) {

#line 1077 "View.MemoryView"
      __pyx_t_5 = (__pyx_v_suboffsets[__pyx_v_dim]);

#line 1077 "View.MemoryView"
    } else {

#line 1077 "View.MemoryView"
      __pyx_t_5 = -1L;

#line 1077 "View.MemoryView"
    }

#line 1077 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_v_dim]) = __pyx_t_5;

#line 1077 "View.MemoryView"
  }

  /* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

#line 1063 "View.MemoryView"


#line 1063 "View.MemoryView"
  /* function exit code */

#line 1063 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1063 "View.MemoryView"
}

/* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

#line 1080 "View.MemoryView"


#line 1080 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice
#line 1080 "View.MemoryView"
;

#line 1080 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 1080 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1080 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1080 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1080 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1080 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1080 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":1083
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 */

#line 1083 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":1084
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 */

#line 1084 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 1084 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1084, __pyx_L1_error)

#line 1084 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1084 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 1084 "View.MemoryView"
  __pyx_t_1 = 0;

#line 1084 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

#line 1080 "View.MemoryView"


#line 1080 "View.MemoryView"
  /* function exit code */

#line 1080 "View.MemoryView"
  __pyx_L1_error:;

#line 1080 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1080 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1080 "View.MemoryView"
  __pyx_r = 0;

#line 1080 "View.MemoryView"
  __pyx_L0:;

#line 1080 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 1080 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1080 "View.MemoryView"
  return __pyx_r;

#line 1080 "View.MemoryView"
}

/* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

#line 1087 "View.MemoryView"


#line 1087 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *)
#line 1087 "View.MemoryView"
;
  int (*__pyx_v_to_dtype_func)(char *, PyObject *)
#line 1087 "View.MemoryView"
;

#line 1087 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 1087 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1087 "View.MemoryView"
  int __pyx_t_1;

#line 1087 "View.MemoryView"
  int __pyx_t_2;

#line 1087 "View.MemoryView"
  PyObject *(*__pyx_t_3)(char *);

#line 1087 "View.MemoryView"
  int (*__pyx_t_4)(char *, PyObject *);

#line 1087 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 1087 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1087 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1087 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1087 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */

#line 1094 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 1094 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 1094 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1095
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 */

#line 1095 "View.MemoryView"
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;

#line 1095 "View.MemoryView"
    __pyx_v_to_object_func = __pyx_t_3;

    /* "View.MemoryView":1096
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
 */

#line 1096 "View.MemoryView"
    __pyx_t_4 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;

#line 1096 "View.MemoryView"
    __pyx_v_to_dtype_func = __pyx_t_4;

    /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */

#line 1094 "View.MemoryView"
    goto __pyx_L3;

#line 1094 "View.MemoryView"
  }

  /* "View.MemoryView":1098
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
 */

#line 1098 "View.MemoryView"
  /*else*/ {

#line 1098 "View.MemoryView"
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1099
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 */

#line 1099 "View.MemoryView"
    __pyx_v_to_dtype_func = NULL;

#line 1099 "View.MemoryView"
  }

#line 1099 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1101
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
 */

#line 1101 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1103
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1103 "View.MemoryView"
  __pyx_t_5 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1101, __pyx_L1_error)

#line 1103 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_5);

#line 1103 "View.MemoryView"
  __pyx_r = __pyx_t_5;

#line 1103 "View.MemoryView"
  __pyx_t_5 = 0;

#line 1103 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

#line 1087 "View.MemoryView"


#line 1087 "View.MemoryView"
  /* function exit code */

#line 1087 "View.MemoryView"
  __pyx_L1_error:;

#line 1087 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 1087 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1087 "View.MemoryView"
  __pyx_r = 0;

#line 1087 "View.MemoryView"
  __pyx_L0:;

#line 1087 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 1087 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1087 "View.MemoryView"
  return __pyx_r;

#line 1087 "View.MemoryView"
}

/* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

#line 1109 "View.MemoryView"


#line 1109 "View.MemoryView"
static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {

#line 1109 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1109 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */

#line 1110 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_arg < 0) != 0);

#line 1110 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1111
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:
 *         return -arg             # <<<<<<<<<<<<<<
 *     else:
 *         return arg
 */

#line 1111 "View.MemoryView"
    __pyx_r = (-__pyx_v_arg);

#line 1111 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */

#line 1110 "View.MemoryView"
  }

  /* "View.MemoryView":1113
 *         return -arg
 *     else:
 *         return arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
 */

#line 1113 "View.MemoryView"
  /*else*/ {

#line 1113 "View.MemoryView"
    __pyx_r = __pyx_v_arg;

#line 1113 "View.MemoryView"
    goto __pyx_L0;

#line 1113 "View.MemoryView"
  }

  /* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

#line 1109 "View.MemoryView"


#line 1109 "View.MemoryView"
  /* function exit code */

#line 1109 "View.MemoryView"
  __pyx_L0:;

#line 1109 "View.MemoryView"
  return __pyx_r;

#line 1109 "View.MemoryView"
}

/* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

#line 1116 "View.MemoryView"


#line 1116 "View.MemoryView"
static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i
#line 1116 "View.MemoryView"
;
  Py_ssize_t __pyx_v_c_stride
#line 1116 "View.MemoryView"
;
  Py_ssize_t __pyx_v_f_stride
#line 1116 "View.MemoryView"
;

#line 1116 "View.MemoryView"
  char __pyx_r;

#line 1116 "View.MemoryView"
  int __pyx_t_1;

#line 1116 "View.MemoryView"
  int __pyx_t_2;

#line 1116 "View.MemoryView"
  int __pyx_t_3;

#line 1116 "View.MemoryView"
  int __pyx_t_4;

  /* "View.MemoryView":1121
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
 */

#line 1121 "View.MemoryView"
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1122
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */

#line 1122 "View.MemoryView"
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1124
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 */

#line 1124 "View.MemoryView"
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {

#line 1124 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */

#line 1125 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);

#line 1125 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1126
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1126 "View.MemoryView"
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1127
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */

#line 1127 "View.MemoryView"
      goto __pyx_L4_break;

      /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */

#line 1125 "View.MemoryView"
    }

#line 1125 "View.MemoryView"
  }

#line 1125 "View.MemoryView"
  __pyx_L4_break:;

  /* "View.MemoryView":1129
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 */

#line 1129 "View.MemoryView"
  __pyx_t_1 = __pyx_v_ndim;

#line 1129 "View.MemoryView"
  __pyx_t_3 = __pyx_t_1;

#line 1129 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1129 "View.MemoryView"
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */

#line 1130 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);

#line 1130 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1131
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1131 "View.MemoryView"
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1132
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 */

#line 1132 "View.MemoryView"
      goto __pyx_L7_break;

      /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */

#line 1130 "View.MemoryView"
    }

#line 1130 "View.MemoryView"
  }

#line 1130 "View.MemoryView"
  __pyx_L7_break:;

  /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */

#line 1134 "View.MemoryView"
  __pyx_t_2 = ((abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride)) != 0);

#line 1134 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1135
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
 */

#line 1135 "View.MemoryView"
    __pyx_r = 'C';

#line 1135 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */

#line 1134 "View.MemoryView"
  }

  /* "View.MemoryView":1137
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */

#line 1137 "View.MemoryView"
  /*else*/ {

#line 1137 "View.MemoryView"
    __pyx_r = 'F';

#line 1137 "View.MemoryView"
    goto __pyx_L0;

#line 1137 "View.MemoryView"
  }

  /* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

#line 1116 "View.MemoryView"


#line 1116 "View.MemoryView"
  /* function exit code */

#line 1116 "View.MemoryView"
  __pyx_L0:;

#line 1116 "View.MemoryView"
  return __pyx_r;

#line 1116 "View.MemoryView"
}

/* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

#line 1140 "View.MemoryView"


#line 1140 "View.MemoryView"
static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1140 "View.MemoryView"
;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dst_extent
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_src_stride
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dst_stride
#line 1140 "View.MemoryView"
;

#line 1140 "View.MemoryView"
  int __pyx_t_1;

#line 1140 "View.MemoryView"
  int __pyx_t_2;

#line 1140 "View.MemoryView"
  int __pyx_t_3;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_4;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

  /* "View.MemoryView":1147
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 */

#line 1147 "View.MemoryView"
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1148
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 */

#line 1148 "View.MemoryView"
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1149
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 */

#line 1149 "View.MemoryView"
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1150
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */

#line 1150 "View.MemoryView"
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */

#line 1152 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);

#line 1152 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_src_stride > 0) != 0);

#line 1153 "View.MemoryView"
    if (__pyx_t_2) {

#line 1153 "View.MemoryView"
    } else {

#line 1153 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 1153 "View.MemoryView"
      goto __pyx_L5_bool_binop_done;

#line 1153 "View.MemoryView"
    }

#line 1153 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_dst_stride > 0) != 0);

#line 1153 "View.MemoryView"
    if (__pyx_t_2) {

#line 1153 "View.MemoryView"
    } else {

#line 1153 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 1153 "View.MemoryView"
      goto __pyx_L5_bool_binop_done;

#line 1153 "View.MemoryView"
    }

    /* "View.MemoryView":1154
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 */

#line 1154 "View.MemoryView"
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);

#line 1154 "View.MemoryView"
    if (__pyx_t_2) {

#line 1154 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));

#line 1154 "View.MemoryView"
    }

#line 1154 "View.MemoryView"
    __pyx_t_3 = (__pyx_t_2 != 0);

#line 1154 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 1154 "View.MemoryView"
    __pyx_L5_bool_binop_done:;

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":1155
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *        else:
 *            for i in range(dst_extent):
 */

#line 1155 "View.MemoryView"
      (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent)));

      /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
      goto __pyx_L4;

#line 1153 "View.MemoryView"
    }

    /* "View.MemoryView":1157
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 *            for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 */

#line 1157 "View.MemoryView"
    /*else*/ {

#line 1157 "View.MemoryView"
      __pyx_t_4 = __pyx_v_dst_extent;

#line 1157 "View.MemoryView"
      __pyx_t_5 = __pyx_t_4;

#line 1157 "View.MemoryView"
      for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1157 "View.MemoryView"
        __pyx_v_i = __pyx_t_6;

        /* "View.MemoryView":1158
 *        else:
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                src_data += src_stride
 *                dst_data += dst_stride
 */

#line 1158 "View.MemoryView"
        (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize));

        /* "View.MemoryView":1159
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride             # <<<<<<<<<<<<<<
 *                dst_data += dst_stride
 *     else:
 */

#line 1159 "View.MemoryView"
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1160
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 *                dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
 */

#line 1160 "View.MemoryView"
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);

#line 1160 "View.MemoryView"
      }

#line 1160 "View.MemoryView"
    }

#line 1160 "View.MemoryView"
    __pyx_L4:;

    /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */

#line 1152 "View.MemoryView"
    goto __pyx_L3;

#line 1152 "View.MemoryView"
  }

  /* "View.MemoryView":1162
 *                dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
 */

#line 1162 "View.MemoryView"
  /*else*/ {

#line 1162 "View.MemoryView"
    __pyx_t_4 = __pyx_v_dst_extent;

#line 1162 "View.MemoryView"
    __pyx_t_5 = __pyx_t_4;

#line 1162 "View.MemoryView"
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1162 "View.MemoryView"
      __pyx_v_i = __pyx_t_6;

      /* "View.MemoryView":1163
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
 */

#line 1163 "View.MemoryView"
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1167
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
 */

#line 1167 "View.MemoryView"
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1168
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
 */

#line 1168 "View.MemoryView"
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);

#line 1168 "View.MemoryView"
    }

#line 1168 "View.MemoryView"
  }

#line 1168 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

#line 1140 "View.MemoryView"


#line 1140 "View.MemoryView"
  /* function exit code */

#line 1140 "View.MemoryView"
}

/* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

#line 1170 "View.MemoryView"


#line 1170 "View.MemoryView"
static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1173
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 */

#line 1173 "View.MemoryView"
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

#line 1170 "View.MemoryView"


#line 1170 "View.MemoryView"
  /* function exit code */

#line 1170 "View.MemoryView"
}

/* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

#line 1177 "View.MemoryView"


#line 1177 "View.MemoryView"
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_shape
#line 1177 "View.MemoryView"
;
  Py_ssize_t __pyx_v_size
#line 1177 "View.MemoryView"
;

#line 1177 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1177 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

  /* "View.MemoryView":1179
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for shape in src.shape[:ndim]:
 */

#line 1179 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;

#line 1179 "View.MemoryView"
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1181
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 * 
 *     for shape in src.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         size *= shape
 * 
 */

#line 1181 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_src->shape + __pyx_v_ndim);

#line 1181 "View.MemoryView"
  for (__pyx_t_4 = __pyx_v_src->shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {

#line 1181 "View.MemoryView"
    __pyx_t_2 = __pyx_t_4;

#line 1181 "View.MemoryView"
    __pyx_v_shape = (__pyx_t_2[0]);

    /* "View.MemoryView":1182
 * 
 *     for shape in src.shape[:ndim]:
 *         size *= shape             # <<<<<<<<<<<<<<
 * 
 *     return size
 */

#line 1182 "View.MemoryView"
    __pyx_v_size = (__pyx_v_size * __pyx_v_shape);

#line 1182 "View.MemoryView"
  }

  /* "View.MemoryView":1184
 *         size *= shape
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
 */

#line 1184 "View.MemoryView"
  __pyx_r = __pyx_v_size;

#line 1184 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

#line 1177 "View.MemoryView"


#line 1177 "View.MemoryView"
  /* function exit code */

#line 1177 "View.MemoryView"
  __pyx_L0:;

#line 1177 "View.MemoryView"
  return __pyx_r;

#line 1177 "View.MemoryView"
}

/* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

#line 1187 "View.MemoryView"


#line 1187 "View.MemoryView"
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx
#line 1187 "View.MemoryView"
;

#line 1187 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1187 "View.MemoryView"
  int __pyx_t_1;

#line 1187 "View.MemoryView"
  int __pyx_t_2;

#line 1187 "View.MemoryView"
  int __pyx_t_3;

#line 1187 "View.MemoryView"
  int __pyx_t_4;

  /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */

#line 1196 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_order == 'F') != 0);

#line 1196 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1197
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */

#line 1197 "View.MemoryView"
    __pyx_t_2 = __pyx_v_ndim;

#line 1197 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1197 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1197 "View.MemoryView"
      __pyx_v_idx = __pyx_t_4;

      /* "View.MemoryView":1198
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 *     else:
 */

#line 1198 "View.MemoryView"
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1199
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 */

#line 1199 "View.MemoryView"
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));

#line 1199 "View.MemoryView"
    }

    /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */

#line 1196 "View.MemoryView"
    goto __pyx_L3;

#line 1196 "View.MemoryView"
  }

  /* "View.MemoryView":1201
 *             stride *= shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */

#line 1201 "View.MemoryView"
  /*else*/ {

#line 1201 "View.MemoryView"
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1; __pyx_t_2-=1) {

#line 1201 "View.MemoryView"
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1202
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 * 
 */

#line 1202 "View.MemoryView"
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1203
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
 */

#line 1203 "View.MemoryView"
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));

#line 1203 "View.MemoryView"
    }

#line 1203 "View.MemoryView"
  }

#line 1203 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1205
 *             stride *= shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 */

#line 1205 "View.MemoryView"
  __pyx_r = __pyx_v_stride;

#line 1205 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

#line 1187 "View.MemoryView"


#line 1187 "View.MemoryView"
  /* function exit code */

#line 1187 "View.MemoryView"
  __pyx_L0:;

#line 1187 "View.MemoryView"
  return __pyx_r;

#line 1187 "View.MemoryView"
}

/* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

#line 1208 "View.MemoryView"


#line 1208 "View.MemoryView"
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i
#line 1208 "View.MemoryView"
;
  void *__pyx_v_result
#line 1208 "View.MemoryView"
;
  size_t __pyx_v_itemsize
#line 1208 "View.MemoryView"
;
  size_t __pyx_v_size
#line 1208 "View.MemoryView"
;

#line 1208 "View.MemoryView"
  void *__pyx_r;

#line 1208 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1208 "View.MemoryView"
  int __pyx_t_2;

#line 1208 "View.MemoryView"
  int __pyx_t_3;

#line 1208 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_t_4;

#line 1208 "View.MemoryView"
  int __pyx_t_5;

#line 1208 "View.MemoryView"
  int __pyx_t_6;

#line 1208 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1208 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1208 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":1219
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 */

#line 1219 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;

#line 1219 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1220
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
 */

#line 1220 "View.MemoryView"
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1222
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err(MemoryError, NULL)
 */

#line 1222 "View.MemoryView"
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */

#line 1223 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_result != 0)) != 0);

#line 1223 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1224
 *     result = malloc(size)
 *     if not result:
 *         _err(MemoryError, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1224 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_err(__pyx_builtin_MemoryError, NULL); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 1224, __pyx_L1_error)

    /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */

#line 1223 "View.MemoryView"
  }

  /* "View.MemoryView":1227
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 */

#line 1227 "View.MemoryView"
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1228
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 */

#line 1228 "View.MemoryView"
  __pyx_t_4 = __pyx_v_src->memview;

#line 1228 "View.MemoryView"
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1229
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
 */

#line 1229 "View.MemoryView"
  __pyx_t_3 = __pyx_v_ndim;

#line 1229 "View.MemoryView"
  __pyx_t_5 = __pyx_t_3;

#line 1229 "View.MemoryView"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1229 "View.MemoryView"
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1230
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
 */

#line 1230 "View.MemoryView"
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1231
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,
 */

#line 1231 "View.MemoryView"
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1L;

#line 1231 "View.MemoryView"
  }

  /* "View.MemoryView":1233
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,             # <<<<<<<<<<<<<<
 *                               ndim, order)
 * 
 */

#line 1233 "View.MemoryView"
  (void)(__pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order));

  /* "View.MemoryView":1237
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
 */

#line 1237 "View.MemoryView"
  __pyx_t_3 = __pyx_v_ndim;

#line 1237 "View.MemoryView"
  __pyx_t_5 = __pyx_t_3;

#line 1237 "View.MemoryView"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1237 "View.MemoryView"
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */

#line 1238 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1) != 0);

#line 1238 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1239
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src[0], order, ndim):
 */

#line 1239 "View.MemoryView"
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;

      /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */

#line 1238 "View.MemoryView"
    }

#line 1238 "View.MemoryView"
  }

  /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */

#line 1241 "View.MemoryView"
  __pyx_t_2 = (__pyx_memviewslice_is_contig((__pyx_v_src[0]), __pyx_v_order, __pyx_v_ndim) != 0);

#line 1241 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1242
 * 
 *     if slice_is_contig(src[0], order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 */

#line 1242 "View.MemoryView"
    (void)(memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size));

    /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */

#line 1241 "View.MemoryView"
    goto __pyx_L9;

#line 1241 "View.MemoryView"
  }

  /* "View.MemoryView":1244
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 1244 "View.MemoryView"
  /*else*/ {

#line 1244 "View.MemoryView"
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);

#line 1244 "View.MemoryView"
  }

#line 1244 "View.MemoryView"
  __pyx_L9:;

  /* "View.MemoryView":1246
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1246 "View.MemoryView"
  __pyx_r = __pyx_v_result;

#line 1246 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

#line 1208 "View.MemoryView"


#line 1208 "View.MemoryView"
  /* function exit code */

#line 1208 "View.MemoryView"
  __pyx_L1_error:;

#line 1208 "View.MemoryView"
  {

#line 1208 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 1208 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1208 "View.MemoryView"
    #endif

#line 1208 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1208 "View.MemoryView"
    #ifdef WITH_THREAD

#line 1208 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1208 "View.MemoryView"
    #endif

#line 1208 "View.MemoryView"
  }

#line 1208 "View.MemoryView"
  __pyx_r = NULL;

#line 1208 "View.MemoryView"
  __pyx_L0:;

#line 1208 "View.MemoryView"
  return __pyx_r;

#line 1208 "View.MemoryView"
}

/* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

#line 1251 "View.MemoryView"


#line 1251 "View.MemoryView"
static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {

#line 1251 "View.MemoryView"
  int __pyx_r;

#line 1251 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1251 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1251 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1251 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1251 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1251 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1251 "View.MemoryView"
  #endif

#line 1251 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1254
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 *                                                         (i, extent1, extent2))             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
 */

#line 1254 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1254 "View.MemoryView"
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_extent1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1254 "View.MemoryView"
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_extent2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1254 "View.MemoryView"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);

#line 1254 "View.MemoryView"
  __pyx_t_1 = 0;

#line 1254 "View.MemoryView"
  __pyx_t_2 = 0;

#line 1254 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":1253
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %             # <<<<<<<<<<<<<<
 *                                                         (i, extent1, extent2))
 * 
 */

#line 1253 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1253, __pyx_L1_error)

#line 1253 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1253 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1253, __pyx_L1_error)

#line 1253 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1253 "View.MemoryView"
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1253 "View.MemoryView"
  __PYX_ERR(1, 1253, __pyx_L1_error)

  /* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

#line 1251 "View.MemoryView"


#line 1251 "View.MemoryView"
  /* function exit code */

#line 1251 "View.MemoryView"
  __pyx_L1_error:;

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1251 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1251 "View.MemoryView"
  __pyx_r = -1;

#line 1251 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1251 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1251 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1251 "View.MemoryView"
  #endif

#line 1251 "View.MemoryView"
  return __pyx_r;

#line 1251 "View.MemoryView"
}

/* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

#line 1257 "View.MemoryView"


#line 1257 "View.MemoryView"
static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, char *__pyx_v_msg, int __pyx_v_dim) {

#line 1257 "View.MemoryView"
  int __pyx_r;

#line 1257 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1257 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1257 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1257 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1257 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1257 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1257 "View.MemoryView"
  #endif

#line 1257 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err_dim", 0);

#line 1257 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1258
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:
 *     raise error(msg.decode('ascii') % dim)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
 */

#line 1258 "View.MemoryView"
  __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1258 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1258 "View.MemoryView"
  __pyx_t_4 = PyUnicode_Format(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1258 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);
  __pyx_t_3 = __pyx_v_error; 
#line 1258 "View.MemoryView"
__pyx_t_2 = NULL;

#line 1258 "View.MemoryView"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 1258 "View.MemoryView"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);

#line 1258 "View.MemoryView"
    if (likely(__pyx_t_2)) {

#line 1258 "View.MemoryView"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 1258 "View.MemoryView"
      __Pyx_INCREF(__pyx_t_2);

#line 1258 "View.MemoryView"
      __Pyx_INCREF(function);

#line 1258 "View.MemoryView"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 1258 "View.MemoryView"
    }

#line 1258 "View.MemoryView"
  }

#line 1258 "View.MemoryView"
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);

#line 1258 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1258 "View.MemoryView"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1258 "View.MemoryView"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1258 "View.MemoryView"
  __PYX_ERR(1, 1258, __pyx_L1_error)

  /* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

#line 1257 "View.MemoryView"


#line 1257 "View.MemoryView"
  /* function exit code */

#line 1257 "View.MemoryView"
  __pyx_L1_error:;

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1257 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1257 "View.MemoryView"
  __pyx_r = -1;

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_error);

#line 1257 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1257 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1257 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1257 "View.MemoryView"
  #endif

#line 1257 "View.MemoryView"
  return __pyx_r;

#line 1257 "View.MemoryView"
}

/* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

#line 1261 "View.MemoryView"


#line 1261 "View.MemoryView"
static int __pyx_memoryview_err(PyObject *__pyx_v_error, char *__pyx_v_msg) {

#line 1261 "View.MemoryView"
  int __pyx_r;

#line 1261 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1261 "View.MemoryView"
  int __pyx_t_1;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 1261 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1261 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1261 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1261 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1261 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1261 "View.MemoryView"
  #endif

#line 1261 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err", 0);

#line 1261 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */

#line 1262 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_msg != NULL) != 0);

#line 1262 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":1263
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))             # <<<<<<<<<<<<<<
 *     else:
 *         raise error
 */

#line 1263 "View.MemoryView"
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1263, __pyx_L1_error)

#line 1263 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 1263 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_error);
    __pyx_t_4 = __pyx_v_error; 
#line 1263 "View.MemoryView"
__pyx_t_5 = NULL;

#line 1263 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 1263 "View.MemoryView"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 1263 "View.MemoryView"
      if (likely(__pyx_t_5)) {

#line 1263 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 1263 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_5);

#line 1263 "View.MemoryView"
        __Pyx_INCREF(function);

#line 1263 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 1263 "View.MemoryView"
      }

#line 1263 "View.MemoryView"
    }

#line 1263 "View.MemoryView"
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);

#line 1263 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1263 "View.MemoryView"
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1263, __pyx_L1_error)

#line 1263 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1263 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1263 "View.MemoryView"
    __PYX_ERR(1, 1263, __pyx_L1_error)

    /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */

#line 1262 "View.MemoryView"
  }

  /* "View.MemoryView":1265
 *         raise error(msg.decode('ascii'))
 *     else:
 *         raise error             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_contents')
 */

#line 1265 "View.MemoryView"
  /*else*/ {

#line 1265 "View.MemoryView"
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);

#line 1265 "View.MemoryView"
    __PYX_ERR(1, 1265, __pyx_L1_error)

#line 1265 "View.MemoryView"
  }

  /* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

#line 1261 "View.MemoryView"


#line 1261 "View.MemoryView"
  /* function exit code */

#line 1261 "View.MemoryView"
  __pyx_L1_error:;

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 1261 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1261 "View.MemoryView"
  __pyx_r = -1;

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_error);

#line 1261 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1261 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1261 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1261 "View.MemoryView"
  #endif

#line 1261 "View.MemoryView"
  return __pyx_r;

#line 1261 "View.MemoryView"
}

/* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

#line 1268 "View.MemoryView"


#line 1268 "View.MemoryView"
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata
#line 1268 "View.MemoryView"
;
  size_t __pyx_v_itemsize
#line 1268 "View.MemoryView"
;
  int __pyx_v_i
#line 1268 "View.MemoryView"
;
  char __pyx_v_order
#line 1268 "View.MemoryView"
;
  int __pyx_v_broadcasting
#line 1268 "View.MemoryView"
;
  int __pyx_v_direct_copy
#line 1268 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 1268 "View.MemoryView"
;
  int __pyx_v_ndim
#line 1268 "View.MemoryView"
;

#line 1268 "View.MemoryView"
  int __pyx_r;

#line 1268 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1268 "View.MemoryView"
  int __pyx_t_2;

#line 1268 "View.MemoryView"
  int __pyx_t_3;

#line 1268 "View.MemoryView"
  int __pyx_t_4;

#line 1268 "View.MemoryView"
  int __pyx_t_5;

#line 1268 "View.MemoryView"
  int __pyx_t_6;

#line 1268 "View.MemoryView"
  void *__pyx_t_7;

#line 1268 "View.MemoryView"
  int __pyx_t_8;

#line 1268 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1268 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1268 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":1276
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 */

#line 1276 "View.MemoryView"
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1277
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 */

#line 1277 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;

#line 1277 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1279
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
 */

#line 1279 "View.MemoryView"
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1280
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
 */

#line 1280 "View.MemoryView"
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1281
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
 */

#line 1281 "View.MemoryView"
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */

#line 1284 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_src_ndim < __pyx_v_dst_ndim) != 0);

#line 1284 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1285
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 */

#line 1285 "View.MemoryView"
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);

    /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */

#line 1284 "View.MemoryView"
    goto __pyx_L3;

#line 1284 "View.MemoryView"
  }

  /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */

#line 1286 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_dst_ndim < __pyx_v_src_ndim) != 0);

#line 1286 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1287
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
 */

#line 1287 "View.MemoryView"
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);

    /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */

#line 1286 "View.MemoryView"
  }

#line 1286 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1289
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */

#line 1289 "View.MemoryView"
  __pyx_t_3 = __pyx_v_dst_ndim;

#line 1289 "View.MemoryView"
  __pyx_t_4 = __pyx_v_src_ndim;

#line 1289 "View.MemoryView"
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {

#line 1289 "View.MemoryView"
    __pyx_t_5 = __pyx_t_3;

#line 1289 "View.MemoryView"
  } else {

#line 1289 "View.MemoryView"
    __pyx_t_5 = __pyx_t_4;

#line 1289 "View.MemoryView"
  }

#line 1289 "View.MemoryView"
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1291
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 */

#line 1291 "View.MemoryView"
  __pyx_t_5 = __pyx_v_ndim;

#line 1291 "View.MemoryView"
  __pyx_t_3 = __pyx_t_5;

#line 1291 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1291 "View.MemoryView"
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */

#line 1292 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i])) != 0);

#line 1292 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */

#line 1293 "View.MemoryView"
      __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) == 1) != 0);

#line 1293 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":1294
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
 */

#line 1294 "View.MemoryView"
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1295
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 */

#line 1295 "View.MemoryView"
        (__pyx_v_src.strides[__pyx_v_i]) = 0;

        /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */

#line 1293 "View.MemoryView"
        goto __pyx_L7;

#line 1293 "View.MemoryView"
      }

      /* "View.MemoryView":1297
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
 */

#line 1297 "View.MemoryView"
      /*else*/ {

#line 1297 "View.MemoryView"
        __pyx_t_6 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 1297, __pyx_L1_error)

#line 1297 "View.MemoryView"
      }

#line 1297 "View.MemoryView"
      __pyx_L7:;

      /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */

#line 1292 "View.MemoryView"
    }

    /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */

#line 1299 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0) != 0);

#line 1299 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1300
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 */

#line 1300 "View.MemoryView"
      __pyx_t_6 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Dimension %d is not direct"), __pyx_v_i); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 1300, __pyx_L1_error)

      /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */

#line 1299 "View.MemoryView"
    }

#line 1299 "View.MemoryView"
  }

  /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */

#line 1302 "View.MemoryView"
  __pyx_t_2 = (__pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize) != 0);

#line 1302 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */

#line 1304 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim) != 0)) != 0);

#line 1304 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1305
 * 
 *         if not slice_is_contig(src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 */

#line 1305 "View.MemoryView"
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);

      /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */

#line 1304 "View.MemoryView"
    }

    /* "View.MemoryView":1307
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
 */

#line 1307 "View.MemoryView"
    __pyx_t_7 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(1, 1307, __pyx_L1_error)

#line 1307 "View.MemoryView"
    __pyx_v_tmpdata = __pyx_t_7;

    /* "View.MemoryView":1308
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
 */

#line 1308 "View.MemoryView"
    __pyx_v_src = __pyx_v_tmp;

    /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */

#line 1302 "View.MemoryView"
  }

  /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1310 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_broadcasting != 0)) != 0);

#line 1310 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */

#line 1313 "View.MemoryView"
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'C', __pyx_v_ndim) != 0);

#line 1313 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1314
 * 
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 */

#line 1314 "View.MemoryView"
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'C', __pyx_v_ndim);

      /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */

#line 1313 "View.MemoryView"
      goto __pyx_L12;

#line 1313 "View.MemoryView"
    }

    /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */

#line 1315 "View.MemoryView"
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'F', __pyx_v_ndim) != 0);

#line 1315 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1316
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
 */

#line 1316 "View.MemoryView"
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'F', __pyx_v_ndim);

      /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */

#line 1315 "View.MemoryView"
    }

#line 1315 "View.MemoryView"
    __pyx_L12:;

    /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1318 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_direct_copy != 0);

#line 1318 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1320
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 */

#line 1320 "View.MemoryView"
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1321
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 */

#line 1321 "View.MemoryView"
      (void)(memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim)));

      /* "View.MemoryView":1322
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
 */

#line 1322 "View.MemoryView"
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1323
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */

#line 1323 "View.MemoryView"
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1324
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
 */

#line 1324 "View.MemoryView"
      __pyx_r = 0;

#line 1324 "View.MemoryView"
      goto __pyx_L0;

      /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1318 "View.MemoryView"
    }

    /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1310 "View.MemoryView"
  }

  /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1326 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_order == 'F');

#line 1326 "View.MemoryView"
  if (__pyx_t_2) {

#line 1326 "View.MemoryView"
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));

#line 1326 "View.MemoryView"
  }

#line 1326 "View.MemoryView"
  __pyx_t_8 = (__pyx_t_2 != 0);

#line 1326 "View.MemoryView"
  if (__pyx_t_8) {

    /* "View.MemoryView":1329
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
 */

#line 1329 "View.MemoryView"
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(1, 1329, __pyx_L1_error)

    /* "View.MemoryView":1330
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1330 "View.MemoryView"
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(1, 1330, __pyx_L1_error)

    /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1326 "View.MemoryView"
  }

  /* "View.MemoryView":1332
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 */

#line 1332 "View.MemoryView"
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1333
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 */

#line 1333 "View.MemoryView"
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1334
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
 */

#line 1334 "View.MemoryView"
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1336
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */

#line 1336 "View.MemoryView"
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1337
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 */

#line 1337 "View.MemoryView"
  __pyx_r = 0;

#line 1337 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

#line 1268 "View.MemoryView"


#line 1268 "View.MemoryView"
  /* function exit code */

#line 1268 "View.MemoryView"
  __pyx_L1_error:;

#line 1268 "View.MemoryView"
  {

#line 1268 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 1268 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1268 "View.MemoryView"
    #endif

#line 1268 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1268 "View.MemoryView"
    #ifdef WITH_THREAD

#line 1268 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1268 "View.MemoryView"
    #endif

#line 1268 "View.MemoryView"
  }

#line 1268 "View.MemoryView"
  __pyx_r = -1;

#line 1268 "View.MemoryView"
  __pyx_L0:;

#line 1268 "View.MemoryView"
  return __pyx_r;

#line 1268 "View.MemoryView"
}

/* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

#line 1340 "View.MemoryView"


#line 1340 "View.MemoryView"
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i
#line 1340 "View.MemoryView"
;
  int __pyx_v_offset
#line 1340 "View.MemoryView"
;

#line 1340 "View.MemoryView"
  int __pyx_t_1;

#line 1340 "View.MemoryView"
  int __pyx_t_2;

#line 1340 "View.MemoryView"
  int __pyx_t_3;

  /* "View.MemoryView":1344
 *                             int ndim_other) nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */

#line 1344 "View.MemoryView"
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1346
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 */

#line 1346 "View.MemoryView"
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {

#line 1346 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1347
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]             # <<<<<<<<<<<<<<
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 */

#line 1347 "View.MemoryView"
    (__pyx_v_mslice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->shape[__pyx_v_i]);

    /* "View.MemoryView":1348
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 */

#line 1348 "View.MemoryView"
    (__pyx_v_mslice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->strides[__pyx_v_i]);

    /* "View.MemoryView":1349
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
 */

#line 1349 "View.MemoryView"
    (__pyx_v_mslice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->suboffsets[__pyx_v_i]);

#line 1349 "View.MemoryView"
  }

  /* "View.MemoryView":1351
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 */

#line 1351 "View.MemoryView"
  __pyx_t_1 = __pyx_v_offset;

#line 1351 "View.MemoryView"
  __pyx_t_2 = __pyx_t_1;

#line 1351 "View.MemoryView"
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {

#line 1351 "View.MemoryView"
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1352
 * 
 *     for i in range(offset):
 *         mslice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1
 */

#line 1352 "View.MemoryView"
    (__pyx_v_mslice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1353
 *     for i in range(offset):
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i] = -1
 * 
 */

#line 1353 "View.MemoryView"
    (__pyx_v_mslice->strides[__pyx_v_i]) = (__pyx_v_mslice->strides[0]);

    /* "View.MemoryView":1354
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1354 "View.MemoryView"
    (__pyx_v_mslice->suboffsets[__pyx_v_i]) = -1L;

#line 1354 "View.MemoryView"
  }

  /* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

#line 1340 "View.MemoryView"


#line 1340 "View.MemoryView"
  /* function exit code */

#line 1340 "View.MemoryView"
}

/* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

#line 1362 "View.MemoryView"


#line 1362 "View.MemoryView"
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {

#line 1362 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */

#line 1366 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_dtype_is_object != 0);

#line 1366 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1367
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,             # <<<<<<<<<<<<<<
 *                                            dst.strides, ndim, inc)
 * 
 */

#line 1367 "View.MemoryView"
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);

    /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */

#line 1366 "View.MemoryView"
  }

  /* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

#line 1362 "View.MemoryView"


#line 1362 "View.MemoryView"
  /* function exit code */

#line 1362 "View.MemoryView"
}

/* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

#line 1371 "View.MemoryView"


#line 1371 "View.MemoryView"
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {

#line 1371 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1371 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1371 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1371 "View.MemoryView"
  #endif

#line 1371 "View.MemoryView"
  __Pyx_RefNannySetupContext("refcount_objects_in_slice_with_gil", 0);

  /* "View.MemoryView":1374
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 */

#line 1374 "View.MemoryView"
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

#line 1371 "View.MemoryView"


#line 1371 "View.MemoryView"
  /* function exit code */

#line 1371 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1371 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1371 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1371 "View.MemoryView"
  #endif

#line 1371 "View.MemoryView"
}

/* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

#line 1377 "View.MemoryView"


#line 1377 "View.MemoryView"
static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1377 "View.MemoryView"
;

#line 1377 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_2;

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 1377 "View.MemoryView"
  int __pyx_t_4;

#line 1377 "View.MemoryView"
  __Pyx_RefNannySetupContext("refcount_objects_in_slice", 0);

  /* "View.MemoryView":1381
 *     cdef Py_ssize_t i
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
 */

#line 1381 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_shape[0]);

#line 1381 "View.MemoryView"
  __pyx_t_2 = __pyx_t_1;

#line 1381 "View.MemoryView"
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {

#line 1381 "View.MemoryView"
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */

#line 1382 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_ndim == 1) != 0);

#line 1382 "View.MemoryView"
    if (__pyx_t_4) {

      /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */

#line 1383 "View.MemoryView"
      __pyx_t_4 = (__pyx_v_inc != 0);

#line 1383 "View.MemoryView"
      if (__pyx_t_4) {

        /* "View.MemoryView":1384
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
 */

#line 1384 "View.MemoryView"
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));

        /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */

#line 1383 "View.MemoryView"
        goto __pyx_L6;

#line 1383 "View.MemoryView"
      }

      /* "View.MemoryView":1386
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 */

#line 1386 "View.MemoryView"
      /*else*/ {

#line 1386 "View.MemoryView"
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));

#line 1386 "View.MemoryView"
      }

#line 1386 "View.MemoryView"
      __pyx_L6:;

      /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */

#line 1382 "View.MemoryView"
      goto __pyx_L5;

#line 1382 "View.MemoryView"
    }

    /* "View.MemoryView":1388
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                       ndim - 1, inc)
 * 
 */

#line 1388 "View.MemoryView"
    /*else*/ {

      /* "View.MemoryView":1389
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 *                                       ndim - 1, inc)             # <<<<<<<<<<<<<<
 * 
 *         data += strides[0]
 */

#line 1389 "View.MemoryView"
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);

#line 1389 "View.MemoryView"
    }

#line 1389 "View.MemoryView"
    __pyx_L5:;

    /* "View.MemoryView":1391
 *                                       ndim - 1, inc)
 * 
 *         data += strides[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1391 "View.MemoryView"
    __pyx_v_data = (__pyx_v_data + (__pyx_v_strides[0]));

#line 1391 "View.MemoryView"
  }

  /* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

#line 1377 "View.MemoryView"


#line 1377 "View.MemoryView"
  /* function exit code */

#line 1377 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1377 "View.MemoryView"
}

/* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

#line 1397 "View.MemoryView"


#line 1397 "View.MemoryView"
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1400
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 */

#line 1400 "View.MemoryView"
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1401
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,             # <<<<<<<<<<<<<<
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)
 */

#line 1401 "View.MemoryView"
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1403
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1403 "View.MemoryView"
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

#line 1397 "View.MemoryView"


#line 1397 "View.MemoryView"
  /* function exit code */

#line 1397 "View.MemoryView"
}

/* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

#line 1407 "View.MemoryView"


#line 1407 "View.MemoryView"
static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1407 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stride
#line 1407 "View.MemoryView"
;
  Py_ssize_t __pyx_v_extent
#line 1407 "View.MemoryView"
;

#line 1407 "View.MemoryView"
  int __pyx_t_1;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_2;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_4;

  /* "View.MemoryView":1411
 *                               size_t itemsize, void *item) nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
 */

#line 1411 "View.MemoryView"
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1412
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */

#line 1412 "View.MemoryView"
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */

#line 1414 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);

#line 1414 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1415
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
 */

#line 1415 "View.MemoryView"
    __pyx_t_2 = __pyx_v_extent;

#line 1415 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1415 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1415 "View.MemoryView"
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1416
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
 */

#line 1416 "View.MemoryView"
      (void)(memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize));

      /* "View.MemoryView":1417
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
 */

#line 1417 "View.MemoryView"
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);

#line 1417 "View.MemoryView"
    }

    /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */

#line 1414 "View.MemoryView"
    goto __pyx_L3;

#line 1414 "View.MemoryView"
  }

  /* "View.MemoryView":1419
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 */

#line 1419 "View.MemoryView"
  /*else*/ {

#line 1419 "View.MemoryView"
    __pyx_t_2 = __pyx_v_extent;

#line 1419 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1419 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1419 "View.MemoryView"
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1420
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                 ndim - 1, itemsize, item)
 *             data += stride
 */

#line 1420 "View.MemoryView"
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1422
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1422 "View.MemoryView"
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);

#line 1422 "View.MemoryView"
    }

#line 1422 "View.MemoryView"
  }

#line 1422 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

#line 1407 "View.MemoryView"


#line 1407 "View.MemoryView"
  /* function exit code */

#line 1407 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum = 
#line 1 "(tree fragment)"
{"__pyx_unpickle_Enum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum, METH_VARARGS|METH_KEYWORDS, 0};

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0
#line 1 "(tree fragment)"
;
  long __pyx_v___pyx_checksum
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_state = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum (wrapper)", 0);

#line 1 "(tree fragment)"
  {

#line 1 "(tree fragment)"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};

#line 1 "(tree fragment)"
    PyObject* values[3] = {0,0,0};

#line 1 "(tree fragment)"
    if (unlikely(__pyx_kwds)) {

#line 1 "(tree fragment)"
      Py_ssize_t kw_args;

#line 1 "(tree fragment)"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1 "(tree fragment)"
      switch (pos_args) {
        case  3: 
#line 1 "(tree fragment)"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1 "(tree fragment)"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1 "(tree fragment)"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  0: break;
        default: 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1 "(tree fragment)"
      switch (pos_args) {

#line 1 "(tree fragment)"
        case  0:

#line 1 "(tree fragment)"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  1:

#line 1 "(tree fragment)"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 1); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  2:

#line 1 "(tree fragment)"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 2); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      if (unlikely(kw_args > 0)) {

#line 1 "(tree fragment)"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Enum") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 1 "(tree fragment)"
      goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
    } else {

#line 1 "(tree fragment)"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
    }

#line 1 "(tree fragment)"
    __pyx_v___pyx_type = values[0];

#line 1 "(tree fragment)"
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
    __pyx_v___pyx_state = values[2];

#line 1 "(tree fragment)"
  }

#line 1 "(tree fragment)"
  goto __pyx_L4_argument_unpacking_done;

#line 1 "(tree fragment)"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 1 "(tree fragment)"
__PYX_ERR(1, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
  __pyx_L3_error:;

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return NULL;

#line 1 "(tree fragment)"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_result = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__30, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 4 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 */

#line 5 "(tree fragment)"
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __pyx_v___pyx_PickleError = __pyx_t_1;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */

#line 6 "(tree fragment)"
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_4, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; 
#line 6 "(tree fragment)"
__pyx_t_6 = NULL;

#line 6 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 6 "(tree fragment)"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);

#line 6 "(tree fragment)"
      if (likely(__pyx_t_6)) {

#line 6 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 6 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_6);

#line 6 "(tree fragment)"
        __Pyx_INCREF(function);

#line 6 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 6 "(tree fragment)"
      }

#line 6 "(tree fragment)"
    }

#line 6 "(tree fragment)"
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 6 "(tree fragment)"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 6 "(tree fragment)"
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 */

#line 7 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_MemviewEnum_type), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "(tree fragment)"
  __pyx_t_5 = NULL;

#line 7 "(tree fragment)"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 7 "(tree fragment)"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);

#line 7 "(tree fragment)"
    if (likely(__pyx_t_5)) {

#line 7 "(tree fragment)"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 7 "(tree fragment)"
      __Pyx_INCREF(__pyx_t_5);

#line 7 "(tree fragment)"
      __Pyx_INCREF(function);

#line 7 "(tree fragment)"
      __Pyx_DECREF_SET(__pyx_t_1, function);

#line 7 "(tree fragment)"
    }

#line 7 "(tree fragment)"
  }

#line 7 "(tree fragment)"
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);

#line 7 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 7 "(tree fragment)"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 7 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 7 "(tree fragment)"
  __pyx_v___pyx_result = __pyx_t_4;

#line 7 "(tree fragment)"
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);

#line 8 "(tree fragment)"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 8 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 */

#line 9 "(tree fragment)"
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __pyx_t_4 = __pyx_unpickle_Enum__set_state(((struct __pyx_MemviewEnum_obj *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 9 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 */

#line 10 "(tree fragment)"
  __Pyx_XDECREF(__pyx_r);

#line 10 "(tree fragment)"
  __Pyx_INCREF(__pyx_v___pyx_result);

#line 10 "(tree fragment)"
  __pyx_r = __pyx_v___pyx_result;

#line 10 "(tree fragment)"
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_result);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {

#line 11 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 11 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 11 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 11 "(tree fragment)"
  int __pyx_t_2;

#line 11 "(tree fragment)"
  Py_ssize_t __pyx_t_3;

#line 11 "(tree fragment)"
  int __pyx_t_4;

#line 11 "(tree fragment)"
  int __pyx_t_5;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 11 "(tree fragment)"
  int __pyx_lineno = 0;

#line 11 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 11 "(tree fragment)"
  int __pyx_clineno = 0;

#line 11 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->name);

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_v___pyx_result->name);

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->name = __pyx_t_1;

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 13 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 13 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 13 "(tree fragment)"
    __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);

#line 13 "(tree fragment)"
  if (__pyx_t_4) {

#line 13 "(tree fragment)"
  } else {

#line 13 "(tree fragment)"
    __pyx_t_2 = __pyx_t_4;

#line 13 "(tree fragment)"
    goto __pyx_L4_bool_binop_done;

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_5 = (__pyx_t_4 != 0);

#line 13 "(tree fragment)"
  __pyx_t_2 = __pyx_t_5;

#line 13 "(tree fragment)"
  __pyx_L4_bool_binop_done:;

#line 13 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 14 "(tree fragment)"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 14 "(tree fragment)"
      __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_8 = NULL;

#line 14 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {

#line 14 "(tree fragment)"
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);

#line 14 "(tree fragment)"
      if (likely(__pyx_t_8)) {

#line 14 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);

#line 14 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_8);

#line 14 "(tree fragment)"
        __Pyx_INCREF(function);

#line 14 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_7, function);

#line 14 "(tree fragment)"
      }

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);

#line 14 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 13 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /* function exit code */

#line 11 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 11 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
  __pyx_r = 0;

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  return __pyx_r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary(PyObject *o) {

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13first_minimum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7maximum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14second_minimum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_19first_minimum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_13maximum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_20second_minimum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_total_distance(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_total_distance(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_14total_distance_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary[] = {

#line 11 "(tree fragment)"
  {"split", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_5split, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_4split},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_7__reduce_cython__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_6__reduce_cython__},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_9__setstate_cython__, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_8__setstate_cython__},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary[] = {

#line 11 "(tree fragment)"
  {(char *)"first_minimum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum, (char *)"first_minimum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"maximum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum, (char *)"maximum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"second_minimum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum, (char *)"second_minimum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"first_minimum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_first_minimum_index, (char *)"first_minimum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"maximum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_maximum_index, (char *)"maximum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"second_minimum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_second_minimum_index, (char *)"second_minimum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"total_distance", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_total_distance, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_total_distance, (char *)"total_distance: 'double'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.PeakBoundary", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_3__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  "PeakBoundary(first_minimum, maximum, second_minimum, first_minimum_index, maximum_index, second_minimum_index)", /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint(PyObject *o) {

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13first_maximum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7minimum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14second_maximum_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_19first_maximum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_13minimum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_20second_maximum_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_total_distance(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_total_distance(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_14total_distance_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint[] = {

#line 11 "(tree fragment)"
  {"split", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_5split, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_4split},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_7__reduce_cython__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_6__reduce_cython__},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_9__setstate_cython__, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_8__setstate_cython__},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint[] = {

#line 11 "(tree fragment)"
  {(char *)"first_maximum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum, (char *)"first_maximum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"minimum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum, (char *)"minimum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"second_maximum", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum, (char *)"second_maximum: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"first_maximum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_first_maximum_index, (char *)"first_maximum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"minimum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_minimum_index, (char *)"minimum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"second_maximum_index", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum_index, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_second_maximum_index, (char *)"second_maximum_index: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"total_distance", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_total_distance, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_total_distance, (char *)"total_distance: 'double'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.ValleyPoint", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_3__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  "ValleyPoint(first_maximum, minimum, second_maximum, first_maximum_index, minimum_index, second_maximum_index)", /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;

#line 11 "(tree fragment)"
  p->profile = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->partition_sites = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->xs = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->ys = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->profile);

#line 11 "(tree fragment)"
  Py_CLEAR(p->partition_sites);

#line 11 "(tree fragment)"
  Py_CLEAR(p->xs);

#line 11 "(tree fragment)"
  Py_CLEAR(p->ys);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)o;

#line 11 "(tree fragment)"
  if (p->profile) {

#line 11 "(tree fragment)"
    e = (*v)(p->profile, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->partition_sites) {

#line 11 "(tree fragment)"
    e = (*v)(p->partition_sites, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->xs) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->xs), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->ys) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->ys), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->profile);

#line 11 "(tree fragment)"
  p->profile = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->partition_sites);

#line 11 "(tree fragment)"
  p->partition_sites = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->xs);

#line 11 "(tree fragment)"
  p->xs = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->ys);

#line 11 "(tree fragment)"
  p->ys = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_profile(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_profile(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7profile_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_partition_sites(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_partition_sites(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15partition_sites_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_xs(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_xs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2xs_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_ys(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_ys(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2ys_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter[] = {

#line 11 "(tree fragment)"
  {"_extreme_indices", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_3_extreme_indices, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_2_extreme_indices},

#line 11 "(tree fragment)"
  {"interpolate", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_5interpolate, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_4interpolate},

#line 11 "(tree fragment)"
  {"locate_peak_boundaries", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_7locate_peak_boundaries, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_6locate_peak_boundaries},

#line 11 "(tree fragment)"
  {"locate_valleys", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_9locate_valleys, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_8locate_valleys},

#line 11 "(tree fragment)"
  {"extract_peak", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_11extract_peak, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_10extract_peak},

#line 11 "(tree fragment)"
  {"split_valley", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_13split_valley, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_12split_valley},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_15__reduce_cython__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_14__reduce_cython__},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_17__setstate_cython__, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_16__setstate_cython__},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter[] = {

#line 11 "(tree fragment)"
  {(char *)"profile", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_profile, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_profile, (char *)"profile: object", 0},

#line 11 "(tree fragment)"
  {(char *)"partition_sites", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_partition_sites, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_partition_sites, (char *)"partition_sites: list", 0},

#line 11 "(tree fragment)"
  {(char *)"xs", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_xs, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_xs, (char *)"xs: numpy.ndarray", 0},

#line 11 "(tree fragment)"
  {(char *)"ys", __pyx_getprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_ys, __pyx_setprop_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_ys, (char *)"ys: numpy.ndarray", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.ProfileSplitter", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "ProfileSplitter(profile)", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *__pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys[--__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_ps);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_splitter);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys[__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys++] = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_v_ps) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_v_ps), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_v_splitter) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_v_splitter, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->__pyx_v_ps);

#line 11 "(tree fragment)"
  p->__pyx_v_ps = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->__pyx_v_splitter);

#line 11 "(tree fragment)"
  p->__pyx_v_splitter = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.__pyx_scope_struct__split_valleys", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *__pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr[--__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_outer_scope);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_p);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_t_0);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr[__pyx_freecount_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_outer_scope) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_v_p) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_v_p, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_t_0) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_t_0, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.__pyx_scope_struct_1_genexpr", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_array __pyx_vtable_array;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_array_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_array_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_array;

#line 11 "(tree fragment)"
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) goto bad;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
  bad:

#line 11 "(tree fragment)"
  Py_DECREF(o); o = 0;

#line 11 "(tree fragment)"
  return NULL;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_array(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_array___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->mode);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_format);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_array___setitem__(o, i, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_Format(PyExc_NotImplementedError,

#line 11 "(tree fragment)"
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {

#line 11 "(tree fragment)"
  PyObject *v = __Pyx_PyObject_GenericGetAttr(o, n);

#line 11 "(tree fragment)"
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {

#line 11 "(tree fragment)"
    PyErr_Clear();

#line 11 "(tree fragment)"
    v = __pyx_array___getattr__(o, n);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return v;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_array[] = {

#line 11 "(tree fragment)"
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_array_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_array_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_array[] = {

#line 11 "(tree fragment)"
  {(char *)"memview", __pyx_getprop___pyx_array_memview, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_array = {

#line 11 "(tree fragment)"
  __pyx_array___len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_array, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_array = {

#line 11 "(tree fragment)"
  __pyx_array___len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_array___getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyBufferProcs __pyx_tp_as_buffer_array = {

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getreadbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getwritebuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getsegcount*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getcharbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_array_getbuffer, /*bf_getbuffer*/

#line 11 "(tree fragment)"
  0, /*bf_releasebuffer*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_array = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.array", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_array, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  __pyx_tp_getattro_array, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_array, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_array, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_array, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_MemviewEnum_obj *)o);

#line 11 "(tree fragment)"
  p->name = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_Enum(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->name);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  if (p->name) {

#line 11 "(tree fragment)"
    e = (*v)(p->name, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_Enum(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->name);

#line 11 "(tree fragment)"
  p->name = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_Enum[] = {

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_MemviewEnum = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.Enum", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_MemviewEnum___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_Enum, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_Enum, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_Enum, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_MemviewEnum___init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_Enum, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_memoryview_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_memoryview;

#line 11 "(tree fragment)"
  p->obj = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_size = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_array_interface = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->view.obj = NULL;

#line 11 "(tree fragment)"
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) goto bad;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
  bad:

#line 11 "(tree fragment)"
  Py_DECREF(o); o = 0;

#line 11 "(tree fragment)"
  return NULL;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_memoryview(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_memoryview___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->obj);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_size);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_array_interface);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  if (p->obj) {

#line 11 "(tree fragment)"
    e = (*v)(p->obj, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->_size) {

#line 11 "(tree fragment)"
    e = (*v)(p->_size, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->_array_interface) {

#line 11 "(tree fragment)"
    e = (*v)(p->_array_interface, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->view.obj) {

#line 11 "(tree fragment)"
    e = (*v)(p->view.obj, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_memoryview(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->obj);

#line 11 "(tree fragment)"
  p->obj = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_size);

#line 11 "(tree fragment)"
  p->_size = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_array_interface);

#line 11 "(tree fragment)"
  p->_array_interface = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  Py_CLEAR(p->view.obj);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_memoryview___setitem__(o, i, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_Format(PyExc_NotImplementedError,

#line 11 "(tree fragment)"
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_memoryview[] = {

#line 11 "(tree fragment)"
  {"is_c_contig", (PyCFunction)__pyx_memoryview_is_c_contig, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"is_f_contig", (PyCFunction)__pyx_memoryview_is_f_contig, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"copy", (PyCFunction)__pyx_memoryview_copy, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"copy_fortran", (PyCFunction)__pyx_memoryview_copy_fortran, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_memoryview[] = {

#line 11 "(tree fragment)"
  {(char *)"T", __pyx_getprop___pyx_memoryview_T, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"base", __pyx_getprop___pyx_memoryview_base, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"shape", __pyx_getprop___pyx_memoryview_shape, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"strides", __pyx_getprop___pyx_memoryview_strides, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"size", __pyx_getprop___pyx_memoryview_size, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_memoryview = {

#line 11 "(tree fragment)"
  __pyx_memoryview___len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_memoryview, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_memoryview = {

#line 11 "(tree fragment)"
  __pyx_memoryview___len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_memoryview___getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyBufferProcs __pyx_tp_as_buffer_memoryview = {

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getreadbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getwritebuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getsegcount*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getcharbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/

#line 11 "(tree fragment)"
  0, /*bf_releasebuffer*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_memoryview = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform.memoryview", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_memoryview___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  __pyx_memoryview___str__, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_memoryview, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_memoryview, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_memoryview, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_memoryview, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_memoryview, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_memoryviewslice_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;

#line 11 "(tree fragment)"
  p->from_object = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->from_slice.memview = NULL;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_memoryviewslice___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->from_object);

#line 11 "(tree fragment)"
  PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_memoryview(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;

#line 11 "(tree fragment)"
  if (p->from_object) {

#line 11 "(tree fragment)"
    e = (*v)(p->from_object, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear__memoryviewslice(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  __pyx_tp_clear_memoryview(o);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->from_object);

#line 11 "(tree fragment)"
  p->from_object = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  __PYX_XDEC_MEMVIEW(&p->from_slice, 1);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryviewslice_base(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods__memoryviewslice[] = {

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets__memoryviewslice[] = {

#line 11 "(tree fragment)"
  {(char *)"base", __pyx_getprop___pyx_memoryviewslice_base, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_memoryviewslice = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.feature_map.profile_transform._memoryviewslice", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_memoryview___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_memoryview___str__, /*tp_str*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "Internal class for passing memoryview slices to Python", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods__memoryviewslice, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets__memoryviewslice, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new__memoryviewslice, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods[] = {

#line 11 "(tree fragment)"
  {"find_immediate_minima", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_1find_immediate_minima, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_find_immediate_minima},

#line 11 "(tree fragment)"
  {"locate_extrema", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_3locate_extrema, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_2locate_extrema},

#line 11 "(tree fragment)"
  {"binsearch", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_9binsearch, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_8binsearch},

#line 11 "(tree fragment)"
  {"gaussian_smooth", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_11gaussian_smooth, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_10gaussian_smooth},

#line 11 "(tree fragment)"
  {"smooth_leveled", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_13smooth_leveled, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_12smooth_leveled},

#line 11 "(tree fragment)"
  {"_get_total_distance", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_11feature_map_17profile_transform_15_get_total_distance, METH_O, __pyx_doc_12ms_deisotope_2_c_11feature_map_17profile_transform_14_get_total_distance},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
#if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
#if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/

#line 11 "(tree fragment)"
static int __pyx_pymod_exec_profile_transform(PyObject* module); /*proto*/

#line 11 "(tree fragment)"
static PyModuleDef_Slot __pyx_moduledef_slots[] = {

#line 11 "(tree fragment)"
  {Py_mod_create, (void*)__pyx_pymod_create},

#line 11 "(tree fragment)"
  {Py_mod_exec, (void*)__pyx_pymod_exec_profile_transform},

#line 11 "(tree fragment)"
  {0, NULL}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
#endif

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyModuleDef __pyx_moduledef = {

#line 11 "(tree fragment)"
    PyModuleDef_HEAD_INIT,

#line 11 "(tree fragment)"
    "profile_transform",

#line 11 "(tree fragment)"
    0, /* m_doc */

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
    0, /* m_size */

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
    -1, /* m_size */

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
    __pyx_methods /* m_methods */,

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
    __pyx_moduledef_slots, /* m_slots */

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
    NULL, /* m_reload */

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
    NULL, /* m_traverse */

#line 11 "(tree fragment)"
    NULL, /* m_clear */

#line 11 "(tree fragment)"
    NULL /* m_free */

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
#endif

#line 11 "(tree fragment)"
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_ASCII, __pyx_k_ASCII, sizeof(__pyx_k_ASCII), 0, 0, 1, 1},
  {&__pyx_kp_s_Buffer_view_does_not_expose_stri, __pyx_k_Buffer_view_does_not_expose_stri, sizeof(__pyx_k_Buffer_view_does_not_expose_stri), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_only_create_a_buffer_that_is, __pyx_k_Can_only_create_a_buffer_that_is, sizeof(__pyx_k_Can_only_create_a_buffer_that_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_assign_to_read_only_memor, __pyx_k_Cannot_assign_to_read_only_memor, sizeof(__pyx_k_Cannot_assign_to_read_only_memor), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_create_writable_memory_vi, __pyx_k_Cannot_create_writable_memory_vi, sizeof(__pyx_k_Cannot_create_writable_memory_vi), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_index_with_type_s, __pyx_k_Cannot_index_with_type_s, sizeof(__pyx_k_Cannot_index_with_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_Ellipsis, __pyx_k_Ellipsis, sizeof(__pyx_k_Ellipsis), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_shape_tuple_for_cython_arr, __pyx_k_Empty_shape_tuple_for_cython_arr, sizeof(__pyx_k_Empty_shape_tuple_for_cython_arr), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_k_Expected_at_least_d_argument_s_g, sizeof(__pyx_k_Expected_at_least_d_argument_s_g), 0, 0, 1, 0},
  {&__pyx_kp_s_Function_call_with_ambiguous_arg, __pyx_k_Function_call_with_ambiguous_arg, sizeof(__pyx_k_Function_call_with_ambiguous_arg), 0, 0, 1, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_k_Incompatible_checksums_0x_x_vs_0_2, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_3, __pyx_k_Incompatible_checksums_0x_x_vs_0_3, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_3), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_4, __pyx_k_Incompatible_checksums_0x_x_vs_0_4, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_4), 0, 0, 1, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_kp_s_Indirect_dimensions_not_supporte, __pyx_k_Indirect_dimensions_not_supporte, sizeof(__pyx_k_Indirect_dimensions_not_supporte), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_k_Invalid_mode_expected_c_or_fortr, sizeof(__pyx_k_Invalid_mode_expected_c_or_fortr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_k_Invalid_shape_in_axis_d_d, sizeof(__pyx_k_Invalid_shape_in_axis_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_k_MemoryView_of_r_at_0x_x, sizeof(__pyx_k_MemoryView_of_r_at_0x_x), 0, 0, 1, 0},
  {&__pyx_kp_s_MemoryView_of_r_object, __pyx_k_MemoryView_of_r_object, sizeof(__pyx_k_MemoryView_of_r_object), 0, 0, 1, 0},
  {&__pyx_kp_s_No_matching_signature_found, __pyx_k_No_matching_signature_found, sizeof(__pyx_k_No_matching_signature_found), 0, 0, 1, 0},
  {&__pyx_n_b_O, __pyx_k_O, sizeof(__pyx_k_O), 0, 0, 0, 1},
  {&__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_k_Out_of_bounds_on_buffer_access_a, sizeof(__pyx_k_Out_of_bounds_on_buffer_access_a), 0, 0, 1, 0},
  {&__pyx_n_s_PeakBoundary, __pyx_k_PeakBoundary, sizeof(__pyx_k_PeakBoundary), 0, 0, 1, 1},
  {&__pyx_kp_s_PeakBoundary_0_2f_0_2f_0_2f_0_3e, __pyx_k_PeakBoundary_0_2f_0_2f_0_2f_0_3e, sizeof(__pyx_k_PeakBoundary_0_2f_0_2f_0_2f_0_3e), 0, 0, 1, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_ProfileSplitter, __pyx_k_ProfileSplitter, sizeof(__pyx_k_ProfileSplitter), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_convert_item_to_object, __pyx_k_Unable_to_convert_item_to_object, sizeof(__pyx_k_Unable_to_convert_item_to_object), 0, 0, 1, 0},
  {&__pyx_n_s_ValleyPoint, __pyx_k_ValleyPoint, sizeof(__pyx_k_ValleyPoint), 0, 0, 1, 1},
  {&__pyx_kp_s_ValleyPoint_0_2f_0_2f_0_2f_0_3e, __pyx_k_ValleyPoint_0_2f_0_2f_0_2f_0_3e, sizeof(__pyx_k_ValleyPoint_0_2f_0_2f_0_2f_0_3e), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_View_MemoryView, __pyx_k_View_MemoryView, sizeof(__pyx_k_View_MemoryView), 0, 0, 1, 1},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_n_s_allocate_buffer, __pyx_k_allocate_buffer, sizeof(__pyx_k_allocate_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_as_arrays, __pyx_k_as_arrays, sizeof(__pyx_k_as_arrays), 0, 0, 1, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_n_s_case, __pyx_k_case, sizeof(__pyx_k_case), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_concatenate, __pyx_k_concatenate, sizeof(__pyx_k_concatenate), 0, 0, 1, 1},
  {&__pyx_kp_s_contiguous_and_direct, __pyx_k_contiguous_and_direct, sizeof(__pyx_k_contiguous_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_contiguous_and_indirect, __pyx_k_contiguous_and_indirect, sizeof(__pyx_k_contiguous_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_defaults, __pyx_k_defaults, sizeof(__pyx_k_defaults), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_kp_s_double_double, __pyx_k_double_double, sizeof(__pyx_k_double_double), 0, 0, 1, 0},
  {&__pyx_kp_s_double_float, __pyx_k_double_float, sizeof(__pyx_k_double_float), 0, 0, 1, 0},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dtype_is_object, __pyx_k_dtype_is_object, sizeof(__pyx_k_dtype_is_object), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_exp, __pyx_k_exp, sizeof(__pyx_k_exp), 0, 0, 1, 1},
  {&__pyx_n_s_extreme_indices, __pyx_k_extreme_indices, sizeof(__pyx_k_extreme_indices), 0, 0, 1, 1},
  {&__pyx_n_s_find_time, __pyx_k_find_time, sizeof(__pyx_k_find_time), 0, 0, 1, 1},
  {&__pyx_n_s_first_maximum, __pyx_k_first_maximum, sizeof(__pyx_k_first_maximum), 0, 0, 1, 1},
  {&__pyx_n_s_first_maximum_index, __pyx_k_first_maximum_index, sizeof(__pyx_k_first_maximum_index), 0, 0, 1, 1},
  {&__pyx_n_s_first_minimum, __pyx_k_first_minimum, sizeof(__pyx_k_first_minimum), 0, 0, 1, 1},
  {&__pyx_n_s_first_minimum_index, __pyx_k_first_minimum_index, sizeof(__pyx_k_first_minimum_index), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_kp_s_float_double, __pyx_k_float_double, sizeof(__pyx_k_float_double), 0, 0, 1, 0},
  {&__pyx_kp_s_float_float, __pyx_k_float_float, sizeof(__pyx_k_float_float), 0, 0, 1, 0},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 0, 1, 1},
  {&__pyx_n_u_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 1, 0, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get_total_distance, __pyx_k_get_total_distance, sizeof(__pyx_k_get_total_distance), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_k_got_differing_extents_in_dimensi, sizeof(__pyx_k_got_differing_extents_in_dimensi), 0, 0, 1, 0},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 0, 1, 1},
  {&__pyx_n_s_intensity, __pyx_k_intensity, sizeof(__pyx_k_intensity), 0, 0, 1, 1},
  {&__pyx_n_s_interp, __pyx_k_interp, sizeof(__pyx_k_interp), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate, __pyx_k_interpolate, sizeof(__pyx_k_interpolate), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_past, __pyx_k_interpolate_past, sizeof(__pyx_k_interpolate_past), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_point, __pyx_k_interpolate_point, sizeof(__pyx_k_interpolate_point), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_kp_s_itemsize_0_for_cython_array, __pyx_k_itemsize_0_for_cython_array, sizeof(__pyx_k_itemsize_0_for_cython_array), 0, 0, 1, 0},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_level, __pyx_k_level, sizeof(__pyx_k_level), 0, 0, 1, 1},
  {&__pyx_n_s_linspace, __pyx_k_linspace, sizeof(__pyx_k_linspace), 0, 0, 1, 1},
  {&__pyx_n_s_locate_peak_boundaries_locals_la, __pyx_k_locate_peak_boundaries_locals_la, sizeof(__pyx_k_locate_peak_boundaries_locals_la), 0, 0, 1, 1},
  {&__pyx_n_s_locate_valleys, __pyx_k_locate_valleys, sizeof(__pyx_k_locate_valleys), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max, __pyx_k_max, sizeof(__pyx_k_max), 0, 0, 1, 1},
  {&__pyx_n_s_maximum, __pyx_k_maximum, sizeof(__pyx_k_maximum), 0, 0, 1, 1},
  {&__pyx_n_s_maximum_index, __pyx_k_maximum_index, sizeof(__pyx_k_maximum_index), 0, 0, 1, 1},
  {&__pyx_n_s_mean, __pyx_k_mean, sizeof(__pyx_k_mean), 0, 0, 1, 1},
  {&__pyx_n_s_median, __pyx_k_median, sizeof(__pyx_k_median), 0, 0, 1, 1},
  {&__pyx_n_s_memview, __pyx_k_memview, sizeof(__pyx_k_memview), 0, 0, 1, 1},
  {&__pyx_n_s_min, __pyx_k_min, sizeof(__pyx_k_min), 0, 0, 1, 1},
  {&__pyx_n_s_minimum, __pyx_k_minimum, sizeof(__pyx_k_minimum), 0, 0, 1, 1},
  {&__pyx_n_s_minimum_index, __pyx_k_minimum_index, sizeof(__pyx_k_minimum_index), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_k_ms_deisotope__c_feature_map_prof, sizeof(__pyx_k_ms_deisotope__c_feature_map_prof), 0, 0, 1, 1},
  {&__pyx_kp_s_ms_deisotope__c_feature_map_prof_2, __pyx_k_ms_deisotope__c_feature_map_prof_2, sizeof(__pyx_k_ms_deisotope__c_feature_map_prof_2), 0, 0, 1, 0},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_n_levels, __pyx_k_n_levels, sizeof(__pyx_k_n_levels), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_new_xs, __pyx_k_new_xs, sizeof(__pyx_k_new_xs), 0, 0, 1, 1},
  {&__pyx_n_s_new_ys, __pyx_k_new_ys, sizeof(__pyx_k_new_ys), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_nodes, __pyx_k_nodes, sizeof(__pyx_k_nodes), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_out, __pyx_k_out, sizeof(__pyx_k_out), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_pi, __pyx_k_pi, sizeof(__pyx_k_pi), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_profile, __pyx_k_profile, sizeof(__pyx_k_profile), 0, 0, 1, 1},
  {&__pyx_n_s_profiles, __pyx_k_profiles, sizeof(__pyx_k_profiles), 0, 0, 1, 1},
  {&__pyx_n_s_ps, __pyx_k_ps, sizeof(__pyx_k_ps), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_0_0interpolate_point, __pyx_k_pyx_fuse_0_0interpolate_point, sizeof(__pyx_k_pyx_fuse_0_0interpolate_point), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_0_1interpolate_point, __pyx_k_pyx_fuse_0_1interpolate_point, sizeof(__pyx_k_pyx_fuse_0_1interpolate_point), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_0sliding_mean, __pyx_k_pyx_fuse_0sliding_mean, sizeof(__pyx_k_pyx_fuse_0sliding_mean), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_1_0interpolate_point, __pyx_k_pyx_fuse_1_0interpolate_point, sizeof(__pyx_k_pyx_fuse_1_0interpolate_point), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_1_1interpolate_point, __pyx_k_pyx_fuse_1_1interpolate_point, sizeof(__pyx_k_pyx_fuse_1_1interpolate_point), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_fuse_1sliding_mean, __pyx_k_pyx_fuse_1sliding_mean, sizeof(__pyx_k_pyx_fuse_1sliding_mean), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_getbuffer, __pyx_k_pyx_getbuffer, sizeof(__pyx_k_pyx_getbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Enum, __pyx_k_pyx_unpickle_Enum, sizeof(__pyx_k_pyx_unpickle_Enum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PeakBoundary, __pyx_k_pyx_unpickle_PeakBoundary, sizeof(__pyx_k_pyx_unpickle_PeakBoundary), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ProfileSplitter, __pyx_k_pyx_unpickle_ProfileSplitter, sizeof(__pyx_k_pyx_unpickle_ProfileSplitter), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ValleyPoint, __pyx_k_pyx_unpickle_ValleyPoint, sizeof(__pyx_k_pyx_unpickle_ValleyPoint), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_scale, __pyx_k_scale, sizeof(__pyx_k_scale), 0, 0, 1, 1},
  {&__pyx_n_s_second_maximum, __pyx_k_second_maximum, sizeof(__pyx_k_second_maximum), 0, 0, 1, 1},
  {&__pyx_n_s_second_maximum_index, __pyx_k_second_maximum_index, sizeof(__pyx_k_second_maximum_index), 0, 0, 1, 1},
  {&__pyx_n_s_second_minimum, __pyx_k_second_minimum, sizeof(__pyx_k_second_minimum), 0, 0, 1, 1},
  {&__pyx_n_s_second_minimum_index, __pyx_k_second_minimum_index, sizeof(__pyx_k_second_minimum_index), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_signatures, __pyx_k_signatures, sizeof(__pyx_k_signatures), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_sliding_mean, __pyx_k_sliding_mean, sizeof(__pyx_k_sliding_mean), 0, 0, 1, 1},
  {&__pyx_n_s_smooth, __pyx_k_smooth, sizeof(__pyx_k_smooth), 0, 0, 1, 1},
  {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_split_valley, __pyx_k_split_valley, sizeof(__pyx_k_split_valley), 0, 0, 1, 1},
  {&__pyx_n_s_split_valleys, __pyx_k_split_valleys, sizeof(__pyx_k_split_valleys), 0, 0, 1, 1},
  {&__pyx_n_s_split_valleys_locals_genexpr, __pyx_k_split_valleys_locals_genexpr, sizeof(__pyx_k_split_valleys_locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_splitter, __pyx_k_splitter, sizeof(__pyx_k_splitter), 0, 0, 1, 1},
  {&__pyx_n_s_sqrt, __pyx_k_sqrt, sizeof(__pyx_k_sqrt), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_strided_and_direct, __pyx_k_strided_and_direct, sizeof(__pyx_k_strided_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_direct_or_indirect, __pyx_k_strided_and_direct_or_indirect, sizeof(__pyx_k_strided_and_direct_or_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_indirect, __pyx_k_strided_and_indirect, sizeof(__pyx_k_strided_and_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_total_distance, __pyx_k_total_distance, sizeof(__pyx_k_total_distance), 0, 0, 1, 1},
  {&__pyx_kp_s_unable_to_allocate_array_data, __pyx_k_unable_to_allocate_array_data, sizeof(__pyx_k_unable_to_allocate_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_unable_to_allocate_shape_and_str, __pyx_k_unable_to_allocate_shape_and_str, sizeof(__pyx_k_unable_to_allocate_shape_and_str), 0, 0, 1, 0},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_vallies, __pyx_k_vallies, sizeof(__pyx_k_vallies), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_var, __pyx_k_var, sizeof(__pyx_k_var), 0, 0, 1, 1},
  {&__pyx_n_s_width, __pyx_k_width, sizeof(__pyx_k_width), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_xs, __pyx_k_xs, sizeof(__pyx_k_xs), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_ys, __pyx_k_ys, sizeof(__pyx_k_ys), 0, 0, 1, 1},
  {&__pyx_n_s_zeros_like, __pyx_k_zeros_like, sizeof(__pyx_k_zeros_like), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 70, __pyx_L1_error)
  __pyx_builtin_sorted = __Pyx_GetBuiltinName(__pyx_n_s_sorted); if (!__pyx_builtin_sorted) __PYX_ERR(0, 311, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 884, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(1, 133, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(1, 148, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(1, 151, __pyx_L1_error)
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_n_s_Ellipsis); if (!__pyx_builtin_Ellipsis) __PYX_ERR(1, 404, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) __PYX_ERR(1, 613, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(1, 832, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__4);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__4);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__5);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":104
 *     for i in range(n):
 *         if i == 0:
 *             value = np.median(np.concatenate(([0], ys[:2]),))             # <<<<<<<<<<<<<<
 *         elif i == n - 1:
 *             value = np.median(np.concatenate(([0], ys[n - 2:]),))
 */

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_slice__6 = PySlice_New(Py_None, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__6)) __PYX_ERR(0, 104, __pyx_L1_error)

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_slice__6);

#line 104 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_slice__6);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xf456ba5, 0xf92ae08, 0x36c730a):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xf456ba5, 0xf92ae08, 0x36c730a) = (first_minimum, first_minimum_index, maximum, maximum_index, second_minimum, second_minimum_index, total_distance))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_tuple__7 = PyTuple_Pack(3, __pyx_int_256207781, __pyx_int_261271048, __pyx_int_57438986); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__7);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__7);

#line 4 "(tree fragment)"
  __pyx_tuple__8 = PyTuple_Pack(3, __pyx_int_19759364, __pyx_int_107829254, __pyx_int_237309992); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__8);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__8);

#line 4 "(tree fragment)"
  __pyx_tuple__9 = PyTuple_Pack(3, __pyx_int_210030367, __pyx_int_164464950, __pyx_int_261528200); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__9);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 884, __pyx_L1_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_tuple__10);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(2, 890, __pyx_L1_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_tuple__11);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */

#line 133 "View.MemoryView"
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_Empty_shape_tuple_for_cython_arr); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 133, __pyx_L1_error)

#line 133 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__12);

#line 133 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */

#line 136 "View.MemoryView"
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_itemsize_0_for_cython_array); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 136, __pyx_L1_error)

#line 136 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__13);

#line 136 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 148 "View.MemoryView"
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_shape_and_str); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(1, 148, __pyx_L1_error)

#line 148 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__14);

#line 148 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */

#line 176 "View.MemoryView"
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_array_data); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 176, __pyx_L1_error)

#line 176 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__15);

#line 176 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */

#line 192 "View.MemoryView"
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_Can_only_create_a_buffer_that_is); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(1, 192, __pyx_L1_error)

#line 192 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__16);

#line 192 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__17);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__18);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */

#line 418 "View.MemoryView"
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_Cannot_assign_to_read_only_memor); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 418, __pyx_L1_error)

#line 418 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__19);

#line 418 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */

#line 495 "View.MemoryView"
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_convert_item_to_object); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(1, 495, __pyx_L1_error)

#line 495 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__20);

#line 495 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */

#line 520 "View.MemoryView"
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_Cannot_create_writable_memory_vi); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 520, __pyx_L1_error)

#line 520 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__21);

#line 520 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */

#line 570 "View.MemoryView"
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_Buffer_view_does_not_expose_stri); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(1, 570, __pyx_L1_error)

#line 570 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__22);

#line 570 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */

#line 577 "View.MemoryView"
  __pyx_tuple__23 = PyTuple_New(1); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__23);

#line 577 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_neg_1);

#line 577 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_int_neg_1);

#line 577 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_tuple__23, 0, __pyx_int_neg_1);

#line 577 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__24);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__25);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */

#line 682 "View.MemoryView"
  __pyx_slice__26 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__26)) __PYX_ERR(1, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
  __Pyx_GOTREF(__pyx_slice__26);

#line 682 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_slice__26);

  /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 703 "View.MemoryView"
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_Indirect_dimensions_not_supporte); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 703, __pyx_L1_error)

#line 703 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__27);

#line 703 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__28);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__29);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__29);

#line 4 "(tree fragment)"
  __pyx_tuple__30 = PyTuple_Pack(3, __pyx_int_184977713, __pyx_int_136983863, __pyx_int_112105877); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__30);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":64
 * 
 * 
 * def interpolate(xs, ys, n=200):             # <<<<<<<<<<<<<<
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)
 */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__31 = PyTuple_Pack(5, __pyx_n_s_xs, __pyx_n_s_ys, __pyx_n_s_n, __pyx_n_s_new_xs, __pyx_n_s_new_ys); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 64, __pyx_L1_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__31);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__31);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_feature_map_prof_2, __pyx_n_s_interpolate, 64, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 64, __pyx_L1_error)

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_n_s_ys); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__33);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__33);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_feature_map_prof_2, __pyx_n_s_pyx_fuse_0sliding_mean, 70, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__35 = PyTuple_Pack(11, __pyx_n_s_profiles, __pyx_n_s_scale, __pyx_n_s_n_levels, __pyx_n_s_i, __pyx_n_s_out, __pyx_n_s_case, __pyx_n_s_ps, __pyx_n_s_vallies, __pyx_n_s_splitter, __pyx_n_s_genexpr, __pyx_n_s_genexpr); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 396, __pyx_L1_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__35);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__35);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(3, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_feature_map_prof_2, __pyx_n_s_split_valleys, 396, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 396, __pyx_L1_error)

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_tuple__37 = PyTuple_Pack(3, __pyx_n_s_time, __pyx_n_s_intensity, __pyx_n_s_x); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_tuple__37);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_tuple__37);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_feature_map_prof_2, __pyx_n_s_pyx_fuse_0_0interpolate_point, 446, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 446, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeakBoundary(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_tuple__39 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__39);

#line 1 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__39);

#line 1 "(tree fragment)"
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PeakBoundary, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __pyx_tuple__41 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__41);

#line 1 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__41);

#line 1 "(tree fragment)"
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ValleyPoint, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __pyx_tuple__43 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__43);

#line 1 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__43);

#line 1 "(tree fragment)"
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ProfileSplitter, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */

#line 286 "View.MemoryView"
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct_or_indirect); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(1, 286, __pyx_L1_error)

#line 286 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__45);

#line 286 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */

#line 287 "View.MemoryView"
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(1, 287, __pyx_L1_error)

#line 287 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__46);

#line 287 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 288 "View.MemoryView"
  __pyx_tuple__47 = PyTuple_Pack(1, __pyx_kp_s_strided_and_indirect); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(1, 288, __pyx_L1_error)

#line 288 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__47);

#line 288 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */

#line 291 "View.MemoryView"
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_direct); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(1, 291, __pyx_L1_error)

#line 291 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__48);

#line 291 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 292 "View.MemoryView"
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_indirect); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(1, 292, __pyx_L1_error)

#line 292 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__49);

#line 292 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_tuple__50 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__50);

#line 1 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__50);

#line 1 "(tree fragment)"
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Enum, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return 0;

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return -1;

#line 1 "(tree fragment)"
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_1_ = PyFloat_FromDouble(1.); if (unlikely(!__pyx_float_1_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_3 = PyFloat_FromDouble(0.3); if (unlikely(!__pyx_float_0_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_200 = PyInt_FromLong(200); if (unlikely(!__pyx_int_200)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_19759364 = PyInt_FromLong(19759364L); if (unlikely(!__pyx_int_19759364)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_57438986 = PyInt_FromLong(57438986L); if (unlikely(!__pyx_int_57438986)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_107829254 = PyInt_FromLong(107829254L); if (unlikely(!__pyx_int_107829254)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_112105877 = PyInt_FromLong(112105877L); if (unlikely(!__pyx_int_112105877)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_136983863 = PyInt_FromLong(136983863L); if (unlikely(!__pyx_int_136983863)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_164464950 = PyInt_FromLong(164464950L); if (unlikely(!__pyx_int_164464950)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_184977713 = PyInt_FromLong(184977713L); if (unlikely(!__pyx_int_184977713)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_210030367 = PyInt_FromLong(210030367L); if (unlikely(!__pyx_int_210030367)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_237309992 = PyInt_FromLong(237309992L); if (unlikely(!__pyx_int_237309992)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_256207781 = PyInt_FromLong(256207781L); if (unlikely(!__pyx_int_256207781)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_261271048 = PyInt_FromLong(261271048L); if (unlikely(!__pyx_int_261271048)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_261528200 = PyInt_FromLong(261528200L); if (unlikely(!__pyx_int_261528200)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary = &__pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;
  __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.compute_distance = (double (*)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary *))__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_12PeakBoundary_compute_distance;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PeakBoundary, (PyObject *)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary) < 0) __PYX_ERR(0, 178, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary = &__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_PeakBoundary;
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint = &__pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;
  __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.compute_distance = (double (*)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint *))__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_11ValleyPoint_compute_distance;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ValleyPoint, (PyObject *)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint = &__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ValleyPoint;
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter = &__pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;
  __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter._extreme_indices = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, PyObject *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter__extreme_indices;
  __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.interpolate = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_interpolate;
  __pyx_vtable_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.locate_valleys = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_11feature_map_17profile_transform_15ProfileSplitter_locate_valleys;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ProfileSplitter, (PyObject *)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter) < 0) __PYX_ERR(0, 252, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter = &__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform_ProfileSplitter;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys) < 0) __PYX_ERR(0, 396, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys = &__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct__split_valleys;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 404, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr = &__pyx_type_12ms_deisotope_2_c_11feature_map_17profile_transform___pyx_scope_struct_1_genexpr;
  __pyx_vtabptr_array = &__pyx_vtable_array;
  __pyx_vtable_array.get_memview = (PyObject *(*)(struct __pyx_array_obj *))__pyx_array_get_memview;
  if (PyType_Ready(&__pyx_type___pyx_array) < 0) __PYX_ERR(1, 105, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_array.tp_print = 0;
  #endif
  if (__Pyx_SetVtable(__pyx_type___pyx_array.tp_dict, __pyx_vtabptr_array) < 0) __PYX_ERR(1, 105, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_array) < 0) __PYX_ERR(1, 105, __pyx_L1_error)
  __pyx_array_type = &__pyx_type___pyx_array;
  if (PyType_Ready(&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(1, 279, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_MemviewEnum.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_MemviewEnum.tp_dictoffset && __pyx_type___pyx_MemviewEnum.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_MemviewEnum.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(1, 279, __pyx_L1_error)
  __pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  if (PyType_Ready(&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(1, 330, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryview.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryview.tp_dictoffset && __pyx_type___pyx_memoryview.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryview.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryview.tp_dict, __pyx_vtabptr_memoryview) < 0) __PYX_ERR(1, 330, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(1, 330, __pyx_L1_error)
  __pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_type___pyx_memoryviewslice.tp_base = __pyx_memoryview_type;
  if (PyType_Ready(&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(1, 965, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryviewslice.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryviewslice.tp_dictoffset && __pyx_type___pyx_memoryviewslice.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryviewslice.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryviewslice.tp_dict, __pyx_vtabptr__memoryviewslice) < 0) __PYX_ERR(1, 965, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(1, 965, __pyx_L1_error)
  __pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 199, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 222, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 226, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakBase", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase) __PYX_ERR(4, 2, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase)) __PYX_ERR(4, 2, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "FittedPeak", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak) __PYX_ERR(4, 12, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak)) __PYX_ERR(4, 12, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSet", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet) __PYX_ERR(4, 29, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet)) __PYX_ERR(4, 29, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSetIndexed", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed) __PYX_ERR(4, 83, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed)) __PYX_ERR(4, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.feature_map.lcms_feature"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "LCMSFeatureTreeList", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList) __PYX_ERR(5, 4, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeList)) __PYX_ERR(5, 4, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "LCMSFeatureTreeNodeBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase) __PYX_ERR(5, 23, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNodeBase)) __PYX_ERR(5, 23, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "LCMSFeatureTreeNode", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode) __PYX_ERR(5, 44, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeatureTreeNode)) __PYX_ERR(5, 44, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "FeatureBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase) __PYX_ERR(5, 56, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureBase)) __PYX_ERR(5, 56, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "LCMSFeature", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature) __PYX_ERR(5, 73, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_LCMSFeature)) __PYX_ERR(5, 73, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "EmptyFeature", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature) __PYX_ERR(5, 104, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_EmptyFeature)) __PYX_ERR(5, 104, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "FeatureSetIterator", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator) __PYX_ERR(5, 109, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_FeatureSetIterator)) __PYX_ERR(5, 109, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "RunningWeightedAverage", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage) __PYX_ERR(5, 137, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverage)) __PYX_ERR(5, 137, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.feature_map.lcms_feature", "RunningWeightedAverageNeutralMass", sizeof(struct __pyx_obj_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass) __PYX_ERR(5, 154, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass = (struct __pyx_vtabstruct_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_11feature_map_12lcms_feature_RunningWeightedAverageNeutralMass)) __PYX_ERR(5, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INTERVAL_INDEX_SIZE", (void **)&__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.feature_map.shape_fitter"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "gaussian_shape", (void (**)(void))&__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_gaussian_shape, "PyObject *(PyArrayObject *, double, double, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "bigaussian_shape", (void (**)(void))&__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_bigaussian_shape, "PyObject *(PyArrayObject *, double, double, double, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "skewed_gaussian_shape", (void (**)(void))&__pyx_f_12ms_deisotope_2_c_11feature_map_12shape_fitter_skewed_gaussian_shape, "PyObject *(PyArrayObject *, double, double, double, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initprofile_transform(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initprofile_transform(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_profile_transform(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_profile_transform(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_profile_transform(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  static PyThread_type_lock __pyx_t_4[8];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'profile_transform' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_profile_transform(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("profile_transform", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ms_deisotope___c__feature_map__profile_transform) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ms_deisotope._c.feature_map.profile_transform")) {
      if (unlikely(PyDict_SetItemString(modules, "ms_deisotope._c.feature_map.profile_transform", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":8
 * cimport cython
 * cimport numpy as np
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 8 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)

#line 8 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 8 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 8, __pyx_L1_error)

#line 8 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":18
 * 
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 18 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_2 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 18, __pyx_L1_error)

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":64
 * 
 * 
 * def interpolate(xs, ys, n=200):             # <<<<<<<<<<<<<<
 *     new_xs = np.linspace(xs.min(), xs.max(), n)
 *     new_ys = np.interp(new_xs, xs, ys)
 */

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_5interpolate, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate, __pyx_t_1) < 0) __PYX_ERR(0, 64, __pyx_L1_error)

#line 64 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":70
 * 
 * 
 * cpdef sliding_mean(np.ndarray[cython.floating, ndim=1, mode='c'] ys):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
 */

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_21__pyx_fuse_0sliding_mean, 0, __pyx_n_s_pyx_fuse_0sliding_mean, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_empty_tuple);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_float, __pyx_t_3) < 0) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_23__pyx_fuse_1sliding_mean, 0, __pyx_n_s_pyx_fuse_1sliding_mean, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_empty_tuple);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_double, __pyx_t_3) < 0) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __pyx_FusedFunction_New(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_7sliding_mean, 0, __pyx_n_s_sliding_mean, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_empty_tuple);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  ((__pyx_FusedFunctionObject *) __pyx_t_3)->__signatures__ = __pyx_t_1;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = 0;

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sliding_mean, __pyx_t_3) < 0) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":396
 * 
 * 
 * def split_valleys(profiles, scale=0.3, n_levels=2):             # <<<<<<<<<<<<<<
 *     for i in range(n_levels):
 *         out = []
 */

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_17split_valleys, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 396, __pyx_L1_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_split_valleys, __pyx_t_3) < 0) __PYX_ERR(0, 396, __pyx_L1_error)

#line 396 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":446
 * @cython.cdivision(True)
 * @cython.boundscheck(False)
 * cpdef double interpolate_point(floating1[::1] time, floating2[::1] intensity, double x):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, n
 */

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_0_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_33__pyx_fuse_0_0interpolate_point, 0, __pyx_n_s_pyx_fuse_0_0interpolate_point, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_kp_s_float_float, __pyx_t_1) < 0) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_0_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_35__pyx_fuse_0_1interpolate_point, 0, __pyx_n_s_pyx_fuse_0_1interpolate_point, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_kp_s_float_double, __pyx_t_1) < 0) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_1_0__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_37__pyx_fuse_1_0interpolate_point, 0, __pyx_n_s_pyx_fuse_1_0interpolate_point, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_kp_s_double_float, __pyx_t_1) < 0) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_1_1__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_39__pyx_fuse_1_1interpolate_point, 0, __pyx_n_s_pyx_fuse_1_1interpolate_point, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_kp_s_double_double, __pyx_t_1) < 0) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_19interpolate_point, 0, __pyx_n_s_interpolate_point, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  ((__pyx_FusedFunctionObject *) __pyx_t_1)->__signatures__ = __pyx_t_3;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_3 = 0;

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_point, __pyx_t_1) < 0) __PYX_ERR(0, 446, __pyx_L1_error)

#line 446 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeakBoundary(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_27__pyx_unpickle_PeakBoundary, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "(tree fragment)"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PeakBoundary, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_PeakBoundary__set_state(<PeakBoundary> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeakBoundary__set_state(PeakBoundary __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.first_minimum = __pyx_state[0]; __pyx_result.first_minimum_index = __pyx_state[1]; __pyx_result.maximum = __pyx_state[2]; __pyx_result.maximum_index = __pyx_state[3]; __pyx_result.second_minimum = __pyx_state[4]; __pyx_result.second_minimum_index = __pyx_state[5]; __pyx_result.total_distance = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_29__pyx_unpickle_ValleyPoint, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 11 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 11 "(tree fragment)"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ValleyPoint, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)

#line 11 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ProfileSplitter(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_11feature_map_17profile_transform_31__pyx_unpickle_ProfileSplitter, NULL, __pyx_n_s_ms_deisotope__c_feature_map_prof); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "(tree fragment)"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ProfileSplitter, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/feature_map/profile_transform.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * #
 * 
 */

#line 1 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)

#line 1 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)

#line 1 "ms_deisotope/_c/feature_map/profile_transform.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":209
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
 */

#line 209 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_array_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 209, __pyx_L1_error)

#line 209 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 209 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_array_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(1, 209, __pyx_L1_error)

#line 209 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 209 "View.MemoryView"
  PyType_Modified(__pyx_array_type);

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */

#line 286 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 286, __pyx_L1_error)

#line 286 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 286 "View.MemoryView"
  __Pyx_XGOTREF(generic);

#line 286 "View.MemoryView"
  __Pyx_DECREF_SET(generic, __pyx_t_1);

#line 286 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 286 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */

#line 287 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 287, __pyx_L1_error)

#line 287 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 287 "View.MemoryView"
  __Pyx_XGOTREF(strided);

#line 287 "View.MemoryView"
  __Pyx_DECREF_SET(strided, __pyx_t_1);

#line 287 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 287 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 288 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 288, __pyx_L1_error)

#line 288 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 288 "View.MemoryView"
  __Pyx_XGOTREF(indirect);

#line 288 "View.MemoryView"
  __Pyx_DECREF_SET(indirect, __pyx_t_1);

#line 288 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 288 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */

#line 291 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__48, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 291, __pyx_L1_error)

#line 291 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 291 "View.MemoryView"
  __Pyx_XGOTREF(contiguous);

#line 291 "View.MemoryView"
  __Pyx_DECREF_SET(contiguous, __pyx_t_1);

#line 291 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 291 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 292 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 292, __pyx_L1_error)

#line 292 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 292 "View.MemoryView"
  __Pyx_XGOTREF(indirect_contiguous);

#line 292 "View.MemoryView"
  __Pyx_DECREF_SET(indirect_contiguous, __pyx_t_1);

#line 292 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 292 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":316
 * 
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0             # <<<<<<<<<<<<<<
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [
 *     PyThread_allocate_lock(),
 */

#line 316 "View.MemoryView"
  __pyx_memoryview_thread_locks_used = 0;

  /* "View.MemoryView":317
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [             # <<<<<<<<<<<<<<
 *     PyThread_allocate_lock(),
 *     PyThread_allocate_lock(),
 */

#line 317 "View.MemoryView"
  __pyx_t_4[0] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[1] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[2] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[3] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[4] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[5] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[6] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_4[7] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  memcpy(&(__pyx_memoryview_thread_locks[0]), __pyx_t_4, sizeof(__pyx_memoryview_thread_locks[0]) * (8));

  /* "View.MemoryView":549
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 549 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 549, __pyx_L1_error)

#line 549 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 549 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_memoryview_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(1, 549, __pyx_L1_error)

#line 549 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 549 "View.MemoryView"
  PyType_Modified(__pyx_memoryview_type);

  /* "View.MemoryView":995
 *         return self.from_object
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 995 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 995, __pyx_L1_error)

#line 995 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 995 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_memoryviewslice_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(1, 995, __pyx_L1_error)

#line 995 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 995 "View.MemoryView"
  PyType_Modified(__pyx_memoryviewslice_type);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum, NULL, __pyx_n_s_View_MemoryView); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "(tree fragment)"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Enum, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /*--- Wrapped vars code ---*/

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_3);

#line 11 "(tree fragment)"
  if (__pyx_m) {

#line 11 "(tree fragment)"
    if (__pyx_d) {

#line 11 "(tree fragment)"
      __Pyx_AddTraceback("init ms_deisotope._c.feature_map.profile_transform", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    Py_CLEAR(__pyx_m);

#line 11 "(tree fragment)"
  } else if (!PyErr_Occurred()) {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_ImportError, "init ms_deisotope._c.feature_map.profile_transform");

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
  return (__pyx_m != NULL) ? 0 : -1;

#line 11 "(tree fragment)"
  #elif PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  return __pyx_m;

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  return;

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
}

#line 1 "cython_utility"

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BufferIndexError */
  static void __Pyx_RaiseBufferIndexError(int axis) {
  PyErr_Format(PyExc_IndexError,
     "Out of bounds on buffer access (axis %d)", axis);
}

/* PyDictVersioning */
  #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
  #if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseArgTupleInvalid */
  static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
  static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
  static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
  static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* GetItemInt */
  static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* DictGetItem */
  #if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* UnicodeAsUCS4 */
  static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
  static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", Py_TYPE(c)->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* SetItemInt */
  static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* IterFinish */
  static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectGetMethod */
  static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
  static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
  static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
  static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
  static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
  static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
  static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyObjectCall2Args */
  static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* BufferFallbackError */
  static void __Pyx_RaiseBufferFallbackError(void) {
  PyErr_SetString(PyExc_ValueError,
     "Buffer acquisition failed on assignment; and then reacquiring the old buffer failed too!");
}

/* ObjectGetItem */
  #if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyFloat_DivideCObj_ZeroDivisionError(operand) if (unlikely(zerodivision_check && ((operand) == 0))) {\
    PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero");\
    return NULL;\
}
static PyObject* __Pyx_PyFloat_DivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
        __Pyx_PyFloat_DivideCObj_ZeroDivisionError(b)
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        __Pyx_PyFloat_DivideCObj_ZeroDivisionError(b)
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: __Pyx_PyFloat_DivideCObj_ZeroDivisionError(0) break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
            __Pyx_PyFloat_DivideCObj_ZeroDivisionError(b)
        }
    } else {
        return (inplace ? __Pyx_PyNumber_InPlaceDivide(op1, op2) : __Pyx_PyNumber_Divide(op1, op2));
    }
        __Pyx_PyFloat_DivideCObj_ZeroDivisionError(b)
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyIntCompare */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyIntBinop */
    #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* SliceObject */
    static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
    #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* PyErrExceptionMatches */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
    static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
    static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* FetchCommonType */
    static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
    #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
    static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* None */
    static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* None */
    static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* MemviewSliceInit */
    static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (unlikely(memviewslice->memview || memviewslice->data)) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF(memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}
#ifndef Py_NO_RETURN
#define Py_NO_RETURN
#endif
static void __pyx_fatalerror(const char *fmt, ...) Py_NO_RETURN {
    va_list vargs;
    char msg[200];
#ifdef HAVE_STDARG_PROTOTYPES
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    va_end(vargs);
    Py_FatalError(msg);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    int first_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None))
        return;
    if (unlikely(__pyx_get_slice_count(memview) < 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    first_time = __pyx_add_acquisition_count(memview) == 0;
    if (unlikely(first_time)) {
        if (have_gil) {
            Py_INCREF((PyObject *) memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_INCREF((PyObject *) memview);
            PyGILState_Release(_gilstate);
        }
    }
}
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    int last_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None)) {
        memslice->memview = NULL;
        return;
    }
    if (unlikely(__pyx_get_slice_count(memview) <= 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    last_time = __pyx_sub_acquisition_count(memview) == 1;
    memslice->data = NULL;
    if (unlikely(last_time)) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        memslice->memview = NULL;
    }
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
    static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* GetTopmostException */
    #if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* DivInt[Py_ssize_t] */
    static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* decode_c_string */
    static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* SwapException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* DivInt[long] */
    static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* PyObject_GenericGetAttrNoDict */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
    static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
    static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* FusedFunction */
    static PyObject *
__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                        PyObject *qualname, PyObject *closure,
                        PyObject *module, PyObject *globals,
                        PyObject *code)
{
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_FusedFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        __pyx_FusedFunctionObject *fusedfunc = (__pyx_FusedFunctionObject *) op;
        fusedfunc->__signatures__ = NULL;
        fusedfunc->type = NULL;
        fusedfunc->self = NULL;
        PyObject_GC_Track(op);
    }
    return op;
}
static void
__pyx_FusedFunction_dealloc(__pyx_FusedFunctionObject *self)
{
    PyObject_GC_UnTrack(self);
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    __Pyx__CyFunction_dealloc((__pyx_CyFunctionObject *) self);
}
static int
__pyx_FusedFunction_traverse(__pyx_FusedFunctionObject *self,
                             visitproc visit,
                             void *arg)
{
    Py_VISIT(self->self);
    Py_VISIT(self->type);
    Py_VISIT(self->__signatures__);
    return __Pyx_CyFunction_traverse((__pyx_CyFunctionObject *) self, visit, arg);
}
static int
__pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self)
{
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    return __Pyx_CyFunction_clear((__pyx_CyFunctionObject *) self);
}
static PyObject *
__pyx_FusedFunction_descr_get(PyObject *self, PyObject *obj, PyObject *type)
{
    __pyx_FusedFunctionObject *func, *meth;
    func = (__pyx_FusedFunctionObject *) self;
    if (func->self || func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(self);
        return self;
    }
    if (obj == Py_None)
        obj = NULL;
    meth = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_New(
                    ((PyCFunctionObject *) func)->m_ml,
                    ((__pyx_CyFunctionObject *) func)->flags,
                    ((__pyx_CyFunctionObject *) func)->func_qualname,
                    ((__pyx_CyFunctionObject *) func)->func_closure,
                    ((PyCFunctionObject *) func)->m_module,
                    ((__pyx_CyFunctionObject *) func)->func_globals,
                    ((__pyx_CyFunctionObject *) func)->func_code);
    if (!meth)
        return NULL;
    if (func->func.defaults) {
        PyObject **pydefaults;
        int i;
        if (!__Pyx_CyFunction_InitDefaults((PyObject*)meth,
                                      func->func.defaults_size,
                                      func->func.defaults_pyobjects)) {
            Py_XDECREF((PyObject*)meth);
            return NULL;
        }
        memcpy(meth->func.defaults, func->func.defaults, func->func.defaults_size);
        pydefaults = __Pyx_CyFunction_Defaults(PyObject *, meth);
        for (i = 0; i < meth->func.defaults_pyobjects; i++)
            Py_XINCREF(pydefaults[i]);
    }
    Py_XINCREF(func->func.func_classobj);
    meth->func.func_classobj = func->func.func_classobj;
    Py_XINCREF(func->__signatures__);
    meth->__signatures__ = func->__signatures__;
    Py_XINCREF(type);
    meth->type = type;
    Py_XINCREF(func->func.defaults_tuple);
    meth->func.defaults_tuple = func->func.defaults_tuple;
    if (func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD)
        obj = type;
    Py_XINCREF(obj);
    meth->self = obj;
    return (PyObject *) meth;
}
static PyObject *
_obj_to_str(PyObject *obj)
{
    if (PyType_Check(obj))
        return PyObject_GetAttr(obj, __pyx_n_s_name_2);
    else
        return PyObject_Str(obj);
}
static PyObject *
__pyx_FusedFunction_getitem(__pyx_FusedFunctionObject *self, PyObject *idx)
{
    PyObject *signature = NULL;
    PyObject *unbound_result_func;
    PyObject *result_func = NULL;
    if (self->__signatures__ == NULL) {
        PyErr_SetString(PyExc_TypeError, "Function is not fused");
        return NULL;
    }
    if (PyTuple_Check(idx)) {
        PyObject *list = PyList_New(0);
        Py_ssize_t n = PyTuple_GET_SIZE(idx);
        PyObject *sep = NULL;
        int i;
        if (unlikely(!list))
            return NULL;
        for (i = 0; i < n; i++) {
            int ret;
            PyObject *string;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(idx, i);
#else
            PyObject *item = PySequence_ITEM(idx, i);  if (unlikely(!item)) goto __pyx_err;
#endif
            string = _obj_to_str(item);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
            Py_DECREF(item);
#endif
            if (unlikely(!string)) goto __pyx_err;
            ret = PyList_Append(list, string);
            Py_DECREF(string);
            if (unlikely(ret < 0)) goto __pyx_err;
        }
        sep = PyUnicode_FromString("|");
        if (likely(sep))
            signature = PyUnicode_Join(sep, list);
__pyx_err:
;
        Py_DECREF(list);
        Py_XDECREF(sep);
    } else {
        signature = _obj_to_str(idx);
    }
    if (!signature)
        return NULL;
    unbound_result_func = PyObject_GetItem(self->__signatures__, signature);
    if (unbound_result_func) {
        if (self->self || self->type) {
            __pyx_FusedFunctionObject *unbound = (__pyx_FusedFunctionObject *) unbound_result_func;
            Py_CLEAR(unbound->func.func_classobj);
            Py_XINCREF(self->func.func_classobj);
            unbound->func.func_classobj = self->func.func_classobj;
            result_func = __pyx_FusedFunction_descr_get(unbound_result_func,
                                                        self->self, self->type);
        } else {
            result_func = unbound_result_func;
            Py_INCREF(result_func);
        }
    }
    Py_DECREF(signature);
    Py_XDECREF(unbound_result_func);
    return result_func;
}
static PyObject *
__pyx_FusedFunction_callfunction(PyObject *func, PyObject *args, PyObject *kw)
{
     __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    int static_specialized = (cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD &&
                              !((__pyx_FusedFunctionObject *) func)->__signatures__);
    if (cyfunc->flags & __Pyx_CYFUNCTION_CCLASS && !static_specialized) {
        return __Pyx_CyFunction_CallAsMethod(func, args, kw);
    } else {
        return __Pyx_CyFunction_Call(func, args, kw);
    }
}
static PyObject *
__pyx_FusedFunction_call(PyObject *func, PyObject *args, PyObject *kw)
{
    __pyx_FusedFunctionObject *binding_func = (__pyx_FusedFunctionObject *) func;
    Py_ssize_t argc = PyTuple_GET_SIZE(args);
    PyObject *new_args = NULL;
    __pyx_FusedFunctionObject *new_func = NULL;
    PyObject *result = NULL;
    PyObject *self = NULL;
    int is_staticmethod = binding_func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD;
    int is_classmethod = binding_func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD;
    if (binding_func->self) {
        Py_ssize_t i;
        new_args = PyTuple_New(argc + 1);
        if (!new_args)
            return NULL;
        self = binding_func->self;
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_INCREF(self);
#endif
        Py_INCREF(self);
        PyTuple_SET_ITEM(new_args, 0, self);
        for (i = 0; i < argc; i++) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(args, i);
            Py_INCREF(item);
#else
            PyObject *item = PySequence_ITEM(args, i);  if (unlikely(!item)) goto bad;
#endif
            PyTuple_SET_ITEM(new_args, i + 1, item);
        }
        args = new_args;
    } else if (binding_func->type) {
        if (argc < 1) {
            PyErr_SetString(PyExc_TypeError, "Need at least one argument, 0 given.");
            return NULL;
        }
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        self = PyTuple_GET_ITEM(args, 0);
#else
        self = PySequence_ITEM(args, 0);  if (unlikely(!self)) return NULL;
#endif
    }
    if (self && !is_classmethod && !is_staticmethod) {
        int is_instance = PyObject_IsInstance(self, binding_func->type);
        if (unlikely(!is_instance)) {
            PyErr_Format(PyExc_TypeError,
                         "First argument should be of type %.200s, got %.200s.",
                         ((PyTypeObject *) binding_func->type)->tp_name,
                         Py_TYPE(self)->tp_name);
            goto bad;
        } else if (unlikely(is_instance == -1)) {
            goto bad;
        }
    }
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
    self = NULL;
#endif
    if (binding_func->__signatures__) {
        PyObject *tup;
        if (is_staticmethod && binding_func->func.flags & __Pyx_CYFUNCTION_CCLASS) {
            tup = PyTuple_Pack(3, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __Pyx_CyFunction_CallMethod(
                func, binding_func->__signatures__, tup, NULL);
        } else {
            tup = PyTuple_Pack(4, binding_func->__signatures__, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_callfunction(func, tup, NULL);
        }
        Py_DECREF(tup);
        if (unlikely(!new_func))
            goto bad;
        Py_XINCREF(binding_func->func.func_classobj);
        Py_CLEAR(new_func->func.func_classobj);
        new_func->func.func_classobj = binding_func->func.func_classobj;
        func = (PyObject *) new_func;
    }
    result = __pyx_FusedFunction_callfunction(func, args, kw);
bad:
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
#endif
    Py_XDECREF(new_args);
    Py_XDECREF((PyObject *) new_func);
    return result;
}
static PyMemberDef __pyx_FusedFunction_members[] = {
    {(char *) "__signatures__",
     T_OBJECT,
     offsetof(__pyx_FusedFunctionObject, __signatures__),
     READONLY,
     0},
    {0, 0, 0, 0, 0},
};
static PyMappingMethods __pyx_FusedFunction_mapping_methods = {
    0,
    (binaryfunc) __pyx_FusedFunction_getitem,
    0,
};
static PyTypeObject __pyx_FusedFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "fused_cython_function",
    sizeof(__pyx_FusedFunctionObject),
    0,
    (destructor) __pyx_FusedFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    0,
    0,
    0,
    &__pyx_FusedFunction_mapping_methods,
    0,
    (ternaryfunc) __pyx_FusedFunction_call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __pyx_FusedFunction_traverse,
    (inquiry) __pyx_FusedFunction_clear,
    0,
    0,
    0,
    0,
    0,
    __pyx_FusedFunction_members,
    __pyx_CyFunction_getsets,
    &__pyx_CyFunctionType_type,
    0,
    __pyx_FusedFunction_descr_get,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_FusedFunction_init(void) {
    __pyx_FusedFunctionType_type.tp_base = __pyx_CyFunctionType;
    __pyx_FusedFunctionType = __Pyx_FetchCommonType(&__pyx_FusedFunctionType_type);
    if (__pyx_FusedFunctionType == NULL) {
        return -1;
    }
    return 0;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_array_type)) return __pyx_array_getbuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_memoryview_type)) return __pyx_memoryview_getbuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


    /* MemviewSliceIsContig */
    static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs.memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs.suboffsets[index] >= 0 || mvs.strides[index] != itemsize)
            return 0;
        itemsize *= mvs.shape[index];
    }
    return 1;
}

/* OverlappingSlices */
    static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

/* Capsule */
    static CYTHON_INLINE PyObject *
__pyx_capsule_create(void *p, CYTHON_UNUSED const char *sig)
{
    PyObject *cobj;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, NULL);
#else
    cobj = PyCObject_FromVoidPtr(p, NULL);
#endif
    return cobj;
}

/* TypeInfoCompare */
    static int
__pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b)
{
    int i;
    if (!a || !b)
        return 0;
    if (a == b)
        return 1;
    if (a->size != b->size || a->typegroup != b->typegroup ||
            a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
        if (a->typegroup == 'H' || b->typegroup == 'H') {
            return a->size == b->size;
        } else {
            return 0;
        }
    }
    if (a->ndim) {
        for (i = 0; i < a->ndim; i++)
            if (a->arraysize[i] != b->arraysize[i])
                return 0;
    }
    if (a->typegroup == 'S') {
        if (a->flags != b->flags)
            return 0;
        if (a->fields || b->fields) {
            if (!(a->fields && b->fields))
                return 0;
            for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                __Pyx_StructField *field_a = a->fields + i;
                __Pyx_StructField *field_b = b->fields + i;
                if (field_a->offset != field_b->offset ||
                    !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                    return 0;
            }
            return !a->fields[i].type && !b->fields[i].type;
        }
    }
    return 1;
}

/* MemviewSliceValidateAndInit */
    static int
__pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
{
    if (buf->shape[dim] <= 1)
        return 1;
    if (buf->strides) {
        if (spec & __Pyx_MEMVIEW_CONTIG) {
            if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                if (unlikely(buf->strides[dim] != sizeof(void *))) {
                    PyErr_Format(PyExc_ValueError,
                                 "Buffer is not indirectly contiguous "
                                 "in dimension %d.", dim);
                    goto fail;
                }
            } else if (unlikely(buf->strides[dim] != buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
        if (spec & __Pyx_MEMVIEW_FOLLOW) {
            Py_ssize_t stride = buf->strides[dim];
            if (stride < 0)
                stride = -stride;
            if (unlikely(stride < buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
    } else {
        if (unlikely(spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1)) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not contiguous in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(spec & (__Pyx_MEMVIEW_PTR))) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not indirect in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(buf->suboffsets)) {
            PyErr_SetString(PyExc_ValueError,
                            "Buffer exposes suboffsets but no strides");
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec)
{
    if (spec & __Pyx_MEMVIEW_DIRECT) {
        if (unlikely(buf->suboffsets && buf->suboffsets[dim] >= 0)) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer not compatible with direct access "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    if (spec & __Pyx_MEMVIEW_PTR) {
        if (unlikely(!buf->suboffsets || (buf->suboffsets[dim] < 0))) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer is not indirectly accessible "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
{
    int i;
    if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = 0; i < ndim; i++) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not fortran contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = ndim - 1; i >- 1; i--) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not C contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    }
    return 1;
fail:
    return 0;
}
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj)
{
    struct __pyx_memoryview_obj *memview, *new_memview;
    __Pyx_RefNannyDeclarations
    Py_buffer *buf;
    int i, spec = 0, retval = -1;
    __Pyx_BufFmt_Context ctx;
    int from_memoryview = __pyx_memoryview_check(original_obj);
    __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
    if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                            original_obj)->typeinfo)) {
        memview = (struct __pyx_memoryview_obj *) original_obj;
        new_memview = NULL;
    } else {
        memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                            original_obj, buf_flags, 0, dtype);
        new_memview = memview;
        if (unlikely(!memview))
            goto fail;
    }
    buf = &memview->view;
    if (unlikely(buf->ndim != ndim)) {
        PyErr_Format(PyExc_ValueError,
                "Buffer has wrong number of dimensions (expected %d, got %d)",
                ndim, buf->ndim);
        goto fail;
    }
    if (new_memview) {
        __Pyx_BufFmt_Init(&ctx, stack, dtype);
        if (unlikely(!__Pyx_BufFmt_CheckString(&ctx, buf->format))) goto fail;
    }
    if (unlikely((unsigned) buf->itemsize != dtype->size)) {
        PyErr_Format(PyExc_ValueError,
                     "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                     "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                     buf->itemsize,
                     (buf->itemsize > 1) ? "s" : "",
                     dtype->name,
                     dtype->size,
                     (dtype->size > 1) ? "s" : "");
        goto fail;
    }
    if (buf->len > 0) {
        for (i = 0; i < ndim; i++) {
            spec = axes_specs[i];
            if (unlikely(!__pyx_check_strides(buf, i, ndim, spec)))
                goto fail;
            if (unlikely(!__pyx_check_suboffsets(buf, i, ndim, spec)))
                goto fail;
        }
        if (unlikely(buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag)))
            goto fail;
    }
    if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                         new_memview != NULL) == -1)) {
        goto fail;
    }
    retval = 0;
    goto no_fail;
fail:
    Py_XDECREF(new_memview);
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

/* ObjectToMemviewSlice */
    static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* ObjectToMemviewSlice */
    static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* ObjectToMemviewSlice */
    static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_float(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT) | writable_flag, 1,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* ObjectToMemviewSlice */
    static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_double(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT) | writable_flag, 1,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
    #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
    #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
    #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
    #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* MemviewSliceCopyTemplate */
    static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object)
{
    __Pyx_RefNannyDeclarations
    int i;
    __Pyx_memviewslice new_mvs = { 0, 0, { 0 }, { 0 }, { 0 } };
    struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
    Py_buffer *buf = &from_memview->view;
    PyObject *shape_tuple = NULL;
    PyObject *temp_int = NULL;
    struct __pyx_array_obj *array_obj = NULL;
    struct __pyx_memoryview_obj *memview_obj = NULL;
    __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
    for (i = 0; i < ndim; i++) {
        if (unlikely(from_mvs->suboffsets[i] >= 0)) {
            PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                           "indirect dimensions (axis %d)", i);
            goto fail;
        }
    }
    shape_tuple = PyTuple_New(ndim);
    if (unlikely(!shape_tuple)) {
        goto fail;
    }
    __Pyx_GOTREF(shape_tuple);
    for(i = 0; i < ndim; i++) {
        temp_int = PyInt_FromSsize_t(from_mvs->shape[i]);
        if(unlikely(!temp_int)) {
            goto fail;
        } else {
            PyTuple_SET_ITEM(shape_tuple, i, temp_int);
            temp_int = NULL;
        }
    }
    array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, (char *) mode, NULL);
    if (unlikely(!array_obj)) {
        goto fail;
    }
    __Pyx_GOTREF(array_obj);
    memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                    (PyObject *) array_obj, contig_flag,
                                    dtype_is_object,
                                    from_mvs->memview->typeinfo);
    if (unlikely(!memview_obj))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
        goto fail;
    if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                dtype_is_object) < 0))
        goto fail;
    goto no_fail;
fail:
    __Pyx_XDECREF(new_mvs.memview);
    new_mvs.memview = NULL;
    new_mvs.data = NULL;
no_fail:
    __Pyx_XDECREF(shape_tuple);
    __Pyx_XDECREF(temp_int);
    __Pyx_XDECREF(array_obj);
    __Pyx_RefNannyFinishContext();
    return new_mvs;
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int64(npy_int64 value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_int64 neg_one = (npy_int64) -1, const_zero = (npy_int64) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_int64) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int64) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_int64) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_int64) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_int64) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_int64),
                                     little, !is_unsigned);
    }
}

/* BytesContains */
    static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character) {
    const Py_ssize_t length = PyBytes_GET_SIZE(bytes);
    char* char_start = PyBytes_AS_STRING(bytes);
    return memchr(char_start, (unsigned char)character, (size_t)length) != NULL;
}

/* ImportNumPyArray */
    static PyObject* __Pyx__ImportNumPyArray(void) {
    PyObject *numpy_module, *ndarray_object = NULL;
    numpy_module = __Pyx_Import(__pyx_n_s_numpy, NULL, 0);
    if (likely(numpy_module)) {
        ndarray_object = PyObject_GetAttrString(numpy_module, "ndarray");
        Py_DECREF(numpy_module);
    }
    if (unlikely(!ndarray_object)) {
        PyErr_Clear();
    }
    if (unlikely(!ndarray_object || !PyObject_TypeCheck(ndarray_object, &PyType_Type))) {
        Py_XDECREF(ndarray_object);
        Py_INCREF(Py_None);
        ndarray_object = Py_None;
    }
    return ndarray_object;
}
static CYTHON_INLINE PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void) {
    if (unlikely(!__pyx_numpy_ndarray)) {
        __pyx_numpy_ndarray = __Pyx__ImportNumPyArray();
    }
    Py_INCREF(__pyx_numpy_ndarray);
    return __pyx_numpy_ndarray;
}

/* CIntFromPy */
    static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const char neg_one = (char) -1, const_zero = (char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, digits[0])
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) ((((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) ((((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) ((((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

/* PyObjectCallMethod1 */
    static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
    #include <structmember.h>
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
    static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
    #ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
    static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
    #ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
    #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
