/* Generated by Cython 0.29.30 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/arrayobject.h",
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include/numpy/ufuncobject.h"
        ],
        "include_dirs": [
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/core/include",
            "/private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/ms_peak_picker/_c"
        ],
        "name": "ms_deisotope._c.peak_set",
        "sources": [
            "ms_deisotope/_c/peak_set.pyx"
        ]
    },
    "module_name": "ms_deisotope._c.peak_set"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_30"
#define CYTHON_HEX_VERSION 0x001D1EF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ms_deisotope___c__peak_set
#define __PYX_HAVE_API__ms_deisotope___c__peak_set
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include <math.h>
#include <stdlib.h>
#include "pythread.h"
#include "pystate.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "ms_deisotope/_c/peak_set.pyx",
  "ms_deisotope/_c/peak_set.pxd",
  "__init__.pxd",
  "stringsource",
  "composition.pxd",
  "isotopic_distribution.pxd",
  "peak_set.pxd",
  "type.pxd",
  "peak_index.pxd",
  "ms_deisotope/_c/averagine.pxd",
  "ms_deisotope/_c/deconvoluter_base.pxd",
  "ms_deisotope/_c/scoring.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* Atomics.proto */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __pyx_atomic_int_type int
#if CYTHON_ATOMICS && __GNUC__ >= 4 && (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL >= 2)) &&\
                    !defined(__i386__)
    #define __pyx_atomic_incr_aligned(value, lock) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_aligned(value, lock) __sync_fetch_and_sub(value, 1)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER) && 0
    #include <Windows.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type LONG
    #define __pyx_atomic_incr_aligned(value, lock) InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#elif CYTHON_ATOMICS && (defined(__ICC) || defined(__INTEL_COMPILER)) && 0
    #define __pyx_atomic_incr_aligned(value, lock) _InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) _InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using Intel atomics"
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* MemviewSliceStruct.proto */
struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;
#define __Pyx_MemoryView_Len(m)  (m.shape[0])


/* "brainpy/_c/composition.pxd":3
 * from brainpy.mass_dict import nist_mass as __nist_mass
 * 
 * ctypedef long count_type             # <<<<<<<<<<<<<<
 * 
 * cdef dict nist_mass
 */

#line 3 "brainpy/_c/composition.pxd"
typedef long __pyx_t_7brainpy_2_c_11composition_count_type;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":689
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */

#line 689 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":690
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */

#line 690 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":691
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */

#line 691 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */

#line 692 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":696
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */

#line 696 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":697
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */

#line 697 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":698
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */

#line 698 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */

#line 699 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":703
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */

#line 703 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":704
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */

#line 704 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":713
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */

#line 713 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":714
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */

#line 714 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":715
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */

#line 715 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":717
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */

#line 717 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":718
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */

#line 718 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":719
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */

#line 719 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":721
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */

#line 721 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":722
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */

#line 722 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":724
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */

#line 724 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":725
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */

#line 725 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":726
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */

#line 726 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition;
struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet;
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;
struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern;
struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase;
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter;
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__;
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr;
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass;
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio;
struct __pyx_t_7brainpy_2_c_11composition_Isotope;
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap;
struct __pyx_t_7brainpy_2_c_11composition_Element;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket;
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable;
struct __pyx_t_7brainpy_2_c_11composition_Composition;
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl;
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop;

/* "brainpy/_c/composition.pxd":10
 * cdef double PROTON = nist_mass["H+"][0][0]
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 */

#line 10 "brainpy/_c/composition.pxd"
struct __pyx_opt_args_7brainpy_2_c_11composition_neutral_mass {

#line 10 "brainpy/_c/composition.pxd"
  int __pyx_n;

#line 10 "brainpy/_c/composition.pxd"
  double charge_carrier;

#line 10 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":11
 * 
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*) nogil
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * cdef char* _parse_isotope_string(char* label, int* isotope_num, char* element_name) nogil
 * 
 */

#line 11 "brainpy/_c/composition.pxd"
struct __pyx_opt_args_7brainpy_2_c_11composition_mass_charge_ratio {

#line 11 "brainpy/_c/composition.pxd"
  int __pyx_n;

#line 11 "brainpy/_c/composition.pxd"
  double charge_carrier;

#line 11 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":17
 * # Isotope and IsotopeMap Declarations
 * 
 * cdef struct Isotope:             # <<<<<<<<<<<<<<
 *     double mass
 *     double abundance
 */

#line 17 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_Isotope {

#line 17 "brainpy/_c/composition.pxd"
  double mass;

#line 17 "brainpy/_c/composition.pxd"
  double abundance;

#line 17 "brainpy/_c/composition.pxd"
  int neutrons;

#line 17 "brainpy/_c/composition.pxd"
  int neutron_shift;

#line 17 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":23
 *     int neutron_shift
 * 
 * cdef struct IsotopeMap:             # <<<<<<<<<<<<<<
 *     Isotope* bins
 *     size_t size
 */

#line 23 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap {

#line 23 "brainpy/_c/composition.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Isotope *bins;

#line 23 "brainpy/_c/composition.pxd"
  size_t size;

#line 23 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":36
 * # Element Declarations
 * 
 * cdef struct Element:             # <<<<<<<<<<<<<<
 *     char* symbol
 *     IsotopeMap* isotopes
 */

#line 36 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_Element {

#line 36 "brainpy/_c/composition.pxd"
  char *symbol;

#line 36 "brainpy/_c/composition.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_IsotopeMap *isotopes;

#line 36 "brainpy/_c/composition.pxd"
  int monoisotopic_isotope_index;

#line 36 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":56
 * # ElementHashTable and ElementHashBucket Declarations
 * 
 * cdef struct ElementHashBucket:             # <<<<<<<<<<<<<<
 *     Element** elements
 *     size_t used
 */

#line 56 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket {

#line 56 "brainpy/_c/composition.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Element **elements;

#line 56 "brainpy/_c/composition.pxd"
  size_t used;

#line 56 "brainpy/_c/composition.pxd"
  size_t size;

#line 56 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":65
 * 
 * 
 * cdef struct ElementHashTable:             # <<<<<<<<<<<<<<
 *     ElementHashBucket* buckets
 *     size_t size
 */

#line 65 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable {

#line 65 "brainpy/_c/composition.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_ElementHashBucket *buckets;

#line 65 "brainpy/_c/composition.pxd"
  size_t size;

#line 65 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":94
 * # Composition Declarations
 * 
 * cdef struct Composition:             # <<<<<<<<<<<<<<
 *     char** elements
 *     count_type* counts
 */

#line 94 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_11composition_Composition {

#line 94 "brainpy/_c/composition.pxd"
  char **elements;

#line 94 "brainpy/_c/composition.pxd"
  __pyx_t_7brainpy_2_c_11composition_count_type *counts;

#line 94 "brainpy/_c/composition.pxd"
  size_t size;

#line 94 "brainpy/_c/composition.pxd"
  size_t used;

#line 94 "brainpy/_c/composition.pxd"
  int max_variants;

#line 94 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":130
 *     @staticmethod
 *     cdef PyComposition _create(Composition* base)
 *     cdef void _set_impl(self, Composition* composition, bint free_existing=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _initialize_from_formula(self, str formula)
 */

#line 130 "brainpy/_c/composition.pxd"
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl {

#line 130 "brainpy/_c/composition.pxd"
  int __pyx_n;

#line 130 "brainpy/_c/composition.pxd"
  int free_existing;

#line 130 "brainpy/_c/composition.pxd"
};

/* "brainpy/_c/composition.pxd":144
 *     cpdef list items(self)
 * 
 *     cpdef pop(self, str key, object default=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef count_type getitem(self, str key)
 */

#line 144 "brainpy/_c/composition.pxd"
struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop {

#line 144 "brainpy/_c/composition.pxd"
  int __pyx_n;

#line 144 "brainpy/_c/composition.pxd"
  PyObject *__pyx_default;

#line 144 "brainpy/_c/composition.pxd"
};

#line 144 "brainpy/_c/composition.pxd"
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector;

/* "brainpy/_c/double_vector.pxd":1
 * cdef struct DoubleVector:             # <<<<<<<<<<<<<<
 *     double* v
 *     size_t used
 */

#line 1 "brainpy/_c/double_vector.pxd"
struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector {

#line 1 "brainpy/_c/double_vector.pxd"
  double *v;

#line 1 "brainpy/_c/double_vector.pxd"
  size_t used;

#line 1 "brainpy/_c/double_vector.pxd"
  size_t size;

#line 1 "brainpy/_c/double_vector.pxd"
};

#line 1 "brainpy/_c/double_vector.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters;

#line 1 "brainpy/_c/double_vector.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants;

#line 1 "brainpy/_c/double_vector.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants;

/* "brainpy/_c/isotopic_constants.pxd":6
 * from brainpy._c.double_vector cimport DoubleVector
 * 
 * ctypedef DoubleVector dvec             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 6 "brainpy/_c/isotopic_constants.pxd"
typedef struct __pyx_t_7brainpy_2_c_13double_vector_DoubleVector __pyx_t_7brainpy_2_c_18isotopic_constants_dvec;

/* "brainpy/_c/isotopic_constants.pxd":9
 * 
 * 
 * cdef struct PolynomialParameters:             # <<<<<<<<<<<<<<
 *     dvec* elementary_symmetric_polynomial
 *     dvec* power_sum
 */

#line 9 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters {

#line 9 "brainpy/_c/isotopic_constants.pxd"
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *elementary_symmetric_polynomial;

#line 9 "brainpy/_c/isotopic_constants.pxd"
  __pyx_t_7brainpy_2_c_18isotopic_constants_dvec *power_sum;

#line 9 "brainpy/_c/isotopic_constants.pxd"
};

/* "brainpy/_c/isotopic_constants.pxd":14
 * 
 * 
 * cdef struct PhiConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     Element* element
 */

#line 14 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants {

#line 14 "brainpy/_c/isotopic_constants.pxd"
  int order;

#line 14 "brainpy/_c/isotopic_constants.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Element *element;

#line 14 "brainpy/_c/isotopic_constants.pxd"
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *element_coefficients;

#line 14 "brainpy/_c/isotopic_constants.pxd"
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PolynomialParameters *mass_coefficients;

#line 14 "brainpy/_c/isotopic_constants.pxd"
};

/* "brainpy/_c/isotopic_constants.pxd":21
 * 
 * 
 * cdef struct IsotopicConstants:             # <<<<<<<<<<<<<<
 *     int order
 *     PhiConstants** constants
 */

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants {

#line 21 "brainpy/_c/isotopic_constants.pxd"
  int order;

#line 21 "brainpy/_c/isotopic_constants.pxd"
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_PhiConstants **constants;

#line 21 "brainpy/_c/isotopic_constants.pxd"
  size_t size;

#line 21 "brainpy/_c/isotopic_constants.pxd"
  size_t used;

#line 21 "brainpy/_c/isotopic_constants.pxd"
};

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_PeakList;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_IsotopicDistribution;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_make_isotopic_distribution;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_guess_npeaks;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution__isotopic_variants;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_isotopic_variants;

#line 21 "brainpy/_c/isotopic_constants.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_max_variants_approx;

/* "brainpy/_c/isotopic_distribution.pxd":7
 * 
 * 
 * cdef struct Peak:             # <<<<<<<<<<<<<<
 *     double mz
 *     double intensity
 */

#line 7 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak {

#line 7 "brainpy/_c/isotopic_distribution.pxd"
  double mz;

#line 7 "brainpy/_c/isotopic_distribution.pxd"
  double intensity;

#line 7 "brainpy/_c/isotopic_distribution.pxd"
  int charge;

#line 7 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":13
 * 
 * 
 * cdef struct PeakList:             # <<<<<<<<<<<<<<
 *     Peak* peaks
 *     size_t used
 */

#line 13 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_PeakList {

#line 13 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak *peaks;

#line 13 "brainpy/_c/isotopic_distribution.pxd"
  size_t used;

#line 13 "brainpy/_c/isotopic_distribution.pxd"
  size_t size;

#line 13 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":35
 * 
 * 
 * cdef struct ElementCache:             # <<<<<<<<<<<<<<
 *     Element** elements
 *     size_t used
 */

#line 35 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache {

#line 35 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Element **elements;

#line 35 "brainpy/_c/isotopic_distribution.pxd"
  size_t used;

#line 35 "brainpy/_c/isotopic_distribution.pxd"
  size_t size;

#line 35 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *source;

#line 35 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":49
 * 
 * 
 * cdef struct IsotopicDistribution:             # <<<<<<<<<<<<<<
 *     Composition* composition
 *     IsotopicConstants* _isotopic_constants
 */

#line 49 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_7brainpy_2_c_21isotopic_distribution_IsotopicDistribution {

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Composition *composition;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_18isotopic_constants_IsotopicConstants *_isotopic_constants;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  int order;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  double average_mass;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_Peak *monoisotopic_peak;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;

#line 49 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":58
 * 
 * 
 * cdef IsotopicDistribution* make_isotopic_distribution(Composition* composition, int order, ElementCache* cache=*) nogil             # <<<<<<<<<<<<<<
 * cdef void free_isotopic_distribution(IsotopicDistribution* distribution)  nogil
 * 
 */

#line 58 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_make_isotopic_distribution {

#line 58 "brainpy/_c/isotopic_distribution.pxd"
  int __pyx_n;

#line 58 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;

#line 58 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":77
 * 
 * cdef int max_variants(Composition* composition, ElementCache* cache) nogil
 * cdef int guess_npeaks(Composition* composition_struct, size_t max_npeaks, ElementCache* cache=*) nogil             # <<<<<<<<<<<<<<
 * 
 * cpdef list _isotopic_variants(object composition, object npeaks=*, int charge=*, double charge_carrier=*)
 */

#line 77 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_guess_npeaks {

#line 77 "brainpy/_c/isotopic_distribution.pxd"
  int __pyx_n;

#line 77 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_t_7brainpy_2_c_21isotopic_distribution_ElementCache *cache;

#line 77 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":79
 * cdef int guess_npeaks(Composition* composition_struct, size_t max_npeaks, ElementCache* cache=*) nogil
 * 
 * cpdef list _isotopic_variants(object composition, object npeaks=*, int charge=*, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 79 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution__isotopic_variants {

#line 79 "brainpy/_c/isotopic_distribution.pxd"
  int __pyx_n;

#line 79 "brainpy/_c/isotopic_distribution.pxd"
  PyObject *npeaks;

#line 79 "brainpy/_c/isotopic_distribution.pxd"
  int charge;

#line 79 "brainpy/_c/isotopic_distribution.pxd"
  double charge_carrier;

#line 79 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":82
 * 
 * 
 * cdef PeakList* isotopic_variants(Composition* composition, int npeaks, int charge=*, double charge_carrier=*) nogil             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 82 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_isotopic_variants {

#line 82 "brainpy/_c/isotopic_distribution.pxd"
  int __pyx_n;

#line 82 "brainpy/_c/isotopic_distribution.pxd"
  int charge;

#line 82 "brainpy/_c/isotopic_distribution.pxd"
  double charge_carrier;

#line 82 "brainpy/_c/isotopic_distribution.pxd"
};

/* "brainpy/_c/isotopic_distribution.pxd":85
 * 
 * 
 * cdef size_t max_variants_approx(double mass, double lambda_factor=*, size_t maxiter=*, double threshold=*) nogil             # <<<<<<<<<<<<<<
 */

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_7brainpy_2_c_21isotopic_distribution_max_variants_approx {

#line 85 "brainpy/_c/isotopic_distribution.pxd"
  int __pyx_n;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
  double lambda_factor;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
  size_t maxiter;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
  double threshold;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
};

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list;

#line 85 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted;

/* "ms_peak_picker/_c/peak_set.pxd":42
 *     cdef FittedPeak _get_nearest_peak(self, double mz, double* errout)
 * 
 *     cdef FittedPeak _has_peak(self, double mz, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakSet clone(self)
 */

#line 42 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak {

#line 42 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 42 "ms_peak_picker/_c/peak_set.pxd"
  double tolerance;

#line 42 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":50
 *     cdef int _between_bounds(self, double m1, double m2, size_t* startp, size_t* endp)
 * 
 *     cpdef tuple all_peaks_for(self, double mz, double error_tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef FittedPeak getitem(self, size_t i)
 */

#line 50 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for {

#line 50 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 50 "ms_peak_picker/_c/peak_set.pxd"
  double error_tolerance;

#line 50 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":64
 * 
 * 
 * cdef struct index_cell:             # <<<<<<<<<<<<<<
 *     double center_value
 *     size_t start
 */

#line 64 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell {

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  double center_value;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  size_t start;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
  size_t end;

#line 64 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":70
 * 
 * 
 * cdef struct index_list:             # <<<<<<<<<<<<<<
 *     index_cell* index
 *     size_t size
 */

#line 70 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list {

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_cell *index;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  size_t size;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  double low;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
  double high;

#line 70 "ms_peak_picker/_c/peak_set.pxd"
};

/* "ms_peak_picker/_c/peak_set.pxd":90
 *     cpdef _allocate_index(self, size_t interval_index_size)
 * 
 *     cpdef FittedPeak has_peak_hinted(self, double mz, double tolerance=*, size_t hint_start=*, size_t hint_end=*)             # <<<<<<<<<<<<<<
 * 
 * cdef size_t double_binary_search_ppm(double* array, double value, double tolerance, size_t n)
 */

#line 90 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted {

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  int __pyx_n;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  double tolerance;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  size_t hint_start;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
  size_t hint_end;

#line 90 "ms_peak_picker/_c/peak_set.pxd"
};

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":728
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */

#line 728 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":729
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */

#line 729 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":730
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */

#line 730 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":732
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
typedef npy_cdouble __pyx_t_5numpy_complex_t;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak;

#line 732 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for;

/* "ms_peak_picker/_c/peak_index.pxd":11
 *         public PeakSet peaks
 * 
 *     cdef FittedPeak _has_peak(self, double mz, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cdef PeakSet _between(self, double start, double stop)
 * 
 */

#line 11 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak {

#line 11 "ms_peak_picker/_c/peak_index.pxd"
  int __pyx_n;

#line 11 "ms_peak_picker/_c/peak_index.pxd"
  double tolerance;

#line 11 "ms_peak_picker/_c/peak_index.pxd"
};

/* "ms_peak_picker/_c/peak_index.pxd":14
 *     cdef PeakSet _between(self, double start, double stop)
 * 
 *     cpdef tuple all_peaks_for(self, double mz, double error_tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self)
 */

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for {

#line 14 "ms_peak_picker/_c/peak_index.pxd"
  int __pyx_n;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
  double error_tolerance;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
};

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_neutral_mass;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_isotopic_shift;

#line 14 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_poisson_approximate;

/* "ms_deisotope/_c/averagine.pxd":7
 * cdef double PROTON
 * 
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*)
 * cdef void slide(double mz, list peaklist)
 */

#line 7 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio {

#line 7 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 7 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 7 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":8
 * 
 * cdef double mass_charge_ratio(double neutral_mass, int z, double charge_carrier=*)
 * cdef double neutral_mass(double mz,  int z, double charge_carrier=*)             # <<<<<<<<<<<<<<
 * cdef void slide(double mz, list peaklist)
 * cdef dict scale_dict(dict data, double factor)
 */

#line 8 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_neutral_mass {

#line 8 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 8 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 8 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":17
 *         public dict base_composition
 * 
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 */

#line 17 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale {

#line 17 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 17 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 17 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 17 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":18
 * 
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 * 
 */

#line 18 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster {

#line 18 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 18 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 18 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 18 "ms_deisotope/_c/averagine.pxd"
  double truncate_after;

#line 18 "ms_deisotope/_c/averagine.pxd"
  double ignore_below;

#line 18 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":19
 *     cpdef dict scale(self, double mz, int charge=*, double charge_carrier=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cdef TheoreticalIsotopicPattern _isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 19 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster {

#line 19 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 19 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 19 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 19 "ms_deisotope/_c/averagine.pxd"
  double truncate_after;

#line 19 "ms_deisotope/_c/averagine.pxd"
  double ignore_below;

#line 19 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":39
 *     cdef bint _eq_inst(self, TheoreticalIsotopicPattern other)
 * 
 *     cpdef TheoreticalIsotopicPattern ignore_below(self, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 */

#line 39 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below {

#line 39 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 39 "ms_deisotope/_c/averagine.pxd"
  double ignore_below;

#line 39 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":40
 * 
 *     cpdef TheoreticalIsotopicPattern ignore_below(self, double ignore_below=*)
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 *     cpdef TheoreticalIsotopicPattern scale(self, list experimental_distribution, str method=*)
 */

#line 40 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after {

#line 40 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 40 "ms_deisotope/_c/averagine.pxd"
  double truncate_after;

#line 40 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":42
 *     cpdef TheoreticalIsotopicPattern truncate_after(self, double truncate_after=*)
 *     cpdef TheoreticalIsotopicPattern shift(self, double mz)
 *     cpdef TheoreticalIsotopicPattern scale(self, list experimental_distribution, str method=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern scale_raw(self, double scale_factor)
 *     cpdef double drop_last_peak(self)
 */

#line 42 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale {

#line 42 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 42 "ms_deisotope/_c/averagine.pxd"
  PyObject *method;

#line 42 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":53
 * 
 *     @cython.final
 *     cdef inline TheoreticalIsotopicPattern _scale(self, list experimental_distribution, str method=*)             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */

#line 53 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale {

#line 53 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 53 "ms_deisotope/_c/averagine.pxd"
  PyObject *method;

#line 53 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":88
 *     cdef double _make_cache_key(self, double mz)
 * 
 *     cdef TheoreticalIsotopicPattern has_mz_charge_pair(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 * 
 */

#line 88 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair {

#line 88 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 88 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 88 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 88 "ms_deisotope/_c/averagine.pxd"
  double truncate_after;

#line 88 "ms_deisotope/_c/averagine.pxd"
  double ignore_below;

#line 88 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":89
 * 
 *     cdef TheoreticalIsotopicPattern has_mz_charge_pair(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)
 *     cpdef TheoreticalIsotopicPattern isotopic_cluster(self, double mz, int charge=*, double charge_carrier=*, double truncate_after=*, double ignore_below=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 89 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster {

#line 89 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 89 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 89 "ms_deisotope/_c/averagine.pxd"
  double charge_carrier;

#line 89 "ms_deisotope/_c/averagine.pxd"
  double truncate_after;

#line 89 "ms_deisotope/_c/averagine.pxd"
  double ignore_below;

#line 89 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":92
 * 
 * 
 * cpdef double isotopic_shift(int charge=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef TheoreticalIsotopicPattern poisson_approximate(double mass, size_t n_peaks, double lambda_factor=*, int charge=*)
 */

#line 92 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_isotopic_shift {

#line 92 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 92 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 92 "ms_deisotope/_c/averagine.pxd"
};

/* "ms_deisotope/_c/averagine.pxd":94
 * cpdef double isotopic_shift(int charge=*)
 * 
 * cpdef TheoreticalIsotopicPattern poisson_approximate(double mass, size_t n_peaks, double lambda_factor=*, int charge=*)             # <<<<<<<<<<<<<<
 */

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_poisson_approximate {

#line 94 "ms_deisotope/_c/averagine.pxd"
  int __pyx_n;

#line 94 "ms_deisotope/_c/averagine.pxd"
  double lambda_factor;

#line 94 "ms_deisotope/_c/averagine.pxd"
  int charge;

#line 94 "ms_deisotope/_c/averagine.pxd"
};

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges;

#line 94 "ms_deisotope/_c/averagine.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base__get_all_peak_charge_pairs;

/* "ms_deisotope/_c/deconvoluter_base.pxd":42
 *     cdef FittedPeak _has_peak(self, double mz, double error_tolerance)
 * 
 *     cpdef list match_theoretical_isotopic_distribution(self, list theoretical_distribution, double error_tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef scale_theoretical_distribution(self, TheoreticalIsotopicPattern theoretical_distribution, list experimental_distribution)
 *     cpdef IsotopicFitRecord _evaluate_theoretical_distribution(self, list experimental, TheoreticalIsotopicPattern theoretical, FittedPeak peak, int charge)
 */

#line 42 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution {

#line 42 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 42 "ms_deisotope/_c/deconvoluter_base.pxd"
  double error_tolerance;

#line 42 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":45
 *     cpdef scale_theoretical_distribution(self, TheoreticalIsotopicPattern theoretical_distribution, list experimental_distribution)
 *     cpdef IsotopicFitRecord _evaluate_theoretical_distribution(self, list experimental, TheoreticalIsotopicPattern theoretical, FittedPeak peak, int charge)
 *     cpdef subtraction(self, TheoreticalIsotopicPattern isotopic_cluster, double error_tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef list fit_incremental_truncation(self, IsotopicFitRecord seed_fit, double lower_bound)
 * 
 */

#line 45 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction {

#line 45 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 45 "ms_deisotope/_c/deconvoluter_base.pxd"
  double error_tolerance;

#line 45 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":50
 *     cpdef bint _check_fit(self, IsotopicFitRecord fit)
 * 
 *     cpdef list _find_next_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 * 
 */

#line 50 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak {

#line 50 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 50 "ms_deisotope/_c/deconvoluter_base.pxd"
  int step;

#line 50 "ms_deisotope/_c/deconvoluter_base.pxd"
  double tolerance;

#line 50 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":51
 * 
 *     cpdef list _find_next_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 */

#line 51 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak {

#line 51 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 51 "ms_deisotope/_c/deconvoluter_base.pxd"
  int step;

#line 51 "ms_deisotope/_c/deconvoluter_base.pxd"
  double tolerance;

#line 51 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":53
 *     cpdef list _find_previous_putative_peak(self, double mz, int charge, int step=*, double tolerance=*)
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 *     cdef int _find_previous_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 * 
 */

#line 53 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace {

#line 53 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 53 "ms_deisotope/_c/deconvoluter_base.pxd"
  int step;

#line 53 "ms_deisotope/_c/deconvoluter_base.pxd"
  double tolerance;

#line 53 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":54
 * 
 *     cdef int _find_next_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)
 *     cdef int _find_previous_putative_peak_inplace(self, double mz, int charge, set result, int step=*, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 54 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace {

#line 54 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 54 "ms_deisotope/_c/deconvoluter_base.pxd"
  int step;

#line 54 "ms_deisotope/_c/deconvoluter_base.pxd"
  double tolerance;

#line 54 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":61
 *         public AveragineCache averagine
 * 
 *     cpdef IsotopicFitRecord fit_theoretical_distribution(self, FittedPeak peak, double error_tolerance, int charge,             # <<<<<<<<<<<<<<
 *                                                          double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 */

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution {

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
  double charge_carrier;

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
  double truncate_after;

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
  double ignore_below;

#line 61 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":65
 *                                                          double ignore_below=*)
 * 
 *     cpdef set _fit_peaks_at_charges(self, set peak_charge_set, double error_tolerance, double charge_carrier=*, double truncate_after=*,             # <<<<<<<<<<<<<<
 *                                     double ignore_below=*)
 * 
 */

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges {

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
  double charge_carrier;

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
  double truncate_after;

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
  double ignore_below;

#line 65 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":73
 *         public list averagines
 * 
 *     cpdef IsotopicFitRecord fit_theoretical_distribution(self, FittedPeak peak, double error_tolerance, int charge,             # <<<<<<<<<<<<<<
 *                                                          AveragineCache  averagine, double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 */

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution {

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
  double charge_carrier;

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
  double truncate_after;

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
  double ignore_below;

#line 73 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":76
 *                                                          AveragineCache  averagine, double charge_carrier=*, double truncate_after=*,
 *                                                          double ignore_below=*)
 *     cpdef set _fit_peaks_at_charges(self, set peak_charge_set, double error_tolerance, double charge_carrier=*, double truncate_after=*,             # <<<<<<<<<<<<<<
 *                                     double ignore_below=*)
 * 
 */

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges {

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
  double charge_carrier;

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
  double truncate_after;

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
  double ignore_below;

#line 76 "ms_deisotope/_c/deconvoluter_base.pxd"
};

/* "ms_deisotope/_c/deconvoluter_base.pxd":83
 * 
 * 
 * cpdef set _get_all_peak_charge_pairs(DeconvoluterBase self, FittedPeak peak, double error_tolerance=*,             # <<<<<<<<<<<<<<
 *                                      object charge_range=*,
 *                                      int left_search_limit=*, int right_search_limit=*,
 */

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base__get_all_peak_charge_pairs {

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  int __pyx_n;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  double error_tolerance;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *charge_range;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  int left_search_limit;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  int right_search_limit;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  int recalculate_starting_peak;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
  int use_quick_charge;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak;

#line 83 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference;

/* "ms_deisotope/_c/scoring.pxd":29
 * 
 *     @staticmethod
 *     cdef IsotopicFitRecord _create(FittedPeak seed_peak, double score, int charge, TheoreticalIsotopicPattern theoretical,             # <<<<<<<<<<<<<<
 *                                    list experimental, object data=*, int missed_peaks=*)
 * 
 */

#line 29 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create {

#line 29 "ms_deisotope/_c/scoring.pxd"
  int __pyx_n;

#line 29 "ms_deisotope/_c/scoring.pxd"
  PyObject *data;

#line 29 "ms_deisotope/_c/scoring.pxd"
  int missed_peaks;

#line 29 "ms_deisotope/_c/scoring.pxd"
};

/* "ms_deisotope/_c/scoring.pxd":75
 *     cdef public double mass_error_tolerance
 * 
 *     cdef double score_peak(self, FittedPeak obs, TheoreticalPeak theo, double mass_error_tolerance=*, double minimum_signal_to_noise=*) nogil             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 75 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak {

#line 75 "ms_deisotope/_c/scoring.pxd"
  int __pyx_n;

#line 75 "ms_deisotope/_c/scoring.pxd"
  double mass_error_tolerance;

#line 75 "ms_deisotope/_c/scoring.pxd"
  double minimum_signal_to_noise;

#line 75 "ms_deisotope/_c/scoring.pxd"
};

/* "ms_deisotope/_c/scoring.pxd":92
 *     cdef public PeakSet peaklist
 * 
 *     cdef double detect_interference(self, list experimental_peaks, double lower=*, double upper=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference {

#line 92 "ms_deisotope/_c/scoring.pxd"
  int __pyx_n;

#line 92 "ms_deisotope/_c/scoring.pxd"
  double lower;

#line 92 "ms_deisotope/_c/scoring.pxd"
  double upper;

#line 92 "ms_deisotope/_c/scoring.pxd"
};

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak;

#line 92 "ms_deisotope/_c/scoring.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for;

/* "ms_deisotope/_c/peak_set.pxd":87
 *     cpdef DeconvolutedPeakSet copy(self)
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 */

#line 87 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak {

#line 87 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_n;

#line 87 "ms_deisotope/_c/peak_set.pxd"
  double error_tolerance;

#line 87 "ms_deisotope/_c/peak_set.pxd"
  int use_mz;

#line 87 "ms_deisotope/_c/peak_set.pxd"
};

/* "ms_deisotope/_c/peak_set.pxd":89
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=*)
 */

#line 89 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak {

#line 89 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_n;

#line 89 "ms_deisotope/_c/peak_set.pxd"
  double error_tolerance;

#line 89 "ms_deisotope/_c/peak_set.pxd"
  int use_mz;

#line 89 "ms_deisotope/_c/peak_set.pxd"
};

/* "ms_deisotope/_c/peak_set.pxd":91
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=*, bint use_mz=*)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout)
 */

#line 91 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for {

#line 91 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_n;

#line 91 "ms_deisotope/_c/peak_set.pxd"
  double tolerance;

#line 91 "ms_deisotope/_c/peak_set.pxd"
};

/* "ms_deisotope/_c/peak_set.pxd":118
 * cdef size_t INTERVAL_INDEX_SIZE
 * 
 * cdef struct index_cell:             # <<<<<<<<<<<<<<
 *     double center_value
 *     size_t start
 */

#line 118 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell {

#line 118 "ms_deisotope/_c/peak_set.pxd"
  double center_value;

#line 118 "ms_deisotope/_c/peak_set.pxd"
  size_t start;

#line 118 "ms_deisotope/_c/peak_set.pxd"
  size_t end;

#line 118 "ms_deisotope/_c/peak_set.pxd"
};

/* "ms_deisotope/_c/peak_set.pxd":124
 * 
 * 
 * cdef struct index_list:             # <<<<<<<<<<<<<<
 *     index_cell* index
 *     size_t size
 */

#line 124 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list {

#line 124 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell *index;

#line 124 "ms_deisotope/_c/peak_set.pxd"
  size_t size;

#line 124 "ms_deisotope/_c/peak_set.pxd"
  double low;

#line 124 "ms_deisotope/_c/peak_set.pxd"
  double high;

#line 124 "ms_deisotope/_c/peak_set.pxd"
};

/* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */

#line 45 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor {

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_n;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *mz;

#line 45 "ms_deisotope/_c/peak_set.pyx"
};

/* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1209 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak {

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_n;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  double error_tolerance;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int use_mz;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
};

/* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_n;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  double tolerance;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
};

/* "brainpy/_c/composition.pxd":123
 * 
 * 
 * cdef class PyComposition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Composition* impl
 */

#line 123 "brainpy/_c/composition.pxd"
struct __pyx_obj_7brainpy_2_c_11composition_PyComposition {

#line 123 "brainpy/_c/composition.pxd"
  PyObject_HEAD

#line 123 "brainpy/_c/composition.pxd"
  struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtab;

#line 123 "brainpy/_c/composition.pxd"
  struct __pyx_t_7brainpy_2_c_11composition_Composition *impl;

#line 123 "brainpy/_c/composition.pxd"
  double cached_mass;

#line 123 "brainpy/_c/composition.pxd"
  int _clean;

#line 123 "brainpy/_c/composition.pxd"
};

#line 123 "brainpy/_c/composition.pxd"


/* "brainpy/_c/isotopic_distribution.pxd":63
 * 
 * @freelist(100000)
 * cdef class TheoreticalPeak(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 63 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak {

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  PyObject_HEAD

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *__pyx_vtab;

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  double mz;

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  double intensity;

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  int charge;

#line 63 "brainpy/_c/isotopic_distribution.pxd"
};

#line 63 "brainpy/_c/isotopic_distribution.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 2 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase {

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  PyObject_HEAD

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtab;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double mz;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double intensity;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  double area;

#line 2 "ms_peak_picker/_c/peak_set.pxd"
};

#line 2 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */

#line 12 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak {

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double signal_to_noise;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double full_width_at_half_max;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double left_width;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  double right_width;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  long peak_count;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  long index;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
};

#line 12 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

#line 29 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet {

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject_HEAD

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtab;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *peaks;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int indexed;

#line 29 "ms_peak_picker/_c/peak_set.pxd"
};

#line 29 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */

#line 83 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  double *mz_index;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_t_14ms_peak_picker_2_c_8peak_set_index_list *interval_index;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
};

#line 83 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_index.pxd":5
 * 
 * 
 * cdef class PeakIndex(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public np.ndarray mz_array
 */

#line 5 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex {

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  PyObject_HEAD

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex *__pyx_vtab;

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  PyArrayObject *mz_array;

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  PyArrayObject *intensity_array;

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaks;

#line 5 "ms_peak_picker/_c/peak_index.pxd"
};

#line 5 "ms_peak_picker/_c/peak_index.pxd"


/* "ms_deisotope/_c/averagine.pxd":12
 * cdef dict scale_dict(dict data, double factor)
 * 
 * cdef class Averagine(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double base_mass
 */

#line 12 "ms_deisotope/_c/averagine.pxd"
struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine {

#line 12 "ms_deisotope/_c/averagine.pxd"
  PyObject_HEAD

#line 12 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine *__pyx_vtab;

#line 12 "ms_deisotope/_c/averagine.pxd"
  double base_mass;

#line 12 "ms_deisotope/_c/averagine.pxd"
  PyObject *base_composition;

#line 12 "ms_deisotope/_c/averagine.pxd"
};

#line 12 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/averagine.pxd":23
 * 
 * @cython.final
 * cdef class TheoreticalIsotopicPattern(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list peaklist
 */

#line 23 "ms_deisotope/_c/averagine.pxd"
struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern {

#line 23 "ms_deisotope/_c/averagine.pxd"
  PyObject_HEAD

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *__pyx_vtab;

#line 23 "ms_deisotope/_c/averagine.pxd"
  PyObject *peaklist;

#line 23 "ms_deisotope/_c/averagine.pxd"
  double origin;

#line 23 "ms_deisotope/_c/averagine.pxd"
  double offset;

#line 23 "ms_deisotope/_c/averagine.pxd"
};

#line 23 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/averagine.pxd":79
 * 
 * @cython.final
 * cdef class AveragineCache(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict backend
 */

#line 79 "ms_deisotope/_c/averagine.pxd"
struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache {

#line 79 "ms_deisotope/_c/averagine.pxd"
  PyObject_HEAD

#line 79 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache *__pyx_vtab;

#line 79 "ms_deisotope/_c/averagine.pxd"
  PyObject *backend;

#line 79 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *averagine;

#line 79 "ms_deisotope/_c/averagine.pxd"
  double cache_truncation;

#line 79 "ms_deisotope/_c/averagine.pxd"
  int enabled;

#line 79 "ms_deisotope/_c/averagine.pxd"
};

#line 79 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":25
 * 
 * 
 * cdef class DeconvoluterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint use_subtraction
 */

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase {

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject_HEAD

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *__pyx_vtab;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int use_subtraction;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *scale_method;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int merge_isobaric_peaks;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  double minimum_intensity;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaklist;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *scorer;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int verbose;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *_slice_cache;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":57
 * 
 * 
 * cdef class AveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AveragineCache averagine
 */

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase {

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *averagine;

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":69
 * 
 * 
 * cdef class MultiAveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list averagines
 */

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase {

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *averagines;

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":91
 * 
 * 
 * cdef class ChargeIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int lower
 */

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator {

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject_HEAD

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *__pyx_vtab;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int lower;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int upper;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int sign;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int *values;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  size_t size;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  size_t index;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/scoring.pxd":16
 * 
 * 
 * cdef class IsotopicFitRecord(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FittedPeak seed_peak
 */

#line 16 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord {

#line 16 "ms_deisotope/_c/scoring.pxd"
  PyObject_HEAD

#line 16 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *__pyx_vtab;

#line 16 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *seed_peak;

#line 16 "ms_deisotope/_c/scoring.pxd"
  double score;

#line 16 "ms_deisotope/_c/scoring.pxd"
  int charge;

#line 16 "ms_deisotope/_c/scoring.pxd"
  PyObject *experimental;

#line 16 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *theoretical;

#line 16 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *monoisotopic_peak;

#line 16 "ms_deisotope/_c/scoring.pxd"
  int missed_peaks;

#line 16 "ms_deisotope/_c/scoring.pxd"
  PyObject *data;

#line 16 "ms_deisotope/_c/scoring.pxd"
  Py_hash_t _hash;

#line 16 "ms_deisotope/_c/scoring.pxd"
};

#line 16 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":38
 * 
 * 
 * cdef class FitSelectorBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double minimum_score
 */

#line 38 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase {

#line 38 "ms_deisotope/_c/scoring.pxd"
  PyObject_HEAD

#line 38 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase *__pyx_vtab;

#line 38 "ms_deisotope/_c/scoring.pxd"
  double minimum_score;

#line 38 "ms_deisotope/_c/scoring.pxd"
};

#line 38 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":50
 * 
 * 
 * cdef class IsotopicFitterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FitSelectorBase select
 */

#line 50 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase {

#line 50 "ms_deisotope/_c/scoring.pxd"
  PyObject_HEAD

#line 50 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *__pyx_vtab;

#line 50 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *select;

#line 50 "ms_deisotope/_c/scoring.pxd"
};

#line 50 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":62
 * 
 * 
 * cdef class LeastSquaresFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 62 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter {

#line 62 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 62 "ms_deisotope/_c/scoring.pxd"
};

#line 62 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":66
 * 
 * 
 * cdef class GTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 66 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter {

#line 66 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 66 "ms_deisotope/_c/scoring.pxd"
};

#line 66 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":69
 *     pass
 * 
 * cdef class ScaledGTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 69 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter {

#line 69 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 69 "ms_deisotope/_c/scoring.pxd"
};

#line 69 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":72
 *     pass
 * 
 * cdef class MSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef public double mass_error_tolerance
 * 
 */

#line 72 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter {

#line 72 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 72 "ms_deisotope/_c/scoring.pxd"
  double mass_error_tolerance;

#line 72 "ms_deisotope/_c/scoring.pxd"
};

#line 72 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":78
 * 
 * 
 * cdef class PenalizedMSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mass_error_tolerance
 */

#line 78 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter {

#line 78 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 78 "ms_deisotope/_c/scoring.pxd"
  double mass_error_tolerance;

#line 78 "ms_deisotope/_c/scoring.pxd"
  double penalty_factor;

#line 78 "ms_deisotope/_c/scoring.pxd"
};

#line 78 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":84
 * 
 * 
 * cdef class FunctionScorer(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         object function
 */

#line 84 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer {

#line 84 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 84 "ms_deisotope/_c/scoring.pxd"
  PyObject *function;

#line 84 "ms_deisotope/_c/scoring.pxd"
};

#line 84 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":89
 * 
 * 
 * cdef class InterferenceDetection(object):             # <<<<<<<<<<<<<<
 *     cdef public PeakSet peaklist
 * 
 */

#line 89 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection {

#line 89 "ms_deisotope/_c/scoring.pxd"
  PyObject_HEAD

#line 89 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection *__pyx_vtab;

#line 89 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *peaklist;

#line 89 "ms_deisotope/_c/scoring.pxd"
};

#line 89 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":95
 * 
 * 
 * cdef class DistinctPatternFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public InterferenceDetection interference_detector
 */

#line 95 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter {

#line 95 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 95 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection *interference_detector;

#line 95 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter *g_test_scaled;

#line 95 "ms_deisotope/_c/scoring.pxd"
  double peak_count_scale;

#line 95 "ms_deisotope/_c/scoring.pxd"
  double domain_scale;

#line 95 "ms_deisotope/_c/scoring.pxd"
};

#line 95 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":102
 *         public double domain_scale
 * 
 * cdef class ScaledPenalizedMSDeconvFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double scale_factor
 */

#line 102 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter {

#line 102 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 102 "ms_deisotope/_c/scoring.pxd"
  double scale_factor;

#line 102 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter *scorer;

#line 102 "ms_deisotope/_c/scoring.pxd"
};

#line 102 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":112
 * 
 * 
 * cdef class DotProductFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 */

#line 112 "ms_deisotope/_c/scoring.pxd"
struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter {

#line 112 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 112 "ms_deisotope/_c/scoring.pxd"
};

#line 112 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/peak_set.pxd":5
 * 
 * 
 * cdef class _Index:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public size_t neutral_mass
 */

#line 5 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index {

#line 5 "ms_deisotope/_c/peak_set.pxd"
  PyObject_HEAD

#line 5 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index *__pyx_vtab;

#line 5 "ms_deisotope/_c/peak_set.pxd"
  size_t neutral_mass;

#line 5 "ms_deisotope/_c/peak_set.pxd"
  size_t mz;

#line 5 "ms_deisotope/_c/peak_set.pxd"
};

#line 5 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":14
 * 
 * 
 * cdef class EnvelopePair:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 14 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair {

#line 14 "ms_deisotope/_c/peak_set.pxd"
  PyObject_HEAD

#line 14 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_vtab;

#line 14 "ms_deisotope/_c/peak_set.pxd"
  double mz;

#line 14 "ms_deisotope/_c/peak_set.pxd"
  double intensity;

#line 14 "ms_deisotope/_c/peak_set.pxd"
};

#line 14 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":25
 * 
 * 
 * cdef class Envelope:             # <<<<<<<<<<<<<<
 *     cdef public tuple pairs
 * 
 */

#line 25 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope {

#line 25 "ms_deisotope/_c/peak_set.pxd"
  PyObject_HEAD

#line 25 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_vtab;

#line 25 "ms_deisotope/_c/peak_set.pxd"
  PyObject *pairs;

#line 25 "ms_deisotope/_c/peak_set.pxd"
};

#line 25 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":40
 * 
 * 
 * cdef class DeconvolutedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double neutral_mass
 */

#line 40 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak {

#line 40 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double neutral_mass;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double signal_to_noise;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *_index;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double full_width_at_half_max;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  int charge;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double a_to_a2_ratio;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double most_abundant_mass;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double average_mass;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  double score;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *envelope;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *fit;

#line 40 "ms_deisotope/_c/peak_set.pxd"
  int chosen_for_msms;

#line 40 "ms_deisotope/_c/peak_set.pxd"
};

#line 40 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":66
 * 
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double drift_time
 */

#line 66 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak {

#line 66 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;

#line 66 "ms_deisotope/_c/peak_set.pxd"
  double drift_time;

#line 66 "ms_deisotope/_c/peak_set.pxd"
};

#line 66 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":71
 * 
 * 
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object solution
 */

#line 71 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution {

#line 71 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;

#line 71 "ms_deisotope/_c/peak_set.pxd"
  PyObject *solution;

#line 71 "ms_deisotope/_c/peak_set.pxd"
};

#line 71 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":75
 *         public object solution
 * 
 * cdef class DeconvolutedPeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

#line 75 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet {

#line 75 "ms_deisotope/_c/peak_set.pxd"
  PyObject_HEAD

#line 75 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_vtab;

#line 75 "ms_deisotope/_c/peak_set.pxd"
  PyObject *peaks;

#line 75 "ms_deisotope/_c/peak_set.pxd"
  PyObject *_mz_ordered;

#line 75 "ms_deisotope/_c/peak_set.pxd"
  int indexed;

#line 75 "ms_deisotope/_c/peak_set.pxd"
};

#line 75 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pxd":104
 * 
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* neutral_mass_array
 */

#line 104 "ms_deisotope/_c/peak_set.pxd"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed {

#line 104 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_base;

#line 104 "ms_deisotope/_c/peak_set.pxd"
  double *neutral_mass_array;

#line 104 "ms_deisotope/_c/peak_set.pxd"
  double *mz_array;

#line 104 "ms_deisotope/_c/peak_set.pxd"
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *interval_index;

#line 104 "ms_deisotope/_c/peak_set.pxd"
  size_t _size;

#line 104 "ms_deisotope/_c/peak_set.pxd"
};

#line 104 "ms_deisotope/_c/peak_set.pxd"


/* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */

#line 143 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ {

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 143 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self;

#line 143 "ms_deisotope/_c/peak_set.pyx"
};

#line 143 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

#line 259 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ {

#line 259 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 259 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self;

#line 259 "ms_deisotope/_c/peak_set.pyx"
};

#line 259 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 260 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr {

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 260 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_outer_scope;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_v_t;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*__pyx_t_2)(PyObject *);

#line 260 "ms_deisotope/_c/peak_set.pyx"
};

#line 260 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */

#line 529 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ {

#line 529 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 529 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self;

#line 529 "ms_deisotope/_c/peak_set.pyx"
};

#line 529 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

#line 599 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ {

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_v_item;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self;

#line 599 "ms_deisotope/_c/peak_set.pyx"
};

#line 599 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */

#line 601 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr {

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject_HEAD

#line 601 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_outer_scope;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_v_p;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*__pyx_t_2)(PyObject *);

#line 601 "ms_deisotope/_c/peak_set.pyx"
};

#line 601 "ms_deisotope/_c/peak_set.pyx"


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

#line 105 "View.MemoryView"
struct __pyx_array_obj {

#line 105 "View.MemoryView"
  PyObject_HEAD

#line 105 "View.MemoryView"
  struct __pyx_vtabstruct_array *__pyx_vtab;

#line 105 "View.MemoryView"
  char *data;

#line 105 "View.MemoryView"
  Py_ssize_t len;

#line 105 "View.MemoryView"
  char *format;

#line 105 "View.MemoryView"
  int ndim;

#line 105 "View.MemoryView"
  Py_ssize_t *_shape;

#line 105 "View.MemoryView"
  Py_ssize_t *_strides;

#line 105 "View.MemoryView"
  Py_ssize_t itemsize;

#line 105 "View.MemoryView"
  PyObject *mode;

#line 105 "View.MemoryView"
  PyObject *_format;

#line 105 "View.MemoryView"
  void (*callback_free_data)(void *);

#line 105 "View.MemoryView"
  int free_data;

#line 105 "View.MemoryView"
  int dtype_is_object;

#line 105 "View.MemoryView"
};

#line 105 "View.MemoryView"


/* "View.MemoryView":279
 * 
 * @cname('__pyx_MemviewEnum')
 * cdef class Enum(object):             # <<<<<<<<<<<<<<
 *     cdef object name
 *     def __init__(self, name):
 */

#line 279 "View.MemoryView"
struct __pyx_MemviewEnum_obj {

#line 279 "View.MemoryView"
  PyObject_HEAD

#line 279 "View.MemoryView"
  PyObject *name;

#line 279 "View.MemoryView"
};

#line 279 "View.MemoryView"


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

#line 330 "View.MemoryView"
struct __pyx_memoryview_obj {

#line 330 "View.MemoryView"
  PyObject_HEAD

#line 330 "View.MemoryView"
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;

#line 330 "View.MemoryView"
  PyObject *obj;

#line 330 "View.MemoryView"
  PyObject *_size;

#line 330 "View.MemoryView"
  PyObject *_array_interface;

#line 330 "View.MemoryView"
  PyThread_type_lock lock;

#line 330 "View.MemoryView"
  __pyx_atomic_int acquisition_count[2];

#line 330 "View.MemoryView"
  __pyx_atomic_int *acquisition_count_aligned_p;

#line 330 "View.MemoryView"
  Py_buffer view;

#line 330 "View.MemoryView"
  int flags;

#line 330 "View.MemoryView"
  int dtype_is_object;

#line 330 "View.MemoryView"
  __Pyx_TypeInfo *typeinfo;

#line 330 "View.MemoryView"
};

#line 330 "View.MemoryView"


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

#line 965 "View.MemoryView"
struct __pyx_memoryviewslice_obj {

#line 965 "View.MemoryView"
  struct __pyx_memoryview_obj __pyx_base;

#line 965 "View.MemoryView"
  __Pyx_memviewslice from_slice;

#line 965 "View.MemoryView"
  PyObject *from_object;

#line 965 "View.MemoryView"
  PyObject *(*to_object_func)(char *);

#line 965 "View.MemoryView"
  int (*to_dtype_func)(char *, PyObject *);

#line 965 "View.MemoryView"
};

#line 965 "View.MemoryView"


#line 965 "View.MemoryView"


/* "brainpy/_c/composition.pxd":123
 * 
 * 
 * cdef class PyComposition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Composition* impl
 */

#line 123 "brainpy/_c/composition.pxd"


#line 123 "brainpy/_c/composition.pxd"
struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition {

#line 123 "brainpy/_c/composition.pxd"
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*_create)(struct __pyx_t_7brainpy_2_c_11composition_Composition *);

#line 123 "brainpy/_c/composition.pxd"
  void (*_set_impl)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_t_7brainpy_2_c_11composition_Composition *, struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition__set_impl *__pyx_optional_args);

#line 123 "brainpy/_c/composition.pxd"
  void (*_initialize_from_formula)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *);

#line 123 "brainpy/_c/composition.pxd"
  double (*mass)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  int (*__pyx___equality_pycomposition)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  int (*__pyx___equality_dict)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *(*copy)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  PyObject *(*update)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  PyObject *(*keys)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  PyObject *(*values)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  PyObject *(*items)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, int __pyx_skip_dispatch);

#line 123 "brainpy/_c/composition.pxd"
  PyObject *(*pop)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_7brainpy_2_c_11composition_13PyComposition_pop *__pyx_optional_args);

#line 123 "brainpy/_c/composition.pxd"
  __pyx_t_7brainpy_2_c_11composition_count_type (*getitem)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *);

#line 123 "brainpy/_c/composition.pxd"
  void (*setitem)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, __pyx_t_7brainpy_2_c_11composition_count_type);

#line 123 "brainpy/_c/composition.pxd"
  void (*increment)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, PyObject *, __pyx_t_7brainpy_2_c_11composition_count_type);

#line 123 "brainpy/_c/composition.pxd"
  void (*add_from)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *);

#line 123 "brainpy/_c/composition.pxd"
  void (*subtract_from)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *);

#line 123 "brainpy/_c/composition.pxd"
  void (*scale_by)(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition *, long);

#line 123 "brainpy/_c/composition.pxd"
};

#line 123 "brainpy/_c/composition.pxd"
static struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition *__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition;

#line 123 "brainpy/_c/composition.pxd"


/* "brainpy/_c/isotopic_distribution.pxd":63
 * 
 * @freelist(100000)
 * cdef class TheoreticalPeak(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 63 "brainpy/_c/isotopic_distribution.pxd"


#line 63 "brainpy/_c/isotopic_distribution.pxd"
struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak {

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*_create)(double, double, int);

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  int (*_eq)(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, int __pyx_skip_dispatch);

#line 63 "brainpy/_c/isotopic_distribution.pxd"
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*clone)(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, int __pyx_skip_dispatch);

#line 63 "brainpy/_c/isotopic_distribution.pxd"
};

#line 63 "brainpy/_c/isotopic_distribution.pxd"
static struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *__pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak;

#line 63 "brainpy/_c/isotopic_distribution.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":2
 * 
 * cdef class PeakBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mz
 */

#line 2 "ms_peak_picker/_c/peak_set.pxd"


#line 2 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase {

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);

#line 2 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch);

#line 2 "ms_peak_picker/_c/peak_set.pxd"
};

#line 2 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase;

#line 2 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":12
 * 
 * 
 * cdef class FittedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double signal_to_noise
 */

#line 12 "ms_peak_picker/_c/peak_set.pxd"


#line 12 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak {

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int __pyx_skip_dispatch);

#line 12 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_create)(double, double, double, double, double, double, long, long, double);

#line 12 "ms_peak_picker/_c/peak_set.pxd"
};

#line 12 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak;

#line 12 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":29
 *                             double area)
 * 
 * cdef class PeakSet(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public tuple peaks
 */

#line 29 "ms_peak_picker/_c/peak_set.pxd"


#line 29 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet {

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_create)(PyObject *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*reindex)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  size_t (*_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_get_nearest_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet__has_peak *__pyx_optional_args);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_between)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int (*_between_bounds)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, double, size_t *, size_t *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*all_peaks_for)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_7PeakSet_all_peaks_for *__pyx_optional_args);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*getitem)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, size_t);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  size_t (*get_size)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
  int (*_eq)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 29 "ms_peak_picker/_c/peak_set.pxd"
};

#line 29 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet;

#line 29 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_set.pxd":83
 * 
 * 
 * cdef class PeakSetIndexed(PeakSet):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* mz_index
 */

#line 83 "ms_peak_picker/_c/peak_set.pxd"


#line 83 "ms_peak_picker/_c/peak_set.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed {

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet __pyx_base;

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  PyObject *(*_allocate_index)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, size_t, int __pyx_skip_dispatch);

#line 83 "ms_peak_picker/_c/peak_set.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*has_peak_hinted)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_8peak_set_14PeakSetIndexed_has_peak_hinted *__pyx_optional_args);

#line 83 "ms_peak_picker/_c/peak_set.pxd"
};

#line 83 "ms_peak_picker/_c/peak_set.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed;

#line 83 "ms_peak_picker/_c/peak_set.pxd"


/* "ms_peak_picker/_c/peak_index.pxd":5
 * 
 * 
 * cdef class PeakIndex(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public np.ndarray mz_array
 */

#line 5 "ms_peak_picker/_c/peak_index.pxd"


#line 5 "ms_peak_picker/_c/peak_index.pxd"
struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex {

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex__has_peak *__pyx_optional_args);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*_between)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, double);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  PyObject *(*all_peaks_for)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_14ms_peak_picker_2_c_10peak_index_9PeakIndex_all_peaks_for *__pyx_optional_args);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  size_t (*get_size)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *(*clone)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, int __pyx_skip_dispatch);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *(*copy)(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex *, int __pyx_skip_dispatch);

#line 5 "ms_peak_picker/_c/peak_index.pxd"
};

#line 5 "ms_peak_picker/_c/peak_index.pxd"
static struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex *__pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex;

#line 5 "ms_peak_picker/_c/peak_index.pxd"


/* "ms_deisotope/_c/averagine.pxd":12
 * cdef dict scale_dict(dict data, double factor)
 * 
 * cdef class Averagine(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double base_mass
 */

#line 12 "ms_deisotope/_c/averagine.pxd"


#line 12 "ms_deisotope/_c/averagine.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine {

#line 12 "ms_deisotope/_c/averagine.pxd"
  PyObject *(*scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_scale *__pyx_optional_args);

#line 12 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine_isotopic_cluster *__pyx_optional_args);

#line 12 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine *, double, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_9Averagine__isotopic_cluster *__pyx_optional_args);

#line 12 "ms_deisotope/_c/averagine.pxd"
};

#line 12 "ms_deisotope/_c/averagine.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine;

#line 12 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/averagine.pxd":23
 * 
 * @cython.final
 * cdef class TheoreticalIsotopicPattern(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list peaklist
 */

#line 23 "ms_deisotope/_c/averagine.pxd"


#line 23 "ms_deisotope/_c/averagine.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern {

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_create)(PyObject *, double, double);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  int (*_eq_inst)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*ignore_below)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_ignore_below *__pyx_optional_args);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*truncate_after)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_truncate_after *__pyx_optional_args);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*shift)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern_scale *__pyx_optional_args);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*scale_raw)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  double (*drop_last_peak)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  double (*total)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*normalize)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*_scale)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_26TheoreticalIsotopicPattern__scale *__pyx_optional_args);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone_shift)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *(*get)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, Py_ssize_t);

#line 23 "ms_deisotope/_c/averagine.pxd"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);

#line 23 "ms_deisotope/_c/averagine.pxd"
  double (*get_monoisotopic_mz)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);

#line 23 "ms_deisotope/_c/averagine.pxd"
  PyObject *(*get_processed_peaks)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);

#line 23 "ms_deisotope/_c/averagine.pxd"
  size_t (*basepeak_index)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  PyObject *(*incremental_truncation)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, double, int __pyx_skip_dispatch);

#line 23 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*clone_drop_last)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *);

#line 23 "ms_deisotope/_c/averagine.pxd"
};

#line 23 "ms_deisotope/_c/averagine.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern;

#line 23 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/averagine.pxd":79
 * 
 * @cython.final
 * cdef class AveragineCache(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict backend
 */

#line 79 "ms_deisotope/_c/averagine.pxd"


#line 79 "ms_deisotope/_c/averagine.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache {

#line 79 "ms_deisotope/_c/averagine.pxd"
  double (*_make_cache_key)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double);

#line 79 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*has_mz_charge_pair)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_has_mz_charge_pair *__pyx_optional_args);

#line 79 "ms_deisotope/_c/averagine.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *(*isotopic_cluster)(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_14AveragineCache_isotopic_cluster *__pyx_optional_args);

#line 79 "ms_deisotope/_c/averagine.pxd"
};

#line 79 "ms_deisotope/_c/averagine.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache *__pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache;

#line 79 "ms_deisotope/_c/averagine.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":25
 * 
 * 
 * cdef class DeconvoluterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint use_subtraction
 */

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"


#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase {

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *(*between)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*has_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *(*_has_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, double);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*match_theoretical_isotopic_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_match_theoretical_isotopic_distribution *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*scale_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_evaluate_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*subtraction)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase_subtraction *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*fit_incremental_truncation)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, double, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int (*_check_fit)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*_find_next_putative_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*_find_previous_putative_peak)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int (*_find_next_putative_peak_inplace)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_next_putative_peak_inplace *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
  int (*_find_previous_putative_peak_inplace)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, double, int, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_16DeconvoluterBase__find_previous_putative_peak_inplace *__pyx_optional_args);

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase;

#line 25 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":57
 * 
 * 
 * cdef class AveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AveragineCache averagine
 */

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"


#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase {

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*fit_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase_fit_theoretical_distribution *__pyx_optional_args);

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*_fit_peaks_at_charges)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *, PyObject *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_25AveragineDeconvoluterBase__fit_peaks_at_charges *__pyx_optional_args);

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase;

#line 57 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":69
 * 
 * 
 * cdef class MultiAveragineDeconvoluterBase(DeconvoluterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list averagines
 */

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"


#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase {

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase __pyx_base;

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*fit_theoretical_distribution)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase_fit_theoretical_distribution *__pyx_optional_args);

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*_fit_peaks_at_charges)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *, PyObject *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_17deconvoluter_base_30MultiAveragineDeconvoluterBase__fit_peaks_at_charges *__pyx_optional_args);

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase;

#line 69 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/deconvoluter_base.pxd":91
 * 
 * 
 * cdef class ChargeIterator(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int lower
 */

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"


#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator {

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*_create)(int, int);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*from_tuple)(PyObject *);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *(*from_quickcharge)(PyObject *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*sequence_from_quickcharge)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, int __pyx_skip_dispatch);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*make_sequence)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  void (*release_sequence)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  void (*set_bounds)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int, int);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int (*has_more)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  PyObject *(*reset)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  int (*get_next_value)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *, int __pyx_skip_dispatch);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *);

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
};

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator *__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator;

#line 91 "ms_deisotope/_c/deconvoluter_base.pxd"


/* "ms_deisotope/_c/scoring.pxd":16
 * 
 * 
 * cdef class IsotopicFitRecord(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FittedPeak seed_peak
 */

#line 16 "ms_deisotope/_c/scoring.pxd"


#line 16 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord {

#line 16 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_create)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, double, int, struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_17IsotopicFitRecord__create *__pyx_optional_args);

#line 16 "ms_deisotope/_c/scoring.pxd"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 16 "ms_deisotope/_c/scoring.pxd"
  int (*_ne)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 16 "ms_deisotope/_c/scoring.pxd"
  int (*_lt)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 16 "ms_deisotope/_c/scoring.pxd"
  int (*_gt)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 16 "ms_deisotope/_c/scoring.pxd"
};

#line 16 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord;

#line 16 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":38
 * 
 * 
 * cdef class FitSelectorBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double minimum_score
 */

#line 38 "ms_deisotope/_c/scoring.pxd"


#line 38 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase {

#line 38 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*best)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, PyObject *, int __pyx_skip_dispatch);

#line 38 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *(*_best_from_set)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, PyObject *);

#line 38 "ms_deisotope/_c/scoring.pxd"
  int (*reject)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 38 "ms_deisotope/_c/scoring.pxd"
  int (*reject_score)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, double, int __pyx_skip_dispatch);

#line 38 "ms_deisotope/_c/scoring.pxd"
  int (*is_maximizing)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase *, int __pyx_skip_dispatch);

#line 38 "ms_deisotope/_c/scoring.pxd"
};

#line 38 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase;

#line 38 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":50
 * 
 * 
 * cdef class IsotopicFitterBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public FitSelectorBase select
 */

#line 50 "ms_deisotope/_c/scoring.pxd"


#line 50 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase {

#line 50 "ms_deisotope/_c/scoring.pxd"
  double (*_evaluate)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, PyObject *, PyObject *, int __pyx_skip_dispatch);

#line 50 "ms_deisotope/_c/scoring.pxd"
  int (*reject)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *, int __pyx_skip_dispatch);

#line 50 "ms_deisotope/_c/scoring.pxd"
  int (*reject_score)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, double, int __pyx_skip_dispatch);

#line 50 "ms_deisotope/_c/scoring.pxd"
  int (*is_maximizing)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, int __pyx_skip_dispatch);

#line 50 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *(*_configure)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *, struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase *, PyObject *, int __pyx_skip_dispatch);

#line 50 "ms_deisotope/_c/scoring.pxd"
};

#line 50 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase;

#line 50 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":62
 * 
 * 
 * cdef class LeastSquaresFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 62 "ms_deisotope/_c/scoring.pxd"


#line 62 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter {

#line 62 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 62 "ms_deisotope/_c/scoring.pxd"
};

#line 62 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter;

#line 62 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":66
 * 
 * 
 * cdef class GTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 66 "ms_deisotope/_c/scoring.pxd"


#line 66 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter {

#line 66 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 66 "ms_deisotope/_c/scoring.pxd"
};

#line 66 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter;

#line 66 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":69
 *     pass
 * 
 * cdef class ScaledGTestFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

#line 69 "ms_deisotope/_c/scoring.pxd"


#line 69 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter {

#line 69 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 69 "ms_deisotope/_c/scoring.pxd"
};

#line 69 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter;

#line 69 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":72
 *     pass
 * 
 * cdef class MSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef public double mass_error_tolerance
 * 
 */

#line 72 "ms_deisotope/_c/scoring.pxd"


#line 72 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter {

#line 72 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 72 "ms_deisotope/_c/scoring.pxd"
  double (*score_peak)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak *, struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_14MSDeconVFitter_score_peak *__pyx_optional_args);

#line 72 "ms_deisotope/_c/scoring.pxd"
};

#line 72 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter;

#line 72 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":78
 * 
 * 
 * cdef class PenalizedMSDeconVFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mass_error_tolerance
 */

#line 78 "ms_deisotope/_c/scoring.pxd"


#line 78 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter {

#line 78 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 78 "ms_deisotope/_c/scoring.pxd"
};

#line 78 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter;

#line 78 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":84
 * 
 * 
 * cdef class FunctionScorer(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         object function
 */

#line 84 "ms_deisotope/_c/scoring.pxd"


#line 84 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer {

#line 84 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 84 "ms_deisotope/_c/scoring.pxd"
};

#line 84 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer;

#line 84 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":89
 * 
 * 
 * cdef class InterferenceDetection(object):             # <<<<<<<<<<<<<<
 *     cdef public PeakSet peaklist
 * 
 */

#line 89 "ms_deisotope/_c/scoring.pxd"


#line 89 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection {

#line 89 "ms_deisotope/_c/scoring.pxd"
  double (*detect_interference)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection *, PyObject *, struct __pyx_opt_args_12ms_deisotope_2_c_7scoring_21InterferenceDetection_detect_interference *__pyx_optional_args);

#line 89 "ms_deisotope/_c/scoring.pxd"
};

#line 89 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection;

#line 89 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":95
 * 
 * 
 * cdef class DistinctPatternFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public InterferenceDetection interference_detector
 */

#line 95 "ms_deisotope/_c/scoring.pxd"


#line 95 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter {

#line 95 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 95 "ms_deisotope/_c/scoring.pxd"
};

#line 95 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter;

#line 95 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":102
 *         public double domain_scale
 * 
 * cdef class ScaledPenalizedMSDeconvFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double scale_factor
 */

#line 102 "ms_deisotope/_c/scoring.pxd"


#line 102 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter {

#line 102 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 102 "ms_deisotope/_c/scoring.pxd"
  double (*_calculate_scale_factor)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet *, int __pyx_skip_dispatch);

#line 102 "ms_deisotope/_c/scoring.pxd"
  void (*scale_fitted_peaks)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, PyObject *, double);

#line 102 "ms_deisotope/_c/scoring.pxd"
  void (*scale_theoretical_peaks)(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *, PyObject *, double);

#line 102 "ms_deisotope/_c/scoring.pxd"
};

#line 102 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter;

#line 102 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/scoring.pxd":112
 * 
 * 
 * cdef class DotProductFitter(IsotopicFitterBase):             # <<<<<<<<<<<<<<
 *     pass
 */

#line 112 "ms_deisotope/_c/scoring.pxd"


#line 112 "ms_deisotope/_c/scoring.pxd"
struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter {

#line 112 "ms_deisotope/_c/scoring.pxd"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase __pyx_base;

#line 112 "ms_deisotope/_c/scoring.pxd"
};

#line 112 "ms_deisotope/_c/scoring.pxd"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter *__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter;

#line 112 "ms_deisotope/_c/scoring.pxd"


/* "ms_deisotope/_c/peak_set.pyx":93
 * 
 * @cython.freelist(100000)
 * cdef class _Index(object):             # <<<<<<<<<<<<<<
 *     """
 *     Stores the ordered index of an object under sort by `mz` and
 */

#line 93 "ms_deisotope/_c/peak_set.pyx"


#line 93 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index {

#line 93 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *(*_create)(size_t, size_t);

#line 93 "ms_deisotope/_c/peak_set.pyx"
};

#line 93 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index;

#line 93 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":129
 * 
 * @cython.freelist(1000000)
 * cdef class EnvelopePair(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, mz, intensity):
 */

#line 129 "ms_deisotope/_c/peak_set.pyx"


#line 129 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair {

#line 129 "ms_deisotope/_c/peak_set.pyx"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, int __pyx_skip_dispatch);

#line 129 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*_create)(double, double);

#line 129 "ms_deisotope/_c/peak_set.pyx"
};

#line 129 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair;

#line 129 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":237
 * 
 * @cython.freelist(100000)
 * cdef class Envelope(object):             # <<<<<<<<<<<<<<
 *     """
 *     Represents a sequence of (mz, intensity) pairs which store peak positions
 */

#line 237 "ms_deisotope/_c/peak_set.pyx"


#line 237 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope {

#line 237 "ms_deisotope/_c/peak_set.pyx"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);

#line 237 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t);

#line 237 "ms_deisotope/_c/peak_set.pyx"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch);

#line 237 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch);

#line 237 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*_create)(PyObject *);

#line 237 "ms_deisotope/_c/peak_set.pyx"
};

#line 237 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope;

#line 237 "ms_deisotope/_c/peak_set.pyx"
static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);

#line 237 "ms_deisotope/_c/peak_set.pyx"
static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t);

#line 237 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":323
 * 
 * 
 * cdef class DeconvolutedPeak(PeakBase):             # <<<<<<<<<<<<<<
 *     """
 *     Represent a single deconvoluted peak which represents an aggregated isotopic
 */

#line 323 "ms_deisotope/_c/peak_set.pyx"


#line 323 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak {

#line 323 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase __pyx_base;

#line 323 "ms_deisotope/_c/peak_set.pyx"
  int (*_eq)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch);

#line 323 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_create_simple)(double, double, int, double, double, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *);

#line 323 "ms_deisotope/_c/peak_set.pyx"
};

#line 323 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;

#line 323 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":455
 * 
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 */

#line 455 "ms_deisotope/_c/peak_set.pyx"


#line 455 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak {

#line 455 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;

#line 455 "ms_deisotope/_c/peak_set.pyx"
};

#line 455 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;

#line 455 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":497
 * 
 * 
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):             # <<<<<<<<<<<<<<
 *     """
 *     Extends :class:`DeconvolutedPeak` to also include a reference to
 */

#line 497 "ms_deisotope/_c/peak_set.pyx"


#line 497 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution {

#line 497 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_base;

#line 497 "ms_deisotope/_c/peak_set.pyx"
};

#line 497 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;

#line 497 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":544
 * 
 * 
 * cdef class DeconvolutedPeakSet:             # <<<<<<<<<<<<<<
 *     """
 *     Represents a collection of :class:`DeconvolutedPeak` instances under multiple orderings.
 */

#line 544 "ms_deisotope/_c/peak_set.pyx"


#line 544 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet {

#line 544 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*reindex)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*clone)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*copy)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_has_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*has_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*all_peaks_for)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*_get_nearest_peak)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, double *);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  size_t (*get_size)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*getitem)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t);

#line 544 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*getslice)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t, size_t);

#line 544 "ms_deisotope/_c/peak_set.pyx"
};

#line 544 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;

#line 544 "ms_deisotope/_c/peak_set.pyx"


/* "ms_deisotope/_c/peak_set.pyx":1139
 * INTERVAL_INDEX_SIZE = 0
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):             # <<<<<<<<<<<<<<
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL
 */

#line 1139 "ms_deisotope/_c/peak_set.pyx"


#line 1139 "ms_deisotope/_c/peak_set.pyx"
struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed {

#line 1139 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_base;

#line 1139 "ms_deisotope/_c/peak_set.pyx"
  void (*_build_index_arrays)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *);

#line 1139 "ms_deisotope/_c/peak_set.pyx"
  int (*_interval_for)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *, double, double, size_t *, size_t *);

#line 1139 "ms_deisotope/_c/peak_set.pyx"
};

#line 1139 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;

#line 1139 "ms_deisotope/_c/peak_set.pyx"


/* "View.MemoryView":105
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

#line 105 "View.MemoryView"


#line 105 "View.MemoryView"
struct __pyx_vtabstruct_array {

#line 105 "View.MemoryView"
  PyObject *(*get_memview)(struct __pyx_array_obj *);

#line 105 "View.MemoryView"
};

#line 105 "View.MemoryView"
static struct __pyx_vtabstruct_array *__pyx_vtabptr_array;

#line 105 "View.MemoryView"


/* "View.MemoryView":330
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

#line 330 "View.MemoryView"


#line 330 "View.MemoryView"
struct __pyx_vtabstruct_memoryview {

#line 330 "View.MemoryView"
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);

#line 330 "View.MemoryView"
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);

#line 330 "View.MemoryView"
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);

#line 330 "View.MemoryView"
};

#line 330 "View.MemoryView"
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;

#line 330 "View.MemoryView"


/* "View.MemoryView":965
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

#line 965 "View.MemoryView"


#line 965 "View.MemoryView"
struct __pyx_vtabstruct__memoryviewslice {

#line 965 "View.MemoryView"
  struct __pyx_vtabstruct_memoryview __pyx_base;

#line 965 "View.MemoryView"
};

#line 965 "View.MemoryView"
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* pyobject_as_double.proto */
static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyInt_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?\
 PyFloat_AS_DOUBLE(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

#define __Pyx_BufPtrCContig1d(type, buf, i0, s0) ((type)buf + i0)
/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* DivInt[Py_ssize_t].proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *); /*proto*/
/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* DivInt[long].proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* ClassMethod.proto */
#include "descrobject.h"
static CYTHON_UNUSED PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* FusedFunction.proto */
typedef struct {
    __pyx_CyFunctionObject func;
    PyObject *__signatures__;
    PyObject *type;
    PyObject *self;
} __pyx_FusedFunctionObject;
static PyObject *__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                                         PyObject *qualname, PyObject *closure,
                                         PyObject *module, PyObject *globals,
                                         PyObject *code);
static int __pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self);
static PyTypeObject *__pyx_FusedFunctionType = NULL;
static int __pyx_FusedFunction_init(void);
#define __Pyx_FusedFunction_USED

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

/* MemviewSliceIsContig.proto */
static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim);

/* OverlappingSlices.proto */
static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

/* Capsule.proto */
static CYTHON_INLINE PyObject *__pyx_capsule_create(void *p, const char *sig);

/* TypeInfoCompare.proto */
static int __pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b);

/* MemviewSliceValidateAndInit.proto */
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *, int writable_flag);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell;
static PyObject* __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell s);
/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* MemviewSliceCopyTemplate.proto */
static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

/* MemviewSliceInit.proto */
#define __Pyx_BUF_MAX_NDIMS %(BUF_MAX_NDIMS)d
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (memview->acquisition_count_aligned_p)
#define __pyx_get_slice_count(memview) (*__pyx_get_slice_count_pointer(memview))
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XDEC_MEMVIEW(slice, have_gil) __Pyx_XDEC_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *, int, int);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint32(npy_uint32 value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* BytesContains.proto */
static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character);

/* ImportNumPyArray.proto */
static PyObject *__pyx_numpy_ndarray = NULL;
static PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(size_t __pyx_v_neutral_mass, size_t __pyx_v_mz); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(double __pyx_v_mz, double __pyx_v_intensity); /* proto*/
static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto*/
static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, size_t __pyx_v_i); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(PyObject *__pyx_v_pairs); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple(double __pyx_v_neutral_mass, double __pyx_v_intensity, int __pyx_v_charge, double __pyx_v_score, double __pyx_v_mz, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_i); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_start, size_t __pyx_v_end); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double *__pyx_v_errout); /* proto*/
static void __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for *__pyx_optional_args); /* proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance, size_t *__pyx_v_start, size_t *__pyx_v_end); /* proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self); /* proto*/
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/

/* Module declarations from 'brainpy._c.composition' */
static PyTypeObject *__pyx_ptype_7brainpy_2_c_11composition_PyComposition = 0;
static PyObject **__pyx_vp_7brainpy_2_c_11composition_nist_mass = 0;
#define __pyx_v_7brainpy_2_c_11composition_nist_mass (*__pyx_vp_7brainpy_2_c_11composition_nist_mass)
static double *__pyx_vp_7brainpy_2_c_11composition_PROTON = 0;
#define __pyx_v_7brainpy_2_c_11composition_PROTON (*__pyx_vp_7brainpy_2_c_11composition_PROTON)
static struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable **__pyx_vp_7brainpy_2_c_11composition__ElementTable = 0;
#define __pyx_v_7brainpy_2_c_11composition__ElementTable (*__pyx_vp_7brainpy_2_c_11composition__ElementTable)

/* Module declarations from 'brainpy._c.double_vector' */

/* Module declarations from 'brainpy._c.isotopic_constants' */
static size_t *__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE = 0;
#define __pyx_v_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE (*__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE)

/* Module declarations from 'brainpy._c.isotopic_distribution' */
static PyTypeObject *__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = 0;

/* Module declarations from 'ms_peak_picker._c.peak_set' */
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = 0;
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = 0;
static size_t *__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE = 0;
#define __pyx_v_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE (*__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE)

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'ms_peak_picker._c.peak_index' */
static PyTypeObject *__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex = 0;

/* Module declarations from 'ms_deisotope._c.averagine' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache = 0;
static double *__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON = 0;
#define __pyx_v_12ms_deisotope_2_c_9averagine_PROTON (*__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON)
static double (*__pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio)(double, int, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio *__pyx_optional_args); /*proto*/

/* Module declarations from 'ms_deisotope._c.deconvoluter_base' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = 0;

/* Module declarations from 'ms_deisotope._c.scoring' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter = 0;
static double *__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY = 0;
#define __pyx_v_12ms_deisotope_2_c_7scoring_INFINITY (*__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY)

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'ms_deisotope._c.peak_set' */
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set__Index = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = 0;
static PyTypeObject *__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = 0;
static PyTypeObject *__pyx_array_type = 0;
static PyTypeObject *__pyx_MemviewEnum_type = 0;
static PyTypeObject *__pyx_memoryview_type = 0;
static PyTypeObject *__pyx_memoryviewslice_type = 0;
static size_t __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE;
static PyObject *__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter = 0;
static PyObject *__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter = 0;
static double __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon;
static double __pyx_v_12ms_deisotope_2_c_8peak_set_INF;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static int __pyx_memoryview_thread_locks_used;
static PyThread_type_lock __pyx_memoryview_thread_locks[8];
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(double *, double, double, size_t, size_t *, size_t *); /*proto*/
static double __pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(double, double); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(PyObject *, double, size_t, double); /*proto*/
static double __pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(double, double); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(double *, double, double, size_t, size_t *); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(PyObject *, double, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(PyObject *, double, size_t, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(PyObject *, double, double); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(PyObject *, double, size_t, size_t, double *); /*proto*/
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(PyObject *, double, double *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(double *, double, double, size_t, size_t, size_t *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(double *, double, double, size_t, size_t, size_t *, size_t *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, size_t); /*proto*/
static double *__pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(double, double, size_t); /*proto*/
static void __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_check_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *); /*proto*/
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, double); /*proto*/
static int __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *, double, size_t *, size_t *); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/
static void *__pyx_align_pointer(void *, size_t); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static PyObject *assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int, int); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, char *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, char *); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *, PyObject *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_float = { "float", NULL, sizeof(float), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_double = { "double", NULL, sizeof(double), { 0 }, 0, 'R', 0, 0 };
#define __Pyx_MODULE_NAME "ms_deisotope._c.peak_set"
extern int __pyx_module_is_main_ms_deisotope___c__peak_set;
int __pyx_module_is_main_ms_deisotope___c__peak_set = 0;

/* Implementation of 'ms_deisotope._c.peak_set' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_map;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_sorted;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_id;
static const char __pyx_k_[] = ", ";
static const char __pyx_k_O[] = "O";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_c[] = "c";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_s[] = "[%s]";
static const char __pyx_k__2[] = "";
static const char __pyx_k__3[] = "()";
static const char __pyx_k__4[] = "|";
static const char __pyx_k_eq[] = "_eq";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_m1[] = "m1";
static const char __pyx_k_m2[] = "m2";
static const char __pyx_k_mz[] = "mz";
static const char __pyx_k_d_d[] = "%d|%d";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_fit[] = "fit";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_inf[] = "inf";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_map[] = "map";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_obj[] = "obj";
static const char __pyx_k_area[] = "area";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_kind[] = "kind";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_ndim[] = "ndim";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_peak[] = "peak";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_step[] = "step";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_ASCII[] = "ASCII";
static const char __pyx_k_Index[] = "_Index";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_clone[] = "clone";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_pairs[] = "pairs";
static const char __pyx_k_peaks[] = "peaks";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_score[] = "score";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_charge[] = "charge";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_unpack[] = "unpack";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_use_mz[] = "use_mz";
static const char __pyx_k_convert[] = "convert";
static const char __pyx_k_fortran[] = "fortran";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_indexed[] = "indexed";
static const char __pyx_k_memview[] = "memview";
static const char __pyx_k_reindex[] = "reindex";
static const char __pyx_k_Ellipsis[] = "Ellipsis";
static const char __pyx_k_Envelope[] = "Envelope";
static const char __pyx_k_defaults[] = "defaults";
static const char __pyx_k_envelope[] = "envelope";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_has_peak[] = "has_peak";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_operator[] = "operator";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_solution[] = "solution";
static const char __pyx_k_0_4f_0_2f[] = "(%0.4f, %0.2f)";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_converted[] = "converted";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_from_peak[] = "from_peak";
static const char __pyx_k_intensity[] = "intensity";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_tolerance[] = "tolerance";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_attrgetter[] = "attrgetter";
static const char __pyx_k_drift_time[] = "drift_time";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_signatures[] = "signatures";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_EnvelopePair[] = "EnvelopePair";
static const char __pyx_k_average_mass[] = "average_mass";
static const char __pyx_k_center_value[] = "center_value";
static const char __pyx_k_envelope_obj[] = "envelope_obj";
static const char __pyx_k_neutral_mass[] = "neutral_mass";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_a_to_a2_ratio[] = "a_to_a2_ratio";
static const char __pyx_k_all_peaks_for[] = "all_peaks_for";
static const char __pyx_k_envelope_list[] = "envelope_list";
static const char __pyx_k_pyx_getbuffer[] = "__pyx_getbuffer";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_View_MemoryView[] = "View.MemoryView";
static const char __pyx_k_allocate_buffer[] = "allocate_buffer";
static const char __pyx_k_chosen_for_msms[] = "chosen_for_msms";
static const char __pyx_k_dtype_is_object[] = "dtype_is_object";
static const char __pyx_k_error_tolerance[] = "error_tolerance";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_release_buffers[] = "_release_buffers";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_signal_to_noise[] = "signal_to_noise";
static const char __pyx_k_DeconvolutedPeak[] = "DeconvolutedPeak";
static const char __pyx_k_current_envelope[] = "current_envelope";
static const char __pyx_k_decode_envelopes[] = "decode_envelopes";
static const char __pyx_k_pyx_unpickle_Enum[] = "__pyx_unpickle_Enum";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_most_abundant_mass[] = "most_abundant_mass";
static const char __pyx_k_strided_and_direct[] = "<strided and direct>";
static const char __pyx_k_DeconvolutedPeakSet[] = "DeconvolutedPeakSet";
static const char __pyx_k_EnvelopePair___iter[] = "EnvelopePair.__iter__";
static const char __pyx_k_INTERVAL_INDEX_SIZE[] = "INTERVAL_INDEX_SIZE";
static const char __pyx_k_Index_reconstructor[] = "_Index_reconstructor";
static const char __pyx_k_init___locals_lambda[] = "__init__.<locals>.<lambda>";
static const char __pyx_k_strided_and_indirect[] = "<strided and indirect>";
static const char __pyx_k_contiguous_and_direct[] = "<contiguous and direct>";
static const char __pyx_k_repr___locals_genexpr[] = "__repr__.<locals>.genexpr";
static const char __pyx_k_Envelope_reconstructor[] = "Envelope_reconstructor";
static const char __pyx_k_MemoryView_of_r_object[] = "<MemoryView of %r object>";
static const char __pyx_k_full_width_at_half_max[] = "full_width_at_half_max";
static const char __pyx_k_MemoryView_of_r_at_0x_x[] = "<MemoryView of %r at 0x%x>";
static const char __pyx_k_contiguous_and_indirect[] = "<contiguous and indirect>";
static const char __pyx_k_Cannot_index_with_type_s[] = "Cannot index with type '%s'";
static const char __pyx_k_DeconvolutedPeakSolution[] = "DeconvolutedPeakSolution";
static const char __pyx_k_getitem___locals_genexpr[] = "__getitem__.<locals>.genexpr";
static const char __pyx_k_ms_deisotope__c_peak_set[] = "ms_deisotope._c.peak_set";
static const char __pyx_k_Invalid_shape_in_axis_d_d[] = "Invalid shape in axis %d: %d.";
static const char __pyx_k_DeconvolutedPeakSetIndexed[] = "DeconvolutedPeakSetIndexed";
static const char __pyx_k_DeconvolutedPeakSet_d_Peaks[] = "<DeconvolutedPeakSet %d Peaks>";
static const char __pyx_k_IonMobilityDeconvolutedPeak[] = "IonMobilityDeconvolutedPeak";
static const char __pyx_k_No_matching_signature_found[] = "No matching signature found";
static const char __pyx_k_itemsize_0_for_cython_array[] = "itemsize <= 0 for cython.array";
static const char __pyx_k_ms_deisotope__c_peak_set_pyx[] = "ms_deisotope/_c/peak_set.pyx";
static const char __pyx_k_unable_to_allocate_array_data[] = "unable to allocate array data.";
static const char __pyx_k_strided_and_direct_or_indirect[] = "<strided and direct or indirect>";
static const char __pyx_k_DeconvolutedPeakSolution___iter[] = "DeconvolutedPeakSolution.__iter__";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_self___class_____name___a_to_a2[] = "{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise})";
static const char __pyx_k_Buffer_view_does_not_expose_stri[] = "Buffer view does not expose strides";
static const char __pyx_k_Can_only_create_a_buffer_that_is[] = "Can only create a buffer that is contiguous in memory.";
static const char __pyx_k_Cannot_assign_to_read_only_memor[] = "Cannot assign to read-only memoryview";
static const char __pyx_k_Cannot_create_writable_memory_vi[] = "Cannot create writable memory view from read-only memoryview";
static const char __pyx_k_Empty_shape_tuple_for_cython_arr[] = "Empty shape tuple for cython.array";
static const char __pyx_k_Expected_at_least_d_argument_s_g[] = "Expected at least %d argument%s, got %d";
static const char __pyx_k_Function_call_with_ambiguous_arg[] = "Function call with ambiguous argument types";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))";
static const char __pyx_k_Indirect_dimensions_not_supporte[] = "Indirect dimensions not supported";
static const char __pyx_k_Invalid_mode_expected_c_or_fortr[] = "Invalid mode, expected 'c' or 'fortran', got %s";
static const char __pyx_k_Out_of_bounds_on_buffer_access_a[] = "Out of bounds on buffer access (axis %d)";
static const char __pyx_k_Unable_to_convert_item_to_object[] = "Unable to convert item to object";
static const char __pyx_k_got_differing_extents_in_dimensi[] = "got differing extents in dimension %d (got %d and %d)";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_unable_to_allocate_shape_and_str[] = "unable to allocate shape and strides.";
static const char __pyx_k_self___class_____name___a_to_a2_2[] = "{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise},drift_time={self.drift_time})";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_0_4f_0_2f;
static PyObject *__pyx_n_s_ASCII;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_kp_s_Buffer_view_does_not_expose_stri;
static PyObject *__pyx_kp_s_Can_only_create_a_buffer_that_is;
static PyObject *__pyx_kp_s_Cannot_assign_to_read_only_memor;
static PyObject *__pyx_kp_s_Cannot_create_writable_memory_vi;
static PyObject *__pyx_kp_s_Cannot_index_with_type_s;
static PyObject *__pyx_n_s_DeconvolutedPeak;
static PyObject *__pyx_n_s_DeconvolutedPeakSet;
static PyObject *__pyx_n_s_DeconvolutedPeakSetIndexed;
static PyObject *__pyx_kp_s_DeconvolutedPeakSet_d_Peaks;
static PyObject *__pyx_n_s_DeconvolutedPeakSolution;
static PyObject *__pyx_n_s_DeconvolutedPeakSolution___iter;
static PyObject *__pyx_n_s_Ellipsis;
static PyObject *__pyx_kp_s_Empty_shape_tuple_for_cython_arr;
static PyObject *__pyx_n_s_Envelope;
static PyObject *__pyx_n_s_EnvelopePair;
static PyObject *__pyx_n_s_EnvelopePair___iter;
static PyObject *__pyx_n_s_Envelope_reconstructor;
static PyObject *__pyx_kp_s_Expected_at_least_d_argument_s_g;
static PyObject *__pyx_kp_s_Function_call_with_ambiguous_arg;
static PyObject *__pyx_n_s_INTERVAL_INDEX_SIZE;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_n_s_Index;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_Index_reconstructor;
static PyObject *__pyx_kp_s_Indirect_dimensions_not_supporte;
static PyObject *__pyx_kp_s_Invalid_mode_expected_c_or_fortr;
static PyObject *__pyx_kp_s_Invalid_shape_in_axis_d_d;
static PyObject *__pyx_n_s_IonMobilityDeconvolutedPeak;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_MemoryView_of_r_at_0x_x;
static PyObject *__pyx_kp_s_MemoryView_of_r_object;
static PyObject *__pyx_kp_s_No_matching_signature_found;
static PyObject *__pyx_n_b_O;
static PyObject *__pyx_kp_s_Out_of_bounds_on_buffer_access_a;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_convert_item_to_object;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_View_MemoryView;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_kp_s__4;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_a_to_a2_ratio;
static PyObject *__pyx_n_s_all_peaks_for;
static PyObject *__pyx_n_s_allocate_buffer;
static PyObject *__pyx_n_s_area;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_attrgetter;
static PyObject *__pyx_n_s_average_mass;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_n_s_center_value;
static PyObject *__pyx_n_s_charge;
static PyObject *__pyx_n_s_chosen_for_msms;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clone;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_kp_s_contiguous_and_direct;
static PyObject *__pyx_kp_s_contiguous_and_indirect;
static PyObject *__pyx_n_s_convert;
static PyObject *__pyx_n_s_converted;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_current_envelope;
static PyObject *__pyx_kp_s_d_d;
static PyObject *__pyx_n_s_decode_envelopes;
static PyObject *__pyx_n_s_defaults;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_n_s_drift_time;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dtype_is_object;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_envelope;
static PyObject *__pyx_n_s_envelope_list;
static PyObject *__pyx_n_s_envelope_obj;
static PyObject *__pyx_n_s_eq;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_error_tolerance;
static PyObject *__pyx_n_s_fit;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_fortran;
static PyObject *__pyx_n_u_fortran;
static PyObject *__pyx_n_s_from_peak;
static PyObject *__pyx_n_s_full_width_at_half_max;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getitem___locals_genexpr;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_kp_s_got_differing_extents_in_dimensi;
static PyObject *__pyx_n_s_has_peak;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_indexed;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_init___locals_lambda;
static PyObject *__pyx_n_s_intensity;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_kp_s_itemsize_0_for_cython_array;
static PyObject *__pyx_n_s_iter;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_m1;
static PyObject *__pyx_n_s_m2;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_s_memview;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_most_abundant_mass;
static PyObject *__pyx_n_s_ms_deisotope__c_peak_set;
static PyObject *__pyx_kp_s_ms_deisotope__c_peak_set_pyx;
static PyObject *__pyx_n_s_mz;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_neutral_mass;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_operator;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_pairs;
static PyObject *__pyx_n_s_peak;
static PyObject *__pyx_n_s_peaks;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_getbuffer;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Enum;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reindex;
static PyObject *__pyx_n_s_release_buffers;
static PyObject *__pyx_n_s_repr___locals_genexpr;
static PyObject *__pyx_kp_s_s;
static PyObject *__pyx_n_s_score;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self___class_____name___a_to_a2;
static PyObject *__pyx_kp_s_self___class_____name___a_to_a2_2;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_signal_to_noise;
static PyObject *__pyx_n_s_signatures;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_solution;
static PyObject *__pyx_n_s_sorted;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_strided_and_direct;
static PyObject *__pyx_kp_s_strided_and_direct_or_indirect;
static PyObject *__pyx_kp_s_strided_and_indirect;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_tolerance;
static PyObject *__pyx_kp_s_unable_to_allocate_array_data;
static PyObject *__pyx_kp_s_unable_to_allocate_shape_and_str;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_mz;
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_mz, PyObject *__pyx_v_intensity); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_pairs); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static Py_hash_t __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area, PyObject *__pyx_v_drift_time); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_solution, PyObject *__pyx_v_fit, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_peak, PyObject *__pyx_v_solution); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_peaks); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance, int __pyx_v_use_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_m1, PyObject *__pyx_v_m2, CYTHON_UNUSED PyObject *__pyx_v_tolerance, PyObject *__pyx_v_use_mz); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array); /* proto */
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_peaks); /* proto */
static void __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_d); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_index_size); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, size_t __pyx_v_i); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1eneg_5;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_112105877;
static PyObject *__pyx_int_136983863;
static PyObject *__pyx_int_184977713;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__24;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__41;
/* Late includes */

/* "ms_deisotope/_c/peak_set.pyx":31
 * 
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

#line 31 "ms_deisotope/_c/peak_set.pyx"


#line 31 "ms_deisotope/_c/peak_set.pyx"
static double __pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(double __pyx_v_x, double __pyx_v_y) {

#line 31 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_r;

#line 31 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 31 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("ppm_error", 0);

  /* "ms_deisotope/_c/peak_set.pyx":32
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):
 *     return (x - y) / y             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 32 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((__pyx_v_x - __pyx_v_y) / __pyx_v_y);

#line 32 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":31
 * 
 * @cython.cdivision
 * cdef double ppm_error(double x, double y):             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

#line 31 "ms_deisotope/_c/peak_set.pyx"


#line 31 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 31 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 31 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 31 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 31 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 45 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(PyObject *__pyx_v_neutral_mass, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor *__pyx_optional_args) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_v_mz = ((PyObject *)Py_None);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_reconstructor", 0);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_mz = __pyx_optional_args->mz;

#line 45 "ms_deisotope/_c/peak_set.pyx"
    }

#line 45 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":46
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):             # <<<<<<<<<<<<<<
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 */

#line 46 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyInt_Check(__pyx_v_neutral_mass); 

#line 46 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 46 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":47
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)             # <<<<<<<<<<<<<<
 *     elif isinstance(neutral_mass, bytearray):
 *         return _Index_from_bytes(neutral_mass)
 */

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_neutral_mass); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 47, __pyx_L1_error)

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_mz); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 47, __pyx_L1_error)

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_t_3, __pyx_t_4)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 47, __pyx_L1_error)

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_5;

#line 47 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

#line 47 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":46
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):
 *     if isinstance(neutral_mass, int):             # <<<<<<<<<<<<<<
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 */

#line 46 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":48
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):             # <<<<<<<<<<<<<<
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 */

#line 48 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyByteArray_Check(__pyx_v_neutral_mass); 

#line 48 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_2 != 0);

#line 48 "ms_deisotope/_c/peak_set.pyx"
  if (likely(__pyx_t_1)) {

    /* "ms_deisotope/_c/peak_set.pyx":49
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):
 *         return _Index_from_bytes(neutral_mass)             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError(type(neutral_mass))
 */

#line 49 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 49 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(PyByteArray_CheckExact(__pyx_v_neutral_mass))||((__pyx_v_neutral_mass) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytearray", Py_TYPE(__pyx_v_neutral_mass)->tp_name), 0))) __PYX_ERR(0, 49, __pyx_L1_error)

#line 49 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(((PyObject*)__pyx_v_neutral_mass), 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 49, __pyx_L1_error)

#line 49 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 49 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_5;

#line 49 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

#line 49 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":48
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 *     elif isinstance(neutral_mass, bytearray):             # <<<<<<<<<<<<<<
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 */

#line 48 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":51
 *         return _Index_from_bytes(neutral_mass)
 *     else:
 *         raise TypeError(type(neutral_mass))             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 51 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 51 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, ((PyObject *)Py_TYPE(__pyx_v_neutral_mass))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 51, __pyx_L1_error)

#line 51 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 51 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);

#line 51 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 51 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 51, __pyx_L1_error)

#line 51 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":45
 * 
 * 
 * cpdef _Index_reconstructor(object neutral_mass, object mz=None):             # <<<<<<<<<<<<<<
 *     if isinstance(neutral_mass, int):
 *         return _Index._create(neutral_mass, mz)
 */

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pyx"
}

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 45 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 45 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set__Index_reconstructor[] = "_Index_reconstructor(neutral_mass, mz=None)";

#line 45 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0
#line 45 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_mz = 0
#line 45 "ms_deisotope/_c/peak_set.pyx"
;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_reconstructor (wrapper)", 0);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  {

#line 45 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_mz,0};

#line 45 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 45 "ms_deisotope/_c/peak_set.pyx"
    values[1] = ((PyObject *)Py_None);

#line 45 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 45 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 45 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 45 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 45 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 45 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 45 "ms_deisotope/_c/peak_set.pyx"
      }

#line 45 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 45 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 45 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 45 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 45 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 45 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 45 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);

#line 45 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 45 "ms_deisotope/_c/peak_set.pyx"
        }

#line 45 "ms_deisotope/_c/peak_set.pyx"
      }

#line 45 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_Index_reconstructor") < 0)) __PYX_ERR(0, 45, __pyx_L3_error)

#line 45 "ms_deisotope/_c/peak_set.pyx"
      }

#line 45 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 45 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 45 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 45 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 45 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 45 "ms_deisotope/_c/peak_set.pyx"
      }

#line 45 "ms_deisotope/_c/peak_set.pyx"
    }

#line 45 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = values[0];

#line 45 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mz = values[1];

#line 45 "ms_deisotope/_c/peak_set.pyx"
  }

#line 45 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_Index_reconstructor", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 45 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 45, __pyx_L3_error)

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 45 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(__pyx_self, __pyx_v_neutral_mass, __pyx_v_mz);

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pyx"
}

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set__Index_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz) {

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set__Index_reconstructor __pyx_t_2;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_reconstructor", 0);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.mz = __pyx_v_mz;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_reconstructor(__pyx_v_neutral_mass, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 45 "ms_deisotope/_c/peak_set.pyx"


#line 45 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 45 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":54
 * 
 * 
 * cpdef _Index_to_bytes(_Index index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j, m
 */

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/

#line 54 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 54 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_m
#line 54 "ms_deisotope/_c/peak_set.pyx"
;
  char *__pyx_v_buff
#line 54 "ms_deisotope/_c/peak_set.pyx"
;
  __pyx_t_5numpy_uint64_t __pyx_v_n
#line 54 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_out = 0
#line 54 "ms_deisotope/_c/peak_set.pyx"
;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 54 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_to_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":61
 *         bytearray out
 * 
 *     m = sizeof(np.uint64_t) * 2             # <<<<<<<<<<<<<<
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 */

#line 61 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_m = ((sizeof(__pyx_t_5numpy_uint64_t)) * 2);

  /* "ms_deisotope/_c/peak_set.pyx":62
 * 
 *     m = sizeof(np.uint64_t) * 2
 *     buff = <char*>malloc(sizeof(char) * m)             # <<<<<<<<<<<<<<
 *     j = 0
 *     n = index.neutral_mass
 */

#line 62 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_buff = ((char *)malloc(((sizeof(char)) * __pyx_v_m)));

  /* "ms_deisotope/_c/peak_set.pyx":63
 *     m = sizeof(np.uint64_t) * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0             # <<<<<<<<<<<<<<
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 */

#line 63 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":64
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 *     n = index.neutral_mass             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */

#line 64 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_index->neutral_mass;

#line 64 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":65
 *     j = 0
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     n = index.mz
 */

#line 65 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":66
 *     n = index.neutral_mass
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 */

#line 66 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":67
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     n = index.mz             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */

#line 67 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_index->mz;

#line 67 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":68
 *     j += sizeof(np.uint64_t)
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 */

#line 68 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":69
 *     n = index.mz
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 */

#line 69 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":70
 *     memcpy(&buff[j], &n, sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)             # <<<<<<<<<<<<<<
 *     free(buff)
 *     return out
 */

#line 70 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyByteArray_FromStringAndSize(((char *)__pyx_v_buff), __pyx_v_m); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)

#line 70 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 70 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_out = ((PyObject*)__pyx_t_2);

#line 70 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":71
 *     j += sizeof(np.uint64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */

#line 71 "ms_deisotope/_c/peak_set.pyx"
  free(__pyx_v_buff);

  /* "ms_deisotope/_c/peak_set.pyx":72
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 72 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 72 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_out);

#line 72 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_out;

#line 72 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":54
 * 
 * 
 * cpdef _Index_to_bytes(_Index index):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j, m
 */

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_out);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 54 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 54 "ms_deisotope/_c/peak_set.pyx"
}

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 54 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/

#line 54 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes[] = "_Index_to_bytes(_Index index)";

#line 54 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_index) {

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_to_bytes (wrapper)", 0);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_index), __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index, 1, "index", 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_r = 
#line 54 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(__pyx_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_index));

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 54 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 54 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 54 "ms_deisotope/_c/peak_set.pyx"
}

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_index) {

#line 54 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 54 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_to_bytes", 0);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_to_bytes(__pyx_v_index, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 54 "ms_deisotope/_c/peak_set.pyx"


#line 54 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 54 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 54 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 54 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":75
 * 
 * 
 * cpdef _Index_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j
 */

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/

#line 75 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(PyObject *__pyx_v_view, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_j
#line 75 "ms_deisotope/_c/peak_set.pyx"
;
  __pyx_t_5numpy_uint64_t __pyx_v_val
#line 75 "ms_deisotope/_c/peak_set.pyx"
;
  char *__pyx_v_buff
#line 75 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_inst = 0
#line 75 "ms_deisotope/_c/peak_set.pyx"
;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 75 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  char *__pyx_t_2;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_from_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":81
 *         char* buff
 *         _Index inst
 *     inst = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *     j = 0
 *     buff = view
 */

#line 81 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)

#line 81 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 81 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);

#line 81 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":82
 *         _Index inst
 *     inst = _Index._create(0, 0)
 *     j = 0             # <<<<<<<<<<<<<<
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 */

#line 82 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":83
 *     inst = _Index._create(0, 0)
 *     j = 0
 *     buff = view             # <<<<<<<<<<<<<<
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */

#line 83 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_AsWritableString(__pyx_v_view); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 83, __pyx_L1_error)

#line 83 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_buff = __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":84
 *     j = 0
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val
 */

#line 84 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&__pyx_v_val), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":85
 *     buff = view
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 */

#line 85 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":86
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val             # <<<<<<<<<<<<<<
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 */

#line 86 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->neutral_mass = __pyx_v_val;

  /* "ms_deisotope/_c/peak_set.pyx":87
 *     j += sizeof(np.uint64_t)
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val
 */

#line 87 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&__pyx_v_val), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint64_t))));

  /* "ms_deisotope/_c/peak_set.pyx":88
 *     inst.neutral_mass = val
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)             # <<<<<<<<<<<<<<
 *     inst.mz = val
 *     return inst
 */

#line 88 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint64_t)));

  /* "ms_deisotope/_c/peak_set.pyx":89
 *     memcpy(&val, &buff[j], sizeof(np.uint64_t))
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val             # <<<<<<<<<<<<<<
 *     return inst
 * 
 */

#line 89 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->mz = __pyx_v_val;

  /* "ms_deisotope/_c/peak_set.pyx":90
 *     j += sizeof(np.uint64_t)
 *     inst.mz = val
 *     return inst             # <<<<<<<<<<<<<<
 * 
 * @cython.freelist(100000)
 */

#line 90 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 90 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));

#line 90 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_v_inst);

#line 90 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":75
 * 
 * 
 * cpdef _Index_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t j
 */

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 75 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 75 "ms_deisotope/_c/peak_set.pyx"
}

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 75 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/

#line 75 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes[] = "_Index_from_bytes(bytearray view)";

#line 75 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view) {

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_from_bytes (wrapper)", 0);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_view), (&PyByteArray_Type), 1, "view", 1))) __PYX_ERR(0, 75, __pyx_L1_error)
  __pyx_r = 
#line 75 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(__pyx_self, ((PyObject*)__pyx_v_view));

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 75 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 75 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 75 "ms_deisotope/_c/peak_set.pyx"
}

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view) {

#line 75 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 75 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_Index_from_bytes", 0);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set__Index_from_bytes(__pyx_v_view, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 75 "ms_deisotope/_c/peak_set.pyx"


#line 75 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 75 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 75 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 75 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":105
 *         Index of this object (or its parent), ordered by neutral mass
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz
 */

#line 105 "ms_deisotope/_c/peak_set.pyx"


#line 105 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 105 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 105 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0
#line 105 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_mz = 0
#line 105 "ms_deisotope/_c/peak_set.pyx"
;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 105 "ms_deisotope/_c/peak_set.pyx"
  {

#line 105 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_mz,0};

#line 105 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 105 "ms_deisotope/_c/peak_set.pyx"
    values[0] = ((PyObject *)__pyx_int_0);

#line 105 "ms_deisotope/_c/peak_set.pyx"
    values[1] = ((PyObject *)__pyx_int_0);

#line 105 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 105 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 105 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 105 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 105 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 105 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 105 "ms_deisotope/_c/peak_set.pyx"
      }

#line 105 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 105 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 105 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass);

#line 105 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[0] = value; kw_args--; }

#line 105 "ms_deisotope/_c/peak_set.pyx"
        }

#line 105 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 105 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 105 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);

#line 105 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 105 "ms_deisotope/_c/peak_set.pyx"
        }

#line 105 "ms_deisotope/_c/peak_set.pyx"
      }

#line 105 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 105, __pyx_L3_error)

#line 105 "ms_deisotope/_c/peak_set.pyx"
      }

#line 105 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 105 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 105 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 105 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  0: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
break;
        default: 
#line 105 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 105 "ms_deisotope/_c/peak_set.pyx"
      }

#line 105 "ms_deisotope/_c/peak_set.pyx"
    }

#line 105 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = values[0];

#line 105 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mz = values[1];

#line 105 "ms_deisotope/_c/peak_set.pyx"
  }

#line 105 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 105 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 105, __pyx_L3_error)

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 105 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 105 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_mz);

#line 105 "ms_deisotope/_c/peak_set.pyx"


#line 105 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 105 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 105 "ms_deisotope/_c/peak_set.pyx"
}

#line 105 "ms_deisotope/_c/peak_set.pyx"


#line 105 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_mz) {

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 105 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":106
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):
 *         self.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         self.mz = mz
 * 
 */

#line 106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_neutral_mass); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 106, __pyx_L1_error)

#line 106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->neutral_mass = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":107
 *     def __init__(self, neutral_mass=0, mz=0):
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 107 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_mz); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L1_error)

#line 107 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->mz = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":105
 *         Index of this object (or its parent), ordered by neutral mass
 *     """
 *     def __init__(self, neutral_mass=0, mz=0):             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.mz = mz
 */

#line 105 "ms_deisotope/_c/peak_set.pyx"


#line 105 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 105 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 105 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 105 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":109
 *         self.mz = mz
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 */

#line 109 "ms_deisotope/_c/peak_set.pyx"


#line 109 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 109 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__(PyObject *__pyx_v_self); /*proto*/

#line 109 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__(PyObject *__pyx_v_self) {

#line 109 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 109 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

#line 109 "ms_deisotope/_c/peak_set.pyx"


#line 109 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 109 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 109 "ms_deisotope/_c/peak_set.pyx"
}

#line 109 "ms_deisotope/_c/peak_set.pyx"


#line 109 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {

#line 109 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 109 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":110
 * 
 *     def __repr__(self):
 *         return "%d|%d" % (self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     def clone(self):
 */

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 110, __pyx_L1_error)

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 110, __pyx_L1_error)

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_d_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 110 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 110 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":109
 *         self.mz = mz
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 */

#line 109 "ms_deisotope/_c/peak_set.pyx"


#line 109 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 109 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 109 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 109 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":112
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 */

#line 112 "ms_deisotope/_c/peak_set.pyx"


#line 112 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 112 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 112 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_4clone[] = "_Index.clone(self)";

#line 112 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 112 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 112 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

#line 112 "ms_deisotope/_c/peak_set.pyx"


#line 112 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 112 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 112 "ms_deisotope/_c/peak_set.pyx"
}

#line 112 "ms_deisotope/_c/peak_set.pyx"


#line 112 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {

#line 112 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 112 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

  /* "ms_deisotope/_c/peak_set.pyx":113
 * 
 *     def clone(self):
 *         return _Index._create(self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 113 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_v_self->neutral_mass, __pyx_v_self->mz)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)

#line 113 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 113 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":112
 *         return "%d|%d" % (self.neutral_mass, self.mz)
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 */

#line 112 "ms_deisotope/_c/peak_set.pyx"


#line 112 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 112 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 112 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 112 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":115
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)
 * 
 */

#line 115 "ms_deisotope/_c/peak_set.pyx"


#line 115 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 115 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 115 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__[] = "_Index.__reduce__(self)";

#line 115 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 115 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

#line 115 "ms_deisotope/_c/peak_set.pyx"


#line 115 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 115 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 115 "ms_deisotope/_c/peak_set.pyx"
}

#line 115 "ms_deisotope/_c/peak_set.pyx"


#line 115 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":116
 * 
 *     def __reduce__(self):
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Index_reconstructor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 116, __pyx_L1_error)

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_3;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 116 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":115
 *         return _Index._create(self.neutral_mass, self.mz)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return _Index_reconstructor, (self.neutral_mass, self.mz)
 * 
 */

#line 115 "ms_deisotope/_c/peak_set.pyx"


#line 115 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 115 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 115 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 115 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":119
 * 
 *     @staticmethod
 *     cdef _Index _create(size_t neutral_mass, size_t mz):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index inst
 */

#line 119 "ms_deisotope/_c/peak_set.pyx"


#line 119 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(size_t __pyx_v_neutral_mass, size_t __pyx_v_mz) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_inst = 0
#line 119 "ms_deisotope/_c/peak_set.pyx"
;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_r = NULL;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 119 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":122
 *         cdef:
 *             _Index inst
 *         inst = _Index.__new__(_Index)             # <<<<<<<<<<<<<<
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz
 */

#line 122 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set__Index), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)

#line 122 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));

#line 122 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);

#line 122 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":123
 *             _Index inst
 *         inst = _Index.__new__(_Index)
 *         inst.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         return inst
 */

#line 123 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->neutral_mass = __pyx_v_neutral_mass;

  /* "ms_deisotope/_c/peak_set.pyx":124
 *         inst = _Index.__new__(_Index)
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */

#line 124 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":125
 *         inst.neutral_mass = neutral_mass
 *         inst.mz = mz
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 125 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 125 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));

#line 125 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_inst;

#line 125 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":119
 * 
 *     @staticmethod
 *     cdef _Index _create(size_t neutral_mass, size_t mz):             # <<<<<<<<<<<<<<
 *         cdef:
 *             _Index inst
 */

#line 119 "ms_deisotope/_c/peak_set.pyx"


#line 119 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index._create", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 119 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 119 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 119 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":7
 * cdef class _Index:
 *     cdef:
 *         public size_t neutral_mass             # <<<<<<<<<<<<<<
 *         public size_t mz
 * 
 */

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 7 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 7 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(PyObject *__pyx_v_self) {

#line 7 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 7 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 7 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
}

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {

#line 7 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 7 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.neutral_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 7 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
}

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 7 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 7 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 7 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 7 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
}

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value) {

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 7 "ms_deisotope/_c/peak_set.pxd"
  size_t __pyx_t_1;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 7, __pyx_L1_error)

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->neutral_mass = __pyx_t_1;

#line 7 "ms_deisotope/_c/peak_set.pxd"


#line 7 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.neutral_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 7 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 7 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 7 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":8
 *     cdef:
 *         public size_t neutral_mass
 *         public size_t mz             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 8 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 8 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(PyObject *__pyx_v_self) {

#line 8 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 8 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self));

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 8 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
}

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self) {

#line 8 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 8 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.mz.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 8 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
}

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 8 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 8 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 8 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 8 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
}

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_6_Index_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_v_self, PyObject *__pyx_v_value) {

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 8 "ms_deisotope/_c/peak_set.pxd"
  size_t __pyx_t_1;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 8, __pyx_L1_error)

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->mz = __pyx_t_1;

#line 8 "ms_deisotope/_c/peak_set.pxd"


#line 8 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._Index.mz.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 8 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 8 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 8 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":131
 * cdef class EnvelopePair(object):
 * 
 *     def __init__(self, mz, intensity):             # <<<<<<<<<<<<<<
 *         self.mz = mz
 *         self.intensity = intensity
 */

#line 131 "ms_deisotope/_c/peak_set.pyx"


#line 131 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 131 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 131 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mz = 0
#line 131 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_intensity = 0
#line 131 "ms_deisotope/_c/peak_set.pyx"
;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 131 "ms_deisotope/_c/peak_set.pyx"
  {

#line 131 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mz,&__pyx_n_s_intensity,0};

#line 131 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 131 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 131 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 131 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 131 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 131 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 131 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 131 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 131 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 131 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 131 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 131 "ms_deisotope/_c/peak_set.pyx"
      }

#line 131 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 131 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 131 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 131 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz)) != 0)) kw_args--;
        else 
#line 131 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 131 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 131 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 131 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 131 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); 
#line 131 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 131, __pyx_L3_error)

#line 131 "ms_deisotope/_c/peak_set.pyx"
        }

#line 131 "ms_deisotope/_c/peak_set.pyx"
      }

#line 131 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 131 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 131, __pyx_L3_error)

#line 131 "ms_deisotope/_c/peak_set.pyx"
      }

#line 131 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {

#line 131 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 131 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 131 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 131 "ms_deisotope/_c/peak_set.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 131 "ms_deisotope/_c/peak_set.pyx"
    }

#line 131 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mz = values[0];

#line 131 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_intensity = values[1];

#line 131 "ms_deisotope/_c/peak_set.pyx"
  }

#line 131 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 131 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 131, __pyx_L3_error)

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 131 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 131 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), __pyx_v_mz, __pyx_v_intensity);

#line 131 "ms_deisotope/_c/peak_set.pyx"


#line 131 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 131 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 131 "ms_deisotope/_c/peak_set.pyx"
}

#line 131 "ms_deisotope/_c/peak_set.pyx"


#line 131 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_mz, PyObject *__pyx_v_intensity) {

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 131 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_1;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":132
 * 
 *     def __init__(self, mz, intensity):
 *         self.mz = mz             # <<<<<<<<<<<<<<
 *         self.intensity = intensity
 * 
 */

#line 132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_mz); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)

#line 132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->mz = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":133
 *     def __init__(self, mz, intensity):
 *         self.mz = mz
 *         self.intensity = intensity             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */

#line 133 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_intensity); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 133, __pyx_L1_error)

#line 133 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->intensity = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":131
 * cdef class EnvelopePair(object):
 * 
 *     def __init__(self, mz, intensity):             # <<<<<<<<<<<<<<
 *         self.mz = mz
 *         self.intensity = intensity
 */

#line 131 "ms_deisotope/_c/peak_set.pyx"


#line 131 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 131 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 131 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":135
 *         self.intensity = intensity
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.mz
 */

#line 135 "ms_deisotope/_c/peak_set.pyx"


#line 135 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 135 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/

#line 135 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {

#line 135 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 135 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_i));

#line 135 "ms_deisotope/_c/peak_set.pyx"


#line 135 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 135 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 135 "ms_deisotope/_c/peak_set.pyx"
}

#line 135 "ms_deisotope/_c/peak_set.pyx"


#line 135 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_i) {

#line 135 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 135 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":136
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.mz
 *         elif i == 1:
 */

#line 136 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)

#line 136 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 136 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 136, __pyx_L1_error)

#line 136 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 136 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":137
 *     def __getitem__(self, i):
 *         if i == 0:
 *             return self.mz             # <<<<<<<<<<<<<<
 *         elif i == 1:
 *             return self.intensity
 */

#line 137 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 137 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)

#line 137 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 137 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 137 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 137 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":136
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.mz
 *         elif i == 1:
 */

#line 136 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":138
 *         if i == 0:
 *             return self.mz
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.intensity
 *         else:
 */

#line 138 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)

#line 138 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 138 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 138, __pyx_L1_error)

#line 138 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 138 "ms_deisotope/_c/peak_set.pyx"
  if (likely(__pyx_t_2)) {

    /* "ms_deisotope/_c/peak_set.pyx":139
 *             return self.mz
 *         elif i == 1:
 *             return self.intensity             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(i)
 */

#line 139 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 139 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)

#line 139 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 139 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 139 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 139 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":138
 *         if i == 0:
 *             return self.mz
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.intensity
 *         else:
 */

#line 138 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":141
 *             return self.intensity
 *         else:
 *             raise IndexError(i)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */

#line 141 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 141 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)

#line 141 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 141 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 141 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 141 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 141, __pyx_L1_error)

#line 141 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":135
 *         self.intensity = intensity
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.mz
 */

#line 135 "ms_deisotope/_c/peak_set.pyx"


#line 135 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 135 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 135 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 135 "ms_deisotope/_c/peak_set.pyx"
}

#line 135 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 143 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__(PyObject *__pyx_v_self); /*proto*/

#line 143 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__(PyObject *__pyx_v_self) {

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = 
#line 143 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 143 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 143 "ms_deisotope/_c/peak_set.pyx"
}

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_cur_scope
#line 143 "ms_deisotope/_c/peak_set.pyx"
;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 143 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, __pyx_empty_tuple, NULL);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)Py_None);

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 143, __pyx_L1_error)

#line 143 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  }

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  {

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_EnvelopePair___iter, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 143, __pyx_L1_error)

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_cur_scope);

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 143 "ms_deisotope/_c/peak_set.pyx"
    return (PyObject *) gen;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  }

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 143 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 143 "ms_deisotope/_c/peak_set.pyx"
}

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_12EnvelopePair_6generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{

#line 143 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)__pyx_generator->closure);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_generator->resume_label) {

#line 143 "ms_deisotope/_c/peak_set.pyx"
    case 0: goto __pyx_L3_first_run;

#line 143 "ms_deisotope/_c/peak_set.pyx"
    case 1: goto __pyx_L4_resume_from_yield;

#line 143 "ms_deisotope/_c/peak_set.pyx"
    case 2: goto __pyx_L5_resume_from_yield;

#line 143 "ms_deisotope/_c/peak_set.pyx"
    default: /* CPython raises the right error here */

#line 143 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 143 "ms_deisotope/_c/peak_set.pyx"
    return NULL;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  }

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_first_run:;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 143, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":144
 * 
 *     def __iter__(self):
 *         yield self.mz             # <<<<<<<<<<<<<<
 *         yield self.intensity
 * 
 */

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 144 "ms_deisotope/_c/peak_set.pyx"
  /* return from generator, yielding value */

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = 1;

#line 144 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_resume_from_yield:;

#line 144 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 144, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":145
 *     def __iter__(self):
 *         yield self.mz
 *         yield self.intensity             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_cur_scope->__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 145 "ms_deisotope/_c/peak_set.pyx"
  /* return from generator, yielding value */

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = 2;

#line 145 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 145 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_resume_from_yield:;

#line 145 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 145, __pyx_L1_error)

#line 145 "ms_deisotope/_c/peak_set.pyx"
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "ms_deisotope/_c/peak_set.pyx":143
 *             raise IndexError(i)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.mz
 *         yield self.intensity
 */

#line 143 "ms_deisotope/_c/peak_set.pyx"


#line 143 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 143 "ms_deisotope/_c/peak_set.pyx"
  PyErr_SetNone(PyExc_StopIteration);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  #if !CYTHON_USE_EXC_INFO_STACK

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = -1;

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);

#line 143 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 143 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 143 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":147
 *         yield self.intensity
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 */

#line 147 "ms_deisotope/_c/peak_set.pyx"


#line 147 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 147 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 147 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__[] = "EnvelopePair.__reduce__(self)";

#line 147 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 147 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 147 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

#line 147 "ms_deisotope/_c/peak_set.pyx"


#line 147 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 147 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 147 "ms_deisotope/_c/peak_set.pyx"
}

#line 147 "ms_deisotope/_c/peak_set.pyx"


#line 147 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {

#line 147 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 147 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":148
 * 
 *     def __reduce__(self):
 *         return EnvelopePair, (self.mz, self.intensity,)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, EnvelopePair other):
 */

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 148, __pyx_L1_error)

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));

#line 148 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair));

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 148 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 148 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":147
 *         yield self.intensity
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 */

#line 147 "ms_deisotope/_c/peak_set.pyx"


#line 147 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 147 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 147 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":150
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 *     cpdef bint _eq(self, EnvelopePair other):             # <<<<<<<<<<<<<<
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)
 */

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 150 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other, int __pyx_skip_dispatch) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 150 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 150 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 150 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 150 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 150 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 150 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)

#line 150 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 150 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq)) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 150 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 150 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 150 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 150 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 150 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 150 "ms_deisotope/_c/peak_set.pyx"
          }

#line 150 "ms_deisotope/_c/peak_set.pyx"
        }

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 150, __pyx_L1_error)

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_5;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
      }

#line 150 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 150 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 150 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 150 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 150 "ms_deisotope/_c/peak_set.pyx"
      }

#line 150 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 150 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 150 "ms_deisotope/_c/peak_set.pyx"
    }

#line 150 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 150 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":151
 * 
 *     cpdef bint _eq(self, EnvelopePair other):
 *         return (abs(self.mz - other.mz) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 */

#line 151 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->mz - __pyx_v_other->mz)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 151 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 151 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 151 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 151 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 151 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":152
 *     cpdef bint _eq(self, EnvelopePair other):
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */

#line 152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->intensity - __pyx_v_other->intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_t_6;

#line 152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_bool_binop_done:;

#line 152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_5;

#line 152 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":150
 *         return EnvelopePair, (self.mz, self.intensity,)
 * 
 *     cpdef bint _eq(self, EnvelopePair other):             # <<<<<<<<<<<<<<
 *         return (abs(self.mz - other.mz) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon)
 */

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.EnvelopePair._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 150 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 150 "ms_deisotope/_c/peak_set.pyx"
}

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 150 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 150 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq[] = "EnvelopePair._eq(self, EnvelopePair other) -> bool";

#line 150 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair, 1, "other", 0))) __PYX_ERR(0, 150, __pyx_L1_error)
  __pyx_r = 
#line 150 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other));

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 150 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 150 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 150 "ms_deisotope/_c/peak_set.pyx"
}

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_other) {

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 150 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 150 "ms_deisotope/_c/peak_set.pyx"


#line 150 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 150 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 150 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":154
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 154 "ms_deisotope/_c/peak_set.pyx"


#line 154 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 154 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/

#line 154 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 154 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = 
#line 154 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

#line 154 "ms_deisotope/_c/peak_set.pyx"


#line 154 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 154 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 154 "ms_deisotope/_c/peak_set.pyx"
}

#line 154 "ms_deisotope/_c/peak_set.pyx"


#line 154 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_11__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 154 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 154 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":155
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 155 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_v_code) {

#line 155 "ms_deisotope/_c/peak_set.pyx"
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":156
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */

#line 156 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 156 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair))))) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)

#line 156 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 156 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 156 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 156 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":155
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 155 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 155 "ms_deisotope/_c/peak_set.pyx"
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":158
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 158 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 158 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair))))) __PYX_ERR(0, 158, __pyx_L1_error)

#line 158 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)

#line 158 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 158 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 158 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 158 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":157
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */

#line 157 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 157 "ms_deisotope/_c/peak_set.pyx"
    default: break;

#line 157 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":154
 *             abs(self.intensity - other.intensity) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 154 "ms_deisotope/_c/peak_set.pyx"


#line 154 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 154 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 154 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 154 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":162
 * 
 *     @staticmethod
 *     cdef EnvelopePair _create(double mz, double intensity):             # <<<<<<<<<<<<<<
 *         cdef:
 *             EnvelopePair inst
 */

#line 162 "ms_deisotope/_c/peak_set.pyx"


#line 162 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(double __pyx_v_mz, double __pyx_v_intensity) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_inst = 0
#line 162 "ms_deisotope/_c/peak_set.pyx"
;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_r = NULL;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 162 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":165
 *         cdef:
 *             EnvelopePair inst
 *         inst = EnvelopePair.__new__(EnvelopePair)             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         inst.intensity = intensity
 */

#line 165 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)

#line 165 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));

#line 165 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1);

#line 165 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":166
 *             EnvelopePair inst
 *         inst = EnvelopePair.__new__(EnvelopePair)
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         inst.intensity = intensity
 *         return inst
 */

#line 166 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":167
 *         inst = EnvelopePair.__new__(EnvelopePair)
 *         inst.mz = mz
 *         inst.intensity = intensity             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */

#line 167 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->intensity = __pyx_v_intensity;

  /* "ms_deisotope/_c/peak_set.pyx":168
 *         inst.mz = mz
 *         inst.intensity = intensity
 *         return inst             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 168 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 168 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));

#line 168 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_inst;

#line 168 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":162
 * 
 *     @staticmethod
 *     cdef EnvelopePair _create(double mz, double intensity):             # <<<<<<<<<<<<<<
 *         cdef:
 *             EnvelopePair inst
 */

#line 162 "ms_deisotope/_c/peak_set.pyx"


#line 162 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair._create", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 162 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 162 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 162 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":170
 *         return inst
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)
 * 
 */

#line 170 "ms_deisotope/_c/peak_set.pyx"


#line 170 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 170 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__(PyObject *__pyx_v_self); /*proto*/

#line 170 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__(PyObject *__pyx_v_self) {

#line 170 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 170 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

#line 170 "ms_deisotope/_c/peak_set.pyx"


#line 170 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 170 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 170 "ms_deisotope/_c/peak_set.pyx"
}

#line 170 "ms_deisotope/_c/peak_set.pyx"


#line 170 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_13__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {

#line 170 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 170 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":171
 * 
 *     def __repr__(self):
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_0_4f_0_2f, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 171 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":170
 *         return inst
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "(%0.4f, %0.2f)" % (self.mz, self.intensity)
 * 
 */

#line 170 "ms_deisotope/_c/peak_set.pyx"


#line 170 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 170 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 170 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 170 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":16
 * cdef class EnvelopePair:
 *     cdef:
 *         public double mz             # <<<<<<<<<<<<<<
 *         public double intensity
 * 
 */

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 16 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 16 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(PyObject *__pyx_v_self) {

#line 16 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 16 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 16 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
}

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {

#line 16 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 16 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.mz.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 16 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
}

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 16 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 16 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 16 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 16 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
}

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value) {

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 16 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L1_error)

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->mz = __pyx_t_1;

#line 16 "ms_deisotope/_c/peak_set.pxd"


#line 16 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.mz.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 16 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 16 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 16 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":17
 *     cdef:
 *         public double mz
 *         public double intensity             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, EnvelopePair other)
 */

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 17 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 17 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(PyObject *__pyx_v_self) {

#line 17 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 17 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self));

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 17 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
}

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self) {

#line 17 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 17 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->intensity); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.intensity.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 17 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
}

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 17 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 17 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 17 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 17 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
}

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_self, PyObject *__pyx_v_value) {

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 17 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 17, __pyx_L1_error)

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->intensity = __pyx_t_1;

#line 17 "ms_deisotope/_c/peak_set.pxd"


#line 17 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.EnvelopePair.intensity.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 17 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 17 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 17 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":175
 * 
 * @cython.boundscheck(False)
 * cpdef bytearray Envelope_to_bytes(Envelope self):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/

#line 175 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_j
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_m
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  char *__pyx_v_buff
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  __pyx_t_5numpy_uint32_t __pyx_v_n
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  __pyx_t_5numpy_float64_t __pyx_v_val
#line 175 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_out = 0
#line 175 "ms_deisotope/_c/peak_set.pyx"
;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5numpy_uint32_t __pyx_t_1;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5numpy_uint32_t __pyx_t_2;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_5;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_to_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":184
 *         bytearray out
 * 
 *     n = self.get_size()             # <<<<<<<<<<<<<<
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 */

#line 184 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":185
 * 
 *     n = self.get_size()
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2             # <<<<<<<<<<<<<<
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 */

#line 185 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_m = ((sizeof(__pyx_t_5numpy_uint32_t)) + (((sizeof(__pyx_t_5numpy_float64_t)) * __pyx_v_n) * 2));

  /* "ms_deisotope/_c/peak_set.pyx":186
 *     n = self.get_size()
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)             # <<<<<<<<<<<<<<
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 */

#line 186 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_buff = ((char *)malloc(((sizeof(char)) * __pyx_v_m)));

  /* "ms_deisotope/_c/peak_set.pyx":187
 *     m = sizeof(np.uint32_t) + sizeof(np.float64_t) * n * 2
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0             # <<<<<<<<<<<<<<
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 */

#line 187 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":188
 *     buff = <char*>malloc(sizeof(char) * m)
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):
 */

#line 188 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_n), (sizeof(__pyx_t_5numpy_uint32_t))));

  /* "ms_deisotope/_c/peak_set.pyx":189
 *     j = 0
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pair = self.getitem(i)
 */

#line 189 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint32_t)));

  /* "ms_deisotope/_c/peak_set.pyx":190
 *     memcpy(&buff[j], &n, sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pair = self.getitem(i)
 *         val = pair.mz
 */

#line 190 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_n;

#line 190 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_t_1;

#line 190 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {

#line 190 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":191
 *     j += sizeof(np.uint32_t)
 *     for i in range(n):
 *         pair = self.getitem(i)             # <<<<<<<<<<<<<<
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 */

#line 191 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 191, __pyx_L1_error)

#line 191 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 191 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_4));

#line 191 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":192
 *     for i in range(n):
 *         pair = self.getitem(i)
 *         val = pair.mz             # <<<<<<<<<<<<<<
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */

#line 192 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_v_pair->mz;

#line 192 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_val = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":193
 *         pair = self.getitem(i)
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity
 */

#line 193 "ms_deisotope/_c/peak_set.pyx"
    (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_val), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":194
 *         val = pair.mz
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 */

#line 194 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":195
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity             # <<<<<<<<<<<<<<
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */

#line 195 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_v_pair->intensity;

#line 195 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_val = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":196
 *         j += sizeof(np.float64_t)
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 */

#line 196 "ms_deisotope/_c/peak_set.pyx"
    (void)(memcpy((&(__pyx_v_buff[__pyx_v_j])), (&__pyx_v_val), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":197
 *         val = pair.intensity
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 */

#line 197 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

#line 197 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":198
 *         memcpy(&buff[j], &val, sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)             # <<<<<<<<<<<<<<
 *     free(buff)
 *     return out
 */

#line 198 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyByteArray_FromStringAndSize(((char *)__pyx_v_buff), __pyx_v_m); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 198, __pyx_L1_error)

#line 198 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 198 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_out = ((PyObject*)__pyx_t_4);

#line 198 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":199
 *         j += sizeof(np.float64_t)
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)             # <<<<<<<<<<<<<<
 *     return out
 * 
 */

#line 199 "ms_deisotope/_c/peak_set.pyx"
  free(__pyx_v_buff);

  /* "ms_deisotope/_c/peak_set.pyx":200
 *     out = PyByteArray_FromStringAndSize(<char*>buff, m)
 *     free(buff)
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 200 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 200 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_out);

#line 200 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_out;

#line 200 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":175
 * 
 * @cython.boundscheck(False)
 * cpdef bytearray Envelope_to_bytes(Envelope self):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_out);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 175 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 175 "ms_deisotope/_c/peak_set.pyx"
}

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 175 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/

#line 175 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes[] = "Envelope_to_bytes(Envelope self) -> bytearray";

#line 175 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes(PyObject *__pyx_self, PyObject *__pyx_v_self) {

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_to_bytes (wrapper)", 0);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_self), __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope, 1, "self", 0))) __PYX_ERR(0, 175, __pyx_L1_error)
  __pyx_r = 
#line 175 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(__pyx_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 175 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 175 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 175 "ms_deisotope/_c/peak_set.pyx"
}

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 175 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 175 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_to_bytes", 0);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 175 "ms_deisotope/_c/peak_set.pyx"


#line 175 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 175 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 175 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 175 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":203
 * 
 * 
 * cpdef Envelope_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/

#line 203 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(PyObject *__pyx_v_view, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 203 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_j
#line 203 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0
#line 203 "ms_deisotope/_c/peak_set.pyx"
;
  __pyx_t_5numpy_uint32_t __pyx_v_n
#line 203 "ms_deisotope/_c/peak_set.pyx"
;
  char *__pyx_v_buff
#line 203 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_result = 0
#line 203 "ms_deisotope/_c/peak_set.pyx"
;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 203 "ms_deisotope/_c/peak_set.pyx"
  char *__pyx_t_1;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5numpy_uint32_t __pyx_t_3;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5numpy_uint32_t __pyx_t_4;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_from_bytes", 0);

  /* "ms_deisotope/_c/peak_set.pyx":211
 *         char* buff
 *         tuple result
 *     j = 0             # <<<<<<<<<<<<<<
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 */

#line 211 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = 0;

  /* "ms_deisotope/_c/peak_set.pyx":212
 *         tuple result
 *     j = 0
 *     buff = view             # <<<<<<<<<<<<<<
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 */

#line 212 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_AsWritableString(__pyx_v_view); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 212, __pyx_L1_error)

#line 212 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_buff = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":213
 *     j = 0
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))             # <<<<<<<<<<<<<<
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)
 */

#line 213 "ms_deisotope/_c/peak_set.pyx"
  (void)(memcpy((&__pyx_v_n), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_uint32_t))));

  /* "ms_deisotope/_c/peak_set.pyx":214
 *     buff = view
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)             # <<<<<<<<<<<<<<
 *     result = PyTuple_New(n)
 *     for i in range(n):
 */

#line 214 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_uint32_t)));

  /* "ms_deisotope/_c/peak_set.pyx":215
 *     memcpy(&n, &buff[j], sizeof(np.uint32_t))
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)
 */

#line 215 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)

#line 215 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 215 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_result = ((PyObject*)__pyx_t_2);

#line 215 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":216
 *     j += sizeof(np.uint32_t)
 *     result = PyTuple_New(n)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 */

#line 216 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_v_n;

#line 216 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_t_3;

#line 216 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {

#line 216 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":217
 *     result = PyTuple_New(n)
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)             # <<<<<<<<<<<<<<
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */

#line 217 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(0.0, 0.0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)

#line 217 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 217 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2));

#line 217 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":218
 *     for i in range(n):
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 */

#line 218 "ms_deisotope/_c/peak_set.pyx"
    (void)(memcpy((&__pyx_v_pair->mz), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":219
 *         pair = EnvelopePair._create(0, 0)
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 */

#line 219 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":220
 *         memcpy(&pair.mz, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))             # <<<<<<<<<<<<<<
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)
 */

#line 220 "ms_deisotope/_c/peak_set.pyx"
    (void)(memcpy((&__pyx_v_pair->intensity), (&(__pyx_v_buff[__pyx_v_j])), (sizeof(__pyx_t_5numpy_float64_t))));

    /* "ms_deisotope/_c/peak_set.pyx":221
 *         j += sizeof(np.float64_t)
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)             # <<<<<<<<<<<<<<
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)
 */

#line 221 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_j = (__pyx_v_j + (sizeof(__pyx_t_5numpy_float64_t)));

    /* "ms_deisotope/_c/peak_set.pyx":222
 *         memcpy(&pair.intensity, &buff[j], sizeof(np.float64_t))
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)             # <<<<<<<<<<<<<<
 *         PyTuple_SetItem(result, i, pair)
 *     return Envelope._create(result)
 */

#line 222 "ms_deisotope/_c/peak_set.pyx"
    Py_INCREF(((PyObject *)__pyx_v_pair));

    /* "ms_deisotope/_c/peak_set.pyx":223
 *         j += sizeof(np.float64_t)
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)             # <<<<<<<<<<<<<<
 *     return Envelope._create(result)
 * 
 */

#line 223 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = PyTuple_SetItem(__pyx_v_result, __pyx_v_i, ((PyObject *)__pyx_v_pair)); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 223, __pyx_L1_error)

#line 223 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":224
 *         Py_INCREF(pair)
 *         PyTuple_SetItem(result, i, pair)
 *     return Envelope._create(result)             # <<<<<<<<<<<<<<
 * 
 * @cython.boundscheck(False)
 */

#line 224 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 224 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_result)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)

#line 224 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 224 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 224 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 224 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":203
 * 
 * 
 * cpdef Envelope_from_bytes(bytearray view):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, j, m
 */

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_result);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 203 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 203 "ms_deisotope/_c/peak_set.pyx"
}

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 203 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view); /*proto*/

#line 203 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes[] = "Envelope_from_bytes(bytearray view)";

#line 203 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes(PyObject *__pyx_self, PyObject *__pyx_v_view) {

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_from_bytes (wrapper)", 0);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_view), (&PyByteArray_Type), 1, "view", 1))) __PYX_ERR(0, 203, __pyx_L1_error)
  __pyx_r = 
#line 203 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(__pyx_self, ((PyObject*)__pyx_v_view));

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 203 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 203 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 203 "ms_deisotope/_c/peak_set.pyx"
}

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_view) {

#line 203 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 203 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_from_bytes", 0);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(__pyx_v_view, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 203 "ms_deisotope/_c/peak_set.pyx"


#line 203 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 203 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 203 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 203 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":227
 * 
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):             # <<<<<<<<<<<<<<
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 */

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value); /*proto*/

#line 227 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(PyObject *__pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch) {

#line 227 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_reconstructor", 0);

  /* "ms_deisotope/_c/peak_set.pyx":228
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 */

#line 228 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_Check(__pyx_v_value); 

#line 228 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 228 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":229
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)             # <<<<<<<<<<<<<<
 *     elif isinstance(value, bytearray):
 *         return Envelope_from_bytes(value)
 */

#line 229 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 229 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 229, __pyx_L1_error)

#line 229 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(((PyObject*)__pyx_v_value))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 229, __pyx_L1_error)

#line 229 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 229 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_3;

#line 229 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 229 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":228
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):
 *     if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 */

#line 228 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":230
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):             # <<<<<<<<<<<<<<
 *         return Envelope_from_bytes(value)
 *     else:
 */

#line 230 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyByteArray_Check(__pyx_v_value); 

#line 230 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_2 != 0);

#line 230 "ms_deisotope/_c/peak_set.pyx"
  if (likely(__pyx_t_1)) {

    /* "ms_deisotope/_c/peak_set.pyx":231
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):
 *         return Envelope_from_bytes(value)             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError(type(value))
 */

#line 231 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 231 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(PyByteArray_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytearray", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 231, __pyx_L1_error)

#line 231 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_from_bytes(((PyObject*)__pyx_v_value), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)

#line 231 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 231 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_3;

#line 231 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 231 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":230
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 *     elif isinstance(value, bytearray):             # <<<<<<<<<<<<<<
 *         return Envelope_from_bytes(value)
 *     else:
 */

#line 230 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":233
 *         return Envelope_from_bytes(value)
 *     else:
 *         raise TypeError(type(value))             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 233 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 233 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, ((PyObject *)Py_TYPE(__pyx_v_value))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)

#line 233 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 233 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 233 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 233 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 233, __pyx_L1_error)

#line 233 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":227
 * 
 * @cython.boundscheck(False)
 * cpdef Envelope_reconstructor(object value):             # <<<<<<<<<<<<<<
 *     if isinstance(value, tuple):
 *         return Envelope._create(value)
 */

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 227 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 227 "ms_deisotope/_c/peak_set.pyx"
}

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 227 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value); /*proto*/

#line 227 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor[] = "Envelope_reconstructor(value)";

#line 227 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor(PyObject *__pyx_self, PyObject *__pyx_v_value) {

#line 227 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_reconstructor (wrapper)", 0);
  __pyx_r = 
#line 227 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(__pyx_self, ((PyObject *)__pyx_v_value));

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 227 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 227 "ms_deisotope/_c/peak_set.pyx"
}

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value) {

#line 227 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 227 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("Envelope_reconstructor", 0);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_reconstructor(__pyx_v_value, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 227 "ms_deisotope/_c/peak_set.pyx"


#line 227 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope_reconstructor", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 227 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 227 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 227 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

#line 250 "ms_deisotope/_c/peak_set.pyx"


#line 250 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 250 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 250 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pairs = 0
#line 250 "ms_deisotope/_c/peak_set.pyx"
;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 250 "ms_deisotope/_c/peak_set.pyx"
  {

#line 250 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pairs,0};

#line 250 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[1] = {0};

#line 250 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 250 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 250 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 250 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  1: 
#line 250 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 250 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 250 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 250 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 250 "ms_deisotope/_c/peak_set.pyx"
      }

#line 250 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 250 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 250 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 250 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pairs)) != 0)) kw_args--;
        else 
#line 250 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 250 "ms_deisotope/_c/peak_set.pyx"
      }

#line 250 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 250 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 250, __pyx_L3_error)

#line 250 "ms_deisotope/_c/peak_set.pyx"
      }

#line 250 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 250 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 250 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 250 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 250 "ms_deisotope/_c/peak_set.pyx"
    }

#line 250 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_pairs = values[0];

#line 250 "ms_deisotope/_c/peak_set.pyx"
  }

#line 250 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 250 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 250, __pyx_L3_error)

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 250 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 250 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), __pyx_v_pairs);

#line 250 "ms_deisotope/_c/peak_set.pyx"


#line 250 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 250 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 250 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":251
 *     """
 *     def __init__(self, pairs):
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */

#line 251 "ms_deisotope/_c/peak_set.pyx"


#line 251 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 251 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda = 
#line 251 "ms_deisotope/_c/peak_set.pyx"
{"lambda", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda, METH_O, 0};

#line 251 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda(PyObject *__pyx_self, PyObject *__pyx_v_x) {

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = 
#line 251 "ms_deisotope/_c/peak_set.pyx"
__pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_x));

#line 251 "ms_deisotope/_c/peak_set.pyx"


#line 251 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 251 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 251 "ms_deisotope/_c/peak_set.pyx"
}

#line 251 "ms_deisotope/_c/peak_set.pyx"


#line 251 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("lambda", 0);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PySequence_Tuple(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 251 "ms_deisotope/_c/peak_set.pyx"


#line 251 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 251 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 251 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

#line 250 "ms_deisotope/_c/peak_set.pyx"


#line 250 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_pairs) {

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 250 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":251
 *     """
 *     def __init__(self, pairs):
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_8Envelope_8__init___lambda, 0, __pyx_n_s_init___locals_lambda, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_pairs);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_pairs);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_pairs);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 251, __pyx_L1_error)

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->pairs);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->pairs);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->pairs = ((PyObject*)__pyx_t_2);

#line 251 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":250
 *         list of (mz, intensity) pairs
 *     """
 *     def __init__(self, pairs):             # <<<<<<<<<<<<<<
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 */

#line 250 "ms_deisotope/_c/peak_set.pyx"


#line 250 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 250 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 250 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 250 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":253
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         return self.pairs[i]
 * 
 */

#line 253 "ms_deisotope/_c/peak_set.pyx"


#line 253 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 253 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/

#line 253 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {

#line 253 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 253 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_i));

#line 253 "ms_deisotope/_c/peak_set.pyx"


#line 253 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 253 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 253 "ms_deisotope/_c/peak_set.pyx"
}

#line 253 "ms_deisotope/_c/peak_set.pyx"


#line 253 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_2__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_i) {

#line 253 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 253 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":254
 * 
 *     def __getitem__(self, i):
 *         return self.pairs[i]             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */

#line 254 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 254 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_self->pairs == Py_None)) {

#line 254 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 254 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 254, __pyx_L1_error)

#line 254 "ms_deisotope/_c/peak_set.pyx"
  }

#line 254 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_self->pairs, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)

#line 254 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 254 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 254 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 254 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":253
 *         self.pairs = tuple(map(lambda x: EnvelopePair(*x), pairs))
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         return self.pairs[i]
 * 
 */

#line 253 "ms_deisotope/_c/peak_set.pyx"


#line 253 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 253 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 253 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 253 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":256
 *         return self.pairs[i]
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.pairs)
 * 
 */

#line 256 "ms_deisotope/_c/peak_set.pyx"


#line 256 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 256 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__(PyObject *__pyx_v_self); /*proto*/

#line 256 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__(PyObject *__pyx_v_self) {

#line 256 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = 
#line 256 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 256 "ms_deisotope/_c/peak_set.pyx"


#line 256 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/peak_set.pyx"
}

#line 256 "ms_deisotope/_c/peak_set.pyx"


#line 256 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 256 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 256 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":257
 * 
 *     def __iter__(self):
 *         return iter(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->pairs;

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 257 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 257 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":256
 *         return self.pairs[i]
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.pairs)
 * 
 */

#line 256 "ms_deisotope/_c/peak_set.pyx"


#line 256 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 256 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 256 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 256 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

#line 259 "ms_deisotope/_c/peak_set.pyx"


#line 259 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 259 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__(PyObject *__pyx_v_self); /*proto*/

#line 259 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__(PyObject *__pyx_v_self) {

#line 259 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 259 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 259 "ms_deisotope/_c/peak_set.pyx"


#line 259 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 259 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 259 "ms_deisotope/_c/peak_set.pyx"
}

#line 259 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 260 "ms_deisotope/_c/peak_set.pyx"


#line 260 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_cur_scope
#line 260 "ms_deisotope/_c/peak_set.pyx"
;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 260 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, __pyx_empty_tuple, NULL);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)Py_None);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  }

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *) __pyx_self;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_repr___locals_genexpr, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_cur_scope);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 260 "ms_deisotope/_c/peak_set.pyx"
    return (PyObject *) gen;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  }

#line 260 "ms_deisotope/_c/peak_set.pyx"


#line 260 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__repr__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 260 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 260 "ms_deisotope/_c/peak_set.pyx"
}

#line 260 "ms_deisotope/_c/peak_set.pyx"


#line 260 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{

#line 260 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)__pyx_generator->closure);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_2;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*__pyx_t_3)(PyObject *);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_generator->resume_label) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    case 0: goto __pyx_L3_first_run;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    case 1: goto __pyx_L6_resume_from_yield;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    default: /* CPython raises the right error here */

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 260 "ms_deisotope/_c/peak_set.pyx"
    return NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  }

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_first_run:;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 260, __pyx_L1_error) }

#line 260 "ms_deisotope/_c/peak_set.pyx"
  if (likely(PyList_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self))) || PyTuple_CheckExact(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self))) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self); __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = NULL;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  } else {
    __pyx_t_2 = -1; 
#line 260 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_1 = PyObject_GetIter(((PyObject *)__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  }

#line 260 "ms_deisotope/_c/peak_set.pyx"
  for (;;) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
    if (likely(!__pyx_t_3)) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
      if (likely(PyList_CheckExact(__pyx_t_1))) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #else

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 260 "ms_deisotope/_c/peak_set.pyx"
      } else {

#line 260 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #else

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 260 "ms_deisotope/_c/peak_set.pyx"
      }
    } else 
#line 260 "ms_deisotope/_c/peak_set.pyx"
{

#line 260 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(!__pyx_t_4)) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
        PyObject* exc_type = PyErr_Occurred();

#line 260 "ms_deisotope/_c/peak_set.pyx"
        if (exc_type) {

#line 260 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 260 "ms_deisotope/_c/peak_set.pyx"
          else __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
        }

#line 260 "ms_deisotope/_c/peak_set.pyx"
        break;

#line 260 "ms_deisotope/_c/peak_set.pyx"
      }

#line 260 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    }

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_t);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_t, __pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_cur_scope->__pyx_v_t); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_0_4f_0_2f, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_5;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_r);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    /* return from generator, yielding value */

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_generator->resume_label = 1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    return __pyx_r;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_resume_from_yield:;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_0 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;

#line 260 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  }

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

#line 260 "ms_deisotope/_c/peak_set.pyx"


#line 260 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 260 "ms_deisotope/_c/peak_set.pyx"
  PyErr_SetNone(PyExc_StopIteration);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  #if !CYTHON_USE_EXC_INFO_STACK

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = -1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 260 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 260 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

#line 259 "ms_deisotope/_c/peak_set.pyx"


#line 259 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_6__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_cur_scope
#line 259 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2 = 0
#line 259 "ms_deisotope/_c/peak_set.pyx"
;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 259 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, __pyx_empty_tuple, NULL);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 259 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)Py_None);

#line 259 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 259 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 259, __pyx_L1_error)

#line 259 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 259 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  }

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":260
 * 
 *     def __repr__(self):
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s_, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 260 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":259
 *         return iter(self.pairs)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 */

#line 259 "ms_deisotope/_c/peak_set.pyx"


#line 259 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_gb_12ms_deisotope_2_c_8peak_set_8Envelope_8__repr___2generator2);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 259 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 259 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 259 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":262
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.pairs)
 * 
 */

#line 262 "ms_deisotope/_c/peak_set.pyx"


#line 262 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 262 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__(PyObject *__pyx_v_self); /*proto*/

#line 262 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__(PyObject *__pyx_v_self) {

#line 262 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_r;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 262 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 262 "ms_deisotope/_c/peak_set.pyx"


#line 262 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 262 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 262 "ms_deisotope/_c/peak_set.pyx"
}

#line 262 "ms_deisotope/_c/peak_set.pyx"


#line 262 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_8__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 262 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_r;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 262 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_2;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":263
 * 
 *     def __len__(self):
 *         return len(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline size_t get_size(self):
 */

#line 263 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->pairs;

#line 263 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 263 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_t_1 == Py_None)) {

#line 263 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 263 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 263, __pyx_L1_error)

#line 263 "ms_deisotope/_c/peak_set.pyx"
  }

#line 263 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 263, __pyx_L1_error)

#line 263 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 263 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 263 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":262
 *         return "[%s]" % (', '.join("(%0.4f, %0.2f)" % tuple(t) for t in self))
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.pairs)
 * 
 */

#line 262 "ms_deisotope/_c/peak_set.pyx"


#line 262 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 262 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 262 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":265
 *         return len(self.pairs)
 * 
 *     cdef inline size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 */

#line 265 "ms_deisotope/_c/peak_set.pyx"


#line 265 "ms_deisotope/_c/peak_set.pyx"
static CYTHON_INLINE size_t __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 265 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_r;

#line 265 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 265 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 265 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":266
 * 
 *     cdef inline size_t get_size(self):
 *         return PyTuple_GET_SIZE(self.pairs)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):
 */

#line 266 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->pairs;

#line 266 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 266 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = PyTuple_GET_SIZE(__pyx_t_1);

#line 266 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 266 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":265
 *         return len(self.pairs)
 * 
 *     cdef inline size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 */

#line 265 "ms_deisotope/_c/peak_set.pyx"


#line 265 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 265 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 265 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 265 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 265 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":268
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 */

#line 268 "ms_deisotope/_c/peak_set.pyx"


#line 268 "ms_deisotope/_c/peak_set.pyx"
static CYTHON_INLINE struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, size_t __pyx_v_i) {

#line 268 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_r = NULL;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 268 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("getitem", 0);

  /* "ms_deisotope/_c/peak_set.pyx":269
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, Envelope other):
 */

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->pairs;

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GetItem(__pyx_t_1, __pyx_v_i); if (unlikely(__pyx_t_2 == ((PyObject *)NULL))) __PYX_ERR(0, 269, __pyx_L1_error)

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2)));

#line 269 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_2);

#line 269 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":268
 *         return PyTuple_GET_SIZE(self.pairs)
 * 
 *     cdef inline EnvelopePair getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 */

#line 268 "ms_deisotope/_c/peak_set.pyx"


#line 268 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.getitem", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 268 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 268 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 268 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":271
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 *     cpdef bint _eq(self, Envelope other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 271 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other, int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 271 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 271 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_a = 0
#line 271 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_b = 0
#line 271 "ms_deisotope/_c/peak_set.pyx"
;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_7;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_8;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 271 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 271 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 271 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 271 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 271 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 271 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)

#line 271 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 271 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq)) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 271 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 271 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 271 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 271 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 271 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 271 "ms_deisotope/_c/peak_set.pyx"
          }

#line 271 "ms_deisotope/_c/peak_set.pyx"
        }

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 271, __pyx_L1_error)

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 271, __pyx_L1_error)

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_5;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
      }

#line 271 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 271 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 271 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 271 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 271 "ms_deisotope/_c/peak_set.pyx"
      }

#line 271 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 271 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 271 "ms_deisotope/_c/peak_set.pyx"
    }

#line 271 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 271 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":276
 *             EnvelopePair a, b
 * 
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         if n != other.get_size():
 *             return False
 */

#line 276 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":277
 * 
 *         n = self.get_size()
 *         if n != other.get_size():             # <<<<<<<<<<<<<<
 *             return False
 * 
 */

#line 277 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = ((__pyx_v_n != __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_other)) != 0);

#line 277 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_5) {

    /* "ms_deisotope/_c/peak_set.pyx":278
 *         n = self.get_size()
 *         if n != other.get_size():
 *             return False             # <<<<<<<<<<<<<<
 * 
 *         i = 0
 */

#line 278 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = 0;

#line 278 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":277
 * 
 *         n = self.get_size()
 *         if n != other.get_size():             # <<<<<<<<<<<<<<
 *             return False
 * 
 */

#line 277 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":280
 *             return False
 * 
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             a = self.getitem(i)
 */

#line 280 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":281
 * 
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 */

#line 281 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_v_n;

#line 281 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = __pyx_t_6;

#line 281 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {

#line 281 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_8;

    /* "ms_deisotope/_c/peak_set.pyx":282
 *         i = 0
 *         for i in range(n):
 *             a = self.getitem(i)             # <<<<<<<<<<<<<<
 *             b = other.getitem(i)
 *             if a != b:
 */

#line 282 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)

#line 282 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 282 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_a, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));

#line 282 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":283
 *         for i in range(n):
 *             a = self.getitem(i)
 *             b = other.getitem(i)             # <<<<<<<<<<<<<<
 *             if a != b:
 *                 return False
 */

#line 283 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_other, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)

#line 283 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 283 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_b, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));

#line 283 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":284
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */

#line 284 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_a), ((PyObject *)__pyx_v_b), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)

#line 284 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 284, __pyx_L1_error)

#line 284 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 284 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_5) {

      /* "ms_deisotope/_c/peak_set.pyx":285
 *             b = other.getitem(i)
 *             if a != b:
 *                 return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */

#line 285 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 0;

#line 285 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":284
 *             a = self.getitem(i)
 *             b = other.getitem(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */

#line 284 "ms_deisotope/_c/peak_set.pyx"
    }

#line 284 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":286
 *             if a != b:
 *                 return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */

#line 286 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 1;

#line 286 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":271
 *         return <EnvelopePair>PyTuple_GetItem(self.pairs, i)
 * 
 *     cpdef bint _eq(self, Envelope other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.Envelope._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_a);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_b);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 271 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 271 "ms_deisotope/_c/peak_set.pyx"
}

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 271 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 271 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq[] = "Envelope._eq(self, Envelope other) -> bool";

#line 271 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope, 1, "other", 0))) __PYX_ERR(0, 271, __pyx_L1_error)
  __pyx_r = 
#line 271 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other));

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 271 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 271 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 271 "ms_deisotope/_c/peak_set.pyx"
}

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_other) {

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 271 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 271 "ms_deisotope/_c/peak_set.pyx"


#line 271 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 271 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 271 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 271 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":288
 *         return True
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 288 "ms_deisotope/_c/peak_set.pyx"


#line 288 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 288 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/

#line 288 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 288 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = 
#line 288 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

#line 288 "ms_deisotope/_c/peak_set.pyx"


#line 288 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 288 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 288 "ms_deisotope/_c/peak_set.pyx"
}

#line 288 "ms_deisotope/_c/peak_set.pyx"


#line 288 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_12__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 288 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 288 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":289
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 289 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_v_code) {

#line 289 "ms_deisotope/_c/peak_set.pyx"
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":290
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */

#line 290 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 290 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 290, __pyx_L1_error)

#line 290 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)

#line 290 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 290 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 290 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 290 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":289
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 289 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 289 "ms_deisotope/_c/peak_set.pyx"
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":292
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 *     cpdef Envelope clone(self):
 */

#line 292 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 292 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 292, __pyx_L1_error)

#line 292 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self->__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)

#line 292 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 292 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 292 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 292 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":291
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */

#line 291 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 291 "ms_deisotope/_c/peak_set.pyx"
    default: break;

#line 291 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":288
 *         return True
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 288 "ms_deisotope/_c/peak_set.pyx"


#line 288 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 288 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 288 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 288 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 288 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":294
 *             return not (self._eq(other))
 * 
 *     cpdef Envelope clone(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate
 */

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 294 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_duplicate = 0
#line 294 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 294 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 294 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_pair = 0
#line 294 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_v_dup_pair = 0
#line 294 "ms_deisotope/_c/peak_set.pyx"
;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_r = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_7;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_8;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 294 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 294 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 294 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 294 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 294 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 294 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)

#line 294 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 294 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone)) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 294 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 294 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 294 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 294 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 294 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 294 "ms_deisotope/_c/peak_set.pyx"
          }

#line 294 "ms_deisotope/_c/peak_set.pyx"
        }

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 294, __pyx_L1_error)

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_2);

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
      }

#line 294 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 294 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 294 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 294 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 294 "ms_deisotope/_c/peak_set.pyx"
      }

#line 294 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 294 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 294 "ms_deisotope/_c/peak_set.pyx"
    }

#line 294 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 294 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":299
 *             size_t i, n
 *             EnvelopePair pair, dup_pair
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):
 */

#line 299 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":300
 *             EnvelopePair pair, dup_pair
 *         n = self.get_size()
 *         duplicate = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             pair = self.getitem(i)
 */

#line 300 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)

#line 300 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 300 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_duplicate = ((PyObject*)__pyx_t_1);

#line 300 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":301
 *         n = self.get_size()
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 */

#line 301 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_v_n;

#line 301 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_t_5;

#line 301 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {

#line 301 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":302
 *         duplicate = PyTuple_New(n)
 *         for i in range(n):
 *             pair = self.getitem(i)             # <<<<<<<<<<<<<<
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)
 */

#line 302 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)

#line 302 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 302 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));

#line 302 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":303
 *         for i in range(n):
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)             # <<<<<<<<<<<<<<
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 */

#line 303 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_pair->mz, __pyx_v_pair->intensity)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)

#line 303 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 303 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_dup_pair, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)__pyx_t_1));

#line 303 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":304
 *             pair = self.getitem(i)
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 *         return Envelope._create(duplicate)
 */

#line 304 "ms_deisotope/_c/peak_set.pyx"
    Py_INCREF(((PyObject *)__pyx_v_dup_pair));

    /* "ms_deisotope/_c/peak_set.pyx":305
 *             dup_pair = EnvelopePair._create(pair.mz, pair.intensity)
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)             # <<<<<<<<<<<<<<
 *         return Envelope._create(duplicate)
 * 
 */

#line 305 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = PyTuple_SetItem(__pyx_v_duplicate, __pyx_v_i, ((PyObject *)__pyx_v_dup_pair)); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 305, __pyx_L1_error)

#line 305 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":306
 *             Py_INCREF(dup_pair)
 *             PyTuple_SetItem(duplicate, i, dup_pair)
 *         return Envelope._create(duplicate)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 306 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 306 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_duplicate)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)

#line 306 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 306 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);

#line 306 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 306 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":294
 *             return not (self._eq(other))
 * 
 *     cpdef Envelope clone(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate
 */

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_duplicate);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_pair);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_dup_pair);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 294 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 294 "ms_deisotope/_c/peak_set.pyx"
}

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 294 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 294 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_14clone[] = "Envelope.clone(self) -> Envelope";

#line 294 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 294 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 294 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 294 "ms_deisotope/_c/peak_set.pyx"
}

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_14clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 294 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 294 "ms_deisotope/_c/peak_set.pyx"


#line 294 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 294 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 294 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 294 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":308
 *         return Envelope._create(duplicate)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             Envelope_reconstructor,
 */

#line 308 "ms_deisotope/_c/peak_set.pyx"


#line 308 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 308 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 308 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__[] = "Envelope.__reduce__(self)";

#line 308 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 308 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 308 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 308 "ms_deisotope/_c/peak_set.pyx"


#line 308 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 308 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 308 "ms_deisotope/_c/peak_set.pyx"
}

#line 308 "ms_deisotope/_c/peak_set.pyx"


#line 308 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 308 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 308 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":309
 * 
 *     def __reduce__(self):
 *         return (             # <<<<<<<<<<<<<<
 *             Envelope_reconstructor,
 *             (Envelope_to_bytes(self),))
 */

#line 309 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":310
 *     def __reduce__(self):
 *         return (
 *             Envelope_reconstructor,             # <<<<<<<<<<<<<<
 *             (Envelope_to_bytes(self),))
 *             # (self.pairs, ))
 */

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Envelope_reconstructor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

  /* "ms_deisotope/_c/peak_set.pyx":311
 *         return (
 *             Envelope_reconstructor,
 *             (Envelope_to_bytes(self),))             # <<<<<<<<<<<<<<
 *             # (self.pairs, ))
 * 
 */

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_f_12ms_deisotope_2_c_8peak_set_Envelope_to_bytes(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 311, __pyx_L1_error)

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 311 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);

#line 311 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":310
 *     def __reduce__(self):
 *         return (
 *             Envelope_reconstructor,             # <<<<<<<<<<<<<<
 *             (Envelope_to_bytes(self),))
 *             # (self.pairs, ))
 */

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 310 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 310 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 310 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 310 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":308
 *         return Envelope._create(duplicate)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             Envelope_reconstructor,
 */

#line 308 "ms_deisotope/_c/peak_set.pyx"


#line 308 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 308 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 308 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 308 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":315
 * 
 *     @staticmethod
 *     cdef Envelope _create(tuple pairs):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Envelope inst
 */

#line 315 "ms_deisotope/_c/peak_set.pyx"


#line 315 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(PyObject *__pyx_v_pairs) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_inst = 0
#line 315 "ms_deisotope/_c/peak_set.pyx"
;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_r = NULL;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 315 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_create", 0);

  /* "ms_deisotope/_c/peak_set.pyx":318
 *         cdef:
 *             Envelope inst
 *         inst = Envelope.__new__(Envelope)             # <<<<<<<<<<<<<<
 *         inst.pairs = pairs
 *         return inst
 */

#line 318 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)

#line 318 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));

#line 318 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);

#line 318 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":319
 *             Envelope inst
 *         inst = Envelope.__new__(Envelope)
 *         inst.pairs = pairs             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */

#line 319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_pairs);

#line 319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_pairs);

#line 319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_inst->pairs);

#line 319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_inst->pairs);

#line 319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->pairs = __pyx_v_pairs;

  /* "ms_deisotope/_c/peak_set.pyx":320
 *         inst = Envelope.__new__(Envelope)
 *         inst.pairs = pairs
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 320 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 320 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));

#line 320 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_inst;

#line 320 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":315
 * 
 *     @staticmethod
 *     cdef Envelope _create(tuple pairs):             # <<<<<<<<<<<<<<
 *         cdef:
 *             Envelope inst
 */

#line 315 "ms_deisotope/_c/peak_set.pyx"


#line 315 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope._create", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 315 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 315 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 315 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":26
 * 
 * cdef class Envelope:
 *     cdef public tuple pairs             # <<<<<<<<<<<<<<
 * 
 *     cdef inline size_t get_size(self)
 */

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 26 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 26 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(PyObject *__pyx_v_self) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 26 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_v_self->pairs);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_v_self->pairs;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 26 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self, PyObject *__pyx_v_value) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 26, __pyx_L1_error)

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->pairs);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->pairs);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->pairs = ((PyObject*)__pyx_t_1);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.Envelope.pairs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(PyObject *__pyx_v_self) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 26 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_v_self));

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_self) {

#line 26 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->pairs);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->pairs);

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->pairs = ((PyObject*)Py_None);

#line 26 "ms_deisotope/_c/peak_set.pxd"


#line 26 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 26 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 26 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 26 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

#line 357 "ms_deisotope/_c/peak_set.pyx"


#line 357 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 357 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 357 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_intensity = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_charge = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_signal_to_noise = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_index = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_full_width_at_half_max = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_a_to_a2_ratio = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_most_abundant_mass = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_average_mass = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_score = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_envelope = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_mz = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_fit = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_chosen_for_msms = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_area = 0
#line 357 "ms_deisotope/_c/peak_set.pyx"
;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  {

#line 357 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_intensity,&__pyx_n_s_charge,&__pyx_n_s_signal_to_noise,&__pyx_n_s_index,&__pyx_n_s_full_width_at_half_max,&__pyx_n_s_a_to_a2_ratio,&__pyx_n_s_most_abundant_mass,&__pyx_n_s_average_mass,&__pyx_n_s_score,&__pyx_n_s_envelope,&__pyx_n_s_mz,&__pyx_n_s_fit,&__pyx_n_s_chosen_for_msms,&__pyx_n_s_area,0};

#line 357 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[15] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

#line 357 "ms_deisotope/_c/peak_set.pyx"
    values[6] = ((PyObject *)__pyx_int_0);

#line 357 "ms_deisotope/_c/peak_set.pyx"
    values[7] = ((PyObject *)__pyx_int_0);

#line 357 "ms_deisotope/_c/peak_set.pyx"
    values[8] = ((PyObject *)__pyx_int_0);

#line 357 "ms_deisotope/_c/peak_set.pyx"
    values[9] = ((PyObject *)__pyx_int_0);

    /* "ms_deisotope/_c/peak_set.pyx":359
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):             # <<<<<<<<<<<<<<
 *         if index is None:
 *             index = _Index._create(0, 0)
 */

#line 359 "ms_deisotope/_c/peak_set.pyx"
    values[10] = ((PyObject *)__pyx_empty_tuple);

#line 359 "ms_deisotope/_c/peak_set.pyx"
    values[11] = ((PyObject *)__pyx_int_0);

#line 359 "ms_deisotope/_c/peak_set.pyx"
    values[12] = ((PyObject *)Py_None);

#line 359 "ms_deisotope/_c/peak_set.pyx"
    values[13] = ((PyObject *)Py_False);

#line 359 "ms_deisotope/_c/peak_set.pyx"
    values[14] = ((PyObject *)__pyx_int_0);

#line 359 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 359 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 359 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case 15: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[14] = PyTuple_GET_ITEM(__pyx_args, 14);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 14: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[13] = PyTuple_GET_ITEM(__pyx_args, 13);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 13: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[12] = PyTuple_GET_ITEM(__pyx_args, 12);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 12: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[11] = PyTuple_GET_ITEM(__pyx_args, 11);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 11: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[10] = PyTuple_GET_ITEM(__pyx_args, 10);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 10: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[9] = PyTuple_GET_ITEM(__pyx_args, 9);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  9: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[8] = PyTuple_GET_ITEM(__pyx_args, 8);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  8: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[7] = PyTuple_GET_ITEM(__pyx_args, 7);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  7: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[6] = PyTuple_GET_ITEM(__pyx_args, 6);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  6: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  5: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 359 "ms_deisotope/_c/peak_set.pyx"
      }

#line 359 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 359 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 359 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 1); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  2:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_charge)) != 0)) kw_args--;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 2); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  3:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signal_to_noise)) != 0)) kw_args--;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 3); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  4:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 4); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  5:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_width_at_half_max)) != 0)) kw_args--;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, 5); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  6:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a_to_a2_ratio);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[6] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  7:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_most_abundant_mass);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[7] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  8:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_average_mass);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[8] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case  9:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_score);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[9] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case 10:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_envelope);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[10] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case 11:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[11] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case 12:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[12] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case 13:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_chosen_for_msms);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[13] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 359 "ms_deisotope/_c/peak_set.pyx"
        case 14:

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_area);

#line 359 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[14] = value; kw_args--; }

#line 359 "ms_deisotope/_c/peak_set.pyx"
        }

#line 359 "ms_deisotope/_c/peak_set.pyx"
      }

#line 359 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 359 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
      }

#line 359 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 359 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 15: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[14] = PyTuple_GET_ITEM(__pyx_args, 14);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 14: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[13] = PyTuple_GET_ITEM(__pyx_args, 13);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 13: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[12] = PyTuple_GET_ITEM(__pyx_args, 12);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 12: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[11] = PyTuple_GET_ITEM(__pyx_args, 11);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 11: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[10] = PyTuple_GET_ITEM(__pyx_args, 10);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 10: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[9] = PyTuple_GET_ITEM(__pyx_args, 9);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  9: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[8] = PyTuple_GET_ITEM(__pyx_args, 8);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  8: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[7] = PyTuple_GET_ITEM(__pyx_args, 7);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  7: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[6] = PyTuple_GET_ITEM(__pyx_args, 6);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  6: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 359 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 359 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 359 "ms_deisotope/_c/peak_set.pyx"
      }

#line 359 "ms_deisotope/_c/peak_set.pyx"
    }

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = values[0];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_intensity = values[1];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_charge = values[2];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_signal_to_noise = values[3];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index = values[4];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_full_width_at_half_max = values[5];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_a_to_a2_ratio = values[6];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_most_abundant_mass = values[7];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_average_mass = values[8];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_score = values[9];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_envelope = values[10];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mz = values[11];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_fit = values[12];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_chosen_for_msms = values[13];

#line 359 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_area = values[14];

#line 359 "ms_deisotope/_c/peak_set.pyx"
  }

#line 359 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 359 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 15, PyTuple_GET_SIZE(__pyx_args)); 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 357, __pyx_L3_error)

#line 359 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 359 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 359 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 359 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 359 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 359 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area);

  /* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

#line 357 "ms_deisotope/_c/peak_set.pyx"


#line 357 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 357 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 357 "ms_deisotope/_c/peak_set.pyx"
}

#line 357 "ms_deisotope/_c/peak_set.pyx"


#line 357 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area) {

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_4;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_6;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_index);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_envelope);

  /* "ms_deisotope/_c/peak_set.pyx":360
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 */

#line 360 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_v_index == Py_None);

#line 360 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 360 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":361
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:
 *             index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 */

#line 361 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)

#line 361 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 361 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);

#line 361 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":360
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 *         if index is None:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 */

#line 360 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 360 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":362
 *         if index is None:
 *             index = _Index._create(0, 0)
 *         elif index == -1:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 */

#line 362 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_index, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)

#line 362 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 362 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 362, __pyx_L1_error)

#line 362 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 362 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":363
 *             index = _Index._create(0, 0)
 *         elif index == -1:
 *             index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         if envelope is None:
 *             envelope = ()
 */

#line 363 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 363, __pyx_L1_error)

#line 363 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 363 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);

#line 363 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":362
 *         if index is None:
 *             index = _Index._create(0, 0)
 *         elif index == -1:             # <<<<<<<<<<<<<<
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 */

#line 362 "ms_deisotope/_c/peak_set.pyx"
  }

#line 362 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":364
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 *         if envelope is None:             # <<<<<<<<<<<<<<
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 */

#line 364 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_v_envelope == Py_None);

#line 364 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_2 != 0);

#line 364 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":365
 *             index = _Index._create(0, 0)
 *         if envelope is None:
 *             envelope = ()             # <<<<<<<<<<<<<<
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity
 */

#line 365 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_empty_tuple);

#line 365 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_envelope, __pyx_empty_tuple);

    /* "ms_deisotope/_c/peak_set.pyx":364
 *         elif index == -1:
 *             index = _Index._create(0, 0)
 *         if envelope is None:             # <<<<<<<<<<<<<<
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 */

#line 364 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":366
 *         if envelope is None:
 *             envelope = ()
 *         self.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise
 */

#line 366 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_neutral_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 366, __pyx_L1_error)

#line 366 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->neutral_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":367
 *             envelope = ()
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity             # <<<<<<<<<<<<<<
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index
 */

#line 367 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_intensity); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)

#line 367 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->__pyx_base.intensity = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":368
 *         self.neutral_mass = neutral_mass
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise             # <<<<<<<<<<<<<<
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max
 */

#line 368 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_signal_to_noise); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L1_error)

#line 368 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->signal_to_noise = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":369
 *         self.intensity = intensity
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index             # <<<<<<<<<<<<<<
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge
 */

#line 369 "ms_deisotope/_c/peak_set.pyx"
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index, __pyx_v_index) < 0) __PYX_ERR(0, 369, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":370
 *         self.signal_to_noise = signal_to_noise
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max             # <<<<<<<<<<<<<<
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio
 */

#line 370 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_full_width_at_half_max); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 370, __pyx_L1_error)

#line 370 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->full_width_at_half_max = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":371
 *         self.index = index
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge             # <<<<<<<<<<<<<<
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass
 */

#line 371 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_charge); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L1_error)

#line 371 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->charge = __pyx_t_5;

  /* "ms_deisotope/_c/peak_set.pyx":372
 *         self.full_width_at_half_max = full_width_at_half_max
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio             # <<<<<<<<<<<<<<
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass
 */

#line 372 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_a_to_a2_ratio); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)

#line 372 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->a_to_a2_ratio = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":373
 *         self.charge = charge
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass             # <<<<<<<<<<<<<<
 *         self.average_mass = average_mass
 *         self.score = score
 */

#line 373 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_most_abundant_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 373, __pyx_L1_error)

#line 373 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->most_abundant_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":374
 *         self.a_to_a2_ratio = a_to_a2_ratio
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass             # <<<<<<<<<<<<<<
 *         self.score = score
 *         if not isinstance(envelope, Envelope):
 */

#line 374 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_average_mass); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 374, __pyx_L1_error)

#line 374 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->average_mass = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":375
 *         self.most_abundant_mass = most_abundant_mass
 *         self.average_mass = average_mass
 *         self.score = score             # <<<<<<<<<<<<<<
 *         if not isinstance(envelope, Envelope):
 *             self.envelope = Envelope(envelope)
 */

#line 375 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_score); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)

#line 375 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->score = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":376
 *         self.average_mass = average_mass
 *         self.score = score
 *         if not isinstance(envelope, Envelope):             # <<<<<<<<<<<<<<
 *             self.envelope = Envelope(envelope)
 *         else:
 */

#line 376 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_envelope, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope); 

#line 376 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 376 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":377
 *         self.score = score
 *         if not isinstance(envelope, Envelope):
 *             self.envelope = Envelope(envelope)             # <<<<<<<<<<<<<<
 *         else:
 *             self.envelope = envelope
 */

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_v_envelope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_self->envelope);

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_3);

#line 377 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":376
 *         self.average_mass = average_mass
 *         self.score = score
 *         if not isinstance(envelope, Envelope):             # <<<<<<<<<<<<<<
 *             self.envelope = Envelope(envelope)
 *         else:
 */

#line 376 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L5;

#line 376 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":379
 *             self.envelope = Envelope(envelope)
 *         else:
 *             self.envelope = envelope             # <<<<<<<<<<<<<<
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit
 */

#line 379 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 379 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_envelope) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_envelope, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(0, 379, __pyx_L1_error)

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_envelope;

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_self->envelope);

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_3);

#line 379 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 379 "ms_deisotope/_c/peak_set.pyx"
  }

#line 379 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5:;

  /* "ms_deisotope/_c/peak_set.pyx":380
 *         else:
 *             self.envelope = envelope
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)             # <<<<<<<<<<<<<<
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms
 */

#line 380 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_mz); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 380, __pyx_L1_error)

#line 380 "ms_deisotope/_c/peak_set.pyx"
  if (!__pyx_t_2) {

#line 380 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 380 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_mz); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)

#line 380 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_t_6;

#line 380 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6_bool_binop_done;

#line 380 "ms_deisotope/_c/peak_set.pyx"
  }

#line 380 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio(__pyx_v_self->neutral_mass, __pyx_v_self->charge, NULL);

#line 380 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_t_6;

#line 380 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6_bool_binop_done:;

#line 380 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->__pyx_base.mz = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":381
 *             self.envelope = envelope
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit             # <<<<<<<<<<<<<<
 *         self.chosen_for_msms = chosen_for_msms
 *         self.area = area
 */

#line 381 "ms_deisotope/_c/peak_set.pyx"
  if (!(likely(((__pyx_v_fit) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_fit, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(0, 381, __pyx_L1_error)

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_v_fit;

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_3);

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->fit);

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_3);

#line 381 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":382
 *         self.mz = mz or mass_charge_ratio(self.neutral_mass, self.charge)
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms             # <<<<<<<<<<<<<<
 *         self.area = area
 * 
 */

#line 382 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_chosen_for_msms); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)

#line 382 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->chosen_for_msms = __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":383
 *         self.fit = fit
 *         self.chosen_for_msms = chosen_for_msms
 *         self.area = area             # <<<<<<<<<<<<<<
 * 
 *     property index:
 */

#line 383 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_area); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)

#line 383 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->__pyx_base.area = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":357
 *         The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures
 *     """
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0):
 */

#line 357 "ms_deisotope/_c/peak_set.pyx"


#line 357 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_index);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_envelope);

#line 357 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 357 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 357 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":386
 * 
 *     property index:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._index
 * 
 */

#line 386 "ms_deisotope/_c/peak_set.pyx"


#line 386 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 386 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 386 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(PyObject *__pyx_v_self) {

#line 386 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 386 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 386 "ms_deisotope/_c/peak_set.pyx"


#line 386 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 386 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 386 "ms_deisotope/_c/peak_set.pyx"
}

#line 386 "ms_deisotope/_c/peak_set.pyx"


#line 386 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 386 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":387
 *     property index:
 *         def __get__(self):
 *             return self._index             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, value):
 */

#line 387 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 387 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_index));

#line 387 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_v_self->_index);

#line 387 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":386
 * 
 *     property index:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._index
 * 
 */

#line 386 "ms_deisotope/_c/peak_set.pyx"


#line 386 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 386 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 386 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 386 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":389
 *             return self._index
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if isinstance(value, _Index):
 *                 self._index = value
 */

#line 389 "ms_deisotope/_c/peak_set.pyx"


#line 389 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 389 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 389 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 389 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 389 "ms_deisotope/_c/peak_set.pyx"


#line 389 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 389 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 389 "ms_deisotope/_c/peak_set.pyx"
}

#line 389 "ms_deisotope/_c/peak_set.pyx"


#line 389 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":390
 * 
 *         def __set__(self, value):
 *             if isinstance(value, _Index):             # <<<<<<<<<<<<<<
 *                 self._index = value
 *             elif value >= 0:
 */

#line 390 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index); 

#line 390 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 390 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":391
 *         def __set__(self, value):
 *             if isinstance(value, _Index):
 *                 self._index = value             # <<<<<<<<<<<<<<
 *             elif value >= 0:
 *                 self._index = _Index._create(value, value)
 */

#line 391 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index))))) __PYX_ERR(0, 391, __pyx_L1_error)

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_value;

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_self->_index);

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);

#line 391 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":390
 * 
 *         def __set__(self, value):
 *             if isinstance(value, _Index):             # <<<<<<<<<<<<<<
 *                 self._index = value
 *             elif value >= 0:
 */

#line 390 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 390 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":392
 *             if isinstance(value, _Index):
 *                 self._index = value
 *             elif value >= 0:             # <<<<<<<<<<<<<<
 *                 self._index = _Index._create(value, value)
 *             else:
 */

#line 392 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_value, __pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)

#line 392 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 392, __pyx_L1_error)

#line 392 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 392 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":393
 *                 self._index = value
 *             elif value >= 0:
 *                 self._index = _Index._create(value, value)             # <<<<<<<<<<<<<<
 *             else:
 *                 self._index = _Index._create(0, 0)
 */

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(__pyx_t_4, __pyx_t_5)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 393, __pyx_L1_error)

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_self->_index);

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);

#line 393 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":392
 *             if isinstance(value, _Index):
 *                 self._index = value
 *             elif value >= 0:             # <<<<<<<<<<<<<<
 *                 self._index = _Index._create(value, value)
 *             else:
 */

#line 392 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 392 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":395
 *                 self._index = _Index._create(value, value)
 *             else:
 *                 self._index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */

#line 395 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 395, __pyx_L1_error)

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_self->_index);

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);

#line 395 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 395 "ms_deisotope/_c/peak_set.pyx"
  }

#line 395 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":389
 *             return self._index
 * 
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             if isinstance(value, _Index):
 *                 self._index = value
 */

#line 389 "ms_deisotope/_c/peak_set.pyx"


#line 389 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 389 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 389 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 389 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":397
 *                 self._index = _Index._create(0, 0)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 */

#line 397 "ms_deisotope/_c/peak_set.pyx"


#line 397 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 397 "ms_deisotope/_c/peak_set.pyx"
static Py_hash_t __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__(PyObject *__pyx_v_self); /*proto*/

#line 397 "ms_deisotope/_c/peak_set.pyx"
static Py_hash_t __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__(PyObject *__pyx_v_self) {

#line 397 "ms_deisotope/_c/peak_set.pyx"
  Py_hash_t __pyx_r;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = 
#line 397 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 397 "ms_deisotope/_c/peak_set.pyx"


#line 397 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 397 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 397 "ms_deisotope/_c/peak_set.pyx"
}

#line 397 "ms_deisotope/_c/peak_set.pyx"


#line 397 "ms_deisotope/_c/peak_set.pyx"
static Py_hash_t __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_2__hash__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 397 "ms_deisotope/_c/peak_set.pyx"
  Py_hash_t __pyx_r;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 397 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  Py_hash_t __pyx_t_5;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":398
 * 
 *     def __hash__(self):
 *         return hash((self.mz, self.intensity, self.charge))             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = PyObject_Hash(__pyx_t_4); if (unlikely(__pyx_t_5 == ((Py_hash_t)-1))) __PYX_ERR(0, 398, __pyx_L1_error)

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 398 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_5;

#line 398 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":397
 *                 self._index = _Index._create(0, 0)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 */

#line 397 "ms_deisotope/_c/peak_set.pyx"


#line 397 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;

#line 397 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 397 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 397 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":400
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 400 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 400 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 400 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 400 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 400 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 400 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 400 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)

#line 400 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 400 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone)) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 400 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 400 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 400 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 400 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 400 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 400 "ms_deisotope/_c/peak_set.pyx"
          }

#line 400 "ms_deisotope/_c/peak_set.pyx"
        }

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 400, __pyx_L1_error)

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
      }

#line 400 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 400 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 400 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 400 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 400 "ms_deisotope/_c/peak_set.pyx"
      }

#line 400 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 400 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 400 "ms_deisotope/_c/peak_set.pyx"
    }

#line 400 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 400 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":401
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 */

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":402
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 */

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 402, __pyx_L1_error)

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 402, __pyx_L1_error)

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 402, __pyx_L1_error)

#line 402 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":403
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 */

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 403, __pyx_L1_error)

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 403, __pyx_L1_error)

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 403, __pyx_L1_error)

#line 403 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":404
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 404, __pyx_L1_error)

#line 404 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":401
 * 
 *     cpdef PeakBase clone(self):
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                 self.most_abundant_mass, self.average_mass, self.score,
 */

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_5);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_6);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_7);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_8);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_9);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_10);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->envelope));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 10, ((PyObject *)__pyx_v_self->envelope));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_11);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->fit));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->fit));

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_t_14, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 401, __pyx_L1_error)

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_13);

#line 401 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 401 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":400
 *         return hash((self.mz, self.intensity, self.charge))
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak(self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                 self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 400 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 400 "ms_deisotope/_c/peak_set.pyx"
}

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 400 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 400 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone[] = "DeconvolutedPeak.clone(self) -> PeakBase";

#line 400 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 400 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 400 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 400 "ms_deisotope/_c/peak_set.pyx"
}

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 400 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 400 "ms_deisotope/_c/peak_set.pyx"


#line 400 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 400 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 400 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 400 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":406
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 406 "ms_deisotope/_c/peak_set.pyx"


#line 406 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 406 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 406 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__[] = "DeconvolutedPeak.__reduce__(self)";

#line 406 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 406 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 406 "ms_deisotope/_c/peak_set.pyx"


#line 406 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 406 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 406 "ms_deisotope/_c/peak_set.pyx"
}

#line 406 "ms_deisotope/_c/peak_set.pyx"


#line 406 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":407
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 */

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":408
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 */

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 408, __pyx_L1_error)

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 408, __pyx_L1_error)

#line 408 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":409
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 */

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 409, __pyx_L1_error)

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 409, __pyx_L1_error)

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 409, __pyx_L1_error)

#line 409 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":410
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 */

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 410, __pyx_L1_error)

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 410, __pyx_L1_error)

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 410, __pyx_L1_error)

#line 410 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":407
 * 
 *     def __reduce__(self):
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *                                   self.most_abundant_mass, self.average_mass, self.score,
 */

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_t_5);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_6);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_7);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_8);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_9);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_10);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->envelope));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 10, ((PyObject *)__pyx_v_self->envelope));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_11);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->fit));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 12, ((PyObject *)__pyx_v_self->fit));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 407, __pyx_L1_error)

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_13, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak));

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_14);

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_13;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 407 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":406
 *                                 self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return DeconvolutedPeak, (self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *                                   self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 406 "ms_deisotope/_c/peak_set.pyx"


#line 406 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 406 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 406 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 406 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":412
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 412 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 412 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 412 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 412 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 412 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 412 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 412 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)

#line 412 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 412 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq)) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 412 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 412 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 412 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 412 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 412 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 412 "ms_deisotope/_c/peak_set.pyx"
          }

#line 412 "ms_deisotope/_c/peak_set.pyx"
        }

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_5;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
      }

#line 412 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 412 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 412 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 412 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 412 "ms_deisotope/_c/peak_set.pyx"
      }

#line 412 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 412 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 412 "ms_deisotope/_c/peak_set.pyx"
    }

#line 412 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 412 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":413
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 */

#line 413 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->neutral_mass - __pyx_v_other->neutral_mass)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 413 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 413 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 413 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 413 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 413 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":414
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (             # <<<<<<<<<<<<<<
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon)
 */

#line 414 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.intensity - __pyx_v_other->__pyx_base.intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 414 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 414 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 414 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 414 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 414 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":415
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (             # <<<<<<<<<<<<<<
 *             abs(self.score - other.score) < epsilon)
 * 
 */

#line 415 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((__pyx_v_self->charge == __pyx_v_other->charge) != 0);

#line 415 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 415 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 415 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 415 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 415 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":416
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, other, int code):
 */

#line 416 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->score - __pyx_v_other->score)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 416 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_t_6;

#line 416 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_bool_binop_done:;

#line 416 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_5;

#line 416 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":412
 *                                   self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 412 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 412 "ms_deisotope/_c/peak_set.pyx"
}

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 412 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 412 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq[] = "DeconvolutedPeak._eq(self, DeconvolutedPeak other) -> bool";

#line 412 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, 1, "other", 0))) __PYX_ERR(0, 412, __pyx_L1_error)
  __pyx_r = 
#line 412 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other));

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 412 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 412 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 412 "ms_deisotope/_c/peak_set.pyx"
}

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other) {

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 412 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 412 "ms_deisotope/_c/peak_set.pyx"


#line 412 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 412 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 412 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 412 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":418
 *             abs(self.score - other.score) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 418 "ms_deisotope/_c/peak_set.pyx"


#line 418 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 418 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code); /*proto*/

#line 418 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 418 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = 
#line 418 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_code));

#line 418 "ms_deisotope/_c/peak_set.pyx"


#line 418 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 418 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 418 "ms_deisotope/_c/peak_set.pyx"
}

#line 418 "ms_deisotope/_c/peak_set.pyx"


#line 418 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_10__richcmp__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_code) {

#line 418 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 418 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":419
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 419 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_v_code) {

#line 419 "ms_deisotope/_c/peak_set.pyx"
    case 2:

    /* "ms_deisotope/_c/peak_set.pyx":420
 *     def __richcmp__(self, other, int code):
 *         if code == 2:
 *             return self._eq(other)             # <<<<<<<<<<<<<<
 *         elif code == 3:
 *             return not (self._eq(other))
 */

#line 420 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 420 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 420, __pyx_L1_error)

#line 420 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self->__pyx_base.__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 420, __pyx_L1_error)

#line 420 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 420 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 420 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 420 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":419
 * 
 *     def __richcmp__(self, other, int code):
 *         if code == 2:             # <<<<<<<<<<<<<<
 *             return self._eq(other)
 *         elif code == 3:
 */

#line 419 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 419 "ms_deisotope/_c/peak_set.pyx"
    case 3:

    /* "ms_deisotope/_c/peak_set.pyx":422
 *             return self._eq(other)
 *         elif code == 3:
 *             return not (self._eq(other))             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 422 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 422 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 422, __pyx_L1_error)

#line 422 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyBool_FromLong((!(((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self->__pyx_base.__pyx_vtab)->_eq(__pyx_v_self, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other), 0) != 0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)

#line 422 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 422 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 422 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 422 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":421
 *         if code == 2:
 *             return self._eq(other)
 *         elif code == 3:             # <<<<<<<<<<<<<<
 *             return not (self._eq(other))
 * 
 */

#line 421 "ms_deisotope/_c/peak_set.pyx"
    break;

#line 421 "ms_deisotope/_c/peak_set.pyx"
    default: break;

#line 421 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":418
 *             abs(self.score - other.score) < epsilon)
 * 
 *     def __richcmp__(self, other, int code):             # <<<<<<<<<<<<<<
 *         if code == 2:
 *             return self._eq(other)
 */

#line 418 "ms_deisotope/_c/peak_set.pyx"


#line 418 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 418 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 418 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 418 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 418 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":424
 *             return not (self._eq(other))
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

#line 424 "ms_deisotope/_c/peak_set.pyx"


#line 424 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 424 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__(PyObject *__pyx_v_self); /*proto*/

#line 424 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__(PyObject *__pyx_v_self) {

#line 424 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 424 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 424 "ms_deisotope/_c/peak_set.pyx"


#line 424 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 424 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 424 "ms_deisotope/_c/peak_set.pyx"
}

#line 424 "ms_deisotope/_c/peak_set.pyx"


#line 424 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 424 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 424 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":425
 * 
 *     def __repr__(self):
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "             # <<<<<<<<<<<<<<
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 */

#line 425 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":429
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 *             "intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, "
 *             "neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise})").format(self=self)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_self___class_____name___a_to_a2, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 429 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 429, __pyx_L1_error)

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 429, __pyx_L1_error)

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_3;

#line 429 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 429 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":424
 *             return not (self._eq(other))
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

#line 424 "ms_deisotope/_c/peak_set.pyx"


#line 424 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 424 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 424 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 424 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":433
 * 
 *     @staticmethod
 *     cdef DeconvolutedPeak _create_simple(double neutral_mass, double intensity, int charge,             # <<<<<<<<<<<<<<
 *                                          double score, double mz, Envelope envelope):
 *         cdef:
 */

#line 433 "ms_deisotope/_c/peak_set.pyx"


#line 433 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple(double __pyx_v_neutral_mass, double __pyx_v_intensity, int __pyx_v_charge, double __pyx_v_score, double __pyx_v_mz, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_inst = 0
#line 433 "ms_deisotope/_c/peak_set.pyx"
;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 433 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_create_simple", 0);

  /* "ms_deisotope/_c/peak_set.pyx":437
 *         cdef:
 *             DeconvolutedPeak inst
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)             # <<<<<<<<<<<<<<
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity
 */

#line 437 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(((PyTypeObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)

#line 437 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));

#line 437 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);

#line 437 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":438
 *             DeconvolutedPeak inst
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)
 *         inst.neutral_mass = neutral_mass             # <<<<<<<<<<<<<<
 *         inst.intensity = intensity
 *         inst.charge = charge
 */

#line 438 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->neutral_mass = __pyx_v_neutral_mass;

  /* "ms_deisotope/_c/peak_set.pyx":439
 *         inst = DeconvolutedPeak.__new__(DeconvolutedPeak)
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity             # <<<<<<<<<<<<<<
 *         inst.charge = charge
 *         inst.score = score
 */

#line 439 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->__pyx_base.intensity = __pyx_v_intensity;

  /* "ms_deisotope/_c/peak_set.pyx":440
 *         inst.neutral_mass = neutral_mass
 *         inst.intensity = intensity
 *         inst.charge = charge             # <<<<<<<<<<<<<<
 *         inst.score = score
 *         inst.signal_to_noise = score
 */

#line 440 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->charge = __pyx_v_charge;

  /* "ms_deisotope/_c/peak_set.pyx":441
 *         inst.intensity = intensity
 *         inst.charge = charge
 *         inst.score = score             # <<<<<<<<<<<<<<
 *         inst.signal_to_noise = score
 *         inst.mz = mz
 */

#line 441 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->score = __pyx_v_score;

  /* "ms_deisotope/_c/peak_set.pyx":442
 *         inst.charge = charge
 *         inst.score = score
 *         inst.signal_to_noise = score             # <<<<<<<<<<<<<<
 *         inst.mz = mz
 *         inst.envelope = envelope
 */

#line 442 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->signal_to_noise = __pyx_v_score;

  /* "ms_deisotope/_c/peak_set.pyx":443
 *         inst.score = score
 *         inst.signal_to_noise = score
 *         inst.mz = mz             # <<<<<<<<<<<<<<
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)
 */

#line 443 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->__pyx_base.mz = __pyx_v_mz;

  /* "ms_deisotope/_c/peak_set.pyx":444
 *         inst.signal_to_noise = score
 *         inst.mz = mz
 *         inst.envelope = envelope             # <<<<<<<<<<<<<<
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0
 */

#line 444 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_envelope));

#line 444 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_envelope));

#line 444 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_inst->envelope);

#line 444 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_inst->envelope));

#line 444 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->envelope = __pyx_v_envelope;

  /* "ms_deisotope/_c/peak_set.pyx":445
 *         inst.mz = mz
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0
 */

#line 445 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)

#line 445 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 445 "ms_deisotope/_c/peak_set.pyx"
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_inst), __pyx_n_s_index, __pyx_t_1) < 0) __PYX_ERR(0, 445, __pyx_L1_error)

#line 445 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":446
 *         inst.envelope = envelope
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0             # <<<<<<<<<<<<<<
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0
 */

#line 446 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->full_width_at_half_max = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":447
 *         inst.index = _Index._create(0, 0)
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0             # <<<<<<<<<<<<<<
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0
 */

#line 447 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->a_to_a2_ratio = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":448
 *         inst.full_width_at_half_max = 0
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0             # <<<<<<<<<<<<<<
 *         inst.average_mass = 0
 *         inst.area = 0
 */

#line 448 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->most_abundant_mass = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":449
 *         inst.a_to_a2_ratio = 0
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0             # <<<<<<<<<<<<<<
 *         inst.area = 0
 * 
 */

#line 449 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->average_mass = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":450
 *         inst.most_abundant_mass = 0
 *         inst.average_mass = 0
 *         inst.area = 0             # <<<<<<<<<<<<<<
 * 
 *         return inst
 */

#line 450 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst->__pyx_base.area = 0.0;

  /* "ms_deisotope/_c/peak_set.pyx":452
 *         inst.area = 0
 * 
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 452 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 452 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));

#line 452 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_inst;

#line 452 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":433
 * 
 *     @staticmethod
 *     cdef DeconvolutedPeak _create_simple(double neutral_mass, double intensity, int charge,             # <<<<<<<<<<<<<<
 *                                          double score, double mz, Envelope envelope):
 *         cdef:
 */

#line 433 "ms_deisotope/_c/peak_set.pyx"


#line 433 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._create_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 433 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 433 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 433 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":42
 * cdef class DeconvolutedPeak(PeakBase):
 *     cdef:
 *         public double neutral_mass             # <<<<<<<<<<<<<<
 *         # public double intensity
 *         public double signal_to_noise
 */

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 42 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 42 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(PyObject *__pyx_v_self) {

#line 42 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 42 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 42 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
}

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 42 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 42 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.neutral_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 42 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
}

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 42 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 42 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 42 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 42 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
}

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 42 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->neutral_mass = __pyx_t_1;

#line 42 "ms_deisotope/_c/peak_set.pxd"


#line 42 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.neutral_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 42 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 42 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 42 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":44
 *         public double neutral_mass
 *         # public double intensity
 *         public double signal_to_noise             # <<<<<<<<<<<<<<
 *         public _Index _index
 *         public double full_width_at_half_max
 */

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 44 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 44 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(PyObject *__pyx_v_self) {

#line 44 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 44 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 44 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
}

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 44 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 44 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->signal_to_noise); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 44, __pyx_L1_error)

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.signal_to_noise.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 44 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
}

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 44 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 44 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 44 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 44 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
}

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 44 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 44, __pyx_L1_error)

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->signal_to_noise = __pyx_t_1;

#line 44 "ms_deisotope/_c/peak_set.pxd"


#line 44 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.signal_to_noise.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 44 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 44 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 44 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":45
 *         # public double intensity
 *         public double signal_to_noise
 *         public _Index _index             # <<<<<<<<<<<<<<
 *         public double full_width_at_half_max
 *         public int charge
 */

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 45 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 45 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(PyObject *__pyx_v_self) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 45 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_index));

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = ((PyObject *)__pyx_v_self->_index);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 45 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index))))) __PYX_ERR(1, 45, __pyx_L1_error)

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->_index);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak._index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(PyObject *__pyx_v_self) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 45 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 45 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->_index);

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_index));

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None);

#line 45 "ms_deisotope/_c/peak_set.pxd"


#line 45 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 45 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 45 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 45 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":46
 *         public double signal_to_noise
 *         public _Index _index
 *         public double full_width_at_half_max             # <<<<<<<<<<<<<<
 *         public int charge
 *         public double a_to_a2_ratio
 */

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 46 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 46 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(PyObject *__pyx_v_self) {

#line 46 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 46 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 46 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
}

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 46 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 46 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->full_width_at_half_max); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 46, __pyx_L1_error)

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.full_width_at_half_max.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 46 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
}

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 46 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 46 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 46 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 46 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
}

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 46 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 46, __pyx_L1_error)

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->full_width_at_half_max = __pyx_t_1;

#line 46 "ms_deisotope/_c/peak_set.pxd"


#line 46 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.full_width_at_half_max.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 46 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 46 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 46 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":47
 *         public _Index _index
 *         public double full_width_at_half_max
 *         public int charge             # <<<<<<<<<<<<<<
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass
 */

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 47 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 47 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(PyObject *__pyx_v_self) {

#line 47 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 47 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 47 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
}

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 47 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 47 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->charge); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.charge.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 47 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
}

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 47 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 47 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 47 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 47 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
}

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_t_1;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 47, __pyx_L1_error)

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->charge = __pyx_t_1;

#line 47 "ms_deisotope/_c/peak_set.pxd"


#line 47 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.charge.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 47 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 47 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 47 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":48
 *         public double full_width_at_half_max
 *         public int charge
 *         public double a_to_a2_ratio             # <<<<<<<<<<<<<<
 *         public double most_abundant_mass
 *         public double average_mass
 */

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 48 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 48 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(PyObject *__pyx_v_self) {

#line 48 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 48 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 48 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
}

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 48 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 48 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->a_to_a2_ratio); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 48, __pyx_L1_error)

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.a_to_a2_ratio.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 48 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
}

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 48 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 48 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 48 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 48 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
}

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 48 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 48, __pyx_L1_error)

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->a_to_a2_ratio = __pyx_t_1;

#line 48 "ms_deisotope/_c/peak_set.pxd"


#line 48 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.a_to_a2_ratio.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 48 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 48 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 48 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":49
 *         public int charge
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass             # <<<<<<<<<<<<<<
 *         public double average_mass
 *         public double score
 */

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 49 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 49 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(PyObject *__pyx_v_self) {

#line 49 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 49 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 49 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
}

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 49 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 49 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->most_abundant_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 49, __pyx_L1_error)

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.most_abundant_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 49 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
}

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 49 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 49 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 49 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 49 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
}

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 49 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 49, __pyx_L1_error)

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->most_abundant_mass = __pyx_t_1;

#line 49 "ms_deisotope/_c/peak_set.pxd"


#line 49 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.most_abundant_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 49 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 49 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 49 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":50
 *         public double a_to_a2_ratio
 *         public double most_abundant_mass
 *         public double average_mass             # <<<<<<<<<<<<<<
 *         public double score
 *         # public double area
 */

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 50 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 50 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(PyObject *__pyx_v_self) {

#line 50 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 50 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 50 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
}

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 50 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 50 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->average_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 50, __pyx_L1_error)

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.average_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 50 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
}

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 50 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 50 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 50 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 50 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
}

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 50 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 50, __pyx_L1_error)

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->average_mass = __pyx_t_1;

#line 50 "ms_deisotope/_c/peak_set.pxd"


#line 50 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.average_mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 50 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 50 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 50 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":51
 *         public double most_abundant_mass
 *         public double average_mass
 *         public double score             # <<<<<<<<<<<<<<
 *         # public double area
 *         public Envelope envelope
 */

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 51 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 51 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(PyObject *__pyx_v_self) {

#line 51 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 51 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 51 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
}

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 51 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 51 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->score); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 51, __pyx_L1_error)

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.score.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 51 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
}

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 51 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 51 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 51 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 51 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
}

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 51 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 51, __pyx_L1_error)

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->score = __pyx_t_1;

#line 51 "ms_deisotope/_c/peak_set.pxd"


#line 51 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.score.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 51 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 51 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 51 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":53
 *         public double score
 *         # public double area
 *         public Envelope envelope             # <<<<<<<<<<<<<<
 *         # public double mz
 *         public IsotopicFitRecord fit
 */

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 53 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 53 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(PyObject *__pyx_v_self) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 53 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->envelope));

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = ((PyObject *)__pyx_v_self->envelope);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 53 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope))))) __PYX_ERR(1, 53, __pyx_L1_error)

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->envelope);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.envelope.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(PyObject *__pyx_v_self) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 53 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 53 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->envelope);

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->envelope));

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None);

#line 53 "ms_deisotope/_c/peak_set.pxd"


#line 53 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 53 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 53 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 53 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":55
 *         public Envelope envelope
 *         # public double mz
 *         public IsotopicFitRecord fit             # <<<<<<<<<<<<<<
 *         public bint chosen_for_msms
 * 
 */

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 55 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 55 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(PyObject *__pyx_v_self) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 55 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->fit));

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = ((PyObject *)__pyx_v_self->fit);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 55 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(1, 55, __pyx_L1_error)

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->fit);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_1);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.fit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(PyObject *__pyx_v_self) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 55 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 55 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->fit);

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->fit));

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None);

#line 55 "ms_deisotope/_c/peak_set.pxd"


#line 55 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 55 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 55 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 55 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":56
 *         # public double mz
 *         public IsotopicFitRecord fit
 *         public bint chosen_for_msms             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other)
 */

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 56 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 56 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(PyObject *__pyx_v_self) {

#line 56 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 56 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self));

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 56 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
}

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self) {

#line 56 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 56 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->chosen_for_msms); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 56, __pyx_L1_error)

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.chosen_for_msms.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 56 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
}

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 56 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 56 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 56 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 56 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
}

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_t_1;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 56, __pyx_L1_error)

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->chosen_for_msms = __pyx_t_1;

#line 56 "ms_deisotope/_c/peak_set.pxd"


#line 56 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeak.chosen_for_msms.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 56 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 56 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 56 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

#line 456 "ms_deisotope/_c/peak_set.pyx"


#line 456 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 456 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 456 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_neutral_mass = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_intensity = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_charge = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_signal_to_noise = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_index = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_full_width_at_half_max = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_a_to_a2_ratio = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_most_abundant_mass = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_average_mass = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_score = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_envelope = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_mz = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_fit = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_chosen_for_msms = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_area = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_drift_time = 0
#line 456 "ms_deisotope/_c/peak_set.pyx"
;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  {

#line 456 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_intensity,&__pyx_n_s_charge,&__pyx_n_s_signal_to_noise,&__pyx_n_s_index,&__pyx_n_s_full_width_at_half_max,&__pyx_n_s_a_to_a2_ratio,&__pyx_n_s_most_abundant_mass,&__pyx_n_s_average_mass,&__pyx_n_s_score,&__pyx_n_s_envelope,&__pyx_n_s_mz,&__pyx_n_s_fit,&__pyx_n_s_chosen_for_msms,&__pyx_n_s_area,&__pyx_n_s_drift_time,0};

#line 456 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

#line 456 "ms_deisotope/_c/peak_set.pyx"
    values[6] = ((PyObject *)__pyx_int_0);

#line 456 "ms_deisotope/_c/peak_set.pyx"
    values[7] = ((PyObject *)__pyx_int_0);

#line 456 "ms_deisotope/_c/peak_set.pyx"
    values[8] = ((PyObject *)__pyx_int_0);

#line 456 "ms_deisotope/_c/peak_set.pyx"
    values[9] = ((PyObject *)__pyx_int_0);

    /* "ms_deisotope/_c/peak_set.pyx":458
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):             # <<<<<<<<<<<<<<
 *         super(IonMobilityDeconvolutedPeak, self).__init__(
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 */

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[10] = ((PyObject *)__pyx_empty_tuple);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[11] = ((PyObject *)__pyx_int_0);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[12] = ((PyObject *)Py_None);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[13] = ((PyObject *)Py_False);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[14] = ((PyObject *)__pyx_int_0);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    values[15] = ((PyObject *)__pyx_int_0);

#line 458 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 458 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 458 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case 16: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[15] = PyTuple_GET_ITEM(__pyx_args, 15);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 15: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[14] = PyTuple_GET_ITEM(__pyx_args, 14);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 14: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[13] = PyTuple_GET_ITEM(__pyx_args, 13);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 13: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[12] = PyTuple_GET_ITEM(__pyx_args, 12);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 12: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[11] = PyTuple_GET_ITEM(__pyx_args, 11);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 11: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[10] = PyTuple_GET_ITEM(__pyx_args, 10);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 10: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[9] = PyTuple_GET_ITEM(__pyx_args, 9);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  9: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[8] = PyTuple_GET_ITEM(__pyx_args, 8);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  8: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[7] = PyTuple_GET_ITEM(__pyx_args, 7);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  7: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[6] = PyTuple_GET_ITEM(__pyx_args, 6);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  6: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  5: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 458 "ms_deisotope/_c/peak_set.pyx"
      }

#line 458 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 458 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 458 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_intensity)) != 0)) kw_args--;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 1); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  2:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_charge)) != 0)) kw_args--;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 2); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  3:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signal_to_noise)) != 0)) kw_args--;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 3); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  4:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 4); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  5:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_full_width_at_half_max)) != 0)) kw_args--;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, 5); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  6:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a_to_a2_ratio);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[6] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  7:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_most_abundant_mass);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[7] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  8:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_average_mass);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[8] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case  9:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_score);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[9] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 10:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_envelope);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[10] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 11:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mz);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[11] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 12:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[12] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 13:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_chosen_for_msms);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[13] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 14:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_area);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[14] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 458 "ms_deisotope/_c/peak_set.pyx"
        case 15:

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_drift_time);

#line 458 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[15] = value; kw_args--; }

#line 458 "ms_deisotope/_c/peak_set.pyx"
        }

#line 458 "ms_deisotope/_c/peak_set.pyx"
      }

#line 458 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 458 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
      }

#line 458 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 458 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case 16: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[15] = PyTuple_GET_ITEM(__pyx_args, 15);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 15: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[14] = PyTuple_GET_ITEM(__pyx_args, 14);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 14: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[13] = PyTuple_GET_ITEM(__pyx_args, 13);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 13: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[12] = PyTuple_GET_ITEM(__pyx_args, 12);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 12: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[11] = PyTuple_GET_ITEM(__pyx_args, 11);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 11: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[10] = PyTuple_GET_ITEM(__pyx_args, 10);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case 10: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[9] = PyTuple_GET_ITEM(__pyx_args, 9);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  9: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[8] = PyTuple_GET_ITEM(__pyx_args, 8);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  8: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[7] = PyTuple_GET_ITEM(__pyx_args, 7);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  7: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[6] = PyTuple_GET_ITEM(__pyx_args, 6);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  6: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
values[5] = PyTuple_GET_ITEM(__pyx_args, 5);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 458 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 458 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 458 "ms_deisotope/_c/peak_set.pyx"
      }

#line 458 "ms_deisotope/_c/peak_set.pyx"
    }

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = values[0];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_intensity = values[1];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_charge = values[2];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_signal_to_noise = values[3];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index = values[4];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_full_width_at_half_max = values[5];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_a_to_a2_ratio = values[6];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_most_abundant_mass = values[7];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_average_mass = values[8];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_score = values[9];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_envelope = values[10];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mz = values[11];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_fit = values[12];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_chosen_for_msms = values[13];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_area = values[14];

#line 458 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_drift_time = values[15];

#line 458 "ms_deisotope/_c/peak_set.pyx"
  }

#line 458 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 458 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 6, 16, PyTuple_GET_SIZE(__pyx_args)); 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 456, __pyx_L3_error)

#line 458 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 458 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 458 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 458 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 458 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 458 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area, __pyx_v_drift_time);

  /* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

#line 456 "ms_deisotope/_c/peak_set.pyx"


#line 456 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 456 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 456 "ms_deisotope/_c/peak_set.pyx"
}

#line 456 "ms_deisotope/_c/peak_set.pyx"


#line 456 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_neutral_mass, PyObject *__pyx_v_intensity, PyObject *__pyx_v_charge, PyObject *__pyx_v_signal_to_noise, PyObject *__pyx_v_index, PyObject *__pyx_v_full_width_at_half_max, PyObject *__pyx_v_a_to_a2_ratio, PyObject *__pyx_v_most_abundant_mass, PyObject *__pyx_v_average_mass, PyObject *__pyx_v_score, PyObject *__pyx_v_envelope, PyObject *__pyx_v_mz, PyObject *__pyx_v_fit, PyObject *__pyx_v_chosen_for_msms, PyObject *__pyx_v_area, PyObject *__pyx_v_drift_time) {

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 456 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_4;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_6;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":459
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 *         super(IonMobilityDeconvolutedPeak, self).__init__(             # <<<<<<<<<<<<<<
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 */

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":462
 *             neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 *             envelope, mz, fit, chosen_for_msms, area)             # <<<<<<<<<<<<<<
 *         self.drift_time = drift_time
 * 
 */

#line 462 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 462 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 462 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 462 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 462 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 462 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 462 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 462 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = 1;

#line 462 "ms_deisotope/_c/peak_set.pyx"
    }

#line 462 "ms_deisotope/_c/peak_set.pyx"
  }

#line 462 "ms_deisotope/_c/peak_set.pyx"
  #if CYTHON_FAST_PYCALL

#line 462 "ms_deisotope/_c/peak_set.pyx"
  if (PyFunction_Check(__pyx_t_2)) {

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyObject *__pyx_temp[16] = {__pyx_t_3, __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area};

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 15+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 462 "ms_deisotope/_c/peak_set.pyx"
  } else

#line 462 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 462 "ms_deisotope/_c/peak_set.pyx"
  #if CYTHON_FAST_PYCCALL

#line 462 "ms_deisotope/_c/peak_set.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyObject *__pyx_temp[16] = {__pyx_t_3, __pyx_v_neutral_mass, __pyx_v_intensity, __pyx_v_charge, __pyx_v_signal_to_noise, __pyx_v_index, __pyx_v_full_width_at_half_max, __pyx_v_a_to_a2_ratio, __pyx_v_most_abundant_mass, __pyx_v_average_mass, __pyx_v_score, __pyx_v_envelope, __pyx_v_mz, __pyx_v_fit, __pyx_v_chosen_for_msms, __pyx_v_area};

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 15+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 462 "ms_deisotope/_c/peak_set.pyx"
  } else

#line 462 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 462 "ms_deisotope/_c/peak_set.pyx"
  {

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = PyTuple_New(15+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

#line 462 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;

#line 462 "ms_deisotope/_c/peak_set.pyx"
    }

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_neutral_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_neutral_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_neutral_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_intensity);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_intensity);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_intensity);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_charge);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_charge);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_charge);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_signal_to_noise);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_signal_to_noise);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_signal_to_noise);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_index);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_index);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 4+__pyx_t_4, __pyx_v_index);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_full_width_at_half_max);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_full_width_at_half_max);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 5+__pyx_t_4, __pyx_v_full_width_at_half_max);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_a_to_a2_ratio);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_a_to_a2_ratio);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 6+__pyx_t_4, __pyx_v_a_to_a2_ratio);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_most_abundant_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_most_abundant_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 7+__pyx_t_4, __pyx_v_most_abundant_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_average_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_average_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 8+__pyx_t_4, __pyx_v_average_mass);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_score);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_score);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 9+__pyx_t_4, __pyx_v_score);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_envelope);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_envelope);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 10+__pyx_t_4, __pyx_v_envelope);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_mz);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_mz);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 11+__pyx_t_4, __pyx_v_mz);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_fit);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_fit);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 12+__pyx_t_4, __pyx_v_fit);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_chosen_for_msms);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_chosen_for_msms);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 13+__pyx_t_4, __pyx_v_chosen_for_msms);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_area);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_area);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_5, 14+__pyx_t_4, __pyx_v_area);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 462 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 462 "ms_deisotope/_c/peak_set.pyx"
  }

#line 462 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 462 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":463
 *             a_to_a2_ratio, most_abundant_mass, average_mass, score,
 *             envelope, mz, fit, chosen_for_msms, area)
 *         self.drift_time = drift_time             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */

#line 463 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_drift_time); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 463, __pyx_L1_error)

#line 463 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->drift_time = __pyx_t_6;

  /* "ms_deisotope/_c/peak_set.pyx":456
 * 
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     def __init__(self, neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max,             # <<<<<<<<<<<<<<
 *                  a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0,
 *                  envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0):
 */

#line 456 "ms_deisotope/_c/peak_set.pyx"


#line 456 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 456 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 456 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 456 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":465
 *         self.drift_time = drift_time
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 465 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, int __pyx_skip_dispatch) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 465 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 465 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 465 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 465 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 465 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 465 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)

#line 465 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 465 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone)) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 465 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 465 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 465 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 465 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 465 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 465 "ms_deisotope/_c/peak_set.pyx"
          }

#line 465 "ms_deisotope/_c/peak_set.pyx"
        }

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 465, __pyx_L1_error)

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
      }

#line 465 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 465 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 465 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 465 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 465 "ms_deisotope/_c/peak_set.pyx"
      }

#line 465 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 465 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 465 "ms_deisotope/_c/peak_set.pyx"
    }

#line 465 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 465 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":466
 * 
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

  /* "ms_deisotope/_c/peak_set.pyx":467
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 467, __pyx_L1_error)

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 467, __pyx_L1_error)

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 467, __pyx_L1_error)

#line 467 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":468
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 */

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 468, __pyx_L1_error)

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 468, __pyx_L1_error)

#line 468 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":469
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)
 */

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 469, __pyx_L1_error)

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 469, __pyx_L1_error)

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 469, __pyx_L1_error)

#line 469 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":470
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,             # <<<<<<<<<<<<<<
 *             self.drift_time)
 * 
 */

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 470, __pyx_L1_error)

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 470, __pyx_L1_error)

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 470, __pyx_L1_error)

#line 470 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":471
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 471 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 471, __pyx_L1_error)

#line 471 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

  /* "ms_deisotope/_c/peak_set.pyx":466
 * 
 *     cpdef PeakBase clone(self):
 *         return IonMobilityDeconvolutedPeak(             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = PyTuple_New(16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 466, __pyx_L1_error)

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_1);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_2);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_3);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_4);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_5);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_6);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_7);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_t_8);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_9);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_10);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 10, ((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_t_11);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 12, ((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_12);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_13);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_14);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak), __pyx_t_15, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 466, __pyx_L1_error)

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_14);

#line 466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 466 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":465
 *         self.drift_time = drift_time
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak(
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 465 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 465 "ms_deisotope/_c/peak_set.pyx"
}

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 465 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 465 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone[] = "IonMobilityDeconvolutedPeak.clone(self) -> PeakBase";

#line 465 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 465 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 465 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 465 "ms_deisotope/_c/peak_set.pyx"
}

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 465 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 465 "ms_deisotope/_c/peak_set.pyx"


#line 465 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 465 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 465 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 465 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":473
 *             self.drift_time)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 473 "ms_deisotope/_c/peak_set.pyx"


#line 473 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 473 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 473 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__[] = "IonMobilityDeconvolutedPeak.__reduce__(self)";

#line 473 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 473 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

#line 473 "ms_deisotope/_c/peak_set.pyx"


#line 473 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 473 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 473 "ms_deisotope/_c/peak_set.pyx"
}

#line 473 "ms_deisotope/_c/peak_set.pyx"


#line 473 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":474
 * 
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":475
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 475, __pyx_L1_error)

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

  /* "ms_deisotope/_c/peak_set.pyx":476
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 */

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 476, __pyx_L1_error)

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 476, __pyx_L1_error)

#line 476 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":477
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)
 */

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 477, __pyx_L1_error)

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 477, __pyx_L1_error)

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 477, __pyx_L1_error)

#line 477 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":478
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,             # <<<<<<<<<<<<<<
 *             self.drift_time)
 * 
 */

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 478, __pyx_L1_error)

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 478, __pyx_L1_error)

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 478, __pyx_L1_error)

#line 478 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":479
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.fit, self.chosen_for_msms, self.area,
 *             self.drift_time)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 */

#line 479 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 479, __pyx_L1_error)

#line 479 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

  /* "ms_deisotope/_c/peak_set.pyx":475
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = PyTuple_New(16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 475, __pyx_L1_error)

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_1);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_2);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_3);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_4);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_5);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_6);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_7);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_t_8);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_9);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_10);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 10, ((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_t_11);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 12, ((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_12);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_13);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_14);

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 475 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":474
 * 
 *     def __reduce__(self):
 *         return IonMobilityDeconvolutedPeak, (             # <<<<<<<<<<<<<<
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 474, __pyx_L1_error)

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 474 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak));

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_15);

#line 474 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_15);

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = 0;

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_14;

#line 474 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 474 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":473
 *             self.drift_time)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return IonMobilityDeconvolutedPeak, (
 *             self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 473 "ms_deisotope/_c/peak_set.pyx"


#line 473 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 473 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 473 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 473 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":481
 *             self.drift_time)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 481 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other, int __pyx_skip_dispatch) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 481 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 481 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 481 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 481 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 481 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 481 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)

#line 481 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 481 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq)) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 481 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 481 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 481 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 481 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 481 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 481 "ms_deisotope/_c/peak_set.pyx"
          }

#line 481 "ms_deisotope/_c/peak_set.pyx"
        }

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 481, __pyx_L1_error)

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 481, __pyx_L1_error)

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_5;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
      }

#line 481 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 481 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 481 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 481 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 481 "ms_deisotope/_c/peak_set.pyx"
      }

#line 481 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 481 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 481 "ms_deisotope/_c/peak_set.pyx"
    }

#line 481 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 481 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":482
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 */

#line 482 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.neutral_mass - __pyx_v_other->neutral_mass)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 482 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 482 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 482 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 482 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 482 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":483
 *     cpdef bint _eq(self, DeconvolutedPeak other):
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (             # <<<<<<<<<<<<<<
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (
 */

#line 483 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.__pyx_base.intensity - __pyx_v_other->__pyx_base.intensity)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 483 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 483 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 483 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 483 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 483 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":484
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (             # <<<<<<<<<<<<<<
 *             abs(self.score - other.score) < epsilon) and (
 *             abs(self.drift_time - other.drift_time) < epsilon)
 */

#line 484 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.charge == __pyx_v_other->charge) != 0);

#line 484 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 484 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 484 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 484 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 484 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":485
 *             abs(self.intensity - other.intensity) < epsilon) and (
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (             # <<<<<<<<<<<<<<
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 */

#line 485 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = ((fabs((__pyx_v_self->__pyx_base.score - __pyx_v_other->score)) < __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon) != 0);

#line 485 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_6) {

#line 485 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 485 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_6;

#line 485 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3_bool_binop_done;

#line 485 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":486
 *             self.charge == other.charge) and (
 *             abs(self.score - other.score) < epsilon) and (
 *             abs(self.drift_time - other.drift_time) < epsilon)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_other), __pyx_n_s_drift_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyNumber_Absolute(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_12ms_deisotope_2_c_8peak_set_epsilon); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 486, __pyx_L1_error)

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_t_6;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_bool_binop_done:;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_5;

#line 486 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":481
 *             self.drift_time)
 * 
 *     cpdef bint _eq(self, DeconvolutedPeak other):             # <<<<<<<<<<<<<<
 *         return (abs(self.neutral_mass - other.neutral_mass) < epsilon) and (
 *             abs(self.intensity - other.intensity) < epsilon) and (
 */

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 481 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 481 "ms_deisotope/_c/peak_set.pyx"
}

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 481 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 481 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq[] = "IonMobilityDeconvolutedPeak._eq(self, DeconvolutedPeak other) -> bool";

#line 481 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq (wrapper)", 0);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, 1, "other", 0))) __PYX_ERR(0, 481, __pyx_L1_error)
  __pyx_r = 
#line 481 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_other));

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 481 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 481 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 481 "ms_deisotope/_c/peak_set.pyx"
}

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_other) {

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 481 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_eq", 0);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 481 "ms_deisotope/_c/peak_set.pyx"


#line 481 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 481 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 481 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 481 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":488
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

#line 488 "ms_deisotope/_c/peak_set.pyx"


#line 488 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 488 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__(PyObject *__pyx_v_self); /*proto*/

#line 488 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__(PyObject *__pyx_v_self) {

#line 488 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 488 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

#line 488 "ms_deisotope/_c/peak_set.pyx"


#line 488 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 488 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 488 "ms_deisotope/_c/peak_set.pyx"
}

#line 488 "ms_deisotope/_c/peak_set.pyx"


#line 488 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {

#line 488 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 488 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":489
 * 
 *     def __repr__(self):
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "             # <<<<<<<<<<<<<<
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 *             "envelope={self.envelope}, full_width_at_half_max={self.full_width_at_half_max}, index={self.index}, "
 */

#line 489 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":494
 *             "intensity={self.intensity}, most_abundant_mass={self.most_abundant_mass}, mz={self.mz}, "
 *             "neutral_mass={self.neutral_mass}, score={self.score}, signal_to_noise={self.signal_to_noise},"
 *             "drift_time={self.drift_time})").format(self=self)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_self___class_____name___a_to_a2_2, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 494 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 494, __pyx_L1_error)

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_3;

#line 494 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 494 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":488
 *             abs(self.drift_time - other.drift_time) < epsilon)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return ("{self.__class__.__name__}(a_to_a2_ratio={self.a_to_a2_ratio}, area={self.area}, "
 *             "average_mass={self.average_mass}, charge={self.charge}, chosen_for_msms={self.chosen_for_msms}, "
 */

#line 488 "ms_deisotope/_c/peak_set.pyx"


#line 488 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 488 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 488 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 488 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":68
 * cdef class IonMobilityDeconvolutedPeak(DeconvolutedPeak):
 *     cdef:
 *         public double drift_time             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 68 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 68 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(PyObject *__pyx_v_self) {

#line 68 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 68 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self));

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 68 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
}

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self) {

#line 68 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 68 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->drift_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 68, __pyx_L1_error)

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.drift_time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 68 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
}

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 68 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 68 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 68 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 68 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
}

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *__pyx_v_self, PyObject *__pyx_v_value) {

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 68 "ms_deisotope/_c/peak_set.pxd"
  double __pyx_t_1;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 68, __pyx_L1_error)

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->drift_time = __pyx_t_1;

#line 68 "ms_deisotope/_c/peak_set.pxd"


#line 68 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak.drift_time.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 68 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 68 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 68 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":510
 *         Representation of the "source" of the isotopic fit
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 */

#line 510 "ms_deisotope/_c/peak_set.pyx"


#line 510 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 510 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 510 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_solution = 0
#line 510 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_fit = 0
#line 510 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_args = 0
#line 510 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_kwargs = 0
#line 510 "ms_deisotope/_c/peak_set.pyx"
;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_kwargs);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  if (PyTuple_GET_SIZE(__pyx_args) > 2) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_args = PyTuple_GetSlice(__pyx_args, 2, PyTuple_GET_SIZE(__pyx_args));

#line 510 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_v_args)) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_RefNannyFinishContext();

#line 510 "ms_deisotope/_c/peak_set.pyx"
      return -1;

#line 510 "ms_deisotope/_c/peak_set.pyx"
    }

#line 510 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_args);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  } else {
    __pyx_v_args = __pyx_empty_tuple; 
#line 510 "ms_deisotope/_c/peak_set.pyx"
__Pyx_INCREF(__pyx_empty_tuple);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  }

#line 510 "ms_deisotope/_c/peak_set.pyx"
  {

#line 510 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_solution,&__pyx_n_s_fit,0};

#line 510 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 510 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 510 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 510 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
        default:
        case  2: 
#line 510 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 510 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 510 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 510 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 510 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;

#line 510 "ms_deisotope/_c/peak_set.pyx"
      }

#line 510 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 510 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 510 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_solution)) != 0)) kw_args--;
        else 
#line 510 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 510 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 510 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 510 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fit)) != 0)) kw_args--;

#line 510 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 2, 1); 
#line 510 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 510, __pyx_L3_error)

#line 510 "ms_deisotope/_c/peak_set.pyx"
        }

#line 510 "ms_deisotope/_c/peak_set.pyx"
      }

#line 510 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
        const Py_ssize_t used_pos_args = (pos_args < 2) ? pos_args : 2;

#line 510 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, used_pos_args, "__init__") < 0)) __PYX_ERR(0, 510, __pyx_L3_error)

#line 510 "ms_deisotope/_c/peak_set.pyx"
      }

#line 510 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) < 2) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 510 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 510 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 510 "ms_deisotope/_c/peak_set.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 510 "ms_deisotope/_c/peak_set.pyx"
    }

#line 510 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_solution = values[0];

#line 510 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_fit = values[1];

#line 510 "ms_deisotope/_c/peak_set.pyx"
  }

#line 510 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 510 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 510, __pyx_L3_error)

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_args); __pyx_v_args = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 510 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 510 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self), __pyx_v_solution, __pyx_v_fit, __pyx_v_args, __pyx_v_kwargs);

#line 510 "ms_deisotope/_c/peak_set.pyx"


#line 510 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_args);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_kwargs);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 510 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 510 "ms_deisotope/_c/peak_set.pyx"
}

#line 510 "ms_deisotope/_c/peak_set.pyx"


#line 510 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_solution, PyObject *__pyx_v_fit, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs) {

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 510 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":511
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):
 *         self.solution = solution             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 *         self.fit = fit
 */

#line 511 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_solution);

#line 511 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_solution);

#line 511 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->solution);

#line 511 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->solution);

#line 511 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->solution = __pyx_v_solution;

  /* "ms_deisotope/_c/peak_set.pyx":512
 *     def __init__(self, solution, fit, *args, **kwargs):
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)             # <<<<<<<<<<<<<<
 *         self.fit = fit
 * 
 */

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_v_args, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 512, __pyx_L1_error)

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 512 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":513
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 *         self.fit = fit             # <<<<<<<<<<<<<<
 * 
 *     cpdef PeakBase clone(self):
 */

#line 513 "ms_deisotope/_c/peak_set.pyx"
  if (!(likely(((__pyx_v_fit) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_fit, __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord))))) __PYX_ERR(0, 513, __pyx_L1_error)

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_v_fit;

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_3);

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.fit);

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->__pyx_base.fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)__pyx_t_3);

#line 513 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":510
 *         Representation of the "source" of the isotopic fit
 *     """
 *     def __init__(self, solution, fit, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         self.solution = solution
 *         super(DeconvolutedPeakSolution, self).__init__(*args, **kwargs)
 */

#line 510 "ms_deisotope/_c/peak_set.pyx"


#line 510 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 510 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 510 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 510 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":515
 *         self.fit = fit
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return self.__class__(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 515 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, int __pyx_skip_dispatch) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *__pyx_r = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_16 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_17;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_18 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 515 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 515 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 515 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 515 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 515 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 515 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)

#line 515 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 515 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone)) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 515 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 515 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 515 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 515 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 515 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 515 "ms_deisotope/_c/peak_set.pyx"
          }

#line 515 "ms_deisotope/_c/peak_set.pyx"
        }

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 515, __pyx_L1_error)

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 515, __pyx_L1_error)

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_2);

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
      }

#line 515 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 515 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 515 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 515 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 515 "ms_deisotope/_c/peak_set.pyx"
      }

#line 515 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 515 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 515 "ms_deisotope/_c/peak_set.pyx"
    }

#line 515 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 515 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":516
 * 
 *     cpdef PeakBase clone(self):
 *         return self.__class__(             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 516 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 516 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)

#line 516 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

  /* "ms_deisotope/_c/peak_set.pyx":517
 *     cpdef PeakBase clone(self):
 *         return self.__class__(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 517, __pyx_L1_error)

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 517, __pyx_L1_error)

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 517, __pyx_L1_error)

#line 517 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

  /* "ms_deisotope/_c/peak_set.pyx":518
 *         return self.__class__(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 */

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 518, __pyx_L1_error)

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 518, __pyx_L1_error)

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 518, __pyx_L1_error)

#line 518 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

  /* "ms_deisotope/_c/peak_set.pyx":519
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 */

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 519, __pyx_L1_error)

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 519, __pyx_L1_error)

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 519, __pyx_L1_error)

#line 519 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

  /* "ms_deisotope/_c/peak_set.pyx":520
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 520, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 520, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 520, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_16 = NULL;

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_17 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_2);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_16)) {

#line 520 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 520 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_16);

#line 520 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 520 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 520 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_17 = 1;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    }

#line 520 "ms_deisotope/_c/peak_set.pyx"
  }

#line 520 "ms_deisotope/_c/peak_set.pyx"
  #if CYTHON_FAST_PYCALL

#line 520 "ms_deisotope/_c/peak_set.pyx"
  if (PyFunction_Check(__pyx_t_2)) {

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyObject *__pyx_temp[17] = {__pyx_t_16, __pyx_v_self->solution, ((PyObject *)__pyx_v_self->__pyx_base.fit), __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_t_10, __pyx_t_11, __pyx_t_12, ((PyObject *)__pyx_v_self->__pyx_base.envelope), __pyx_t_13, __pyx_t_14, __pyx_t_15};

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_17, 16+__pyx_t_17); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
  } else

#line 520 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 520 "ms_deisotope/_c/peak_set.pyx"
  #if CYTHON_FAST_PYCCALL

#line 520 "ms_deisotope/_c/peak_set.pyx"
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyObject *__pyx_temp[17] = {__pyx_t_16, __pyx_v_self->solution, ((PyObject *)__pyx_v_self->__pyx_base.fit), __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, __pyx_t_10, __pyx_t_11, __pyx_t_12, ((PyObject *)__pyx_v_self->__pyx_base.envelope), __pyx_t_13, __pyx_t_14, __pyx_t_15};

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_17, 16+__pyx_t_17); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
  } else

#line 520 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 520 "ms_deisotope/_c/peak_set.pyx"
  {

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_18 = PyTuple_New(16+__pyx_t_17); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 516, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_18);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_16) {

#line 520 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_t_16); __pyx_t_16 = NULL;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    }

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_self->solution);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_v_self->solution);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 0+__pyx_t_17, __pyx_v_self->solution);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 1+__pyx_t_17, ((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 2+__pyx_t_17, __pyx_t_3);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_4);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 3+__pyx_t_17, __pyx_t_4);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_5);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 4+__pyx_t_17, __pyx_t_5);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_6);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 5+__pyx_t_17, __pyx_t_6);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_7);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 6+__pyx_t_17, __pyx_t_7);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_8);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 7+__pyx_t_17, __pyx_t_8);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_9);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 8+__pyx_t_17, __pyx_t_9);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_10);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 9+__pyx_t_17, __pyx_t_10);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_11);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 10+__pyx_t_17, __pyx_t_11);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_12);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 11+__pyx_t_17, __pyx_t_12);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 12+__pyx_t_17, ((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_13);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 13+__pyx_t_17, __pyx_t_13);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_14);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 14+__pyx_t_17, __pyx_t_14);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_15);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_18, 15+__pyx_t_17, __pyx_t_15);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_7 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_9 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_10 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_11 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_12 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_13 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_14 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_15 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 520 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

#line 520 "ms_deisotope/_c/peak_set.pyx"
  }

#line 520 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":516
 * 
 *     cpdef PeakBase clone(self):
 *         return self.__class__(             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 516 "ms_deisotope/_c/peak_set.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase))))) __PYX_ERR(0, 516, __pyx_L1_error)

#line 516 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_t_1);

#line 516 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 516 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":515
 *         self.fit = fit
 * 
 *     cpdef PeakBase clone(self):             # <<<<<<<<<<<<<<
 *         return self.__class__(
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_16);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_18);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 515 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 515 "ms_deisotope/_c/peak_set.pyx"
}

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 515 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 515 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone[] = "DeconvolutedPeakSolution.clone(self) -> PeakBase";

#line 515 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 515 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 515 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 515 "ms_deisotope/_c/peak_set.pyx"
}

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 515 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 515 "ms_deisotope/_c/peak_set.pyx"


#line 515 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 515 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 515 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 515 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":522
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 522 "ms_deisotope/_c/peak_set.pyx"


#line 522 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 522 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 522 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__[] = "DeconvolutedPeakSolution.__reduce__(self)";

#line 522 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 522 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

#line 522 "ms_deisotope/_c/peak_set.pyx"


#line 522 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 522 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 522 "ms_deisotope/_c/peak_set.pyx"
}

#line 522 "ms_deisotope/_c/peak_set.pyx"


#line 522 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":523
 * 
 *     def __reduce__(self):
 *         return self.__class__, (             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

  /* "ms_deisotope/_c/peak_set.pyx":524
 *     def __reduce__(self):
 *         return self.__class__, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.neutral_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 524, __pyx_L1_error)

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.intensity); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.charge); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 524, __pyx_L1_error)

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.signal_to_noise); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 524, __pyx_L1_error)

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

  /* "ms_deisotope/_c/peak_set.pyx":525
 *         return self.__class__, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,             # <<<<<<<<<<<<<<
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 */

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 525, __pyx_L1_error)

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.full_width_at_half_max); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 525, __pyx_L1_error)

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.a_to_a2_ratio); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 525, __pyx_L1_error)

#line 525 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

  /* "ms_deisotope/_c/peak_set.pyx":526
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,             # <<<<<<<<<<<<<<
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 */

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.most_abundant_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 526, __pyx_L1_error)

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.average_mass); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 526, __pyx_L1_error)

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.score); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 526, __pyx_L1_error)

#line 526 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

  /* "ms_deisotope/_c/peak_set.pyx":527
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.mz); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 527, __pyx_L1_error)

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.chosen_for_msms); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 527, __pyx_L1_error)

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.area); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 527, __pyx_L1_error)

#line 527 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

  /* "ms_deisotope/_c/peak_set.pyx":524
 *     def __reduce__(self):
 *         return self.__class__, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,             # <<<<<<<<<<<<<<
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 *             self.most_abundant_mass, self.average_mass, self.score,
 */

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = PyTuple_New(16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 524, __pyx_L1_error)

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_self->solution);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_self->solution);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_v_self->solution);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 1, ((PyObject *)__pyx_v_self->__pyx_base.fit));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_t_2);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_3);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_4);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_5);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_6);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_t_7);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_8);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_9);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 10, __pyx_t_10);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_t_11);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 12, ((PyObject *)__pyx_v_self->__pyx_base.envelope));

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_12);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_13);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_14);

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 524 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":523
 * 
 *     def __reduce__(self):
 *         return self.__class__, (             # <<<<<<<<<<<<<<
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 *             self.index, self.full_width_at_half_max, self.a_to_a2_ratio,
 */

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 523, __pyx_L1_error)

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_15);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_15);

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = 0;

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_14;

#line 523 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 523 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":522
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (
 *             self.solution, self.fit, self.neutral_mass, self.intensity, self.charge, self.signal_to_noise,
 */

#line 522 "ms_deisotope/_c/peak_set.pyx"


#line 522 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 522 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 522 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 522 "ms_deisotope/_c/peak_set.pyx"
}

#line 522 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 529 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__(PyObject *__pyx_v_self); /*proto*/

#line 529 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__(PyObject *__pyx_v_self) {

#line 529 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = 
#line 529 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 529 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 529 "ms_deisotope/_c/peak_set.pyx"
}

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_6__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_cur_scope
#line 529 "ms_deisotope/_c/peak_set.pyx"
;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, __pyx_empty_tuple, NULL);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)Py_None);

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 529, __pyx_L1_error)

#line 529 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  }

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  {

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_DeconvolutedPeakSolution___iter, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 529, __pyx_L1_error)

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_cur_scope);

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 529 "ms_deisotope/_c/peak_set.pyx"
    return (PyObject *) gen;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  }

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 529 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 529 "ms_deisotope/_c/peak_set.pyx"
}

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{

#line 529 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)__pyx_generator->closure);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_generator->resume_label) {

#line 529 "ms_deisotope/_c/peak_set.pyx"
    case 0: goto __pyx_L3_first_run;

#line 529 "ms_deisotope/_c/peak_set.pyx"
    case 1: goto __pyx_L4_resume_from_yield;

#line 529 "ms_deisotope/_c/peak_set.pyx"
    case 2: goto __pyx_L5_resume_from_yield;

#line 529 "ms_deisotope/_c/peak_set.pyx"
    case 3: goto __pyx_L6_resume_from_yield;

#line 529 "ms_deisotope/_c/peak_set.pyx"
    default: /* CPython raises the right error here */

#line 529 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 529 "ms_deisotope/_c/peak_set.pyx"
    return NULL;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  }

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_first_run:;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 529, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":530
 * 
 *     def __iter__(self):
 *         yield self.solution             # <<<<<<<<<<<<<<
 *         yield self
 *         yield self.fit
 */

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->solution);

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_cur_scope->__pyx_v_self->solution;

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 530 "ms_deisotope/_c/peak_set.pyx"
  /* return from generator, yielding value */

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = 1;

#line 530 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 530 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_resume_from_yield:;

#line 530 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 530, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":531
 *     def __iter__(self):
 *         yield self.solution
 *         yield self             # <<<<<<<<<<<<<<
 *         yield self.fit
 * 
 */

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_self));

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 531 "ms_deisotope/_c/peak_set.pyx"
  /* return from generator, yielding value */

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = 2;

#line 531 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 531 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_resume_from_yield:;

#line 531 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 531, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":532
 *         yield self.solution
 *         yield self
 *         yield self.fit             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.fit));

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_self->__pyx_base.fit);

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 532 "ms_deisotope/_c/peak_set.pyx"
  /* return from generator, yielding value */

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = 3;

#line 532 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 532 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6_resume_from_yield:;

#line 532 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 532, __pyx_L1_error)

#line 532 "ms_deisotope/_c/peak_set.pyx"
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "ms_deisotope/_c/peak_set.pyx":529
 *             self.envelope, self.mz, self.chosen_for_msms, self.area)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.solution
 *         yield self
 */

#line 529 "ms_deisotope/_c/peak_set.pyx"


#line 529 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 529 "ms_deisotope/_c/peak_set.pyx"
  PyErr_SetNone(PyExc_StopIteration);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  #if !CYTHON_USE_EXC_INFO_STACK

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = -1;

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);

#line 529 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 529 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 529 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */

#line 535 "ms_deisotope/_c/peak_set.pyx"


#line 535 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 535 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 535 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak[] = "DeconvolutedPeakSolution.from_peak(type cls, peak, solution)";

#line 535 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peak = 0
#line 535 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_solution = 0
#line 535 "ms_deisotope/_c/peak_set.pyx"
;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("from_peak (wrapper)", 0);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  {

#line 535 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peak,&__pyx_n_s_solution,0};

#line 535 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 535 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 535 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 535 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 535 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 535 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 535 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 535 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 535 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 535 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 535 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 535 "ms_deisotope/_c/peak_set.pyx"
      }

#line 535 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 535 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 535 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 535 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peak)) != 0)) kw_args--;
        else 
#line 535 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 535 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 535 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 535 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_solution)) != 0)) kw_args--;

#line 535 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("from_peak", 1, 2, 2, 1); 
#line 535 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 535, __pyx_L3_error)

#line 535 "ms_deisotope/_c/peak_set.pyx"
        }

#line 535 "ms_deisotope/_c/peak_set.pyx"
      }

#line 535 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 535 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "from_peak") < 0)) __PYX_ERR(0, 535, __pyx_L3_error)

#line 535 "ms_deisotope/_c/peak_set.pyx"
      }

#line 535 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {

#line 535 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 535 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 535 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 535 "ms_deisotope/_c/peak_set.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 535 "ms_deisotope/_c/peak_set.pyx"
    }

#line 535 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peak = values[0];

#line 535 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_solution = values[1];

#line 535 "ms_deisotope/_c/peak_set.pyx"
  }

#line 535 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_peak", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 535 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 535, __pyx_L3_error)

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.from_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 535 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 535 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(((PyTypeObject*)__pyx_v_cls), __pyx_v_peak, __pyx_v_solution);

#line 535 "ms_deisotope/_c/peak_set.pyx"


#line 535 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 535 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 535 "ms_deisotope/_c/peak_set.pyx"
}

#line 535 "ms_deisotope/_c/peak_set.pyx"


#line 535 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_peak, PyObject *__pyx_v_solution) {
  PyObject *__pyx_v_inst = NULL
#line 535 "ms_deisotope/_c/peak_set.pyx"
;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_15 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_16 = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("from_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":537
 *     def from_peak(cls, peak, solution):
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,             # <<<<<<<<<<<<<<
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 */

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_fit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_intensity); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 537, __pyx_L1_error)

#line 537 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

  /* "ms_deisotope/_c/peak_set.pyx":538
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,             # <<<<<<<<<<<<<<
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 */

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_charge); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 538, __pyx_L1_error)

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_signal_to_noise); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 538, __pyx_L1_error)

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 538, __pyx_L1_error)

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_full_width_at_half_max); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 538, __pyx_L1_error)

#line 538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

  /* "ms_deisotope/_c/peak_set.pyx":539
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,             # <<<<<<<<<<<<<<
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 *         return inst
 */

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_a_to_a2_ratio); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 539, __pyx_L1_error)

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_most_abundant_mass); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 539, __pyx_L1_error)

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_average_mass); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)

#line 539 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

  /* "ms_deisotope/_c/peak_set.pyx":540
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_score); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 540, __pyx_L1_error)

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_envelope); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 540, __pyx_L1_error)

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_mz); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 540, __pyx_L1_error)

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_chosen_for_msms); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 540, __pyx_L1_error)

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_area); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 540, __pyx_L1_error)

#line 540 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

  /* "ms_deisotope/_c/peak_set.pyx":536
 *     @classmethod
 *     def from_peak(cls, peak, solution):
 *         inst = cls(             # <<<<<<<<<<<<<<
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 *             peak.charge, peak.signal_to_noise, peak.index, peak.full_width_at_half_max,
 */

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_16 = PyTuple_New(16); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 536, __pyx_L1_error)

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_16);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_solution);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_solution);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_v_solution);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_1);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_t_2);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 3, __pyx_t_3);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 4, __pyx_t_4);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 5, __pyx_t_5);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 6, __pyx_t_6);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 7, __pyx_t_7);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 8, __pyx_t_8);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 9, __pyx_t_9);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 10, __pyx_t_10);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 11, __pyx_t_11);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 12, __pyx_t_12);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 13, __pyx_t_13);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_14);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 14, __pyx_t_14);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_15);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_16, 15, __pyx_t_15);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_16, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 536, __pyx_L1_error)

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_15);

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_inst = __pyx_t_15;

#line 536 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_15 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":541
 *             peak.a_to_a2_ratio, peak.most_abundant_mass, peak.average_mass,
 *             peak.score, peak.envelope, peak.mz, peak.chosen_for_msms, peak.area)
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 541 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 541 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_inst);

#line 541 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_inst;

#line 541 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */

#line 535 "ms_deisotope/_c/peak_set.pyx"


#line 535 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_15);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_16);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSolution.from_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_inst);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 535 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 535 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":73
 * cdef class DeconvolutedPeakSolution(DeconvolutedPeak):
 *     cdef:
 *         public object solution             # <<<<<<<<<<<<<<
 * 
 * cdef class DeconvolutedPeakSet(object):
 */

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 73 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 73 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(PyObject *__pyx_v_self) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 73 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_v_self->solution);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_v_self->solution;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 73 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self, PyObject *__pyx_v_value) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_v_value);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_v_value);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->solution);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->solution);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->solution = __pyx_v_value;

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(PyObject *__pyx_v_self) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 73 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)__pyx_v_self));

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *__pyx_v_self) {

#line 73 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->solution);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->solution);

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->solution = Py_None;

#line 73 "ms_deisotope/_c/peak_set.pxd"


#line 73 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 73 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 73 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 73 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":555
 *         Collection of peaks ordered by `mz`
 *     """
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 */

#line 555 "ms_deisotope/_c/peak_set.pyx"


#line 555 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 555 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 555 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peaks = 0
#line 555 "ms_deisotope/_c/peak_set.pyx"
;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 555 "ms_deisotope/_c/peak_set.pyx"
  {

#line 555 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peaks,0};

#line 555 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[1] = {0};

#line 555 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 555 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 555 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 555 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  1: 
#line 555 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 555 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 555 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 555 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 555 "ms_deisotope/_c/peak_set.pyx"
      }

#line 555 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 555 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 555 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 555 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peaks)) != 0)) kw_args--;
        else 
#line 555 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 555 "ms_deisotope/_c/peak_set.pyx"
      }

#line 555 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 555 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 555, __pyx_L3_error)

#line 555 "ms_deisotope/_c/peak_set.pyx"
      }

#line 555 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 555 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 555 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 555 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 555 "ms_deisotope/_c/peak_set.pyx"
    }

#line 555 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peaks = values[0];

#line 555 "ms_deisotope/_c/peak_set.pyx"
  }

#line 555 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 555 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 555, __pyx_L3_error)

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 555 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 555 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_peaks);

#line 555 "ms_deisotope/_c/peak_set.pyx"


#line 555 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 555 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 555 "ms_deisotope/_c/peak_set.pyx"
}

#line 555 "ms_deisotope/_c/peak_set.pyx"


#line 555 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_peaks) {

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 555 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":556
 *     """
 *     def __init__(self, peaks):
 *         self.peaks = tuple(peaks)             # <<<<<<<<<<<<<<
 *         self._mz_ordered = None
 *         self.indexed = False
 */

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PySequence_Tuple(__pyx_v_peaks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->peaks);

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->peaks);

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_1);

#line 556 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":557
 *     def __init__(self, peaks):
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None             # <<<<<<<<<<<<<<
 *         self.indexed = False
 * 
 */

#line 557 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(Py_None);

#line 557 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(Py_None);

#line 557 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);

#line 557 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);

#line 557 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->_mz_ordered = ((PyObject*)Py_None);

  /* "ms_deisotope/_c/peak_set.pyx":558
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 *         self.indexed = False             # <<<<<<<<<<<<<<
 * 
 *     cpdef reindex(self):
 */

#line 558 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->indexed = 0;

  /* "ms_deisotope/_c/peak_set.pyx":555
 *         Collection of peaks ordered by `mz`
 *     """
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.peaks = tuple(peaks)
 *         self._mz_ordered = None
 */

#line 555 "ms_deisotope/_c/peak_set.pyx"


#line 555 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 555 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 555 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 555 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":560
 *         self.indexed = False
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         """
 *         Updates the :attr:`index` of each peak in `self` and updates the
 */

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 560 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_i
#line 560 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 560 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 560 "ms_deisotope/_c/peak_set.pyx"
;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_7;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex", 0);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 560 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 560 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 560 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 560 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 560 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 560 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_reindex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)

#line 560 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 560 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex)) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 560 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 560 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 560 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 560 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 560 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 560 "ms_deisotope/_c/peak_set.pyx"
          }

#line 560 "ms_deisotope/_c/peak_set.pyx"
        }

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_2;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
      }

#line 560 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 560 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 560 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 560 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 560 "ms_deisotope/_c/peak_set.pyx"
      }

#line 560 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 560 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 560 "ms_deisotope/_c/peak_set.pyx"
    }

#line 560 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 560 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":572
 *             size_t i, n
 *             DeconvolutedPeak peak
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))             # <<<<<<<<<<<<<<
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)
 */

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 572, __pyx_L1_error)

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_self->peaks);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_self->peaks);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->peaks);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 572, __pyx_L1_error)

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter) < 0) __PYX_ERR(0, 572, __pyx_L1_error)

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 572, __pyx_L1_error)

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 572, __pyx_L1_error)

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->peaks);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->peaks);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_2);

#line 572 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":573
 *             DeconvolutedPeak peak
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))             # <<<<<<<<<<<<<<
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0
 */

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_self->peaks);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_self->peaks);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->peaks);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_key, __pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter) < 0) __PYX_ERR(0, 573, __pyx_L1_error)

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->_mz_ordered = ((PyObject*)__pyx_t_3);

#line 573 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":574
 *         self.peaks = tuple(sorted(self.peaks, key=neutral_mass_getter))
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(n):
 */

#line 574 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_v_self->peaks;

#line 574 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_3);

#line 574 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_t_3);

#line 574 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":575
 *         self._mz_ordered = tuple(sorted(self.peaks, key=mz_getter))
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = self.getitem(i)
 */

#line 575 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":576
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)
 */

#line 576 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_v_n;

#line 576 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_t_5;

#line 576 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {

#line 576 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":577
 *         i = 0
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i
 */

#line 577 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)

#line 577 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 577 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));

#line 577 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":578
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)             # <<<<<<<<<<<<<<
 *             peak._index.neutral_mass = i
 *         i = 0
 */

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create(0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 578, __pyx_L1_error)

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_3);

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_v_peak->_index);

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(((PyObject *)__pyx_v_peak->_index));

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peak->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)__pyx_t_3);

#line 578 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":579
 *             peak = self.getitem(i)
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(n):
 */

#line 579 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peak->_index->neutral_mass = __pyx_v_i;

#line 579 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":580
 *             peak._index = _Index._create(0, 0)
 *             peak._index.neutral_mass = i
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 */

#line 580 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":581
 *             peak._index.neutral_mass = i
 *         i = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i
 */

#line 581 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_v_n;

#line 581 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_t_5;

#line 581 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {

#line 581 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":582
 *         i = 0
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)             # <<<<<<<<<<<<<<
 *             peak._index.mz = i
 *         self.indexed = True
 */

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_self->_mz_ordered;

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_v_i);

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_t_8);

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));

#line 582 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":583
 *         for i in range(n):
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i             # <<<<<<<<<<<<<<
 *         self.indexed = True
 *         return self
 */

#line 583 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peak->_index->mz = __pyx_v_i;

#line 583 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":584
 *             peak = <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *             peak._index.mz = i
 *         self.indexed = True             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

#line 584 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->indexed = 1;

  /* "ms_deisotope/_c/peak_set.pyx":585
 *             peak._index.mz = i
 *         self.indexed = True
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */

#line 585 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 585 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 585 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_v_self);

#line 585 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":560
 *         self.indexed = False
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         """
 *         Updates the :attr:`index` of each peak in `self` and updates the
 */

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 560 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 560 "ms_deisotope/_c/peak_set.pyx"
}

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 560 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 560 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex[] = "DeconvolutedPeakSet.reindex(self)\n\n        Updates the :attr:`index` of each peak in `self` and updates the\n        sorted order.\n\n        Returns\n        -------\n        self: DeconvolutedPeakSet\n        ";

#line 560 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex (wrapper)", 0);
  __pyx_r = 
#line 560 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 560 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 560 "ms_deisotope/_c/peak_set.pyx"
}

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 560 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex", 0);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 560 "ms_deisotope/_c/peak_set.pyx"


#line 560 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 560 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 560 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 560 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":587
 *         return self
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

#line 587 "ms_deisotope/_c/peak_set.pyx"


#line 587 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 587 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5__iter__(PyObject *__pyx_v_self); /*proto*/

#line 587 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5__iter__(PyObject *__pyx_v_self) {

#line 587 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = 
#line 587 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 587 "ms_deisotope/_c/peak_set.pyx"


#line 587 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 587 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 587 "ms_deisotope/_c/peak_set.pyx"
}

#line 587 "ms_deisotope/_c/peak_set.pyx"


#line 587 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_4__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 587 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 587 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":588
 * 
 *     def __iter__(self):
 *         return iter(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 588, __pyx_L1_error)

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 588 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 588 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":587
 *         return self
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

#line 587 "ms_deisotope/_c/peak_set.pyx"


#line 587 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 587 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 587 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 587 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":590
 *         return iter(self.peaks)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
 */

#line 590 "ms_deisotope/_c/peak_set.pyx"


#line 590 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 590 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__(PyObject *__pyx_v_self); /*proto*/

#line 590 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__(PyObject *__pyx_v_self) {

#line 590 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_r;

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 590 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 590 "ms_deisotope/_c/peak_set.pyx"


#line 590 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 590 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 590 "ms_deisotope/_c/peak_set.pyx"
}

#line 590 "ms_deisotope/_c/peak_set.pyx"


#line 590 "ms_deisotope/_c/peak_set.pyx"
static Py_ssize_t __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_6__len__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 590 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_r;

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":591
 * 
 *     def __len__(self):
 *         return self.get_size()             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self):
 */

#line 591 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->get_size(__pyx_v_self);

#line 591 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":590
 *         return iter(self.peaks)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
 */

#line 590 "ms_deisotope/_c/peak_set.pyx"


#line 590 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 590 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 590 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 590 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":593
 *         return self.get_size()
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 */

#line 593 "ms_deisotope/_c/peak_set.pyx"


#line 593 "ms_deisotope/_c/peak_set.pyx"
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 593 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_r;

#line 593 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 593 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 593 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":594
 * 
 *     cdef size_t get_size(self):
 *         return PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 594 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 594 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 594 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = PyTuple_GET_SIZE(__pyx_t_1);

#line 594 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 594 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":593
 *         return self.get_size()
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 */

#line 593 "ms_deisotope/_c/peak_set.pyx"


#line 593 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 593 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 593 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 593 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 593 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":596
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 */

#line 596 "ms_deisotope/_c/peak_set.pyx"


#line 596 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 596 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__(PyObject *__pyx_v_self); /*proto*/

#line 596 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__(PyObject *__pyx_v_self) {

#line 596 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 596 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 596 "ms_deisotope/_c/peak_set.pyx"


#line 596 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 596 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 596 "ms_deisotope/_c/peak_set.pyx"
}

#line 596 "ms_deisotope/_c/peak_set.pyx"


#line 596 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_8__repr__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 596 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 596 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_1;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":597
 * 
 *     def __repr__(self):
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyObject_Length(((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 597, __pyx_L1_error)

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 597, __pyx_L1_error)

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_DeconvolutedPeakSet_d_Peaks, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_3;

#line 597 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 597 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":596
 *         return PyTuple_GET_SIZE(self.peaks)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 */

#line 596 "ms_deisotope/_c/peak_set.pyx"


#line 596 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 596 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 596 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 596 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

#line 599 "ms_deisotope/_c/peak_set.pyx"


#line 599 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 599 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/

#line 599 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 599 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_item));

#line 599 "ms_deisotope/_c/peak_set.pyx"


#line 599 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 599 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 599 "ms_deisotope/_c/peak_set.pyx"
}

#line 599 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */

#line 601 "ms_deisotope/_c/peak_set.pyx"


#line 601 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_cur_scope
#line 601 "ms_deisotope/_c/peak_set.pyx"
;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 601 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, __pyx_empty_tuple, NULL);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)Py_None);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *) __pyx_self;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_getitem___locals_genexpr, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!gen)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_cur_scope);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 601 "ms_deisotope/_c/peak_set.pyx"
    return (PyObject *) gen;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"


#line 601 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__getitem__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 601 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 601 "ms_deisotope/_c/peak_set.pyx"
}

#line 601 "ms_deisotope/_c/peak_set.pyx"


#line 601 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{

#line 601 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)__pyx_generator->closure);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_3;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *(*__pyx_t_4)(PyObject *);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("genexpr", 0);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  switch (__pyx_generator->resume_label) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    case 0: goto __pyx_L3_first_run;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    case 1: goto __pyx_L6_resume_from_yield;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    default: /* CPython raises the right error here */

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 601 "ms_deisotope/_c/peak_set.pyx"
    return NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_first_run:;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 601, __pyx_L1_error) }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->peaks == Py_None)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_item)) { __Pyx_RaiseClosureNameError("item"); __PYX_ERR(0, 601, __pyx_L1_error) }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->peaks, __pyx_cur_scope->__pyx_outer_scope->__pyx_v_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  } else {
    __pyx_t_3 = -1; 
#line 601 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  for (;;) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (likely(!__pyx_t_4)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
      if (likely(PyList_CheckExact(__pyx_t_2))) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #else

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 601 "ms_deisotope/_c/peak_set.pyx"
      } else {

#line 601 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #else

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 601 "ms_deisotope/_c/peak_set.pyx"
      }
    } else 
#line 601 "ms_deisotope/_c/peak_set.pyx"
{

#line 601 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);

#line 601 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(!__pyx_t_1)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
        PyObject* exc_type = PyErr_Occurred();

#line 601 "ms_deisotope/_c/peak_set.pyx"
        if (exc_type) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 601 "ms_deisotope/_c/peak_set.pyx"
          else __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
        }

#line 601 "ms_deisotope/_c/peak_set.pyx"
        break;

#line 601 "ms_deisotope/_c/peak_set.pyx"
      }

#line 601 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    }

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_p);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_p, __pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_p, __pyx_n_s_clone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_6)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_6);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_5, function);

#line 601 "ms_deisotope/_c/peak_set.pyx"
      }

#line 601 "ms_deisotope/_c/peak_set.pyx"
    }

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_4;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_r);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_RefNannyFinishContext();

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    /* return from generator, yielding value */

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_generator->resume_label = 1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    return __pyx_r;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_resume_from_yield:;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope->__pyx_t_0 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_2);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_2;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
  }

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

#line 601 "ms_deisotope/_c/peak_set.pyx"


#line 601 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 601 "ms_deisotope/_c/peak_set.pyx"
  PyErr_SetNone(PyExc_StopIteration);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  #if !CYTHON_USE_EXC_INFO_STACK

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  #endif

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __pyx_generator->resume_label = -1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);

#line 601 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 601 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 601 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

#line 599 "ms_deisotope/_c/peak_set.pyx"


#line 599 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_10__getitem__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_item) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_cur_scope
#line 599 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3 = 0
#line 599 "ms_deisotope/_c/peak_set.pyx"
;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_3;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getitem__", 0);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(__pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, __pyx_empty_tuple, NULL);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_cur_scope)) {

#line 599 "ms_deisotope/_c/peak_set.pyx"
    __pyx_cur_scope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)Py_None);

#line 599 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 599 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 599, __pyx_L1_error)

#line 599 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 599 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_cur_scope);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  }

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_cur_scope->__pyx_v_item = __pyx_v_item;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_item);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_item);

  /* "ms_deisotope/_c/peak_set.pyx":600
 * 
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]
 */

#line 600 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_item;

#line 600 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 600 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PySlice_Check(__pyx_t_1); 

#line 600 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 600 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 600 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":601
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))             # <<<<<<<<<<<<<<
 *         return self.peaks[item]
 * 
 */

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PySequence_Tuple(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = NULL;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 601 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_5)) {

#line 601 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_5);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 601 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 601 "ms_deisotope/_c/peak_set.pyx"
      }

#line 601 "ms_deisotope/_c/peak_set.pyx"
    }

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_1;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 601 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":600
 * 
 *     def __getitem__(self, item):
 *         if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]
 */

#line 600 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":602
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 *         return self.peaks[item]             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */

#line 602 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 602 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_cur_scope->__pyx_v_self->peaks == Py_None)) {

#line 602 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 602 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 602, __pyx_L1_error)

#line 602 "ms_deisotope/_c/peak_set.pyx"
  }

#line 602 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_self->peaks, __pyx_cur_scope->__pyx_v_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)

#line 602 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 602 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 602 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 602 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":599
 *         return "<DeconvolutedPeakSet %d Peaks>" % (len(self))
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         if isinstance(item, slice):
 *             return self.__class__(tuple(p.clone() for p in self.peaks[item]))
 */

#line 599 "ms_deisotope/_c/peak_set.pyx"


#line 599 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_gb_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem___2generator3);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 599 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 599 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 599 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":604
 *         return self.peaks[item]
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks == other.peaks
 */

#line 604 "ms_deisotope/_c/peak_set.pyx"


#line 604 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 604 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 604 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = 
#line 604 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_other));

#line 604 "ms_deisotope/_c/peak_set.pyx"


#line 604 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 604 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 604 "ms_deisotope/_c/peak_set.pyx"
}

#line 604 "ms_deisotope/_c/peak_set.pyx"


#line 604 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_12__eq__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other) {

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */

#line 605 "ms_deisotope/_c/peak_set.pyx"
  {

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_declare

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_assign

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_1);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_2);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    /*try:*/ {

      /* "ms_deisotope/_c/peak_set.pyx":606
 *     def __eq__(self, other):
 *         try:
 *             return self.peaks == other.peaks             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return tuple(self) == tuple(other)
 */

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(__pyx_r);

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_peaks); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 606, __pyx_L3_error)

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_self->peaks, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 606, __pyx_L3_error)

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_t_5;

#line 606 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = 0;

#line 606 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7_try_return;

      /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */

#line 605 "ms_deisotope/_c/peak_set.pyx"
    }

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L3_error:;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":607
 *         try:
 *             return self.peaks == other.peaks
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return tuple(self) == tuple(other)
 * 
 */

#line 607 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);

#line 607 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_6) {

#line 607 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 607 "ms_deisotope/_c/peak_set.pyx"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 607, __pyx_L5_except_error)

#line 607 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 607 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 607 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_7);

      /* "ms_deisotope/_c/peak_set.pyx":608
 *             return self.peaks == other.peaks
 *         except AttributeError:
 *             return tuple(self) == tuple(other)             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(__pyx_r);

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_8 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 608, __pyx_L5_except_error)

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_8);

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_9 = __Pyx_PySequence_Tuple(__pyx_v_other); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 608, __pyx_L5_except_error)

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_9);

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_8, __pyx_t_9, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 608, __pyx_L5_except_error)

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_t_10;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_10 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 608 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6_except_return;

#line 608 "ms_deisotope/_c/peak_set.pyx"
    }

#line 608 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L5_except_error;

#line 608 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5_except_error:;

    /* "ms_deisotope/_c/peak_set.pyx":605
 * 
 *     def __eq__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks == other.peaks
 *         except AttributeError:
 */

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L1_error;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L7_try_return:;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_except_return:;

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 605 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 605 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":604
 *         return self.peaks[item]
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks == other.peaks
 */

#line 604 "ms_deisotope/_c/peak_set.pyx"


#line 604 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 604 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 604 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 604 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":610
 *             return tuple(self) == tuple(other)
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks != other.peaks
 */

#line 610 "ms_deisotope/_c/peak_set.pyx"


#line 610 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 610 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/

#line 610 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = 
#line 610 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_other));

#line 610 "ms_deisotope/_c/peak_set.pyx"


#line 610 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 610 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 610 "ms_deisotope/_c/peak_set.pyx"
}

#line 610 "ms_deisotope/_c/peak_set.pyx"


#line 610 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_14__ne__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_other) {

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_6;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */

#line 611 "ms_deisotope/_c/peak_set.pyx"
  {

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_declare

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_assign

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_1);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_2);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGOTREF(__pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    /*try:*/ {

      /* "ms_deisotope/_c/peak_set.pyx":612
 *     def __ne__(self, other):
 *         try:
 *             return self.peaks != other.peaks             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return tuple(self) != tuple(other)
 */

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(__pyx_r);

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_peaks); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 612, __pyx_L3_error)

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_self->peaks, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 612, __pyx_L3_error)

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_t_5;

#line 612 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = 0;

#line 612 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7_try_return;

      /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */

#line 611 "ms_deisotope/_c/peak_set.pyx"
    }

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L3_error:;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":613
 *         try:
 *             return self.peaks != other.peaks
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return tuple(self) != tuple(other)
 * 
 */

#line 613 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);

#line 613 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_6) {

#line 613 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 613 "ms_deisotope/_c/peak_set.pyx"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 613, __pyx_L5_except_error)

#line 613 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 613 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 613 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_7);

      /* "ms_deisotope/_c/peak_set.pyx":614
 *             return self.peaks != other.peaks
 *         except AttributeError:
 *             return tuple(self) != tuple(other)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(__pyx_r);

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_8 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 614, __pyx_L5_except_error)

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_8);

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_9 = __Pyx_PySequence_Tuple(__pyx_v_other); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 614, __pyx_L5_except_error)

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_9);

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_8, __pyx_t_9, Py_NE); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 614, __pyx_L5_except_error)

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_t_10;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_10 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 614 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6_except_return;

#line 614 "ms_deisotope/_c/peak_set.pyx"
    }

#line 614 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L5_except_error;

#line 614 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5_except_error:;

    /* "ms_deisotope/_c/peak_set.pyx":611
 * 
 *     def __ne__(self, other):
 *         try:             # <<<<<<<<<<<<<<
 *             return self.peaks != other.peaks
 *         except AttributeError:
 */

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L1_error;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L7_try_return:;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_except_return:;

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 611 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 611 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":610
 *             return tuple(self) == tuple(other)
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         try:
 *             return self.peaks != other.peaks
 */

#line 610 "ms_deisotope/_c/peak_set.pyx"


#line 610 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 610 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 610 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 610 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":616
 *             return tuple(self) != tuple(other)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

#line 616 "ms_deisotope/_c/peak_set.pyx"


#line 616 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 616 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__(PyObject *__pyx_v_self); /*proto*/

#line 616 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__(PyObject *__pyx_v_self) {

#line 616 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = 
#line 616 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 616 "ms_deisotope/_c/peak_set.pyx"


#line 616 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 616 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 616 "ms_deisotope/_c/peak_set.pyx"
}

#line 616 "ms_deisotope/_c/peak_set.pyx"


#line 616 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_16__iter__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 616 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 616 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":617
 * 
 *     def __iter__(self):
 *         return iter(self.peaks)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeakSet clone(self):
 */

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L1_error)

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 617 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 617 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":616
 *             return tuple(self) != tuple(other)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.peaks)
 * 
 */

#line 616 "ms_deisotope/_c/peak_set.pyx"


#line 616 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 616 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 616 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 616 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":619
 *         return iter(self.peaks)
 * 
 *     cpdef DeconvolutedPeakSet clone(self):             # <<<<<<<<<<<<<<
 *         return self.copy()
 * 
 */

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 619 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_r = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 619 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 619 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 619 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 619 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 619 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 619 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)

#line 619 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 619 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone)) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 619 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 619 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 619 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 619 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 619 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 619 "ms_deisotope/_c/peak_set.pyx"
          }

#line 619 "ms_deisotope/_c/peak_set.pyx"
        }

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 619, __pyx_L1_error)

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_2);

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
      }

#line 619 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 619 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 619 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 619 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 619 "ms_deisotope/_c/peak_set.pyx"
      }

#line 619 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 619 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 619 "ms_deisotope/_c/peak_set.pyx"
    }

#line 619 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 619 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":620
 * 
 *     cpdef DeconvolutedPeakSet clone(self):
 *         return self.copy()             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeakSet copy(self):
 */

#line 620 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 620 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)

#line 620 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 620 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_1);

#line 620 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 620 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":619
 *         return iter(self.peaks)
 * 
 *     cpdef DeconvolutedPeakSet clone(self):             # <<<<<<<<<<<<<<
 *         return self.copy()
 * 
 */

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 619 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 619 "ms_deisotope/_c/peak_set.pyx"
}

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 619 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 619 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone[] = "DeconvolutedPeakSet.clone(self) -> DeconvolutedPeakSet";

#line 619 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = 
#line 619 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 619 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 619 "ms_deisotope/_c/peak_set.pyx"
}

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 619 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("clone", 0);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 619 "ms_deisotope/_c/peak_set.pyx"


#line 619 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 619 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 619 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 619 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":622
 *         return self.copy()
 * 
 *     cpdef DeconvolutedPeakSet copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate_peaks
 */

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 622 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_duplicate_peaks = 0
#line 622 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 622 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 622 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 622 "ms_deisotope/_c/peak_set.pyx"
;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_r = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_7;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_8;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("copy", 0);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 622 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 622 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 622 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 622 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 622 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 622 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)

#line 622 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 622 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy)) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 622 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 622 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 622 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 622 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 622 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 622 "ms_deisotope/_c/peak_set.pyx"
          }

#line 622 "ms_deisotope/_c/peak_set.pyx"
        }

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 622, __pyx_L1_error)

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 622, __pyx_L1_error)

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_2);

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
      }

#line 622 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 622 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 622 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 622 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 622 "ms_deisotope/_c/peak_set.pyx"
      }

#line 622 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 622 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 622 "ms_deisotope/_c/peak_set.pyx"
    }

#line 622 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 622 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":627
 *             DeconvolutedPeak peak
 *             size_t i, n
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):
 */

#line 627 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->get_size(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":628
 *             size_t i, n
 *         n = self.get_size()
 *         duplicate_peaks = PyTuple_New(n)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             peak = self.getitem(i)
 */

#line 628 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)

#line 628 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 628 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_duplicate_peaks = ((PyObject*)__pyx_t_1);

#line 628 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":629
 *         n = self.get_size()
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             peak = peak.clone()
 */

#line 629 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_v_n;

#line 629 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_t_5;

#line 629 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {

#line 629 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":630
 *         duplicate_peaks = PyTuple_New(n)
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             peak = peak.clone()
 *             Py_INCREF(peak)
 */

#line 630 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->getitem(__pyx_v_self, __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)

#line 630 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 630 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));

#line 630 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":631
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             peak = peak.clone()             # <<<<<<<<<<<<<<
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 */

#line 631 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_v_peak->__pyx_base.__pyx_vtab)->__pyx_base.clone(((struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *)__pyx_v_peak), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)

#line 631 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 631 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 631, __pyx_L1_error)

#line 631 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));

#line 631 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":632
 *             peak = self.getitem(i)
 *             peak = peak.clone()
 *             Py_INCREF(peak)             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 *         return self.__class__(duplicate_peaks)
 */

#line 632 "ms_deisotope/_c/peak_set.pyx"
    Py_INCREF(((PyObject *)__pyx_v_peak));

    /* "ms_deisotope/_c/peak_set.pyx":633
 *             peak = peak.clone()
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)             # <<<<<<<<<<<<<<
 *         return self.__class__(duplicate_peaks)
 * 
 */

#line 633 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = PyTuple_SetItem(__pyx_v_duplicate_peaks, __pyx_v_i, ((PyObject *)__pyx_v_peak)); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 633, __pyx_L1_error)

#line 633 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":634
 *             Py_INCREF(peak)
 *             PyTuple_SetItem(duplicate_peaks, i, peak)
 *         return self.__class__(duplicate_peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 634 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 634 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 634 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 634 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 634 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 634 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 634 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 634 "ms_deisotope/_c/peak_set.pyx"
    }

#line 634 "ms_deisotope/_c/peak_set.pyx"
  }

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_duplicate_peaks) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_duplicate_peaks);

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 634 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 634 "ms_deisotope/_c/peak_set.pyx"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet))))) __PYX_ERR(0, 634, __pyx_L1_error)

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_t_1);

#line 634 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 634 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":622
 *         return self.copy()
 * 
 *     cpdef DeconvolutedPeakSet copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple duplicate_peaks
 */

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_duplicate_peaks);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 622 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 622 "ms_deisotope/_c/peak_set.pyx"
}

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 622 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 622 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy[] = "DeconvolutedPeakSet.copy(self) -> DeconvolutedPeakSet";

#line 622 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = 
#line 622 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 622 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 622 "ms_deisotope/_c/peak_set.pyx"
}

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 622 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("copy", 0);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 622 "ms_deisotope/_c/peak_set.pyx"


#line 622 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 622 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 622 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 622 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":636
 *         return self.__class__(duplicate_peaks)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, )
 * 
 */

#line 636 "ms_deisotope/_c/peak_set.pyx"


#line 636 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 636 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 636 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__[] = "DeconvolutedPeakSet.__reduce__(self)";

#line 636 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 636 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 636 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 636 "ms_deisotope/_c/peak_set.pyx"


#line 636 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 636 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 636 "ms_deisotope/_c/peak_set.pyx"
}

#line 636 "ms_deisotope/_c/peak_set.pyx"


#line 636 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 636 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 636 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":637
 * 
 *     def __reduce__(self):
 *         return self.__class__, (self.peaks, )             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 637, __pyx_L1_error)

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_self->peaks);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_self->peaks);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->peaks);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_3;

#line 637 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 637 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":636
 *         return self.__class__(duplicate_peaks)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, )
 * 
 */

#line 636 "ms_deisotope/_c/peak_set.pyx"


#line 636 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 636 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 636 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 636 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":639
 *         return self.__class__, (self.peaks, )
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 639 "ms_deisotope/_c/peak_set.pyx"


#line 639 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args) {

#line 639 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_v_error_tolerance = ((double)1e-5);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_v_use_mz = ((int)0);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 639 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_has_peak", 0);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 639 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 639 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;

#line 639 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_optional_args->__pyx_n > 1) {

#line 639 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_use_mz = __pyx_optional_args->use_mz;

#line 639 "ms_deisotope/_c/peak_set.pyx"
      }

#line 639 "ms_deisotope/_c/peak_set.pyx"
    }

#line 639 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":658
 *             The found peak, or None if no peak is found
 *         '''
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 */

#line 658 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_v_use_mz != 0);

#line 658 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":659
 *         '''
 *         if use_mz:
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)             # <<<<<<<<<<<<<<
 *         else:
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 */

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_v_self->_mz_ordered;

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_2);

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(((PyObject*)__pyx_t_2), __pyx_v_neutral_mass, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 659, __pyx_L1_error)

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3);

#line 659 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

#line 659 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":658
 *             The found peak, or None if no peak is found
 *         '''
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 */

#line 658 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":661
 *             return binary_search_mz(self._mz_ordered, neutral_mass, error_tolerance)
 *         else:
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)             # <<<<<<<<<<<<<<
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */

#line 661 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_self->peaks;

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(((PyObject*)__pyx_t_3), __pyx_v_neutral_mass, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 661, __pyx_L1_error)

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);

#line 661 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = 0;

#line 661 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 661 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":639
 *         return self.__class__, (self.peaks, )
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 639 "ms_deisotope/_c/peak_set.pyx"


#line 639 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 639 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 639 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 639 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":663
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 663 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_v_error_tolerance = ((double)1e-5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_v_use_mz = ((int)0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_8;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_10;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak __pyx_t_11;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("has_peak", 0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_optional_args->__pyx_n > 1) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_use_mz = __pyx_optional_args->use_mz;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    }

#line 663 "ms_deisotope/_c/peak_set.pyx"
  }

#line 663 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 663 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 663 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 663 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 663 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 663 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_has_peak); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_3);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_error_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_use_mz); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_6 = __pyx_t_1; 
#line 663 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_7 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_8 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_7)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);

#line 663 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_7);

#line 663 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 663 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_6, function);

#line 663 "ms_deisotope/_c/peak_set.pyx"
            __pyx_t_8 = 1;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCALL

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (PyFunction_Check(__pyx_t_6)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 663 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 663 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCCALL

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 663 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 663 "ms_deisotope/_c/peak_set.pyx"
        {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_9);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_7) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          }

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_3);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_4);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_3 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_5 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 663 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 663 "ms_deisotope/_c/peak_set.pyx"
    }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 663 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":684
 *             The found peak, or None if no peak is found
 *         '''
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 */

#line 684 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = ((!(__pyx_v_self->indexed != 0)) != 0);

#line 684 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":685
 *         '''
 *         if not self.indexed:
 *             self.reindex()             # <<<<<<<<<<<<<<
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 */

#line 685 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->reindex(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)

#line 685 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 685 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":684
 *             The found peak, or None if no peak is found
 *         '''
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 */

#line 684 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":686
 *         if not self.indexed:
 *             self.reindex()
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):
 */

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11.__pyx_n = 2;

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11.error_tolerance = __pyx_v_error_tolerance;

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11.use_mz = __pyx_v_use_mz;

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->_has_peak(__pyx_v_self, __pyx_v_neutral_mass, &__pyx_t_11)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);

#line 686 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 686 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":663
 *             return binary_search_neutral_mass(self.peaks, neutral_mass, error_tolerance)
 * 
 *     cpdef DeconvolutedPeak has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 663 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 663 "ms_deisotope/_c/peak_set.pyx"
}

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 663 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 663 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak[] = "DeconvolutedPeakSet.has_peak(self, double neutral_mass, double error_tolerance=1e-5, bool use_mz=False) -> DeconvolutedPeak\nFind the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        If ``use_mz`` is True, instead of matching neutral masses, match peaks using m/z instead.\n\n        If :attr:`indexed` is not :const:`True`, then :meth:`reindex` will be called.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n        use_mz: bool\n            Whether to search using m/z instead of neutral mass\n\n        Returns\n        -------\n        DeconvolutedPeak\n            The found peak, or None if no peak is found\n        ";

#line 663 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass
#line 663 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_error_tolerance
#line 663 "ms_deisotope/_c/peak_set.pyx"
;
  int __pyx_v_use_mz
#line 663 "ms_deisotope/_c/peak_set.pyx"
;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("has_peak (wrapper)", 0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  {

#line 663 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_error_tolerance,&__pyx_n_s_use_mz,0};

#line 663 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[3] = {0,0,0};

#line 663 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 663 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  3: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 663 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 663 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_error_tolerance);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 663 "ms_deisotope/_c/peak_set.pyx"
        case  2:

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_mz);

#line 663 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 663 "ms_deisotope/_c/peak_set.pyx"
        }

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "has_peak") < 0)) __PYX_ERR(0, 663, __pyx_L3_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 663 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 663 "ms_deisotope/_c/peak_set.pyx"
      }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
    if (values[1]) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_error_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_error_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_error_tolerance = ((double)1e-5);

#line 663 "ms_deisotope/_c/peak_set.pyx"
    }

#line 663 "ms_deisotope/_c/peak_set.pyx"
    if (values[2]) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_use_mz = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_use_mz == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 663, __pyx_L3_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 663 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_use_mz = ((int)0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
    }

#line 663 "ms_deisotope/_c/peak_set.pyx"
  }

#line 663 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("has_peak", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 663 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 663, __pyx_L3_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 663 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 663 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_use_mz);

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 663 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 663 "ms_deisotope/_c/peak_set.pyx"
}

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance, int __pyx_v_use_mz) {

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 663 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak __pyx_t_2;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("has_peak", 0);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.__pyx_n = 2;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.error_tolerance = __pyx_v_error_tolerance;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.use_mz = __pyx_v_use_mz;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet->has_peak(__pyx_v_self, __pyx_v_neutral_mass, 1, &__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 663, __pyx_L1_error)

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 663 "ms_deisotope/_c/peak_set.pyx"


#line 663 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 663 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 663 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 663 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":688
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 */

#line 688 "ms_deisotope/_c/peak_set.pyx"


#line 688 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_i) {

#line 688 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 688 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 688 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 688 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2;

#line 688 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("getitem", 0);

  /* "ms_deisotope/_c/peak_set.pyx":689
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)             # <<<<<<<<<<<<<<
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):
 */

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_v_i);

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)));

#line 689 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);

#line 689 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":688
 *         return self._has_peak(neutral_mass, error_tolerance, use_mz)
 * 
 *     cdef DeconvolutedPeak getitem(self, size_t i):             # <<<<<<<<<<<<<<
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 */

#line 688 "ms_deisotope/_c/peak_set.pyx"


#line 688 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 688 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 688 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 688 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 688 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 688 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":691
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */

#line 691 "ms_deisotope/_c/peak_set.pyx"


#line 691 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, size_t __pyx_v_start, size_t __pyx_v_end) {

#line 691 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 691 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("getslice", 0);

  /* "ms_deisotope/_c/peak_set.pyx":692
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):
 */

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 692, __pyx_L1_error)

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject*)__pyx_t_2));

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject*)__pyx_t_2);

#line 692 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 692 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":691
 *         return <DeconvolutedPeak>PyTuple_GET_ITEM(self.peaks, i)
 * 
 *     cdef tuple getslice(self, size_t start, size_t end):             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */

#line 691 "ms_deisotope/_c/peak_set.pyx"


#line 691 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.getslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 691 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 691 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 691 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":694
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 694 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_v_tolerance = ((double)1e-5);
  double __pyx_v_lo
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_hi
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_lo_err
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_hi_err
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  int __pyx_v_lo_ix
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  int __pyx_v_hi_ix
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_lo_peak = 0
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_hi_peak = 0
#line 694 "ms_deisotope/_c/peak_set.pyx"
;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_7;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_9;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_10;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_11;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for", 0);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = __pyx_optional_args->tolerance;

#line 694 "ms_deisotope/_c/peak_set.pyx"
    }

#line 694 "ms_deisotope/_c/peak_set.pyx"
  }

#line 694 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 694 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 694 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 694 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 694 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 694 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_all_peaks_for); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_3);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; 
#line 694 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_6 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_7 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_6)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 694 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_6);

#line 694 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 694 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_5, function);

#line 694 "ms_deisotope/_c/peak_set.pyx"
            __pyx_t_7 = 1;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          }

#line 694 "ms_deisotope/_c/peak_set.pyx"
        }

#line 694 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCALL

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (PyFunction_Check(__pyx_t_5)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 694 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 694 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCCALL

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 694 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 694 "ms_deisotope/_c/peak_set.pyx"
        {

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_8);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_6) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          }

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_3);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_4);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_3 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        }

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((PyObject*)__pyx_t_2);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 694 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 694 "ms_deisotope/_c/peak_set.pyx"
    }

#line 694 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 694 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":714
 *             DeconvolutedPeak lo_peak
 *             DeconvolutedPeak hi_peak
 *         lo = neutral_mass - neutral_mass * tolerance             # <<<<<<<<<<<<<<
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 */

#line 714 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = (__pyx_v_neutral_mass - (__pyx_v_neutral_mass * __pyx_v_tolerance));

  /* "ms_deisotope/_c/peak_set.pyx":715
 *             DeconvolutedPeak hi_peak
 *         lo = neutral_mass - neutral_mass * tolerance
 *         hi = neutral_mass + neutral_mass * tolerance             # <<<<<<<<<<<<<<
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 */

#line 715 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = (__pyx_v_neutral_mass + (__pyx_v_neutral_mass * __pyx_v_tolerance));

  /* "ms_deisotope/_c/peak_set.pyx":716
 *         lo = neutral_mass - neutral_mass * tolerance
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)             # <<<<<<<<<<<<<<
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 */

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_1), __pyx_v_lo, (&__pyx_v_lo_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);

#line 716 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":717
 *         hi = neutral_mass + neutral_mass * tolerance
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)             # <<<<<<<<<<<<<<
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 */

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_v_self->peaks;

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_2);

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_2), __pyx_v_hi, (&__pyx_v_hi_err))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);

#line 717 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":718
 *         lo_peak = binary_search_nearest_neutral_mass(self.peaks, lo, &lo_err)
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 */

#line 718 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = __pyx_v_lo_peak->_index->neutral_mass;

#line 718 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo_ix = __pyx_t_9;

  /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */

#line 719 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 719 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 719 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = ((__pyx_v_lo_ix < PyTuple_GET_SIZE(__pyx_t_1)) != 0);

#line 719 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 719 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_11) {

#line 719 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 719 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_10 = __pyx_t_11;

#line 719 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L4_bool_binop_done;

#line 719 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":720
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:             # <<<<<<<<<<<<<<
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 */

#line 720 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_lo_peak->neutral_mass, __pyx_v_neutral_mass)) > __pyx_v_tolerance) != 0);

#line 720 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = __pyx_t_11;

#line 720 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_bool_binop_done:;

  /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */

#line 719 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":721
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1             # <<<<<<<<<<<<<<
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(
 */

#line 721 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_lo_ix = (__pyx_v_lo_ix + 1);

    /* "ms_deisotope/_c/peak_set.pyx":719
 *         hi_peak = binary_search_nearest_neutral_mass(self.peaks, hi, &hi_err)
 *         lo_ix = lo_peak._index.neutral_mass
 *         if lo_ix < PyTuple_GET_SIZE(self.peaks) and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 */

#line 719 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":722
 *                 ppm_error(lo_peak.neutral_mass, neutral_mass)) > tolerance:
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1             # <<<<<<<<<<<<<<
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 */

#line 722 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi_ix = (__pyx_v_hi_peak->_index->neutral_mass + 1);

  /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */

#line 723 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = ((__pyx_v_hi_ix != 0) != 0);

#line 723 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_11) {

#line 723 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 723 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_10 = __pyx_t_11;

#line 723 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L7_bool_binop_done;

#line 723 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":724
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:             # <<<<<<<<<<<<<<
 *             hi_ix -= 1
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 */

#line 724 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_hi_peak->neutral_mass, __pyx_v_neutral_mass)) > __pyx_v_tolerance) != 0);

#line 724 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = __pyx_t_11;

#line 724 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L7_bool_binop_done:;

  /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */

#line 723 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":725
 *         if hi_ix != 0 and abs(
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 */

#line 725 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_hi_ix = (__pyx_v_hi_ix - 1);

    /* "ms_deisotope/_c/peak_set.pyx":723
 *             lo_ix += 1
 *         hi_ix = hi_peak._index.neutral_mass + 1
 *         if hi_ix != 0 and abs(             # <<<<<<<<<<<<<<
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 */

#line 723 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":726
 *                 ppm_error(hi_peak.neutral_mass, neutral_mass)) > tolerance:
 *             hi_ix -= 1
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):
 */

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_lo_ix, __pyx_v_hi_ix); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject*)__pyx_t_2));

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject*)__pyx_t_2);

#line 726 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 726 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":694
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_lo_peak);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_hi_peak);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 694 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 694 "ms_deisotope/_c/peak_set.pyx"
}

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 694 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 694 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for[] = "DeconvolutedPeakSet.all_peaks_for(self, double neutral_mass, double tolerance=1e-5) -> tuple\nFind all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n\n        Returns\n        -------\n        tuple of DeconvolutedPeak\n            The found peaks\n        ";

#line 694 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass
#line 694 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_tolerance
#line 694 "ms_deisotope/_c/peak_set.pyx"
;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for (wrapper)", 0);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  {

#line 694 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};

#line 694 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 694 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 694 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 694 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 694 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 694 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);

#line 694 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 694 "ms_deisotope/_c/peak_set.pyx"
        }

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "all_peaks_for") < 0)) __PYX_ERR(0, 694, __pyx_L3_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 694 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 694 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 694 "ms_deisotope/_c/peak_set.pyx"
      }

#line 694 "ms_deisotope/_c/peak_set.pyx"
    }

#line 694 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 694, __pyx_L3_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
    if (values[1]) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 694, __pyx_L3_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 694 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = ((double)1e-5);

#line 694 "ms_deisotope/_c/peak_set.pyx"
    }

#line 694 "ms_deisotope/_c/peak_set.pyx"
  }

#line 694 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("all_peaks_for", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 694 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 694, __pyx_L3_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 694 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 694 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 694 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 694 "ms_deisotope/_c/peak_set.pyx"
}

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 694 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for __pyx_t_2;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for", 0);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.tolerance = __pyx_v_tolerance;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet->all_peaks_for(__pyx_v_self, __pyx_v_neutral_mass, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 694 "ms_deisotope/_c/peak_set.pyx"


#line 694 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 694 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 694 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 694 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":728
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

#line 728 "ms_deisotope/_c/peak_set.pyx"


#line 728 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass, double *__pyx_v_errout) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 728 "ms_deisotope/_c/peak_set.pyx"
;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 728 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_get_nearest_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":744
 *         '''
 *         cdef DeconvolutedPeak peak
 *         peak = binary_search_nearest_neutral_mass(self.peaks, neutral_mass, errout)             # <<<<<<<<<<<<<<
 *         return peak
 * 
 */

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->peaks;

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(((PyObject*)__pyx_t_1), __pyx_v_neutral_mass, __pyx_v_errout)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2);

#line 744 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":745
 *         cdef DeconvolutedPeak peak
 *         peak = binary_search_nearest_neutral_mass(self.peaks, neutral_mass, errout)
 *         return peak             # <<<<<<<<<<<<<<
 * 
 *     def get_nearest_peak(self, double neutral_mass):
 */

#line 745 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 745 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_peak));

#line 745 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_peak;

#line 745 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":728
 *         return <tuple>PyTuple_GetSlice(self.peaks, lo_ix, hi_ix)
 * 
 *     cdef DeconvolutedPeak _get_nearest_peak(self, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

#line 728 "ms_deisotope/_c/peak_set.pyx"


#line 728 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 728 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 728 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 728 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":747
 *         return peak
 * 
 *     def get_nearest_peak(self, double neutral_mass):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

#line 747 "ms_deisotope/_c/peak_set.pyx"


#line 747 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 747 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak(PyObject *__pyx_v_self, PyObject *__pyx_arg_neutral_mass); /*proto*/

#line 747 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak[] = "DeconvolutedPeakSet.get_nearest_peak(self, double neutral_mass)\nFind the peak nearest to ``neutral_mass``, regardless of error.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n\n        Returns\n        -------\n        DeconvolutedPeak\n            The nearest peak\n        double\n            The error between ``neutral_mass`` and the found peak\n        ";

#line 747 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak(PyObject *__pyx_v_self, PyObject *__pyx_arg_neutral_mass) {
  double __pyx_v_neutral_mass
#line 747 "ms_deisotope/_c/peak_set.pyx"
;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("get_nearest_peak (wrapper)", 0);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  assert(__pyx_arg_neutral_mass); {

#line 747 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(__pyx_arg_neutral_mass); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 747, __pyx_L3_error)

#line 747 "ms_deisotope/_c/peak_set.pyx"
  }

#line 747 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 747 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 747 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((double)__pyx_v_neutral_mass));

#line 747 "ms_deisotope/_c/peak_set.pyx"


#line 747 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 747 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 747 "ms_deisotope/_c/peak_set.pyx"
}

#line 747 "ms_deisotope/_c/peak_set.pyx"


#line 747 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, double __pyx_v_neutral_mass) {
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 747 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_errout
#line 747 "ms_deisotope/_c/peak_set.pyx"
;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 747 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("get_nearest_peak", 0);

  /* "ms_deisotope/_c/peak_set.pyx":765
 *             DeconvolutedPeak peak
 *             double errout
 *         peak = self._get_nearest_peak(neutral_mass, &errout)             # <<<<<<<<<<<<<<
 *         return peak, errout
 * 
 */

#line 765 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self->__pyx_vtab)->_get_nearest_peak(__pyx_v_self, __pyx_v_neutral_mass, (&__pyx_v_errout))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)

#line 765 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 765 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1);

#line 765 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":766
 *             double errout
 *         peak = self._get_nearest_peak(neutral_mass, &errout)
 *         return peak, errout             # <<<<<<<<<<<<<<
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):
 */

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_errout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 766, __pyx_L1_error)

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_peak));

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_peak));

#line 766 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_peak));

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 766 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_2;

#line 766 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 766 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":747
 *         return peak
 * 
 *     def get_nearest_peak(self, double neutral_mass):             # <<<<<<<<<<<<<<
 *         '''Find the peak nearest to ``neutral_mass``, regardless of error.
 * 
 */

#line 747 "ms_deisotope/_c/peak_set.pyx"


#line 747 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.get_nearest_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 747 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 747 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 747 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":768
 *         return peak, errout
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):             # <<<<<<<<<<<<<<
 *         """Retrieve a :class:`DeconvolutedPeakSet` containing all the peaks
 *         whose mass is between ``m1`` and ``m2``.
 */

#line 768 "ms_deisotope/_c/peak_set.pyx"


#line 768 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 768 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 768 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between[] = "DeconvolutedPeakSet.between(self, m1, m2, tolerance=1e-5, use_mz=False)\nRetrieve a :class:`DeconvolutedPeakSet` containing all the peaks\n        whose mass is between ``m1`` and ``m2``.\n\n        If ``use_mz`` is :const:`True` then search by m/z instead of mass\n\n        Parameters\n        ----------\n        m1 : float\n            The lower mass limit\n        m2 : float\n            The upper mass limit\n        use_mz: bool\n            Whether to search for m/z instead of neutral mass\n\n        Returns\n        -------\n        DeconvolutedPeakSet\n        ";

#line 768 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_m1 = 0
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_m2 = 0
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  CYTHON_UNUSED PyObject *__pyx_v_tolerance = 0
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_use_mz = 0
#line 768 "ms_deisotope/_c/peak_set.pyx"
;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("between (wrapper)", 0);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  {

#line 768 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_m1,&__pyx_n_s_m2,&__pyx_n_s_tolerance,&__pyx_n_s_use_mz,0};

#line 768 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[4] = {0,0,0,0};

#line 768 "ms_deisotope/_c/peak_set.pyx"
    values[2] = ((PyObject *)__pyx_float_1eneg_5);

#line 768 "ms_deisotope/_c/peak_set.pyx"
    values[3] = ((PyObject *)Py_False);

#line 768 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 768 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 768 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 768 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  4: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 768 "ms_deisotope/_c/peak_set.pyx"
      }

#line 768 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 768 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 768 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 768 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_m1)) != 0)) kw_args--;
        else 
#line 768 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 768 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_m2)) != 0)) kw_args--;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("between", 0, 2, 4, 1); 
#line 768 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 768, __pyx_L3_error)

#line 768 "ms_deisotope/_c/peak_set.pyx"
        }

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        case  2:

#line 768 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 768 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);

#line 768 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[2] = value; kw_args--; }

#line 768 "ms_deisotope/_c/peak_set.pyx"
        }

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 768 "ms_deisotope/_c/peak_set.pyx"
        case  3:

#line 768 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 768 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_mz);

#line 768 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[3] = value; kw_args--; }

#line 768 "ms_deisotope/_c/peak_set.pyx"
        }

#line 768 "ms_deisotope/_c/peak_set.pyx"
      }

#line 768 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 768 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "between") < 0)) __PYX_ERR(0, 768, __pyx_L3_error)

#line 768 "ms_deisotope/_c/peak_set.pyx"
      }

#line 768 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 768 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 768 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 768 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 768 "ms_deisotope/_c/peak_set.pyx"
      }

#line 768 "ms_deisotope/_c/peak_set.pyx"
    }

#line 768 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_m1 = values[0];

#line 768 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_m2 = values[1];

#line 768 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_tolerance = values[2];

#line 768 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_use_mz = values[3];

#line 768 "ms_deisotope/_c/peak_set.pyx"
  }

#line 768 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("between", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); 
#line 768 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 768, __pyx_L3_error)

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.between", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 768 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 768 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_m1, __pyx_v_m2, __pyx_v_tolerance, __pyx_v_use_mz);

#line 768 "ms_deisotope/_c/peak_set.pyx"


#line 768 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 768 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 768 "ms_deisotope/_c/peak_set.pyx"
}

#line 768 "ms_deisotope/_c/peak_set.pyx"


#line 768 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_m1, PyObject *__pyx_v_m2, CYTHON_UNUSED PyObject *__pyx_v_tolerance, PyObject *__pyx_v_use_mz) {
  PyObject *__pyx_v_acc = NULL
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  int __pyx_v_collecting
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_value = NULL
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_getter = NULL
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_iterator = NULL
#line 768 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_peak = NULL
#line 768 "ms_deisotope/_c/peak_set.pyx"
;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_3;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_6;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_8;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("between", 0);

  /* "ms_deisotope/_c/peak_set.pyx":787
 *         DeconvolutedPeakSet
 *         """
 *         acc = []             # <<<<<<<<<<<<<<
 *         collecting = False
 *         value = None
 */

#line 787 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)

#line 787 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 787 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_acc = ((PyObject*)__pyx_t_1);

#line 787 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":788
 *         """
 *         acc = []
 *         collecting = False             # <<<<<<<<<<<<<<
 *         value = None
 *         if not use_mz:
 */

#line 788 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_collecting = 0;

  /* "ms_deisotope/_c/peak_set.pyx":789
 *         acc = []
 *         collecting = False
 *         value = None             # <<<<<<<<<<<<<<
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")
 */

#line 789 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(Py_None);

#line 789 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_value = Py_None;

  /* "ms_deisotope/_c/peak_set.pyx":790
 *         collecting = False
 *         value = None
 *         if not use_mz:             # <<<<<<<<<<<<<<
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks
 */

#line 790 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_use_mz); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 790, __pyx_L1_error)

#line 790 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = ((!__pyx_t_2) != 0);

#line 790 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":791
 *         value = None
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 *             iterator = self.peaks
 *         else:
 */

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_operator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 791, __pyx_L1_error)

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 791, __pyx_L1_error)

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = NULL;

#line 791 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {

#line 791 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);

#line 791 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_4)) {

#line 791 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 791 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_4);

#line 791 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 791 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_5, function);

#line 791 "ms_deisotope/_c/peak_set.pyx"
      }

#line 791 "ms_deisotope/_c/peak_set.pyx"
    }

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_n_s_neutral_mass) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_neutral_mass);

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 791 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 791, __pyx_L1_error)

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_getter = __pyx_t_1;

#line 791 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":792
 *         if not use_mz:
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks             # <<<<<<<<<<<<<<
 *         else:
 *             getter = operator.attrgetter("mz")
 */

#line 792 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_v_self->peaks;

#line 792 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_1);

#line 792 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_iterator = ((PyObject*)__pyx_t_1);

#line 792 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":790
 *         collecting = False
 *         value = None
 *         if not use_mz:             # <<<<<<<<<<<<<<
 *             getter = operator.attrgetter("neutral_mass")
 *             iterator = self.peaks
 */

#line 790 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 790 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":794
 *             iterator = self.peaks
 *         else:
 *             getter = operator.attrgetter("mz")             # <<<<<<<<<<<<<<
 *             iterator = self._mz_ordered
 *         for peak in iterator:
 */

#line 794 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_operator); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 794, __pyx_L1_error)

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 794, __pyx_L1_error)

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = NULL;

#line 794 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 794 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 794 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_5)) {

#line 794 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 794 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_5);

#line 794 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 794 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 794 "ms_deisotope/_c/peak_set.pyx"
      }

#line 794 "ms_deisotope/_c/peak_set.pyx"
    }

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_n_s_mz) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_n_s_mz);

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 794 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_getter = __pyx_t_1;

#line 794 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":795
 *         else:
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered             # <<<<<<<<<<<<<<
 *         for peak in iterator:
 *             value = getter(peak)
 */

#line 795 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_v_self->_mz_ordered;

#line 795 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_1);

#line 795 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_iterator = ((PyObject*)__pyx_t_1);

#line 795 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 795 "ms_deisotope/_c/peak_set.pyx"
  }

#line 795 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":796
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered
 *         for peak in iterator:             # <<<<<<<<<<<<<<
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 */

#line 796 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_iterator == Py_None)) {

#line 796 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 796, __pyx_L1_error)

#line 796 "ms_deisotope/_c/peak_set.pyx"
  }

#line 796 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_iterator; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;

#line 796 "ms_deisotope/_c/peak_set.pyx"
  for (;;) {

#line 796 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;

#line 796 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 796, __pyx_L1_error)

#line 796 "ms_deisotope/_c/peak_set.pyx"
    #else

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 796, __pyx_L1_error)

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 796 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, __pyx_t_4);

#line 796 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":797
 *             iterator = self._mz_ordered
 *         for peak in iterator:
 *             value = getter(peak)             # <<<<<<<<<<<<<<
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 */

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_v_getter);
    __pyx_t_5 = __pyx_v_getter; 
#line 797 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_7 = NULL;

#line 797 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {

#line 797 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);

#line 797 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_7)) {

#line 797 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 797 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_7);

#line 797 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 797 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_5, function);

#line 797 "ms_deisotope/_c/peak_set.pyx"
      }

#line 797 "ms_deisotope/_c/peak_set.pyx"
    }

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_v_peak) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_peak);

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 797 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 797, __pyx_L1_error)

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);

#line 797 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":798
 *         for peak in iterator:
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:             # <<<<<<<<<<<<<<
 *                 collecting = True
 *             elif collecting and value > m2:
 */

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((!(__pyx_v_collecting != 0)) != 0);

#line 798 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 798 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 798 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 798 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7_bool_binop_done;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    }

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m1, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 798 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 798 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 798 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7_bool_binop_done;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    }

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m2, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 798, __pyx_L1_error)

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L7_bool_binop_done:;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":799
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True             # <<<<<<<<<<<<<<
 *             elif collecting and value > m2:
 *                 break
 */

#line 799 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_collecting = 1;

      /* "ms_deisotope/_c/peak_set.pyx":798
 *         for peak in iterator:
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:             # <<<<<<<<<<<<<<
 *                 collecting = True
 *             elif collecting and value > m2:
 */

#line 798 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6;

#line 798 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":800
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 *             elif collecting and value > m2:             # <<<<<<<<<<<<<<
 *                 break
 * 
 */

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (__pyx_v_collecting != 0);

#line 800 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 800 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 800 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 800 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L10_bool_binop_done;

#line 800 "ms_deisotope/_c/peak_set.pyx"
    }

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_value, __pyx_v_m2, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 800, __pyx_L1_error)

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 800, __pyx_L1_error)

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L10_bool_binop_done:;

#line 800 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":801
 *                 collecting = True
 *             elif collecting and value > m2:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             if collecting:
 */

#line 801 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_break;

      /* "ms_deisotope/_c/peak_set.pyx":800
 *             if not collecting and value >= m1 and value < m2:
 *                 collecting = True
 *             elif collecting and value > m2:             # <<<<<<<<<<<<<<
 *                 break
 * 
 */

#line 800 "ms_deisotope/_c/peak_set.pyx"
    }

#line 800 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6:;

    /* "ms_deisotope/_c/peak_set.pyx":803
 *                 break
 * 
 *             if collecting:             # <<<<<<<<<<<<<<
 *                 acc.append(peak.clone())
 * 
 */

#line 803 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_collecting != 0);

#line 803 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

      /* "ms_deisotope/_c/peak_set.pyx":804
 * 
 *             if collecting:
 *                 acc.append(peak.clone())             # <<<<<<<<<<<<<<
 * 
 *         return self.__class__(acc).reindex()
 */

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_peak, __pyx_n_s_clone); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_5);

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_7 = NULL;

#line 804 "ms_deisotope/_c/peak_set.pyx"
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 804 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);

#line 804 "ms_deisotope/_c/peak_set.pyx"
        if (likely(__pyx_t_7)) {

#line 804 "ms_deisotope/_c/peak_set.pyx"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 804 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_INCREF(__pyx_t_7);

#line 804 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_INCREF(function);

#line 804 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF_SET(__pyx_t_5, function);

#line 804 "ms_deisotope/_c/peak_set.pyx"
        }

#line 804 "ms_deisotope/_c/peak_set.pyx"
      }

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_5);

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 804 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 804, __pyx_L1_error)

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_acc, __pyx_t_4); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 804, __pyx_L1_error)

#line 804 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":803
 *                 break
 * 
 *             if collecting:             # <<<<<<<<<<<<<<
 *                 acc.append(peak.clone())
 * 
 */

#line 803 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":796
 *             getter = operator.attrgetter("mz")
 *             iterator = self._mz_ordered
 *         for peak in iterator:             # <<<<<<<<<<<<<<
 *             value = getter(peak)
 *             if not collecting and value >= m1 and value < m2:
 */

#line 796 "ms_deisotope/_c/peak_set.pyx"
  }

#line 796 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_break:;

#line 796 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":806
 *                 acc.append(peak.clone())
 * 
 *         return self.__class__(acc).reindex()             # <<<<<<<<<<<<<<
 * 
 * cdef double INF
 */

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 806, __pyx_L1_error)

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = NULL;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 806 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_7)) {

#line 806 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_7);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_5, function);

#line 806 "ms_deisotope/_c/peak_set.pyx"
    }

#line 806 "ms_deisotope/_c/peak_set.pyx"
  }

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_v_acc) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_acc);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 806, __pyx_L1_error)

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_reindex); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 806, __pyx_L1_error)

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = NULL;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 806 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_4)) {

#line 806 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_4);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 806 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_5, function);

#line 806 "ms_deisotope/_c/peak_set.pyx"
    }

#line 806 "ms_deisotope/_c/peak_set.pyx"
  }

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 806, __pyx_L1_error)

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 806 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":768
 *         return peak, errout
 * 
 *     def between(self, m1, m2, tolerance=1e-5, use_mz=False):             # <<<<<<<<<<<<<<
 *         """Retrieve a :class:`DeconvolutedPeakSet` containing all the peaks
 *         whose mass is between ``m1`` and ``m2``.
 */

#line 768 "ms_deisotope/_c/peak_set.pyx"


#line 768 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.between", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_acc);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_value);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_getter);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_iterator);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_peak);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 768 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 768 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 768 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pxd":77
 * cdef class DeconvolutedPeakSet(object):
 *     cdef:
 *         public tuple peaks             # <<<<<<<<<<<<<<
 *         public tuple _mz_ordered
 *         public bint indexed
 */

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 77 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 77 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(PyObject *__pyx_v_self) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 77 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_v_self->peaks);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_v_self->peaks;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 77 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 77, __pyx_L1_error)

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->peaks);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->peaks);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->peaks = ((PyObject*)__pyx_t_1);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.peaks.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(PyObject *__pyx_v_self) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 77 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 77 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->peaks);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->peaks);

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->peaks = ((PyObject*)Py_None);

#line 77 "ms_deisotope/_c/peak_set.pxd"


#line 77 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 77 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 77 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 77 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":78
 *     cdef:
 *         public tuple peaks
 *         public tuple _mz_ordered             # <<<<<<<<<<<<<<
 *         public bint indexed
 * 
 */

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 78 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 78 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(PyObject *__pyx_v_self) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 78 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_v_self->_mz_ordered);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_v_self->_mz_ordered;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 78 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  if (!(likely(PyTuple_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 78, __pyx_L1_error)

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __pyx_v_value;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(__pyx_t_1);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(__pyx_t_1);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->_mz_ordered = ((PyObject*)__pyx_t_1);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet._mz_ordered.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(PyObject *__pyx_v_self); /*proto*/

#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(PyObject *__pyx_v_self) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = 
#line 78 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_4__del__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 78 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__del__", 0);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_INCREF(Py_None);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GIVEREF(Py_None);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_v_self->_mz_ordered);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_DECREF(__pyx_v_self->_mz_ordered);

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->_mz_ordered = ((PyObject*)Py_None);

#line 78 "ms_deisotope/_c/peak_set.pxd"


#line 78 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 78 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 78 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 78 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pxd":79
 *         public tuple peaks
 *         public tuple _mz_ordered
 *         public bint indexed             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 79 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 79 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(PyObject *__pyx_v_self) {

#line 79 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 79 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self));

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 79 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
}

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed___get__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self) {

#line 79 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_r = NULL;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 79 "ms_deisotope/_c/peak_set.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__get__", 0);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->indexed); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = __pyx_t_1;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.indexed.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = NULL;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 79 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
}

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
/* Python wrapper */

#line 79 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/

#line 79 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = 
#line 79 "ms_deisotope/_c/peak_set.pxd"
__pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), ((PyObject *)__pyx_v_value));

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 79 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
}

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_2__set__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_self, PyObject *__pyx_v_value) {

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyDeclarations

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_t_1;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_lineno = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  const char *__pyx_filename = NULL;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  int __pyx_clineno = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannySetupContext("__set__", 0);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 79, __pyx_L1_error)

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_v_self->indexed = __pyx_t_1;

#line 79 "ms_deisotope/_c/peak_set.pxd"


#line 79 "ms_deisotope/_c/peak_set.pxd"
  /* function exit code */

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = 0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  goto __pyx_L0;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L1_error:;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSet.indexed.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_r = -1;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __pyx_L0:;

#line 79 "ms_deisotope/_c/peak_set.pxd"
  __Pyx_RefNannyFinishContext();

#line 79 "ms_deisotope/_c/peak_set.pxd"
  return __pyx_r;

#line 79 "ms_deisotope/_c/peak_set.pxd"
}

/* "ms_deisotope/_c/peak_set.pyx":812
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_neutral_mass(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

#line 812 "ms_deisotope/_c/peak_set.pyx"


#line 812 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_mid, double __pyx_v_tolerance) {
  size_t __pyx_v_best_index
#line 812 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 812 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 812 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 812 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 812 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_target = 0
#line 812 "ms_deisotope/_c/peak_set.pyx"
;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 812 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_sweep_solution_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":818
 *         DeconvolutedPeak target
 * 
 *     best_index = mid             # <<<<<<<<<<<<<<
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)
 */

#line 818 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_index = __pyx_v_mid;

  /* "ms_deisotope/_c/peak_set.pyx":819
 * 
 *     best_index = mid
 *     best_error = INF             # <<<<<<<<<<<<<<
 *     n = PyTuple_GET_SIZE(array)
 * 
 */

#line 819 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

  /* "ms_deisotope/_c/peak_set.pyx":820
 *     best_index = mid
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */

#line 820 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_v_array);

  /* "ms_deisotope/_c/peak_set.pyx":822
 *     n = PyTuple_GET_SIZE(array)
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 */

#line 822 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":823
 * 
 *     i = 0
 *     while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 */

#line 823 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 823 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 823 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 823 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 823 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __pyx_t_2;

#line 823 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_bool_binop_done;

#line 823 "ms_deisotope/_c/peak_set.pyx"
    }

#line 823 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 823 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_t_2;

#line 823 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5_bool_binop_done:;

#line 823 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":824
 *     i = 0
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 */

#line 824 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 824 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 824 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 824, __pyx_L1_error)

#line 824 "ms_deisotope/_c/peak_set.pyx"
    }

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_mid - __pyx_v_i);

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 824, __pyx_L1_error)

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_4;

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_5);

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_5));

#line 824 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":825
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */

#line 825 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->neutral_mass));

    /* "ms_deisotope/_c/peak_set.pyx":826
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */

#line 826 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);

#line 826 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":827
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */

#line 827 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 827 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":828
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid - i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */

#line 828 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":829
 *             if abs_error < best_error:
 *                 best_index = mid - i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */

#line 829 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":827
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */

#line 827 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":826
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */

#line 826 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7;

#line 826 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":831
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     i = 1
 */

#line 831 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 831 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L4_break;

#line 831 "ms_deisotope/_c/peak_set.pyx"
    }

#line 831 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L7:;

    /* "ms_deisotope/_c/peak_set.pyx":832
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     i = 1
 *     while (mid + i) < (n - 1):
 */

#line 832 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 1);

#line 832 "ms_deisotope/_c/peak_set.pyx"
  }

#line 832 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_break:;

  /* "ms_deisotope/_c/peak_set.pyx":833
 *             break
 *         i += 1
 *     i = 1             # <<<<<<<<<<<<<<
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 */

#line 833 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 1;

  /* "ms_deisotope/_c/peak_set.pyx":834
 *         i += 1
 *     i = 1
 *     while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 */

#line 834 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 834 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 834 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":835
 *     i = 1
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 */

#line 835 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 835 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 835 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 835, __pyx_L1_error)

#line 835 "ms_deisotope/_c/peak_set.pyx"
    }

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_mid + __pyx_v_i);

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_t_5;

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_4);

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 835 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":836
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */

#line 836 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->neutral_mass));

    /* "ms_deisotope/_c/peak_set.pyx":837
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */

#line 837 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);

#line 837 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":838
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */

#line 838 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 838 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":839
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid + i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */

#line 839 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":840
 *             if abs_error < best_error:
 *                 best_index = mid + i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */

#line 840 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":838
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */

#line 838 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":837
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.neutral_mass))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */

#line 837 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L11;

#line 837 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":842
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     if best_error == INF:
 */

#line 842 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 842 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L10_break;

#line 842 "ms_deisotope/_c/peak_set.pyx"
    }

#line 842 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L11:;

    /* "ms_deisotope/_c/peak_set.pyx":843
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     if best_error == INF:
 *         return None
 */

#line 843 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 1);

#line 843 "ms_deisotope/_c/peak_set.pyx"
  }

#line 843 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L10_break:;

  /* "ms_deisotope/_c/peak_set.pyx":844
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */

#line 844 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);

#line 844 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":845
 *         i += 1
 *     if best_error == INF:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return array[best_index]
 */

#line 845 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 845 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 845 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":844
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */

#line 844 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":847
 *         return None
 *     else:
 *         return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 847 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 847 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 847 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 847 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 847 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 847, __pyx_L1_error)

#line 847 "ms_deisotope/_c/peak_set.pyx"
    }

#line 847 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 847, __pyx_L1_error)

#line 847 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 847 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 847, __pyx_L1_error)

#line 847 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);

#line 847 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

#line 847 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 847 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":812
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_neutral_mass(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

#line 812 "ms_deisotope/_c/peak_set.pyx"


#line 812 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_solution_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_target);

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 812 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 812 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 812 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":851
 * 
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

#line 851 "ms_deisotope/_c/peak_set.pyx"


#line 851 "ms_deisotope/_c/peak_set.pyx"
static double __pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(double __pyx_v_x, double __pyx_v_y) {

#line 851 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_r;

  /* "ms_deisotope/_c/peak_set.pyx":852
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:
 *     return (x - y) / y             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 852 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((__pyx_v_x - __pyx_v_y) / __pyx_v_y);

#line 852 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":851
 * 
 * @cython.cdivision(True)
 * cdef double _ppm_error(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     return (x - y) / y
 * 
 */

#line 851 "ms_deisotope/_c/peak_set.pyx"


#line 851 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 851 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 851 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 851 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":855
 * 
 * 
 * cdef int _binary_search(double* array, double target, double error_tolerance, size_t n, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 855 "ms_deisotope/_c/peak_set.pyx"


#line 855 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t *__pyx_v_out) {
  size_t __pyx_v_lo
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_best_index
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 855 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 855 "ms_deisotope/_c/peak_set.pyx"
;

#line 855 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 855 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 855 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":862
 *         double best_error, abs_error
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */

#line 862 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":863
 * 
 *     lo = 0
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 863 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":865
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 */

#line 865 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 865 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 865 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":866
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */

#line 866 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":867
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */

#line 867 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":868
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 868 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);

#line 868 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":869
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */

#line 869 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":870
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */

#line 870 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":871
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */

#line 871 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":872
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 872 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 872 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 872 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 872 "ms_deisotope/_c/peak_set.pyx"
        } else {

#line 872 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = __pyx_t_2;

#line 872 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L8_bool_binop_done;

#line 872 "ms_deisotope/_c/peak_set.pyx"
        }

#line 872 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 872 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = __pyx_t_2;

#line 872 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L8_bool_binop_done:;

#line 872 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":873
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 873 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":874
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 874 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":875
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 875 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 875 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":876
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 876 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 876 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":877
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 877 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":878
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 878 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":876
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 876 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":875
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 875 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L10;

#line 875 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":880
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             i = 1
 */

#line 880 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 880 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L7_break;

#line 880 "ms_deisotope/_c/peak_set.pyx"
        }

#line 880 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":881
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */

#line 881 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 881 "ms_deisotope/_c/peak_set.pyx"
      }

#line 881 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":882
 *                     break
 *                 i += 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */

#line 882 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":883
 *                 i += 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 883 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 883 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 883 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":884
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 884 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":885
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 885 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":886
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 886 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 886 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":887
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 887 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 887 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":888
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 888 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":889
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 889 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":887
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 887 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":886
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 886 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L14;

#line 886 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":891
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             out[0] = best_index
 */

#line 891 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 891 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L13_break;

#line 891 "ms_deisotope/_c/peak_set.pyx"
        }

#line 891 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":892
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             out[0] = best_index
 *             if best_error == INF:
 */

#line 892 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 892 "ms_deisotope/_c/peak_set.pyx"
      }

#line 892 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":893
 *                     break
 *                 i += 1
 *             out[0] = best_index             # <<<<<<<<<<<<<<
 *             if best_error == INF:
 *                 return 3
 */

#line 893 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_out[0]) = __pyx_v_best_index;

      /* "ms_deisotope/_c/peak_set.pyx":894
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */

#line 894 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);

#line 894 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":895
 *             out[0] = best_index
 *             if best_error == INF:
 *                 return 3             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */

#line 895 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = 3;

#line 895 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

        /* "ms_deisotope/_c/peak_set.pyx":894
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */

#line 894 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":896
 *             if best_error == INF:
 *                 return 3
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             out[0] = 0
 */

#line 896 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 0;

#line 896 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":868
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 868 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":897
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */

#line 897 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 897 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":898
 *             return 0
 *         elif hi - lo == 1:
 *             out[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */

#line 898 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_out[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":899
 *         elif hi - lo == 1:
 *             out[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */

#line 899 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 1;

#line 899 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":897
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */

#line 897 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":900
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 900 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);

#line 900 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":901
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 901 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":900
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 900 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 900 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":903
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     out[0] = 0
 *     return 2
 */

#line 903 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 903 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 903 "ms_deisotope/_c/peak_set.pyx"
    }

#line 903 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 903 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":904
 *         else:
 *             lo = mid
 *     out[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */

#line 904 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_out[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":905
 *             lo = mid
 *     out[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 905 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 2;

#line 905 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":855
 * 
 * 
 * cdef int _binary_search(double* array, double target, double error_tolerance, size_t n, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 855 "ms_deisotope/_c/peak_set.pyx"


#line 855 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 855 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 855 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 855 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":908
 * 
 * 
 * cdef int _binary_search_interval(double* array, double target, double error_tolerance, size_t n, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 908 "ms_deisotope/_c/peak_set.pyx"


#line 908 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_lo
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  CYTHON_UNUSED size_t __pyx_v_best_index
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 908 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 908 "ms_deisotope/_c/peak_set.pyx"
;

#line 908 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 908 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 908 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":915
 *         double best_error, abs_error
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */

#line 915 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":916
 * 
 *     lo = 0
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 916 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":918
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 */

#line 918 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 918 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 918 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":919
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */

#line 919 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":920
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */

#line 920 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":921
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 921 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);

#line 921 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":922
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */

#line 922 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":923
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */

#line 923 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":924
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */

#line 924 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":925
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 925 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 925 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 925 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 925 "ms_deisotope/_c/peak_set.pyx"
        } else {

#line 925 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = __pyx_t_2;

#line 925 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L8_bool_binop_done;

#line 925 "ms_deisotope/_c/peak_set.pyx"
        }

#line 925 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 925 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = __pyx_t_2;

#line 925 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L8_bool_binop_done:;

#line 925 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":926
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 926 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":927
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 927 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":928
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 928 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 928 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":929
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 929 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 929 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":930
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 930 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":931
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 931 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":929
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 929 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":928
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 928 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L10;

#line 928 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":933
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             start[0] = mid - i + 1
 */

#line 933 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 933 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L7_break;

#line 933 "ms_deisotope/_c/peak_set.pyx"
        }

#line 933 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":934
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             start[0] = mid - i + 1
 *             i = 1
 */

#line 934 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 934 "ms_deisotope/_c/peak_set.pyx"
      }

#line 934 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":935
 *                     break
 *                 i += 1
 *             start[0] = mid - i + 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */

#line 935 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = ((__pyx_v_mid - __pyx_v_i) + 1);

      /* "ms_deisotope/_c/peak_set.pyx":936
 *                 i += 1
 *             start[0] = mid - i + 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */

#line 936 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":937
 *             start[0] = mid - i + 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 937 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 937 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 937 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":938
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 938 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":939
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 939 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":940
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 940 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 940 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":941
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 941 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 941 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":942
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 942 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":943
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 943 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":941
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 941 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":940
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 940 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L14;

#line 940 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":945
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             end[0] = mid + i
 */

#line 945 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 945 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L13_break;

#line 945 "ms_deisotope/_c/peak_set.pyx"
        }

#line 945 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":946
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             end[0] = mid + i
 *             return 0
 */

#line 946 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 946 "ms_deisotope/_c/peak_set.pyx"
      }

#line 946 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":947
 *                     break
 *                 i += 1
 *             end[0] = mid + i             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */

#line 947 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_end[0]) = (__pyx_v_mid + __pyx_v_i);

      /* "ms_deisotope/_c/peak_set.pyx":948
 *                 i += 1
 *             end[0] = mid + i
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             start[0] = 0
 */

#line 948 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 0;

#line 948 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":921
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 921 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":949
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */

#line 949 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 949 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":950
 *             return 0
 *         elif hi - lo == 1:
 *             start[0] = 0             # <<<<<<<<<<<<<<
 *             end[0] = 0
 *             return 1
 */

#line 950 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":951
 *         elif hi - lo == 1:
 *             start[0] = 0
 *             end[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */

#line 951 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_end[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":952
 *             start[0] = 0
 *             end[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */

#line 952 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 1;

#line 952 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":949
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */

#line 949 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":953
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 953 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);

#line 953 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":954
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 954 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":953
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 953 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 953 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":956
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     start[0] = 0
 *     end[0] = 0
 */

#line 956 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 956 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 956 "ms_deisotope/_c/peak_set.pyx"
    }

#line 956 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 956 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":957
 *         else:
 *             lo = mid
 *     start[0] = 0             # <<<<<<<<<<<<<<
 *     end[0] = 0
 *     return 2
 */

#line 957 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_start[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":958
 *             lo = mid
 *     start[0] = 0
 *     end[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */

#line 958 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_end[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":959
 *     start[0] = 0
 *     end[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 959 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 2;

#line 959 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":908
 * 
 * 
 * cdef int _binary_search_interval(double* array, double target, double error_tolerance, size_t n, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 908 "ms_deisotope/_c/peak_set.pyx"


#line 908 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 908 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 908 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 908 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":962
 * 
 * 
 * cdef DeconvolutedPeak binary_search_neutral_mass(tuple peak_set, double neutral_mass, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 962 "ms_deisotope/_c/peak_set.pyx"


#line 962 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_neutral_mass(PyObject *__pyx_v_peak_set, double __pyx_v_neutral_mass, double __pyx_v_error_tolerance) {
  size_t __pyx_v_lo
#line 962 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 962 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 962 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 962 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0
#line 962 "ms_deisotope/_c/peak_set.pyx"
;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 962 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_4;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("binary_search_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":968
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = PyTuple_GET_SIZE(peak_set)
 * 
 */

#line 968 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":969
 * 
 *     lo = 0
 *     hi = PyTuple_GET_SIZE(peak_set)             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 969 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = PyTuple_GET_SIZE(__pyx_v_peak_set);

  /* "ms_deisotope/_c/peak_set.pyx":971
 *     hi = PyTuple_GET_SIZE(peak_set)
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */

#line 971 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 971 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 971 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":972
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass
 */

#line 972 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":973
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.neutral_mass
 * 
 */

#line 973 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_peak_set == Py_None)) {

#line 973 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 973 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 973, __pyx_L1_error)

#line 973 "ms_deisotope/_c/peak_set.pyx"
    }

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_3);

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));

#line 973 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":974
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 */

#line 974 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_v_found_peak->neutral_mass;

#line 974 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":976
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 */

#line 976 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_neutral_mass)) < __pyx_v_error_tolerance) != 0);

#line 976 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":977
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */

#line 977 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_neutral_mass(__pyx_v_peak_set, __pyx_v_neutral_mass, __pyx_v_mid, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 977, __pyx_L1_error)

#line 977 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_3);

#line 977 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3));

#line 977 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":978
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             return None
 */

#line 978 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 978 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));

#line 978 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_v_found_peak;

#line 978 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":976
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 */

#line 976 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":979
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > neutral_mass:
 */

#line 979 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 979 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":980
 *             return found_peak
 *         elif hi - lo == 1:
 *             return None             # <<<<<<<<<<<<<<
 *         elif found_mass > neutral_mass:
 *             hi = mid
 */

#line 980 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 980 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 980 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":979
 *             found_peak = _sweep_solution_neutral_mass(peak_set, neutral_mass, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > neutral_mass:
 */

#line 979 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":981
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 981 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_neutral_mass) != 0);

#line 981 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":982
 *             return None
 *         elif found_mass > neutral_mass:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 982 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":981
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 981 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 981 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":984
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 984 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 984 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 984 "ms_deisotope/_c/peak_set.pyx"
    }

#line 984 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 984 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":962
 * 
 * 
 * cdef DeconvolutedPeak binary_search_neutral_mass(tuple peak_set, double neutral_mass, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 962 "ms_deisotope/_c/peak_set.pyx"


#line 962 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 962 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 962 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 962 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":987
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_mz(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

#line 987 "ms_deisotope/_c/peak_set.pyx"


#line 987 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_mid, double __pyx_v_tolerance) {
  size_t __pyx_v_best_index
#line 987 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 987 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 987 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 987 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 987 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_target = 0
#line 987 "ms_deisotope/_c/peak_set.pyx"
;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 987 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_sweep_solution_mz", 0);

  /* "ms_deisotope/_c/peak_set.pyx":993
 *         DeconvolutedPeak target
 * 
 *     best_index = mid             # <<<<<<<<<<<<<<
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)
 */

#line 993 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_index = __pyx_v_mid;

  /* "ms_deisotope/_c/peak_set.pyx":994
 * 
 *     best_index = mid
 *     best_error = INF             # <<<<<<<<<<<<<<
 *     n = PyTuple_GET_SIZE(array)
 * 
 */

#line 994 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

  /* "ms_deisotope/_c/peak_set.pyx":995
 *     best_index = mid
 *     best_error = INF
 *     n = PyTuple_GET_SIZE(array)             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */

#line 995 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_v_array);

  /* "ms_deisotope/_c/peak_set.pyx":997
 *     n = PyTuple_GET_SIZE(array)
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 */

#line 997 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":998
 * 
 *     i = 0
 *     while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 */

#line 998 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 998 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 998 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 998 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 998 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __pyx_t_2;

#line 998 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_bool_binop_done;

#line 998 "ms_deisotope/_c/peak_set.pyx"
    }

#line 998 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 998 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_t_2;

#line 998 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5_bool_binop_done:;

#line 998 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":999
 *     i = 0
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 */

#line 999 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 999 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 999 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 999, __pyx_L1_error)

#line 999 "ms_deisotope/_c/peak_set.pyx"
    }

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_mid - __pyx_v_i);

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 999, __pyx_L1_error)

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_t_4;

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_5);

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_5));

#line 999 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1000
 *     while mid - i >= 0 and i <= mid:
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */

#line 1000 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->__pyx_base.mz));

    /* "ms_deisotope/_c/peak_set.pyx":1001
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */

#line 1001 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);

#line 1001 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1002
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */

#line 1002 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1002 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1003
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid - i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */

#line 1003 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":1004
 *             if abs_error < best_error:
 *                 best_index = mid - i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */

#line 1004 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":1002
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid - i
 *                 best_error = abs_error
 */

#line 1002 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1001
 *         target = <DeconvolutedPeak>array[mid - i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid - i
 */

#line 1001 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L7;

#line 1001 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1006
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     i = 1
 */

#line 1006 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1006 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L4_break;

#line 1006 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1006 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L7:;

    /* "ms_deisotope/_c/peak_set.pyx":1007
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     i = 1
 *     while (mid + i) < (n - 1):
 */

#line 1007 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 1);

#line 1007 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1007 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_break:;

  /* "ms_deisotope/_c/peak_set.pyx":1008
 *             break
 *         i += 1
 *     i = 1             # <<<<<<<<<<<<<<
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 */

#line 1008 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 1;

  /* "ms_deisotope/_c/peak_set.pyx":1009
 *         i += 1
 *     i = 1
 *     while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 */

#line 1009 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1009 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 1009 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1010
 *     i = 1
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]             # <<<<<<<<<<<<<<
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 */

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 1010 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1010 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1010, __pyx_L1_error)

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_mid + __pyx_v_i);

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_t_3, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1010, __pyx_L1_error)

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_t_5;

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_4);

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_target, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 1010 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1011
 *     while (mid + i) < (n - 1):
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))             # <<<<<<<<<<<<<<
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 */

#line 1011 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_value, __pyx_v_target->__pyx_base.mz));

    /* "ms_deisotope/_c/peak_set.pyx":1012
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */

#line 1012 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_tolerance) != 0);

#line 1012 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1013
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */

#line 1013 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1013 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1014
 *         if abs_error < tolerance:
 *             if abs_error < best_error:
 *                 best_index = mid + i             # <<<<<<<<<<<<<<
 *                 best_error = abs_error
 *         else:
 */

#line 1014 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

        /* "ms_deisotope/_c/peak_set.pyx":1015
 *             if abs_error < best_error:
 *                 best_index = mid + i
 *                 best_error = abs_error             # <<<<<<<<<<<<<<
 *         else:
 *             break
 */

#line 1015 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_best_error = __pyx_v_abs_error;

        /* "ms_deisotope/_c/peak_set.pyx":1013
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:
 *             if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                 best_index = mid + i
 *                 best_error = abs_error
 */

#line 1013 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1012
 *         target = <DeconvolutedPeak>array[mid + i]
 *         abs_error = abs(ppm_error(value, target.mz))
 *         if abs_error < tolerance:             # <<<<<<<<<<<<<<
 *             if abs_error < best_error:
 *                 best_index = mid + i
 */

#line 1012 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L11;

#line 1012 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1017
 *                 best_error = abs_error
 *         else:
 *             break             # <<<<<<<<<<<<<<
 *         i += 1
 *     if best_error == INF:
 */

#line 1017 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1017 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L10_break;

#line 1017 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1017 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L11:;

    /* "ms_deisotope/_c/peak_set.pyx":1018
 *         else:
 *             break
 *         i += 1             # <<<<<<<<<<<<<<
 *     if best_error == INF:
 *         return None
 */

#line 1018 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 1);

#line 1018 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1018 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L10_break:;

  /* "ms_deisotope/_c/peak_set.pyx":1019
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */

#line 1019 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);

#line 1019 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1020
 *         i += 1
 *     if best_error == INF:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return array[best_index]
 */

#line 1020 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1020 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1020 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1019
 *             break
 *         i += 1
 *     if best_error == INF:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */

#line 1019 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1022
 *         return None
 *     else:
 *         return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1022 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 1022 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1022 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1022, __pyx_L1_error)

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L1_error)

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_4);

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 1022, __pyx_L1_error)

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

#line 1022 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 1022 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":987
 * 
 * 
 * cdef DeconvolutedPeak _sweep_solution_mz(tuple array, double value, size_t mid, double tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t best_index, i, n
 */

#line 987 "ms_deisotope/_c/peak_set.pyx"


#line 987 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_solution_mz", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_target);

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 987 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 987 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 987 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1025
 * 
 * 
 * cdef DeconvolutedPeak binary_search_mz(tuple peak_set, double mz, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1025 "ms_deisotope/_c/peak_set.pyx"


#line 1025 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_mz(PyObject *__pyx_v_peak_set, double __pyx_v_mz, double __pyx_v_error_tolerance) {
  size_t __pyx_v_lo
#line 1025 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 1025 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 1025 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 1025 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0
#line 1025 "ms_deisotope/_c/peak_set.pyx"
;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_1;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_5;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("binary_search_mz", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1031
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = len(peak_set)
 * 
 */

#line 1031 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1032
 * 
 *     lo = 0
 *     hi = len(peak_set)             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 1032 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_peak_set == Py_None)) {

#line 1032 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 1032 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1032, __pyx_L1_error)

#line 1032 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1032 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_peak_set); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1032, __pyx_L1_error)

#line 1032 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1034
 *     hi = len(peak_set)
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */

#line 1034 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1034 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 1034 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1035
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.mz
 */

#line 1035 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1036
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.mz
 * 
 */

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_peak_set == Py_None)) {

#line 1036 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1036 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1036, __pyx_L1_error)

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1036, __pyx_L1_error)

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_t_3;

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_4);

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 1036 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1037
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.mz             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 */

#line 1037 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_v_found_peak->__pyx_base.mz;

#line 1037 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":1039
 *         found_mass = found_peak.mz
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 */

#line 1039 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_mz)) < __pyx_v_error_tolerance) != 0);

#line 1039 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1040
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */

#line 1040 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_solution_mz(__pyx_v_peak_set, __pyx_v_mz, __pyx_v_mid, __pyx_v_error_tolerance)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1040, __pyx_L1_error)

#line 1040 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 1040 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 1040 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1041
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             return None
 */

#line 1041 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1041 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));

#line 1041 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_v_found_peak;

#line 1041 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1039
 *         found_mass = found_peak.mz
 * 
 *         if abs(ppm_error(found_mass, mz)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 */

#line 1039 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1042
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > mz:
 */

#line 1042 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 1042 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1043
 *             return found_peak
 *         elif hi - lo == 1:
 *             return None             # <<<<<<<<<<<<<<
 *         elif found_mass > mz:
 *             hi = mid
 */

#line 1043 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1043 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1043 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1042
 *             found_peak = _sweep_solution_mz(peak_set, mz, mid, error_tolerance)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             return None
 *         elif found_mass > mz:
 */

#line 1042 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1044
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > mz:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1044 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_found_mass > __pyx_v_mz) != 0);

#line 1044 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1045
 *             return None
 *         elif found_mass > mz:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 1045 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1044
 *         elif hi - lo == 1:
 *             return None
 *         elif found_mass > mz:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1044 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1044 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1047
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1047 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1047 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 1047 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1047 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1047 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1025
 * 
 * 
 * cdef DeconvolutedPeak binary_search_mz(tuple peak_set, double mz, double error_tolerance):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1025 "ms_deisotope/_c/peak_set.pyx"


#line 1025 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_mz", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1025 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1025 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1050
 * 
 * 
 * cdef DeconvolutedPeak _sweep_nearest_match_neutral_mass(tuple array, double value, size_t lo, size_t hi, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

#line 1050 "ms_deisotope/_c/peak_set.pyx"


#line 1050 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(PyObject *__pyx_v_array, double __pyx_v_value, size_t __pyx_v_lo, size_t __pyx_v_hi, double *__pyx_v_errout) {
  size_t __pyx_v_i
#line 1050 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_best_index
#line 1050 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 1050 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_err
#line 1050 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_v
#line 1050 "ms_deisotope/_c/peak_set.pyx"
;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_1;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_2;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_7;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_sweep_nearest_match_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1057
 *         double v
 * 
 *     best_error = float('inf')             # <<<<<<<<<<<<<<
 *     best_index = -1
 *     for i in range(hi - lo):
 */

#line 1057 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_AsDouble(__pyx_n_s_inf); if (unlikely(__pyx_t_1 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1057, __pyx_L1_error)

#line 1057 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_error = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1058
 * 
 *     best_error = float('inf')
 *     best_index = -1             # <<<<<<<<<<<<<<
 *     for i in range(hi - lo):
 *         i += lo
 */

#line 1058 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_best_index = -1L;

  /* "ms_deisotope/_c/peak_set.pyx":1059
 *     best_error = float('inf')
 *     best_index = -1
 *     for i in range(hi - lo):             # <<<<<<<<<<<<<<
 *         i += lo
 *         v = array[i].neutral_mass
 */

#line 1059 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_v_hi - __pyx_v_lo);

#line 1059 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_t_2;

#line 1059 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1059 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":1060
 *     best_index = -1
 *     for i in range(hi - lo):
 *         i += lo             # <<<<<<<<<<<<<<
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 */

#line 1060 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + __pyx_v_lo);

    /* "ms_deisotope/_c/peak_set.pyx":1061
 *     for i in range(hi - lo):
 *         i += lo
 *         v = array[i].neutral_mass             # <<<<<<<<<<<<<<
 *         err = abs(v - value)
 *         if err < best_error:
 */

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_array == Py_None)) {

#line 1061 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1061 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1061, __pyx_L1_error)

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_i, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1061, __pyx_L1_error)

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_5);

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1061, __pyx_L1_error)

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1061, __pyx_L1_error)

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1061 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_v = __pyx_t_1;

    /* "ms_deisotope/_c/peak_set.pyx":1062
 *         i += lo
 *         v = array[i].neutral_mass
 *         err = abs(v - value)             # <<<<<<<<<<<<<<
 *         if err < best_error:
 *             best_error = err
 */

#line 1062 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_err = fabs((__pyx_v_v - __pyx_v_value));

    /* "ms_deisotope/_c/peak_set.pyx":1063
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 *         if err < best_error:             # <<<<<<<<<<<<<<
 *             best_error = err
 *             best_index = i
 */

#line 1063 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_7 = ((__pyx_v_err < __pyx_v_best_error) != 0);

#line 1063 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_7) {

      /* "ms_deisotope/_c/peak_set.pyx":1064
 *         err = abs(v - value)
 *         if err < best_error:
 *             best_error = err             # <<<<<<<<<<<<<<
 *             best_index = i
 *     errout[0] = best_error
 */

#line 1064 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_error = __pyx_v_err;

      /* "ms_deisotope/_c/peak_set.pyx":1065
 *         if err < best_error:
 *             best_error = err
 *             best_index = i             # <<<<<<<<<<<<<<
 *     errout[0] = best_error
 *     return array[best_index]
 */

#line 1065 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_index = __pyx_v_i;

      /* "ms_deisotope/_c/peak_set.pyx":1063
 *         v = array[i].neutral_mass
 *         err = abs(v - value)
 *         if err < best_error:             # <<<<<<<<<<<<<<
 *             best_error = err
 *             best_index = i
 */

#line 1063 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1063 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1066
 *             best_error = err
 *             best_index = i
 *     errout[0] = best_error             # <<<<<<<<<<<<<<
 *     return array[best_index]
 * 
 */

#line 1066 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_errout[0]) = __pyx_v_best_error;

  /* "ms_deisotope/_c/peak_set.pyx":1067
 *             best_index = i
 *     errout[0] = best_error
 *     return array[best_index]             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_array == Py_None)) {

#line 1067 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1067 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1067, __pyx_L1_error)

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v_array, __pyx_v_best_index, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1067, __pyx_L1_error)

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak))))) __PYX_ERR(0, 1067, __pyx_L1_error)

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_6);

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 1067 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1050
 * 
 * 
 * cdef DeconvolutedPeak _sweep_nearest_match_neutral_mass(tuple array, double value, size_t lo, size_t hi, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

#line 1050 "ms_deisotope/_c/peak_set.pyx"


#line 1050 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set._sweep_nearest_match_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1050 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1050 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1070
 * 
 * 
 * cdef DeconvolutedPeak binary_search_nearest_neutral_mass(tuple peak_set, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1070 "ms_deisotope/_c/peak_set.pyx"


#line 1070 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_binary_search_nearest_neutral_mass(PyObject *__pyx_v_peak_set, double __pyx_v_neutral_mass, double *__pyx_v_errout) {
  size_t __pyx_v_lo
#line 1070 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 1070 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 1070 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 1070 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_found_peak = 0
#line 1070 "ms_deisotope/_c/peak_set.pyx"
;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_1;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_5;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("binary_search_nearest_neutral_mass", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1076
 *         DeconvolutedPeak found_peak
 * 
 *     lo = 0             # <<<<<<<<<<<<<<
 *     hi = len(peak_set)
 *     while hi != lo:
 */

#line 1076 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1077
 * 
 *     lo = 0
 *     hi = len(peak_set)             # <<<<<<<<<<<<<<
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 */

#line 1077 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_peak_set == Py_None)) {

#line 1077 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 1077 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1077, __pyx_L1_error)

#line 1077 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1077 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_peak_set); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1077, __pyx_L1_error)

#line 1077 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1078
 *     lo = 0
 *     hi = len(peak_set)
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 */

#line 1078 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1078 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 1078 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1079
 *     hi = len(peak_set)
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass
 */

#line 1079 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1080
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]             # <<<<<<<<<<<<<<
 *         found_mass = found_peak.neutral_mass
 * 
 */

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_peak_set == Py_None)) {

#line 1080 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1080 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1080, __pyx_L1_error)

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_peak_set, __pyx_v_mid, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1080, __pyx_L1_error)

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_3);

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = __pyx_t_3;

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_4);

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 1080 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1081
 *         mid = (hi + lo) // 2
 *         found_peak = <DeconvolutedPeak>peak_set[mid]
 *         found_mass = found_peak.neutral_mass             # <<<<<<<<<<<<<<
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 */

#line 1081 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = __pyx_v_found_peak->neutral_mass;

#line 1081 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = __pyx_t_5;

    /* "ms_deisotope/_c/peak_set.pyx":1083
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 */

#line 1083 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set_ppm_error(__pyx_v_found_mass, __pyx_v_neutral_mass)) < 1.0) != 0);

#line 1083 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1084
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif hi - lo == 1:
 */

#line 1084 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set__sweep_nearest_match_neutral_mass(__pyx_v_peak_set, __pyx_v_neutral_mass, __pyx_v_lo, __pyx_v_hi, __pyx_v_errout)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1084, __pyx_L1_error)

#line 1084 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_4);

#line 1084 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_v_found_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4));

#line 1084 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1085
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass
 */

#line 1085 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1085 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));

#line 1085 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_v_found_peak;

#line 1085 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1083
 *         found_mass = found_peak.neutral_mass
 * 
 *         if abs(ppm_error(found_mass, neutral_mass)) < 1:             # <<<<<<<<<<<<<<
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 */

#line 1083 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1086
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 */

#line 1086 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 1086 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1087
 *             return found_peak
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass             # <<<<<<<<<<<<<<
 *             return found_peak
 *         elif found_mass > neutral_mass:
 */

#line 1087 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_errout[0]) = (__pyx_v_found_mass - __pyx_v_neutral_mass);

      /* "ms_deisotope/_c/peak_set.pyx":1088
 *         elif hi - lo == 1:
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak             # <<<<<<<<<<<<<<
 *         elif found_mass > neutral_mass:
 *             hi = mid
 */

#line 1088 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1088 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(((PyObject *)__pyx_v_found_peak));

#line 1088 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = __pyx_v_found_peak;

#line 1088 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1086
 *             found_peak = _sweep_nearest_match_neutral_mass(peak_set, neutral_mass, lo, hi, errout)
 *             return found_peak
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 */

#line 1086 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1089
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1089 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_found_mass > __pyx_v_neutral_mass) != 0);

#line 1089 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1090
 *             return found_peak
 *         elif found_mass > neutral_mass:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 1090 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1089
 *             errout[0] = found_mass - neutral_mass
 *             return found_peak
 *         elif found_mass > neutral_mass:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1089 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1089 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1092
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1092 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1092 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 1092 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1092 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1092 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1070
 * 
 * 
 * cdef DeconvolutedPeak binary_search_nearest_neutral_mass(tuple peak_set, double neutral_mass, double* errout):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1070 "ms_deisotope/_c/peak_set.pyx"


#line 1070 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.binary_search_nearest_neutral_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_found_peak);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1070 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1070 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

#line 1095 "ms_deisotope/_c/peak_set.pyx"


#line 1095 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1095 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/

#line 1095 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_12convert[] = "convert(self)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_13convert = 
#line 1095 "ms_deisotope/_c/peak_set.pyx"
{"convert", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_12convert};

#line 1095 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_13convert(PyObject *__pyx_self, PyObject *__pyx_v_self) {

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("convert (wrapper)", 0);
  __pyx_r = 
#line 1095 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(__pyx_self, ((PyObject *)__pyx_v_self));

#line 1095 "ms_deisotope/_c/peak_set.pyx"


#line 1095 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
}

#line 1095 "ms_deisotope/_c/peak_set.pyx"


#line 1095 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_12convert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_7 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_9 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_10 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_11 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("convert", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1096
 * 
 * def convert(self):
 *     return DeconvolutedPeak(             # <<<<<<<<<<<<<<
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 */

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

  /* "ms_deisotope/_c/peak_set.pyx":1097
 * def convert(self):
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,             # <<<<<<<<<<<<<<
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 */

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1097, __pyx_L1_error)

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_intensity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1097, __pyx_L1_error)

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_charge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1097, __pyx_L1_error)

#line 1097 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

  /* "ms_deisotope/_c/peak_set.pyx":1098
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,             # <<<<<<<<<<<<<<
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 */

#line 1098 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_signal_to_noise); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1098, __pyx_L1_error)

#line 1098 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 1098 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_full_width_at_half_max); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1098, __pyx_L1_error)

#line 1098 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

  /* "ms_deisotope/_c/peak_set.pyx":1099
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,             # <<<<<<<<<<<<<<
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 *         self.area)
 */

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a_to_a2_ratio); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1099, __pyx_L1_error)

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_most_abundant_mass); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1099, __pyx_L1_error)

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_7);

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_average_mass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1099, __pyx_L1_error)

#line 1099 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_8);

  /* "ms_deisotope/_c/peak_set.pyx":1100
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,             # <<<<<<<<<<<<<<
 *         self.area)
 * 
 */

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_score); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1100, __pyx_L1_error)

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_9);

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_envelope); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1100, __pyx_L1_error)

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope), __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1100, __pyx_L1_error)

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_11);

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_mz); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1100, __pyx_L1_error)

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_10);

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_chosen_for_msms); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1100, __pyx_L1_error)

#line 1100 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_12);

  /* "ms_deisotope/_c/peak_set.pyx":1101
 *         self.a_to_a2_ratio, self.most_abundant_mass, self.average_mass,
 *         self.score, Envelope(self.envelope), self.mz, None, self.chosen_for_msms,
 *         self.area)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1101 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_area); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1101, __pyx_L1_error)

#line 1101 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

  /* "ms_deisotope/_c/peak_set.pyx":1096
 * 
 * def convert(self):
 *     return DeconvolutedPeak(             # <<<<<<<<<<<<<<
 *         self.neutral_mass, self.intensity, self.charge,
 *         self.signal_to_noise, -1, self.full_width_at_half_max,
 */

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_14 = PyTuple_New(15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1096, __pyx_L1_error)

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_14);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_1);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_2);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_t_3);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_4);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_int_neg_1);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_int_neg_1);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_int_neg_1);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_5);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_5);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_6);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_t_6);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_7);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_7);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_8);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 8, __pyx_t_8);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_9);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 9, __pyx_t_9);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_11);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 10, __pyx_t_11);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_10);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 11, __pyx_t_10);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(Py_None);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(Py_None);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 12, Py_None);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_12);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 13, __pyx_t_12);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_13);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_14, 14, __pyx_t_13);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_7 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_11 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_12 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), __pyx_t_14, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1096, __pyx_L1_error)

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_13);

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_13;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_13 = 0;

#line 1096 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

#line 1095 "ms_deisotope/_c/peak_set.pyx"


#line 1095 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_7);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_9);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_10);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_11);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1095 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes[] = "decode_envelopes(signatures, args, kwargs, defaults)";
static PyMethodDef __pyx_mdef_12ms_deisotope_2_c_8peak_set_15decode_envelopes = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
{"decode_envelopes", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_15decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_args = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_kwargs = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[4] = {0,0,0,0};

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  4: 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1106, __pyx_L3_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        case  2:

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1106, __pyx_L3_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        case  3:

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1106, __pyx_L3_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) __PYX_ERR(0, 1106, __pyx_L3_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_signatures = values[0];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_args = values[1];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_kwargs = values[2];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_defaults = values[3];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1106, __pyx_L3_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_14decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  Py_ssize_t __pyx_v_i
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyTypeObject *__pyx_v_ndarray = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  __Pyx_memviewslice __pyx_v_memslice
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  Py_ssize_t __pyx_v_itemsize
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  CYTHON_UNUSED int __pyx_v_dtype_signed
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  char __pyx_v_kind
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_arg = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_dtype = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_arg_base = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_candidates = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_sig = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  int __pyx_v_match_found
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_src_sig = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_dst_type = NULL
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_4;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_5;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  long __pyx_t_7;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_memviewslice __pyx_t_8;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_9;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_10;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_11;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_12 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_13 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_14 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_15;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_16;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  Py_ssize_t __pyx_t_17;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_18;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("decode_envelopes", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_kwargs);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_4) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L4_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = ((!__pyx_t_4) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF_SET(__pyx_v_kwargs, Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__Pyx_ImportNumPyArrayTypeIfAvailable()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_itemsize = -1L;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_args == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((0 < __pyx_t_5) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = (__pyx_v_kwargs != Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = (__pyx_t_3 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_4) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L7_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_array, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = (__pyx_t_4 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L7_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_kwargs == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_arg = __pyx_t_1;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_args == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_int_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_int_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_int_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_kp_s__2);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_kp_s__2);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_kp_s__2);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GIVEREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_t_2 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_dtype = __pyx_t_6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L12;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = __pyx_memoryview_check(__pyx_v_arg); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_arg_base = __pyx_t_6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_3 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_v_dtype = __pyx_t_6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L13;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_INCREF(Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_v_dtype = Py_None;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L13:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L12;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_dtype = Py_None;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L12:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_itemsize = -1L;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_v_dtype != Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = (__pyx_t_2 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_itemsize = __pyx_t_5;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_7 = __Pyx_PyObject_Ord(__pyx_t_6); if (unlikely(__pyx_t_7 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_kind = __pyx_t_7;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        switch (__pyx_v_kind) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          case 'i':

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          case 'u':

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          case 'f':

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            goto __pyx_L16_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_L16_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            goto __pyx_L10_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            goto __pyx_L19_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_5) == 1) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_L19_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
            goto __pyx_L10_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          case 'c':

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          case 'O':

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          default: break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L22_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L22_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_v_arg, 0); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L10_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        PyErr_Clear(); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L26_bool_binop_done;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L26_bool_binop_done:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_v_arg, 0); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_memslice = __pyx_t_8;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L10_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        PyErr_Clear(); 

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L10_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L10_break:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_candidates = ((PyObject*)__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_signatures == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_9), (&__pyx_t_10)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_6 = __pyx_t_1;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_6, __pyx_t_9, &__pyx_t_5, &__pyx_t_1, NULL, NULL, __pyx_t_10);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_t_11 == 0)) break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_match_found = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_14 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_14)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_14);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_12 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_kp_s__3) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__3);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_12);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_split); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_12 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (likely(__pyx_t_12)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_12);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(function);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_t_13, function);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_12, __pyx_kp_s__4) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_kp_s__4);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_src_sig, __pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_15 = PyList_GET_SIZE(__pyx_v_dest_sig); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_16 = __pyx_t_15;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = __pyx_t_17;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = PyList_GET_ITEM(__pyx_v_dest_sig, __pyx_v_i);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = (__pyx_v_dst_type != Py_None);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_t_3 != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_src_sig, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_13 = PyObject_RichCompare(__pyx_t_1, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_v_match_found = 1;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L34;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          __pyx_v_match_found = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L32_break;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L34:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L32_break:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (__pyx_v_match_found != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_18 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_18 == ((int)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (PyList_GET_SIZE(__pyx_v_candidates) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = ((!__pyx_t_2) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = ((__pyx_t_9 > 1) != 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_3) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_v_signatures == Py_None)) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 1106 "ms_deisotope/_c/peak_set.pyx"
      __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_t_6;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_6 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_12);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_13);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_14);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_dest_sig);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_ndarray);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_arg);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_dtype);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_arg_base);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_candidates);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_sig);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_src_sig);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_dst_type);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_kwargs);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_8peak_set_17decode_envelopes = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
{"__pyx_fuse_0decode_envelopes", (PyCFunction)__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_fuse_0__pyx_pw_12ms_deisotope_2_c_8peak_set_17decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("decode_envelopes (wrapper)", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array), __pyx_ptype_5numpy_ndarray, 1, "array", 0))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_r = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(__pyx_self, ((PyArrayObject *)__pyx_v_array));

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_16decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array) {
  PyObject *__pyx_v_envelope_list = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_current_envelope = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_converted = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope_obj = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  float __pyx_v_a
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  float __pyx_v_b
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_array
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_4;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_0decode_envelopes", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffer_array.pybuffer.buf = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffer_array.refcount = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.data = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_float, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/peak_set.pyx":1113
 *         size_t i, n
 *         cython.floating a, b
 *     envelope_list = []             # <<<<<<<<<<<<<<
 *     current_envelope = []
 * 
 */

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_envelope_list = ((PyObject*)__pyx_t_1);

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1114
 *         cython.floating a, b
 *     envelope_list = []
 *     current_envelope = []             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_current_envelope = ((PyObject*)__pyx_t_1);

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1116
 *     current_envelope = []
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     n = array.shape[0]
 *     while i < n:
 */

#line 1116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1117
 * 
 *     i = 0
 *     n = array.shape[0]             # <<<<<<<<<<<<<<
 *     while i < n:
 *         a = array[i]
 */

#line 1117 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = (__pyx_v_array->dimensions[0]);

  /* "ms_deisotope/_c/peak_set.pyx":1118
 *     i = 0
 *     n = array.shape[0]
 *     while i < n:             # <<<<<<<<<<<<<<
 *         a = array[i]
 *         b = array[i + 1]
 */

#line 1118 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1118 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_i < __pyx_v_n) != 0);

#line 1118 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1119
 *     n = array.shape[0]
 *     while i < n:
 *         a = array[i]             # <<<<<<<<<<<<<<
 *         b = array[i + 1]
 *         i += 2
 */

#line 1119 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_i;

#line 1119 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_a = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1120
 *     while i < n:
 *         a = array[i]
 *         b = array[i + 1]             # <<<<<<<<<<<<<<
 *         i += 2
 *         if a == 0 and b == 0:
 */

#line 1120 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_i + 1);

#line 1120 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_b = (*__Pyx_BufPtrCContig1d(float *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1121
 *         a = array[i]
 *         b = array[i + 1]
 *         i += 2             # <<<<<<<<<<<<<<
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 */

#line 1121 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 2);

    /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = ((__pyx_v_a == 0.0) != 0);

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_4) {

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = __pyx_t_4;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6_bool_binop_done;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = ((__pyx_v_b == 0.0) != 0);

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_bool_binop_done:;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_v_current_envelope != ((PyObject*)Py_None));

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = (__pyx_t_2 != 0);

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_4) {

        /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = (PyList_GET_SIZE(__pyx_v_current_envelope) != 0);

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_4) {

          /* "ms_deisotope/_c/peak_set.pyx":1125
 *             if current_envelope is not None:
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 */

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1125, __pyx_L1_error)

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1126
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []
 */

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1127
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *                 current_envelope = []
 *         else:
 */

#line 1127 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1127, __pyx_L1_error)

          /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1128
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []             # <<<<<<<<<<<<<<
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 */

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_v_current_envelope, ((PyObject*)__pyx_t_1));

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = 0;

        /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1130
 *                 current_envelope = []
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))             # <<<<<<<<<<<<<<
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 */

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_a, __pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1130, __pyx_L1_error)

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_current_envelope, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1130, __pyx_L1_error)

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1130 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1131
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 */

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1132
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *     envelope_list.append(envelope_obj)
 *     return envelope_list
 */

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1133
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *     return envelope_list
 * 
 */

#line 1133 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1133, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1134
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 *     return envelope_list             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_envelope_list);

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_envelope_list;

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_declare

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_assign

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.decode_envelopes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L2:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_envelope_list);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_current_envelope);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_converted);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_envelope_obj);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_8peak_set_19decode_envelopes = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
{"__pyx_fuse_1decode_envelopes", (PyCFunction)__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_14decode_envelopes};

#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_fuse_1__pyx_pw_12ms_deisotope_2_c_8peak_set_19decode_envelopes(PyObject *__pyx_self, PyObject *__pyx_v_array) {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("decode_envelopes (wrapper)", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_array), __pyx_ptype_5numpy_ndarray, 1, "array", 0))) __PYX_ERR(0, 1106, __pyx_L1_error)
  __pyx_r = 
#line 1106 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(__pyx_self, ((PyArrayObject *)__pyx_v_array));

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_18decode_envelopes(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_array) {
  PyObject *__pyx_v_envelope_list = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_current_envelope = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  PyObject *__pyx_v_converted = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_v_envelope_obj = 0
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_a
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_b
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_array
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;
  __Pyx_Buffer __pyx_pybuffer_array
#line 1106 "ms_deisotope/_c/peak_set.pyx"
;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_4;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_5;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__pyx_fuse_1decode_envelopes", 0);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffer_array.pybuffer.buf = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffer_array.refcount = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.data = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.rcbuffer = &__pyx_pybuffer_array;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_BufFmt_StackElem __pyx_stack[1];

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_array.rcbuffer->pybuffer, (PyObject*)__pyx_v_array, &__Pyx_TypeInfo_double, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pybuffernd_array.diminfo[0].strides = __pyx_pybuffernd_array.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_array.diminfo[0].shape = __pyx_pybuffernd_array.rcbuffer->pybuffer.shape[0];

  /* "ms_deisotope/_c/peak_set.pyx":1113
 *         size_t i, n
 *         cython.floating a, b
 *     envelope_list = []             # <<<<<<<<<<<<<<
 *     current_envelope = []
 * 
 */

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_envelope_list = ((PyObject*)__pyx_t_1);

#line 1113 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1114
 *         cython.floating a, b
 *     envelope_list = []
 *     current_envelope = []             # <<<<<<<<<<<<<<
 * 
 *     i = 0
 */

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_current_envelope = ((PyObject*)__pyx_t_1);

#line 1114 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1116
 *     current_envelope = []
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     n = array.shape[0]
 *     while i < n:
 */

#line 1116 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1117
 * 
 *     i = 0
 *     n = array.shape[0]             # <<<<<<<<<<<<<<
 *     while i < n:
 *         a = array[i]
 */

#line 1117 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = (__pyx_v_array->dimensions[0]);

  /* "ms_deisotope/_c/peak_set.pyx":1118
 *     i = 0
 *     n = array.shape[0]
 *     while i < n:             # <<<<<<<<<<<<<<
 *         a = array[i]
 *         b = array[i + 1]
 */

#line 1118 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1118 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_i < __pyx_v_n) != 0);

#line 1118 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_2) break;

    /* "ms_deisotope/_c/peak_set.pyx":1119
 *     n = array.shape[0]
 *     while i < n:
 *         a = array[i]             # <<<<<<<<<<<<<<
 *         b = array[i + 1]
 *         i += 2
 */

#line 1119 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = __pyx_v_i;

#line 1119 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_a = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1120
 *     while i < n:
 *         a = array[i]
 *         b = array[i + 1]             # <<<<<<<<<<<<<<
 *         i += 2
 *         if a == 0 and b == 0:
 */

#line 1120 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = (__pyx_v_i + 1);

#line 1120 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_b = (*__Pyx_BufPtrCContig1d(double *, __pyx_pybuffernd_array.rcbuffer->pybuffer.buf, __pyx_t_3, __pyx_pybuffernd_array.diminfo[0].strides));

    /* "ms_deisotope/_c/peak_set.pyx":1121
 *         a = array[i]
 *         b = array[i + 1]
 *         i += 2             # <<<<<<<<<<<<<<
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 */

#line 1121 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_i + 2);

    /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = ((__pyx_v_a == 0.0) != 0);

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_4) {

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = __pyx_t_4;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6_bool_binop_done;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_4 = ((__pyx_v_b == 0.0) != 0);

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = __pyx_t_4;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6_bool_binop_done:;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_v_current_envelope != ((PyObject*)Py_None));

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = (__pyx_t_2 != 0);

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_4) {

        /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = (PyList_GET_SIZE(__pyx_v_current_envelope) != 0);

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_4) {

          /* "ms_deisotope/_c/peak_set.pyx":1125
 *             if current_envelope is not None:
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 */

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1125, __pyx_L1_error)

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));

#line 1125 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1126
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []
 */

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1126, __pyx_L1_error)

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_1);

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));

#line 1126 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1127
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *                 current_envelope = []
 *         else:
 */

#line 1127 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1127, __pyx_L1_error)

          /* "ms_deisotope/_c/peak_set.pyx":1124
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:
 *                 if current_envelope:             # <<<<<<<<<<<<<<
 *                     converted = tuple(current_envelope)
 *                     envelope_obj = Envelope._create(converted)
 */

#line 1124 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1128
 *                     envelope_obj = Envelope._create(converted)
 *                     envelope_list.append(envelope_obj)
 *                 current_envelope = []             # <<<<<<<<<<<<<<
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 */

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1128, __pyx_L1_error)

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_1);

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF_SET(__pyx_v_current_envelope, ((PyObject*)__pyx_t_1));

#line 1128 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = 0;

        /* "ms_deisotope/_c/peak_set.pyx":1123
 *         i += 2
 *         if a == 0 and b == 0:
 *             if current_envelope is not None:             # <<<<<<<<<<<<<<
 *                 if current_envelope:
 *                     converted = tuple(current_envelope)
 */

#line 1123 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1122
 *         b = array[i + 1]
 *         i += 2
 *         if a == 0 and b == 0:             # <<<<<<<<<<<<<<
 *             if current_envelope is not None:
 *                 if current_envelope:
 */

#line 1122 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1122 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1130
 *                 current_envelope = []
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))             # <<<<<<<<<<<<<<
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 */

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create(__pyx_v_a, __pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1130, __pyx_L1_error)

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_current_envelope, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1130, __pyx_L1_error)

#line 1130 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1130 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1130 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1131
 *         else:
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)             # <<<<<<<<<<<<<<
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 */

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = PyList_AsTuple(__pyx_v_current_envelope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF_SET(__pyx_v_converted, ((PyObject*)__pyx_t_1));

#line 1131 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1132
 *             current_envelope.append(EnvelopePair._create(a, b))
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)             # <<<<<<<<<<<<<<
 *     envelope_list.append(envelope_obj)
 *     return envelope_list
 */

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((PyObject *)__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create(__pyx_v_converted)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1132, __pyx_L1_error)

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF_SET(__pyx_v_envelope_obj, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)__pyx_t_1));

#line 1132 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1133
 *     converted = tuple(current_envelope)
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)             # <<<<<<<<<<<<<<
 *     return envelope_list
 * 
 */

#line 1133 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_envelope_list, ((PyObject *)__pyx_v_envelope_obj)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1133, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1134
 *     envelope_obj = Envelope._create(converted)
 *     envelope_list.append(envelope_obj)
 *     return envelope_list             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_envelope_list);

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_envelope_list;

#line 1134 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"


#line 1106 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_declare

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_PyThreadState_assign

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.decode_envelopes", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L2;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_array.rcbuffer->pybuffer);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L2:;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_envelope_list);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_current_envelope);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_converted);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_envelope_obj);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1140
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 */

#line 1140 "ms_deisotope/_c/peak_set.pyx"


#line 1140 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1140 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1140 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peaks = 0
#line 1140 "ms_deisotope/_c/peak_set.pyx"
;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peaks,0};

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[1] = {0};

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  1: 
#line 1140 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 1140 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 1140 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peaks)) != 0)) kw_args--;
        else 
#line 1140 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 1140, __pyx_L3_error)

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5_argtuple_error;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1140 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_peaks = values[0];

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 1140 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1140, __pyx_L3_error)

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  return -1;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1140 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_peaks);

#line 1140 "ms_deisotope/_c/peak_set.pyx"


#line 1140 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
}

#line 1140 "ms_deisotope/_c/peak_set.pyx"


#line 1140 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed___init__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_peaks) {

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1141
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL             # <<<<<<<<<<<<<<
 *         self.mz_array = NULL
 *         self.interval_index = NULL
 */

#line 1141 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->neutral_mass_array = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1142
 *     def __init__(self, peaks):
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL             # <<<<<<<<<<<<<<
 *         self.interval_index = NULL
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 */

#line 1142 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->mz_array = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1143
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 *         self.interval_index = NULL             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 */

#line 1143 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->interval_index = NULL;

  /* "ms_deisotope/_c/peak_set.pyx":1144
 *         self.mz_array = NULL
 *         self.interval_index = NULL
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1144, __pyx_L1_error)

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 1144 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 1144 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_peaks) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_peaks);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1144 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1140
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 *     def __init__(self, peaks):             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = NULL
 *         self.mz_array = NULL
 */

#line 1140 "ms_deisotope/_c/peak_set.pyx"


#line 1140 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = -1;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1140 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1140 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1146
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 * 
 */

#line 1146 "ms_deisotope/_c/peak_set.pyx"


#line 1146 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1146 "ms_deisotope/_c/peak_set.pyx"
static void __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 1146 "ms_deisotope/_c/peak_set.pyx"
static void __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(PyObject *__pyx_v_self) {

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

#line 1146 "ms_deisotope/_c/peak_set.pyx"


#line 1146 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1146 "ms_deisotope/_c/peak_set.pyx"
}

#line 1146 "ms_deisotope/_c/peak_set.pyx"


#line 1146 "ms_deisotope/_c/peak_set.pyx"
static void __pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_2__dealloc__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1147
 * 
 *     def __dealloc__(self):
 *         self._release_buffers()             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_release_buffers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1147, __pyx_L1_error)

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 1147 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 1147 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1147, __pyx_L1_error)

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1147 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1146
 *         super(DeconvolutedPeakSetIndexed, self).__init__(peaks)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 * 
 */

#line 1146 "ms_deisotope/_c/peak_set.pyx"


#line 1146 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1146 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1146 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1149
 *         self._release_buffers()
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 */

#line 1149 "ms_deisotope/_c/peak_set.pyx"


#line 1149 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1149 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1149 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__[] = "DeconvolutedPeakSetIndexed.__reduce__(self)";

#line 1149 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = 
#line 1149 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

#line 1149 "ms_deisotope/_c/peak_set.pyx"


#line 1149 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
}

#line 1149 "ms_deisotope/_c/peak_set.pyx"


#line 1149 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1150
 * 
 *     def __reduce__(self):
 *         return self.__class__, (self.peaks, ), self.__getstate__()             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self):
 */

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1150, __pyx_L1_error)

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1150, __pyx_L1_error)

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_self->__pyx_base.peaks);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.peaks);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->__pyx_base.peaks);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1150, __pyx_L1_error)

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = NULL;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {

#line 1150 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_5)) {

#line 1150 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_5);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_4, function);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1150, __pyx_L1_error)

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1150, __pyx_L1_error)

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_4;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 1150 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1149
 *         self._release_buffers()
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 */

#line 1149 "ms_deisotope/_c/peak_set.pyx"


#line 1149 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1149 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1149 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1152
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         d = {"indexed": self.indexed}
 *         return d
 */

#line 1152 "ms_deisotope/_c/peak_set.pyx"


#line 1152 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1152 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1152 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__[] = "DeconvolutedPeakSetIndexed.__getstate__(self)";

#line 1152 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = 
#line 1152 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

#line 1152 "ms_deisotope/_c/peak_set.pyx"


#line 1152 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
}

#line 1152 "ms_deisotope/_c/peak_set.pyx"


#line 1152 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  PyObject *__pyx_v_d = NULL
#line 1152 "ms_deisotope/_c/peak_set.pyx"
;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1153
 * 
 *     def __getstate__(self):
 *         d = {"indexed": self.indexed}             # <<<<<<<<<<<<<<
 *         return d
 * 
 */

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1153, __pyx_L1_error)

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.indexed); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1153, __pyx_L1_error)

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_indexed, __pyx_t_2) < 0) __PYX_ERR(0, 1153, __pyx_L1_error)

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_d = ((PyObject*)__pyx_t_1);

#line 1153 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1154
 *     def __getstate__(self):
 *         d = {"indexed": self.indexed}
 *         return d             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, d):
 */

#line 1154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1154 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_v_d);

#line 1154 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_d;

#line 1154 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1152
 *         return self.__class__, (self.peaks, ), self.__getstate__()
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         d = {"indexed": self.indexed}
 *         return d
 */

#line 1152 "ms_deisotope/_c/peak_set.pyx"


#line 1152 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_v_d);

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1152 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1152 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1156
 *         return d
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         if d.get("indexed", False):
 *             self.reindex()
 */

#line 1156 "ms_deisotope/_c/peak_set.pyx"


#line 1156 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1156 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d); /*proto*/

#line 1156 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__[] = "DeconvolutedPeakSetIndexed.__setstate__(self, d)";

#line 1156 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d) {

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = 
#line 1156 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((PyObject *)__pyx_v_d));

#line 1156 "ms_deisotope/_c/peak_set.pyx"


#line 1156 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
}

#line 1156 "ms_deisotope/_c/peak_set.pyx"


#line 1156 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_d) {

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_3;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1157, __pyx_L1_error)

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1157, __pyx_L1_error)

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_3) {

    /* "ms_deisotope/_c/peak_set.pyx":1158
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):
 *             self.reindex()             # <<<<<<<<<<<<<<
 * 
 *     def _release_buffers(self):
 */

#line 1158 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1158, __pyx_L1_error)

#line 1158 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 1158 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1156
 *         return d
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         if d.get("indexed", False):
 *             self.reindex()
 */

#line 1156 "ms_deisotope/_c/peak_set.pyx"


#line 1156 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1156 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1156 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1160
 *             self.reindex()
 * 
 *     def _release_buffers(self):             # <<<<<<<<<<<<<<
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 */

#line 1160 "ms_deisotope/_c/peak_set.pyx"


#line 1160 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1160 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1160 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers[] = "DeconvolutedPeakSetIndexed._release_buffers(self)";

#line 1160 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_release_buffers (wrapper)", 0);
  __pyx_r = 
#line 1160 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

#line 1160 "ms_deisotope/_c/peak_set.pyx"


#line 1160 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1160 "ms_deisotope/_c/peak_set.pyx"
}

#line 1160 "ms_deisotope/_c/peak_set.pyx"


#line 1160 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_release_buffers", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1161
 * 
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 */

#line 1161 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_self->neutral_mass_array != NULL) != 0);

#line 1161 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1162
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:
 */

#line 1162 "ms_deisotope/_c/peak_set.pyx"
    free(__pyx_v_self->neutral_mass_array);

    /* "ms_deisotope/_c/peak_set.pyx":1163
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL             # <<<<<<<<<<<<<<
 *         if self.mz_array != NULL:
 *             free(self.mz_array)
 */

#line 1163 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->neutral_mass_array = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1161
 * 
 *     def _release_buffers(self):
 *         if self.neutral_mass_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 */

#line 1161 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1164
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.mz_array)
 *             self.mz_array = NULL
 */

#line 1164 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_self->mz_array != NULL) != 0);

#line 1164 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1165
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:
 *             free(self.mz_array)             # <<<<<<<<<<<<<<
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:
 */

#line 1165 "ms_deisotope/_c/peak_set.pyx"
    free(__pyx_v_self->mz_array);

    /* "ms_deisotope/_c/peak_set.pyx":1166
 *         if self.mz_array != NULL:
 *             free(self.mz_array)
 *             self.mz_array = NULL             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)
 */

#line 1166 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->mz_array = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1164
 *             free(self.neutral_mass_array)
 *             self.neutral_mass_array = NULL
 *         if self.mz_array != NULL:             # <<<<<<<<<<<<<<
 *             free(self.mz_array)
 *             self.mz_array = NULL
 */

#line 1164 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1167
 *             free(self.mz_array)
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL
 */

#line 1167 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_self->interval_index != NULL) != 0);

#line 1167 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1168
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)             # <<<<<<<<<<<<<<
 *             self.interval_index = NULL
 * 
 */

#line 1168 "ms_deisotope/_c/peak_set.pyx"
    __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_self->interval_index);

    /* "ms_deisotope/_c/peak_set.pyx":1169
 *         if self.interval_index != NULL:
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL             # <<<<<<<<<<<<<<
 * 
 *     def set_interval_index_size(self, index_size):
 */

#line 1169 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_self->interval_index = NULL;

    /* "ms_deisotope/_c/peak_set.pyx":1167
 *             free(self.mz_array)
 *             self.mz_array = NULL
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             free_index_list(self.interval_index)
 *             self.interval_index = NULL
 */

#line 1167 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1160
 *             self.reindex()
 * 
 *     def _release_buffers(self):             # <<<<<<<<<<<<<<
 *         if self.neutral_mass_array != NULL:
 *             free(self.neutral_mass_array)
 */

#line 1160 "ms_deisotope/_c/peak_set.pyx"


#line 1160 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1160 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1160 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1171
 *             self.interval_index = NULL
 * 
 *     def set_interval_index_size(self, index_size):             # <<<<<<<<<<<<<<
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 */

#line 1171 "ms_deisotope/_c/peak_set.pyx"


#line 1171 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1171 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size(PyObject *__pyx_v_self, PyObject *__pyx_v_index_size); /*proto*/

#line 1171 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size[] = "DeconvolutedPeakSetIndexed.set_interval_index_size(self, index_size)";

#line 1171 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size(PyObject *__pyx_v_self, PyObject *__pyx_v_index_size) {

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("set_interval_index_size (wrapper)", 0);
  __pyx_r = 
#line 1171 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((PyObject *)__pyx_v_index_size));

#line 1171 "ms_deisotope/_c/peak_set.pyx"


#line 1171 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
}

#line 1171 "ms_deisotope/_c/peak_set.pyx"


#line 1171 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, PyObject *__pyx_v_index_size) {

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("set_interval_index_size", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1173
 *     def set_interval_index_size(self, index_size):
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size             # <<<<<<<<<<<<<<
 *         self.reindex()
 * 
 */

#line 1173 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_index_size); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L1_error)

#line 1173 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1174
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 *         self.reindex()             # <<<<<<<<<<<<<<
 * 
 *     cdef void _build_index_arrays(self):
 */

#line 1174 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1174, __pyx_L1_error)

#line 1174 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1174 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1171
 *             self.interval_index = NULL
 * 
 *     def set_interval_index_size(self, index_size):             # <<<<<<<<<<<<<<
 *         global INTERVAL_INDEX_SIZE
 *         INTERVAL_INDEX_SIZE = index_size
 */

#line 1171 "ms_deisotope/_c/peak_set.pyx"


#line 1171 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.set_interval_index_size", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1171 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1171 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1176
 *         self.reindex()
 * 
 *     cdef void _build_index_arrays(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

#line 1176 "ms_deisotope/_c/peak_set.pyx"


#line 1176 "ms_deisotope/_c/peak_set.pyx"
static void __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {
  size_t __pyx_v_i
#line 1176 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1176 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 1176 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_interval_index
#line 1176 "ms_deisotope/_c/peak_set.pyx"
;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_7;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_8;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_9;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_build_index_arrays", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1180
 *             size_t i, n
 *             DeconvolutedPeak peak
 *         n = PyTuple_GET_SIZE(self.peaks)             # <<<<<<<<<<<<<<
 *         self._size = n
 *         self._release_buffers()
 */

#line 1180 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->__pyx_base.peaks;

#line 1180 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 1180 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = PyTuple_GET_SIZE(__pyx_t_1);

#line 1180 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1181
 *             DeconvolutedPeak peak
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         self._size = n             # <<<<<<<<<<<<<<
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 */

#line 1181 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->_size = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1182
 *         n = PyTuple_GET_SIZE(self.peaks)
 *         self._size = n
 *         self._release_buffers()             # <<<<<<<<<<<<<<
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 */

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_release_buffers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1182, __pyx_L1_error)

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 1182 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 1182 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1182, __pyx_L1_error)

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1182 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1183
 *         self._size = n
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 * 
 */

#line 1183 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->neutral_mass_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1184
 *         self._release_buffers()
 *         self.neutral_mass_array = <double*>malloc(sizeof(double) * n)
 *         self.mz_array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n):
 */

#line 1184 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_self->mz_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1186
 *         self.mz_array = <double*>malloc(sizeof(double) * n)
 * 
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass
 */

#line 1186 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_v_n;

#line 1186 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_t_4;

#line 1186 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1186 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_6;

    /* "ms_deisotope/_c/peak_set.pyx":1187
 * 
 *         for i in range(n):
 *             peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array[i] = peak.neutral_mass
 *             self.mz_array[peak._index.mz] = peak.mz
 */

#line 1187 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.getitem(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1187, __pyx_L1_error)

#line 1187 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1187 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_1));

#line 1187 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1188
 *         for i in range(n):
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass             # <<<<<<<<<<<<<<
 *             self.mz_array[peak._index.mz] = peak.mz
 * 
 */

#line 1188 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_7 = __pyx_v_peak->neutral_mass;

#line 1188 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_self->neutral_mass_array[__pyx_v_i]) = __pyx_t_7;

    /* "ms_deisotope/_c/peak_set.pyx":1189
 *             peak = self.getitem(i)
 *             self.neutral_mass_array[i] = peak.neutral_mass
 *             self.mz_array[peak._index.mz] = peak.mz             # <<<<<<<<<<<<<<
 * 
 *         # The interpolating interval index is just a bit slower than
 */

#line 1189 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_7 = __pyx_v_peak->__pyx_base.mz;

#line 1189 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_self->mz_array[__pyx_v_peak->_index->mz]) = __pyx_t_7;

#line 1189 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1193
 *         # The interpolating interval index is just a bit slower than
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:             # <<<<<<<<<<<<<<
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 */

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = ((__pyx_v_n > 2) != 0);

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_9) {

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  } else {

#line 1193 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = __pyx_t_9;

#line 1193 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6_bool_binop_done;

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = ((__pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE > 0) != 0);

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_8 = __pyx_t_9;

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6_bool_binop_done:;

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_8) {

    /* "ms_deisotope/_c/peak_set.pyx":1194
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 */

#line 1194 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = ((__pyx_v_self->interval_index != NULL) != 0);

#line 1194 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_8) {

      /* "ms_deisotope/_c/peak_set.pyx":1195
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)             # <<<<<<<<<<<<<<
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 */

#line 1195 "ms_deisotope/_c/peak_set.pyx"
      __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_self->interval_index);

      /* "ms_deisotope/_c/peak_set.pyx":1196
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL             # <<<<<<<<<<<<<<
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 */

#line 1196 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_self->interval_index = NULL;

      /* "ms_deisotope/_c/peak_set.pyx":1194
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:
 *             if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 */

#line 1194 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1197
 *                 free_index_list(self.interval_index)
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))             # <<<<<<<<<<<<<<
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:
 */

#line 1197 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_interval_index = ((struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *)malloc((sizeof(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list))));

    /* "ms_deisotope/_c/peak_set.pyx":1198
 *                 self.interval_index = NULL
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)             # <<<<<<<<<<<<<<
 *             if check_index(interval_index) != 0:
 *                 free_index_list(interval_index)
 */

#line 1198 "ms_deisotope/_c/peak_set.pyx"
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_interval_index, __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE));

    /* "ms_deisotope/_c/peak_set.pyx":1199
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:             # <<<<<<<<<<<<<<
 *                 free_index_list(interval_index)
 *             else:
 */

#line 1199 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_8 = ((__pyx_f_12ms_deisotope_2_c_8peak_set_check_index(__pyx_v_interval_index) != 0) != 0);

#line 1199 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_8) {

      /* "ms_deisotope/_c/peak_set.pyx":1200
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:
 *                 free_index_list(interval_index)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.interval_index = interval_index
 */

#line 1200 "ms_deisotope/_c/peak_set.pyx"
      __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(__pyx_v_interval_index);

      /* "ms_deisotope/_c/peak_set.pyx":1199
 *             interval_index = <index_list*>malloc(sizeof(index_list))
 *             build_interval_index(self, interval_index, INTERVAL_INDEX_SIZE)
 *             if check_index(interval_index) != 0:             # <<<<<<<<<<<<<<
 *                 free_index_list(interval_index)
 *             else:
 */

#line 1199 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L9;

#line 1199 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1202
 *                 free_index_list(interval_index)
 *             else:
 *                 self.interval_index = interval_index             # <<<<<<<<<<<<<<
 * 
 *     cpdef reindex(self):
 */

#line 1202 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1202 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_self->interval_index = __pyx_v_interval_index;

#line 1202 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1202 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L9:;

    /* "ms_deisotope/_c/peak_set.pyx":1193
 *         # The interpolating interval index is just a bit slower than
 *         # the normal binary search, likely due to cache friendliness.
 *         if n > 2 and INTERVAL_INDEX_SIZE > 0:             # <<<<<<<<<<<<<<
 *             if self.interval_index != NULL:
 *                 free_index_list(self.interval_index)
 */

#line 1193 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1176
 *         self.reindex()
 * 
 *     cdef void _build_index_arrays(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

#line 1176 "ms_deisotope/_c/peak_set.pyx"


#line 1176 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._build_index_arrays", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 1176 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1176 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1204
 *                 self.interval_index = interval_index
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 */

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1204 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, int __pyx_skip_dispatch) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex", 0);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1204 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_reindex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex)) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; 
#line 1204 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_4 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_4)) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_4);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_3, function);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1204, __pyx_L1_error)

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_2);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = __pyx_t_2;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1204 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1204 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1205
 * 
 *     cpdef reindex(self):
 *         super(DeconvolutedPeakSetIndexed, self).reindex()             # <<<<<<<<<<<<<<
 *         self._build_index_arrays()
 *         return self
 */

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1205, __pyx_L1_error)

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1205, __pyx_L1_error)

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_reindex); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1205, __pyx_L1_error)

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = NULL;

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {

#line 1205 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
    if (likely(__pyx_t_3)) {

#line 1205 "ms_deisotope/_c/peak_set.pyx"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(function);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF_SET(__pyx_t_2, function);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1205, __pyx_L1_error)

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1205 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1206
 *     cpdef reindex(self):
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()             # <<<<<<<<<<<<<<
 *         return self
 * 
 */

#line 1206 "ms_deisotope/_c/peak_set.pyx"
  ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->_build_index_arrays(__pyx_v_self);

  /* "ms_deisotope/_c/peak_set.pyx":1207
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):
 */

#line 1207 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1207 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 1207 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject *)__pyx_v_self);

#line 1207 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1204
 *                 self.interval_index = interval_index
 * 
 *     cpdef reindex(self):             # <<<<<<<<<<<<<<
 *         super(DeconvolutedPeakSetIndexed, self).reindex()
 *         self._build_index_arrays()
 */

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
}

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1204 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex[] = "DeconvolutedPeakSetIndexed.reindex(self)";

#line 1204 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex (wrapper)", 0);
  __pyx_r = 
#line 1204 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self));

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
}

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self) {

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("reindex", 0);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1204 "ms_deisotope/_c/peak_set.pyx"


#line 1204 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.reindex", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1204 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1204 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1209 "ms_deisotope/_c/peak_set.pyx"


#line 1209 "ms_deisotope/_c/peak_set.pyx"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak *__pyx_optional_args) {

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_v_error_tolerance = ((double)1e-5);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_v_use_mz = ((int)0);
  int __pyx_v_status
#line 1209 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1209 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1209 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_s
#line 1209 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 1209 "ms_deisotope/_c/peak_set.pyx"
;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_r = NULL;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_5;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_has_peak", 0);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 1209 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 1209 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_error_tolerance = __pyx_optional_args->error_tolerance;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_optional_args->__pyx_n > 1) {

#line 1209 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_use_mz = __pyx_optional_args->use_mz;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1209 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1233
 *             DeconvolutedPeak peak
 * 
 *         n = self._size             # <<<<<<<<<<<<<<
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 */

#line 1233 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->_size;

#line 1233 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1234
 * 
 *         n = self._size
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 */

#line 1234 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_v_use_mz != 0);

#line 1234 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1235
 *         n = self._size
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)             # <<<<<<<<<<<<<<
 *             if status != 0:
 *                 return None
 */

#line 1235 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(__pyx_v_self->mz_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, (&__pyx_v_i));

    /* "ms_deisotope/_c/peak_set.pyx":1236
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */

#line 1236 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((__pyx_v_status != 0) != 0);

#line 1236 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_2) {

      /* "ms_deisotope/_c/peak_set.pyx":1237
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 *                 return None             # <<<<<<<<<<<<<<
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 */

#line 1237 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1237 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1237 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1236
 *         if use_mz:
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:             # <<<<<<<<<<<<<<
 *                 return None
 *             else:
 */

#line 1236 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1239
 *                 return None
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return None
 */

#line 1239 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_3 = __pyx_v_self->__pyx_base._mz_ordered;

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(__pyx_t_3);

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_v_i);

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_INCREF(((PyObject *)((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4)));

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_4);

#line 1239 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

#line 1239 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1234
 * 
 *         n = self._size
 *         if use_mz:             # <<<<<<<<<<<<<<
 *             status = _binary_search(self.mz_array, neutral_mass, error_tolerance, n, &i)
 *             if status != 0:
 */

#line 1234 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1240
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return None
 *         if self.interval_index != NULL:
 */

#line 1240 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((__pyx_v_n == 0) != 0);

#line 1240 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1241
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:
 *             return None             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 */

#line 1241 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1241 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1241 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1240
 *             else:
 *                 return <DeconvolutedPeak>PyTuple_GET_ITEM(self._mz_ordered, i)
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return None
 *         if self.interval_index != NULL:
 */

#line 1240 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1242
 *         if n == 0:
 *             return None
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 */

#line 1242 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((__pyx_v_self->interval_index != NULL) != 0);

#line 1242 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1243
 *             return None
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)             # <<<<<<<<<<<<<<
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 *         else:
 */

#line 1243 "ms_deisotope/_c/peak_set.pyx"
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_neutral_mass, (&__pyx_v_s), (&__pyx_v_n)));

    /* "ms_deisotope/_c/peak_set.pyx":1244
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)             # <<<<<<<<<<<<<<
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 */

#line 1244 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, __pyx_v_s, (&__pyx_v_i));

    /* "ms_deisotope/_c/peak_set.pyx":1242
 *         if n == 0:
 *             return None
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 */

#line 1242 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6;

#line 1242 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1246
 *             status = _binary_search_with_hint(self.neutral_mass_array, neutral_mass, error_tolerance, n, s, &i)
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)             # <<<<<<<<<<<<<<
 *         if status != 0:
 *             return None
 */

#line 1246 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1246 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_error_tolerance, __pyx_v_n, (&__pyx_v_i));

#line 1246 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1246 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6:;

  /* "ms_deisotope/_c/peak_set.pyx":1247
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return None
 *         peak = self.getitem(i)
 */

#line 1247 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((__pyx_v_status != 0) != 0);

#line 1247 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1248
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:
 *             return None             # <<<<<<<<<<<<<<
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 */

#line 1248 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1248 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1248 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1247
 *         else:
 *             status = _binary_search(self.neutral_mass_array, neutral_mass, error_tolerance, n, &i)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return None
 *         peak = self.getitem(i)
 */

#line 1247 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1249
 *         if status != 0:
 *             return None
 *         peak = self.getitem(i)             # <<<<<<<<<<<<<<
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 *             return peak
 */

#line 1249 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.getitem(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1249, __pyx_L1_error)

#line 1249 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1249 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_peak = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_3);

#line 1249 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1250
 *             return None
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:             # <<<<<<<<<<<<<<
 *             return peak
 *         else:
 */

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = (__pyx_v_peak->neutral_mass - __pyx_v_neutral_mass);

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_v_neutral_mass == 0)) {

#line 1250 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");

#line 1250 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1250, __pyx_L1_error)

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = ((fabs((__pyx_t_5 / __pyx_v_neutral_mass)) < __pyx_v_error_tolerance) != 0);

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_2) {

    /* "ms_deisotope/_c/peak_set.pyx":1251
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:
 *             return peak             # <<<<<<<<<<<<<<
 *         else:
 *             return None
 */

#line 1251 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1251 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(((PyObject *)__pyx_v_peak));

#line 1251 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_v_peak;

#line 1251 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1250
 *             return None
 *         peak = self.getitem(i)
 *         if abs((peak.neutral_mass - neutral_mass) / neutral_mass) < error_tolerance:             # <<<<<<<<<<<<<<
 *             return peak
 *         else:
 */

#line 1250 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1253
 *             return peak
 *         else:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):
 */

#line 1253 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1253 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 1253 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)Py_None); __Pyx_INCREF(Py_None);

#line 1253 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 1253 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1209
 *         return self
 * 
 *     cdef DeconvolutedPeak _has_peak(self, double neutral_mass, double error_tolerance=1e-5, bint use_mz=False):             # <<<<<<<<<<<<<<
 *         '''Find the peak that best matches ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1209 "ms_deisotope/_c/peak_set.pyx"


#line 1209 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._has_peak", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1209 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1209 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1255
 *             return None
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1255 "ms_deisotope/_c/peak_set.pyx"


#line 1255 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1255 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1255 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval[] = "DeconvolutedPeakSetIndexed._test_interval(self, double neutral_mass, double tolerance=1e-5)";

#line 1255 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_tolerance
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_test_interval (wrapper)", 0);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_test_interval") < 0)) __PYX_ERR(0, 1255, __pyx_L3_error)

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1255, __pyx_L3_error)

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    if (values[1]) {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1255, __pyx_L3_error)

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1255 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = ((double)1e-5);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_test_interval", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1255, __pyx_L3_error)

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1255 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

#line 1255 "ms_deisotope/_c/peak_set.pyx"


#line 1255 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
}

#line 1255 "ms_deisotope/_c/peak_set.pyx"


#line 1255 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {
  int __pyx_v_status
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_start
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_end
#line 1255 "ms_deisotope/_c/peak_set.pyx"
;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("_test_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1259
 *             int status
 *             size_t n, start, end
 *         n = self._size             # <<<<<<<<<<<<<<
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 */

#line 1259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->_size;

#line 1259 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1260
 *             size_t n, start, end
 *         n = self._size
 *         status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         return status, start, end
 */

#line 1260 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, (&__pyx_v_start), (&__pyx_v_end));

  /* "ms_deisotope/_c/peak_set.pyx":1262
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         return status, start, end             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):
 */

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1262, __pyx_L1_error)

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1262, __pyx_L1_error)

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1262, __pyx_L1_error)

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1262, __pyx_L1_error)

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_5);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_4);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_5;

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = 0;

#line 1262 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1255
 *             return None
 * 
 *     def _test_interval(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1255 "ms_deisotope/_c/peak_set.pyx"


#line 1255 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed._test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1255 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1255 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1264 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for *__pyx_optional_args) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_v_tolerance = ((double)1e-5);
  int __pyx_v_status
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_s
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_start
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_end
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_5 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_6 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_7;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_8 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_9;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_10;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for", 0);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_optional_args) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_optional_args->__pyx_n > 0) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = __pyx_optional_args->tolerance;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  /* Check if called by wrapper */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_skip_dispatch)) ;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  /* Check if overridden in Python */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_all_peaks_for); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_GOTREF(__pyx_t_1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_XDECREF(__pyx_r);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_neutral_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_3);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_tolerance); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_GOTREF(__pyx_t_4);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
__pyx_t_6 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_7 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          if (likely(__pyx_t_6)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(__pyx_t_6);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_INCREF(function);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_DECREF_SET(__pyx_t_5, function);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            __pyx_t_7 = 1;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCALL

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (PyFunction_Check(__pyx_t_5)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        #if CYTHON_FAST_PYCCALL

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        } else

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        #endif

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_8);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_6) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_3);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GIVEREF(__pyx_t_4);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_3 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_4 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = ((PyObject*)__pyx_t_2);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      #endif

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    #endif

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1282
 *             int status
 *             size_t n, s, start, end
 *         n = self._size             # <<<<<<<<<<<<<<
 *         if not self.indexed:
 *             self.reindex()
 */

#line 1282 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_9 = __pyx_v_self->_size;

#line 1282 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_9;

  /* "ms_deisotope/_c/peak_set.pyx":1283
 *             size_t n, s, start, end
 *         n = self._size
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         if n == 0:
 */

#line 1283 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = ((!(__pyx_v_self->__pyx_base.indexed != 0)) != 0);

#line 1283 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1284
 *         n = self._size
 *         if not self.indexed:
 *             self.reindex()             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return ()
 */

#line 1284 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reindex(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1284, __pyx_L1_error)

#line 1284 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_1);

#line 1284 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1283
 *             size_t n, s, start, end
 *         n = self._size
 *         if not self.indexed:             # <<<<<<<<<<<<<<
 *             self.reindex()
 *         if n == 0:
 */

#line 1283 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1285
 *         if not self.indexed:
 *             self.reindex()
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         if self.interval_index != NULL:
 */

#line 1285 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = ((__pyx_v_n == 0) != 0);

#line 1285 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1286
 *             self.reindex()
 *         if n == 0:
 *             return ()             # <<<<<<<<<<<<<<
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 */

#line 1286 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 1286 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_empty_tuple);

#line 1286 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_empty_tuple;

#line 1286 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1285
 *         if not self.indexed:
 *             self.reindex()
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         if self.interval_index != NULL:
 */

#line 1285 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1287
 *         if n == 0:
 *             return ()
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(
 */

#line 1287 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = ((__pyx_v_self->interval_index != NULL) != 0);

#line 1287 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1288
 *             return ()
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)             # <<<<<<<<<<<<<<
 *             status = _binary_search_interval_with_hint(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 */

#line 1288 "ms_deisotope/_c/peak_set.pyx"
    (void)(__pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_neutral_mass, (&__pyx_v_s), (&__pyx_v_n)));

    /* "ms_deisotope/_c/peak_set.pyx":1289
 *         if self.interval_index != NULL:
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(             # <<<<<<<<<<<<<<
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 *         else:
 */

#line 1289 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, __pyx_v_s, (&__pyx_v_start), (&__pyx_v_end));

    /* "ms_deisotope/_c/peak_set.pyx":1287
 *         if n == 0:
 *             return ()
 *         if self.interval_index != NULL:             # <<<<<<<<<<<<<<
 *             find_search_interval(self.interval_index, neutral_mass, &s, &n)
 *             status = _binary_search_interval_with_hint(
 */

#line 1287 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L5;

#line 1287 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1292
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, s, &start, &end)
 *         else:
 *             status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:
 */

#line 1292 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

    /* "ms_deisotope/_c/peak_set.pyx":1293
 *         else:
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)             # <<<<<<<<<<<<<<
 *         if status != 0:
 *             return ()
 */

#line 1293 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, (&__pyx_v_start), (&__pyx_v_end));

#line 1293 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1293 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5:;

  /* "ms_deisotope/_c/peak_set.pyx":1294
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 */

#line 1294 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_10 = ((__pyx_v_status != 0) != 0);

#line 1294 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_10) {

    /* "ms_deisotope/_c/peak_set.pyx":1295
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:
 *             return ()             # <<<<<<<<<<<<<<
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 */

#line 1295 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF(__pyx_r);

#line 1295 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_INCREF(__pyx_empty_tuple);

#line 1295 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = __pyx_empty_tuple;

#line 1295 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1294
 *             status = _binary_search_interval(
 *                 self.neutral_mass_array, neutral_mass, tolerance, n, &start, &end)
 *         if status != 0:             # <<<<<<<<<<<<<<
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 */

#line 1294 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1296
 *         if status != 0:
 *             return ()
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:
 */

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->__pyx_base.peaks;

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(__pyx_t_1);

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = PyTuple_GetSlice(__pyx_t_1, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_INCREF(((PyObject*)__pyx_t_2));

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = ((PyObject*)__pyx_t_2);

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1296 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1264
 *         return status, start, end
 * 
 *     cpdef tuple all_peaks_for(self, double neutral_mass, double tolerance=1e-5):             # <<<<<<<<<<<<<<
 *         '''Find all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.
 * 
 */

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_5);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_6);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_8);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
}

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 1264 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for[] = "DeconvolutedPeakSetIndexed.all_peaks_for(self, double neutral_mass, double tolerance=1e-5) -> tuple\nFind all peaks that match ``neutral_mass`` within ``error_tolerance`` mass accuracy ppm.\n\n        Parameters\n        ----------\n        neutral_mass: double\n            The mass to search for\n        error_tolerance: double\n            The PPM error tolerance to apply\n\n        Returns\n        -------\n        tuple of DeconvolutedPeak\n            The found peaks\n        ";

#line 1264 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_neutral_mass
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_tolerance
#line 1264 "ms_deisotope/_c/peak_set.pyx"
;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for (wrapper)", 0);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_neutral_mass,&__pyx_n_s_tolerance,0};

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    PyObject* values[2] = {0,0};

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    if (unlikely(__pyx_kwds)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      Py_ssize_t kw_args;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {
        case  2: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        case  0: break;
        default: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      switch (pos_args) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        case  0:

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_neutral_mass)) != 0)) kw_args--;
        else 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        case  1:

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (kw_args > 0) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tolerance);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
          if (value) { values[1] = value; kw_args--; }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      if (unlikely(kw_args > 0)) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "all_peaks_for") < 0)) __PYX_ERR(0, 1264, __pyx_L3_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
        break;
        default: 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
goto __pyx_L5_argtuple_error;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_neutral_mass = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_neutral_mass == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L3_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    if (values[1]) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1264, __pyx_L3_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    } else {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_tolerance = ((double)1e-5);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("all_peaks_for", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
__PYX_ERR(0, 1264, __pyx_L3_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1264 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), __pyx_v_neutral_mass, __pyx_v_tolerance);

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
}

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance) {

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for __pyx_t_2;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("all_peaks_for", 0);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.__pyx_n = 1;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2.tolerance = __pyx_v_tolerance;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed->__pyx_base.all_peaks_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_self), __pyx_v_neutral_mass, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1264, __pyx_L1_error)

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

#line 1264 "ms_deisotope/_c/peak_set.pyx"


#line 1264 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.all_peaks_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1264 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1264 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1298
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1298 "ms_deisotope/_c/peak_set.pyx"


#line 1298 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_neutral_mass, double __pyx_v_tolerance, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  int __pyx_v_status
#line 1298 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_n
#line 1298 "ms_deisotope/_c/peak_set.pyx"
;

#line 1298 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1298 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1302
 *             int status
 *             size_t n
 *         n = self._size             # <<<<<<<<<<<<<<
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 */

#line 1302 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_v_self->_size;

#line 1302 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_n = __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1303
 *             size_t n
 *         n = self._size
 *         status = _binary_search_interval(             # <<<<<<<<<<<<<<
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 *         return status
 */

#line 1303 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval(__pyx_v_self->neutral_mass_array, __pyx_v_neutral_mass, __pyx_v_tolerance, __pyx_v_n, __pyx_v_start, __pyx_v_end);

  /* "ms_deisotope/_c/peak_set.pyx":1305
 *         status = _binary_search_interval(
 *             self.neutral_mass_array, neutral_mass, tolerance, n, start, end)
 *         return status             # <<<<<<<<<<<<<<
 * 
 *     def test_interval(self, double value):
 */

#line 1305 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_status;

#line 1305 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1298
 *         return <tuple>PyTuple_GetSlice(self.peaks, start, end)
 * 
 *     cdef int _interval_for(self, double neutral_mass, double tolerance, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1298 "ms_deisotope/_c/peak_set.pyx"


#line 1298 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1298 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1298 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1298 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1307
 *         return status
 * 
 *     def test_interval(self, double value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1307 "ms_deisotope/_c/peak_set.pyx"


#line 1307 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1307 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/

#line 1307 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval[] = "DeconvolutedPeakSetIndexed.test_interval(self, double value)";

#line 1307 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value
#line 1307 "ms_deisotope/_c/peak_set.pyx"
;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("test_interval (wrapper)", 0);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  assert(__pyx_arg_value); {

#line 1307 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1307, __pyx_L3_error)

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1307 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((double)__pyx_v_value));

#line 1307 "ms_deisotope/_c/peak_set.pyx"


#line 1307 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
}

#line 1307 "ms_deisotope/_c/peak_set.pyx"


#line 1307 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value) {
  CYTHON_UNUSED int __pyx_v_status
#line 1307 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_start
#line 1307 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_end
#line 1307 "ms_deisotope/_c/peak_set.pyx"
;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_3 = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_4 = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("test_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1313
 *             size_t i
 * 
 *         status = find_search_interval(self.interval_index, value, &start, &end)             # <<<<<<<<<<<<<<
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 */

#line 1313 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_status = __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(__pyx_v_self->interval_index, __pyx_v_value, (&__pyx_v_start), (&__pyx_v_end));

  /* "ms_deisotope/_c/peak_set.pyx":1314
 * 
 *         status = find_search_interval(self.interval_index, value, &start, &end)
 *         return start, end, interpolate_index(self.interval_index, value)             # <<<<<<<<<<<<<<
 * 
 *     def find_interval_for(self, double value):
 */

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1314, __pyx_L1_error)

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1314, __pyx_L1_error)

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_2);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_self->interval_index, __pyx_v_value)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1314, __pyx_L1_error)

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1314, __pyx_L1_error)

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_4);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = 0;

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_4;

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = 0;

#line 1314 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1307
 *         return status
 * 
 *     def test_interval(self, double value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int status
 */

#line 1307 "ms_deisotope/_c/peak_set.pyx"


#line 1307 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_3);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_4);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.test_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1307 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1307 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1316
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 *     def find_interval_for(self, double value):             # <<<<<<<<<<<<<<
 *         return interpolate_index(self.interval_index, value)
 * 
 */

#line 1316 "ms_deisotope/_c/peak_set.pyx"


#line 1316 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1316 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/

#line 1316 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for[] = "DeconvolutedPeakSetIndexed.find_interval_for(self, double value)";

#line 1316 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value
#line 1316 "ms_deisotope/_c/peak_set.pyx"
;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("find_interval_for (wrapper)", 0);

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  assert(__pyx_arg_value); {

#line 1316 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1316, __pyx_L3_error)

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.find_interval_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1316 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((double)__pyx_v_value));

#line 1316 "ms_deisotope/_c/peak_set.pyx"


#line 1316 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
}

#line 1316 "ms_deisotope/_c/peak_set.pyx"


#line 1316 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, double __pyx_v_value) {

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("find_interval_for", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1317
 * 
 *     def find_interval_for(self, double value):
 *         return interpolate_index(self.interval_index, value)             # <<<<<<<<<<<<<<
 * 
 *     def check_interval(self, size_t i):
 */

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_self->interval_index, __pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1317, __pyx_L1_error)

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1317 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1316
 *         return start, end, interpolate_index(self.interval_index, value)
 * 
 *     def find_interval_for(self, double value):             # <<<<<<<<<<<<<<
 *         return interpolate_index(self.interval_index, value)
 * 
 */

#line 1316 "ms_deisotope/_c/peak_set.pyx"


#line 1316 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.find_interval_for", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1316 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1316 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1319
 *         return interpolate_index(self.interval_index, value)
 * 
 *     def check_interval(self, size_t i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             index_cell cell
 */

#line 1319 "ms_deisotope/_c/peak_set.pyx"


#line 1319 "ms_deisotope/_c/peak_set.pyx"
/* Python wrapper */

#line 1319 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_i); /*proto*/

#line 1319 "ms_deisotope/_c/peak_set.pyx"
static char __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval[] = "DeconvolutedPeakSetIndexed.check_interval(self, size_t i)";

#line 1319 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval(PyObject *__pyx_v_self, PyObject *__pyx_arg_i) {
  size_t __pyx_v_i
#line 1319 "ms_deisotope/_c/peak_set.pyx"
;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = 0;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("check_interval (wrapper)", 0);

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  assert(__pyx_arg_i); {

#line 1319 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __Pyx_PyInt_As_size_t(__pyx_arg_i); if (unlikely((__pyx_v_i == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1319, __pyx_L3_error)

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L4_argument_unpacking_done;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3_error:;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.check_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  return NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1319 "ms_deisotope/_c/peak_set.pyx"
__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)__pyx_v_self), ((size_t)__pyx_v_i));

#line 1319 "ms_deisotope/_c/peak_set.pyx"


#line 1319 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
}

#line 1319 "ms_deisotope/_c/peak_set.pyx"


#line 1319 "ms_deisotope/_c/peak_set.pyx"
static PyObject *__pyx_pf_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *__pyx_v_self, size_t __pyx_v_i) {
  struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell __pyx_v_cell
#line 1319 "ms_deisotope/_c/peak_set.pyx"
;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_r = NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_1 = NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("check_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1322
 *         cdef:
 *             index_cell cell
 *         cell = self.interval_index.index[i]             # <<<<<<<<<<<<<<
 *         return cell
 * 
 */

#line 1322 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_cell = (__pyx_v_self->interval_index->index[__pyx_v_i]);

  /* "ms_deisotope/_c/peak_set.pyx":1323
 *             index_cell cell
 *         cell = self.interval_index.index[i]
 *         return cell             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_r);

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(__pyx_v_cell); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1323, __pyx_L1_error)

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_t_1;

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

#line 1323 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1319
 *         return interpolate_index(self.interval_index, value)
 * 
 *     def check_interval(self, size_t i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             index_cell cell
 */

#line 1319 "ms_deisotope/_c/peak_set.pyx"


#line 1319 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_1);

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_AddTraceback("ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed.check_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = NULL;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGIVEREF(__pyx_r);

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1319 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1319 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1327
 * 
 * 
 * cdef int _binary_search_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     '''Performs a best-matching binary search with a hint at the lower bound as well as the upper bound.
 * 
 */

#line 1327 "ms_deisotope/_c/peak_set.pyx"


#line 1327 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_with_hint(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t __pyx_v_hint, size_t *__pyx_v_out) {
  size_t __pyx_v_lo
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_best_index
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 1327 "ms_deisotope/_c/peak_set.pyx"
;

#line 1327 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1327 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 1327 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":1359
 *         double best_error, abs_error
 * 
 *     lo = hint             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */

#line 1359 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = __pyx_v_hint;

  /* "ms_deisotope/_c/peak_set.pyx":1360
 * 
 *     lo = hint
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 1360 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1362
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 */

#line 1362 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1362 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 1362 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1363
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */

#line 1363 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1364
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */

#line 1364 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":1365
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 1365 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);

#line 1365 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1366
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */

#line 1366 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1367
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */

#line 1367 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":1368
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */

#line 1368 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1369
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 1369 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        } else {

#line 1369 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = __pyx_t_2;

#line 1369 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L8_bool_binop_done;

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = __pyx_t_2;

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L8_bool_binop_done:;

#line 1369 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1370
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 1370 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1371
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 1371 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1372
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 1372 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 1372 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1373
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 1373 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1373 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1374
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 1374 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1375
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 1375 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1373
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 1373 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":1372
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 1372 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L10;

#line 1372 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1377
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             i = 1
 */

#line 1377 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1377 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L7_break;

#line 1377 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1377 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":1378
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */

#line 1378 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 1378 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1378 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1379
 *                     break
 *                 i += 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */

#line 1379 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":1380
 *                 i += 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 1380 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 1380 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 1380 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1381
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 1381 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1382
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 1382 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1383
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 1383 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 1383 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1384
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 1384 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1384 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1385
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 1385 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1386
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 1386 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1384
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 1384 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":1383
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 1383 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L14;

#line 1383 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1388
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             out[0] = best_index
 */

#line 1388 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1388 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L13_break;

#line 1388 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1388 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":1389
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             out[0] = best_index
 *             if best_error == INF:
 */

#line 1389 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 1389 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1389 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1390
 *                     break
 *                 i += 1
 *             out[0] = best_index             # <<<<<<<<<<<<<<
 *             if best_error == INF:
 *                 return 3
 */

#line 1390 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_out[0]) = __pyx_v_best_index;

      /* "ms_deisotope/_c/peak_set.pyx":1391
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */

#line 1391 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_best_error == __pyx_v_12ms_deisotope_2_c_8peak_set_INF) != 0);

#line 1391 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1392
 *             out[0] = best_index
 *             if best_error == INF:
 *                 return 3             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */

#line 1392 "ms_deisotope/_c/peak_set.pyx"
        __pyx_r = 3;

#line 1392 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L0;

        /* "ms_deisotope/_c/peak_set.pyx":1391
 *                 i += 1
 *             out[0] = best_index
 *             if best_error == INF:             # <<<<<<<<<<<<<<
 *                 return 3
 *             return 0
 */

#line 1391 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1393
 *             if best_error == INF:
 *                 return 3
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             out[0] = 0
 */

#line 1393 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 0;

#line 1393 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1365
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 1365 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1394
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */

#line 1394 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 1394 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1395
 *             return 0
 *         elif hi - lo == 1:
 *             out[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */

#line 1395 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_out[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1396
 *         elif hi - lo == 1:
 *             out[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */

#line 1396 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 1;

#line 1396 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1394
 *                 return 3
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             out[0] = 0
 *             return 1
 */

#line 1394 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1397
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1397 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);

#line 1397 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1398
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 1398 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1397
 *             out[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1397 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1397 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1400
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     out[0] = 0
 *     return 2
 */

#line 1400 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1400 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 1400 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1400 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1400 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1401
 *         else:
 *             lo = mid
 *     out[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */

#line 1401 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_out[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1402
 *             lo = mid
 *     out[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1402 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 2;

#line 1402 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1327
 * 
 * 
 * cdef int _binary_search_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* out) nogil:             # <<<<<<<<<<<<<<
 *     '''Performs a best-matching binary search with a hint at the lower bound as well as the upper bound.
 * 
 */

#line 1327 "ms_deisotope/_c/peak_set.pyx"


#line 1327 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1327 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1327 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1327 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1405
 * 
 * 
 * cdef int _binary_search_interval_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1405 "ms_deisotope/_c/peak_set.pyx"


#line 1405 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval_with_hint(double *__pyx_v_array, double __pyx_v_target, double __pyx_v_error_tolerance, size_t __pyx_v_n, size_t __pyx_v_hint, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_lo
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_hi
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_mid
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  CYTHON_UNUSED size_t __pyx_v_best_index
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_found_mass
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_best_error
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_abs_error
#line 1405 "ms_deisotope/_c/peak_set.pyx"
;

#line 1405 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1405 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 1405 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_2;

  /* "ms_deisotope/_c/peak_set.pyx":1412
 *         double best_error, abs_error
 * 
 *     lo = hint             # <<<<<<<<<<<<<<
 *     hi = n
 * 
 */

#line 1412 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_lo = __pyx_v_hint;

  /* "ms_deisotope/_c/peak_set.pyx":1413
 * 
 *     lo = hint
 *     hi = n             # <<<<<<<<<<<<<<
 * 
 *     while hi != lo:
 */

#line 1413 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_hi = __pyx_v_n;

  /* "ms_deisotope/_c/peak_set.pyx":1415
 *     hi = n
 * 
 *     while hi != lo:             # <<<<<<<<<<<<<<
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 */

#line 1415 "ms_deisotope/_c/peak_set.pyx"
  while (1) {

#line 1415 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_hi != __pyx_v_lo) != 0);

#line 1415 "ms_deisotope/_c/peak_set.pyx"
    if (!__pyx_t_1) break;

    /* "ms_deisotope/_c/peak_set.pyx":1416
 * 
 *     while hi != lo:
 *         mid = (hi + lo) // 2             # <<<<<<<<<<<<<<
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 */

#line 1416 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_mid = ((__pyx_v_hi + __pyx_v_lo) / 2);

    /* "ms_deisotope/_c/peak_set.pyx":1417
 *     while hi != lo:
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]             # <<<<<<<<<<<<<<
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 */

#line 1417 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_found_mass = (__pyx_v_array[__pyx_v_mid]);

    /* "ms_deisotope/_c/peak_set.pyx":1418
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 1418 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_found_mass, __pyx_v_target)) < __pyx_v_error_tolerance) != 0);

#line 1418 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1419
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid             # <<<<<<<<<<<<<<
 *             best_error = INF
 *             i = 0
 */

#line 1419 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_index = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1420
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:
 *             best_index = mid
 *             best_error = INF             # <<<<<<<<<<<<<<
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 */

#line 1420 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_best_error = __pyx_v_12ms_deisotope_2_c_8peak_set_INF;

      /* "ms_deisotope/_c/peak_set.pyx":1421
 *             best_index = mid
 *             best_error = INF
 *             i = 0             # <<<<<<<<<<<<<<
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 */

#line 1421 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1422
 *             best_error = INF
 *             i = 0
 *             while mid - i >= 0 and i <= mid:             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 1422 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = (((__pyx_v_mid - __pyx_v_i) >= 0) != 0);

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_2) {

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        } else {

#line 1422 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = __pyx_t_2;

#line 1422 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L8_bool_binop_done;

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_2 = ((__pyx_v_i <= __pyx_v_mid) != 0);

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = __pyx_t_2;

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L8_bool_binop_done:;

#line 1422 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1423
 *             i = 0
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 1423 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid - __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1424
 *             while mid - i >= 0 and i <= mid:
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 1424 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1425
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 1425 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 1425 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1426
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 1426 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1426 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1427
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid - i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 1427 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid - __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1428
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 1428 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1426
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid - i
 *                         best_error = abs_error
 */

#line 1426 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":1425
 *                 found_mass = array[mid - i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid - i
 */

#line 1425 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L10;

#line 1425 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1430
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             start[0] = mid - i + 1
 */

#line 1430 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1430 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L7_break;

#line 1430 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1430 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L10:;

        /* "ms_deisotope/_c/peak_set.pyx":1431
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             start[0] = mid - i + 1
 *             i = 1
 */

#line 1431 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 1431 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1431 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L7_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1432
 *                     break
 *                 i += 1
 *             start[0] = mid - i + 1             # <<<<<<<<<<<<<<
 *             i = 1
 *             while (mid + i) < (n - 1):
 */

#line 1432 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = ((__pyx_v_mid - __pyx_v_i) + 1);

      /* "ms_deisotope/_c/peak_set.pyx":1433
 *                 i += 1
 *             start[0] = mid - i + 1
 *             i = 1             # <<<<<<<<<<<<<<
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 */

#line 1433 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_i = 1;

      /* "ms_deisotope/_c/peak_set.pyx":1434
 *             start[0] = mid - i + 1
 *             i = 1
 *             while (mid + i) < (n - 1):             # <<<<<<<<<<<<<<
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 */

#line 1434 "ms_deisotope/_c/peak_set.pyx"
      while (1) {

#line 1434 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = (((__pyx_v_mid + __pyx_v_i) < (__pyx_v_n - 1)) != 0);

#line 1434 "ms_deisotope/_c/peak_set.pyx"
        if (!__pyx_t_1) break;

        /* "ms_deisotope/_c/peak_set.pyx":1435
 *             i = 1
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]             # <<<<<<<<<<<<<<
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 */

#line 1435 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_found_mass = (__pyx_v_array[(__pyx_v_mid + __pyx_v_i)]);

        /* "ms_deisotope/_c/peak_set.pyx":1436
 *             while (mid + i) < (n - 1):
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))             # <<<<<<<<<<<<<<
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 */

#line 1436 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_abs_error = fabs(__pyx_f_12ms_deisotope_2_c_8peak_set__ppm_error(__pyx_v_target, __pyx_v_found_mass));

        /* "ms_deisotope/_c/peak_set.pyx":1437
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 1437 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_error_tolerance) != 0);

#line 1437 "ms_deisotope/_c/peak_set.pyx"
        if (__pyx_t_1) {

          /* "ms_deisotope/_c/peak_set.pyx":1438
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 1438 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_abs_error < __pyx_v_best_error) != 0);

#line 1438 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1439
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:
 *                         best_index = mid + i             # <<<<<<<<<<<<<<
 *                         best_error = abs_error
 *                 else:
 */

#line 1439 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_index = (__pyx_v_mid + __pyx_v_i);

            /* "ms_deisotope/_c/peak_set.pyx":1440
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 *                         best_error = abs_error             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */

#line 1440 "ms_deisotope/_c/peak_set.pyx"
            __pyx_v_best_error = __pyx_v_abs_error;

            /* "ms_deisotope/_c/peak_set.pyx":1438
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:
 *                     if abs_error < best_error:             # <<<<<<<<<<<<<<
 *                         best_index = mid + i
 *                         best_error = abs_error
 */

#line 1438 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":1437
 *                 found_mass = array[mid + i]
 *                 abs_error = fabs(_ppm_error(target, found_mass))
 *                 if abs_error < error_tolerance:             # <<<<<<<<<<<<<<
 *                     if abs_error < best_error:
 *                         best_index = mid + i
 */

#line 1437 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L14;

#line 1437 "ms_deisotope/_c/peak_set.pyx"
        }

        /* "ms_deisotope/_c/peak_set.pyx":1442
 *                         best_error = abs_error
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *                 i += 1
 *             end[0] = mid + i
 */

#line 1442 "ms_deisotope/_c/peak_set.pyx"
        /*else*/ {

#line 1442 "ms_deisotope/_c/peak_set.pyx"
          goto __pyx_L13_break;

#line 1442 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1442 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L14:;

        /* "ms_deisotope/_c/peak_set.pyx":1443
 *                 else:
 *                     break
 *                 i += 1             # <<<<<<<<<<<<<<
 *             end[0] = mid + i
 *             return 0
 */

#line 1443 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = (__pyx_v_i + 1);

#line 1443 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1443 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L13_break:;

      /* "ms_deisotope/_c/peak_set.pyx":1444
 *                     break
 *                 i += 1
 *             end[0] = mid + i             # <<<<<<<<<<<<<<
 *             return 0
 *         elif hi - lo == 1:
 */

#line 1444 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_end[0]) = (__pyx_v_mid + __pyx_v_i);

      /* "ms_deisotope/_c/peak_set.pyx":1445
 *                 i += 1
 *             end[0] = mid + i
 *             return 0             # <<<<<<<<<<<<<<
 *         elif hi - lo == 1:
 *             start[0] = 0
 */

#line 1445 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 0;

#line 1445 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1418
 *         mid = (hi + lo) // 2
 *         found_mass = array[mid]
 *         if fabs(_ppm_error(found_mass, target)) < error_tolerance:             # <<<<<<<<<<<<<<
 *             best_index = mid
 *             best_error = INF
 */

#line 1418 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1446
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */

#line 1446 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = (((__pyx_v_hi - __pyx_v_lo) == 1) != 0);

#line 1446 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1447
 *             return 0
 *         elif hi - lo == 1:
 *             start[0] = 0             # <<<<<<<<<<<<<<
 *             end[0] = 0
 *             return 1
 */

#line 1447 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1448
 *         elif hi - lo == 1:
 *             start[0] = 0
 *             end[0] = 0             # <<<<<<<<<<<<<<
 *             return 1
 *         elif found_mass > target:
 */

#line 1448 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_end[0]) = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1449
 *             start[0] = 0
 *             end[0] = 0
 *             return 1             # <<<<<<<<<<<<<<
 *         elif found_mass > target:
 *             hi = mid
 */

#line 1449 "ms_deisotope/_c/peak_set.pyx"
      __pyx_r = 1;

#line 1449 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L0;

      /* "ms_deisotope/_c/peak_set.pyx":1446
 *             end[0] = mid + i
 *             return 0
 *         elif hi - lo == 1:             # <<<<<<<<<<<<<<
 *             start[0] = 0
 *             end[0] = 0
 */

#line 1446 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1450
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1450 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_found_mass > __pyx_v_target) != 0);

#line 1450 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1451
 *             return 1
 *         elif found_mass > target:
 *             hi = mid             # <<<<<<<<<<<<<<
 *         else:
 *             lo = mid
 */

#line 1451 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_hi = __pyx_v_mid;

      /* "ms_deisotope/_c/peak_set.pyx":1450
 *             end[0] = 0
 *             return 1
 *         elif found_mass > target:             # <<<<<<<<<<<<<<
 *             hi = mid
 *         else:
 */

#line 1450 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1450 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1453
 *             hi = mid
 *         else:
 *             lo = mid             # <<<<<<<<<<<<<<
 *     start[0] = 0
 *     end[0] = 0
 */

#line 1453 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1453 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_lo = __pyx_v_mid;

#line 1453 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1453 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

#line 1453 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1454
 *         else:
 *             lo = mid
 *     start[0] = 0             # <<<<<<<<<<<<<<
 *     end[0] = 0
 *     return 2
 */

#line 1454 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_start[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1455
 *             lo = mid
 *     start[0] = 0
 *     end[0] = 0             # <<<<<<<<<<<<<<
 *     return 2
 * 
 */

#line 1455 "ms_deisotope/_c/peak_set.pyx"
  (__pyx_v_end[0]) = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1456
 *     start[0] = 0
 *     end[0] = 0
 *     return 2             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1456 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 2;

#line 1456 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1405
 * 
 * 
 * cdef int _binary_search_interval_with_hint(double* array, double target, double error_tolerance, size_t n, size_t hint, size_t* start, size_t* end) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t lo, hi, mid, i, j
 */

#line 1405 "ms_deisotope/_c/peak_set.pyx"


#line 1405 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1405 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1405 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1405 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1459
 * 
 * 
 * cdef int build_interval_index(DeconvolutedPeakSet peaks, index_list* index, size_t index_size):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* linear_spacing
 */

#line 1459 "ms_deisotope/_c/peak_set.pyx"


#line 1459 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_build_interval_index(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *__pyx_v_peaks, struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, size_t __pyx_v_index_size) {
  double *__pyx_v_linear_spacing
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_current_value
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_err
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_next_value
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_start_i
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_end_i
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_index_i
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_peaks_size
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *__pyx_v_peak = 0
#line 1459 "ms_deisotope/_c/peak_set.pyx"
;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  PyObject *__pyx_t_2 = NULL;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_3;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_5;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_6;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_7;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("build_interval_index", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1465
 *         size_t i, start_i, end_i, index_i, peaks_size
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()             # <<<<<<<<<<<<<<
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass
 */

#line 1465 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_peaks_size = ((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->get_size(__pyx_v_peaks);

  /* "ms_deisotope/_c/peak_set.pyx":1466
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:             # <<<<<<<<<<<<<<
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 */

#line 1466 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_peaks_size > 0) != 0);

#line 1466 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1467
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass             # <<<<<<<<<<<<<<
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 *     else:
 */

#line 1467 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1467, __pyx_L1_error)

#line 1467 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 1467 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)->neutral_mass;

#line 1467 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1467 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index->low = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":1468
 *     if peaks_size > 0:
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass             # <<<<<<<<<<<<<<
 *     else:
 *         index.low = 0
 */

#line 1468 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, (__pyx_v_peaks_size - 1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1468, __pyx_L1_error)

#line 1468 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 1468 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_3 = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2)->neutral_mass;

#line 1468 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1468 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index->high = __pyx_t_3;

    /* "ms_deisotope/_c/peak_set.pyx":1466
 *         DeconvolutedPeak peak
 *     peaks_size = peaks.get_size()
 *     if peaks_size > 0:             # <<<<<<<<<<<<<<
 *         index.low = peaks.getitem(0).neutral_mass
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 */

#line 1466 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 1466 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1470
 *         index.high = peaks.getitem(peaks_size - 1).neutral_mass
 *     else:
 *         index.low = 0             # <<<<<<<<<<<<<<
 *         index.high = 1
 *     index.size = index_size
 */

#line 1470 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1470 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index->low = 0.0;

    /* "ms_deisotope/_c/peak_set.pyx":1471
 *     else:
 *         index.low = 0
 *         index.high = 1             # <<<<<<<<<<<<<<
 *     index.size = index_size
 *     linear_spacing = build_linear_spaced_array(
 */

#line 1471 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index->high = 1.0;

#line 1471 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1471 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":1472
 *         index.low = 0
 *         index.high = 1
 *     index.size = index_size             # <<<<<<<<<<<<<<
 *     linear_spacing = build_linear_spaced_array(
 *         index.low,
 */

#line 1472 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_index->size = __pyx_v_index_size;

  /* "ms_deisotope/_c/peak_set.pyx":1473
 *         index.high = 1
 *     index.size = index_size
 *     linear_spacing = build_linear_spaced_array(             # <<<<<<<<<<<<<<
 *         index.low,
 *         index.high,
 */

#line 1473 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_linear_spacing = __pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(__pyx_v_index->low, __pyx_v_index->high, __pyx_v_index_size);

  /* "ms_deisotope/_c/peak_set.pyx":1478
 *         index_size)
 * 
 *     index.index = <index_cell*>malloc(sizeof(index_cell) * index_size)             # <<<<<<<<<<<<<<
 * 
 *     for index_i in range(index_size):
 */

#line 1478 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_index->index = ((struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell *)malloc(((sizeof(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell)) * __pyx_v_index_size)));

  /* "ms_deisotope/_c/peak_set.pyx":1480
 *     index.index = <index_cell*>malloc(sizeof(index_cell) * index_size)
 * 
 *     for index_i in range(index_size):             # <<<<<<<<<<<<<<
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 */

#line 1480 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __pyx_v_index_size;

#line 1480 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_5 = __pyx_t_4;

#line 1480 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1480 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_index_i = __pyx_t_6;

    /* "ms_deisotope/_c/peak_set.pyx":1481
 * 
 *     for index_i in range(index_size):
 *         current_value = linear_spacing[index_i]             # <<<<<<<<<<<<<<
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:
 */

#line 1481 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_current_value = (__pyx_v_linear_spacing[__pyx_v_index_i]);

    /* "ms_deisotope/_c/peak_set.pyx":1482
 *     for index_i in range(index_size):
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)             # <<<<<<<<<<<<<<
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 */

#line 1482 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->_get_nearest_peak(__pyx_v_peaks, __pyx_v_current_value, (&__pyx_v_err))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1482, __pyx_L1_error)

#line 1482 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_GOTREF(__pyx_t_2);

#line 1482 "ms_deisotope/_c/peak_set.pyx"
    __Pyx_XDECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2));

#line 1482 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1483
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:             # <<<<<<<<<<<<<<
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 */

#line 1483 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_peaks_size > 0) != 0);

#line 1483 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1484
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 */

#line 1484 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_7 = __pyx_v_peak->_index->neutral_mass;

#line 1484 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_start_i = __pyx_t_7;

      /* "ms_deisotope/_c/peak_set.pyx":1485
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:             # <<<<<<<<<<<<<<
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 */

#line 1485 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_index_i > 0) != 0);

#line 1485 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1486
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1             # <<<<<<<<<<<<<<
 *             if index_i == index_size - 1:
 *                 end_i = peaks_size - 1
 */

#line 1486 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_start_i = ((__pyx_v_index->index[(__pyx_v_index_i - 1)]).end - 1);

        /* "ms_deisotope/_c/peak_set.pyx":1485
 *         if peaks_size > 0:
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:             # <<<<<<<<<<<<<<
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 */

#line 1485 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1487
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:             # <<<<<<<<<<<<<<
 *                 end_i = peaks_size - 1
 *             else:
 */

#line 1487 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_1 = ((__pyx_v_index_i == (__pyx_v_index_size - 1)) != 0);

#line 1487 "ms_deisotope/_c/peak_set.pyx"
      if (__pyx_t_1) {

        /* "ms_deisotope/_c/peak_set.pyx":1488
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:
 *                 end_i = peaks_size - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 next_value = linear_spacing[index_i + 1]
 */

#line 1488 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_end_i = (__pyx_v_peaks_size - 1);

        /* "ms_deisotope/_c/peak_set.pyx":1487
 *             if index_i > 0:
 *                 start_i = index.index[index_i - 1].end - 1
 *             if index_i == index_size - 1:             # <<<<<<<<<<<<<<
 *                 end_i = peaks_size - 1
 *             else:
 */

#line 1487 "ms_deisotope/_c/peak_set.pyx"
        goto __pyx_L8;

#line 1487 "ms_deisotope/_c/peak_set.pyx"
      }

      /* "ms_deisotope/_c/peak_set.pyx":1490
 *                 end_i = peaks_size - 1
 *             else:
 *                 next_value = linear_spacing[index_i + 1]             # <<<<<<<<<<<<<<
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:
 */

#line 1490 "ms_deisotope/_c/peak_set.pyx"
      /*else*/ {

#line 1490 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_next_value = (__pyx_v_linear_spacing[(__pyx_v_index_i + 1)]);

        /* "ms_deisotope/_c/peak_set.pyx":1491
 *             else:
 *                 next_value = linear_spacing[index_i + 1]
 *                 i = peak._index.neutral_mass             # <<<<<<<<<<<<<<
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 */

#line 1491 "ms_deisotope/_c/peak_set.pyx"
        __pyx_t_7 = __pyx_v_peak->_index->neutral_mass;

#line 1491 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_i = __pyx_t_7;

        /* "ms_deisotope/_c/peak_set.pyx":1492
 *                 next_value = linear_spacing[index_i + 1]
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:             # <<<<<<<<<<<<<<
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 */

#line 1492 "ms_deisotope/_c/peak_set.pyx"
        while (1) {

#line 1492 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((__pyx_v_i < __pyx_v_peaks_size) != 0);

#line 1492 "ms_deisotope/_c/peak_set.pyx"
          if (!__pyx_t_1) break;

          /* "ms_deisotope/_c/peak_set.pyx":1493
 *                 i = peak._index.neutral_mass
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)             # <<<<<<<<<<<<<<
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break
 */

#line 1493 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)__pyx_v_peaks->__pyx_vtab)->getitem(__pyx_v_peaks, __pyx_v_i)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1493, __pyx_L1_error)

#line 1493 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_GOTREF(__pyx_t_2);

#line 1493 "ms_deisotope/_c/peak_set.pyx"
          __Pyx_DECREF_SET(__pyx_v_peak, ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)__pyx_t_2));

#line 1493 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_2 = 0;

          /* "ms_deisotope/_c/peak_set.pyx":1494
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */

#line 1494 "ms_deisotope/_c/peak_set.pyx"
          __pyx_t_1 = ((fabs((__pyx_v_current_value - __pyx_v_peak->neutral_mass)) > fabs((__pyx_v_next_value - __pyx_v_peak->neutral_mass))) != 0);

#line 1494 "ms_deisotope/_c/peak_set.pyx"
          if (__pyx_t_1) {

            /* "ms_deisotope/_c/peak_set.pyx":1495
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 end_i = i
 */

#line 1495 "ms_deisotope/_c/peak_set.pyx"
            goto __pyx_L10_break;

            /* "ms_deisotope/_c/peak_set.pyx":1494
 *                 while i < peaks_size:
 *                     peak = peaks.getitem(i)
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */

#line 1494 "ms_deisotope/_c/peak_set.pyx"
          }

          /* "ms_deisotope/_c/peak_set.pyx":1496
 *                     if abs(current_value - peak.neutral_mass) > abs(next_value - peak.neutral_mass):
 *                         break
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 end_i = i
 *         else:
 */

#line 1496 "ms_deisotope/_c/peak_set.pyx"
          __pyx_v_i = (__pyx_v_i + 1);

#line 1496 "ms_deisotope/_c/peak_set.pyx"
        }

#line 1496 "ms_deisotope/_c/peak_set.pyx"
        __pyx_L10_break:;

        /* "ms_deisotope/_c/peak_set.pyx":1497
 *                         break
 *                     i += 1
 *                 end_i = i             # <<<<<<<<<<<<<<
 *         else:
 *             start_i = 0
 */

#line 1497 "ms_deisotope/_c/peak_set.pyx"
        __pyx_v_end_i = __pyx_v_i;

#line 1497 "ms_deisotope/_c/peak_set.pyx"
      }

#line 1497 "ms_deisotope/_c/peak_set.pyx"
      __pyx_L8:;

      /* "ms_deisotope/_c/peak_set.pyx":1483
 *         current_value = linear_spacing[index_i]
 *         peak = peaks._get_nearest_peak(current_value, &err)
 *         if peaks_size > 0:             # <<<<<<<<<<<<<<
 *             start_i = peak._index.neutral_mass
 *             if index_i > 0:
 */

#line 1483 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L6;

#line 1483 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1499
 *                 end_i = i
 *         else:
 *             start_i = 0             # <<<<<<<<<<<<<<
 *             end_i = 0
 *         index.index[index_i].center_value = current_value
 */

#line 1499 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1499 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_start_i = 0;

      /* "ms_deisotope/_c/peak_set.pyx":1500
 *         else:
 *             start_i = 0
 *             end_i = 0             # <<<<<<<<<<<<<<
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i
 */

#line 1500 "ms_deisotope/_c/peak_set.pyx"
      __pyx_v_end_i = 0;

#line 1500 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1500 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L6:;

    /* "ms_deisotope/_c/peak_set.pyx":1501
 *             start_i = 0
 *             end_i = 0
 *         index.index[index_i].center_value = current_value             # <<<<<<<<<<<<<<
 *         index.index[index_i].start = start_i
 *         index.index[index_i].end = end_i
 */

#line 1501 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_index->index[__pyx_v_index_i]).center_value = __pyx_v_current_value;

    /* "ms_deisotope/_c/peak_set.pyx":1502
 *             end_i = 0
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i             # <<<<<<<<<<<<<<
 *         index.index[index_i].end = end_i
 * 
 */

#line 1502 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_index->index[__pyx_v_index_i]).start = __pyx_v_start_i;

    /* "ms_deisotope/_c/peak_set.pyx":1503
 *         index.index[index_i].center_value = current_value
 *         index.index[index_i].start = start_i
 *         index.index[index_i].end = end_i             # <<<<<<<<<<<<<<
 * 
 *     free(linear_spacing)
 */

#line 1503 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_index->index[__pyx_v_index_i]).end = __pyx_v_end_i;

#line 1503 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1505
 *         index.index[index_i].end = end_i
 * 
 *     free(linear_spacing)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */

#line 1505 "ms_deisotope/_c/peak_set.pyx"
  free(__pyx_v_linear_spacing);

  /* "ms_deisotope/_c/peak_set.pyx":1506
 * 
 *     free(linear_spacing)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1506 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1506 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1459
 * 
 * 
 * cdef int build_interval_index(DeconvolutedPeakSet peaks, index_list* index, size_t index_size):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* linear_spacing
 */

#line 1459 "ms_deisotope/_c/peak_set.pyx"


#line 1459 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF(__pyx_t_2);

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.build_interval_index", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XDECREF((PyObject *)__pyx_v_peak);

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1459 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1459 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1509
 * 
 * 
 * cdef double* build_linear_spaced_array(double low, double high, size_t n):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* array
 */

#line 1509 "ms_deisotope/_c/peak_set.pyx"


#line 1509 "ms_deisotope/_c/peak_set.pyx"
static double *__pyx_f_12ms_deisotope_2_c_8peak_set_build_linear_spaced_array(double __pyx_v_low, double __pyx_v_high, size_t __pyx_v_n) {
  double *__pyx_v_array
#line 1509 "ms_deisotope/_c/peak_set.pyx"
;
  double __pyx_v_delta
#line 1509 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1509 "ms_deisotope/_c/peak_set.pyx"
;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  double *__pyx_r;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_1;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_2;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_3;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_4;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("build_linear_spaced_array", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1515
 *         size_t i
 * 
 *     delta = (high - low) / (n - 1)             # <<<<<<<<<<<<<<
 * 
 *     array = <double*>malloc(sizeof(double) * n)
 */

#line 1515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_v_high - __pyx_v_low);

#line 1515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_v_n - 1);

#line 1515 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_t_2 == 0)) {

#line 1515 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");

#line 1515 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1515, __pyx_L1_error)

#line 1515 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1515 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_delta = (__pyx_t_1 / __pyx_t_2);

  /* "ms_deisotope/_c/peak_set.pyx":1517
 *     delta = (high - low) / (n - 1)
 * 
 *     array = <double*>malloc(sizeof(double) * n)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(n):
 */

#line 1517 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_array = ((double *)malloc(((sizeof(double)) * __pyx_v_n)));

  /* "ms_deisotope/_c/peak_set.pyx":1519
 *     array = <double*>malloc(sizeof(double) * n)
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         array[i] = low + i * delta
 *     return array
 */

#line 1519 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_v_n;

#line 1519 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __pyx_t_2;

#line 1519 "ms_deisotope/_c/peak_set.pyx"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1519 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_t_4;

    /* "ms_deisotope/_c/peak_set.pyx":1520
 * 
 *     for i in range(n):
 *         array[i] = low + i * delta             # <<<<<<<<<<<<<<
 *     return array
 * 
 */

#line 1520 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_array[__pyx_v_i]) = (__pyx_v_low + (__pyx_v_i * __pyx_v_delta));

#line 1520 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1521
 *     for i in range(n):
 *         array[i] = low + i * delta
 *     return array             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1521 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_array;

#line 1521 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1509
 * 
 * 
 * cdef double* build_linear_spaced_array(double low, double high, size_t n):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double* array
 */

#line 1509 "ms_deisotope/_c/peak_set.pyx"


#line 1509 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.build_linear_spaced_array", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1509 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1509 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1524
 * 
 * 
 * cdef void free_index_list(index_list* index):             # <<<<<<<<<<<<<<
 *     free(index.index)
 *     free(index)
 */

#line 1524 "ms_deisotope/_c/peak_set.pyx"


#line 1524 "ms_deisotope/_c/peak_set.pyx"
static void __pyx_f_12ms_deisotope_2_c_8peak_set_free_index_list(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index) {

#line 1524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("free_index_list", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1525
 * 
 * cdef void free_index_list(index_list* index):
 *     free(index.index)             # <<<<<<<<<<<<<<
 *     free(index)
 * 
 */

#line 1525 "ms_deisotope/_c/peak_set.pyx"
  free(__pyx_v_index->index);

  /* "ms_deisotope/_c/peak_set.pyx":1526
 * cdef void free_index_list(index_list* index):
 *     free(index.index)
 *     free(index)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1526 "ms_deisotope/_c/peak_set.pyx"
  free(__pyx_v_index);

  /* "ms_deisotope/_c/peak_set.pyx":1524
 * 
 * 
 * cdef void free_index_list(index_list* index):             # <<<<<<<<<<<<<<
 *     free(index.index)
 *     free(index)
 */

#line 1524 "ms_deisotope/_c/peak_set.pyx"


#line 1524 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1524 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1524 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1529
 * 
 * 
 * cdef int check_index(index_list* index) nogil:             # <<<<<<<<<<<<<<
 *     if index.size == 0:
 *         return 1
 */

#line 1529 "ms_deisotope/_c/peak_set.pyx"


#line 1529 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_check_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index) {

#line 1529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1529 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

  /* "ms_deisotope/_c/peak_set.pyx":1530
 * 
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif (index.high - index.low) == 0:
 */

#line 1530 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_index->size == 0) != 0);

#line 1530 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1531
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:
 *         return 1             # <<<<<<<<<<<<<<
 *     elif (index.high - index.low) == 0:
 *         return 2
 */

#line 1531 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = 1;

#line 1531 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1530
 * 
 * cdef int check_index(index_list* index) nogil:
 *     if index.size == 0:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif (index.high - index.low) == 0:
 */

#line 1530 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1532
 *     if index.size == 0:
 *         return 1
 *     elif (index.high - index.low) == 0:             # <<<<<<<<<<<<<<
 *         return 2
 *     else:
 */

#line 1532 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (((__pyx_v_index->high - __pyx_v_index->low) == 0.0) != 0);

#line 1532 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1533
 *         return 1
 *     elif (index.high - index.low) == 0:
 *         return 2             # <<<<<<<<<<<<<<
 *     else:
 *         return 0
 */

#line 1533 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = 2;

#line 1533 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

    /* "ms_deisotope/_c/peak_set.pyx":1532
 *     if index.size == 0:
 *         return 1
 *     elif (index.high - index.low) == 0:             # <<<<<<<<<<<<<<
 *         return 2
 *     else:
 */

#line 1532 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1535
 *         return 2
 *     else:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1535 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1535 "ms_deisotope/_c/peak_set.pyx"
    __pyx_r = 0;

#line 1535 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L0;

#line 1535 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1529
 * 
 * 
 * cdef int check_index(index_list* index) nogil:             # <<<<<<<<<<<<<<
 *     if index.size == 0:
 *         return 1
 */

#line 1529 "ms_deisotope/_c/peak_set.pyx"


#line 1529 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1529 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1529 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1529 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1538
 * 
 * 
 * cdef size_t interpolate_index(index_list* index, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double v
 */

#line 1538 "ms_deisotope/_c/peak_set.pyx"


#line 1538 "ms_deisotope/_c/peak_set.pyx"
static size_t __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, double __pyx_v_value) {
  double __pyx_v_v
#line 1538 "ms_deisotope/_c/peak_set.pyx"
;
  size_t __pyx_v_i
#line 1538 "ms_deisotope/_c/peak_set.pyx"
;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_r;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_1;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  double __pyx_t_2;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_lineno = 0;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  const char *__pyx_filename = NULL;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_clineno = 0;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("interpolate_index", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1542
 *         double v
 *         size_t i
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)             # <<<<<<<<<<<<<<
 *     i = <size_t>v
 *     return i
 */

#line 1542 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = (__pyx_v_value - __pyx_v_index->low);

#line 1542 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = (__pyx_v_index->high - __pyx_v_index->low);

#line 1542 "ms_deisotope/_c/peak_set.pyx"
  if (unlikely(__pyx_t_2 == 0)) {

#line 1542 "ms_deisotope/_c/peak_set.pyx"
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");

#line 1542 "ms_deisotope/_c/peak_set.pyx"
    __PYX_ERR(0, 1542, __pyx_L1_error)

#line 1542 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1542 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_v = ((__pyx_t_1 / __pyx_t_2) * __pyx_v_index->size);

  /* "ms_deisotope/_c/peak_set.pyx":1543
 *         size_t i
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)
 *     i = <size_t>v             # <<<<<<<<<<<<<<
 *     return i
 * 
 */

#line 1543 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_i = ((size_t)__pyx_v_v);

  /* "ms_deisotope/_c/peak_set.pyx":1544
 *     v = (((value - index.low) / (index.high - index.low)) * index.size)
 *     i = <size_t>v
 *     return i             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1544 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = __pyx_v_i;

#line 1544 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1538
 * 
 * 
 * cdef size_t interpolate_index(index_list* index, double value):             # <<<<<<<<<<<<<<
 *     cdef:
 *         double v
 */

#line 1538 "ms_deisotope/_c/peak_set.pyx"


#line 1538 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L1_error:;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_WriteUnraisable("ms_deisotope._c.peak_set.interpolate_index", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1538 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1538 "ms_deisotope/_c/peak_set.pyx"
}

/* "ms_deisotope/_c/peak_set.pyx":1547
 * 
 * 
 * cdef int find_search_interval(index_list* index, double value, size_t* start, size_t* end):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

#line 1547 "ms_deisotope/_c/peak_set.pyx"


#line 1547 "ms_deisotope/_c/peak_set.pyx"
static int __pyx_f_12ms_deisotope_2_c_8peak_set_find_search_interval(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_list *__pyx_v_index, double __pyx_v_value, size_t *__pyx_v_start, size_t *__pyx_v_end) {
  size_t __pyx_v_i
#line 1547 "ms_deisotope/_c/peak_set.pyx"
;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_r;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyDeclarations

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  int __pyx_t_1;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  size_t __pyx_t_2;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannySetupContext("find_search_interval", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1550
 *     cdef:
 *         size_t i
 *     if value > index.high:             # <<<<<<<<<<<<<<
 *         i = index.size - 1
 *     elif value < index.low:
 */

#line 1550 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_value > __pyx_v_index->high) != 0);

#line 1550 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1551
 *         size_t i
 *     if value > index.high:
 *         i = index.size - 1             # <<<<<<<<<<<<<<
 *     elif value < index.low:
 *         i = 0
 */

#line 1551 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = (__pyx_v_index->size - 1);

    /* "ms_deisotope/_c/peak_set.pyx":1550
 *     cdef:
 *         size_t i
 *     if value > index.high:             # <<<<<<<<<<<<<<
 *         i = index.size - 1
 *     elif value < index.low:
 */

#line 1550 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 1550 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1552
 *     if value > index.high:
 *         i = index.size - 1
 *     elif value < index.low:             # <<<<<<<<<<<<<<
 *         i = 0
 *     else:
 */

#line 1552 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_value < __pyx_v_index->low) != 0);

#line 1552 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1553
 *         i = index.size - 1
 *     elif value < index.low:
 *         i = 0             # <<<<<<<<<<<<<<
 *     else:
 *         i = interpolate_index(index, value)
 */

#line 1553 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = 0;

    /* "ms_deisotope/_c/peak_set.pyx":1552
 *     if value > index.high:
 *         i = index.size - 1
 *     elif value < index.low:             # <<<<<<<<<<<<<<
 *         i = 0
 *     else:
 */

#line 1552 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L3;

#line 1552 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1555
 *         i = 0
 *     else:
 *         i = interpolate_index(index, value)             # <<<<<<<<<<<<<<
 *     if i > 0:
 *         if i < index.size:
 */

#line 1555 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1555 "ms_deisotope/_c/peak_set.pyx"
    __pyx_v_i = __pyx_f_12ms_deisotope_2_c_8peak_set_interpolate_index(__pyx_v_index, __pyx_v_value);

#line 1555 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1555 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L3:;

  /* "ms_deisotope/_c/peak_set.pyx":1556
 *     else:
 *         i = interpolate_index(index, value)
 *     if i > 0:             # <<<<<<<<<<<<<<
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start
 */

#line 1556 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_i > 0) != 0);

#line 1556 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1557
 *         i = interpolate_index(index, value)
 *     if i > 0:
 *         if i < index.size:             # <<<<<<<<<<<<<<
 *             start[0] = index.index[i - 1].start
 *         else:
 */

#line 1557 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_index->size) != 0);

#line 1557 "ms_deisotope/_c/peak_set.pyx"
    if (__pyx_t_1) {

      /* "ms_deisotope/_c/peak_set.pyx":1558
 *     if i > 0:
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start             # <<<<<<<<<<<<<<
 *         else:
 *             # if we're at index.size or greater, act as if we're at the last
 */

#line 1558 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_v_index->index[(__pyx_v_i - 1)]).start;

#line 1558 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = __pyx_t_2;

      /* "ms_deisotope/_c/peak_set.pyx":1557
 *         i = interpolate_index(index, value)
 *     if i > 0:
 *         if i < index.size:             # <<<<<<<<<<<<<<
 *             start[0] = index.index[i - 1].start
 *         else:
 */

#line 1557 "ms_deisotope/_c/peak_set.pyx"
      goto __pyx_L5;

#line 1557 "ms_deisotope/_c/peak_set.pyx"
    }

    /* "ms_deisotope/_c/peak_set.pyx":1562
 *             # if we're at index.size or greater, act as if we're at the last
 *             # cell of the index
 *             start[0] = index.index[index.size - 2].start             # <<<<<<<<<<<<<<
 *     else:
 *         # if somehow the index were negative, this could end badly.
 */

#line 1562 "ms_deisotope/_c/peak_set.pyx"
    /*else*/ {

#line 1562 "ms_deisotope/_c/peak_set.pyx"
      __pyx_t_2 = (__pyx_v_index->index[(__pyx_v_index->size - 2)]).start;

#line 1562 "ms_deisotope/_c/peak_set.pyx"
      (__pyx_v_start[0]) = __pyx_t_2;

#line 1562 "ms_deisotope/_c/peak_set.pyx"
    }

#line 1562 "ms_deisotope/_c/peak_set.pyx"
    __pyx_L5:;

    /* "ms_deisotope/_c/peak_set.pyx":1556
 *     else:
 *         i = interpolate_index(index, value)
 *     if i > 0:             # <<<<<<<<<<<<<<
 *         if i < index.size:
 *             start[0] = index.index[i - 1].start
 */

#line 1556 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L4;

#line 1556 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1565
 *     else:
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start             # <<<<<<<<<<<<<<
 *     if i >= (index.size - 1):
 *         end[0] = index.index[index.size - 1].end + 1
 */

#line 1565 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1565 "ms_deisotope/_c/peak_set.pyx"
    __pyx_t_2 = (__pyx_v_index->index[__pyx_v_i]).start;

#line 1565 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_start[0]) = __pyx_t_2;

#line 1565 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1565 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L4:;

  /* "ms_deisotope/_c/peak_set.pyx":1566
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):             # <<<<<<<<<<<<<<
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 */

#line 1566 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = ((__pyx_v_i >= (__pyx_v_index->size - 1)) != 0);

#line 1566 "ms_deisotope/_c/peak_set.pyx"
  if (__pyx_t_1) {

    /* "ms_deisotope/_c/peak_set.pyx":1567
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):
 *         end[0] = index.index[index.size - 1].end + 1             # <<<<<<<<<<<<<<
 *     else:
 *         end[0] = index.index[i + 1].end + 1
 */

#line 1567 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_end[0]) = ((__pyx_v_index->index[(__pyx_v_index->size - 1)]).end + 1);

    /* "ms_deisotope/_c/peak_set.pyx":1566
 *         # if somehow the index were negative, this could end badly.
 *         start[0] = index.index[i].start
 *     if i >= (index.size - 1):             # <<<<<<<<<<<<<<
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 */

#line 1566 "ms_deisotope/_c/peak_set.pyx"
    goto __pyx_L6;

#line 1566 "ms_deisotope/_c/peak_set.pyx"
  }

  /* "ms_deisotope/_c/peak_set.pyx":1569
 *         end[0] = index.index[index.size - 1].end + 1
 *     else:
 *         end[0] = index.index[i + 1].end + 1             # <<<<<<<<<<<<<<
 *     return 0
 */

#line 1569 "ms_deisotope/_c/peak_set.pyx"
  /*else*/ {

#line 1569 "ms_deisotope/_c/peak_set.pyx"
    (__pyx_v_end[0]) = ((__pyx_v_index->index[(__pyx_v_i + 1)]).end + 1);

#line 1569 "ms_deisotope/_c/peak_set.pyx"
  }

#line 1569 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L6:;

  /* "ms_deisotope/_c/peak_set.pyx":1570
 *     else:
 *         end[0] = index.index[i + 1].end + 1
 *     return 0             # <<<<<<<<<<<<<<
 */

#line 1570 "ms_deisotope/_c/peak_set.pyx"
  __pyx_r = 0;

#line 1570 "ms_deisotope/_c/peak_set.pyx"
  goto __pyx_L0;

  /* "ms_deisotope/_c/peak_set.pyx":1547
 * 
 * 
 * cdef int find_search_interval(index_list* index, double value, size_t* start, size_t* end):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i
 */

#line 1547 "ms_deisotope/_c/peak_set.pyx"


#line 1547 "ms_deisotope/_c/peak_set.pyx"
  /* function exit code */

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  __pyx_L0:;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_RefNannyFinishContext();

#line 1547 "ms_deisotope/_c/peak_set.pyx"
  return __pyx_r;

#line 1547 "ms_deisotope/_c/peak_set.pyx"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":735
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 735, __pyx_L1_error)

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 735 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 734 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":738
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 738, __pyx_L1_error)

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 738 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 737 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":741
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 741, __pyx_L1_error)

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 741 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 740 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":744
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 744, __pyx_L1_error)

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 744 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 743 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":747
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 747, __pyx_L1_error)

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_t_1);

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = __pyx_t_1;

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = 0;

#line 747 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_1);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 746 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_1;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":751
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);

#line 751 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */

#line 750 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":753
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /*else*/ {

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_INCREF(__pyx_empty_tuple);

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = __pyx_empty_tuple;

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

#line 753 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 749 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":869
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */

#line 869 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  Py_INCREF(__pyx_v_base);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":870
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */

#line 870 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 868 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base
#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_r = NULL;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_1;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":873
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */

#line 873 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":875
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_r);

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 875 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L0;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */

#line 874 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":876
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_r);

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_INCREF(((PyObject *)__pyx_v_base));

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = ((PyObject *)__pyx_v_base);

#line 876 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XGIVEREF(__pyx_r);

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 872 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":882
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */

#line 882 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 882, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":883
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 883, __pyx_L5_except_error)

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 883 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 884, __pyx_L5_except_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 884, __pyx_L5_except_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 881 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 880 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":888
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */

#line 888 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 888, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":889
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 889, __pyx_L5_except_error)

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 889 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 890, __pyx_L5_except_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 890, __pyx_L5_except_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 887 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 886 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_r;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyDeclarations

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_1 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_2 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_3 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_t_4;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_5 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_6 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_7 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  PyObject *__pyx_t_8 = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_lineno = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  const char *__pyx_filename = NULL;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  int __pyx_clineno = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  {

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_declare

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_PyThreadState_assign

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_1);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_2);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGOTREF(__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    /*try:*/ {

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":894
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */

#line 894 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 894, __pyx_L3_error)

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L8_try_end;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":895
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    if (__pyx_t_4) {

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 895, __pyx_L5_except_error)

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_5);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_6);

#line 895 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":896
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 896, __pyx_L5_except_error)

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_GOTREF(__pyx_t_8);

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
      __PYX_ERR(2, 896, __pyx_L5_except_error)

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    }

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L5_except_error;

#line 896 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L5_except_error:;

    /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_1);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    goto __pyx_L1_error;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
    __pyx_L8_try_end:;

#line 893 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  }

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"


#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  /* function exit code */

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = 0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  goto __pyx_L0;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L1_error:;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_5);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_6);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_7);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_XDECREF(__pyx_t_8);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_r = -1;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_L0:;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_RefNannyFinishContext();

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  return __pyx_r;

#line 892 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
}

/* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
/* Python wrapper */

#line 122 "View.MemoryView"
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 122 "View.MemoryView"
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_itemsize
#line 122 "View.MemoryView"
;
  PyObject *__pyx_v_format = 0
#line 122 "View.MemoryView"
;
  PyObject *__pyx_v_mode = 0
#line 122 "View.MemoryView"
;
  int __pyx_v_allocate_buffer
#line 122 "View.MemoryView"
;

#line 122 "View.MemoryView"
  int __pyx_lineno = 0;

#line 122 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 122 "View.MemoryView"
  int __pyx_clineno = 0;

#line 122 "View.MemoryView"
  int __pyx_r;

#line 122 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 122 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);

#line 122 "View.MemoryView"
  {

#line 122 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,&__pyx_n_s_itemsize,&__pyx_n_s_format,&__pyx_n_s_mode,&__pyx_n_s_allocate_buffer,0};

#line 122 "View.MemoryView"
    PyObject* values[5] = {0,0,0,0,0};

#line 122 "View.MemoryView"
    values[3] = ((PyObject *)__pyx_n_s_c);

#line 122 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 122 "View.MemoryView"
      Py_ssize_t kw_args;

#line 122 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 122 "View.MemoryView"
      switch (pos_args) {
        case  5: 
#line 122 "View.MemoryView"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 122 "View.MemoryView"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 122 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 122 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 122 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  0: break;
        default: 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 122 "View.MemoryView"
      switch (pos_args) {

#line 122 "View.MemoryView"
        case  0:

#line 122 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shape)) != 0)) kw_args--;
        else 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  1:

#line 122 "View.MemoryView"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_itemsize)) != 0)) kw_args--;

#line 122 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 1); 
#line 122 "View.MemoryView"
__PYX_ERR(3, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  2:

#line 122 "View.MemoryView"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;

#line 122 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 2); 
#line 122 "View.MemoryView"
__PYX_ERR(3, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  3:

#line 122 "View.MemoryView"
        if (kw_args > 0) {

#line 122 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode);

#line 122 "View.MemoryView"
          if (value) { values[3] = value; kw_args--; }

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 122 "View.MemoryView"
        case  4:

#line 122 "View.MemoryView"
        if (kw_args > 0) {

#line 122 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_allocate_buffer);

#line 122 "View.MemoryView"
          if (value) { values[4] = value; kw_args--; }

#line 122 "View.MemoryView"
        }

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 122 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(3, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
    } else {

#line 122 "View.MemoryView"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: 
#line 122 "View.MemoryView"
values[4] = PyTuple_GET_ITEM(__pyx_args, 4);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  4: 
#line 122 "View.MemoryView"
values[3] = PyTuple_GET_ITEM(__pyx_args, 3);

#line 122 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  3: 
#line 122 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 122 "View.MemoryView"
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 122 "View.MemoryView"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 122 "View.MemoryView"
        break;
        default: 
#line 122 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 122 "View.MemoryView"
      }

#line 122 "View.MemoryView"
    }

#line 122 "View.MemoryView"
    __pyx_v_shape = ((PyObject*)values[0]);

#line 122 "View.MemoryView"
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 122, __pyx_L3_error)

#line 122 "View.MemoryView"
    __pyx_v_format = values[2];

#line 122 "View.MemoryView"
    __pyx_v_mode = values[3];

#line 122 "View.MemoryView"
    if (values[4]) {

#line 122 "View.MemoryView"
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 123, __pyx_L3_error)

#line 122 "View.MemoryView"
    } else {

      /* "View.MemoryView":123
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
 */

#line 123 "View.MemoryView"
      __pyx_v_allocate_buffer = ((int)1);

#line 123 "View.MemoryView"
    }

#line 123 "View.MemoryView"
  }

#line 123 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 123 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); 
#line 123 "View.MemoryView"
__PYX_ERR(3, 122, __pyx_L3_error)

#line 123 "View.MemoryView"
  __pyx_L3_error:;

#line 123 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 123 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 123 "View.MemoryView"
  return -1;

#line 123 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;

#line 123 "View.MemoryView"
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) __PYX_ERR(3, 122, __pyx_L1_error)

#line 123 "View.MemoryView"
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {

#line 123 "View.MemoryView"
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); __PYX_ERR(3, 122, __pyx_L1_error)

#line 123 "View.MemoryView"
  }
  __pyx_r = 
#line 123 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
  /* function exit code */

#line 122 "View.MemoryView"
  goto __pyx_L0;

#line 122 "View.MemoryView"
  __pyx_L1_error:;

#line 122 "View.MemoryView"
  __pyx_r = -1;

#line 122 "View.MemoryView"
  __pyx_L0:;

#line 122 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 122 "View.MemoryView"
  return __pyx_r;

#line 122 "View.MemoryView"
}

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_i
#line 122 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dim
#line 122 "View.MemoryView"
;
  PyObject **__pyx_v_p
#line 122 "View.MemoryView"
;
  char __pyx_v_order
#line 122 "View.MemoryView"
;

#line 122 "View.MemoryView"
  int __pyx_r;

#line 122 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 122 "View.MemoryView"
  int __pyx_t_2;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 122 "View.MemoryView"
  int __pyx_t_4;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 122 "View.MemoryView"
  char *__pyx_t_7;

#line 122 "View.MemoryView"
  int __pyx_t_8;

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 122 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 122 "View.MemoryView"
  Py_ssize_t __pyx_t_11;

#line 122 "View.MemoryView"
  int __pyx_lineno = 0;

#line 122 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 122 "View.MemoryView"
  int __pyx_clineno = 0;

#line 122 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__", 0);

#line 122 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":129
 *         cdef PyObject **p
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
 */

#line 129 "View.MemoryView"
  if (unlikely(__pyx_v_shape == Py_None)) {

#line 129 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 129 "View.MemoryView"
    __PYX_ERR(3, 129, __pyx_L1_error)

#line 129 "View.MemoryView"
  }

#line 129 "View.MemoryView"
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(3, 129, __pyx_L1_error)

#line 129 "View.MemoryView"
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":130
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
 */

#line 130 "View.MemoryView"
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */

#line 132 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_self->ndim != 0)) != 0);

#line 132 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */

#line 133 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 133, __pyx_L1_error)

#line 133 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 133 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 133 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 133 "View.MemoryView"
    __PYX_ERR(3, 133, __pyx_L1_error)

    /* "View.MemoryView":132
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */

#line 132 "View.MemoryView"
  }

  /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */

#line 135 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_itemsize <= 0) != 0);

#line 135 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */

#line 136 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 136, __pyx_L1_error)

#line 136 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 136 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 136 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 136 "View.MemoryView"
    __PYX_ERR(3, 136, __pyx_L1_error)

    /* "View.MemoryView":135
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */

#line 135 "View.MemoryView"
  }

  /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */

#line 138 "View.MemoryView"
  __pyx_t_2 = PyBytes_Check(__pyx_v_format); 

#line 138 "View.MemoryView"
  __pyx_t_4 = ((!(__pyx_t_2 != 0)) != 0);

#line 138 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":139
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */

#line 139 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_format, __pyx_n_s_encode); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 139, __pyx_L1_error)

#line 139 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 139 "View.MemoryView"
    __pyx_t_6 = NULL;

#line 139 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 139 "View.MemoryView"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);

#line 139 "View.MemoryView"
      if (likely(__pyx_t_6)) {

#line 139 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 139 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_6);

#line 139 "View.MemoryView"
        __Pyx_INCREF(function);

#line 139 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_5, function);

#line 139 "View.MemoryView"
      }

#line 139 "View.MemoryView"
    }

#line 139 "View.MemoryView"
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_n_s_ASCII) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_ASCII);

#line 139 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 139 "View.MemoryView"
    if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 139, __pyx_L1_error)

#line 139 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 139 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 139 "View.MemoryView"
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_3);

#line 139 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":138
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */

#line 138 "View.MemoryView"
  }

  /* "View.MemoryView":140
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
 */

#line 140 "View.MemoryView"
  if (!(likely(PyBytes_CheckExact(__pyx_v_format))||((__pyx_v_format) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_format)->tp_name), 0))) __PYX_ERR(3, 140, __pyx_L1_error)

#line 140 "View.MemoryView"
  __pyx_t_3 = __pyx_v_format;

#line 140 "View.MemoryView"
  __Pyx_INCREF(__pyx_t_3);

#line 140 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 140 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->_format);

#line 140 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->_format);

#line 140 "View.MemoryView"
  __pyx_v_self->_format = ((PyObject*)__pyx_t_3);

#line 140 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":141
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 141 "View.MemoryView"
  if (unlikely(__pyx_v_self->_format == Py_None)) {

#line 141 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");

#line 141 "View.MemoryView"
    __PYX_ERR(3, 141, __pyx_L1_error)

#line 141 "View.MemoryView"
  }

#line 141 "View.MemoryView"
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_self->_format); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(3, 141, __pyx_L1_error)

#line 141 "View.MemoryView"
  __pyx_v_self->format = __pyx_t_7;

  /* "View.MemoryView":144
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
 */

#line 144 "View.MemoryView"
  __pyx_v_self->_shape = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":145
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
 */

#line 145 "View.MemoryView"
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */

#line 147 "View.MemoryView"
  __pyx_t_4 = ((!(__pyx_v_self->_shape != 0)) != 0);

#line 147 "View.MemoryView"
  if (unlikely(__pyx_t_4)) {

    /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 148 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 148, __pyx_L1_error)

#line 148 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 148 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 148 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 148 "View.MemoryView"
    __PYX_ERR(3, 148, __pyx_L1_error)

    /* "View.MemoryView":147
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */

#line 147 "View.MemoryView"
  }

  /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */

#line 151 "View.MemoryView"
  __pyx_t_8 = 0;

#line 151 "View.MemoryView"
  __pyx_t_3 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_3); __pyx_t_1 = 0;

#line 151 "View.MemoryView"
  for (;;) {

#line 151 "View.MemoryView"
    if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_3)) break;

#line 151 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 151 "View.MemoryView"
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_5); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(3, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    #else

#line 151 "View.MemoryView"
    __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 151 "View.MemoryView"
    #endif

#line 151 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 151, __pyx_L1_error)

#line 151 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 151 "View.MemoryView"
    __pyx_v_dim = __pyx_t_9;

#line 151 "View.MemoryView"
    __pyx_v_idx = __pyx_t_8;

#line 151 "View.MemoryView"
    __pyx_t_8 = (__pyx_t_8 + 1);

    /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */

#line 152 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_dim <= 0) != 0);

#line 152 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":153
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
 */

#line 153 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 153 "View.MemoryView"
      __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 153 "View.MemoryView"
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 153 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_5);

#line 153 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5);

#line 153 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_6);

#line 153 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_6);

#line 153 "View.MemoryView"
      __pyx_t_5 = 0;

#line 153 "View.MemoryView"
      __pyx_t_6 = 0;

#line 153 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 153 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 153, __pyx_L1_error)

#line 153 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 153 "View.MemoryView"
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 153 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 153 "View.MemoryView"
      __PYX_ERR(3, 153, __pyx_L1_error)

      /* "View.MemoryView":152
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */

#line 152 "View.MemoryView"
    }

    /* "View.MemoryView":154
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
 */

#line 154 "View.MemoryView"
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":151
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */

#line 151 "View.MemoryView"
  }

#line 151 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */

#line 157 "View.MemoryView"
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_fortran, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(3, 157, __pyx_L1_error)

#line 157 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":158
 *         cdef char order
 *         if mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 */

#line 158 "View.MemoryView"
    __pyx_v_order = 'F';

    /* "View.MemoryView":159
 *         if mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         elif mode == 'c':
 *             order = b'C'
 */

#line 159 "View.MemoryView"
    __Pyx_INCREF(__pyx_n_u_fortran);

#line 159 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_n_u_fortran);

#line 159 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->mode);

#line 159 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->mode);

#line 159 "View.MemoryView"
    __pyx_v_self->mode = __pyx_n_u_fortran;

    /* "View.MemoryView":157
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */

#line 157 "View.MemoryView"
    goto __pyx_L10;

#line 157 "View.MemoryView"
  }

  /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */

#line 160 "View.MemoryView"
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_c, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(3, 160, __pyx_L1_error)

#line 160 "View.MemoryView"
  if (likely(__pyx_t_4)) {

    /* "View.MemoryView":161
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         else:
 */

#line 161 "View.MemoryView"
    __pyx_v_order = 'C';

    /* "View.MemoryView":162
 *         elif mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 */

#line 162 "View.MemoryView"
    __Pyx_INCREF(__pyx_n_u_c);

#line 162 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_n_u_c);

#line 162 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->mode);

#line 162 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->mode);

#line 162 "View.MemoryView"
    __pyx_v_self->mode = __pyx_n_u_c;

    /* "View.MemoryView":160
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */

#line 160 "View.MemoryView"
    goto __pyx_L10;

#line 160 "View.MemoryView"
  }

  /* "View.MemoryView":164
 *             self.mode = u'c'
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,
 */

#line 164 "View.MemoryView"
  /*else*/ {

#line 164 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_v_mode); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 164 "View.MemoryView"
    __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_10);

#line 164 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 164 "View.MemoryView"
    __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 164 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 164 "View.MemoryView"
    __PYX_ERR(3, 164, __pyx_L1_error)

#line 164 "View.MemoryView"
  }

#line 164 "View.MemoryView"
  __pyx_L10:;

  /* "View.MemoryView":166
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,             # <<<<<<<<<<<<<<
 *                                              itemsize, self.ndim, order)
 * 
 */

#line 166 "View.MemoryView"
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":169
 *                                              itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:
 */

#line 169 "View.MemoryView"
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":170
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 *         if allocate_buffer:
 * 
 */

#line 170 "View.MemoryView"
  __pyx_t_10 = PyObject_RichCompare(__pyx_v_format, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 170, __pyx_L1_error)

#line 170 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 170, __pyx_L1_error)

#line 170 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 170 "View.MemoryView"
  __pyx_v_self->dtype_is_object = __pyx_t_4;

  /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 171 "View.MemoryView"
  __pyx_t_4 = (__pyx_v_allocate_buffer != 0);

#line 171 "View.MemoryView"
  if (__pyx_t_4) {

    /* "View.MemoryView":174
 * 
 * 
 *             self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")
 */

#line 174 "View.MemoryView"
    __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

    /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */

#line 175 "View.MemoryView"
    __pyx_t_4 = ((!(__pyx_v_self->data != 0)) != 0);

#line 175 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */

#line 176 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(3, 176, __pyx_L1_error)

#line 176 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_10);

#line 176 "View.MemoryView"
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);

#line 176 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

#line 176 "View.MemoryView"
      __PYX_ERR(3, 176, __pyx_L1_error)

      /* "View.MemoryView":175
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */

#line 175 "View.MemoryView"
    }

    /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */

#line 178 "View.MemoryView"
    __pyx_t_4 = (__pyx_v_self->dtype_is_object != 0);

#line 178 "View.MemoryView"
    if (__pyx_t_4) {

      /* "View.MemoryView":179
 * 
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 */

#line 179 "View.MemoryView"
      __pyx_v_p = ((PyObject **)__pyx_v_self->data);

      /* "View.MemoryView":180
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):             # <<<<<<<<<<<<<<
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)
 */

#line 180 "View.MemoryView"
      if (unlikely(__pyx_v_itemsize == 0)) {

#line 180 "View.MemoryView"
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");

#line 180 "View.MemoryView"
        __PYX_ERR(3, 180, __pyx_L1_error)

#line 180 "View.MemoryView"
      }

#line 180 "View.MemoryView"
      else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {

#line 180 "View.MemoryView"
        PyErr_SetString(PyExc_OverflowError, "value too large to perform division");

#line 180 "View.MemoryView"
        __PYX_ERR(3, 180, __pyx_L1_error)

#line 180 "View.MemoryView"
      }

#line 180 "View.MemoryView"
      __pyx_t_1 = __Pyx_div_Py_ssize_t(__pyx_v_self->len, __pyx_v_itemsize);

#line 180 "View.MemoryView"
      __pyx_t_9 = __pyx_t_1;

#line 180 "View.MemoryView"
      for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_9; __pyx_t_11+=1) {

#line 180 "View.MemoryView"
        __pyx_v_i = __pyx_t_11;

        /* "View.MemoryView":181
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None             # <<<<<<<<<<<<<<
 *                     Py_INCREF(Py_None)
 * 
 */

#line 181 "View.MemoryView"
        (__pyx_v_p[__pyx_v_i]) = Py_None;

        /* "View.MemoryView":182
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 182 "View.MemoryView"
        Py_INCREF(Py_None);

#line 182 "View.MemoryView"
      }

      /* "View.MemoryView":178
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */

#line 178 "View.MemoryView"
    }

    /* "View.MemoryView":171
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 171 "View.MemoryView"
  }

  /* "View.MemoryView":122
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

#line 122 "View.MemoryView"


#line 122 "View.MemoryView"
  /* function exit code */

#line 122 "View.MemoryView"
  __pyx_r = 0;

#line 122 "View.MemoryView"
  goto __pyx_L0;

#line 122 "View.MemoryView"
  __pyx_L1_error:;

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_10);

#line 122 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 122 "View.MemoryView"
  __pyx_r = -1;

#line 122 "View.MemoryView"
  __pyx_L0:;

#line 122 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_format);

#line 122 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 122 "View.MemoryView"
  return __pyx_r;

#line 122 "View.MemoryView"
}

/* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
/* Python wrapper */

#line 185 "View.MemoryView"
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/

#line 185 "View.MemoryView"
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 185 "View.MemoryView"
  int __pyx_r;

#line 185 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 185 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = 
#line 185 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
  /* function exit code */

#line 185 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 185 "View.MemoryView"
  return __pyx_r;

#line 185 "View.MemoryView"
}

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode
#line 185 "View.MemoryView"
;

#line 185 "View.MemoryView"
  int __pyx_r;

#line 185 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 185 "View.MemoryView"
  int __pyx_t_1;

#line 185 "View.MemoryView"
  int __pyx_t_2;

#line 185 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 185 "View.MemoryView"
  char *__pyx_t_4;

#line 185 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 185 "View.MemoryView"
  int __pyx_t_6;

#line 185 "View.MemoryView"
  Py_ssize_t *__pyx_t_7;

#line 185 "View.MemoryView"
  int __pyx_lineno = 0;

#line 185 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 185 "View.MemoryView"
  int __pyx_clineno = 0;

#line 185 "View.MemoryView"
  if (__pyx_v_info == NULL) {

#line 185 "View.MemoryView"
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");

#line 185 "View.MemoryView"
    return -1;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__", 0);

#line 185 "View.MemoryView"
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);

#line 185 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":186
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */

#line 186 "View.MemoryView"
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */

#line 187 "View.MemoryView"
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_c, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 187, __pyx_L1_error)

#line 187 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 187 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":188
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */

#line 188 "View.MemoryView"
    __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":187
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */

#line 187 "View.MemoryView"
    goto __pyx_L3;

#line 187 "View.MemoryView"
  }

  /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */

#line 189 "View.MemoryView"
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_fortran, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 189, __pyx_L1_error)

#line 189 "View.MemoryView"
  __pyx_t_1 = (__pyx_t_2 != 0);

#line 189 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":190
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 */

#line 190 "View.MemoryView"
    __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":189
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */

#line 189 "View.MemoryView"
  }

#line 189 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */

#line 191 "View.MemoryView"
  __pyx_t_1 = ((!((__pyx_v_flags & __pyx_v_bufmode) != 0)) != 0);

#line 191 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */

#line 192 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 192, __pyx_L1_error)

#line 192 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 192 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 192 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 192 "View.MemoryView"
    __PYX_ERR(3, 192, __pyx_L1_error)

    /* "View.MemoryView":191
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */

#line 191 "View.MemoryView"
  }

  /* "View.MemoryView":193
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 *         info.ndim = self.ndim
 */

#line 193 "View.MemoryView"
  __pyx_t_4 = __pyx_v_self->data;

#line 193 "View.MemoryView"
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":194
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 */

#line 194 "View.MemoryView"
  __pyx_t_5 = __pyx_v_self->len;

#line 194 "View.MemoryView"
  __pyx_v_info->len = __pyx_t_5;

  /* "View.MemoryView":195
 *         info.buf = self.data
 *         info.len = self.len
 *         info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *         info.shape = self._shape
 *         info.strides = self._strides
 */

#line 195 "View.MemoryView"
  __pyx_t_6 = __pyx_v_self->ndim;

#line 195 "View.MemoryView"
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":196
 *         info.len = self.len
 *         info.ndim = self.ndim
 *         info.shape = self._shape             # <<<<<<<<<<<<<<
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 */

#line 196 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->_shape;

#line 196 "View.MemoryView"
  __pyx_v_info->shape = __pyx_t_7;

  /* "View.MemoryView":197
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 *         info.strides = self._strides             # <<<<<<<<<<<<<<
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 */

#line 197 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->_strides;

#line 197 "View.MemoryView"
  __pyx_v_info->strides = __pyx_t_7;

  /* "View.MemoryView":198
 *         info.shape = self._shape
 *         info.strides = self._strides
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 */

#line 198 "View.MemoryView"
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":199
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 * 
 */

#line 199 "View.MemoryView"
  __pyx_t_5 = __pyx_v_self->itemsize;

#line 199 "View.MemoryView"
  __pyx_v_info->itemsize = __pyx_t_5;

  /* "View.MemoryView":200
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 200 "View.MemoryView"
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */

#line 202 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 202 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":203
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */

#line 203 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->format;

#line 203 "View.MemoryView"
    __pyx_v_info->format = __pyx_t_4;

    /* "View.MemoryView":202
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */

#line 202 "View.MemoryView"
    goto __pyx_L5;

#line 202 "View.MemoryView"
  }

  /* "View.MemoryView":205
 *             info.format = self.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.obj = self
 */

#line 205 "View.MemoryView"
  /*else*/ {

#line 205 "View.MemoryView"
    __pyx_v_info->format = NULL;

#line 205 "View.MemoryView"
  }

#line 205 "View.MemoryView"
  __pyx_L5:;

  /* "View.MemoryView":207
 *             info.format = NULL
 * 
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 207 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 207 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 207 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_info->obj);

#line 207 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_info->obj);

#line 207 "View.MemoryView"
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":185
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

#line 185 "View.MemoryView"


#line 185 "View.MemoryView"
  /* function exit code */

#line 185 "View.MemoryView"
  __pyx_r = 0;

#line 185 "View.MemoryView"
  goto __pyx_L0;

#line 185 "View.MemoryView"
  __pyx_L1_error:;

#line 185 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 185 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 185 "View.MemoryView"
  __pyx_r = -1;

#line 185 "View.MemoryView"
  if (__pyx_v_info->obj != NULL) {

#line 185 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 185 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  goto __pyx_L2;

#line 185 "View.MemoryView"
  __pyx_L0:;

#line 185 "View.MemoryView"
  if (__pyx_v_info->obj == Py_None) {

#line 185 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 185 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 185 "View.MemoryView"
  }

#line 185 "View.MemoryView"
  __pyx_L2:;

#line 185 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 185 "View.MemoryView"
  return __pyx_r;

#line 185 "View.MemoryView"
}

/* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
/* Python wrapper */

#line 211 "View.MemoryView"
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 211 "View.MemoryView"
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {

#line 211 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 211 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 211 "View.MemoryView"
  __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
  /* function exit code */

#line 211 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 211 "View.MemoryView"
}

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {

#line 211 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 211 "View.MemoryView"
  int __pyx_t_1;

#line 211 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */

#line 212 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->callback_free_data != NULL) != 0);

#line 212 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":213
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data:
 *             if self.dtype_is_object:
 */

#line 213 "View.MemoryView"
    __pyx_v_self->callback_free_data(__pyx_v_self->data);

    /* "View.MemoryView":212
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */

#line 212 "View.MemoryView"
    goto __pyx_L3;

#line 212 "View.MemoryView"
  }

  /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */

#line 214 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->free_data != 0);

#line 214 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */

#line 215 "View.MemoryView"
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);

#line 215 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":216
 *         elif self.free_data:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,             # <<<<<<<<<<<<<<
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 */

#line 216 "View.MemoryView"
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);

      /* "View.MemoryView":215
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */

#line 215 "View.MemoryView"
    }

    /* "View.MemoryView":218
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyObject_Free(self._shape)
 * 
 */

#line 218 "View.MemoryView"
    free(__pyx_v_self->data);

    /* "View.MemoryView":214
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */

#line 214 "View.MemoryView"
  }

#line 214 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":219
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 *         PyObject_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 219 "View.MemoryView"
  PyObject_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":211
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

#line 211 "View.MemoryView"


#line 211 "View.MemoryView"
  /* function exit code */

#line 211 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 211 "View.MemoryView"
}

/* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
/* Python wrapper */

#line 222 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 222 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self) {

#line 222 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 222 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 222 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 222 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
  /* function exit code */

#line 222 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 222 "View.MemoryView"
  return __pyx_r;

#line 222 "View.MemoryView"
}

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {

#line 222 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 222 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 222 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 222 "View.MemoryView"
  int __pyx_lineno = 0;

#line 222 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 222 "View.MemoryView"
  int __pyx_clineno = 0;

#line 222 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":223
 *     @property
 *     def memview(self):
 *         return self.get_memview()             # <<<<<<<<<<<<<<
 * 
 *     @cname('get_memview')
 */

#line 223 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 223 "View.MemoryView"
  __pyx_t_1 = ((struct __pyx_vtabstruct_array *)__pyx_v_self->__pyx_vtab)->get_memview(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 223, __pyx_L1_error)

#line 223 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 223 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 223 "View.MemoryView"
  __pyx_t_1 = 0;

#line 223 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":222
 * 
 *     @property
 *     def memview(self):             # <<<<<<<<<<<<<<
 *         return self.get_memview()
 * 
 */

#line 222 "View.MemoryView"


#line 222 "View.MemoryView"
  /* function exit code */

#line 222 "View.MemoryView"
  __pyx_L1_error:;

#line 222 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 222 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 222 "View.MemoryView"
  __pyx_r = NULL;

#line 222 "View.MemoryView"
  __pyx_L0:;

#line 222 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 222 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 222 "View.MemoryView"
  return __pyx_r;

#line 222 "View.MemoryView"
}

/* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

#line 226 "View.MemoryView"


#line 226 "View.MemoryView"
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags
#line 226 "View.MemoryView"
;

#line 226 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 226 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 226 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 226 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 226 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 226 "View.MemoryView"
  int __pyx_lineno = 0;

#line 226 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 226 "View.MemoryView"
  int __pyx_clineno = 0;

#line 226 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_memview", 0);

  /* "View.MemoryView":227
 *     @cname('get_memview')
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 */

#line 227 "View.MemoryView"
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":228
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 228 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 228 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 228 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 228 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 228 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);

#line 228 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 228 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 228 "View.MemoryView"
  __pyx_t_1 = 0;

#line 228 "View.MemoryView"
  __pyx_t_2 = 0;

#line 228 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 228, __pyx_L1_error)

#line 228 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 228 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 228 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 228 "View.MemoryView"
  __pyx_t_2 = 0;

#line 228 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":226
 * 
 *     @cname('get_memview')
 *     cdef get_memview(self):             # <<<<<<<<<<<<<<
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)
 */

#line 226 "View.MemoryView"


#line 226 "View.MemoryView"
  /* function exit code */

#line 226 "View.MemoryView"
  __pyx_L1_error:;

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 226 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 226 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.get_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 226 "View.MemoryView"
  __pyx_r = 0;

#line 226 "View.MemoryView"
  __pyx_L0:;

#line 226 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 226 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 226 "View.MemoryView"
  return __pyx_r;

#line 226 "View.MemoryView"
}

/* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
/* Python wrapper */

#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self); /*proto*/

#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self) {

#line 230 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 230 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 230 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 230 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(((struct __pyx_array_obj *)__pyx_v_self));

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
  /* function exit code */

#line 230 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 230 "View.MemoryView"
  return __pyx_r;

#line 230 "View.MemoryView"
}

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self) {

#line 230 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 230 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 230 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":231
 * 
 *     def __len__(self):
 *         return self._shape[0]             # <<<<<<<<<<<<<<
 * 
 *     def __getattr__(self, attr):
 */

#line 231 "View.MemoryView"
  __pyx_r = (__pyx_v_self->_shape[0]);

#line 231 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":230
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
 */

#line 230 "View.MemoryView"


#line 230 "View.MemoryView"
  /* function exit code */

#line 230 "View.MemoryView"
  __pyx_L0:;

#line 230 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 230 "View.MemoryView"
  return __pyx_r;

#line 230 "View.MemoryView"
}

/* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
/* Python wrapper */

#line 233 "View.MemoryView"
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/

#line 233 "View.MemoryView"
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {

#line 233 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 233 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 233 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_r = 
#line 233 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
  /* function exit code */

#line 233 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 233 "View.MemoryView"
  return __pyx_r;

#line 233 "View.MemoryView"
}

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {

#line 233 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 233 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 233 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 233 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 233 "View.MemoryView"
  int __pyx_lineno = 0;

#line 233 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 233 "View.MemoryView"
  int __pyx_clineno = 0;

#line 233 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":234
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */

#line 234 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 234 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 234, __pyx_L1_error)

#line 234 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 234 "View.MemoryView"
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 234, __pyx_L1_error)

#line 234 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 234 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 234 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 234 "View.MemoryView"
  __pyx_t_2 = 0;

#line 234 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":233
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

#line 233 "View.MemoryView"


#line 233 "View.MemoryView"
  /* function exit code */

#line 233 "View.MemoryView"
  __pyx_L1_error:;

#line 233 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 233 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 233 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 233 "View.MemoryView"
  __pyx_r = NULL;

#line 233 "View.MemoryView"
  __pyx_L0:;

#line 233 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 233 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 233 "View.MemoryView"
  return __pyx_r;

#line 233 "View.MemoryView"
}

/* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
/* Python wrapper */

#line 236 "View.MemoryView"
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/

#line 236 "View.MemoryView"
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {

#line 236 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 236 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 236 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 236 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
  /* function exit code */

#line 236 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 236 "View.MemoryView"
  return __pyx_r;

#line 236 "View.MemoryView"
}

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {

#line 236 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 236 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 236 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 236 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 236 "View.MemoryView"
  int __pyx_lineno = 0;

#line 236 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 236 "View.MemoryView"
  int __pyx_clineno = 0;

#line 236 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":237
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
 */

#line 237 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 237 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 237, __pyx_L1_error)

#line 237 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 237 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 237, __pyx_L1_error)

#line 237 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 237 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 237 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 237 "View.MemoryView"
  __pyx_t_2 = 0;

#line 237 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":236
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

#line 236 "View.MemoryView"


#line 236 "View.MemoryView"
  /* function exit code */

#line 236 "View.MemoryView"
  __pyx_L1_error:;

#line 236 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 236 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 236 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 236 "View.MemoryView"
  __pyx_r = NULL;

#line 236 "View.MemoryView"
  __pyx_L0:;

#line 236 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 236 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 236 "View.MemoryView"
  return __pyx_r;

#line 236 "View.MemoryView"
}

/* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
/* Python wrapper */

#line 239 "View.MemoryView"
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/

#line 239 "View.MemoryView"
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {

#line 239 "View.MemoryView"
  int __pyx_r;

#line 239 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 239 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = 
#line 239 "View.MemoryView"
__pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
  /* function exit code */

#line 239 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 239 "View.MemoryView"
  return __pyx_r;

#line 239 "View.MemoryView"
}

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {

#line 239 "View.MemoryView"
  int __pyx_r;

#line 239 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 239 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 239 "View.MemoryView"
  int __pyx_lineno = 0;

#line 239 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 239 "View.MemoryView"
  int __pyx_clineno = 0;

#line 239 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":240
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 240 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 240, __pyx_L1_error)

#line 240 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 240 "View.MemoryView"
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0)) __PYX_ERR(3, 240, __pyx_L1_error)

#line 240 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":239
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

#line 239 "View.MemoryView"


#line 239 "View.MemoryView"
  /* function exit code */

#line 239 "View.MemoryView"
  __pyx_r = 0;

#line 239 "View.MemoryView"
  goto __pyx_L0;

#line 239 "View.MemoryView"
  __pyx_L1_error:;

#line 239 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 239 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 239 "View.MemoryView"
  __pyx_r = -1;

#line 239 "View.MemoryView"
  __pyx_L0:;

#line 239 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 239 "View.MemoryView"
  return __pyx_r;

#line 239 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_array___reduce_cython__(((struct __pyx_array_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.array.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_array_2__setstate_cython__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

#line 244 "View.MemoryView"


#line 244 "View.MemoryView"
static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char *__pyx_v_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0
#line 244 "View.MemoryView"
;

#line 244 "View.MemoryView"
  struct __pyx_array_obj *__pyx_r = NULL;

#line 244 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 244 "View.MemoryView"
  int __pyx_t_1;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 244 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 244 "View.MemoryView"
  int __pyx_lineno = 0;

#line 244 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 244 "View.MemoryView"
  int __pyx_clineno = 0;

#line 244 "View.MemoryView"
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */

#line 248 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_buf == NULL) != 0);

#line 248 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":249
 * 
 *     if buf == NULL:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))             # <<<<<<<<<<<<<<
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 */

#line 249 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 249 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 249 "View.MemoryView"
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 249 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_shape);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_shape);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_2);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_3);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);

#line 249 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 249 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_2 = 0;

#line 249 "View.MemoryView"
    __pyx_t_3 = 0;

#line 249 "View.MemoryView"
    __pyx_t_4 = 0;

#line 249 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 249, __pyx_L1_error)

#line 249 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 249 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 249 "View.MemoryView"
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_4);

#line 249 "View.MemoryView"
    __pyx_t_4 = 0;

    /* "View.MemoryView":248
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */

#line 248 "View.MemoryView"
    goto __pyx_L3;

#line 248 "View.MemoryView"
  }

  /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */

#line 251 "View.MemoryView"
  /*else*/ {

#line 251 "View.MemoryView"
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 251 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 251 "View.MemoryView"
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 251 "View.MemoryView"
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 251 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_shape);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_shape);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_5);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);

#line 251 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_3);

#line 251 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);

#line 251 "View.MemoryView"
    __pyx_t_4 = 0;

#line 251 "View.MemoryView"
    __pyx_t_5 = 0;

#line 251 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":252
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
 */

#line 252 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 252, __pyx_L1_error)

#line 252 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 252 "View.MemoryView"
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_allocate_buffer, Py_False) < 0) __PYX_ERR(3, 252, __pyx_L1_error)

    /* "View.MemoryView":251
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */

#line 251 "View.MemoryView"
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 251, __pyx_L1_error)

#line 251 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 251 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 251 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 251 "View.MemoryView"
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_5);

#line 251 "View.MemoryView"
    __pyx_t_5 = 0;

    /* "View.MemoryView":253
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 253 "View.MemoryView"
    __pyx_v_result->data = __pyx_v_buf;

#line 253 "View.MemoryView"
  }

#line 253 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":255
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 255 "View.MemoryView"
  __Pyx_XDECREF(((PyObject *)__pyx_r));

#line 255 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 255 "View.MemoryView"
  __pyx_r = __pyx_v_result;

#line 255 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":244
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

#line 244 "View.MemoryView"


#line 244 "View.MemoryView"
  /* function exit code */

#line 244 "View.MemoryView"
  __pyx_L1_error:;

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 244 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 244 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 244 "View.MemoryView"
  __pyx_r = 0;

#line 244 "View.MemoryView"
  __pyx_L0:;

#line 244 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 244 "View.MemoryView"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 244 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 244 "View.MemoryView"
  return __pyx_r;

#line 244 "View.MemoryView"
}

/* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
/* Python wrapper */

#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0
#line 281 "View.MemoryView"
;

#line 281 "View.MemoryView"
  int __pyx_lineno = 0;

#line 281 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 281 "View.MemoryView"
  int __pyx_clineno = 0;

#line 281 "View.MemoryView"
  int __pyx_r;

#line 281 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 281 "View.MemoryView"
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);

#line 281 "View.MemoryView"
  {

#line 281 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};

#line 281 "View.MemoryView"
    PyObject* values[1] = {0};

#line 281 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 281 "View.MemoryView"
      Py_ssize_t kw_args;

#line 281 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 281 "View.MemoryView"
      switch (pos_args) {
        case  1: 
#line 281 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 281 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 281 "View.MemoryView"
        case  0: break;
        default: 
#line 281 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 281 "View.MemoryView"
      switch (pos_args) {

#line 281 "View.MemoryView"
        case  0:

#line 281 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else 
#line 281 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 281 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(3, 281, __pyx_L3_error)

#line 281 "View.MemoryView"
      }

#line 281 "View.MemoryView"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {

#line 281 "View.MemoryView"
      goto __pyx_L5_argtuple_error;

#line 281 "View.MemoryView"
    } else {

#line 281 "View.MemoryView"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 281 "View.MemoryView"
    }

#line 281 "View.MemoryView"
    __pyx_v_name = values[0];

#line 281 "View.MemoryView"
  }

#line 281 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 281 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); 
#line 281 "View.MemoryView"
__PYX_ERR(3, 281, __pyx_L3_error)

#line 281 "View.MemoryView"
  __pyx_L3_error:;

#line 281 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return -1;

#line 281 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 281 "View.MemoryView"
__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
  /* function exit code */

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return __pyx_r;

#line 281 "View.MemoryView"
}

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {

#line 281 "View.MemoryView"
  int __pyx_r;

#line 281 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 281 "View.MemoryView"
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":282
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
 */

#line 282 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_name);

#line 282 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_name);

#line 282 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->name);

#line 282 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->name);

#line 282 "View.MemoryView"
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":281
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

#line 281 "View.MemoryView"


#line 281 "View.MemoryView"
  /* function exit code */

#line 281 "View.MemoryView"
  __pyx_r = 0;

#line 281 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 281 "View.MemoryView"
  return __pyx_r;

#line 281 "View.MemoryView"
}

/* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
/* Python wrapper */

#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/

#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {

#line 283 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 283 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 283 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 283 "View.MemoryView"
__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
  /* function exit code */

#line 283 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 283 "View.MemoryView"
  return __pyx_r;

#line 283 "View.MemoryView"
}

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {

#line 283 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 283 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 283 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":284
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 */

#line 284 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 284 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->name);

#line 284 "View.MemoryView"
  __pyx_r = __pyx_v_self->name;

#line 284 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":283
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

#line 283 "View.MemoryView"


#line 283 "View.MemoryView"
  /* function exit code */

#line 283 "View.MemoryView"
  __pyx_L0:;

#line 283 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 283 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 283 "View.MemoryView"
  return __pyx_r;

#line 283 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_MemviewEnum___reduce_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_v_state = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v__dict = 0
#line 1 "(tree fragment)"
;
  int __pyx_v_use_setstate
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */

#line 5 "(tree fragment)"
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
  __Pyx_INCREF(__pyx_v_self->name);

#line 5 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_v_self->name);

#line 5 "(tree fragment)"
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name);

#line 5 "(tree fragment)"
  __pyx_v_state = ((PyObject*)__pyx_t_1);

#line 5 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */

#line 6 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
  __pyx_v__dict = __pyx_t_1;

#line 6 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
  __pyx_t_2 = (__pyx_v__dict != Py_None);

#line 7 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 7 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */

#line 8 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 8 "(tree fragment)"
    __Pyx_INCREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v__dict);

#line 8 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);

#line 8 "(tree fragment)"
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 8, __pyx_L1_error)

#line 8 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 8 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 8 "(tree fragment)"
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));

#line 8 "(tree fragment)"
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
 */

#line 9 "(tree fragment)"
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */

#line 7 "(tree fragment)"
    goto __pyx_L3;

#line 7 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 */

#line 11 "(tree fragment)"
  /*else*/ {

#line 11 "(tree fragment)"
    __pyx_t_3 = (__pyx_v_self->name != Py_None);

#line 11 "(tree fragment)"
    __pyx_v_use_setstate = __pyx_t_3;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  __pyx_t_3 = (__pyx_v_use_setstate != 0);

#line 12 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 */

#line 13 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 13 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 13 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_184977713);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_184977713);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);

#line 13 "(tree fragment)"
    __Pyx_INCREF(Py_None);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(Py_None);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);

#line 13 "(tree fragment)"
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_4);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);

#line 13 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 13 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 13 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);

#line 13 "(tree fragment)"
    __pyx_t_4 = 0;

#line 13 "(tree fragment)"
    __pyx_t_1 = 0;

#line 13 "(tree fragment)"
    __pyx_r = __pyx_t_5;

#line 13 "(tree fragment)"
    __pyx_t_5 = 0;

#line 13 "(tree fragment)"
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 */

#line 12 "(tree fragment)"
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, None), state
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 15 "(tree fragment)"
  /*else*/ {

#line 15 "(tree fragment)"
    __Pyx_XDECREF(__pyx_r);

#line 15 "(tree fragment)"
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_Enum); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 15 "(tree fragment)"
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 15 "(tree fragment)"
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_int_184977713);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_int_184977713);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_184977713);

#line 15 "(tree fragment)"
    __Pyx_INCREF(__pyx_v_state);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_v_state);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);

#line 15 "(tree fragment)"
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 15, __pyx_L1_error)

#line 15 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_5);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);

#line 15 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_t_1);

#line 15 "(tree fragment)"
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);

#line 15 "(tree fragment)"
    __pyx_t_5 = 0;

#line 15 "(tree fragment)"
    __pyx_t_1 = 0;

#line 15 "(tree fragment)"
    __pyx_r = __pyx_t_4;

#line 15 "(tree fragment)"
    __pyx_t_4 = 0;

#line 15 "(tree fragment)"
    goto __pyx_L0;

#line 15 "(tree fragment)"
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.Enum.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v_state);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v__dict);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
/* Python wrapper */

#line 16 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 16 "(tree fragment)"
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 16 "(tree fragment)"
__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 16 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 16 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 16 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 16 "(tree fragment)"
  int __pyx_lineno = 0;

#line 16 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 16 "(tree fragment)"
  int __pyx_clineno = 0;

#line 16 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */

#line 17 "(tree fragment)"
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(3, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __pyx_t_1 = __pyx_unpickle_Enum__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 17, __pyx_L1_error)

#line 17 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 17 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0xb068931, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
 */

#line 16 "(tree fragment)"


#line 16 "(tree fragment)"
  /* function exit code */

#line 16 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 16 "(tree fragment)"
  goto __pyx_L0;

#line 16 "(tree fragment)"
  __pyx_L1_error:;

#line 16 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 16 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.Enum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 16 "(tree fragment)"
  __pyx_r = NULL;

#line 16 "(tree fragment)"
  __pyx_L0:;

#line 16 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 16 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 16 "(tree fragment)"
  return __pyx_r;

#line 16 "(tree fragment)"
}

/* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

#line 298 "View.MemoryView"


#line 298 "View.MemoryView"
static void *__pyx_align_pointer(void *__pyx_v_memory, size_t __pyx_v_alignment) {
  Py_intptr_t __pyx_v_aligned_p
#line 298 "View.MemoryView"
;
  size_t __pyx_v_offset
#line 298 "View.MemoryView"
;

#line 298 "View.MemoryView"
  void *__pyx_r;

#line 298 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":300
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory             # <<<<<<<<<<<<<<
 *     cdef size_t offset
 * 
 */

#line 300 "View.MemoryView"
  __pyx_v_aligned_p = ((Py_intptr_t)__pyx_v_memory);

  /* "View.MemoryView":304
 * 
 *     with cython.cdivision(True):
 *         offset = aligned_p % alignment             # <<<<<<<<<<<<<<
 * 
 *     if offset > 0:
 */

#line 304 "View.MemoryView"
  __pyx_v_offset = (__pyx_v_aligned_p % __pyx_v_alignment);

  /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */

#line 306 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_offset > 0) != 0);

#line 306 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":307
 * 
 *     if offset > 0:
 *         aligned_p += alignment - offset             # <<<<<<<<<<<<<<
 * 
 *     return <void *> aligned_p
 */

#line 307 "View.MemoryView"
    __pyx_v_aligned_p = (__pyx_v_aligned_p + (__pyx_v_alignment - __pyx_v_offset));

    /* "View.MemoryView":306
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */

#line 306 "View.MemoryView"
  }

  /* "View.MemoryView":309
 *         aligned_p += alignment - offset
 * 
 *     return <void *> aligned_p             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 309 "View.MemoryView"
  __pyx_r = ((void *)__pyx_v_aligned_p);

#line 309 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":298
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

#line 298 "View.MemoryView"


#line 298 "View.MemoryView"
  /* function exit code */

#line 298 "View.MemoryView"
  __pyx_L0:;

#line 298 "View.MemoryView"
  return __pyx_r;

#line 298 "View.MemoryView"
}

/* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
/* Python wrapper */

#line 345 "View.MemoryView"
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/

#line 345 "View.MemoryView"
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0
#line 345 "View.MemoryView"
;
  int __pyx_v_flags
#line 345 "View.MemoryView"
;
  int __pyx_v_dtype_is_object
#line 345 "View.MemoryView"
;

#line 345 "View.MemoryView"
  int __pyx_lineno = 0;

#line 345 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 345 "View.MemoryView"
  int __pyx_clineno = 0;

#line 345 "View.MemoryView"
  int __pyx_r;

#line 345 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 345 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);

#line 345 "View.MemoryView"
  {

#line 345 "View.MemoryView"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_obj,&__pyx_n_s_flags,&__pyx_n_s_dtype_is_object,0};

#line 345 "View.MemoryView"
    PyObject* values[3] = {0,0,0};

#line 345 "View.MemoryView"
    if (unlikely(__pyx_kwds)) {

#line 345 "View.MemoryView"
      Py_ssize_t kw_args;

#line 345 "View.MemoryView"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 345 "View.MemoryView"
      switch (pos_args) {
        case  3: 
#line 345 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 345 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 345 "View.MemoryView"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  0: break;
        default: 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
      kw_args = PyDict_Size(__pyx_kwds);

#line 345 "View.MemoryView"
      switch (pos_args) {

#line 345 "View.MemoryView"
        case  0:

#line 345 "View.MemoryView"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_obj)) != 0)) kw_args--;
        else 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  1:

#line 345 "View.MemoryView"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;

#line 345 "View.MemoryView"
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); 
#line 345 "View.MemoryView"
__PYX_ERR(3, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
        }

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;

#line 345 "View.MemoryView"
        case  2:

#line 345 "View.MemoryView"
        if (kw_args > 0) {

#line 345 "View.MemoryView"
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype_is_object);

#line 345 "View.MemoryView"
          if (value) { values[2] = value; kw_args--; }

#line 345 "View.MemoryView"
        }

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
      if (unlikely(kw_args > 0)) {

#line 345 "View.MemoryView"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(3, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
    } else {

#line 345 "View.MemoryView"
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: 
#line 345 "View.MemoryView"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 345 "View.MemoryView"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 345 "View.MemoryView"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 345 "View.MemoryView"
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 345 "View.MemoryView"
        break;
        default: 
#line 345 "View.MemoryView"
goto __pyx_L5_argtuple_error;

#line 345 "View.MemoryView"
      }

#line 345 "View.MemoryView"
    }

#line 345 "View.MemoryView"
    __pyx_v_obj = values[0];

#line 345 "View.MemoryView"
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
    if (values[2]) {

#line 345 "View.MemoryView"
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
    } else {

#line 345 "View.MemoryView"
      __pyx_v_dtype_is_object = ((int)0);

#line 345 "View.MemoryView"
    }

#line 345 "View.MemoryView"
  }

#line 345 "View.MemoryView"
  goto __pyx_L4_argument_unpacking_done;

#line 345 "View.MemoryView"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 345 "View.MemoryView"
__PYX_ERR(3, 345, __pyx_L3_error)

#line 345 "View.MemoryView"
  __pyx_L3_error:;

#line 345 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return -1;

#line 345 "View.MemoryView"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 345 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
  /* function exit code */

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return __pyx_r;

#line 345 "View.MemoryView"
}

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {

#line 345 "View.MemoryView"
  int __pyx_r;

#line 345 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 345 "View.MemoryView"
  int __pyx_t_1;

#line 345 "View.MemoryView"
  int __pyx_t_2;

#line 345 "View.MemoryView"
  int __pyx_t_3;

#line 345 "View.MemoryView"
  int __pyx_t_4;

#line 345 "View.MemoryView"
  int __pyx_lineno = 0;

#line 345 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 345 "View.MemoryView"
  int __pyx_clineno = 0;

#line 345 "View.MemoryView"
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":346
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 */

#line 346 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

#line 346 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_obj);

#line 346 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_self->obj);

#line 346 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_self->obj);

#line 346 "View.MemoryView"
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":347
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 */

#line 347 "View.MemoryView"
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */

#line 348 "View.MemoryView"
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)__pyx_memoryview_type));

#line 348 "View.MemoryView"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 348 "View.MemoryView"
  if (!__pyx_t_3) {

#line 348 "View.MemoryView"
  } else {

#line 348 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 348 "View.MemoryView"
    goto __pyx_L4_bool_binop_done;

#line 348 "View.MemoryView"
  }

#line 348 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_obj != Py_None);

#line 348 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 348 "View.MemoryView"
  __pyx_t_1 = __pyx_t_2;

#line 348 "View.MemoryView"
  __pyx_L4_bool_binop_done:;

#line 348 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":349
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 */

#line 349 "View.MemoryView"
    __pyx_t_4 = __Pyx_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 349, __pyx_L1_error)

    /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */

#line 350 "View.MemoryView"
    __pyx_t_1 = ((((PyObject *)__pyx_v_self->view.obj) == NULL) != 0);

#line 350 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":351
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
 */

#line 351 "View.MemoryView"
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":352
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         global __pyx_memoryview_thread_locks_used
 */

#line 352 "View.MemoryView"
      Py_INCREF(Py_None);

      /* "View.MemoryView":350
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */

#line 350 "View.MemoryView"
    }

    /* "View.MemoryView":348
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */

#line 348 "View.MemoryView"
  }

  /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */

#line 355 "View.MemoryView"
  __pyx_t_1 = ((__pyx_memoryview_thread_locks_used < 8) != 0);

#line 355 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":356
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]             # <<<<<<<<<<<<<<
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 */

#line 356 "View.MemoryView"
    __pyx_v_self->lock = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

    /* "View.MemoryView":357
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1             # <<<<<<<<<<<<<<
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 */

#line 357 "View.MemoryView"
    __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used + 1);

    /* "View.MemoryView":355
 * 
 *         global __pyx_memoryview_thread_locks_used
 *         if __pyx_memoryview_thread_locks_used < THREAD_LOCKS_PREALLOCATED:             # <<<<<<<<<<<<<<
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 */

#line 355 "View.MemoryView"
  }

  /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */

#line 358 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);

#line 358 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":359
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *             if self.lock is NULL:
 *                 raise MemoryError
 */

#line 359 "View.MemoryView"
    __pyx_v_self->lock = PyThread_allocate_lock();

    /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */

#line 360 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);

#line 360 "View.MemoryView"
    if (unlikely(__pyx_t_1)) {

      /* "View.MemoryView":361
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 361 "View.MemoryView"
      PyErr_NoMemory(); __PYX_ERR(3, 361, __pyx_L1_error)

      /* "View.MemoryView":360
 *         if self.lock is NULL:
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 * 
 */

#line 360 "View.MemoryView"
    }

    /* "View.MemoryView":358
 *             self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *             __pyx_memoryview_thread_locks_used += 1
 *         if self.lock is NULL:             # <<<<<<<<<<<<<<
 *             self.lock = PyThread_allocate_lock()
 *             if self.lock is NULL:
 */

#line 358 "View.MemoryView"
  }

  /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */

#line 363 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 363 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":364
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
 */

#line 364 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_self->view.format[0]) == 'O') != 0);

#line 364 "View.MemoryView"
    if (__pyx_t_2) {

#line 364 "View.MemoryView"
    } else {

#line 364 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 364 "View.MemoryView"
      goto __pyx_L11_bool_binop_done;

#line 364 "View.MemoryView"
    }

#line 364 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_self->view.format[1]) == '\x00') != 0);

#line 364 "View.MemoryView"
    __pyx_t_1 = __pyx_t_2;

#line 364 "View.MemoryView"
    __pyx_L11_bool_binop_done:;

#line 364 "View.MemoryView"
    __pyx_v_self->dtype_is_object = __pyx_t_1;

    /* "View.MemoryView":363
 *                 raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 */

#line 363 "View.MemoryView"
    goto __pyx_L10;

#line 363 "View.MemoryView"
  }

  /* "View.MemoryView":366
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 */

#line 366 "View.MemoryView"
  /*else*/ {

#line 366 "View.MemoryView"
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;

#line 366 "View.MemoryView"
  }

#line 366 "View.MemoryView"
  __pyx_L10:;

  /* "View.MemoryView":368
 *             self.dtype_is_object = dtype_is_object
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(             # <<<<<<<<<<<<<<
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL
 */

#line 368 "View.MemoryView"
  __pyx_v_self->acquisition_count_aligned_p = ((__pyx_atomic_int *)__pyx_align_pointer(((void *)(&(__pyx_v_self->acquisition_count[0]))), (sizeof(__pyx_atomic_int))));

  /* "View.MemoryView":370
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
 */

#line 370 "View.MemoryView"
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":345
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

#line 345 "View.MemoryView"


#line 345 "View.MemoryView"
  /* function exit code */

#line 345 "View.MemoryView"
  __pyx_r = 0;

#line 345 "View.MemoryView"
  goto __pyx_L0;

#line 345 "View.MemoryView"
  __pyx_L1_error:;

#line 345 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 345 "View.MemoryView"
  __pyx_r = -1;

#line 345 "View.MemoryView"
  __pyx_L0:;

#line 345 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 345 "View.MemoryView"
  return __pyx_r;

#line 345 "View.MemoryView"
}

/* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
/* Python wrapper */

#line 372 "View.MemoryView"
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 372 "View.MemoryView"
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {

#line 372 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 372 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 372 "View.MemoryView"
  __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
  /* function exit code */

#line 372 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 372 "View.MemoryView"
}

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i
#line 372 "View.MemoryView"
;

#line 372 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 372 "View.MemoryView"
  int __pyx_t_1;

#line 372 "View.MemoryView"
  int __pyx_t_2;

#line 372 "View.MemoryView"
  int __pyx_t_3;

#line 372 "View.MemoryView"
  int __pyx_t_4;

#line 372 "View.MemoryView"
  int __pyx_t_5;

#line 372 "View.MemoryView"
  PyThread_type_lock __pyx_t_6;

#line 372 "View.MemoryView"
  PyThread_type_lock __pyx_t_7;

#line 372 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */

#line 373 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);

#line 373 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 373 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":374
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)             # <<<<<<<<<<<<<<
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 */

#line 374 "View.MemoryView"
    __Pyx_ReleaseBuffer((&__pyx_v_self->view));

    /* "View.MemoryView":373
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 */

#line 373 "View.MemoryView"
    goto __pyx_L3;

#line 373 "View.MemoryView"
  }

  /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */

#line 375 "View.MemoryView"
  __pyx_t_2 = ((((Py_buffer *)(&__pyx_v_self->view))->obj == Py_None) != 0);

#line 375 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":377
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL             # <<<<<<<<<<<<<<
 *             Py_DECREF(Py_None)
 * 
 */

#line 377 "View.MemoryView"
    ((Py_buffer *)(&__pyx_v_self->view))->obj = NULL;

    /* "View.MemoryView":378
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 *             Py_DECREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */

#line 378 "View.MemoryView"
    Py_DECREF(Py_None);

    /* "View.MemoryView":375
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 */

#line 375 "View.MemoryView"
  }

#line 375 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */

#line 382 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_self->lock != NULL) != 0);

#line 382 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":383
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):             # <<<<<<<<<<<<<<
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 */

#line 383 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_thread_locks_used;

#line 383 "View.MemoryView"
    __pyx_t_4 = __pyx_t_3;

#line 383 "View.MemoryView"
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {

#line 383 "View.MemoryView"
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */

#line 384 "View.MemoryView"
      __pyx_t_2 = (((__pyx_memoryview_thread_locks[__pyx_v_i]) == __pyx_v_self->lock) != 0);

#line 384 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":385
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1             # <<<<<<<<<<<<<<
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 */

#line 385 "View.MemoryView"
        __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used - 1);

        /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */

#line 386 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_i != __pyx_memoryview_thread_locks_used) != 0);

#line 386 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":388
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])             # <<<<<<<<<<<<<<
 *                     break
 *             else:
 */

#line 388 "View.MemoryView"
          __pyx_t_6 = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

#line 388 "View.MemoryView"
          __pyx_t_7 = (__pyx_memoryview_thread_locks[__pyx_v_i]);

          /* "View.MemoryView":387
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (             # <<<<<<<<<<<<<<
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break
 */

#line 387 "View.MemoryView"
          (__pyx_memoryview_thread_locks[__pyx_v_i]) = __pyx_t_6;

#line 387 "View.MemoryView"
          (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]) = __pyx_t_7;

          /* "View.MemoryView":386
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 */

#line 386 "View.MemoryView"
        }

        /* "View.MemoryView":389
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break             # <<<<<<<<<<<<<<
 *             else:
 *                 PyThread_free_lock(self.lock)
 */

#line 389 "View.MemoryView"
        goto __pyx_L6_break;

        /* "View.MemoryView":384
 *         if self.lock != NULL:
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 */

#line 384 "View.MemoryView"
      }

#line 384 "View.MemoryView"
    }

#line 384 "View.MemoryView"
    /*else*/ {

      /* "View.MemoryView":391
 *                     break
 *             else:
 *                 PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 */

#line 391 "View.MemoryView"
      PyThread_free_lock(__pyx_v_self->lock);

#line 391 "View.MemoryView"
    }

#line 391 "View.MemoryView"
    __pyx_L6_break:;

    /* "View.MemoryView":382
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(__pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 */

#line 382 "View.MemoryView"
  }

  /* "View.MemoryView":372
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

#line 372 "View.MemoryView"


#line 372 "View.MemoryView"
  /* function exit code */

#line 372 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 372 "View.MemoryView"
}

/* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

#line 393 "View.MemoryView"


#line 393 "View.MemoryView"
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim
#line 393 "View.MemoryView"
;
  char *__pyx_v_itemp
#line 393 "View.MemoryView"
;
  PyObject *__pyx_v_idx = NULL
#line 393 "View.MemoryView"
;

#line 393 "View.MemoryView"
  char *__pyx_r;

#line 393 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 393 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 393 "View.MemoryView"
  PyObject *(*__pyx_t_4)(PyObject *);

#line 393 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 393 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

#line 393 "View.MemoryView"
  char *__pyx_t_7;

#line 393 "View.MemoryView"
  int __pyx_lineno = 0;

#line 393 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 393 "View.MemoryView"
  int __pyx_clineno = 0;

#line 393 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":395
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
 */

#line 395 "View.MemoryView"
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */

#line 397 "View.MemoryView"
  __pyx_t_1 = 0;

#line 397 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {

#line 397 "View.MemoryView"
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;

#line 397 "View.MemoryView"
    __pyx_t_4 = NULL;

#line 397 "View.MemoryView"
  } else {
    __pyx_t_3 = -1; 
#line 397 "View.MemoryView"
__pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 397 "View.MemoryView"
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
  }

#line 397 "View.MemoryView"
  for (;;) {

#line 397 "View.MemoryView"
    if (likely(!__pyx_t_4)) {

#line 397 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_2))) {

#line 397 "View.MemoryView"
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;

#line 397 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "View.MemoryView"
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        #else

#line 397 "View.MemoryView"
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
        #endif

#line 397 "View.MemoryView"
      } else {

#line 397 "View.MemoryView"
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;

#line 397 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 397 "View.MemoryView"
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        #else

#line 397 "View.MemoryView"
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
        #endif

#line 397 "View.MemoryView"
      }
    } else 
#line 397 "View.MemoryView"
{

#line 397 "View.MemoryView"
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);

#line 397 "View.MemoryView"
      if (unlikely(!__pyx_t_5)) {

#line 397 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 397 "View.MemoryView"
        if (exc_type) {

#line 397 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 397 "View.MemoryView"
          else __PYX_ERR(3, 397, __pyx_L1_error)

#line 397 "View.MemoryView"
        }

#line 397 "View.MemoryView"
        break;

#line 397 "View.MemoryView"
      }

#line 397 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 397 "View.MemoryView"
    }

#line 397 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);

#line 397 "View.MemoryView"
    __pyx_t_5 = 0;

#line 397 "View.MemoryView"
    __pyx_v_dim = __pyx_t_1;

#line 397 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":398
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
 */

#line 398 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 398, __pyx_L1_error)

#line 398 "View.MemoryView"
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == ((char *)NULL))) __PYX_ERR(3, 398, __pyx_L1_error)

#line 398 "View.MemoryView"
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":397
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */

#line 397 "View.MemoryView"
  }

#line 397 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":400
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 400 "View.MemoryView"
  __pyx_r = __pyx_v_itemp;

#line 400 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":393
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

#line 393 "View.MemoryView"


#line 393 "View.MemoryView"
  /* function exit code */

#line 393 "View.MemoryView"
  __pyx_L1_error:;

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 393 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 393 "View.MemoryView"
  __pyx_r = NULL;

#line 393 "View.MemoryView"
  __pyx_L0:;

#line 393 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_idx);

#line 393 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 393 "View.MemoryView"
  return __pyx_r;

#line 393 "View.MemoryView"
}

/* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
/* Python wrapper */

#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/

#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {

#line 403 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 403 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 403 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = 
#line 403 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
  /* function exit code */

#line 403 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 403 "View.MemoryView"
  return __pyx_r;

#line 403 "View.MemoryView"
}

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL
#line 403 "View.MemoryView"
;
  PyObject *__pyx_v_indices = NULL
#line 403 "View.MemoryView"
;
  char *__pyx_v_itemp
#line 403 "View.MemoryView"
;

#line 403 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 403 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 403 "View.MemoryView"
  int __pyx_t_1;

#line 403 "View.MemoryView"
  int __pyx_t_2;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 403 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 403 "View.MemoryView"
  char *__pyx_t_6;

#line 403 "View.MemoryView"
  int __pyx_lineno = 0;

#line 403 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 403 "View.MemoryView"
  int __pyx_clineno = 0;

#line 403 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */

#line 404 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);

#line 404 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 404 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":405
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
 */

#line 405 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 405 "View.MemoryView"
    __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 405 "View.MemoryView"
    __pyx_r = ((PyObject *)__pyx_v_self);

#line 405 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":404
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */

#line 404 "View.MemoryView"
  }

  /* "View.MemoryView":407
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
 */

#line 407 "View.MemoryView"
  __pyx_t_3 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 407 "View.MemoryView"
  if (likely(__pyx_t_3 != Py_None)) {

#line 407 "View.MemoryView"
    PyObject* sequence = __pyx_t_3;

#line 407 "View.MemoryView"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 407 "View.MemoryView"
    if (unlikely(size != 2)) {

#line 407 "View.MemoryView"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 407 "View.MemoryView"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 407 "View.MemoryView"
      __PYX_ERR(3, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    }

#line 407 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 407 "View.MemoryView"
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 

#line 407 "View.MemoryView"
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 

#line 407 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_4);

#line 407 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_5);

#line 407 "View.MemoryView"
    #else

#line 407 "View.MemoryView"
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 407 "View.MemoryView"
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 407 "View.MemoryView"
    #endif

#line 407 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 407 "View.MemoryView"
  } else {

#line 407 "View.MemoryView"
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(3, 407, __pyx_L1_error)

#line 407 "View.MemoryView"
  }

#line 407 "View.MemoryView"
  __pyx_v_have_slices = __pyx_t_4;

#line 407 "View.MemoryView"
  __pyx_t_4 = 0;

#line 407 "View.MemoryView"
  __pyx_v_indices = __pyx_t_5;

#line 407 "View.MemoryView"
  __pyx_t_5 = 0;

  /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */

#line 410 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 410, __pyx_L1_error)

#line 410 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":411
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
 */

#line 411 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 411 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 411, __pyx_L1_error)

#line 411 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 411 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 411 "View.MemoryView"
    __pyx_t_3 = 0;

#line 411 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":410
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */

#line 410 "View.MemoryView"
  }

  /* "View.MemoryView":413
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
 */

#line 413 "View.MemoryView"
  /*else*/ {

#line 413 "View.MemoryView"
    __pyx_t_6 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_6 == ((char *)NULL))) __PYX_ERR(3, 413, __pyx_L1_error)

#line 413 "View.MemoryView"
    __pyx_v_itemp = __pyx_t_6;

    /* "View.MemoryView":414
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
 */

#line 414 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 414 "View.MemoryView"
    __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 414, __pyx_L1_error)

#line 414 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 414 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 414 "View.MemoryView"
    __pyx_t_3 = 0;

#line 414 "View.MemoryView"
    goto __pyx_L0;

#line 414 "View.MemoryView"
  }

  /* "View.MemoryView":403
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

#line 403 "View.MemoryView"


#line 403 "View.MemoryView"
  /* function exit code */

#line 403 "View.MemoryView"
  __pyx_L1_error:;

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 403 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 403 "View.MemoryView"
  __pyx_r = NULL;

#line 403 "View.MemoryView"
  __pyx_L0:;

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_have_slices);

#line 403 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_indices);

#line 403 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 403 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 403 "View.MemoryView"
  return __pyx_r;

#line 403 "View.MemoryView"
}

/* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
/* Python wrapper */

#line 416 "View.MemoryView"
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/

#line 416 "View.MemoryView"
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {

#line 416 "View.MemoryView"
  int __pyx_r;

#line 416 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 416 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = 
#line 416 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
  /* function exit code */

#line 416 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 416 "View.MemoryView"
  return __pyx_r;

#line 416 "View.MemoryView"
}

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL
#line 416 "View.MemoryView"
;
  PyObject *__pyx_v_obj = NULL
#line 416 "View.MemoryView"
;

#line 416 "View.MemoryView"
  int __pyx_r;

#line 416 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 416 "View.MemoryView"
  int __pyx_t_1;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 416 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 416 "View.MemoryView"
  int __pyx_lineno = 0;

#line 416 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 416 "View.MemoryView"
  int __pyx_clineno = 0;

#line 416 "View.MemoryView"
  __Pyx_RefNannySetupContext("__setitem__", 0);

#line 416 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */

#line 417 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->view.readonly != 0);

#line 417 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */

#line 418 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 418, __pyx_L1_error)

#line 418 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 418 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 418 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 418 "View.MemoryView"
    __PYX_ERR(3, 418, __pyx_L1_error)

    /* "View.MemoryView":417
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 */

#line 417 "View.MemoryView"
  }

  /* "View.MemoryView":420
 *             raise TypeError("Cannot assign to read-only memoryview")
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
 */

#line 420 "View.MemoryView"
  __pyx_t_2 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 420 "View.MemoryView"
  if (likely(__pyx_t_2 != Py_None)) {

#line 420 "View.MemoryView"
    PyObject* sequence = __pyx_t_2;

#line 420 "View.MemoryView"
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);

#line 420 "View.MemoryView"
    if (unlikely(size != 2)) {

#line 420 "View.MemoryView"
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);

#line 420 "View.MemoryView"
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);

#line 420 "View.MemoryView"
      __PYX_ERR(3, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    }

#line 420 "View.MemoryView"
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 420 "View.MemoryView"
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 

#line 420 "View.MemoryView"
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 

#line 420 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 420 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_4);

#line 420 "View.MemoryView"
    #else

#line 420 "View.MemoryView"
    __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 420 "View.MemoryView"
    __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 420 "View.MemoryView"
    #endif

#line 420 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 420 "View.MemoryView"
  } else {

#line 420 "View.MemoryView"
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(3, 420, __pyx_L1_error)

#line 420 "View.MemoryView"
  }

#line 420 "View.MemoryView"
  __pyx_v_have_slices = __pyx_t_3;

#line 420 "View.MemoryView"
  __pyx_t_3 = 0;

#line 420 "View.MemoryView"
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_4);

#line 420 "View.MemoryView"
  __pyx_t_4 = 0;

  /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */

#line 422 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 422, __pyx_L1_error)

#line 422 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":423
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)
 */

#line 423 "View.MemoryView"
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 423, __pyx_L1_error)

#line 423 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 423 "View.MemoryView"
    __pyx_v_obj = __pyx_t_2;

#line 423 "View.MemoryView"
    __pyx_t_2 = 0;

    /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */

#line 424 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_obj); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 424, __pyx_L1_error)

#line 424 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":425
 *             obj = self.is_slice(value)
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
 */

#line 425 "View.MemoryView"
      __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 425, __pyx_L1_error)

#line 425 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_2);

#line 425 "View.MemoryView"
      __pyx_t_4 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_2, __pyx_v_obj); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 425, __pyx_L1_error)

#line 425 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 425 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 425 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":424
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */

#line 424 "View.MemoryView"
      goto __pyx_L5;

#line 424 "View.MemoryView"
    }

    /* "View.MemoryView":427
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
 */

#line 427 "View.MemoryView"
    /*else*/ {

#line 427 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 427 "View.MemoryView"
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_memoryview_type))))) __PYX_ERR(3, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_4), __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 427, __pyx_L1_error)

#line 427 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_2);

#line 427 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 427 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 427 "View.MemoryView"
    }

#line 427 "View.MemoryView"
    __pyx_L5:;

    /* "View.MemoryView":422
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */

#line 422 "View.MemoryView"
    goto __pyx_L4;

#line 422 "View.MemoryView"
  }

  /* "View.MemoryView":429
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
 */

#line 429 "View.MemoryView"
  /*else*/ {

#line 429 "View.MemoryView"
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 429, __pyx_L1_error)

#line 429 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 429 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 429 "View.MemoryView"
  }

#line 429 "View.MemoryView"
  __pyx_L4:;

  /* "View.MemoryView":416
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")
 */

#line 416 "View.MemoryView"


#line 416 "View.MemoryView"
  /* function exit code */

#line 416 "View.MemoryView"
  __pyx_r = 0;

#line 416 "View.MemoryView"
  goto __pyx_L0;

#line 416 "View.MemoryView"
  __pyx_L1_error:;

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 416 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 416 "View.MemoryView"
  __pyx_r = -1;

#line 416 "View.MemoryView"
  __pyx_L0:;

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_have_slices);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_obj);

#line 416 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_index);

#line 416 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 416 "View.MemoryView"
  return __pyx_r;

#line 416 "View.MemoryView"
}

/* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

#line 431 "View.MemoryView"


#line 431 "View.MemoryView"
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {

#line 431 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 431 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 431 "View.MemoryView"
  int __pyx_t_1;

#line 431 "View.MemoryView"
  int __pyx_t_2;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 431 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 431 "View.MemoryView"
  int __pyx_t_9;

#line 431 "View.MemoryView"
  int __pyx_lineno = 0;

#line 431 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 431 "View.MemoryView"
  int __pyx_clineno = 0;

#line 431 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_slice", 0);

#line 431 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */

#line 432 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_memoryview_type); 

#line 432 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 432 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
    {

#line 433 "View.MemoryView"
      __Pyx_PyThreadState_declare

#line 433 "View.MemoryView"
      __Pyx_PyThreadState_assign

#line 433 "View.MemoryView"
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_5);

#line 433 "View.MemoryView"
      /*try:*/ {

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */

#line 434 "View.MemoryView"
        __pyx_t_6 = __Pyx_PyInt_From_int(((__pyx_v_self->flags & (~PyBUF_WRITABLE)) | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":435
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
 */

#line 435 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 435, __pyx_L4_error)

#line 435 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

        /* "View.MemoryView":434
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */

#line 434 "View.MemoryView"
        __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_8);

#line 434 "View.MemoryView"
        __Pyx_INCREF(__pyx_v_obj);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_v_obj);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_obj);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_6);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);

#line 434 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_7);

#line 434 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);

#line 434 "View.MemoryView"
        __pyx_t_6 = 0;

#line 434 "View.MemoryView"
        __pyx_t_7 = 0;

#line 434 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 434, __pyx_L4_error)

#line 434 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 434 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 434 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_v_obj, __pyx_t_7);

#line 434 "View.MemoryView"
        __pyx_t_7 = 0;

        /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
      }

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 433 "View.MemoryView"
      goto __pyx_L9_try_end;

#line 433 "View.MemoryView"
      __pyx_L4_error:;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 433 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "View.MemoryView":436
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */

#line 436 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);

#line 436 "View.MemoryView"
      if (__pyx_t_9) {

#line 436 "View.MemoryView"
        __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 436 "View.MemoryView"
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_6) < 0) __PYX_ERR(3, 436, __pyx_L6_except_error)

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_8);

#line 436 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":437
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
 */

#line 437 "View.MemoryView"
        __Pyx_XDECREF(__pyx_r);

#line 437 "View.MemoryView"
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 437 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 437 "View.MemoryView"
        goto __pyx_L7_except_return;

#line 437 "View.MemoryView"
      }

#line 437 "View.MemoryView"
      goto __pyx_L6_except_error;

#line 437 "View.MemoryView"
      __pyx_L6_except_error:;

      /* "View.MemoryView":433
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 433 "View.MemoryView"
      goto __pyx_L1_error;

#line 433 "View.MemoryView"
      __pyx_L7_except_return:;

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_3);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_4);

#line 433 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_5);

#line 433 "View.MemoryView"
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);

#line 433 "View.MemoryView"
      goto __pyx_L0;

#line 433 "View.MemoryView"
      __pyx_L9_try_end:;

#line 433 "View.MemoryView"
    }

    /* "View.MemoryView":432
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 */

#line 432 "View.MemoryView"
  }

  /* "View.MemoryView":439
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
 */

#line 439 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 439 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_obj);

#line 439 "View.MemoryView"
  __pyx_r = __pyx_v_obj;

#line 439 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":431
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

#line 431 "View.MemoryView"


#line 431 "View.MemoryView"
  /* function exit code */

#line 431 "View.MemoryView"
  __pyx_L1_error:;

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_8);

#line 431 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 431 "View.MemoryView"
  __pyx_r = 0;

#line 431 "View.MemoryView"
  __pyx_L0:;

#line 431 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_obj);

#line 431 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 431 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 431 "View.MemoryView"
  return __pyx_r;

#line 431 "View.MemoryView"
}

/* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

#line 441 "View.MemoryView"


#line 441 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice
#line 441 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_src_slice
#line 441 "View.MemoryView"
;

#line 441 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 441 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 441 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 441 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_2;

#line 441 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 441 "View.MemoryView"
  int __pyx_t_4;

#line 441 "View.MemoryView"
  int __pyx_t_5;

#line 441 "View.MemoryView"
  int __pyx_t_6;

#line 441 "View.MemoryView"
  int __pyx_lineno = 0;

#line 441 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 441 "View.MemoryView"
  int __pyx_clineno = 0;

#line 441 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */

#line 445 "View.MemoryView"
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_memoryview_type))))) __PYX_ERR(3, 445, __pyx_L1_error)

#line 445 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 445, __pyx_L1_error)

  /* "View.MemoryView":446
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],             # <<<<<<<<<<<<<<
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 */

#line 446 "View.MemoryView"
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_memoryview_type))))) __PYX_ERR(3, 446, __pyx_L1_error)

#line 446 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice)); if (unlikely(__pyx_t_2 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 446, __pyx_L1_error)

  /* "View.MemoryView":447
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 */

#line 447 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 447 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 447 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 447 "View.MemoryView"
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 447, __pyx_L1_error)

#line 447 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":445
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */

#line 445 "View.MemoryView"
  __pyx_t_6 = __pyx_memoryview_copy_contents((__pyx_t_1[0]), (__pyx_t_2[0]), __pyx_t_4, __pyx_t_5, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 445, __pyx_L1_error)

  /* "View.MemoryView":441
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

#line 441 "View.MemoryView"


#line 441 "View.MemoryView"
  /* function exit code */

#line 441 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 441 "View.MemoryView"
  goto __pyx_L0;

#line 441 "View.MemoryView"
  __pyx_L1_error:;

#line 441 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 441 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 441 "View.MemoryView"
  __pyx_r = 0;

#line 441 "View.MemoryView"
  __pyx_L0:;

#line 441 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 441 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 441 "View.MemoryView"
  return __pyx_r;

#line 441 "View.MemoryView"
}

/* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

#line 449 "View.MemoryView"


#line 449 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[0x80]
#line 449 "View.MemoryView"
;
  void *__pyx_v_tmp
#line 449 "View.MemoryView"
;
  void *__pyx_v_item
#line 449 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_dst_slice
#line 449 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp_slice
#line 449 "View.MemoryView"
;

#line 449 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 449 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 449 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 449 "View.MemoryView"
  int __pyx_t_2;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 449 "View.MemoryView"
  int __pyx_t_4;

#line 449 "View.MemoryView"
  int __pyx_t_5;

#line 449 "View.MemoryView"
  char const *__pyx_t_6;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_11 = NULL;

#line 449 "View.MemoryView"
  PyObject *__pyx_t_12 = NULL;

#line 449 "View.MemoryView"
  int __pyx_lineno = 0;

#line 449 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 449 "View.MemoryView"
  int __pyx_clineno = 0;

#line 449 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":451
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
 */

#line 451 "View.MemoryView"
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":456
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 */

#line 456 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 456, __pyx_L1_error)

#line 456 "View.MemoryView"
  __pyx_v_dst_slice = __pyx_t_1;

  /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */

#line 458 "View.MemoryView"
  __pyx_t_2 = ((((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array))) != 0);

#line 458 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":459
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
 */

#line 459 "View.MemoryView"
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */

#line 460 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_tmp == NULL) != 0);

#line 460 "View.MemoryView"
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":461
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
 */

#line 461 "View.MemoryView"
      PyErr_NoMemory(); __PYX_ERR(3, 461, __pyx_L1_error)

      /* "View.MemoryView":460
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */

#line 460 "View.MemoryView"
    }

    /* "View.MemoryView":462
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
 */

#line 462 "View.MemoryView"
    __pyx_v_item = __pyx_v_tmp;

    /* "View.MemoryView":458
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */

#line 458 "View.MemoryView"
    goto __pyx_L3;

#line 458 "View.MemoryView"
  }

  /* "View.MemoryView":464
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
 */

#line 464 "View.MemoryView"
  /*else*/ {

#line 464 "View.MemoryView"
    __pyx_v_item = ((void *)__pyx_v_array);

#line 464 "View.MemoryView"
  }

#line 464 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":466
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
 */

#line 466 "View.MemoryView"
  /*try:*/ {

    /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */

#line 467 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_self->dtype_is_object != 0);

#line 467 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":468
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
 */

#line 468 "View.MemoryView"
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);

      /* "View.MemoryView":467
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */

#line 467 "View.MemoryView"
      goto __pyx_L8;

#line 467 "View.MemoryView"
    }

    /* "View.MemoryView":470
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 470 "View.MemoryView"
    /*else*/ {

#line 470 "View.MemoryView"
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 470, __pyx_L6_error)

#line 470 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 470 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 470 "View.MemoryView"
    }

#line 470 "View.MemoryView"
    __pyx_L8:;

    /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */

#line 474 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_self->view.suboffsets != NULL) != 0);

#line 474 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":475
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
 */

#line 475 "View.MemoryView"
      __pyx_t_3 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 475, __pyx_L6_error)

#line 475 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 475 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "View.MemoryView":474
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */

#line 474 "View.MemoryView"
    }

    /* "View.MemoryView":476
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
 */

#line 476 "View.MemoryView"
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);

#line 476 "View.MemoryView"
  }

  /* "View.MemoryView":479
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
 */

#line 479 "View.MemoryView"
  /*finally:*/ {

#line 479 "View.MemoryView"
    /*normal exit:*/{

#line 479 "View.MemoryView"
      PyMem_Free(__pyx_v_tmp);

#line 479 "View.MemoryView"
      goto __pyx_L7;

#line 479 "View.MemoryView"
    }

#line 479 "View.MemoryView"
    __pyx_L6_error:;

#line 479 "View.MemoryView"
    /*exception exit:*/{

#line 479 "View.MemoryView"
      __Pyx_PyThreadState_declare

#line 479 "View.MemoryView"
      __Pyx_PyThreadState_assign

#line 479 "View.MemoryView"
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;

#line 479 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 479 "View.MemoryView"
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);

#line 479 "View.MemoryView"
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_7);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_8);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_10);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_11);

#line 479 "View.MemoryView"
      __Pyx_XGOTREF(__pyx_t_12);

#line 479 "View.MemoryView"
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;

#line 479 "View.MemoryView"
      {

#line 479 "View.MemoryView"
        PyMem_Free(__pyx_v_tmp);

#line 479 "View.MemoryView"
      }

#line 479 "View.MemoryView"
      if (PY_MAJOR_VERSION >= 3) {

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_10);

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_11);

#line 479 "View.MemoryView"
        __Pyx_XGIVEREF(__pyx_t_12);

#line 479 "View.MemoryView"
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);

#line 479 "View.MemoryView"
      }

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_7);

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_8);

#line 479 "View.MemoryView"
      __Pyx_XGIVEREF(__pyx_t_9);

#line 479 "View.MemoryView"
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);

#line 479 "View.MemoryView"
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;

#line 479 "View.MemoryView"
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;

#line 479 "View.MemoryView"
      goto __pyx_L1_error;

#line 479 "View.MemoryView"
    }

#line 479 "View.MemoryView"
    __pyx_L7:;

#line 479 "View.MemoryView"
  }

  /* "View.MemoryView":449
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

#line 449 "View.MemoryView"


#line 449 "View.MemoryView"
  /* function exit code */

#line 449 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 449 "View.MemoryView"
  goto __pyx_L0;

#line 449 "View.MemoryView"
  __pyx_L1_error:;

#line 449 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 449 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 449 "View.MemoryView"
  __pyx_r = 0;

#line 449 "View.MemoryView"
  __pyx_L0:;

#line 449 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 449 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 449 "View.MemoryView"
  return __pyx_r;

#line 449 "View.MemoryView"
}

/* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

#line 481 "View.MemoryView"


#line 481 "View.MemoryView"
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp
#line 481 "View.MemoryView"
;

#line 481 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 481 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 481 "View.MemoryView"
  char *__pyx_t_1;

#line 481 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 481 "View.MemoryView"
  int __pyx_lineno = 0;

#line 481 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 481 "View.MemoryView"
  int __pyx_clineno = 0;

#line 481 "View.MemoryView"
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":482
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
 */

#line 482 "View.MemoryView"
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(3, 482, __pyx_L1_error)

#line 482 "View.MemoryView"
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":483
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */

#line 483 "View.MemoryView"
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 483, __pyx_L1_error)

#line 483 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 483 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":481
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

#line 481 "View.MemoryView"


#line 481 "View.MemoryView"
  /* function exit code */

#line 481 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 481 "View.MemoryView"
  goto __pyx_L0;

#line 481 "View.MemoryView"
  __pyx_L1_error:;

#line 481 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 481 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 481 "View.MemoryView"
  __pyx_r = 0;

#line 481 "View.MemoryView"
  __pyx_L0:;

#line 481 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 481 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 481 "View.MemoryView"
  return __pyx_r;

#line 481 "View.MemoryView"
}

/* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 485 "View.MemoryView"


#line 485 "View.MemoryView"
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL
#line 485 "View.MemoryView"
;
  PyObject *__pyx_v_bytesitem = 0
#line 485 "View.MemoryView"
;
  PyObject *__pyx_v_result = NULL
#line 485 "View.MemoryView"
;

#line 485 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 485 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 485 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 485 "View.MemoryView"
  int __pyx_t_8;

#line 485 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 485 "View.MemoryView"
  size_t __pyx_t_10;

#line 485 "View.MemoryView"
  int __pyx_t_11;

#line 485 "View.MemoryView"
  int __pyx_lineno = 0;

#line 485 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 485 "View.MemoryView"
  int __pyx_clineno = 0;

#line 485 "View.MemoryView"
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":488
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
 */

#line 488 "View.MemoryView"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 488, __pyx_L1_error)

#line 488 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 488 "View.MemoryView"
  __pyx_v_struct = __pyx_t_1;

#line 488 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":491
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 */

#line 491 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 491, __pyx_L1_error)

#line 491 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 491 "View.MemoryView"
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);

#line 491 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
  {

#line 492 "View.MemoryView"
    __Pyx_PyThreadState_declare

#line 492 "View.MemoryView"
    __Pyx_PyThreadState_assign

#line 492 "View.MemoryView"
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGOTREF(__pyx_t_4);

#line 492 "View.MemoryView"
    /*try:*/ {

      /* "View.MemoryView":493
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")
 */

#line 493 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 493 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 493 "View.MemoryView"
      __pyx_t_7 = NULL;

#line 493 "View.MemoryView"
      __pyx_t_8 = 0;

#line 493 "View.MemoryView"
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {

#line 493 "View.MemoryView"
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);

#line 493 "View.MemoryView"
        if (likely(__pyx_t_7)) {

#line 493 "View.MemoryView"
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);

#line 493 "View.MemoryView"
          __Pyx_INCREF(__pyx_t_7);

#line 493 "View.MemoryView"
          __Pyx_INCREF(function);

#line 493 "View.MemoryView"
          __Pyx_DECREF_SET(__pyx_t_5, function);

#line 493 "View.MemoryView"
          __pyx_t_8 = 1;

#line 493 "View.MemoryView"
        }

#line 493 "View.MemoryView"
      }

#line 493 "View.MemoryView"
      #if CYTHON_FAST_PYCALL

#line 493 "View.MemoryView"
      if (PyFunction_Check(__pyx_t_5)) {

#line 493 "View.MemoryView"
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 493 "View.MemoryView"
      } else

#line 493 "View.MemoryView"
      #endif

#line 493 "View.MemoryView"
      #if CYTHON_FAST_PYCCALL

#line 493 "View.MemoryView"
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {

#line 493 "View.MemoryView"
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_t_6, __pyx_v_bytesitem};

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 493 "View.MemoryView"
      } else

#line 493 "View.MemoryView"
      #endif

#line 493 "View.MemoryView"
      {

#line 493 "View.MemoryView"
        __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 493 "View.MemoryView"
        if (__pyx_t_7) {

#line 493 "View.MemoryView"
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;

#line 493 "View.MemoryView"
        }

#line 493 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_6);

#line 493 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);

#line 493 "View.MemoryView"
        __Pyx_INCREF(__pyx_v_bytesitem);

#line 493 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_v_bytesitem);

#line 493 "View.MemoryView"
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_bytesitem);

#line 493 "View.MemoryView"
        __pyx_t_6 = 0;

#line 493 "View.MemoryView"
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 493, __pyx_L3_error)

#line 493 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 493 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 493 "View.MemoryView"
      }

#line 493 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 493 "View.MemoryView"
      __pyx_v_result = __pyx_t_1;

#line 493 "View.MemoryView"
      __pyx_t_1 = 0;

      /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
    }

    /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */

#line 497 "View.MemoryView"
    /*else:*/ {

#line 497 "View.MemoryView"
      __pyx_t_10 = strlen(__pyx_v_self->view.format); 

#line 497 "View.MemoryView"
      __pyx_t_11 = ((__pyx_t_10 == 1) != 0);

#line 497 "View.MemoryView"
      if (__pyx_t_11) {

        /* "View.MemoryView":498
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
 */

#line 498 "View.MemoryView"
        __Pyx_XDECREF(__pyx_r);

#line 498 "View.MemoryView"
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 498, __pyx_L5_except_error)

#line 498 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_1);

#line 498 "View.MemoryView"
        __pyx_r = __pyx_t_1;

#line 498 "View.MemoryView"
        __pyx_t_1 = 0;

#line 498 "View.MemoryView"
        goto __pyx_L6_except_return;

        /* "View.MemoryView":497
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */

#line 497 "View.MemoryView"
      }

      /* "View.MemoryView":499
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */

#line 499 "View.MemoryView"
      __Pyx_XDECREF(__pyx_r);

#line 499 "View.MemoryView"
      __Pyx_INCREF(__pyx_v_result);

#line 499 "View.MemoryView"
      __pyx_r = __pyx_v_result;

#line 499 "View.MemoryView"
      goto __pyx_L6_except_return;

#line 499 "View.MemoryView"
    }

#line 499 "View.MemoryView"
    __pyx_L3_error:;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 499 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "View.MemoryView":494
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError("Unable to convert item to object")
 *         else:
 */

#line 494 "View.MemoryView"
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_5, &__pyx_t_9);

#line 494 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 494, __pyx_L5_except_error)

#line 494 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 494 "View.MemoryView"
    __pyx_t_8 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_6);

#line 494 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 494 "View.MemoryView"
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_5, __pyx_t_9);

#line 494 "View.MemoryView"
    __pyx_t_1 = 0; __pyx_t_5 = 0; __pyx_t_9 = 0;

#line 494 "View.MemoryView"
    if (__pyx_t_8) {

#line 494 "View.MemoryView"
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 494 "View.MemoryView"
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_5, &__pyx_t_1) < 0) __PYX_ERR(3, 494, __pyx_L5_except_error)

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 494 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_1);

      /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */

#line 495 "View.MemoryView"
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 495, __pyx_L5_except_error)

#line 495 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 495 "View.MemoryView"
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);

#line 495 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 495 "View.MemoryView"
      __PYX_ERR(3, 495, __pyx_L5_except_error)

#line 495 "View.MemoryView"
    }

#line 495 "View.MemoryView"
    goto __pyx_L5_except_error;

#line 495 "View.MemoryView"
    __pyx_L5_except_error:;

    /* "View.MemoryView":492
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);

#line 492 "View.MemoryView"
    goto __pyx_L1_error;

#line 492 "View.MemoryView"
    __pyx_L6_except_return:;

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_2);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_3);

#line 492 "View.MemoryView"
    __Pyx_XGIVEREF(__pyx_t_4);

#line 492 "View.MemoryView"
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);

#line 492 "View.MemoryView"
    goto __pyx_L0;

#line 492 "View.MemoryView"
  }

  /* "View.MemoryView":485
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 485 "View.MemoryView"


#line 485 "View.MemoryView"
  /* function exit code */

#line 485 "View.MemoryView"
  __pyx_L1_error:;

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_9);

#line 485 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 485 "View.MemoryView"
  __pyx_r = 0;

#line 485 "View.MemoryView"
  __pyx_L0:;

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_struct);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_bytesitem);

#line 485 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 485 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 485 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 485 "View.MemoryView"
  return __pyx_r;

#line 485 "View.MemoryView"
}

/* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 501 "View.MemoryView"


#line 501 "View.MemoryView"
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL
#line 501 "View.MemoryView"
;
  char __pyx_v_c
#line 501 "View.MemoryView"
;
  PyObject *__pyx_v_bytesvalue = 0
#line 501 "View.MemoryView"
;
  Py_ssize_t __pyx_v_i
#line 501 "View.MemoryView"
;

#line 501 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 501 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 501 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 501 "View.MemoryView"
  int __pyx_t_2;

#line 501 "View.MemoryView"
  int __pyx_t_3;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 501 "View.MemoryView"
  int __pyx_t_7;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_8 = NULL;

#line 501 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 501 "View.MemoryView"
  PyObject *__pyx_t_10 = NULL;

#line 501 "View.MemoryView"
  char *__pyx_t_11;

#line 501 "View.MemoryView"
  char *__pyx_t_12;

#line 501 "View.MemoryView"
  char *__pyx_t_13;

#line 501 "View.MemoryView"
  char *__pyx_t_14;

#line 501 "View.MemoryView"
  int __pyx_lineno = 0;

#line 501 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 501 "View.MemoryView"
  int __pyx_clineno = 0;

#line 501 "View.MemoryView"
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":504
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
 */

#line 504 "View.MemoryView"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 504, __pyx_L1_error)

#line 504 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 504 "View.MemoryView"
  __pyx_v_struct = __pyx_t_1;

#line 504 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */

#line 509 "View.MemoryView"
  __pyx_t_2 = PyTuple_Check(__pyx_v_value); 

#line 509 "View.MemoryView"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 509 "View.MemoryView"
  if (__pyx_t_3) {

    /* "View.MemoryView":510
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
 */

#line 510 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_1);

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 510 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_t_4);

#line 510 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_4 = 0;

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 510 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 510 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 510 "View.MemoryView"
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(3, 510, __pyx_L1_error)

#line 510 "View.MemoryView"
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);

#line 510 "View.MemoryView"
    __pyx_t_4 = 0;

    /* "View.MemoryView":509
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */

#line 509 "View.MemoryView"
    goto __pyx_L3;

#line 509 "View.MemoryView"
  }

  /* "View.MemoryView":512
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
 */

#line 512 "View.MemoryView"
  /*else*/ {

#line 512 "View.MemoryView"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 512 "View.MemoryView"
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_1);

#line 512 "View.MemoryView"
    __pyx_t_5 = NULL;

#line 512 "View.MemoryView"
    __pyx_t_7 = 0;

#line 512 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {

#line 512 "View.MemoryView"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);

#line 512 "View.MemoryView"
      if (likely(__pyx_t_5)) {

#line 512 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);

#line 512 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_5);

#line 512 "View.MemoryView"
        __Pyx_INCREF(function);

#line 512 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_6, function);

#line 512 "View.MemoryView"
        __pyx_t_7 = 1;

#line 512 "View.MemoryView"
      }

#line 512 "View.MemoryView"
    }

#line 512 "View.MemoryView"
    #if CYTHON_FAST_PYCALL

#line 512 "View.MemoryView"
    if (PyFunction_Check(__pyx_t_6)) {

#line 512 "View.MemoryView"
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "View.MemoryView"
    } else

#line 512 "View.MemoryView"
    #endif

#line 512 "View.MemoryView"
    #if CYTHON_FAST_PYCCALL

#line 512 "View.MemoryView"
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {

#line 512 "View.MemoryView"
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_1, __pyx_v_value};

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 512 "View.MemoryView"
    } else

#line 512 "View.MemoryView"
    #endif

#line 512 "View.MemoryView"
    {

#line 512 "View.MemoryView"
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_8);

#line 512 "View.MemoryView"
      if (__pyx_t_5) {

#line 512 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;

#line 512 "View.MemoryView"
      }

#line 512 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_t_1);

#line 512 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_1);

#line 512 "View.MemoryView"
      __Pyx_INCREF(__pyx_v_value);

#line 512 "View.MemoryView"
      __Pyx_GIVEREF(__pyx_v_value);

#line 512 "View.MemoryView"
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_value);

#line 512 "View.MemoryView"
      __pyx_t_1 = 0;

#line 512 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 512 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 512 "View.MemoryView"
    }

#line 512 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 512 "View.MemoryView"
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(3, 512, __pyx_L1_error)

#line 512 "View.MemoryView"
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);

#line 512 "View.MemoryView"
    __pyx_t_4 = 0;

#line 512 "View.MemoryView"
  }

#line 512 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */

#line 514 "View.MemoryView"
  __pyx_t_9 = 0;

#line 514 "View.MemoryView"
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {

#line 514 "View.MemoryView"
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");

#line 514 "View.MemoryView"
    __PYX_ERR(3, 514, __pyx_L1_error)

#line 514 "View.MemoryView"
  }

#line 514 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_bytesvalue);

#line 514 "View.MemoryView"
  __pyx_t_10 = __pyx_v_bytesvalue;

#line 514 "View.MemoryView"
  __pyx_t_12 = PyBytes_AS_STRING(__pyx_t_10);

#line 514 "View.MemoryView"
  __pyx_t_13 = (__pyx_t_12 + PyBytes_GET_SIZE(__pyx_t_10));

#line 514 "View.MemoryView"
  for (__pyx_t_14 = __pyx_t_12; __pyx_t_14 < __pyx_t_13; __pyx_t_14++) {

#line 514 "View.MemoryView"
    __pyx_t_11 = __pyx_t_14;

#line 514 "View.MemoryView"
    __pyx_v_c = (__pyx_t_11[0]);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 515 "View.MemoryView"
    __pyx_v_i = __pyx_t_9;

    /* "View.MemoryView":514
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */

#line 514 "View.MemoryView"
    __pyx_t_9 = (__pyx_t_9 + 1);

    /* "View.MemoryView":515
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */

#line 515 "View.MemoryView"
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;

#line 515 "View.MemoryView"
  }

#line 515 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "View.MemoryView":501
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

#line 501 "View.MemoryView"


#line 501 "View.MemoryView"
  /* function exit code */

#line 501 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 501 "View.MemoryView"
  goto __pyx_L0;

#line 501 "View.MemoryView"
  __pyx_L1_error:;

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_8);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_10);

#line 501 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 501 "View.MemoryView"
  __pyx_r = 0;

#line 501 "View.MemoryView"
  __pyx_L0:;

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_struct);

#line 501 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_bytesvalue);

#line 501 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 501 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 501 "View.MemoryView"
  return __pyx_r;

#line 501 "View.MemoryView"
}

/* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
/* Python wrapper */

#line 518 "View.MemoryView"
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/

#line 518 "View.MemoryView"
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 518 "View.MemoryView"
  int __pyx_r;

#line 518 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 518 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = 
#line 518 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
  /* function exit code */

#line 518 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 518 "View.MemoryView"
  return __pyx_r;

#line 518 "View.MemoryView"
}

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {

#line 518 "View.MemoryView"
  int __pyx_r;

#line 518 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 518 "View.MemoryView"
  int __pyx_t_1;

#line 518 "View.MemoryView"
  int __pyx_t_2;

#line 518 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 518 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 518 "View.MemoryView"
  char *__pyx_t_5;

#line 518 "View.MemoryView"
  void *__pyx_t_6;

#line 518 "View.MemoryView"
  int __pyx_t_7;

#line 518 "View.MemoryView"
  Py_ssize_t __pyx_t_8;

#line 518 "View.MemoryView"
  int __pyx_lineno = 0;

#line 518 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 518 "View.MemoryView"
  int __pyx_clineno = 0;

#line 518 "View.MemoryView"
  if (__pyx_v_info == NULL) {

#line 518 "View.MemoryView"
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");

#line 518 "View.MemoryView"
    return -1;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  __Pyx_RefNannySetupContext("__getbuffer__", 0);

#line 518 "View.MemoryView"
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);

#line 518 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */

#line 519 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_flags & PyBUF_WRITABLE) != 0);

#line 519 "View.MemoryView"
  if (__pyx_t_2) {

#line 519 "View.MemoryView"
  } else {

#line 519 "View.MemoryView"
    __pyx_t_1 = __pyx_t_2;

#line 519 "View.MemoryView"
    goto __pyx_L4_bool_binop_done;

#line 519 "View.MemoryView"
  }

#line 519 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_self->view.readonly != 0);

#line 519 "View.MemoryView"
  __pyx_t_1 = __pyx_t_2;

#line 519 "View.MemoryView"
  __pyx_L4_bool_binop_done:;

#line 519 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */

#line 520 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 520, __pyx_L1_error)

#line 520 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 520 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 520 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 520 "View.MemoryView"
    __PYX_ERR(3, 520, __pyx_L1_error)

    /* "View.MemoryView":519
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 */

#line 519 "View.MemoryView"
  }

  /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */

#line 522 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_ND) != 0);

#line 522 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":523
 * 
 *         if flags & PyBUF_ND:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
 */

#line 523 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.shape;

#line 523 "View.MemoryView"
    __pyx_v_info->shape = __pyx_t_4;

    /* "View.MemoryView":522
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */

#line 522 "View.MemoryView"
    goto __pyx_L6;

#line 522 "View.MemoryView"
  }

  /* "View.MemoryView":525
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
 */

#line 525 "View.MemoryView"
  /*else*/ {

#line 525 "View.MemoryView"
    __pyx_v_info->shape = NULL;

#line 525 "View.MemoryView"
  }

#line 525 "View.MemoryView"
  __pyx_L6:;

  /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */

#line 527 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);

#line 527 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":528
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
 */

#line 528 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.strides;

#line 528 "View.MemoryView"
    __pyx_v_info->strides = __pyx_t_4;

    /* "View.MemoryView":527
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */

#line 527 "View.MemoryView"
    goto __pyx_L7;

#line 527 "View.MemoryView"
  }

  /* "View.MemoryView":530
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
 */

#line 530 "View.MemoryView"
  /*else*/ {

#line 530 "View.MemoryView"
    __pyx_v_info->strides = NULL;

#line 530 "View.MemoryView"
  }

#line 530 "View.MemoryView"
  __pyx_L7:;

  /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */

#line 532 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);

#line 532 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":533
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
 */

#line 533 "View.MemoryView"
    __pyx_t_4 = __pyx_v_self->view.suboffsets;

#line 533 "View.MemoryView"
    __pyx_v_info->suboffsets = __pyx_t_4;

    /* "View.MemoryView":532
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */

#line 532 "View.MemoryView"
    goto __pyx_L8;

#line 532 "View.MemoryView"
  }

  /* "View.MemoryView":535
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */

#line 535 "View.MemoryView"
  /*else*/ {

#line 535 "View.MemoryView"
    __pyx_v_info->suboffsets = NULL;

#line 535 "View.MemoryView"
  }

#line 535 "View.MemoryView"
  __pyx_L8:;

  /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */

#line 537 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);

#line 537 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":538
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */

#line 538 "View.MemoryView"
    __pyx_t_5 = __pyx_v_self->view.format;

#line 538 "View.MemoryView"
    __pyx_v_info->format = __pyx_t_5;

    /* "View.MemoryView":537
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */

#line 537 "View.MemoryView"
    goto __pyx_L9;

#line 537 "View.MemoryView"
  }

  /* "View.MemoryView":540
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
 */

#line 540 "View.MemoryView"
  /*else*/ {

#line 540 "View.MemoryView"
    __pyx_v_info->format = NULL;

#line 540 "View.MemoryView"
  }

#line 540 "View.MemoryView"
  __pyx_L9:;

  /* "View.MemoryView":542
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 */

#line 542 "View.MemoryView"
  __pyx_t_6 = __pyx_v_self->view.buf;

#line 542 "View.MemoryView"
  __pyx_v_info->buf = __pyx_t_6;

  /* "View.MemoryView":543
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 */

#line 543 "View.MemoryView"
  __pyx_t_7 = __pyx_v_self->view.ndim;

#line 543 "View.MemoryView"
  __pyx_v_info->ndim = __pyx_t_7;

  /* "View.MemoryView":544
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 */

#line 544 "View.MemoryView"
  __pyx_t_8 = __pyx_v_self->view.itemsize;

#line 544 "View.MemoryView"
  __pyx_v_info->itemsize = __pyx_t_8;

  /* "View.MemoryView":545
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = self.view.readonly
 *         info.obj = self
 */

#line 545 "View.MemoryView"
  __pyx_t_8 = __pyx_v_self->view.len;

#line 545 "View.MemoryView"
  __pyx_v_info->len = __pyx_t_8;

  /* "View.MemoryView":546
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
 */

#line 546 "View.MemoryView"
  __pyx_t_1 = __pyx_v_self->view.readonly;

#line 546 "View.MemoryView"
  __pyx_v_info->readonly = __pyx_t_1;

  /* "View.MemoryView":547
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 547 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_self));

#line 547 "View.MemoryView"
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));

#line 547 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_info->obj);

#line 547 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_info->obj);

#line 547 "View.MemoryView"
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":518
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")
 */

#line 518 "View.MemoryView"


#line 518 "View.MemoryView"
  /* function exit code */

#line 518 "View.MemoryView"
  __pyx_r = 0;

#line 518 "View.MemoryView"
  goto __pyx_L0;

#line 518 "View.MemoryView"
  __pyx_L1_error:;

#line 518 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 518 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 518 "View.MemoryView"
  __pyx_r = -1;

#line 518 "View.MemoryView"
  if (__pyx_v_info->obj != NULL) {

#line 518 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 518 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  goto __pyx_L2;

#line 518 "View.MemoryView"
  __pyx_L0:;

#line 518 "View.MemoryView"
  if (__pyx_v_info->obj == Py_None) {

#line 518 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_info->obj);

#line 518 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;

#line 518 "View.MemoryView"
  }

#line 518 "View.MemoryView"
  __pyx_L2:;

#line 518 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 518 "View.MemoryView"
  return __pyx_r;

#line 518 "View.MemoryView"
}

/* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
/* Python wrapper */

#line 553 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 553 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self) {

#line 553 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 553 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 553 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 553 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
  /* function exit code */

#line 553 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 553 "View.MemoryView"
  return __pyx_r;

#line 553 "View.MemoryView"
}

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0
#line 553 "View.MemoryView"
;

#line 553 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 553 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 553 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 553 "View.MemoryView"
  int __pyx_t_2;

#line 553 "View.MemoryView"
  int __pyx_lineno = 0;

#line 553 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 553 "View.MemoryView"
  int __pyx_clineno = 0;

#line 553 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":554
 *     @property
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&result.from_slice)
 *         return result
 */

#line 554 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 554, __pyx_L1_error)

#line 554 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 554 "View.MemoryView"
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_memoryviewslice_type))))) __PYX_ERR(3, 554, __pyx_L1_error)

#line 554 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);

#line 554 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":555
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */

#line 555 "View.MemoryView"
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(3, 555, __pyx_L1_error)

  /* "View.MemoryView":556
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 556 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 556 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 556 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 556 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":553
 * 
 *     @property
 *     def T(self):             # <<<<<<<<<<<<<<
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 */

#line 553 "View.MemoryView"


#line 553 "View.MemoryView"
  /* function exit code */

#line 553 "View.MemoryView"
  __pyx_L1_error:;

#line 553 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 553 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 553 "View.MemoryView"
  __pyx_r = NULL;

#line 553 "View.MemoryView"
  __pyx_L0:;

#line 553 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 553 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 553 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 553 "View.MemoryView"
  return __pyx_r;

#line 553 "View.MemoryView"
}

/* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
/* Python wrapper */

#line 559 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 559 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self) {

#line 559 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 559 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 559 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 559 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
  /* function exit code */

#line 559 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 559 "View.MemoryView"
  return __pyx_r;

#line 559 "View.MemoryView"
}

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 559 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 559 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 559 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":560
 *     @property
 *     def base(self):
 *         return self.obj             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 560 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 560 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->obj);

#line 560 "View.MemoryView"
  __pyx_r = __pyx_v_self->obj;

#line 560 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":559
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
 */

#line 559 "View.MemoryView"


#line 559 "View.MemoryView"
  /* function exit code */

#line 559 "View.MemoryView"
  __pyx_L0:;

#line 559 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 559 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 559 "View.MemoryView"
  return __pyx_r;

#line 559 "View.MemoryView"
}

/* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
/* Python wrapper */

#line 563 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 563 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self) {

#line 563 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 563 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 563 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 563 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
  /* function exit code */

#line 563 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 563 "View.MemoryView"
  return __pyx_r;

#line 563 "View.MemoryView"
}

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_length
#line 563 "View.MemoryView"
;

#line 563 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 563 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 563 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 563 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 563 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 563 "View.MemoryView"
  int __pyx_lineno = 0;

#line 563 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 563 "View.MemoryView"
  int __pyx_clineno = 0;

#line 563 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":564
 *     @property
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 564 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 564 "View.MemoryView"
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 564 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);

#line 564 "View.MemoryView"
  for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {

#line 564 "View.MemoryView"
    __pyx_t_2 = __pyx_t_4;

#line 564 "View.MemoryView"
    __pyx_v_length = (__pyx_t_2[0]);

#line 564 "View.MemoryView"
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_5);

#line 564 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(3, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 564 "View.MemoryView"
  }

#line 564 "View.MemoryView"
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 564, __pyx_L1_error)

#line 564 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_5);

#line 564 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 564 "View.MemoryView"
  __pyx_r = __pyx_t_5;

#line 564 "View.MemoryView"
  __pyx_t_5 = 0;

#line 564 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":563
 * 
 *     @property
 *     def shape(self):             # <<<<<<<<<<<<<<
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

#line 563 "View.MemoryView"


#line 563 "View.MemoryView"
  /* function exit code */

#line 563 "View.MemoryView"
  __pyx_L1_error:;

#line 563 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 563 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 563 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 563 "View.MemoryView"
  __pyx_r = NULL;

#line 563 "View.MemoryView"
  __pyx_L0:;

#line 563 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 563 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 563 "View.MemoryView"
  return __pyx_r;

#line 563 "View.MemoryView"
}

/* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
/* Python wrapper */

#line 567 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 567 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self) {

#line 567 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 567 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 567 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 567 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
  /* function exit code */

#line 567 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 567 "View.MemoryView"
  return __pyx_r;

#line 567 "View.MemoryView"
}

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_stride
#line 567 "View.MemoryView"
;

#line 567 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 567 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 567 "View.MemoryView"
  int __pyx_t_1;

#line 567 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 567 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 567 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 567 "View.MemoryView"
  int __pyx_lineno = 0;

#line 567 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 567 "View.MemoryView"
  int __pyx_clineno = 0;

#line 567 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */

#line 568 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.strides == NULL) != 0);

#line 568 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */

#line 570 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 570, __pyx_L1_error)

#line 570 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 570 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 570 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 570 "View.MemoryView"
    __PYX_ERR(3, 570, __pyx_L1_error)

    /* "View.MemoryView":568
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError("Buffer view does not expose strides")
 */

#line 568 "View.MemoryView"
  }

  /* "View.MemoryView":572
 *             raise ValueError("Buffer view does not expose strides")
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 572 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 572 "View.MemoryView"
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 572 "View.MemoryView"
  __pyx_t_4 = (__pyx_v_self->view.strides + __pyx_v_self->view.ndim);

#line 572 "View.MemoryView"
  for (__pyx_t_5 = __pyx_v_self->view.strides; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {

#line 572 "View.MemoryView"
    __pyx_t_3 = __pyx_t_5;

#line 572 "View.MemoryView"
    __pyx_v_stride = (__pyx_t_3[0]);

#line 572 "View.MemoryView"
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_stride); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_6);

#line 572 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) __PYX_ERR(3, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 572 "View.MemoryView"
  }

#line 572 "View.MemoryView"
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 572, __pyx_L1_error)

#line 572 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_6);

#line 572 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 572 "View.MemoryView"
  __pyx_r = __pyx_t_6;

#line 572 "View.MemoryView"
  __pyx_t_6 = 0;

#line 572 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":567
 * 
 *     @property
 *     def strides(self):             # <<<<<<<<<<<<<<
 *         if self.view.strides == NULL:
 * 
 */

#line 567 "View.MemoryView"


#line 567 "View.MemoryView"
  /* function exit code */

#line 567 "View.MemoryView"
  __pyx_L1_error:;

#line 567 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 567 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 567 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 567 "View.MemoryView"
  __pyx_r = NULL;

#line 567 "View.MemoryView"
  __pyx_L0:;

#line 567 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 567 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 567 "View.MemoryView"
  return __pyx_r;

#line 567 "View.MemoryView"
}

/* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
/* Python wrapper */

#line 575 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 575 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self) {

#line 575 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 575 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 575 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 575 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
  /* function exit code */

#line 575 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 575 "View.MemoryView"
  return __pyx_r;

#line 575 "View.MemoryView"
}

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_suboffset
#line 575 "View.MemoryView"
;

#line 575 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 575 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 575 "View.MemoryView"
  int __pyx_t_1;

#line 575 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 575 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 575 "View.MemoryView"
  Py_ssize_t *__pyx_t_6;

#line 575 "View.MemoryView"
  int __pyx_lineno = 0;

#line 575 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 575 "View.MemoryView"
  int __pyx_clineno = 0;

#line 575 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */

#line 576 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.suboffsets == NULL) != 0);

#line 576 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */

#line 577 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 577 "View.MemoryView"
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 577 "View.MemoryView"
    __pyx_t_3 = PyNumber_Multiply(__pyx_tuple__21, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 577 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 577 "View.MemoryView"
    __pyx_r = __pyx_t_3;

#line 577 "View.MemoryView"
    __pyx_t_3 = 0;

#line 577 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":576
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
 */

#line 576 "View.MemoryView"
  }

  /* "View.MemoryView":579
 *             return (-1,) * self.view.ndim
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 579 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 579 "View.MemoryView"
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 579 "View.MemoryView"
  __pyx_t_5 = (__pyx_v_self->view.suboffsets + __pyx_v_self->view.ndim);

#line 579 "View.MemoryView"
  for (__pyx_t_6 = __pyx_v_self->view.suboffsets; __pyx_t_6 < __pyx_t_5; __pyx_t_6++) {

#line 579 "View.MemoryView"
    __pyx_t_4 = __pyx_t_6;

#line 579 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_4[0]);

#line 579 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_suboffset); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 579 "View.MemoryView"
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_2))) __PYX_ERR(3, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 579 "View.MemoryView"
  }

#line 579 "View.MemoryView"
  __pyx_t_2 = PyList_AsTuple(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 579, __pyx_L1_error)

#line 579 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 579 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 579 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 579 "View.MemoryView"
  __pyx_t_2 = 0;

#line 579 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":575
 * 
 *     @property
 *     def suboffsets(self):             # <<<<<<<<<<<<<<
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim
 */

#line 575 "View.MemoryView"


#line 575 "View.MemoryView"
  /* function exit code */

#line 575 "View.MemoryView"
  __pyx_L1_error:;

#line 575 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 575 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 575 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 575 "View.MemoryView"
  __pyx_r = NULL;

#line 575 "View.MemoryView"
  __pyx_L0:;

#line 575 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 575 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 575 "View.MemoryView"
  return __pyx_r;

#line 575 "View.MemoryView"
}

/* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
/* Python wrapper */

#line 582 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 582 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self) {

#line 582 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 582 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 582 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 582 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
  /* function exit code */

#line 582 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 582 "View.MemoryView"
  return __pyx_r;

#line 582 "View.MemoryView"
}

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 582 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 582 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 582 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 582 "View.MemoryView"
  int __pyx_lineno = 0;

#line 582 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 582 "View.MemoryView"
  int __pyx_clineno = 0;

#line 582 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":583
 *     @property
 *     def ndim(self):
 *         return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 583 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 583 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 583, __pyx_L1_error)

#line 583 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 583 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 583 "View.MemoryView"
  __pyx_t_1 = 0;

#line 583 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":582
 * 
 *     @property
 *     def ndim(self):             # <<<<<<<<<<<<<<
 *         return self.view.ndim
 * 
 */

#line 582 "View.MemoryView"


#line 582 "View.MemoryView"
  /* function exit code */

#line 582 "View.MemoryView"
  __pyx_L1_error:;

#line 582 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 582 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 582 "View.MemoryView"
  __pyx_r = NULL;

#line 582 "View.MemoryView"
  __pyx_L0:;

#line 582 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 582 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 582 "View.MemoryView"
  return __pyx_r;

#line 582 "View.MemoryView"
}

/* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
/* Python wrapper */

#line 586 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 586 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self) {

#line 586 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 586 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 586 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 586 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
  /* function exit code */

#line 586 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 586 "View.MemoryView"
  return __pyx_r;

#line 586 "View.MemoryView"
}

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 586 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 586 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 586 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 586 "View.MemoryView"
  int __pyx_lineno = 0;

#line 586 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 586 "View.MemoryView"
  int __pyx_clineno = 0;

#line 586 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":587
 *     @property
 *     def itemsize(self):
 *         return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 587 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 587 "View.MemoryView"
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 587, __pyx_L1_error)

#line 587 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 587 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 587 "View.MemoryView"
  __pyx_t_1 = 0;

#line 587 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":586
 * 
 *     @property
 *     def itemsize(self):             # <<<<<<<<<<<<<<
 *         return self.view.itemsize
 * 
 */

#line 586 "View.MemoryView"


#line 586 "View.MemoryView"
  /* function exit code */

#line 586 "View.MemoryView"
  __pyx_L1_error:;

#line 586 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 586 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 586 "View.MemoryView"
  __pyx_r = NULL;

#line 586 "View.MemoryView"
  __pyx_L0:;

#line 586 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 586 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 586 "View.MemoryView"
  return __pyx_r;

#line 586 "View.MemoryView"
}

/* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
/* Python wrapper */

#line 590 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 590 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self) {

#line 590 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 590 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 590 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 590 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
  /* function exit code */

#line 590 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 590 "View.MemoryView"
  return __pyx_r;

#line 590 "View.MemoryView"
}

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 590 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 590 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 590 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 590 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 590 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 590 "View.MemoryView"
  int __pyx_lineno = 0;

#line 590 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 590 "View.MemoryView"
  int __pyx_clineno = 0;

#line 590 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":591
 *     @property
 *     def nbytes(self):
 *         return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 591 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 591 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 591 "View.MemoryView"
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 591 "View.MemoryView"
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 591, __pyx_L1_error)

#line 591 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 591 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 591 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 591 "View.MemoryView"
  __pyx_r = __pyx_t_3;

#line 591 "View.MemoryView"
  __pyx_t_3 = 0;

#line 591 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":590
 * 
 *     @property
 *     def nbytes(self):             # <<<<<<<<<<<<<<
 *         return self.size * self.view.itemsize
 * 
 */

#line 590 "View.MemoryView"


#line 590 "View.MemoryView"
  /* function exit code */

#line 590 "View.MemoryView"
  __pyx_L1_error:;

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 590 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 590 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 590 "View.MemoryView"
  __pyx_r = NULL;

#line 590 "View.MemoryView"
  __pyx_L0:;

#line 590 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 590 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 590 "View.MemoryView"
  return __pyx_r;

#line 590 "View.MemoryView"
}

/* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
/* Python wrapper */

#line 594 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 594 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self) {

#line 594 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 594 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 594 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 594 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
  /* function exit code */

#line 594 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 594 "View.MemoryView"
  return __pyx_r;

#line 594 "View.MemoryView"
}

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL
#line 594 "View.MemoryView"
;
  PyObject *__pyx_v_length = NULL
#line 594 "View.MemoryView"
;

#line 594 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 594 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 594 "View.MemoryView"
  int __pyx_t_1;

#line 594 "View.MemoryView"
  int __pyx_t_2;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

#line 594 "View.MemoryView"
  Py_ssize_t *__pyx_t_5;

#line 594 "View.MemoryView"
  PyObject *__pyx_t_6 = NULL;

#line 594 "View.MemoryView"
  int __pyx_lineno = 0;

#line 594 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 594 "View.MemoryView"
  int __pyx_clineno = 0;

#line 594 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */

#line 595 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);

#line 595 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 595 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":596
 *     def size(self):
 *         if self._size is None:
 *             result = 1             # <<<<<<<<<<<<<<
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 */

#line 596 "View.MemoryView"
    __Pyx_INCREF(__pyx_int_1);

#line 596 "View.MemoryView"
    __pyx_v_result = __pyx_int_1;

    /* "View.MemoryView":598
 *             result = 1
 * 
 *             for length in self.view.shape[:self.view.ndim]:             # <<<<<<<<<<<<<<
 *                 result *= length
 * 
 */

#line 598 "View.MemoryView"
    __pyx_t_4 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);

#line 598 "View.MemoryView"
    for (__pyx_t_5 = __pyx_v_self->view.shape; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {

#line 598 "View.MemoryView"
      __pyx_t_3 = __pyx_t_5;

#line 598 "View.MemoryView"
      __pyx_t_6 = PyInt_FromSsize_t((__pyx_t_3[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 598, __pyx_L1_error)

#line 598 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 598 "View.MemoryView"
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_6);

#line 598 "View.MemoryView"
      __pyx_t_6 = 0;

      /* "View.MemoryView":599
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 *                 result *= length             # <<<<<<<<<<<<<<
 * 
 *             self._size = result
 */

#line 599 "View.MemoryView"
      __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 599, __pyx_L1_error)

#line 599 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_6);

#line 599 "View.MemoryView"
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_6);

#line 599 "View.MemoryView"
      __pyx_t_6 = 0;

#line 599 "View.MemoryView"
    }

    /* "View.MemoryView":601
 *                 result *= length
 * 
 *             self._size = result             # <<<<<<<<<<<<<<
 * 
 *         return self._size
 */

#line 601 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_result);

#line 601 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_result);

#line 601 "View.MemoryView"
    __Pyx_GOTREF(__pyx_v_self->_size);

#line 601 "View.MemoryView"
    __Pyx_DECREF(__pyx_v_self->_size);

#line 601 "View.MemoryView"
    __pyx_v_self->_size = __pyx_v_result;

    /* "View.MemoryView":595
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
 */

#line 595 "View.MemoryView"
  }

  /* "View.MemoryView":603
 *             self._size = result
 * 
 *         return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */

#line 603 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 603 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->_size);

#line 603 "View.MemoryView"
  __pyx_r = __pyx_v_self->_size;

#line 603 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":594
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         if self._size is None:
 *             result = 1
 */

#line 594 "View.MemoryView"


#line 594 "View.MemoryView"
  /* function exit code */

#line 594 "View.MemoryView"
  __pyx_L1_error:;

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_6);

#line 594 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 594 "View.MemoryView"
  __pyx_r = NULL;

#line 594 "View.MemoryView"
  __pyx_L0:;

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 594 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_length);

#line 594 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 594 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 594 "View.MemoryView"
  return __pyx_r;

#line 594 "View.MemoryView"
}

/* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
/* Python wrapper */

#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/

#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {

#line 605 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 605 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 605 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = 
#line 605 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
  /* function exit code */

#line 605 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 605 "View.MemoryView"
  return __pyx_r;

#line 605 "View.MemoryView"
}

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 605 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 605 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 605 "View.MemoryView"
  int __pyx_t_1;

#line 605 "View.MemoryView"
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */

#line 606 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->view.ndim >= 1) != 0);

#line 606 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":607
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */

#line 607 "View.MemoryView"
    __pyx_r = (__pyx_v_self->view.shape[0]);

#line 607 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":606
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */

#line 606 "View.MemoryView"
  }

  /* "View.MemoryView":609
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */

#line 609 "View.MemoryView"
  __pyx_r = 0;

#line 609 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":605
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

#line 605 "View.MemoryView"


#line 605 "View.MemoryView"
  /* function exit code */

#line 605 "View.MemoryView"
  __pyx_L0:;

#line 605 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 605 "View.MemoryView"
  return __pyx_r;

#line 605 "View.MemoryView"
}

/* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
/* Python wrapper */

#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/

#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {

#line 611 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 611 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 611 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = 
#line 611 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
  /* function exit code */

#line 611 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 611 "View.MemoryView"
  return __pyx_r;

#line 611 "View.MemoryView"
}

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 611 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 611 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 611 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 611 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 611 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 611 "View.MemoryView"
  int __pyx_lineno = 0;

#line 611 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 611 "View.MemoryView"
  int __pyx_clineno = 0;

#line 611 "View.MemoryView"
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */

#line 612 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 612 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 612 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 612 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":613
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */

#line 613 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 613, __pyx_L1_error)

#line 613 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

  /* "View.MemoryView":612
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */

#line 612 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 612 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 612 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);

#line 612 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 612 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);

#line 612 "View.MemoryView"
  __pyx_t_1 = 0;

#line 612 "View.MemoryView"
  __pyx_t_2 = 0;

#line 612 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 612, __pyx_L1_error)

#line 612 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 612 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 612 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 612 "View.MemoryView"
  __pyx_t_2 = 0;

#line 612 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":611
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

#line 611 "View.MemoryView"


#line 611 "View.MemoryView"
  /* function exit code */

#line 611 "View.MemoryView"
  __pyx_L1_error:;

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 611 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 611 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 611 "View.MemoryView"
  __pyx_r = NULL;

#line 611 "View.MemoryView"
  __pyx_L0:;

#line 611 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 611 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 611 "View.MemoryView"
  return __pyx_r;

#line 611 "View.MemoryView"
}

/* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
/* Python wrapper */

#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/

#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {

#line 615 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 615 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 615 "View.MemoryView"
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = 
#line 615 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
  /* function exit code */

#line 615 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 615 "View.MemoryView"
  return __pyx_r;

#line 615 "View.MemoryView"
}

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {

#line 615 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 615 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 615 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 615 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 615 "View.MemoryView"
  int __pyx_lineno = 0;

#line 615 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 615 "View.MemoryView"
  int __pyx_clineno = 0;

#line 615 "View.MemoryView"
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":616
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 616 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 616 "View.MemoryView"
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 616 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 616 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);

#line 616 "View.MemoryView"
  __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 616, __pyx_L1_error)

#line 616 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 616 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 616 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 616 "View.MemoryView"
  __pyx_t_1 = 0;

#line 616 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":615
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

#line 615 "View.MemoryView"


#line 615 "View.MemoryView"
  /* function exit code */

#line 615 "View.MemoryView"
  __pyx_L1_error:;

#line 615 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 615 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 615 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 615 "View.MemoryView"
  __pyx_r = NULL;

#line 615 "View.MemoryView"
  __pyx_L0:;

#line 615 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 615 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 615 "View.MemoryView"
  return __pyx_r;

#line 615 "View.MemoryView"
}

/* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
/* Python wrapper */

#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 619 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 619 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 619 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  __pyx_r = 
#line 619 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
  /* function exit code */

#line 619 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 619 "View.MemoryView"
  return __pyx_r;

#line 619 "View.MemoryView"
}

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice
#line 619 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 619 "View.MemoryView"
;

#line 619 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 619 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 619 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 619 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 619 "View.MemoryView"
  int __pyx_lineno = 0;

#line 619 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 619 "View.MemoryView"
  int __pyx_clineno = 0;

#line 619 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":622
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 */

#line 622 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 622, __pyx_L1_error)

#line 622 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":623
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
 */

#line 623 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 623 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 623, __pyx_L1_error)

#line 623 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 623 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 623 "View.MemoryView"
  __pyx_t_2 = 0;

#line 623 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":619
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 619 "View.MemoryView"


#line 619 "View.MemoryView"
  /* function exit code */

#line 619 "View.MemoryView"
  __pyx_L1_error:;

#line 619 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 619 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 619 "View.MemoryView"
  __pyx_r = NULL;

#line 619 "View.MemoryView"
  __pyx_L0:;

#line 619 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 619 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 619 "View.MemoryView"
  return __pyx_r;

#line 619 "View.MemoryView"
}

/* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
/* Python wrapper */

#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 625 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 625 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 625 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  __pyx_r = 
#line 625 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
  /* function exit code */

#line 625 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 625 "View.MemoryView"
  return __pyx_r;

#line 625 "View.MemoryView"
}

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice
#line 625 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 625 "View.MemoryView"
;

#line 625 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 625 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 625 "View.MemoryView"
  __Pyx_memviewslice *__pyx_t_1;

#line 625 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 625 "View.MemoryView"
  int __pyx_lineno = 0;

#line 625 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 625 "View.MemoryView"
  int __pyx_clineno = 0;

#line 625 "View.MemoryView"
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":628
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 */

#line 628 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((__Pyx_memviewslice *)NULL))) __PYX_ERR(3, 628, __pyx_L1_error)

#line 628 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":629
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
 */

#line 629 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 629 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 629, __pyx_L1_error)

#line 629 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 629 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 629 "View.MemoryView"
  __pyx_t_2 = 0;

#line 629 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":625
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

#line 625 "View.MemoryView"


#line 625 "View.MemoryView"
  /* function exit code */

#line 625 "View.MemoryView"
  __pyx_L1_error:;

#line 625 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 625 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 625 "View.MemoryView"
  __pyx_r = NULL;

#line 625 "View.MemoryView"
  __pyx_L0:;

#line 625 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 625 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 625 "View.MemoryView"
  return __pyx_r;

#line 625 "View.MemoryView"
}

/* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
/* Python wrapper */

#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 631 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 631 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 631 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = 
#line 631 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
  /* function exit code */

#line 631 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 631 "View.MemoryView"
  return __pyx_r;

#line 631 "View.MemoryView"
}

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice
#line 631 "View.MemoryView"
;
  int __pyx_v_flags
#line 631 "View.MemoryView"
;

#line 631 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 631 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 631 "View.MemoryView"
  __Pyx_memviewslice __pyx_t_1;

#line 631 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 631 "View.MemoryView"
  int __pyx_lineno = 0;

#line 631 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 631 "View.MemoryView"
  int __pyx_clineno = 0;

#line 631 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":633
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
 */

#line 633 "View.MemoryView"
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":635
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
 */

#line 635 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":636
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
 */

#line 636 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), ((char *)"c"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 636, __pyx_L1_error)

#line 636 "View.MemoryView"
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":641
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
 */

#line 641 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 641 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 641, __pyx_L1_error)

#line 641 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 641 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 641 "View.MemoryView"
  __pyx_t_2 = 0;

#line 641 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":631
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

#line 631 "View.MemoryView"


#line 631 "View.MemoryView"
  /* function exit code */

#line 631 "View.MemoryView"
  __pyx_L1_error:;

#line 631 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 631 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 631 "View.MemoryView"
  __pyx_r = NULL;

#line 631 "View.MemoryView"
  __pyx_L0:;

#line 631 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 631 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 631 "View.MemoryView"
  return __pyx_r;

#line 631 "View.MemoryView"
}

/* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
/* Python wrapper */

#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 643 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 643 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 643 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  __pyx_r = 
#line 643 "View.MemoryView"
__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
  /* function exit code */

#line 643 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 643 "View.MemoryView"
  return __pyx_r;

#line 643 "View.MemoryView"
}

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src
#line 643 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_dst
#line 643 "View.MemoryView"
;
  int __pyx_v_flags
#line 643 "View.MemoryView"
;

#line 643 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 643 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 643 "View.MemoryView"
  __Pyx_memviewslice __pyx_t_1;

#line 643 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 643 "View.MemoryView"
  int __pyx_lineno = 0;

#line 643 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 643 "View.MemoryView"
  int __pyx_clineno = 0;

#line 643 "View.MemoryView"
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":645
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
 */

#line 645 "View.MemoryView"
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":647
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
 */

#line 647 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":648
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
 */

#line 648 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), ((char *)"fortran"), __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 648, __pyx_L1_error)

#line 648 "View.MemoryView"
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":653
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 653 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 653 "View.MemoryView"
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 653, __pyx_L1_error)

#line 653 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 653 "View.MemoryView"
  __pyx_r = __pyx_t_2;

#line 653 "View.MemoryView"
  __pyx_t_2 = 0;

#line 653 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":643
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

#line 643 "View.MemoryView"


#line 643 "View.MemoryView"
  /* function exit code */

#line 643 "View.MemoryView"
  __pyx_L1_error:;

#line 643 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 643 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 643 "View.MemoryView"
  __pyx_r = NULL;

#line 643 "View.MemoryView"
  __pyx_L0:;

#line 643 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 643 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 643 "View.MemoryView"
  return __pyx_r;

#line 643 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_memoryview___reduce_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_memoryview_2__setstate_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

#line 657 "View.MemoryView"


#line 657 "View.MemoryView"
static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0
#line 657 "View.MemoryView"
;

#line 657 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 657 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 657 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 657 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 657 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 657 "View.MemoryView"
  int __pyx_lineno = 0;

#line 657 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 657 "View.MemoryView"
  int __pyx_clineno = 0;

#line 657 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":658
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
 */

#line 658 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 658 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 658 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_o);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_v_o);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);

#line 658 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 658 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_1 = 0;

#line 658 "View.MemoryView"
  __pyx_t_2 = 0;

#line 658 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 658, __pyx_L1_error)

#line 658 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 658 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 658 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_2);

#line 658 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":659
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
 */

#line 659 "View.MemoryView"
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":660
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
 */

#line 660 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 660 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 660 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 660 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":657
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

#line 657 "View.MemoryView"


#line 657 "View.MemoryView"
  /* function exit code */

#line 657 "View.MemoryView"
  __pyx_L1_error:;

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 657 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 657 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 657 "View.MemoryView"
  __pyx_r = 0;

#line 657 "View.MemoryView"
  __pyx_L0:;

#line 657 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 657 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 657 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 657 "View.MemoryView"
  return __pyx_r;

#line 657 "View.MemoryView"
}

/* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

#line 663 "View.MemoryView"


#line 663 "View.MemoryView"
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {

#line 663 "View.MemoryView"
  int __pyx_r;

#line 663 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 663 "View.MemoryView"
  int __pyx_t_1;

#line 663 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_check", 0);

  /* "View.MemoryView":664
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
 */

#line 664 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_memoryview_type); 

#line 664 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 664 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":663
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

#line 663 "View.MemoryView"


#line 663 "View.MemoryView"
  /* function exit code */

#line 663 "View.MemoryView"
  __pyx_L0:;

#line 663 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 663 "View.MemoryView"
  return __pyx_r;

#line 663 "View.MemoryView"
}

/* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

#line 666 "View.MemoryView"


#line 666 "View.MemoryView"
static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  PyObject *__pyx_v_tup = NULL
#line 666 "View.MemoryView"
;
  PyObject *__pyx_v_result = NULL
#line 666 "View.MemoryView"
;
  int __pyx_v_have_slices
#line 666 "View.MemoryView"
;
  int __pyx_v_seen_ellipsis
#line 666 "View.MemoryView"
;
  CYTHON_UNUSED PyObject *__pyx_v_idx = NULL
#line 666 "View.MemoryView"
;
  PyObject *__pyx_v_item = NULL
#line 666 "View.MemoryView"
;
  Py_ssize_t __pyx_v_nslices
#line 666 "View.MemoryView"
;

#line 666 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 666 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 666 "View.MemoryView"
  int __pyx_t_1;

#line 666 "View.MemoryView"
  int __pyx_t_2;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 666 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 666 "View.MemoryView"
  PyObject *(*__pyx_t_6)(PyObject *);

#line 666 "View.MemoryView"
  PyObject *__pyx_t_7 = NULL;

#line 666 "View.MemoryView"
  Py_ssize_t __pyx_t_8;

#line 666 "View.MemoryView"
  int __pyx_t_9;

#line 666 "View.MemoryView"
  int __pyx_t_10;

#line 666 "View.MemoryView"
  PyObject *__pyx_t_11 = NULL;

#line 666 "View.MemoryView"
  int __pyx_lineno = 0;

#line 666 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 666 "View.MemoryView"
  int __pyx_clineno = 0;

#line 666 "View.MemoryView"
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */

#line 671 "View.MemoryView"
  __pyx_t_1 = PyTuple_Check(__pyx_v_index); 

#line 671 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 671 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":672
 *     """
 *     if not isinstance(index, tuple):
 *         tup = (index,)             # <<<<<<<<<<<<<<
 *     else:
 *         tup = index
 */

#line 672 "View.MemoryView"
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 672, __pyx_L1_error)

#line 672 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 672 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_index);

#line 672 "View.MemoryView"
    __Pyx_GIVEREF(__pyx_v_index);

#line 672 "View.MemoryView"
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index);

#line 672 "View.MemoryView"
    __pyx_v_tup = __pyx_t_3;

#line 672 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":671
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */

#line 671 "View.MemoryView"
    goto __pyx_L3;

#line 671 "View.MemoryView"
  }

  /* "View.MemoryView":674
 *         tup = (index,)
 *     else:
 *         tup = index             # <<<<<<<<<<<<<<
 * 
 *     result = []
 */

#line 674 "View.MemoryView"
  /*else*/ {

#line 674 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_index);

#line 674 "View.MemoryView"
    __pyx_v_tup = __pyx_v_index;

#line 674 "View.MemoryView"
  }

#line 674 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":676
 *         tup = index
 * 
 *     result = []             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
 */

#line 676 "View.MemoryView"
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 676, __pyx_L1_error)

#line 676 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 676 "View.MemoryView"
  __pyx_v_result = ((PyObject*)__pyx_t_3);

#line 676 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":677
 * 
 *     result = []
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 */

#line 677 "View.MemoryView"
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":678
 *     result = []
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 */

#line 678 "View.MemoryView"
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */

#line 679 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_0);

#line 679 "View.MemoryView"
  __pyx_t_3 = __pyx_int_0;

#line 679 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_tup)) || PyTuple_CheckExact(__pyx_v_tup)) {

#line 679 "View.MemoryView"
    __pyx_t_4 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;

#line 679 "View.MemoryView"
    __pyx_t_6 = NULL;

#line 679 "View.MemoryView"
  } else {
    __pyx_t_5 = -1; 
#line 679 "View.MemoryView"
__pyx_t_4 = PyObject_GetIter(__pyx_v_tup); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 679 "View.MemoryView"
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
  }

#line 679 "View.MemoryView"
  for (;;) {

#line 679 "View.MemoryView"
    if (likely(!__pyx_t_6)) {

#line 679 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_4))) {

#line 679 "View.MemoryView"
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;

#line 679 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 679 "View.MemoryView"
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        #else

#line 679 "View.MemoryView"
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
        #endif

#line 679 "View.MemoryView"
      } else {

#line 679 "View.MemoryView"
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;

#line 679 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 679 "View.MemoryView"
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        #else

#line 679 "View.MemoryView"
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
        #endif

#line 679 "View.MemoryView"
      }
    } else 
#line 679 "View.MemoryView"
{

#line 679 "View.MemoryView"
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);

#line 679 "View.MemoryView"
      if (unlikely(!__pyx_t_7)) {

#line 679 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 679 "View.MemoryView"
        if (exc_type) {

#line 679 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 679 "View.MemoryView"
          else __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
        }

#line 679 "View.MemoryView"
        break;

#line 679 "View.MemoryView"
      }

#line 679 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
    }

#line 679 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_7);

#line 679 "View.MemoryView"
    __pyx_t_7 = 0;

#line 679 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 679 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_3);

#line 679 "View.MemoryView"
    __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 679, __pyx_L1_error)

#line 679 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_7);

#line 679 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3);

#line 679 "View.MemoryView"
    __pyx_t_3 = __pyx_t_7;

#line 679 "View.MemoryView"
    __pyx_t_7 = 0;

    /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */

#line 680 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);

#line 680 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_2 != 0);

#line 680 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */

#line 681 "View.MemoryView"
      __pyx_t_1 = ((!(__pyx_v_seen_ellipsis != 0)) != 0);

#line 681 "View.MemoryView"
      if (__pyx_t_1) {

        /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */

#line 682 "View.MemoryView"
        __pyx_t_8 = PyObject_Length(__pyx_v_tup); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(3, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __pyx_t_7 = PyList_New(1 * ((((__pyx_v_ndim - __pyx_t_8) + 1)<0) ? 0:((__pyx_v_ndim - __pyx_t_8) + 1))); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 682 "View.MemoryView"
        { Py_ssize_t __pyx_temp;

#line 682 "View.MemoryView"
          for (__pyx_temp=0; __pyx_temp < ((__pyx_v_ndim - __pyx_t_8) + 1); __pyx_temp++) {

#line 682 "View.MemoryView"
            __Pyx_INCREF(__pyx_slice__24);

#line 682 "View.MemoryView"
            __Pyx_GIVEREF(__pyx_slice__24);

#line 682 "View.MemoryView"
            PyList_SET_ITEM(__pyx_t_7, __pyx_temp, __pyx_slice__24);

#line 682 "View.MemoryView"
          }

#line 682 "View.MemoryView"
        }

#line 682 "View.MemoryView"
        __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "View.MemoryView":683
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(slice(None))
 */

#line 683 "View.MemoryView"
        __pyx_v_seen_ellipsis = 1;

        /* "View.MemoryView":681
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */

#line 681 "View.MemoryView"
        goto __pyx_L7;

#line 681 "View.MemoryView"
      }

      /* "View.MemoryView":685
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */

#line 685 "View.MemoryView"
      /*else*/ {

#line 685 "View.MemoryView"
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_slice__24); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 685, __pyx_L1_error)

#line 685 "View.MemoryView"
      }

#line 685 "View.MemoryView"
      __pyx_L7:;

      /* "View.MemoryView":686
 *             else:
 *                 result.append(slice(None))
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 */

#line 686 "View.MemoryView"
      __pyx_v_have_slices = 1;

      /* "View.MemoryView":680
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */

#line 680 "View.MemoryView"
      goto __pyx_L6;

#line 680 "View.MemoryView"
    }

    /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */

#line 688 "View.MemoryView"
    /*else*/ {

#line 688 "View.MemoryView"
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 

#line 688 "View.MemoryView"
      __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);

#line 688 "View.MemoryView"
      if (__pyx_t_10) {

#line 688 "View.MemoryView"
      } else {

#line 688 "View.MemoryView"
        __pyx_t_1 = __pyx_t_10;

#line 688 "View.MemoryView"
        goto __pyx_L9_bool_binop_done;

#line 688 "View.MemoryView"
      }

#line 688 "View.MemoryView"
      __pyx_t_10 = ((!(PyIndex_Check(__pyx_v_item) != 0)) != 0);

#line 688 "View.MemoryView"
      __pyx_t_1 = __pyx_t_10;

#line 688 "View.MemoryView"
      __pyx_L9_bool_binop_done:;

#line 688 "View.MemoryView"
      if (unlikely(__pyx_t_1)) {

        /* "View.MemoryView":689
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 *                 raise TypeError("Cannot index with type '%s'" % type(item))             # <<<<<<<<<<<<<<
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 */

#line 689 "View.MemoryView"
        __pyx_t_7 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Cannot_index_with_type_s, ((PyObject *)Py_TYPE(__pyx_v_item))); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 689, __pyx_L1_error)

#line 689 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_7);

#line 689 "View.MemoryView"
        __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 689, __pyx_L1_error)

#line 689 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_11);

#line 689 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 689 "View.MemoryView"
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);

#line 689 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

#line 689 "View.MemoryView"
        __PYX_ERR(3, 689, __pyx_L1_error)

        /* "View.MemoryView":688
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */

#line 688 "View.MemoryView"
      }

      /* "View.MemoryView":691
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 *             have_slices = have_slices or isinstance(item, slice)             # <<<<<<<<<<<<<<
 *             result.append(item)
 * 
 */

#line 691 "View.MemoryView"
      __pyx_t_10 = (__pyx_v_have_slices != 0);

#line 691 "View.MemoryView"
      if (!__pyx_t_10) {

#line 691 "View.MemoryView"
      } else {

#line 691 "View.MemoryView"
        __pyx_t_1 = __pyx_t_10;

#line 691 "View.MemoryView"
        goto __pyx_L11_bool_binop_done;

#line 691 "View.MemoryView"
      }

#line 691 "View.MemoryView"
      __pyx_t_10 = PySlice_Check(__pyx_v_item); 

#line 691 "View.MemoryView"
      __pyx_t_2 = (__pyx_t_10 != 0);

#line 691 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 691 "View.MemoryView"
      __pyx_L11_bool_binop_done:;

#line 691 "View.MemoryView"
      __pyx_v_have_slices = __pyx_t_1;

      /* "View.MemoryView":692
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 *             result.append(item)             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - len(result)
 */

#line 692 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_item); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 692, __pyx_L1_error)

#line 692 "View.MemoryView"
    }

#line 692 "View.MemoryView"
    __pyx_L6:;

    /* "View.MemoryView":679
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */

#line 679 "View.MemoryView"
  }

#line 679 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 679 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":694
 *             result.append(item)
 * 
 *     nslices = ndim - len(result)             # <<<<<<<<<<<<<<
 *     if nslices:
 *         result.extend([slice(None)] * nslices)
 */

#line 694 "View.MemoryView"
  __pyx_t_5 = PyList_GET_SIZE(__pyx_v_result); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(3, 694, __pyx_L1_error)

#line 694 "View.MemoryView"
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_t_5);

  /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */

#line 695 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_nslices != 0);

#line 695 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":696
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */

#line 696 "View.MemoryView"
    __pyx_t_3 = PyList_New(1 * ((__pyx_v_nslices<0) ? 0:__pyx_v_nslices)); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 696, __pyx_L1_error)

#line 696 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 696 "View.MemoryView"
    { Py_ssize_t __pyx_temp;

#line 696 "View.MemoryView"
      for (__pyx_temp=0; __pyx_temp < __pyx_v_nslices; __pyx_temp++) {

#line 696 "View.MemoryView"
        __Pyx_INCREF(__pyx_slice__24);

#line 696 "View.MemoryView"
        __Pyx_GIVEREF(__pyx_slice__24);

#line 696 "View.MemoryView"
        PyList_SET_ITEM(__pyx_t_3, __pyx_temp, __pyx_slice__24);

#line 696 "View.MemoryView"
      }

#line 696 "View.MemoryView"
    }

#line 696 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 696, __pyx_L1_error)

#line 696 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "View.MemoryView":695
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */

#line 695 "View.MemoryView"
  }

  /* "View.MemoryView":698
 *         result.extend([slice(None)] * nslices)
 * 
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 */

#line 698 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 698 "View.MemoryView"
  if (!__pyx_v_have_slices) {

#line 698 "View.MemoryView"
  } else {

#line 698 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
    __pyx_t_3 = __pyx_t_4;

#line 698 "View.MemoryView"
    __pyx_t_4 = 0;

#line 698 "View.MemoryView"
    goto __pyx_L14_bool_binop_done;

#line 698 "View.MemoryView"
  }

#line 698 "View.MemoryView"
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_3 = __pyx_t_4;

#line 698 "View.MemoryView"
  __pyx_t_4 = 0;

#line 698 "View.MemoryView"
  __pyx_L14_bool_binop_done:;

#line 698 "View.MemoryView"
  __pyx_t_4 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_11 = PyTuple_New(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(3, 698, __pyx_L1_error)

#line 698 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_11);

#line 698 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 698 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_3);

#line 698 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_4);

#line 698 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_4);

#line 698 "View.MemoryView"
  __pyx_t_3 = 0;

#line 698 "View.MemoryView"
  __pyx_t_4 = 0;

#line 698 "View.MemoryView"
  __pyx_r = ((PyObject*)__pyx_t_11);

#line 698 "View.MemoryView"
  __pyx_t_11 = 0;

#line 698 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":666
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

#line 666 "View.MemoryView"


#line 666 "View.MemoryView"
  /* function exit code */

#line 666 "View.MemoryView"
  __pyx_L1_error:;

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_7);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_11);

#line 666 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 666 "View.MemoryView"
  __pyx_r = 0;

#line 666 "View.MemoryView"
  __pyx_L0:;

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_tup);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_result);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_idx);

#line 666 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_item);

#line 666 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 666 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 666 "View.MemoryView"
  return __pyx_r;

#line 666 "View.MemoryView"
}

/* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 700 "View.MemoryView"


#line 700 "View.MemoryView"
static PyObject *assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_suboffset
#line 700 "View.MemoryView"
;

#line 700 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 700 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_1;

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 700 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 700 "View.MemoryView"
  int __pyx_t_4;

#line 700 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 700 "View.MemoryView"
  int __pyx_lineno = 0;

#line 700 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 700 "View.MemoryView"
  int __pyx_clineno = 0;

#line 700 "View.MemoryView"
  __Pyx_RefNannySetupContext("assert_direct_dimensions", 0);

  /* "View.MemoryView":701
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")
 */

#line 701 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_suboffsets + __pyx_v_ndim);

#line 701 "View.MemoryView"
  for (__pyx_t_3 = __pyx_v_suboffsets; __pyx_t_3 < __pyx_t_2; __pyx_t_3++) {

#line 701 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 701 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_1[0]);

    /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */

#line 702 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_suboffset >= 0) != 0);

#line 702 "View.MemoryView"
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 703 "View.MemoryView"
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 703, __pyx_L1_error)

#line 703 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_5);

#line 703 "View.MemoryView"
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);

#line 703 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 703 "View.MemoryView"
      __PYX_ERR(3, 703, __pyx_L1_error)

      /* "View.MemoryView":702
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */

#line 702 "View.MemoryView"
    }

#line 702 "View.MemoryView"
  }

  /* "View.MemoryView":700
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 700 "View.MemoryView"


#line 700 "View.MemoryView"
  /* function exit code */

#line 700 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 700 "View.MemoryView"
  goto __pyx_L0;

#line 700 "View.MemoryView"
  __pyx_L1_error:;

#line 700 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 700 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 700 "View.MemoryView"
  __pyx_r = 0;

#line 700 "View.MemoryView"
  __pyx_L0:;

#line 700 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 700 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 700 "View.MemoryView"
  return __pyx_r;

#line 700 "View.MemoryView"
}

/* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

#line 710 "View.MemoryView"


#line 710 "View.MemoryView"
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim
#line 710 "View.MemoryView"
;
  int __pyx_v_suboffset_dim
#line 710 "View.MemoryView"
;
  int __pyx_v_dim
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_src
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_dst
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_p_src
#line 710 "View.MemoryView"
;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0
#line 710 "View.MemoryView"
;
  __Pyx_memviewslice *__pyx_v_p_dst
#line 710 "View.MemoryView"
;
  int *__pyx_v_p_suboffset_dim
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_start
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stop
#line 710 "View.MemoryView"
;
  Py_ssize_t __pyx_v_step
#line 710 "View.MemoryView"
;
  int __pyx_v_have_start
#line 710 "View.MemoryView"
;
  int __pyx_v_have_stop
#line 710 "View.MemoryView"
;
  int __pyx_v_have_step
#line 710 "View.MemoryView"
;
  PyObject *__pyx_v_index = NULL
#line 710 "View.MemoryView"
;

#line 710 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_r = NULL;

#line 710 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 710 "View.MemoryView"
  int __pyx_t_1;

#line 710 "View.MemoryView"
  int __pyx_t_2;

#line 710 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 710 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_t_4;

#line 710 "View.MemoryView"
  char *__pyx_t_5;

#line 710 "View.MemoryView"
  int __pyx_t_6;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_7;

#line 710 "View.MemoryView"
  PyObject *(*__pyx_t_8)(PyObject *);

#line 710 "View.MemoryView"
  PyObject *__pyx_t_9 = NULL;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_10;

#line 710 "View.MemoryView"
  int __pyx_t_11;

#line 710 "View.MemoryView"
  Py_ssize_t __pyx_t_12;

#line 710 "View.MemoryView"
  int __pyx_lineno = 0;

#line 710 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 710 "View.MemoryView"
  int __pyx_clineno = 0;

#line 710 "View.MemoryView"
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":711
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
 */

#line 711 "View.MemoryView"
  __pyx_v_new_ndim = 0;

#line 711 "View.MemoryView"
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":718
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
 */

#line 718 "View.MemoryView"
  (void)(memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst))));

  /* "View.MemoryView":722
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */

#line 722 "View.MemoryView"
  #ifndef CYTHON_WITHOUT_ASSERTIONS

#line 722 "View.MemoryView"
  if (unlikely(!Py_OptimizeFlag)) {

#line 722 "View.MemoryView"
    if (unlikely(!((__pyx_v_memview->view.ndim > 0) != 0))) {

#line 722 "View.MemoryView"
      PyErr_SetNone(PyExc_AssertionError);

#line 722 "View.MemoryView"
      __PYX_ERR(3, 722, __pyx_L1_error)

#line 722 "View.MemoryView"
    }

#line 722 "View.MemoryView"
  }

#line 722 "View.MemoryView"
  #endif

  /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */

#line 724 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 724 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 724 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":725
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 */

#line 725 "View.MemoryView"
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(3, 725, __pyx_L1_error)

#line 725 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);

#line 725 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 725 "View.MemoryView"
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);

#line 725 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":726
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
 */

#line 726 "View.MemoryView"
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);

    /* "View.MemoryView":724
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */

#line 724 "View.MemoryView"
    goto __pyx_L3;

#line 724 "View.MemoryView"
  }

  /* "View.MemoryView":728
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
 */

#line 728 "View.MemoryView"
  /*else*/ {

#line 728 "View.MemoryView"
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":729
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 729 "View.MemoryView"
    __pyx_v_p_src = (&__pyx_v_src);

#line 729 "View.MemoryView"
  }

#line 729 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":735
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
 */

#line 735 "View.MemoryView"
  __pyx_t_4 = __pyx_v_p_src->memview;

#line 735 "View.MemoryView"
  __pyx_v_dst.memview = __pyx_t_4;

  /* "View.MemoryView":736
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 736 "View.MemoryView"
  __pyx_t_5 = __pyx_v_p_src->data;

#line 736 "View.MemoryView"
  __pyx_v_dst.data = __pyx_t_5;

  /* "View.MemoryView":741
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step
 */

#line 741 "View.MemoryView"
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":742
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step
 *     cdef bint have_start, have_stop, have_step
 */

#line 742 "View.MemoryView"
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */

#line 746 "View.MemoryView"
  __pyx_t_6 = 0;

#line 746 "View.MemoryView"
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {

#line 746 "View.MemoryView"
    __pyx_t_3 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;

#line 746 "View.MemoryView"
    __pyx_t_8 = NULL;

#line 746 "View.MemoryView"
  } else {
    __pyx_t_7 = -1; 
#line 746 "View.MemoryView"
__pyx_t_3 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 746 "View.MemoryView"
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
  }

#line 746 "View.MemoryView"
  for (;;) {

#line 746 "View.MemoryView"
    if (likely(!__pyx_t_8)) {

#line 746 "View.MemoryView"
      if (likely(PyList_CheckExact(__pyx_t_3))) {

#line 746 "View.MemoryView"
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;

#line 746 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 746 "View.MemoryView"
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        #else

#line 746 "View.MemoryView"
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
        #endif

#line 746 "View.MemoryView"
      } else {

#line 746 "View.MemoryView"
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;

#line 746 "View.MemoryView"
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS

#line 746 "View.MemoryView"
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        #else

#line 746 "View.MemoryView"
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
        #endif

#line 746 "View.MemoryView"
      }
    } else 
#line 746 "View.MemoryView"
{

#line 746 "View.MemoryView"
      __pyx_t_9 = __pyx_t_8(__pyx_t_3);

#line 746 "View.MemoryView"
      if (unlikely(!__pyx_t_9)) {

#line 746 "View.MemoryView"
        PyObject* exc_type = PyErr_Occurred();

#line 746 "View.MemoryView"
        if (exc_type) {

#line 746 "View.MemoryView"
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();

#line 746 "View.MemoryView"
          else __PYX_ERR(3, 746, __pyx_L1_error)

#line 746 "View.MemoryView"
        }

#line 746 "View.MemoryView"
        break;

#line 746 "View.MemoryView"
      }

#line 746 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 746 "View.MemoryView"
    }

#line 746 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_9);

#line 746 "View.MemoryView"
    __pyx_t_9 = 0;

#line 746 "View.MemoryView"
    __pyx_v_dim = __pyx_t_6;

#line 746 "View.MemoryView"
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */

#line 747 "View.MemoryView"
    __pyx_t_2 = (PyIndex_Check(__pyx_v_index) != 0);

#line 747 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":751
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 *                 index, 0, 0, # start, stop, step             # <<<<<<<<<<<<<<
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 */

#line 751 "View.MemoryView"
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 751, __pyx_L1_error)

      /* "View.MemoryView":748
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */

#line 748 "View.MemoryView"
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_t_10, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(3, 748, __pyx_L1_error)

      /* "View.MemoryView":747
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */

#line 747 "View.MemoryView"
      goto __pyx_L6;

#line 747 "View.MemoryView"
    }

    /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */

#line 754 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_index == Py_None);

#line 754 "View.MemoryView"
    __pyx_t_1 = (__pyx_t_2 != 0);

#line 754 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":755
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 */

#line 755 "View.MemoryView"
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":756
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
 */

#line 756 "View.MemoryView"
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":757
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
 */

#line 757 "View.MemoryView"
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1L;

      /* "View.MemoryView":758
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
 */

#line 758 "View.MemoryView"
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

      /* "View.MemoryView":754
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */

#line 754 "View.MemoryView"
      goto __pyx_L6;

#line 754 "View.MemoryView"
    }

    /* "View.MemoryView":760
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
 */

#line 760 "View.MemoryView"
    /*else*/ {

#line 760 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 760 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
      if (!__pyx_t_1) {

#line 760 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 760 "View.MemoryView"
      } else {

#line 760 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 760, __pyx_L1_error)

#line 760 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 760 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 760 "View.MemoryView"
        goto __pyx_L7_bool_binop_done;

#line 760 "View.MemoryView"
      }

#line 760 "View.MemoryView"
      __pyx_t_10 = 0;

#line 760 "View.MemoryView"
      __pyx_L7_bool_binop_done:;

#line 760 "View.MemoryView"
      __pyx_v_start = __pyx_t_10;

      /* "View.MemoryView":761
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
 */

#line 761 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 761 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
      if (!__pyx_t_1) {

#line 761 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 761 "View.MemoryView"
      } else {

#line 761 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 761, __pyx_L1_error)

#line 761 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 761 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 761 "View.MemoryView"
        goto __pyx_L9_bool_binop_done;

#line 761 "View.MemoryView"
      }

#line 761 "View.MemoryView"
      __pyx_t_10 = 0;

#line 761 "View.MemoryView"
      __pyx_L9_bool_binop_done:;

#line 761 "View.MemoryView"
      __pyx_v_stop = __pyx_t_10;

      /* "View.MemoryView":762
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
 */

#line 762 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 762 "View.MemoryView"
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(3, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
      if (!__pyx_t_1) {

#line 762 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 762 "View.MemoryView"
      } else {

#line 762 "View.MemoryView"
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 762, __pyx_L1_error)

#line 762 "View.MemoryView"
        __pyx_t_10 = __pyx_t_12;

#line 762 "View.MemoryView"
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 762 "View.MemoryView"
        goto __pyx_L11_bool_binop_done;

#line 762 "View.MemoryView"
      }

#line 762 "View.MemoryView"
      __pyx_t_10 = 0;

#line 762 "View.MemoryView"
      __pyx_L11_bool_binop_done:;

#line 762 "View.MemoryView"
      __pyx_v_step = __pyx_t_10;

      /* "View.MemoryView":764
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
 */

#line 764 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 764, __pyx_L1_error)

#line 764 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 764 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 764 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 764 "View.MemoryView"
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":765
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
 */

#line 765 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 765, __pyx_L1_error)

#line 765 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 765 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 765 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 765 "View.MemoryView"
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":766
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
 */

#line 766 "View.MemoryView"
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) __PYX_ERR(3, 766, __pyx_L1_error)

#line 766 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_9);

#line 766 "View.MemoryView"
      __pyx_t_1 = (__pyx_t_9 != Py_None);

#line 766 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

#line 766 "View.MemoryView"
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":768
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */

#line 768 "View.MemoryView"
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(3, 768, __pyx_L1_error)

      /* "View.MemoryView":774
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */

#line 774 "View.MemoryView"
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

#line 774 "View.MemoryView"
    }

#line 774 "View.MemoryView"
    __pyx_L6:;

    /* "View.MemoryView":746
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */

#line 746 "View.MemoryView"
  }

#line 746 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */

#line 776 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 776 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 776 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */

#line 777 "View.MemoryView"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":778
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
 */

#line 778 "View.MemoryView"
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(3, 778, __pyx_L1_error) }

    /* "View.MemoryView":779
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
 */

#line 779 "View.MemoryView"
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(3, 779, __pyx_L1_error) }

    /* "View.MemoryView":777
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */

#line 777 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 777, __pyx_L1_error)

#line 777 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 777 "View.MemoryView"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(3, 777, __pyx_L1_error)

#line 777 "View.MemoryView"
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);

#line 777 "View.MemoryView"
    __pyx_t_3 = 0;

#line 777 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":776
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */

#line 776 "View.MemoryView"
  }

  /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */

#line 782 "View.MemoryView"
  /*else*/ {

#line 782 "View.MemoryView"
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":783
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 783 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 782, __pyx_L1_error)

#line 783 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":782
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */

#line 782 "View.MemoryView"
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) __PYX_ERR(3, 782, __pyx_L1_error)

#line 782 "View.MemoryView"
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);

#line 782 "View.MemoryView"
    __pyx_t_3 = 0;

#line 782 "View.MemoryView"
    goto __pyx_L0;

#line 782 "View.MemoryView"
  }

  /* "View.MemoryView":710
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

#line 710 "View.MemoryView"


#line 710 "View.MemoryView"
  /* function exit code */

#line 710 "View.MemoryView"
  __pyx_L1_error:;

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_9);

#line 710 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 710 "View.MemoryView"
  __pyx_r = 0;

#line 710 "View.MemoryView"
  __pyx_L0:;

#line 710 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);

#line 710 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_index);

#line 710 "View.MemoryView"
  __Pyx_XGIVEREF((PyObject *)__pyx_r);

#line 710 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 710 "View.MemoryView"
  return __pyx_r;

#line 710 "View.MemoryView"
}

/* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

#line 807 "View.MemoryView"


#line 807 "View.MemoryView"
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *__pyx_v_dst, Py_ssize_t __pyx_v_shape, Py_ssize_t __pyx_v_stride, Py_ssize_t __pyx_v_suboffset, int __pyx_v_dim, int __pyx_v_new_ndim, int *__pyx_v_suboffset_dim, Py_ssize_t __pyx_v_start, Py_ssize_t __pyx_v_stop, Py_ssize_t __pyx_v_step, int __pyx_v_have_start, int __pyx_v_have_stop, int __pyx_v_have_step, int __pyx_v_is_slice) {
  Py_ssize_t __pyx_v_new_shape
#line 807 "View.MemoryView"
;
  int __pyx_v_negative_step
#line 807 "View.MemoryView"
;

#line 807 "View.MemoryView"
  int __pyx_r;

#line 807 "View.MemoryView"
  int __pyx_t_1;

#line 807 "View.MemoryView"
  int __pyx_t_2;

#line 807 "View.MemoryView"
  int __pyx_t_3;

#line 807 "View.MemoryView"
  int __pyx_lineno = 0;

#line 807 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 807 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */

#line 827 "View.MemoryView"
  __pyx_t_1 = ((!(__pyx_v_is_slice != 0)) != 0);

#line 827 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */

#line 829 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_start < 0) != 0);

#line 829 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":830
 * 
 *         if start < 0:
 *             start += shape             # <<<<<<<<<<<<<<
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 */

#line 830 "View.MemoryView"
      __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

      /* "View.MemoryView":829
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */

#line 829 "View.MemoryView"
    }

    /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */

#line 831 "View.MemoryView"
    __pyx_t_1 = (0 <= __pyx_v_start);

#line 831 "View.MemoryView"
    if (__pyx_t_1) {

#line 831 "View.MemoryView"
      __pyx_t_1 = (__pyx_v_start < __pyx_v_shape);

#line 831 "View.MemoryView"
    }

#line 831 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);

#line 831 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":832
 *             start += shape
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)             # <<<<<<<<<<<<<<
 *     else:
 * 
 */

#line 832 "View.MemoryView"
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"Index out of bounds (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 832, __pyx_L1_error)

      /* "View.MemoryView":831
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */

#line 831 "View.MemoryView"
    }

    /* "View.MemoryView":827
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */

#line 827 "View.MemoryView"
    goto __pyx_L3;

#line 827 "View.MemoryView"
  }

  /* "View.MemoryView":835
 *     else:
 * 
 *         negative_step = have_step != 0 and step < 0             # <<<<<<<<<<<<<<
 * 
 *         if have_step and step == 0:
 */

#line 835 "View.MemoryView"
  /*else*/ {

#line 835 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_have_step != 0) != 0);

#line 835 "View.MemoryView"
    if (__pyx_t_1) {

#line 835 "View.MemoryView"
    } else {

#line 835 "View.MemoryView"
      __pyx_t_2 = __pyx_t_1;

#line 835 "View.MemoryView"
      goto __pyx_L6_bool_binop_done;

#line 835 "View.MemoryView"
    }

#line 835 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_step < 0) != 0);

#line 835 "View.MemoryView"
    __pyx_t_2 = __pyx_t_1;

#line 835 "View.MemoryView"
    __pyx_L6_bool_binop_done:;

#line 835 "View.MemoryView"
    __pyx_v_negative_step = __pyx_t_2;

    /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */

#line 837 "View.MemoryView"
    __pyx_t_1 = (__pyx_v_have_step != 0);

#line 837 "View.MemoryView"
    if (__pyx_t_1) {

#line 837 "View.MemoryView"
    } else {

#line 837 "View.MemoryView"
      __pyx_t_2 = __pyx_t_1;

#line 837 "View.MemoryView"
      goto __pyx_L9_bool_binop_done;

#line 837 "View.MemoryView"
    }

#line 837 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_step == 0) != 0);

#line 837 "View.MemoryView"
    __pyx_t_2 = __pyx_t_1;

#line 837 "View.MemoryView"
    __pyx_L9_bool_binop_done:;

#line 837 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":838
 * 
 *         if have_step and step == 0:
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 838 "View.MemoryView"
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Step may not be zero (axis %d)"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 838, __pyx_L1_error)

      /* "View.MemoryView":837
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */

#line 837 "View.MemoryView"
    }

    /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */

#line 841 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_have_start != 0);

#line 841 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */

#line 842 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_start < 0) != 0);

#line 842 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":843
 *         if have_start:
 *             if start < 0:
 *                 start += shape             # <<<<<<<<<<<<<<
 *                 if start < 0:
 *                     start = 0
 */

#line 843 "View.MemoryView"
        __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

        /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */

#line 844 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_start < 0) != 0);

#line 844 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":845
 *                 start += shape
 *                 if start < 0:
 *                     start = 0             # <<<<<<<<<<<<<<
 *             elif start >= shape:
 *                 if negative_step:
 */

#line 845 "View.MemoryView"
          __pyx_v_start = 0;

          /* "View.MemoryView":844
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */

#line 844 "View.MemoryView"
        }

        /* "View.MemoryView":842
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */

#line 842 "View.MemoryView"
        goto __pyx_L12;

#line 842 "View.MemoryView"
      }

      /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */

#line 846 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_start >= __pyx_v_shape) != 0);

#line 846 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */

#line 847 "View.MemoryView"
        __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 847 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":848
 *             elif start >= shape:
 *                 if negative_step:
 *                     start = shape - 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = shape
 */

#line 848 "View.MemoryView"
          __pyx_v_start = (__pyx_v_shape - 1);

          /* "View.MemoryView":847
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */

#line 847 "View.MemoryView"
          goto __pyx_L14;

#line 847 "View.MemoryView"
        }

        /* "View.MemoryView":850
 *                     start = shape - 1
 *                 else:
 *                     start = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */

#line 850 "View.MemoryView"
        /*else*/ {

#line 850 "View.MemoryView"
          __pyx_v_start = __pyx_v_shape;

#line 850 "View.MemoryView"
        }

#line 850 "View.MemoryView"
        __pyx_L14:;

        /* "View.MemoryView":846
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */

#line 846 "View.MemoryView"
      }

#line 846 "View.MemoryView"
      __pyx_L12:;

      /* "View.MemoryView":841
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */

#line 841 "View.MemoryView"
      goto __pyx_L11;

#line 841 "View.MemoryView"
    }

    /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */

#line 852 "View.MemoryView"
    /*else*/ {

#line 852 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 852 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":853
 *         else:
 *             if negative_step:
 *                 start = shape - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 start = 0
 */

#line 853 "View.MemoryView"
        __pyx_v_start = (__pyx_v_shape - 1);

        /* "View.MemoryView":852
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */

#line 852 "View.MemoryView"
        goto __pyx_L15;

#line 852 "View.MemoryView"
      }

      /* "View.MemoryView":855
 *                 start = shape - 1
 *             else:
 *                 start = 0             # <<<<<<<<<<<<<<
 * 
 *         if have_stop:
 */

#line 855 "View.MemoryView"
      /*else*/ {

#line 855 "View.MemoryView"
        __pyx_v_start = 0;

#line 855 "View.MemoryView"
      }

#line 855 "View.MemoryView"
      __pyx_L15:;

#line 855 "View.MemoryView"
    }

#line 855 "View.MemoryView"
    __pyx_L11:;

    /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */

#line 857 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_have_stop != 0);

#line 857 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */

#line 858 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_stop < 0) != 0);

#line 858 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":859
 *         if have_stop:
 *             if stop < 0:
 *                 stop += shape             # <<<<<<<<<<<<<<
 *                 if stop < 0:
 *                     stop = 0
 */

#line 859 "View.MemoryView"
        __pyx_v_stop = (__pyx_v_stop + __pyx_v_shape);

        /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */

#line 860 "View.MemoryView"
        __pyx_t_2 = ((__pyx_v_stop < 0) != 0);

#line 860 "View.MemoryView"
        if (__pyx_t_2) {

          /* "View.MemoryView":861
 *                 stop += shape
 *                 if stop < 0:
 *                     stop = 0             # <<<<<<<<<<<<<<
 *             elif stop > shape:
 *                 stop = shape
 */

#line 861 "View.MemoryView"
          __pyx_v_stop = 0;

          /* "View.MemoryView":860
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */

#line 860 "View.MemoryView"
        }

        /* "View.MemoryView":858
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */

#line 858 "View.MemoryView"
        goto __pyx_L17;

#line 858 "View.MemoryView"
      }

      /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */

#line 862 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_stop > __pyx_v_shape) != 0);

#line 862 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":863
 *                     stop = 0
 *             elif stop > shape:
 *                 stop = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */

#line 863 "View.MemoryView"
        __pyx_v_stop = __pyx_v_shape;

        /* "View.MemoryView":862
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */

#line 862 "View.MemoryView"
      }

#line 862 "View.MemoryView"
      __pyx_L17:;

      /* "View.MemoryView":857
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */

#line 857 "View.MemoryView"
      goto __pyx_L16;

#line 857 "View.MemoryView"
    }

    /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */

#line 865 "View.MemoryView"
    /*else*/ {

#line 865 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_negative_step != 0);

#line 865 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":866
 *         else:
 *             if negative_step:
 *                 stop = -1             # <<<<<<<<<<<<<<
 *             else:
 *                 stop = shape
 */

#line 866 "View.MemoryView"
        __pyx_v_stop = -1L;

        /* "View.MemoryView":865
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */

#line 865 "View.MemoryView"
        goto __pyx_L19;

#line 865 "View.MemoryView"
      }

      /* "View.MemoryView":868
 *                 stop = -1
 *             else:
 *                 stop = shape             # <<<<<<<<<<<<<<
 * 
 *         if not have_step:
 */

#line 868 "View.MemoryView"
      /*else*/ {

#line 868 "View.MemoryView"
        __pyx_v_stop = __pyx_v_shape;

#line 868 "View.MemoryView"
      }

#line 868 "View.MemoryView"
      __pyx_L19:;

#line 868 "View.MemoryView"
    }

#line 868 "View.MemoryView"
    __pyx_L16:;

    /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */

#line 870 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_v_have_step != 0)) != 0);

#line 870 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":871
 * 
 *         if not have_step:
 *             step = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 871 "View.MemoryView"
      __pyx_v_step = 1;

      /* "View.MemoryView":870
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */

#line 870 "View.MemoryView"
    }

    /* "View.MemoryView":875
 * 
 *         with cython.cdivision(True):
 *             new_shape = (stop - start) // step             # <<<<<<<<<<<<<<
 * 
 *             if (stop - start) - step * new_shape:
 */

#line 875 "View.MemoryView"
    __pyx_v_new_shape = ((__pyx_v_stop - __pyx_v_start) / __pyx_v_step);

    /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */

#line 877 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_stop - __pyx_v_start) - (__pyx_v_step * __pyx_v_new_shape)) != 0);

#line 877 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":878
 * 
 *             if (stop - start) - step * new_shape:
 *                 new_shape += 1             # <<<<<<<<<<<<<<
 * 
 *         if new_shape < 0:
 */

#line 878 "View.MemoryView"
      __pyx_v_new_shape = (__pyx_v_new_shape + 1);

      /* "View.MemoryView":877
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */

#line 877 "View.MemoryView"
    }

    /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */

#line 880 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_new_shape < 0) != 0);

#line 880 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":881
 * 
 *         if new_shape < 0:
 *             new_shape = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 881 "View.MemoryView"
      __pyx_v_new_shape = 0;

      /* "View.MemoryView":880
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */

#line 880 "View.MemoryView"
    }

    /* "View.MemoryView":884
 * 
 * 
 *         dst.strides[new_ndim] = stride * step             # <<<<<<<<<<<<<<
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset
 */

#line 884 "View.MemoryView"
    (__pyx_v_dst->strides[__pyx_v_new_ndim]) = (__pyx_v_stride * __pyx_v_step);

    /* "View.MemoryView":885
 * 
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape             # <<<<<<<<<<<<<<
 *         dst.suboffsets[new_ndim] = suboffset
 * 
 */

#line 885 "View.MemoryView"
    (__pyx_v_dst->shape[__pyx_v_new_ndim]) = __pyx_v_new_shape;

    /* "View.MemoryView":886
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 886 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_v_new_ndim]) = __pyx_v_suboffset;

#line 886 "View.MemoryView"
  }

#line 886 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */

#line 889 "View.MemoryView"
  __pyx_t_2 = (((__pyx_v_suboffset_dim[0]) < 0) != 0);

#line 889 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":890
 * 
 *     if suboffset_dim[0] < 0:
 *         dst.data += start * stride             # <<<<<<<<<<<<<<
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 */

#line 890 "View.MemoryView"
    __pyx_v_dst->data = (__pyx_v_dst->data + (__pyx_v_start * __pyx_v_stride));

    /* "View.MemoryView":889
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */

#line 889 "View.MemoryView"
    goto __pyx_L23;

#line 889 "View.MemoryView"
  }

  /* "View.MemoryView":892
 *         dst.data += start * stride
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride             # <<<<<<<<<<<<<<
 * 
 *     if suboffset >= 0:
 */

#line 892 "View.MemoryView"
  /*else*/ {

#line 892 "View.MemoryView"
    __pyx_t_3 = (__pyx_v_suboffset_dim[0]);

#line 892 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_t_3]) = ((__pyx_v_dst->suboffsets[__pyx_t_3]) + (__pyx_v_start * __pyx_v_stride));

#line 892 "View.MemoryView"
  }

#line 892 "View.MemoryView"
  __pyx_L23:;

  /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */

#line 894 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);

#line 894 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */

#line 895 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_v_is_slice != 0)) != 0);

#line 895 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */

#line 896 "View.MemoryView"
      __pyx_t_2 = ((__pyx_v_new_ndim == 0) != 0);

#line 896 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":897
 *         if not is_slice:
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 */

#line 897 "View.MemoryView"
        __pyx_v_dst->data = ((((char **)__pyx_v_dst->data)[0]) + __pyx_v_suboffset);

        /* "View.MemoryView":896
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */

#line 896 "View.MemoryView"
        goto __pyx_L26;

#line 896 "View.MemoryView"
      }

      /* "View.MemoryView":899
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "             # <<<<<<<<<<<<<<
 *                                      "must be indexed and not sliced", dim)
 *         else:
 */

#line 899 "View.MemoryView"
      /*else*/ {

        /* "View.MemoryView":900
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 *                                      "must be indexed and not sliced", dim)             # <<<<<<<<<<<<<<
 *         else:
 *             suboffset_dim[0] = new_ndim
 */

#line 900 "View.MemoryView"
        __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, ((char *)"All dimensions preceding dimension %d must be indexed and not sliced"), __pyx_v_dim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 899, __pyx_L1_error)

#line 900 "View.MemoryView"
      }

#line 900 "View.MemoryView"
      __pyx_L26:;

      /* "View.MemoryView":895
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */

#line 895 "View.MemoryView"
      goto __pyx_L25;

#line 895 "View.MemoryView"
    }

    /* "View.MemoryView":902
 *                                      "must be indexed and not sliced", dim)
 *         else:
 *             suboffset_dim[0] = new_ndim             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */

#line 902 "View.MemoryView"
    /*else*/ {

#line 902 "View.MemoryView"
      (__pyx_v_suboffset_dim[0]) = __pyx_v_new_ndim;

#line 902 "View.MemoryView"
    }

#line 902 "View.MemoryView"
    __pyx_L25:;

    /* "View.MemoryView":894
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */

#line 894 "View.MemoryView"
  }

  /* "View.MemoryView":904
 *             suboffset_dim[0] = new_ndim
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 904 "View.MemoryView"
  __pyx_r = 0;

#line 904 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":807
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

#line 807 "View.MemoryView"


#line 807 "View.MemoryView"
  /* function exit code */

#line 807 "View.MemoryView"
  __pyx_L1_error:;

#line 807 "View.MemoryView"
  {

#line 807 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 807 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 807 "View.MemoryView"
    #endif

#line 807 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.slice_memviewslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 807 "View.MemoryView"
    #ifdef WITH_THREAD

#line 807 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 807 "View.MemoryView"
    #endif

#line 807 "View.MemoryView"
  }

#line 807 "View.MemoryView"
  __pyx_r = -1;

#line 807 "View.MemoryView"
  __pyx_L0:;

#line 807 "View.MemoryView"
  return __pyx_r;

#line 807 "View.MemoryView"
}

/* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

#line 910 "View.MemoryView"


#line 910 "View.MemoryView"
static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stride
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_suboffset
#line 910 "View.MemoryView"
;
  Py_ssize_t __pyx_v_itemsize
#line 910 "View.MemoryView"
;
  char *__pyx_v_resultp
#line 910 "View.MemoryView"
;

#line 910 "View.MemoryView"
  char *__pyx_r;

#line 910 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 910 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 910 "View.MemoryView"
  int __pyx_t_2;

#line 910 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 910 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 910 "View.MemoryView"
  int __pyx_lineno = 0;

#line 910 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 910 "View.MemoryView"
  int __pyx_clineno = 0;

#line 910 "View.MemoryView"
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":912
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
 */

#line 912 "View.MemoryView"
  __pyx_v_suboffset = -1L;

  /* "View.MemoryView":913
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
 */

#line 913 "View.MemoryView"
  __pyx_t_1 = __pyx_v_view->itemsize;

#line 913 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */

#line 916 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_view->ndim == 0) != 0);

#line 916 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":917
 * 
 *     if view.ndim == 0:
 *         shape = view.len / itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
 */

#line 917 "View.MemoryView"
    if (unlikely(__pyx_v_itemsize == 0)) {

#line 917 "View.MemoryView"
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");

#line 917 "View.MemoryView"
      __PYX_ERR(3, 917, __pyx_L1_error)

#line 917 "View.MemoryView"
    }

#line 917 "View.MemoryView"
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {

#line 917 "View.MemoryView"
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");

#line 917 "View.MemoryView"
      __PYX_ERR(3, 917, __pyx_L1_error)

#line 917 "View.MemoryView"
    }

#line 917 "View.MemoryView"
    __pyx_v_shape = __Pyx_div_Py_ssize_t(__pyx_v_view->len, __pyx_v_itemsize);

    /* "View.MemoryView":918
 *     if view.ndim == 0:
 *         shape = view.len / itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
 */

#line 918 "View.MemoryView"
    __pyx_v_stride = __pyx_v_itemsize;

    /* "View.MemoryView":916
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */

#line 916 "View.MemoryView"
    goto __pyx_L3;

#line 916 "View.MemoryView"
  }

  /* "View.MemoryView":920
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 */

#line 920 "View.MemoryView"
  /*else*/ {

#line 920 "View.MemoryView"
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":921
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
 */

#line 921 "View.MemoryView"
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */

#line 922 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_view->suboffsets != NULL) != 0);

#line 922 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":923
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
 */

#line 923 "View.MemoryView"
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);

      /* "View.MemoryView":922
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */

#line 922 "View.MemoryView"
    }

#line 922 "View.MemoryView"
  }

#line 922 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */

#line 925 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_index < 0) != 0);

#line 925 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":926
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 */

#line 926 "View.MemoryView"
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 927 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_index < 0) != 0);

#line 927 "View.MemoryView"
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":928
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
 */

#line 928 "View.MemoryView"
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 928 "View.MemoryView"
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_4);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 928 "View.MemoryView"
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 928, __pyx_L1_error)

#line 928 "View.MemoryView"
      __Pyx_GOTREF(__pyx_t_3);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 928 "View.MemoryView"
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 928 "View.MemoryView"
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 928 "View.MemoryView"
      __PYX_ERR(3, 928, __pyx_L1_error)

      /* "View.MemoryView":927
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 927 "View.MemoryView"
    }

    /* "View.MemoryView":925
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */

#line 925 "View.MemoryView"
  }

  /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 930 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_index >= __pyx_v_shape) != 0);

#line 930 "View.MemoryView"
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":931
 * 
 *     if index >= shape:
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
 */

#line 931 "View.MemoryView"
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 931 "View.MemoryView"
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_4);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 931 "View.MemoryView"
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 931, __pyx_L1_error)

#line 931 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 931 "View.MemoryView"
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);

#line 931 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 931 "View.MemoryView"
    __PYX_ERR(3, 931, __pyx_L1_error)

    /* "View.MemoryView":930
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */

#line 930 "View.MemoryView"
  }

  /* "View.MemoryView":933
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
 */

#line 933 "View.MemoryView"
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */

#line 934 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);

#line 934 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":935
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
 */

#line 935 "View.MemoryView"
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);

    /* "View.MemoryView":934
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */

#line 934 "View.MemoryView"
  }

  /* "View.MemoryView":937
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 937 "View.MemoryView"
  __pyx_r = __pyx_v_resultp;

#line 937 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":910
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

#line 910 "View.MemoryView"


#line 910 "View.MemoryView"
  /* function exit code */

#line 910 "View.MemoryView"
  __pyx_L1_error:;

#line 910 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 910 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 910 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 910 "View.MemoryView"
  __pyx_r = NULL;

#line 910 "View.MemoryView"
  __pyx_L0:;

#line 910 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 910 "View.MemoryView"
  return __pyx_r;

#line 910 "View.MemoryView"
}

/* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

#line 943 "View.MemoryView"


#line 943 "View.MemoryView"
static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim
#line 943 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_shape
#line 943 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_strides
#line 943 "View.MemoryView"
;
  int __pyx_v_i
#line 943 "View.MemoryView"
;
  int __pyx_v_j
#line 943 "View.MemoryView"
;

#line 943 "View.MemoryView"
  int __pyx_r;

#line 943 "View.MemoryView"
  int __pyx_t_1;

#line 943 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 943 "View.MemoryView"
  long __pyx_t_3;

#line 943 "View.MemoryView"
  long __pyx_t_4;

#line 943 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 943 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

#line 943 "View.MemoryView"
  int __pyx_t_7;

#line 943 "View.MemoryView"
  int __pyx_t_8;

#line 943 "View.MemoryView"
  int __pyx_t_9;

#line 943 "View.MemoryView"
  int __pyx_lineno = 0;

#line 943 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 943 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":944
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 */

#line 944 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;

#line 944 "View.MemoryView"
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":946
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
 */

#line 946 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memslice->shape;

#line 946 "View.MemoryView"
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":947
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 947 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memslice->strides;

#line 947 "View.MemoryView"
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":951
 * 
 *     cdef int i, j
 *     for i in range(ndim / 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 */

#line 951 "View.MemoryView"
  __pyx_t_3 = __Pyx_div_long(__pyx_v_ndim, 2);

#line 951 "View.MemoryView"
  __pyx_t_4 = __pyx_t_3;

#line 951 "View.MemoryView"
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_4; __pyx_t_1+=1) {

#line 951 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":952
 *     cdef int i, j
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
 */

#line 952 "View.MemoryView"
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":953
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 */

#line 953 "View.MemoryView"
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_j]);

#line 953 "View.MemoryView"
    __pyx_t_6 = (__pyx_v_strides[__pyx_v_i]);

#line 953 "View.MemoryView"
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_5;

#line 953 "View.MemoryView"
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_6;

    /* "View.MemoryView":954
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 */

#line 954 "View.MemoryView"
    __pyx_t_6 = (__pyx_v_shape[__pyx_v_j]);

#line 954 "View.MemoryView"
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_i]);

#line 954 "View.MemoryView"
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_6;

#line 954 "View.MemoryView"
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */

#line 956 "View.MemoryView"
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0) != 0);

#line 956 "View.MemoryView"
    if (!__pyx_t_8) {

#line 956 "View.MemoryView"
    } else {

#line 956 "View.MemoryView"
      __pyx_t_7 = __pyx_t_8;

#line 956 "View.MemoryView"
      goto __pyx_L6_bool_binop_done;

#line 956 "View.MemoryView"
    }

#line 956 "View.MemoryView"
    __pyx_t_8 = (((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0) != 0);

#line 956 "View.MemoryView"
    __pyx_t_7 = __pyx_t_8;

#line 956 "View.MemoryView"
    __pyx_L6_bool_binop_done:;

#line 956 "View.MemoryView"
    if (__pyx_t_7) {

      /* "View.MemoryView":957
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 1
 */

#line 957 "View.MemoryView"
      __pyx_t_9 = __pyx_memoryview_err(__pyx_builtin_ValueError, ((char *)"Cannot transpose memoryview with indirect dimensions")); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(3, 957, __pyx_L1_error)

      /* "View.MemoryView":956
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */

#line 956 "View.MemoryView"
    }

#line 956 "View.MemoryView"
  }

  /* "View.MemoryView":959
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 959 "View.MemoryView"
  __pyx_r = 1;

#line 959 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":943
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

#line 943 "View.MemoryView"


#line 943 "View.MemoryView"
  /* function exit code */

#line 943 "View.MemoryView"
  __pyx_L1_error:;

#line 943 "View.MemoryView"
  {

#line 943 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 943 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 943 "View.MemoryView"
    #endif

#line 943 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 943 "View.MemoryView"
    #ifdef WITH_THREAD

#line 943 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 943 "View.MemoryView"
    #endif

#line 943 "View.MemoryView"
  }

#line 943 "View.MemoryView"
  __pyx_r = 0;

#line 943 "View.MemoryView"
  __pyx_L0:;

#line 943 "View.MemoryView"
  return __pyx_r;

#line 943 "View.MemoryView"
}

/* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
/* Python wrapper */

#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/

#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {

#line 976 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 976 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);

#line 976 "View.MemoryView"
  __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
  /* function exit code */

#line 976 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 976 "View.MemoryView"
}

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 976 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 976 "View.MemoryView"
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":977
 * 
 *     def __dealloc__(self):
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */

#line 977 "View.MemoryView"
  __PYX_XDEC_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":976
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

#line 976 "View.MemoryView"


#line 976 "View.MemoryView"
  /* function exit code */

#line 976 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 976 "View.MemoryView"
}

/* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

#line 979 "View.MemoryView"


#line 979 "View.MemoryView"
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {

#line 979 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 979 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 979 "View.MemoryView"
  int __pyx_t_1;

#line 979 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 979 "View.MemoryView"
  int __pyx_lineno = 0;

#line 979 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 979 "View.MemoryView"
  int __pyx_clineno = 0;

#line 979 "View.MemoryView"
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */

#line 980 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->to_object_func != NULL) != 0);

#line 980 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":981
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
 */

#line 981 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 981 "View.MemoryView"
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 981, __pyx_L1_error)

#line 981 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 981 "View.MemoryView"
    __pyx_r = __pyx_t_2;

#line 981 "View.MemoryView"
    __pyx_t_2 = 0;

#line 981 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":980
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */

#line 980 "View.MemoryView"
  }

  /* "View.MemoryView":983
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */

#line 983 "View.MemoryView"
  /*else*/ {

#line 983 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 983 "View.MemoryView"
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 983, __pyx_L1_error)

#line 983 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 983 "View.MemoryView"
    __pyx_r = __pyx_t_2;

#line 983 "View.MemoryView"
    __pyx_t_2 = 0;

#line 983 "View.MemoryView"
    goto __pyx_L0;

#line 983 "View.MemoryView"
  }

  /* "View.MemoryView":979
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

#line 979 "View.MemoryView"


#line 979 "View.MemoryView"
  /* function exit code */

#line 979 "View.MemoryView"
  __pyx_L1_error:;

#line 979 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 979 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 979 "View.MemoryView"
  __pyx_r = 0;

#line 979 "View.MemoryView"
  __pyx_L0:;

#line 979 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 979 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 979 "View.MemoryView"
  return __pyx_r;

#line 979 "View.MemoryView"
}

/* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

#line 985 "View.MemoryView"


#line 985 "View.MemoryView"
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {

#line 985 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 985 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 985 "View.MemoryView"
  int __pyx_t_1;

#line 985 "View.MemoryView"
  int __pyx_t_2;

#line 985 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 985 "View.MemoryView"
  int __pyx_lineno = 0;

#line 985 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 985 "View.MemoryView"
  int __pyx_clineno = 0;

#line 985 "View.MemoryView"
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */

#line 986 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_self->to_dtype_func != NULL) != 0);

#line 986 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":987
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
 */

#line 987 "View.MemoryView"
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(3, 987, __pyx_L1_error)

    /* "View.MemoryView":986
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */

#line 986 "View.MemoryView"
    goto __pyx_L3;

#line 986 "View.MemoryView"
  }

  /* "View.MemoryView":989
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */

#line 989 "View.MemoryView"
  /*else*/ {

#line 989 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 989, __pyx_L1_error)

#line 989 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 989 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 989 "View.MemoryView"
  }

#line 989 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":985
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

#line 985 "View.MemoryView"


#line 985 "View.MemoryView"
  /* function exit code */

#line 985 "View.MemoryView"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 985 "View.MemoryView"
  goto __pyx_L0;

#line 985 "View.MemoryView"
  __pyx_L1_error:;

#line 985 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 985 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 985 "View.MemoryView"
  __pyx_r = 0;

#line 985 "View.MemoryView"
  __pyx_L0:;

#line 985 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 985 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 985 "View.MemoryView"
  return __pyx_r;

#line 985 "View.MemoryView"
}

/* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
/* Python wrapper */

#line 992 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self); /*proto*/

#line 992 "View.MemoryView"
static PyObject *__pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(PyObject *__pyx_v_self) {

#line 992 "View.MemoryView"
  PyObject *__pyx_r = 0;

#line 992 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 992 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = 
#line 992 "View.MemoryView"
__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
  /* function exit code */

#line 992 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 992 "View.MemoryView"
  return __pyx_r;

#line 992 "View.MemoryView"
}

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 992 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 992 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 992 "View.MemoryView"
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":993
 *     @property
 *     def base(self):
 *         return self.from_object             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */

#line 993 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 993 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_self->from_object);

#line 993 "View.MemoryView"
  __pyx_r = __pyx_v_self->from_object;

#line 993 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":992
 * 
 *     @property
 *     def base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
 */

#line 992 "View.MemoryView"


#line 992 "View.MemoryView"
  /* function exit code */

#line 992 "View.MemoryView"
  __pyx_L0:;

#line 992 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 992 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 992 "View.MemoryView"
  return __pyx_r;

#line 992 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/

#line 1 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf___pyx_memoryviewslice___reduce_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self) {

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 2 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 2 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 2 "(tree fragment)"
  __PYX_ERR(3, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
/* Python wrapper */

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/

#line 3 "(tree fragment)"
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = 
#line 3 "(tree fragment)"
__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {

#line 3 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 3 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 3 "(tree fragment)"
  int __pyx_lineno = 0;

#line 3 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 3 "(tree fragment)"
  int __pyx_clineno = 0;

#line 3 "(tree fragment)"
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __PYX_ERR(3, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 3 "(tree fragment)"


#line 3 "(tree fragment)"
  /* function exit code */

#line 3 "(tree fragment)"
  __pyx_L1_error:;

#line 3 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 3 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 3 "(tree fragment)"
  __pyx_r = NULL;

#line 3 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 3 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 3 "(tree fragment)"
  return __pyx_r;

#line 3 "(tree fragment)"
}

/* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

#line 999 "View.MemoryView"


#line 999 "View.MemoryView"
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0
#line 999 "View.MemoryView"
;
  Py_ssize_t __pyx_v_suboffset
#line 999 "View.MemoryView"
;
  PyObject *__pyx_v_length = NULL
#line 999 "View.MemoryView"
;

#line 999 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 999 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 999 "View.MemoryView"
  int __pyx_t_1;

#line 999 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 999 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 999 "View.MemoryView"
  __Pyx_TypeInfo *__pyx_t_4;

#line 999 "View.MemoryView"
  Py_buffer __pyx_t_5;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_6;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_7;

#line 999 "View.MemoryView"
  Py_ssize_t *__pyx_t_8;

#line 999 "View.MemoryView"
  Py_ssize_t __pyx_t_9;

#line 999 "View.MemoryView"
  int __pyx_lineno = 0;

#line 999 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 999 "View.MemoryView"
  int __pyx_clineno = 0;

#line 999 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */

#line 1007 "View.MemoryView"
  __pyx_t_1 = ((((PyObject *)__pyx_v_memviewslice.memview) == Py_None) != 0);

#line 1007 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1008
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1008 "View.MemoryView"
    __Pyx_XDECREF(__pyx_r);

#line 1008 "View.MemoryView"
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 1008 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1007
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */

#line 1007 "View.MemoryView"
  }

  /* "View.MemoryView":1013
 * 
 * 
 *     result = _memoryviewslice(None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
 */

#line 1013 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1013 "View.MemoryView"
  __Pyx_INCREF(Py_None);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(Py_None);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None);

#line 1013 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_0);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_int_0);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_0);

#line 1013 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_2 = 0;

#line 1013 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryviewslice_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1013, __pyx_L1_error)

#line 1013 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1013 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1013 "View.MemoryView"
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);

#line 1013 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":1015
 *     result = _memoryviewslice(None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 */

#line 1015 "View.MemoryView"
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":1016
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 */

#line 1016 "View.MemoryView"
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":1018
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 */

#line 1018 "View.MemoryView"
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_memviewslice.memview), __pyx_n_s_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1018, __pyx_L1_error)

#line 1018 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1018 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1018 "View.MemoryView"
  __Pyx_GOTREF(__pyx_v_result->from_object);

#line 1018 "View.MemoryView"
  __Pyx_DECREF(__pyx_v_result->from_object);

#line 1018 "View.MemoryView"
  __pyx_v_result->from_object = __pyx_t_2;

#line 1018 "View.MemoryView"
  __pyx_t_2 = 0;

  /* "View.MemoryView":1019
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
 */

#line 1019 "View.MemoryView"
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;

#line 1019 "View.MemoryView"
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":1021
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 */

#line 1021 "View.MemoryView"
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;

#line 1021 "View.MemoryView"
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":1022
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 */

#line 1022 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":1023
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
 */

#line 1023 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":1024
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
 */

#line 1024 "View.MemoryView"
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":1025
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 */

#line 1025 "View.MemoryView"
  Py_INCREF(Py_None);

  /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */

#line 1027 "View.MemoryView"
  __pyx_t_1 = ((((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)->flags & PyBUF_WRITABLE) != 0);

#line 1027 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1028
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 *         result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 *     else:
 *         result.flags = PyBUF_RECORDS_RO
 */

#line 1028 "View.MemoryView"
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

    /* "View.MemoryView":1027
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
 */

#line 1027 "View.MemoryView"
    goto __pyx_L4;

#line 1027 "View.MemoryView"
  }

  /* "View.MemoryView":1030
 *         result.flags = PyBUF_RECORDS
 *     else:
 *         result.flags = PyBUF_RECORDS_RO             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 */

#line 1030 "View.MemoryView"
  /*else*/ {

#line 1030 "View.MemoryView"
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS_RO;

#line 1030 "View.MemoryView"
  }

#line 1030 "View.MemoryView"
  __pyx_L4:;

  /* "View.MemoryView":1032
 *         result.flags = PyBUF_RECORDS_RO
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 * 
 */

#line 1032 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":1033
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1033 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":1036
 * 
 * 
 *     result.view.suboffsets = NULL             # <<<<<<<<<<<<<<
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

#line 1036 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.suboffsets = NULL;

  /* "View.MemoryView":1037
 * 
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 */

#line 1037 "View.MemoryView"
  __pyx_t_7 = (__pyx_v_result->from_slice.suboffsets + __pyx_v_ndim);

#line 1037 "View.MemoryView"
  for (__pyx_t_8 = __pyx_v_result->from_slice.suboffsets; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {

#line 1037 "View.MemoryView"
    __pyx_t_6 = __pyx_t_8;

#line 1037 "View.MemoryView"
    __pyx_v_suboffset = (__pyx_t_6[0]);

    /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */

#line 1038 "View.MemoryView"
    __pyx_t_1 = ((__pyx_v_suboffset >= 0) != 0);

#line 1038 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":1039
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1039 "View.MemoryView"
      __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

      /* "View.MemoryView":1040
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
 */

#line 1040 "View.MemoryView"
      goto __pyx_L6_break;

      /* "View.MemoryView":1038
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */

#line 1038 "View.MemoryView"
    }

#line 1038 "View.MemoryView"
  }

#line 1038 "View.MemoryView"
  __pyx_L6_break:;

  /* "View.MemoryView":1042
 *             break
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length
 */

#line 1042 "View.MemoryView"
  __pyx_t_9 = __pyx_v_result->__pyx_base.view.itemsize;

#line 1042 "View.MemoryView"
  __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

  /* "View.MemoryView":1043
 * 
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         result.view.len *= length
 * 
 */

#line 1043 "View.MemoryView"
  __pyx_t_7 = (__pyx_v_result->__pyx_base.view.shape + __pyx_v_ndim);

#line 1043 "View.MemoryView"
  for (__pyx_t_8 = __pyx_v_result->__pyx_base.view.shape; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {

#line 1043 "View.MemoryView"
    __pyx_t_6 = __pyx_t_8;

#line 1043 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t((__pyx_t_6[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1043, __pyx_L1_error)

#line 1043 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1043 "View.MemoryView"
    __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_2);

#line 1043 "View.MemoryView"
    __pyx_t_2 = 0;

    /* "View.MemoryView":1044
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
 */

#line 1044 "View.MemoryView"
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_result->__pyx_base.view.len); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1044 "View.MemoryView"
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_2, __pyx_v_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 1044 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1044 "View.MemoryView"
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(3, 1044, __pyx_L1_error)

#line 1044 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1044 "View.MemoryView"
    __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

#line 1044 "View.MemoryView"
  }

  /* "View.MemoryView":1046
 *         result.view.len *= length
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
 */

#line 1046 "View.MemoryView"
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":1047
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 1047 "View.MemoryView"
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":1049
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 */

#line 1049 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 1049 "View.MemoryView"
  __Pyx_INCREF(((PyObject *)__pyx_v_result));

#line 1049 "View.MemoryView"
  __pyx_r = ((PyObject *)__pyx_v_result);

#line 1049 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":999
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

#line 999 "View.MemoryView"


#line 999 "View.MemoryView"
  /* function exit code */

#line 999 "View.MemoryView"
  __pyx_L1_error:;

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 999 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 999 "View.MemoryView"
  __pyx_r = 0;

#line 999 "View.MemoryView"
  __pyx_L0:;

#line 999 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_result);

#line 999 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_length);

#line 999 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 999 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 999 "View.MemoryView"
  return __pyx_r;

#line 999 "View.MemoryView"
}

/* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

#line 1052 "View.MemoryView"


#line 1052 "View.MemoryView"
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0
#line 1052 "View.MemoryView"
;

#line 1052 "View.MemoryView"
  __Pyx_memviewslice *__pyx_r;

#line 1052 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1052 "View.MemoryView"
  int __pyx_t_1;

#line 1052 "View.MemoryView"
  int __pyx_t_2;

#line 1052 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1052 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1052 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1052 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1052 "View.MemoryView"
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */

#line 1055 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 1055 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 1055 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1056
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
 */

#line 1056 "View.MemoryView"
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) __PYX_ERR(3, 1056, __pyx_L1_error)

#line 1056 "View.MemoryView"
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);

#line 1056 "View.MemoryView"
    __Pyx_INCREF(__pyx_t_3);

#line 1056 "View.MemoryView"
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);

#line 1056 "View.MemoryView"
    __pyx_t_3 = 0;

    /* "View.MemoryView":1057
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
 */

#line 1057 "View.MemoryView"
    __pyx_r = (&__pyx_v_obj->from_slice);

#line 1057 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1055
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */

#line 1055 "View.MemoryView"
  }

  /* "View.MemoryView":1059
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
 */

#line 1059 "View.MemoryView"
  /*else*/ {

#line 1059 "View.MemoryView"
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":1060
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
 */

#line 1060 "View.MemoryView"
    __pyx_r = __pyx_v_mslice;

#line 1060 "View.MemoryView"
    goto __pyx_L0;

#line 1060 "View.MemoryView"
  }

  /* "View.MemoryView":1052
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 */

#line 1052 "View.MemoryView"


#line 1052 "View.MemoryView"
  /* function exit code */

#line 1052 "View.MemoryView"
  __pyx_L1_error:;

#line 1052 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1052 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1052 "View.MemoryView"
  __pyx_r = NULL;

#line 1052 "View.MemoryView"
  __pyx_L0:;

#line 1052 "View.MemoryView"
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);

#line 1052 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1052 "View.MemoryView"
  return __pyx_r;

#line 1052 "View.MemoryView"
}

/* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

#line 1063 "View.MemoryView"


#line 1063 "View.MemoryView"
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_shape
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_strides
#line 1063 "View.MemoryView"
;
  Py_ssize_t *__pyx_v_suboffsets
#line 1063 "View.MemoryView"
;

#line 1063 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1063 "View.MemoryView"
  Py_ssize_t *__pyx_t_1;

#line 1063 "View.MemoryView"
  int __pyx_t_2;

#line 1063 "View.MemoryView"
  int __pyx_t_3;

#line 1063 "View.MemoryView"
  int __pyx_t_4;

#line 1063 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 1063 "View.MemoryView"
  __Pyx_RefNannySetupContext("slice_copy", 0);

  /* "View.MemoryView":1067
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
 */

#line 1067 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.shape;

#line 1067 "View.MemoryView"
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":1068
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
 */

#line 1068 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.strides;

#line 1068 "View.MemoryView"
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":1069
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 */

#line 1069 "View.MemoryView"
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;

#line 1069 "View.MemoryView"
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":1071
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
 */

#line 1071 "View.MemoryView"
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":1072
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
 */

#line 1072 "View.MemoryView"
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":1074
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 */

#line 1074 "View.MemoryView"
  __pyx_t_2 = __pyx_v_memview->view.ndim;

#line 1074 "View.MemoryView"
  __pyx_t_3 = __pyx_t_2;

#line 1074 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1074 "View.MemoryView"
    __pyx_v_dim = __pyx_t_4;

    /* "View.MemoryView":1075
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 */

#line 1075 "View.MemoryView"
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":1076
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 */

#line 1076 "View.MemoryView"
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":1077
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
 */

#line 1077 "View.MemoryView"
    if ((__pyx_v_suboffsets != 0)) {

#line 1077 "View.MemoryView"
      __pyx_t_5 = (__pyx_v_suboffsets[__pyx_v_dim]);

#line 1077 "View.MemoryView"
    } else {

#line 1077 "View.MemoryView"
      __pyx_t_5 = -1L;

#line 1077 "View.MemoryView"
    }

#line 1077 "View.MemoryView"
    (__pyx_v_dst->suboffsets[__pyx_v_dim]) = __pyx_t_5;

#line 1077 "View.MemoryView"
  }

  /* "View.MemoryView":1063
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

#line 1063 "View.MemoryView"


#line 1063 "View.MemoryView"
  /* function exit code */

#line 1063 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1063 "View.MemoryView"
}

/* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

#line 1080 "View.MemoryView"


#line 1080 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice
#line 1080 "View.MemoryView"
;

#line 1080 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 1080 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1080 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1080 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1080 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1080 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1080 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":1083
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 */

#line 1083 "View.MemoryView"
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":1084
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 */

#line 1084 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

#line 1084 "View.MemoryView"
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1084, __pyx_L1_error)

#line 1084 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1084 "View.MemoryView"
  __pyx_r = __pyx_t_1;

#line 1084 "View.MemoryView"
  __pyx_t_1 = 0;

#line 1084 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1080
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

#line 1080 "View.MemoryView"


#line 1080 "View.MemoryView"
  /* function exit code */

#line 1080 "View.MemoryView"
  __pyx_L1_error:;

#line 1080 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1080 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1080 "View.MemoryView"
  __pyx_r = 0;

#line 1080 "View.MemoryView"
  __pyx_L0:;

#line 1080 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 1080 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1080 "View.MemoryView"
  return __pyx_r;

#line 1080 "View.MemoryView"
}

/* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

#line 1087 "View.MemoryView"


#line 1087 "View.MemoryView"
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *)
#line 1087 "View.MemoryView"
;
  int (*__pyx_v_to_dtype_func)(char *, PyObject *)
#line 1087 "View.MemoryView"
;

#line 1087 "View.MemoryView"
  PyObject *__pyx_r = NULL;

#line 1087 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1087 "View.MemoryView"
  int __pyx_t_1;

#line 1087 "View.MemoryView"
  int __pyx_t_2;

#line 1087 "View.MemoryView"
  PyObject *(*__pyx_t_3)(char *);

#line 1087 "View.MemoryView"
  int (*__pyx_t_4)(char *, PyObject *);

#line 1087 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 1087 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1087 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1087 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1087 "View.MemoryView"
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */

#line 1094 "View.MemoryView"
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 

#line 1094 "View.MemoryView"
  __pyx_t_2 = (__pyx_t_1 != 0);

#line 1094 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1095
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 */

#line 1095 "View.MemoryView"
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;

#line 1095 "View.MemoryView"
    __pyx_v_to_object_func = __pyx_t_3;

    /* "View.MemoryView":1096
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
 */

#line 1096 "View.MemoryView"
    __pyx_t_4 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;

#line 1096 "View.MemoryView"
    __pyx_v_to_dtype_func = __pyx_t_4;

    /* "View.MemoryView":1094
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */

#line 1094 "View.MemoryView"
    goto __pyx_L3;

#line 1094 "View.MemoryView"
  }

  /* "View.MemoryView":1098
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
 */

#line 1098 "View.MemoryView"
  /*else*/ {

#line 1098 "View.MemoryView"
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1099
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 */

#line 1099 "View.MemoryView"
    __pyx_v_to_dtype_func = NULL;

#line 1099 "View.MemoryView"
  }

#line 1099 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1101
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
 */

#line 1101 "View.MemoryView"
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1103
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1103 "View.MemoryView"
  __pyx_t_5 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 1101, __pyx_L1_error)

#line 1103 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_5);

#line 1103 "View.MemoryView"
  __pyx_r = __pyx_t_5;

#line 1103 "View.MemoryView"
  __pyx_t_5 = 0;

#line 1103 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1087
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

#line 1087 "View.MemoryView"


#line 1087 "View.MemoryView"
  /* function exit code */

#line 1087 "View.MemoryView"
  __pyx_L1_error:;

#line 1087 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 1087 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1087 "View.MemoryView"
  __pyx_r = 0;

#line 1087 "View.MemoryView"
  __pyx_L0:;

#line 1087 "View.MemoryView"
  __Pyx_XGIVEREF(__pyx_r);

#line 1087 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1087 "View.MemoryView"
  return __pyx_r;

#line 1087 "View.MemoryView"
}

/* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

#line 1109 "View.MemoryView"


#line 1109 "View.MemoryView"
static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {

#line 1109 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1109 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */

#line 1110 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_arg < 0) != 0);

#line 1110 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1111
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:
 *         return -arg             # <<<<<<<<<<<<<<
 *     else:
 *         return arg
 */

#line 1111 "View.MemoryView"
    __pyx_r = (-__pyx_v_arg);

#line 1111 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1110
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */

#line 1110 "View.MemoryView"
  }

  /* "View.MemoryView":1113
 *         return -arg
 *     else:
 *         return arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
 */

#line 1113 "View.MemoryView"
  /*else*/ {

#line 1113 "View.MemoryView"
    __pyx_r = __pyx_v_arg;

#line 1113 "View.MemoryView"
    goto __pyx_L0;

#line 1113 "View.MemoryView"
  }

  /* "View.MemoryView":1109
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

#line 1109 "View.MemoryView"


#line 1109 "View.MemoryView"
  /* function exit code */

#line 1109 "View.MemoryView"
  __pyx_L0:;

#line 1109 "View.MemoryView"
  return __pyx_r;

#line 1109 "View.MemoryView"
}

/* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

#line 1116 "View.MemoryView"


#line 1116 "View.MemoryView"
static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i
#line 1116 "View.MemoryView"
;
  Py_ssize_t __pyx_v_c_stride
#line 1116 "View.MemoryView"
;
  Py_ssize_t __pyx_v_f_stride
#line 1116 "View.MemoryView"
;

#line 1116 "View.MemoryView"
  char __pyx_r;

#line 1116 "View.MemoryView"
  int __pyx_t_1;

#line 1116 "View.MemoryView"
  int __pyx_t_2;

#line 1116 "View.MemoryView"
  int __pyx_t_3;

#line 1116 "View.MemoryView"
  int __pyx_t_4;

  /* "View.MemoryView":1121
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
 */

#line 1121 "View.MemoryView"
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1122
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */

#line 1122 "View.MemoryView"
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1124
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 */

#line 1124 "View.MemoryView"
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {

#line 1124 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */

#line 1125 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);

#line 1125 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1126
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1126 "View.MemoryView"
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1127
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */

#line 1127 "View.MemoryView"
      goto __pyx_L4_break;

      /* "View.MemoryView":1125
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */

#line 1125 "View.MemoryView"
    }

#line 1125 "View.MemoryView"
  }

#line 1125 "View.MemoryView"
  __pyx_L4_break:;

  /* "View.MemoryView":1129
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 */

#line 1129 "View.MemoryView"
  __pyx_t_1 = __pyx_v_ndim;

#line 1129 "View.MemoryView"
  __pyx_t_3 = __pyx_t_1;

#line 1129 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1129 "View.MemoryView"
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */

#line 1130 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);

#line 1130 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1131
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */

#line 1131 "View.MemoryView"
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1132
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 */

#line 1132 "View.MemoryView"
      goto __pyx_L7_break;

      /* "View.MemoryView":1130
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */

#line 1130 "View.MemoryView"
    }

#line 1130 "View.MemoryView"
  }

#line 1130 "View.MemoryView"
  __pyx_L7_break:;

  /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */

#line 1134 "View.MemoryView"
  __pyx_t_2 = ((abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride)) != 0);

#line 1134 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1135
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
 */

#line 1135 "View.MemoryView"
    __pyx_r = 'C';

#line 1135 "View.MemoryView"
    goto __pyx_L0;

    /* "View.MemoryView":1134
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */

#line 1134 "View.MemoryView"
  }

  /* "View.MemoryView":1137
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */

#line 1137 "View.MemoryView"
  /*else*/ {

#line 1137 "View.MemoryView"
    __pyx_r = 'F';

#line 1137 "View.MemoryView"
    goto __pyx_L0;

#line 1137 "View.MemoryView"
  }

  /* "View.MemoryView":1116
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

#line 1116 "View.MemoryView"


#line 1116 "View.MemoryView"
  /* function exit code */

#line 1116 "View.MemoryView"
  __pyx_L0:;

#line 1116 "View.MemoryView"
  return __pyx_r;

#line 1116 "View.MemoryView"
}

/* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

#line 1140 "View.MemoryView"


#line 1140 "View.MemoryView"
static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1140 "View.MemoryView"
;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dst_extent
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_src_stride
#line 1140 "View.MemoryView"
;
  Py_ssize_t __pyx_v_dst_stride
#line 1140 "View.MemoryView"
;

#line 1140 "View.MemoryView"
  int __pyx_t_1;

#line 1140 "View.MemoryView"
  int __pyx_t_2;

#line 1140 "View.MemoryView"
  int __pyx_t_3;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_4;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_5;

#line 1140 "View.MemoryView"
  Py_ssize_t __pyx_t_6;

  /* "View.MemoryView":1147
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 */

#line 1147 "View.MemoryView"
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1148
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 */

#line 1148 "View.MemoryView"
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1149
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 */

#line 1149 "View.MemoryView"
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1150
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */

#line 1150 "View.MemoryView"
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */

#line 1152 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);

#line 1152 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_src_stride > 0) != 0);

#line 1153 "View.MemoryView"
    if (__pyx_t_2) {

#line 1153 "View.MemoryView"
    } else {

#line 1153 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 1153 "View.MemoryView"
      goto __pyx_L5_bool_binop_done;

#line 1153 "View.MemoryView"
    }

#line 1153 "View.MemoryView"
    __pyx_t_2 = ((__pyx_v_dst_stride > 0) != 0);

#line 1153 "View.MemoryView"
    if (__pyx_t_2) {

#line 1153 "View.MemoryView"
    } else {

#line 1153 "View.MemoryView"
      __pyx_t_1 = __pyx_t_2;

#line 1153 "View.MemoryView"
      goto __pyx_L5_bool_binop_done;

#line 1153 "View.MemoryView"
    }

    /* "View.MemoryView":1154
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 */

#line 1154 "View.MemoryView"
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);

#line 1154 "View.MemoryView"
    if (__pyx_t_2) {

#line 1154 "View.MemoryView"
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));

#line 1154 "View.MemoryView"
    }

#line 1154 "View.MemoryView"
    __pyx_t_3 = (__pyx_t_2 != 0);

#line 1154 "View.MemoryView"
    __pyx_t_1 = __pyx_t_3;

#line 1154 "View.MemoryView"
    __pyx_L5_bool_binop_done:;

    /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
    if (__pyx_t_1) {

      /* "View.MemoryView":1155
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *        else:
 *            for i in range(dst_extent):
 */

#line 1155 "View.MemoryView"
      (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent)));

      /* "View.MemoryView":1153
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */

#line 1153 "View.MemoryView"
      goto __pyx_L4;

#line 1153 "View.MemoryView"
    }

    /* "View.MemoryView":1157
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 *            for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 */

#line 1157 "View.MemoryView"
    /*else*/ {

#line 1157 "View.MemoryView"
      __pyx_t_4 = __pyx_v_dst_extent;

#line 1157 "View.MemoryView"
      __pyx_t_5 = __pyx_t_4;

#line 1157 "View.MemoryView"
      for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1157 "View.MemoryView"
        __pyx_v_i = __pyx_t_6;

        /* "View.MemoryView":1158
 *        else:
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                src_data += src_stride
 *                dst_data += dst_stride
 */

#line 1158 "View.MemoryView"
        (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize));

        /* "View.MemoryView":1159
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride             # <<<<<<<<<<<<<<
 *                dst_data += dst_stride
 *     else:
 */

#line 1159 "View.MemoryView"
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1160
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 *                dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
 */

#line 1160 "View.MemoryView"
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);

#line 1160 "View.MemoryView"
      }

#line 1160 "View.MemoryView"
    }

#line 1160 "View.MemoryView"
    __pyx_L4:;

    /* "View.MemoryView":1152
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */

#line 1152 "View.MemoryView"
    goto __pyx_L3;

#line 1152 "View.MemoryView"
  }

  /* "View.MemoryView":1162
 *                dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
 */

#line 1162 "View.MemoryView"
  /*else*/ {

#line 1162 "View.MemoryView"
    __pyx_t_4 = __pyx_v_dst_extent;

#line 1162 "View.MemoryView"
    __pyx_t_5 = __pyx_t_4;

#line 1162 "View.MemoryView"
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1162 "View.MemoryView"
      __pyx_v_i = __pyx_t_6;

      /* "View.MemoryView":1163
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
 */

#line 1163 "View.MemoryView"
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1167
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
 */

#line 1167 "View.MemoryView"
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1168
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
 */

#line 1168 "View.MemoryView"
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);

#line 1168 "View.MemoryView"
    }

#line 1168 "View.MemoryView"
  }

#line 1168 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1140
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

#line 1140 "View.MemoryView"


#line 1140 "View.MemoryView"
  /* function exit code */

#line 1140 "View.MemoryView"
}

/* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

#line 1170 "View.MemoryView"


#line 1170 "View.MemoryView"
static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1173
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 */

#line 1173 "View.MemoryView"
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1170
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

#line 1170 "View.MemoryView"


#line 1170 "View.MemoryView"
  /* function exit code */

#line 1170 "View.MemoryView"
}

/* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

#line 1177 "View.MemoryView"


#line 1177 "View.MemoryView"
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_shape
#line 1177 "View.MemoryView"
;
  Py_ssize_t __pyx_v_size
#line 1177 "View.MemoryView"
;

#line 1177 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1177 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_2;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_3;

#line 1177 "View.MemoryView"
  Py_ssize_t *__pyx_t_4;

  /* "View.MemoryView":1179
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for shape in src.shape[:ndim]:
 */

#line 1179 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;

#line 1179 "View.MemoryView"
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1181
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 * 
 *     for shape in src.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         size *= shape
 * 
 */

#line 1181 "View.MemoryView"
  __pyx_t_3 = (__pyx_v_src->shape + __pyx_v_ndim);

#line 1181 "View.MemoryView"
  for (__pyx_t_4 = __pyx_v_src->shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {

#line 1181 "View.MemoryView"
    __pyx_t_2 = __pyx_t_4;

#line 1181 "View.MemoryView"
    __pyx_v_shape = (__pyx_t_2[0]);

    /* "View.MemoryView":1182
 * 
 *     for shape in src.shape[:ndim]:
 *         size *= shape             # <<<<<<<<<<<<<<
 * 
 *     return size
 */

#line 1182 "View.MemoryView"
    __pyx_v_size = (__pyx_v_size * __pyx_v_shape);

#line 1182 "View.MemoryView"
  }

  /* "View.MemoryView":1184
 *         size *= shape
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
 */

#line 1184 "View.MemoryView"
  __pyx_r = __pyx_v_size;

#line 1184 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1177
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 */

#line 1177 "View.MemoryView"


#line 1177 "View.MemoryView"
  /* function exit code */

#line 1177 "View.MemoryView"
  __pyx_L0:;

#line 1177 "View.MemoryView"
  return __pyx_r;

#line 1177 "View.MemoryView"
}

/* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

#line 1187 "View.MemoryView"


#line 1187 "View.MemoryView"
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx
#line 1187 "View.MemoryView"
;

#line 1187 "View.MemoryView"
  Py_ssize_t __pyx_r;

#line 1187 "View.MemoryView"
  int __pyx_t_1;

#line 1187 "View.MemoryView"
  int __pyx_t_2;

#line 1187 "View.MemoryView"
  int __pyx_t_3;

#line 1187 "View.MemoryView"
  int __pyx_t_4;

  /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */

#line 1196 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_order == 'F') != 0);

#line 1196 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1197
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */

#line 1197 "View.MemoryView"
    __pyx_t_2 = __pyx_v_ndim;

#line 1197 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1197 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1197 "View.MemoryView"
      __pyx_v_idx = __pyx_t_4;

      /* "View.MemoryView":1198
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 *     else:
 */

#line 1198 "View.MemoryView"
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1199
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 */

#line 1199 "View.MemoryView"
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));

#line 1199 "View.MemoryView"
    }

    /* "View.MemoryView":1196
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */

#line 1196 "View.MemoryView"
    goto __pyx_L3;

#line 1196 "View.MemoryView"
  }

  /* "View.MemoryView":1201
 *             stride *= shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
 */

#line 1201 "View.MemoryView"
  /*else*/ {

#line 1201 "View.MemoryView"
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1; __pyx_t_2-=1) {

#line 1201 "View.MemoryView"
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1202
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 * 
 */

#line 1202 "View.MemoryView"
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1203
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
 */

#line 1203 "View.MemoryView"
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));

#line 1203 "View.MemoryView"
    }

#line 1203 "View.MemoryView"
  }

#line 1203 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1205
 *             stride *= shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 */

#line 1205 "View.MemoryView"
  __pyx_r = __pyx_v_stride;

#line 1205 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1187
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

#line 1187 "View.MemoryView"


#line 1187 "View.MemoryView"
  /* function exit code */

#line 1187 "View.MemoryView"
  __pyx_L0:;

#line 1187 "View.MemoryView"
  return __pyx_r;

#line 1187 "View.MemoryView"
}

/* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

#line 1208 "View.MemoryView"


#line 1208 "View.MemoryView"
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i
#line 1208 "View.MemoryView"
;
  void *__pyx_v_result
#line 1208 "View.MemoryView"
;
  size_t __pyx_v_itemsize
#line 1208 "View.MemoryView"
;
  size_t __pyx_v_size
#line 1208 "View.MemoryView"
;

#line 1208 "View.MemoryView"
  void *__pyx_r;

#line 1208 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1208 "View.MemoryView"
  int __pyx_t_2;

#line 1208 "View.MemoryView"
  int __pyx_t_3;

#line 1208 "View.MemoryView"
  struct __pyx_memoryview_obj *__pyx_t_4;

#line 1208 "View.MemoryView"
  int __pyx_t_5;

#line 1208 "View.MemoryView"
  int __pyx_t_6;

#line 1208 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1208 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1208 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":1219
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 */

#line 1219 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;

#line 1219 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1220
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
 */

#line 1220 "View.MemoryView"
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1222
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err(MemoryError, NULL)
 */

#line 1222 "View.MemoryView"
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */

#line 1223 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_result != 0)) != 0);

#line 1223 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1224
 *     result = malloc(size)
 *     if not result:
 *         _err(MemoryError, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1224 "View.MemoryView"
    __pyx_t_3 = __pyx_memoryview_err(__pyx_builtin_MemoryError, NULL); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(3, 1224, __pyx_L1_error)

    /* "View.MemoryView":1223
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */

#line 1223 "View.MemoryView"
  }

  /* "View.MemoryView":1227
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 */

#line 1227 "View.MemoryView"
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1228
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 */

#line 1228 "View.MemoryView"
  __pyx_t_4 = __pyx_v_src->memview;

#line 1228 "View.MemoryView"
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1229
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
 */

#line 1229 "View.MemoryView"
  __pyx_t_3 = __pyx_v_ndim;

#line 1229 "View.MemoryView"
  __pyx_t_5 = __pyx_t_3;

#line 1229 "View.MemoryView"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1229 "View.MemoryView"
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1230
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
 */

#line 1230 "View.MemoryView"
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1231
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,
 */

#line 1231 "View.MemoryView"
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1L;

#line 1231 "View.MemoryView"
  }

  /* "View.MemoryView":1233
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,             # <<<<<<<<<<<<<<
 *                               ndim, order)
 * 
 */

#line 1233 "View.MemoryView"
  (void)(__pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order));

  /* "View.MemoryView":1237
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
 */

#line 1237 "View.MemoryView"
  __pyx_t_3 = __pyx_v_ndim;

#line 1237 "View.MemoryView"
  __pyx_t_5 = __pyx_t_3;

#line 1237 "View.MemoryView"
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {

#line 1237 "View.MemoryView"
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */

#line 1238 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1) != 0);

#line 1238 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1239
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src[0], order, ndim):
 */

#line 1239 "View.MemoryView"
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;

      /* "View.MemoryView":1238
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */

#line 1238 "View.MemoryView"
    }

#line 1238 "View.MemoryView"
  }

  /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */

#line 1241 "View.MemoryView"
  __pyx_t_2 = (__pyx_memviewslice_is_contig((__pyx_v_src[0]), __pyx_v_order, __pyx_v_ndim) != 0);

#line 1241 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1242
 * 
 *     if slice_is_contig(src[0], order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 */

#line 1242 "View.MemoryView"
    (void)(memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size));

    /* "View.MemoryView":1241
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */

#line 1241 "View.MemoryView"
    goto __pyx_L9;

#line 1241 "View.MemoryView"
  }

  /* "View.MemoryView":1244
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
 */

#line 1244 "View.MemoryView"
  /*else*/ {

#line 1244 "View.MemoryView"
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);

#line 1244 "View.MemoryView"
  }

#line 1244 "View.MemoryView"
  __pyx_L9:;

  /* "View.MemoryView":1246
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1246 "View.MemoryView"
  __pyx_r = __pyx_v_result;

#line 1246 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

#line 1208 "View.MemoryView"


#line 1208 "View.MemoryView"
  /* function exit code */

#line 1208 "View.MemoryView"
  __pyx_L1_error:;

#line 1208 "View.MemoryView"
  {

#line 1208 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 1208 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1208 "View.MemoryView"
    #endif

#line 1208 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1208 "View.MemoryView"
    #ifdef WITH_THREAD

#line 1208 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1208 "View.MemoryView"
    #endif

#line 1208 "View.MemoryView"
  }

#line 1208 "View.MemoryView"
  __pyx_r = NULL;

#line 1208 "View.MemoryView"
  __pyx_L0:;

#line 1208 "View.MemoryView"
  return __pyx_r;

#line 1208 "View.MemoryView"
}

/* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

#line 1251 "View.MemoryView"


#line 1251 "View.MemoryView"
static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {

#line 1251 "View.MemoryView"
  int __pyx_r;

#line 1251 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1251 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1251 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1251 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1251 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1251 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1251 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1251 "View.MemoryView"
  #endif

#line 1251 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1254
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 *                                                         (i, extent1, extent2))             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
 */

#line 1254 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1254 "View.MemoryView"
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_extent1); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1254 "View.MemoryView"
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_extent2); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1254 "View.MemoryView"
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1254, __pyx_L1_error)

#line 1254 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_2);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);

#line 1254 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1254 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);

#line 1254 "View.MemoryView"
  __pyx_t_1 = 0;

#line 1254 "View.MemoryView"
  __pyx_t_2 = 0;

#line 1254 "View.MemoryView"
  __pyx_t_3 = 0;

  /* "View.MemoryView":1253
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %             # <<<<<<<<<<<<<<
 *                                                         (i, extent1, extent2))
 * 
 */

#line 1253 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1253, __pyx_L1_error)

#line 1253 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1253 "View.MemoryView"
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1253, __pyx_L1_error)

#line 1253 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1253 "View.MemoryView"
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 1253 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1253 "View.MemoryView"
  __PYX_ERR(3, 1253, __pyx_L1_error)

  /* "View.MemoryView":1251
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

#line 1251 "View.MemoryView"


#line 1251 "View.MemoryView"
  /* function exit code */

#line 1251 "View.MemoryView"
  __pyx_L1_error:;

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1251 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1251 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1251 "View.MemoryView"
  __pyx_r = -1;

#line 1251 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1251 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1251 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1251 "View.MemoryView"
  #endif

#line 1251 "View.MemoryView"
  return __pyx_r;

#line 1251 "View.MemoryView"
}

/* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

#line 1257 "View.MemoryView"


#line 1257 "View.MemoryView"
static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, char *__pyx_v_msg, int __pyx_v_dim) {

#line 1257 "View.MemoryView"
  int __pyx_r;

#line 1257 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_1 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1257 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1257 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1257 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1257 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1257 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1257 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1257 "View.MemoryView"
  #endif

#line 1257 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err_dim", 0);

#line 1257 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1258
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:
 *     raise error(msg.decode('ascii') % dim)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
 */

#line 1258 "View.MemoryView"
  __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_2);

#line 1258 "View.MemoryView"
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_3);

#line 1258 "View.MemoryView"
  __pyx_t_4 = PyUnicode_Format(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_4);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1258 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);
  __pyx_t_3 = __pyx_v_error; 
#line 1258 "View.MemoryView"
__pyx_t_2 = NULL;

#line 1258 "View.MemoryView"
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {

#line 1258 "View.MemoryView"
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);

#line 1258 "View.MemoryView"
    if (likely(__pyx_t_2)) {

#line 1258 "View.MemoryView"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);

#line 1258 "View.MemoryView"
      __Pyx_INCREF(__pyx_t_2);

#line 1258 "View.MemoryView"
      __Pyx_INCREF(function);

#line 1258 "View.MemoryView"
      __Pyx_DECREF_SET(__pyx_t_3, function);

#line 1258 "View.MemoryView"
    }

#line 1258 "View.MemoryView"
  }

#line 1258 "View.MemoryView"
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);

#line 1258 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1258 "View.MemoryView"
  if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1258, __pyx_L1_error)

#line 1258 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1258 "View.MemoryView"
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);

#line 1258 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1258 "View.MemoryView"
  __PYX_ERR(3, 1258, __pyx_L1_error)

  /* "View.MemoryView":1257
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

#line 1257 "View.MemoryView"


#line 1257 "View.MemoryView"
  /* function exit code */

#line 1257 "View.MemoryView"
  __pyx_L1_error:;

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_1);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1257 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1257 "View.MemoryView"
  __pyx_r = -1;

#line 1257 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_error);

#line 1257 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1257 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1257 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1257 "View.MemoryView"
  #endif

#line 1257 "View.MemoryView"
  return __pyx_r;

#line 1257 "View.MemoryView"
}

/* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

#line 1261 "View.MemoryView"


#line 1261 "View.MemoryView"
static int __pyx_memoryview_err(PyObject *__pyx_v_error, char *__pyx_v_msg) {

#line 1261 "View.MemoryView"
  int __pyx_r;

#line 1261 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1261 "View.MemoryView"
  int __pyx_t_1;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_2 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_3 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_4 = NULL;

#line 1261 "View.MemoryView"
  PyObject *__pyx_t_5 = NULL;

#line 1261 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1261 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1261 "View.MemoryView"
  int __pyx_clineno = 0;

#line 1261 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1261 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1261 "View.MemoryView"
  #endif

#line 1261 "View.MemoryView"
  __Pyx_RefNannySetupContext("_err", 0);

#line 1261 "View.MemoryView"
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */

#line 1262 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_msg != NULL) != 0);

#line 1262 "View.MemoryView"
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":1263
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))             # <<<<<<<<<<<<<<
 *     else:
 *         raise error
 */

#line 1263 "View.MemoryView"
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) __PYX_ERR(3, 1263, __pyx_L1_error)

#line 1263 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_3);

#line 1263 "View.MemoryView"
    __Pyx_INCREF(__pyx_v_error);
    __pyx_t_4 = __pyx_v_error; 
#line 1263 "View.MemoryView"
__pyx_t_5 = NULL;

#line 1263 "View.MemoryView"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {

#line 1263 "View.MemoryView"
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);

#line 1263 "View.MemoryView"
      if (likely(__pyx_t_5)) {

#line 1263 "View.MemoryView"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);

#line 1263 "View.MemoryView"
        __Pyx_INCREF(__pyx_t_5);

#line 1263 "View.MemoryView"
        __Pyx_INCREF(function);

#line 1263 "View.MemoryView"
        __Pyx_DECREF_SET(__pyx_t_4, function);

#line 1263 "View.MemoryView"
      }

#line 1263 "View.MemoryView"
    }

#line 1263 "View.MemoryView"
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);

#line 1263 "View.MemoryView"
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 1263 "View.MemoryView"
    if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 1263, __pyx_L1_error)

#line 1263 "View.MemoryView"
    __Pyx_GOTREF(__pyx_t_2);

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 1263 "View.MemoryView"
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);

#line 1263 "View.MemoryView"
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

#line 1263 "View.MemoryView"
    __PYX_ERR(3, 1263, __pyx_L1_error)

    /* "View.MemoryView":1262
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */

#line 1262 "View.MemoryView"
  }

  /* "View.MemoryView":1265
 *         raise error(msg.decode('ascii'))
 *     else:
 *         raise error             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_contents')
 */

#line 1265 "View.MemoryView"
  /*else*/ {

#line 1265 "View.MemoryView"
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);

#line 1265 "View.MemoryView"
    __PYX_ERR(3, 1265, __pyx_L1_error)

#line 1265 "View.MemoryView"
  }

  /* "View.MemoryView":1261
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

#line 1261 "View.MemoryView"


#line 1261 "View.MemoryView"
  /* function exit code */

#line 1261 "View.MemoryView"
  __pyx_L1_error:;

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_2);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_3);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_4);

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_t_5);

#line 1261 "View.MemoryView"
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1261 "View.MemoryView"
  __pyx_r = -1;

#line 1261 "View.MemoryView"
  __Pyx_XDECREF(__pyx_v_error);

#line 1261 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1261 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1261 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1261 "View.MemoryView"
  #endif

#line 1261 "View.MemoryView"
  return __pyx_r;

#line 1261 "View.MemoryView"
}

/* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

#line 1268 "View.MemoryView"


#line 1268 "View.MemoryView"
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata
#line 1268 "View.MemoryView"
;
  size_t __pyx_v_itemsize
#line 1268 "View.MemoryView"
;
  int __pyx_v_i
#line 1268 "View.MemoryView"
;
  char __pyx_v_order
#line 1268 "View.MemoryView"
;
  int __pyx_v_broadcasting
#line 1268 "View.MemoryView"
;
  int __pyx_v_direct_copy
#line 1268 "View.MemoryView"
;
  __Pyx_memviewslice __pyx_v_tmp
#line 1268 "View.MemoryView"
;
  int __pyx_v_ndim
#line 1268 "View.MemoryView"
;

#line 1268 "View.MemoryView"
  int __pyx_r;

#line 1268 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1268 "View.MemoryView"
  int __pyx_t_2;

#line 1268 "View.MemoryView"
  int __pyx_t_3;

#line 1268 "View.MemoryView"
  int __pyx_t_4;

#line 1268 "View.MemoryView"
  int __pyx_t_5;

#line 1268 "View.MemoryView"
  int __pyx_t_6;

#line 1268 "View.MemoryView"
  void *__pyx_t_7;

#line 1268 "View.MemoryView"
  int __pyx_t_8;

#line 1268 "View.MemoryView"
  int __pyx_lineno = 0;

#line 1268 "View.MemoryView"
  const char *__pyx_filename = NULL;

#line 1268 "View.MemoryView"
  int __pyx_clineno = 0;

  /* "View.MemoryView":1276
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 */

#line 1276 "View.MemoryView"
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1277
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 */

#line 1277 "View.MemoryView"
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;

#line 1277 "View.MemoryView"
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1279
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
 */

#line 1279 "View.MemoryView"
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1280
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
 */

#line 1280 "View.MemoryView"
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1281
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
 */

#line 1281 "View.MemoryView"
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */

#line 1284 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_src_ndim < __pyx_v_dst_ndim) != 0);

#line 1284 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1285
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 */

#line 1285 "View.MemoryView"
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);

    /* "View.MemoryView":1284
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */

#line 1284 "View.MemoryView"
    goto __pyx_L3;

#line 1284 "View.MemoryView"
  }

  /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */

#line 1286 "View.MemoryView"
  __pyx_t_2 = ((__pyx_v_dst_ndim < __pyx_v_src_ndim) != 0);

#line 1286 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1287
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
 */

#line 1287 "View.MemoryView"
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);

    /* "View.MemoryView":1286
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */

#line 1286 "View.MemoryView"
  }

#line 1286 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1289
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */

#line 1289 "View.MemoryView"
  __pyx_t_3 = __pyx_v_dst_ndim;

#line 1289 "View.MemoryView"
  __pyx_t_4 = __pyx_v_src_ndim;

#line 1289 "View.MemoryView"
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {

#line 1289 "View.MemoryView"
    __pyx_t_5 = __pyx_t_3;

#line 1289 "View.MemoryView"
  } else {

#line 1289 "View.MemoryView"
    __pyx_t_5 = __pyx_t_4;

#line 1289 "View.MemoryView"
  }

#line 1289 "View.MemoryView"
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1291
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 */

#line 1291 "View.MemoryView"
  __pyx_t_5 = __pyx_v_ndim;

#line 1291 "View.MemoryView"
  __pyx_t_3 = __pyx_t_5;

#line 1291 "View.MemoryView"
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1291 "View.MemoryView"
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */

#line 1292 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i])) != 0);

#line 1292 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */

#line 1293 "View.MemoryView"
      __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) == 1) != 0);

#line 1293 "View.MemoryView"
      if (__pyx_t_2) {

        /* "View.MemoryView":1294
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
 */

#line 1294 "View.MemoryView"
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1295
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 */

#line 1295 "View.MemoryView"
        (__pyx_v_src.strides[__pyx_v_i]) = 0;

        /* "View.MemoryView":1293
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */

#line 1293 "View.MemoryView"
        goto __pyx_L7;

#line 1293 "View.MemoryView"
      }

      /* "View.MemoryView":1297
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
 */

#line 1297 "View.MemoryView"
      /*else*/ {

#line 1297 "View.MemoryView"
        __pyx_t_6 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 1297, __pyx_L1_error)

#line 1297 "View.MemoryView"
      }

#line 1297 "View.MemoryView"
      __pyx_L7:;

      /* "View.MemoryView":1292
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */

#line 1292 "View.MemoryView"
    }

    /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */

#line 1299 "View.MemoryView"
    __pyx_t_2 = (((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0) != 0);

#line 1299 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1300
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 */

#line 1300 "View.MemoryView"
      __pyx_t_6 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, ((char *)"Dimension %d is not direct"), __pyx_v_i); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(3, 1300, __pyx_L1_error)

      /* "View.MemoryView":1299
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */

#line 1299 "View.MemoryView"
    }

#line 1299 "View.MemoryView"
  }

  /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */

#line 1302 "View.MemoryView"
  __pyx_t_2 = (__pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize) != 0);

#line 1302 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */

#line 1304 "View.MemoryView"
    __pyx_t_2 = ((!(__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim) != 0)) != 0);

#line 1304 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1305
 * 
 *         if not slice_is_contig(src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 */

#line 1305 "View.MemoryView"
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);

      /* "View.MemoryView":1304
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */

#line 1304 "View.MemoryView"
    }

    /* "View.MemoryView":1307
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
 */

#line 1307 "View.MemoryView"
    __pyx_t_7 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(3, 1307, __pyx_L1_error)

#line 1307 "View.MemoryView"
    __pyx_v_tmpdata = __pyx_t_7;

    /* "View.MemoryView":1308
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
 */

#line 1308 "View.MemoryView"
    __pyx_v_src = __pyx_v_tmp;

    /* "View.MemoryView":1302
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
 */

#line 1302 "View.MemoryView"
  }

  /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1310 "View.MemoryView"
  __pyx_t_2 = ((!(__pyx_v_broadcasting != 0)) != 0);

#line 1310 "View.MemoryView"
  if (__pyx_t_2) {

    /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */

#line 1313 "View.MemoryView"
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'C', __pyx_v_ndim) != 0);

#line 1313 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1314
 * 
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 */

#line 1314 "View.MemoryView"
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'C', __pyx_v_ndim);

      /* "View.MemoryView":1313
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 */

#line 1313 "View.MemoryView"
      goto __pyx_L12;

#line 1313 "View.MemoryView"
    }

    /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */

#line 1315 "View.MemoryView"
    __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, 'F', __pyx_v_ndim) != 0);

#line 1315 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1316
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
 */

#line 1316 "View.MemoryView"
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'F', __pyx_v_ndim);

      /* "View.MemoryView":1315
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 */

#line 1315 "View.MemoryView"
    }

#line 1315 "View.MemoryView"
    __pyx_L12:;

    /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1318 "View.MemoryView"
    __pyx_t_2 = (__pyx_v_direct_copy != 0);

#line 1318 "View.MemoryView"
    if (__pyx_t_2) {

      /* "View.MemoryView":1320
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 */

#line 1320 "View.MemoryView"
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1321
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 */

#line 1321 "View.MemoryView"
      (void)(memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim)));

      /* "View.MemoryView":1322
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
 */

#line 1322 "View.MemoryView"
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1323
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */

#line 1323 "View.MemoryView"
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1324
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
 */

#line 1324 "View.MemoryView"
      __pyx_r = 0;

#line 1324 "View.MemoryView"
      goto __pyx_L0;

      /* "View.MemoryView":1318
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1318 "View.MemoryView"
    }

    /* "View.MemoryView":1310
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1310 "View.MemoryView"
  }

  /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1326 "View.MemoryView"
  __pyx_t_2 = (__pyx_v_order == 'F');

#line 1326 "View.MemoryView"
  if (__pyx_t_2) {

#line 1326 "View.MemoryView"
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));

#line 1326 "View.MemoryView"
  }

#line 1326 "View.MemoryView"
  __pyx_t_8 = (__pyx_t_2 != 0);

#line 1326 "View.MemoryView"
  if (__pyx_t_8) {

    /* "View.MemoryView":1329
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
 */

#line 1329 "View.MemoryView"
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(3, 1329, __pyx_L1_error)

    /* "View.MemoryView":1330
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 */

#line 1330 "View.MemoryView"
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == ((int)0))) __PYX_ERR(3, 1330, __pyx_L1_error)

    /* "View.MemoryView":1326
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1326 "View.MemoryView"
  }

  /* "View.MemoryView":1332
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 */

#line 1332 "View.MemoryView"
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1333
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 */

#line 1333 "View.MemoryView"
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1334
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
 */

#line 1334 "View.MemoryView"
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1336
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */

#line 1336 "View.MemoryView"
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1337
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 */

#line 1337 "View.MemoryView"
  __pyx_r = 0;

#line 1337 "View.MemoryView"
  goto __pyx_L0;

  /* "View.MemoryView":1268
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

#line 1268 "View.MemoryView"


#line 1268 "View.MemoryView"
  /* function exit code */

#line 1268 "View.MemoryView"
  __pyx_L1_error:;

#line 1268 "View.MemoryView"
  {

#line 1268 "View.MemoryView"
    #ifdef WITH_THREAD
    PyGILState_STATE 
#line 1268 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1268 "View.MemoryView"
    #endif

#line 1268 "View.MemoryView"
    __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1268 "View.MemoryView"
    #ifdef WITH_THREAD

#line 1268 "View.MemoryView"
    __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1268 "View.MemoryView"
    #endif

#line 1268 "View.MemoryView"
  }

#line 1268 "View.MemoryView"
  __pyx_r = -1;

#line 1268 "View.MemoryView"
  __pyx_L0:;

#line 1268 "View.MemoryView"
  return __pyx_r;

#line 1268 "View.MemoryView"
}

/* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

#line 1340 "View.MemoryView"


#line 1340 "View.MemoryView"
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i
#line 1340 "View.MemoryView"
;
  int __pyx_v_offset
#line 1340 "View.MemoryView"
;

#line 1340 "View.MemoryView"
  int __pyx_t_1;

#line 1340 "View.MemoryView"
  int __pyx_t_2;

#line 1340 "View.MemoryView"
  int __pyx_t_3;

  /* "View.MemoryView":1344
 *                             int ndim_other) nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */

#line 1344 "View.MemoryView"
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1346
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 */

#line 1346 "View.MemoryView"
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {

#line 1346 "View.MemoryView"
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1347
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]             # <<<<<<<<<<<<<<
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 */

#line 1347 "View.MemoryView"
    (__pyx_v_mslice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->shape[__pyx_v_i]);

    /* "View.MemoryView":1348
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 */

#line 1348 "View.MemoryView"
    (__pyx_v_mslice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->strides[__pyx_v_i]);

    /* "View.MemoryView":1349
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
 */

#line 1349 "View.MemoryView"
    (__pyx_v_mslice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->suboffsets[__pyx_v_i]);

#line 1349 "View.MemoryView"
  }

  /* "View.MemoryView":1351
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 */

#line 1351 "View.MemoryView"
  __pyx_t_1 = __pyx_v_offset;

#line 1351 "View.MemoryView"
  __pyx_t_2 = __pyx_t_1;

#line 1351 "View.MemoryView"
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {

#line 1351 "View.MemoryView"
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1352
 * 
 *     for i in range(offset):
 *         mslice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1
 */

#line 1352 "View.MemoryView"
    (__pyx_v_mslice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1353
 *     for i in range(offset):
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i] = -1
 * 
 */

#line 1353 "View.MemoryView"
    (__pyx_v_mslice->strides[__pyx_v_i]) = (__pyx_v_mslice->strides[0]);

    /* "View.MemoryView":1354
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1354 "View.MemoryView"
    (__pyx_v_mslice->suboffsets[__pyx_v_i]) = -1L;

#line 1354 "View.MemoryView"
  }

  /* "View.MemoryView":1340
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

#line 1340 "View.MemoryView"


#line 1340 "View.MemoryView"
  /* function exit code */

#line 1340 "View.MemoryView"
}

/* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

#line 1362 "View.MemoryView"


#line 1362 "View.MemoryView"
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {

#line 1362 "View.MemoryView"
  int __pyx_t_1;

  /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */

#line 1366 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_dtype_is_object != 0);

#line 1366 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1367
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,             # <<<<<<<<<<<<<<
 *                                            dst.strides, ndim, inc)
 * 
 */

#line 1367 "View.MemoryView"
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);

    /* "View.MemoryView":1366
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */

#line 1366 "View.MemoryView"
  }

  /* "View.MemoryView":1362
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

#line 1362 "View.MemoryView"


#line 1362 "View.MemoryView"
  /* function exit code */

#line 1362 "View.MemoryView"
}

/* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

#line 1371 "View.MemoryView"


#line 1371 "View.MemoryView"
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {

#line 1371 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1371 "View.MemoryView"
  #ifdef WITH_THREAD
  PyGILState_STATE 
#line 1371 "View.MemoryView"
__pyx_gilstate_save = __Pyx_PyGILState_Ensure();

#line 1371 "View.MemoryView"
  #endif

#line 1371 "View.MemoryView"
  __Pyx_RefNannySetupContext("refcount_objects_in_slice_with_gil", 0);

  /* "View.MemoryView":1374
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 */

#line 1374 "View.MemoryView"
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1371
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

#line 1371 "View.MemoryView"


#line 1371 "View.MemoryView"
  /* function exit code */

#line 1371 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1371 "View.MemoryView"
  #ifdef WITH_THREAD

#line 1371 "View.MemoryView"
  __Pyx_PyGILState_Release(__pyx_gilstate_save);

#line 1371 "View.MemoryView"
  #endif

#line 1371 "View.MemoryView"
}

/* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

#line 1377 "View.MemoryView"


#line 1377 "View.MemoryView"
static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1377 "View.MemoryView"
;

#line 1377 "View.MemoryView"
  __Pyx_RefNannyDeclarations

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_1;

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_2;

#line 1377 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 1377 "View.MemoryView"
  int __pyx_t_4;

#line 1377 "View.MemoryView"
  __Pyx_RefNannySetupContext("refcount_objects_in_slice", 0);

  /* "View.MemoryView":1381
 *     cdef Py_ssize_t i
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
 */

#line 1381 "View.MemoryView"
  __pyx_t_1 = (__pyx_v_shape[0]);

#line 1381 "View.MemoryView"
  __pyx_t_2 = __pyx_t_1;

#line 1381 "View.MemoryView"
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {

#line 1381 "View.MemoryView"
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */

#line 1382 "View.MemoryView"
    __pyx_t_4 = ((__pyx_v_ndim == 1) != 0);

#line 1382 "View.MemoryView"
    if (__pyx_t_4) {

      /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */

#line 1383 "View.MemoryView"
      __pyx_t_4 = (__pyx_v_inc != 0);

#line 1383 "View.MemoryView"
      if (__pyx_t_4) {

        /* "View.MemoryView":1384
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
 */

#line 1384 "View.MemoryView"
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));

        /* "View.MemoryView":1383
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */

#line 1383 "View.MemoryView"
        goto __pyx_L6;

#line 1383 "View.MemoryView"
      }

      /* "View.MemoryView":1386
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 */

#line 1386 "View.MemoryView"
      /*else*/ {

#line 1386 "View.MemoryView"
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));

#line 1386 "View.MemoryView"
      }

#line 1386 "View.MemoryView"
      __pyx_L6:;

      /* "View.MemoryView":1382
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */

#line 1382 "View.MemoryView"
      goto __pyx_L5;

#line 1382 "View.MemoryView"
    }

    /* "View.MemoryView":1388
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                       ndim - 1, inc)
 * 
 */

#line 1388 "View.MemoryView"
    /*else*/ {

      /* "View.MemoryView":1389
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 *                                       ndim - 1, inc)             # <<<<<<<<<<<<<<
 * 
 *         data += strides[0]
 */

#line 1389 "View.MemoryView"
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);

#line 1389 "View.MemoryView"
    }

#line 1389 "View.MemoryView"
    __pyx_L5:;

    /* "View.MemoryView":1391
 *                                       ndim - 1, inc)
 * 
 *         data += strides[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1391 "View.MemoryView"
    __pyx_v_data = (__pyx_v_data + (__pyx_v_strides[0]));

#line 1391 "View.MemoryView"
  }

  /* "View.MemoryView":1377
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

#line 1377 "View.MemoryView"


#line 1377 "View.MemoryView"
  /* function exit code */

#line 1377 "View.MemoryView"
  __Pyx_RefNannyFinishContext();

#line 1377 "View.MemoryView"
}

/* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

#line 1397 "View.MemoryView"


#line 1397 "View.MemoryView"
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1400
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 */

#line 1400 "View.MemoryView"
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1401
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,             # <<<<<<<<<<<<<<
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)
 */

#line 1401 "View.MemoryView"
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1403
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1403 "View.MemoryView"
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1397
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

#line 1397 "View.MemoryView"


#line 1397 "View.MemoryView"
  /* function exit code */

#line 1397 "View.MemoryView"
}

/* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

#line 1407 "View.MemoryView"


#line 1407 "View.MemoryView"
static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i
#line 1407 "View.MemoryView"
;
  Py_ssize_t __pyx_v_stride
#line 1407 "View.MemoryView"
;
  Py_ssize_t __pyx_v_extent
#line 1407 "View.MemoryView"
;

#line 1407 "View.MemoryView"
  int __pyx_t_1;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_2;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_3;

#line 1407 "View.MemoryView"
  Py_ssize_t __pyx_t_4;

  /* "View.MemoryView":1411
 *                               size_t itemsize, void *item) nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
 */

#line 1411 "View.MemoryView"
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1412
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */

#line 1412 "View.MemoryView"
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */

#line 1414 "View.MemoryView"
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);

#line 1414 "View.MemoryView"
  if (__pyx_t_1) {

    /* "View.MemoryView":1415
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
 */

#line 1415 "View.MemoryView"
    __pyx_t_2 = __pyx_v_extent;

#line 1415 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1415 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1415 "View.MemoryView"
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1416
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
 */

#line 1416 "View.MemoryView"
      (void)(memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize));

      /* "View.MemoryView":1417
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
 */

#line 1417 "View.MemoryView"
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);

#line 1417 "View.MemoryView"
    }

    /* "View.MemoryView":1414
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */

#line 1414 "View.MemoryView"
    goto __pyx_L3;

#line 1414 "View.MemoryView"
  }

  /* "View.MemoryView":1419
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 */

#line 1419 "View.MemoryView"
  /*else*/ {

#line 1419 "View.MemoryView"
    __pyx_t_2 = __pyx_v_extent;

#line 1419 "View.MemoryView"
    __pyx_t_3 = __pyx_t_2;

#line 1419 "View.MemoryView"
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {

#line 1419 "View.MemoryView"
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1420
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                 ndim - 1, itemsize, item)
 *             data += stride
 */

#line 1420 "View.MemoryView"
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1422
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 1422 "View.MemoryView"
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);

#line 1422 "View.MemoryView"
    }

#line 1422 "View.MemoryView"
  }

#line 1422 "View.MemoryView"
  __pyx_L3:;

  /* "View.MemoryView":1407
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

#line 1407 "View.MemoryView"


#line 1407 "View.MemoryView"
  /* function exit code */

#line 1407 "View.MemoryView"
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
/* Python wrapper */

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum = 
#line 1 "(tree fragment)"
{"__pyx_unpickle_Enum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum, METH_VARARGS|METH_KEYWORDS, 0};

#line 1 "(tree fragment)"
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0
#line 1 "(tree fragment)"
;
  long __pyx_v___pyx_checksum
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_state = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum (wrapper)", 0);

#line 1 "(tree fragment)"
  {

#line 1 "(tree fragment)"
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};

#line 1 "(tree fragment)"
    PyObject* values[3] = {0,0,0};

#line 1 "(tree fragment)"
    if (unlikely(__pyx_kwds)) {

#line 1 "(tree fragment)"
      Py_ssize_t kw_args;

#line 1 "(tree fragment)"
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);

#line 1 "(tree fragment)"
      switch (pos_args) {
        case  3: 
#line 1 "(tree fragment)"
values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  2: 
#line 1 "(tree fragment)"
values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;
        case  1: 
#line 1 "(tree fragment)"
values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  0: break;
        default: 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      kw_args = PyDict_Size(__pyx_kwds);

#line 1 "(tree fragment)"
      switch (pos_args) {

#line 1 "(tree fragment)"
        case  0:

#line 1 "(tree fragment)"
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else 
#line 1 "(tree fragment)"
goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  1:

#line 1 "(tree fragment)"
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 1); 
#line 1 "(tree fragment)"
__PYX_ERR(3, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
        CYTHON_FALLTHROUGH;

#line 1 "(tree fragment)"
        case  2:

#line 1 "(tree fragment)"
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;

#line 1 "(tree fragment)"
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, 2); 
#line 1 "(tree fragment)"
__PYX_ERR(3, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
        }

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
      if (unlikely(kw_args > 0)) {

#line 1 "(tree fragment)"
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Enum") < 0)) __PYX_ERR(3, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
      }

#line 1 "(tree fragment)"
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {

#line 1 "(tree fragment)"
      goto __pyx_L5_argtuple_error;

#line 1 "(tree fragment)"
    } else {

#line 1 "(tree fragment)"
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);

#line 1 "(tree fragment)"
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);

#line 1 "(tree fragment)"
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);

#line 1 "(tree fragment)"
    }

#line 1 "(tree fragment)"
    __pyx_v___pyx_type = values[0];

#line 1 "(tree fragment)"
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(3, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
    __pyx_v___pyx_state = values[2];

#line 1 "(tree fragment)"
  }

#line 1 "(tree fragment)"
  goto __pyx_L4_argument_unpacking_done;

#line 1 "(tree fragment)"
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); 
#line 1 "(tree fragment)"
__PYX_ERR(3, 1, __pyx_L3_error)

#line 1 "(tree fragment)"
  __pyx_L3_error:;

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return NULL;

#line 1 "(tree fragment)"
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = 
#line 1 "(tree fragment)"
__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0
#line 1 "(tree fragment)"
;
  PyObject *__pyx_v___pyx_result = 0
#line 1 "(tree fragment)"
;

#line 1 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 1 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 1 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 1 "(tree fragment)"
  int __pyx_t_2;

#line 1 "(tree fragment)"
  int __pyx_t_3;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_4 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_5 = NULL;

#line 1 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 1 "(tree fragment)"
  int __pyx_lineno = 0;

#line 1 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 1 "(tree fragment)"
  int __pyx_clineno = 0;

#line 1 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 4 "(tree fragment)"
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__28, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 4 "(tree fragment)"
  __pyx_t_3 = (__pyx_t_2 != 0);

#line 4 "(tree fragment)"
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 */

#line 5 "(tree fragment)"
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __Pyx_GIVEREF(__pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);

#line 5 "(tree fragment)"
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)

#line 5 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __Pyx_INCREF(__pyx_t_1);

#line 5 "(tree fragment)"
    __pyx_v___pyx_PickleError = __pyx_t_1;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 5 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */

#line 6 "(tree fragment)"
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 6 "(tree fragment)"
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; 
#line 6 "(tree fragment)"
__pyx_t_6 = NULL;

#line 6 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {

#line 6 "(tree fragment)"
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);

#line 6 "(tree fragment)"
      if (likely(__pyx_t_6)) {

#line 6 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 6 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_6);

#line 6 "(tree fragment)"
        __Pyx_INCREF(function);

#line 6 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_1, function);

#line 6 "(tree fragment)"
      }

#line 6 "(tree fragment)"
    }

#line 6 "(tree fragment)"
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);

#line 6 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 6 "(tree fragment)"
    if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 6, __pyx_L1_error)

#line 6 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 6 "(tree fragment)"
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);

#line 6 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

#line 6 "(tree fragment)"
    __PYX_ERR(3, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb068931, 0x82a3537, 0x6ae9995):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 */

#line 4 "(tree fragment)"
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 */

#line 7 "(tree fragment)"
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_MemviewEnum_type), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 7 "(tree fragment)"
  __pyx_t_5 = NULL;

#line 7 "(tree fragment)"
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {

#line 7 "(tree fragment)"
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);

#line 7 "(tree fragment)"
    if (likely(__pyx_t_5)) {

#line 7 "(tree fragment)"
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);

#line 7 "(tree fragment)"
      __Pyx_INCREF(__pyx_t_5);

#line 7 "(tree fragment)"
      __Pyx_INCREF(function);

#line 7 "(tree fragment)"
      __Pyx_DECREF_SET(__pyx_t_1, function);

#line 7 "(tree fragment)"
    }

#line 7 "(tree fragment)"
  }

#line 7 "(tree fragment)"
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);

#line 7 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

#line 7 "(tree fragment)"
  if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 7, __pyx_L1_error)

#line 7 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_4);

#line 7 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 7 "(tree fragment)"
  __pyx_v___pyx_result = __pyx_t_4;

#line 7 "(tree fragment)"
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);

#line 8 "(tree fragment)"
  __pyx_t_2 = (__pyx_t_3 != 0);

#line 8 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 */

#line 9 "(tree fragment)"
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(3, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __pyx_t_4 = __pyx_unpickle_Enum__set_state(((struct __pyx_MemviewEnum_obj *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 9, __pyx_L1_error)

#line 9 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_4);

#line 9 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb068931, 0x82a3537, 0x6ae9995) = (name))" % __pyx_checksum)
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */

#line 8 "(tree fragment)"
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 */

#line 10 "(tree fragment)"
  __Pyx_XDECREF(__pyx_r);

#line 10 "(tree fragment)"
  __Pyx_INCREF(__pyx_v___pyx_result);

#line 10 "(tree fragment)"
  __pyx_r = __pyx_v___pyx_result;

#line 10 "(tree fragment)"
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"


#line 1 "(tree fragment)"
  /* function exit code */

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_4);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_5);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 1 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 1 "(tree fragment)"
  __pyx_r = NULL;

#line 1 "(tree fragment)"
  __pyx_L0:;

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);

#line 1 "(tree fragment)"
  __Pyx_XDECREF(__pyx_v___pyx_result);

#line 1 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return __pyx_r;

#line 1 "(tree fragment)"
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {

#line 11 "(tree fragment)"
  PyObject *__pyx_r = NULL;

#line 11 "(tree fragment)"
  __Pyx_RefNannyDeclarations

#line 11 "(tree fragment)"
  PyObject *__pyx_t_1 = NULL;

#line 11 "(tree fragment)"
  int __pyx_t_2;

#line 11 "(tree fragment)"
  Py_ssize_t __pyx_t_3;

#line 11 "(tree fragment)"
  int __pyx_t_4;

#line 11 "(tree fragment)"
  int __pyx_t_5;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_6 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_7 = NULL;

#line 11 "(tree fragment)"
  PyObject *__pyx_t_8 = NULL;

#line 11 "(tree fragment)"
  int __pyx_lineno = 0;

#line 11 "(tree fragment)"
  const char *__pyx_filename = NULL;

#line 11 "(tree fragment)"
  int __pyx_clineno = 0;

#line 11 "(tree fragment)"
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 12 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 12 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 12 "(tree fragment)"
    __PYX_ERR(3, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  }

#line 12 "(tree fragment)"
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_t_1);

#line 12 "(tree fragment)"
  __Pyx_GOTREF(__pyx_v___pyx_result->name);

#line 12 "(tree fragment)"
  __Pyx_DECREF(__pyx_v___pyx_result->name);

#line 12 "(tree fragment)"
  __pyx_v___pyx_result->name = __pyx_t_1;

#line 12 "(tree fragment)"
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 13 "(tree fragment)"
  if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 13 "(tree fragment)"
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");

#line 13 "(tree fragment)"
    __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);

#line 13 "(tree fragment)"
  if (__pyx_t_4) {

#line 13 "(tree fragment)"
  } else {

#line 13 "(tree fragment)"
    __pyx_t_2 = __pyx_t_4;

#line 13 "(tree fragment)"
    goto __pyx_L4_bool_binop_done;

#line 13 "(tree fragment)"
  }

#line 13 "(tree fragment)"
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 13, __pyx_L1_error)

#line 13 "(tree fragment)"
  __pyx_t_5 = (__pyx_t_4 != 0);

#line 13 "(tree fragment)"
  __pyx_t_2 = __pyx_t_5;

#line 13 "(tree fragment)"
  __pyx_L4_bool_binop_done:;

#line 13 "(tree fragment)"
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(3, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_7);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(__pyx_v___pyx_state == Py_None)) {

#line 14 "(tree fragment)"
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");

#line 14 "(tree fragment)"
      __PYX_ERR(3, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(3, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_6);

#line 14 "(tree fragment)"
    __pyx_t_8 = NULL;

#line 14 "(tree fragment)"
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {

#line 14 "(tree fragment)"
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);

#line 14 "(tree fragment)"
      if (likely(__pyx_t_8)) {

#line 14 "(tree fragment)"
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);

#line 14 "(tree fragment)"
        __Pyx_INCREF(__pyx_t_8);

#line 14 "(tree fragment)"
        __Pyx_INCREF(function);

#line 14 "(tree fragment)"
        __Pyx_DECREF_SET(__pyx_t_7, function);

#line 14 "(tree fragment)"
      }

#line 14 "(tree fragment)"
    }

#line 14 "(tree fragment)"
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);

#line 14 "(tree fragment)"
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

#line 14 "(tree fragment)"
    if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 14, __pyx_L1_error)

#line 14 "(tree fragment)"
    __Pyx_GOTREF(__pyx_t_1);

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

#line 14 "(tree fragment)"
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */

#line 13 "(tree fragment)"
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /* function exit code */

#line 11 "(tree fragment)"
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);

#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_6);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_7);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_8);

#line 11 "(tree fragment)"
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
  __pyx_r = 0;

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_XGIVEREF(__pyx_r);

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  return __pyx_r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set__Index __pyx_vtable_12ms_deisotope_2_c_8peak_set__Index;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *__pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[100000];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set__Index = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[--__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
      o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set__Index(PyObject *o) {

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index < 100000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set__Index[__pyx_freecount_12ms_deisotope_2_c_8peak_set__Index++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_12neutral_mass_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_mz(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_mz(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_2mz_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set__Index[] = {

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_5clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_4clone},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_7__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_6_Index_6__reduce__},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set__Index[] = {

#line 11 "(tree fragment)"
  {(char *)"neutral_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_neutral_mass, (char *)"neutral_mass: 'size_t'", 0},

#line 11 "(tree fragment)"
  {(char *)"mz", __pyx_getprop_12ms_deisotope_2_c_8peak_set_6_Index_mz, __pyx_setprop_12ms_deisotope_2_c_8peak_set_6_Index_mz, (char *)"mz: 'size_t'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set__Index = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set._Index", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set__Index, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_3__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  "_Index(neutral_mass=0, mz=0)\n\n    Stores the ordered index of an object under sort by `mz` and\n    sort by `neutral_mass`\n\n    Attributes\n    ----------\n    mz : int\n        Index of this object (or its parent), ordered by mz\n    neutral_mass : int\n        Index of this object (or its parent), ordered by neutral mass\n    ", /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set__Index, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set__Index, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_6_Index_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set__Index, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_EnvelopePair __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *__pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[1000000];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[--__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
      o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyObject *o) {

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair < 1000000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set_EnvelopePair[__pyx_freecount_12ms_deisotope_2_c_8peak_set_EnvelopePair++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_EnvelopePair(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_2mz_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9intensity_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_EnvelopePair[] = {

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_8__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_7__reduce__},

#line 11 "(tree fragment)"
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_10_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_12EnvelopePair_9_eq},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_EnvelopePair[] = {

#line 11 "(tree fragment)"
  {(char *)"mz", __pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz, __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_mz, (char *)"mz: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"intensity", __pyx_getprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity, __pyx_setprop_12ms_deisotope_2_c_8peak_set_12EnvelopePair_intensity, (char *)"intensity: 'double'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_EnvelopePair = {

#line 11 "(tree fragment)"
  0, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_EnvelopePair = {

#line 11 "(tree fragment)"
  0, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_3__getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  0, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.EnvelopePair", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_14__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_EnvelopePair, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_EnvelopePair, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  "EnvelopePair(mz, intensity)", /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_12__richcmp__, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_5__iter__, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_12EnvelopePair_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_EnvelopePair, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_Envelope __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *__pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[100000];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[--__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
      o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    } else {

#line 11 "(tree fragment)"
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope;

#line 11 "(tree fragment)"
  p->pairs = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->pairs);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope < 100000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set_Envelope[__pyx_freecount_12ms_deisotope_2_c_8peak_set_Envelope++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;

#line 11 "(tree fragment)"
  if (p->pairs) {

#line 11 "(tree fragment)"
    e = (*v)(p->pairs, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->pairs);

#line 11 "(tree fragment)"
  p->pairs = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_Envelope(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5pairs_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_Envelope[] = {

#line 11 "(tree fragment)"
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_11_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_10_eq},

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_15clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_14clone},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_17__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_16__reduce__},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_Envelope[] = {

#line 11 "(tree fragment)"
  {(char *)"pairs", __pyx_getprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs, __pyx_setprop_12ms_deisotope_2_c_8peak_set_8Envelope_pairs, (char *)"pairs: tuple", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_Envelope = {

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_Envelope, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_Envelope = {

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_9__len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_3__getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  0, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.Envelope", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_7__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_Envelope, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_Envelope, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "Envelope(pairs)\n\n    Represents a sequence of (mz, intensity) pairs which store peak positions\n    that were matched. Since these peaks may later be mutated by subtraction,\n    this structure stores copies of the numbers\n\n    Supports the Sequence protocol, and is read-only.\n\n    Attributes\n    ----------\n    pairs : tuple of tuple of (float, float)\n        list of (mz, intensity) pairs\n    ", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_clear*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_13__richcmp__, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_5__iter__, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_8Envelope_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_Envelope, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_new(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;

#line 11 "(tree fragment)"
  p->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_index);

#line 11 "(tree fragment)"
  Py_CLEAR(p->envelope);

#line 11 "(tree fragment)"
  Py_CLEAR(p->fit);

#line 11 "(tree fragment)"
  #if CYTHON_USE_TYPE_SLOTS

#line 11 "(tree fragment)"
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  if (likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;

#line 11 "(tree fragment)"
  e = ((likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) ? ((__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_traverse) ? __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak)); if (e) return e;

#line 11 "(tree fragment)"
  if (p->_index) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->_index), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->envelope) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->envelope), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->fit) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->fit), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *)o;

#line 11 "(tree fragment)"
  if (likely(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase)) { if (__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_clear) __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_index);

#line 11 "(tree fragment)"
  p->_index = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->envelope);

#line 11 "(tree fragment)"
  p->envelope = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->fit);

#line 11 "(tree fragment)"
  p->fit = ((struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12neutral_mass_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15signal_to_noise_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6_index_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_22full_width_at_half_max_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6charge_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13a_to_a2_ratio_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_18most_abundant_mass_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_12average_mass_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5score_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8envelope_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3fit_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_15chosen_for_msms_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak[] = {

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_5clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_4clone},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_7__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_6__reduce__},

#line 11 "(tree fragment)"
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_9_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_8_eq},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak[] = {

#line 11 "(tree fragment)"
  {(char *)"index", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_index, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"neutral_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_neutral_mass, (char *)"neutral_mass: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"signal_to_noise", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_signal_to_noise, (char *)"signal_to_noise: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"_index", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__index, (char *)"_index: ms_deisotope._c.peak_set._Index", 0},

#line 11 "(tree fragment)"
  {(char *)"full_width_at_half_max", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_full_width_at_half_max, (char *)"full_width_at_half_max: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"charge", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_charge, (char *)"charge: 'int'", 0},

#line 11 "(tree fragment)"
  {(char *)"a_to_a2_ratio", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_a_to_a2_ratio, (char *)"a_to_a2_ratio: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"most_abundant_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_most_abundant_mass, (char *)"most_abundant_mass: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"average_mass", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_average_mass, (char *)"average_mass: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"score", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_score, (char *)"score: 'double'", 0},

#line 11 "(tree fragment)"
  {(char *)"envelope", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_envelope, (char *)"envelope: ms_deisotope._c.peak_set.Envelope", 0},

#line 11 "(tree fragment)"
  {(char *)"fit", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_fit, (char *)"fit: ms_deisotope._c.scoring.IsotopicFitRecord", 0},

#line 11 "(tree fragment)"
  {(char *)"chosen_for_msms", __pyx_getprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms, __pyx_setprop_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_chosen_for_msms, (char *)"chosen_for_msms: 'bool'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.DeconvolutedPeak", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_3__hash__, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "DeconvolutedPeak(neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max, a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0, envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0)\n\n    Represent a single deconvoluted peak which represents an aggregated isotopic\n    pattern collapsed to its monoisotopic peak, with a known charge state\n\n    Attributes\n    ----------\n    a_to_a2_ratio : float\n        Ratio of intensities of A peak to A+2 peak\n    average_mass : float\n        The averaged neutral mass of the composition, the weighted average\n        of the envelope peaks.\n    charge : int\n        The signed charge state of the isotopic pattern\n    envelope : Envelope\n        The sequence of (mz, intensity) pairs which map to this peak\n    full_width_at_half_max : float\n        The averaged full width at half max of this isotopic pattern\n    index : _Index\n        The position of this peak in different orderings\n    intensity : float\n        The summed height of the peaks in the isotopic pattern this peak captures\n    most_abundant_mass : float\n        The neutral mass of the most abundant peak in the isotopic pattern this peak captures\n    mz : float\n        The mass-charge-ratio of the monoisotopic peak\n    neutral_mass : float\n        The neutral mass of the monoisotopic peak\n    score : float\n        An assigned value describing the quality of this peak's fit. The semantics of this score\n        depends upon the scoring function\n    signal_to_noise : float\n        The average signal-to-noise ratio of the peaks in the isotopic pattern this peak captures\n    ", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_clear*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_11__richcmp__, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_10drift_time_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak[] = {

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_3clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_2clone},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_4__reduce__},

#line 11 "(tree fragment)"
  {"_eq", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_7_eq, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_6_eq},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak[] = {

#line 11 "(tree fragment)"
  {(char *)"drift_time", __pyx_getprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time, __pyx_setprop_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_drift_time, (char *)"drift_time: 'double'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.IonMobilityDeconvolutedPeak", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_9__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "IonMobilityDeconvolutedPeak(neutral_mass, intensity, charge, signal_to_noise, index, full_width_at_half_max, a_to_a2_ratio=0, most_abundant_mass=0, average_mass=0, score=0, envelope=(), mz=0, fit=None, chosen_for_msms=False, area=0, drift_time=0)", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;

#line 11 "(tree fragment)"
  p->solution = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->solution);

#line 11 "(tree fragment)"
  PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;

#line 11 "(tree fragment)"
  e = __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o, v, a); if (e) return e;

#line 11 "(tree fragment)"
  if (p->solution) {

#line 11 "(tree fragment)"
    e = (*v)(p->solution, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution *)o;

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak(o);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->solution);

#line 11 "(tree fragment)"
  p->solution = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_8solution_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution[] = {

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_3clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_2clone},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_4__reduce__},

#line 11 "(tree fragment)"
  {"from_peak", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_10from_peak, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_9from_peak},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution[] = {

#line 11 "(tree fragment)"
  {(char *)"solution", __pyx_getprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution, __pyx_setprop_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_solution, (char *)"solution: object", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.DeconvolutedPeakSolution", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_13__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "DeconvolutedPeakSolution(solution, fit, *args, **kwargs)\n\n    Extends :class:`DeconvolutedPeak` to also include a reference to\n    the :class:`IsotopicFitRecord` instance it is derived from, and optionally an\n    object which the fit derives from.\n\n    Attributes\n    ----------\n    fit : IsotpicFitRecord\n        The isotopic pattern fit used to construct this peak\n    solution : object\n        Representation of the \"source\" of the isotopic fit\n    ", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_7__iter__, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;

#line 11 "(tree fragment)"
  p->peaks = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_mz_ordered = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->peaks);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_mz_ordered);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;

#line 11 "(tree fragment)"
  if (p->peaks) {

#line 11 "(tree fragment)"
    e = (*v)(p->peaks, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->_mz_ordered) {

#line 11 "(tree fragment)"
    e = (*v)(p->_mz_ordered, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->peaks);

#line 11 "(tree fragment)"
  p->peaks = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_mz_ordered);

#line 11 "(tree fragment)"
  p->_mz_ordered = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_richcompare_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(PyObject *o1, PyObject *o2, int op) {

#line 11 "(tree fragment)"
  switch (op) {

#line 11 "(tree fragment)"
    case Py_EQ: {

#line 11 "(tree fragment)"
      return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_13__eq__(o1, o2);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    case Py_NE: {

#line 11 "(tree fragment)"
      return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_15__ne__(o1, o2);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    default: {

#line 11 "(tree fragment)"
      return __Pyx_NewRef(Py_NotImplemented);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_5peaks_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11_mz_ordered_5__del__(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7indexed_3__set__(o, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_NotImplementedError, "__del__");

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet[] = {

#line 11 "(tree fragment)"
  {"reindex", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_3reindex, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_2reindex},

#line 11 "(tree fragment)"
  {"clone", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_19clone, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_18clone},

#line 11 "(tree fragment)"
  {"copy", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_21copy, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_20copy},

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_23__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_22__reduce__},

#line 11 "(tree fragment)"
  {"has_peak", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_25has_peak, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_24has_peak},

#line 11 "(tree fragment)"
  {"all_peaks_for", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_27all_peaks_for, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_26all_peaks_for},

#line 11 "(tree fragment)"
  {"get_nearest_peak", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_29get_nearest_peak, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_28get_nearest_peak},

#line 11 "(tree fragment)"
  {"between", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_31between, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_30between},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet[] = {

#line 11 "(tree fragment)"
  {(char *)"peaks", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_peaks, (char *)"peaks: tuple", 0},

#line 11 "(tree fragment)"
  {(char *)"_mz_ordered", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__mz_ordered, (char *)"_mz_ordered: tuple", 0},

#line 11 "(tree fragment)"
  {(char *)"indexed", __pyx_getprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed, __pyx_setprop_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_indexed, (char *)"indexed: 'bool'", 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_DeconvolutedPeakSet = {

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_DeconvolutedPeakSet = {

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_7__len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_11__getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  0, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.DeconvolutedPeakSet", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_DeconvolutedPeakSet, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_DeconvolutedPeakSet, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "DeconvolutedPeakSet(peaks)\n\n    Represents a collection of :class:`DeconvolutedPeak` instances under multiple orderings.\n\n    Attributes\n    ----------\n    peaks : tuple of DeconvolutedPeak\n        Collection of peaks ordered by `neutral_mass`\n    _mz_ordered: tuple of DeconvolutedPeak\n        Collection of peaks ordered by `mz`\n    ", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_clear*/

#line 11 "(tree fragment)"
  __pyx_tp_richcompare_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet*)__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed(PyObject *o) {

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_3__dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed[] = {

#line 11 "(tree fragment)"
  {"__reduce__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_5__reduce__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_4__reduce__},

#line 11 "(tree fragment)"
  {"__getstate__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_7__getstate__, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_6__getstate__},

#line 11 "(tree fragment)"
  {"__setstate__", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_9__setstate__, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_8__setstate__},

#line 11 "(tree fragment)"
  {"_release_buffers", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_11_release_buffers, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_10_release_buffers},

#line 11 "(tree fragment)"
  {"set_interval_index_size", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_13set_interval_index_size, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_12set_interval_index_size},

#line 11 "(tree fragment)"
  {"reindex", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_15reindex, METH_NOARGS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_14reindex},

#line 11 "(tree fragment)"
  {"_test_interval", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_17_test_interval, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_16_test_interval},

#line 11 "(tree fragment)"
  {"all_peaks_for", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_19all_peaks_for, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_18all_peaks_for},

#line 11 "(tree fragment)"
  {"test_interval", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_21test_interval, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_20test_interval},

#line 11 "(tree fragment)"
  {"find_interval_for", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_23find_interval_for, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_22find_interval_for},

#line 11 "(tree fragment)"
  {"check_interval", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_25check_interval, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_24check_interval},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.DeconvolutedPeakSetIndexed", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_9__repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "DeconvolutedPeakSetIndexed(peaks)", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_17__iter__, /*tp_iter*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_pw_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_1__init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_self);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_v_self) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct____iter__", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_self);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_v_self) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->__pyx_v_self);

#line 11 "(tree fragment)"
  p->__pyx_v_self = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct_1___repr__", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_outer_scope);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_t);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_t_0);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_outer_scope) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_v_t) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_v_t, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_t_0) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_t_0, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct_2_genexpr", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_self);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_v_self) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct_3___iter__", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_item);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_self);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_v_item) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_v_item, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_v_self) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->__pyx_v_item);

#line 11 "(tree fragment)"
  p->__pyx_v_item = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->__pyx_v_self);

#line 11 "(tree fragment)"
  p->__pyx_v_self = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct_4___getitem__", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[8];

#line 11 "(tree fragment)"
static int __pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = 0;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr)))) {

#line 11 "(tree fragment)"
    o = (PyObject*)__pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[--__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr];

#line 11 "(tree fragment)"
    memset(o, 0, sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr));

#line 11 "(tree fragment)"
    (void) PyObject_INIT(o, t);

#line 11 "(tree fragment)"
    PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
    if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o;

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_outer_scope);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_v_p);

#line 11 "(tree fragment)"
  Py_CLEAR(p->__pyx_t_0);

#line 11 "(tree fragment)"
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr)))) {

#line 11 "(tree fragment)"
    __pyx_freelist_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr[__pyx_freecount_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr++] = ((struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *p = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr *)o;

#line 11 "(tree fragment)"
  if (p->__pyx_outer_scope) {

#line 11 "(tree fragment)"
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_v_p) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_v_p, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->__pyx_t_0) {

#line 11 "(tree fragment)"
    e = (*v)(p->__pyx_t_0, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.__pyx_scope_struct_5_genexpr", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_obj_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  0, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_array __pyx_vtable_array;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_array_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_array_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_array;

#line 11 "(tree fragment)"
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) goto bad;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
  bad:

#line 11 "(tree fragment)"
  Py_DECREF(o); o = 0;

#line 11 "(tree fragment)"
  return NULL;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_array(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_array___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->mode);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_format);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_array___setitem__(o, i, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_Format(PyExc_NotImplementedError,

#line 11 "(tree fragment)"
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {

#line 11 "(tree fragment)"
  PyObject *v = __Pyx_PyObject_GenericGetAttr(o, n);

#line 11 "(tree fragment)"
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {

#line 11 "(tree fragment)"
    PyErr_Clear();

#line 11 "(tree fragment)"
    v = __pyx_array___getattr__(o, n);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return v;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_array[] = {

#line 11 "(tree fragment)"
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_array_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_array_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_array[] = {

#line 11 "(tree fragment)"
  {(char *)"memview", __pyx_getprop___pyx_array_memview, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_array = {

#line 11 "(tree fragment)"
  __pyx_array___len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_array, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_array = {

#line 11 "(tree fragment)"
  __pyx_array___len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_array___getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyBufferProcs __pyx_tp_as_buffer_array = {

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getreadbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getwritebuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getsegcount*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getcharbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_array_getbuffer, /*bf_getbuffer*/

#line 11 "(tree fragment)"
  0, /*bf_releasebuffer*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_array = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.array", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_array, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  __pyx_tp_getattro_array, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  0, /*tp_traverse*/

#line 11 "(tree fragment)"
  0, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_array, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_array, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_array, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_MemviewEnum_obj *)o);

#line 11 "(tree fragment)"
  p->name = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_Enum(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  Py_CLEAR(p->name);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  if (p->name) {

#line 11 "(tree fragment)"
    e = (*v)(p->name, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_Enum(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->name);

#line 11 "(tree fragment)"
  p->name = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_Enum[] = {

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_MemviewEnum_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_MemviewEnum = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.Enum", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_MemviewEnum___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_Enum, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_Enum, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_Enum, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  0, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  __pyx_MemviewEnum___init__, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_Enum, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p;

#line 11 "(tree fragment)"
  PyObject *o;

#line 11 "(tree fragment)"
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {

#line 11 "(tree fragment)"
    o = (*t->tp_alloc)(t, 0);

#line 11 "(tree fragment)"
  } else {

#line 11 "(tree fragment)"
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_memoryview_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_vtab = __pyx_vtabptr_memoryview;

#line 11 "(tree fragment)"
  p->obj = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_size = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->_array_interface = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->view.obj = NULL;

#line 11 "(tree fragment)"
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) goto bad;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
  bad:

#line 11 "(tree fragment)"
  Py_DECREF(o); o = 0;

#line 11 "(tree fragment)"
  return NULL;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc_memoryview(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_memoryview___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->obj);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_size);

#line 11 "(tree fragment)"
  Py_CLEAR(p->_array_interface);

#line 11 "(tree fragment)"
  (*Py_TYPE(o)->tp_free)(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  if (p->obj) {

#line 11 "(tree fragment)"
    e = (*v)(p->obj, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->_size) {

#line 11 "(tree fragment)"
    e = (*v)(p->_size, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->_array_interface) {

#line 11 "(tree fragment)"
    e = (*v)(p->_array_interface, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  if (p->view.obj) {

#line 11 "(tree fragment)"
    e = (*v)(p->view.obj, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear_memoryview(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->obj);

#line 11 "(tree fragment)"
  p->obj = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_size);

#line 11 "(tree fragment)"
  p->_size = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->_array_interface);

#line 11 "(tree fragment)"
  p->_array_interface = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  Py_CLEAR(p->view.obj);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"
static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {

#line 11 "(tree fragment)"
  PyObject *r;

#line 11 "(tree fragment)"
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;

#line 11 "(tree fragment)"
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);

#line 11 "(tree fragment)"
  Py_DECREF(x);

#line 11 "(tree fragment)"
  return r;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {

#line 11 "(tree fragment)"
  if (v) {

#line 11 "(tree fragment)"
    return __pyx_memoryview___setitem__(o, i, v);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  else {

#line 11 "(tree fragment)"
    PyErr_Format(PyExc_NotImplementedError,

#line 11 "(tree fragment)"
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);

#line 11 "(tree fragment)"
    return -1;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods_memoryview[] = {

#line 11 "(tree fragment)"
  {"is_c_contig", (PyCFunction)__pyx_memoryview_is_c_contig, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"is_f_contig", (PyCFunction)__pyx_memoryview_is_f_contig, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"copy", (PyCFunction)__pyx_memoryview_copy, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"copy_fortran", (PyCFunction)__pyx_memoryview_copy_fortran, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryview_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets_memoryview[] = {

#line 11 "(tree fragment)"
  {(char *)"T", __pyx_getprop___pyx_memoryview_T, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"base", __pyx_getprop___pyx_memoryview_base, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"shape", __pyx_getprop___pyx_memoryview_shape, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"strides", __pyx_getprop___pyx_memoryview_strides, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {(char *)"size", __pyx_getprop___pyx_memoryview_size, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PySequenceMethods __pyx_tp_as_sequence_memoryview = {

#line 11 "(tree fragment)"
  __pyx_memoryview___len__, /*sq_length*/

#line 11 "(tree fragment)"
  0, /*sq_concat*/

#line 11 "(tree fragment)"
  0, /*sq_repeat*/

#line 11 "(tree fragment)"
  __pyx_sq_item_memoryview, /*sq_item*/

#line 11 "(tree fragment)"
  0, /*sq_slice*/

#line 11 "(tree fragment)"
  0, /*sq_ass_item*/

#line 11 "(tree fragment)"
  0, /*sq_ass_slice*/

#line 11 "(tree fragment)"
  0, /*sq_contains*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_concat*/

#line 11 "(tree fragment)"
  0, /*sq_inplace_repeat*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMappingMethods __pyx_tp_as_mapping_memoryview = {

#line 11 "(tree fragment)"
  __pyx_memoryview___len__, /*mp_length*/

#line 11 "(tree fragment)"
  __pyx_memoryview___getitem__, /*mp_subscript*/

#line 11 "(tree fragment)"
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyBufferProcs __pyx_tp_as_buffer_memoryview = {

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getreadbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getwritebuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getsegcount*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*bf_getcharbuffer*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/

#line 11 "(tree fragment)"
  0, /*bf_releasebuffer*/

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_memoryview = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set.memoryview", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  __pyx_memoryview___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  __pyx_memoryview___str__, /*tp_str*/

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  0, /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse_memoryview, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear_memoryview, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods_memoryview, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets_memoryview, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new_memoryview, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p;

#line 11 "(tree fragment)"
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);

#line 11 "(tree fragment)"
  if (unlikely(!o)) return 0;

#line 11 "(tree fragment)"
  p = ((struct __pyx_memoryviewslice_obj *)o);

#line 11 "(tree fragment)"
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;

#line 11 "(tree fragment)"
  p->from_object = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  p->from_slice.memview = NULL;

#line 11 "(tree fragment)"
  return o;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  #if CYTHON_USE_TP_FINALIZE

#line 11 "(tree fragment)"
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {

#line 11 "(tree fragment)"
    if (PyObject_CallFinalizerFromDealloc(o)) return;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  PyObject_GC_UnTrack(o);

#line 11 "(tree fragment)"
  {

#line 11 "(tree fragment)"
    PyObject *etype, *eval, *etb;

#line 11 "(tree fragment)"
    PyErr_Fetch(&etype, &eval, &etb);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);

#line 11 "(tree fragment)"
    __pyx_memoryviewslice___dealloc__(o);

#line 11 "(tree fragment)"
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);

#line 11 "(tree fragment)"
    PyErr_Restore(etype, eval, etb);

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  Py_CLEAR(p->from_object);

#line 11 "(tree fragment)"
  PyObject_GC_Track(o);

#line 11 "(tree fragment)"
  __pyx_tp_dealloc_memoryview(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {

#line 11 "(tree fragment)"
  int e;

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;

#line 11 "(tree fragment)"
  if (p->from_object) {

#line 11 "(tree fragment)"
    e = (*v)(p->from_object, a); if (e) return e;

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static int __pyx_tp_clear__memoryviewslice(PyObject *o) {

#line 11 "(tree fragment)"
  PyObject* tmp;

#line 11 "(tree fragment)"
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;

#line 11 "(tree fragment)"
  __pyx_tp_clear_memoryview(o);

#line 11 "(tree fragment)"
  tmp = ((PyObject*)p->from_object);

#line 11 "(tree fragment)"
  p->from_object = Py_None; Py_INCREF(Py_None);

#line 11 "(tree fragment)"
  Py_XDECREF(tmp);

#line 11 "(tree fragment)"
  __PYX_XDEC_MEMVIEW(&p->from_slice, 1);

#line 11 "(tree fragment)"
  return 0;

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyObject *__pyx_getprop___pyx_memoryviewslice_base(PyObject *o, CYTHON_UNUSED void *x) {

#line 11 "(tree fragment)"
  return __pyx_pw_15View_dot_MemoryView_16_memoryviewslice_4base_1__get__(o);

#line 11 "(tree fragment)"
}

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods__memoryviewslice[] = {

#line 11 "(tree fragment)"
  {"__reduce_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_1__reduce_cython__, METH_NOARGS, 0},

#line 11 "(tree fragment)"
  {"__setstate_cython__", (PyCFunction)__pyx_pw___pyx_memoryviewslice_3__setstate_cython__, METH_O, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyGetSetDef __pyx_getsets__memoryviewslice[] = {

#line 11 "(tree fragment)"
  {(char *)"base", __pyx_getprop___pyx_memoryviewslice_base, 0, (char *)0, 0},

#line 11 "(tree fragment)"
  {0, 0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyTypeObject __pyx_type___pyx_memoryviewslice = {

#line 11 "(tree fragment)"
  PyVarObject_HEAD_INIT(0, 0)

#line 11 "(tree fragment)"
  "ms_deisotope._c.peak_set._memoryviewslice", /*tp_name*/

#line 11 "(tree fragment)"
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/

#line 11 "(tree fragment)"
  0, /*tp_itemsize*/

#line 11 "(tree fragment)"
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX < 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4

#line 11 "(tree fragment)"
  0, /*tp_vectorcall_offset*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattr*/

#line 11 "(tree fragment)"
  0, /*tp_setattr*/

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION < 3

#line 11 "(tree fragment)"
  0, /*tp_compare*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  0, /*tp_as_async*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_memoryview___repr__, /*tp_repr*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_repr*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_as_number*/

#line 11 "(tree fragment)"
  0, /*tp_as_sequence*/

#line 11 "(tree fragment)"
  0, /*tp_as_mapping*/

#line 11 "(tree fragment)"
  0, /*tp_hash*/

#line 11 "(tree fragment)"
  0, /*tp_call*/

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY

#line 11 "(tree fragment)"
  __pyx_memoryview___str__, /*tp_str*/

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  0, /*tp_str*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  0, /*tp_getattro*/

#line 11 "(tree fragment)"
  0, /*tp_setattro*/

#line 11 "(tree fragment)"
  0, /*tp_as_buffer*/

#line 11 "(tree fragment)"
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/

#line 11 "(tree fragment)"
  "Internal class for passing memoryview slices to Python", /*tp_doc*/

#line 11 "(tree fragment)"
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/

#line 11 "(tree fragment)"
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/

#line 11 "(tree fragment)"
  0, /*tp_richcompare*/

#line 11 "(tree fragment)"
  0, /*tp_weaklistoffset*/

#line 11 "(tree fragment)"
  0, /*tp_iter*/

#line 11 "(tree fragment)"
  0, /*tp_iternext*/

#line 11 "(tree fragment)"
  __pyx_methods__memoryviewslice, /*tp_methods*/

#line 11 "(tree fragment)"
  0, /*tp_members*/

#line 11 "(tree fragment)"
  __pyx_getsets__memoryviewslice, /*tp_getset*/

#line 11 "(tree fragment)"
  0, /*tp_base*/

#line 11 "(tree fragment)"
  0, /*tp_dict*/

#line 11 "(tree fragment)"
  0, /*tp_descr_get*/

#line 11 "(tree fragment)"
  0, /*tp_descr_set*/

#line 11 "(tree fragment)"
  0, /*tp_dictoffset*/

#line 11 "(tree fragment)"
  0, /*tp_init*/

#line 11 "(tree fragment)"
  0, /*tp_alloc*/

#line 11 "(tree fragment)"
  __pyx_tp_new__memoryviewslice, /*tp_new*/

#line 11 "(tree fragment)"
  0, /*tp_free*/

#line 11 "(tree fragment)"
  0, /*tp_is_gc*/

#line 11 "(tree fragment)"
  0, /*tp_bases*/

#line 11 "(tree fragment)"
  0, /*tp_mro*/

#line 11 "(tree fragment)"
  0, /*tp_cache*/

#line 11 "(tree fragment)"
  0, /*tp_subclasses*/

#line 11 "(tree fragment)"
  0, /*tp_weaklist*/

#line 11 "(tree fragment)"
  0, /*tp_del*/

#line 11 "(tree fragment)"
  0, /*tp_version_tag*/

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030400a1

#line 11 "(tree fragment)"
  0, /*tp_finalize*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)

#line 11 "(tree fragment)"
  0, /*tp_vectorcall*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_print*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000

#line 11 "(tree fragment)"
  0, /*tp_pypy_flags*/

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static PyMethodDef __pyx_methods[] = {

#line 11 "(tree fragment)"
  {"_Index_reconstructor", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_12ms_deisotope_2_c_8peak_set_1_Index_reconstructor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_12ms_deisotope_2_c_8peak_set__Index_reconstructor},

#line 11 "(tree fragment)"
  {"_Index_to_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_3_Index_to_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_2_Index_to_bytes},

#line 11 "(tree fragment)"
  {"_Index_from_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_5_Index_from_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_4_Index_from_bytes},

#line 11 "(tree fragment)"
  {"Envelope_to_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_7Envelope_to_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_6Envelope_to_bytes},

#line 11 "(tree fragment)"
  {"Envelope_from_bytes", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_9Envelope_from_bytes, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_8Envelope_from_bytes},

#line 11 "(tree fragment)"
  {"Envelope_reconstructor", (PyCFunction)__pyx_pw_12ms_deisotope_2_c_8peak_set_11Envelope_reconstructor, METH_O, __pyx_doc_12ms_deisotope_2_c_8peak_set_10Envelope_reconstructor},

#line 11 "(tree fragment)"
  {0, 0, 0, 0}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
#if PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
#if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/

#line 11 "(tree fragment)"
static int __pyx_pymod_exec_peak_set(PyObject* module); /*proto*/

#line 11 "(tree fragment)"
static PyModuleDef_Slot __pyx_moduledef_slots[] = {

#line 11 "(tree fragment)"
  {Py_mod_create, (void*)__pyx_pymod_create},

#line 11 "(tree fragment)"
  {Py_mod_exec, (void*)__pyx_pymod_exec_peak_set},

#line 11 "(tree fragment)"
  {0, NULL}

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
#endif

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
static struct PyModuleDef __pyx_moduledef = {

#line 11 "(tree fragment)"
    PyModuleDef_HEAD_INIT,

#line 11 "(tree fragment)"
    "peak_set",

#line 11 "(tree fragment)"
    0, /* m_doc */

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
    0, /* m_size */

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
    -1, /* m_size */

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
    __pyx_methods /* m_methods */,

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
    __pyx_moduledef_slots, /* m_slots */

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
    NULL, /* m_reload */

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
    NULL, /* m_traverse */

#line 11 "(tree fragment)"
    NULL, /* m_clear */

#line 11 "(tree fragment)"
    NULL /* m_free */

#line 11 "(tree fragment)"
};

#line 11 "(tree fragment)"
#endif

#line 11 "(tree fragment)"
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_0_4f_0_2f, __pyx_k_0_4f_0_2f, sizeof(__pyx_k_0_4f_0_2f), 0, 0, 1, 0},
  {&__pyx_n_s_ASCII, __pyx_k_ASCII, sizeof(__pyx_k_ASCII), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Buffer_view_does_not_expose_stri, __pyx_k_Buffer_view_does_not_expose_stri, sizeof(__pyx_k_Buffer_view_does_not_expose_stri), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_only_create_a_buffer_that_is, __pyx_k_Can_only_create_a_buffer_that_is, sizeof(__pyx_k_Can_only_create_a_buffer_that_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_assign_to_read_only_memor, __pyx_k_Cannot_assign_to_read_only_memor, sizeof(__pyx_k_Cannot_assign_to_read_only_memor), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_create_writable_memory_vi, __pyx_k_Cannot_create_writable_memory_vi, sizeof(__pyx_k_Cannot_create_writable_memory_vi), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_index_with_type_s, __pyx_k_Cannot_index_with_type_s, sizeof(__pyx_k_Cannot_index_with_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_DeconvolutedPeak, __pyx_k_DeconvolutedPeak, sizeof(__pyx_k_DeconvolutedPeak), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSet, __pyx_k_DeconvolutedPeakSet, sizeof(__pyx_k_DeconvolutedPeakSet), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSetIndexed, __pyx_k_DeconvolutedPeakSetIndexed, sizeof(__pyx_k_DeconvolutedPeakSetIndexed), 0, 0, 1, 1},
  {&__pyx_kp_s_DeconvolutedPeakSet_d_Peaks, __pyx_k_DeconvolutedPeakSet_d_Peaks, sizeof(__pyx_k_DeconvolutedPeakSet_d_Peaks), 0, 0, 1, 0},
  {&__pyx_n_s_DeconvolutedPeakSolution, __pyx_k_DeconvolutedPeakSolution, sizeof(__pyx_k_DeconvolutedPeakSolution), 0, 0, 1, 1},
  {&__pyx_n_s_DeconvolutedPeakSolution___iter, __pyx_k_DeconvolutedPeakSolution___iter, sizeof(__pyx_k_DeconvolutedPeakSolution___iter), 0, 0, 1, 1},
  {&__pyx_n_s_Ellipsis, __pyx_k_Ellipsis, sizeof(__pyx_k_Ellipsis), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_shape_tuple_for_cython_arr, __pyx_k_Empty_shape_tuple_for_cython_arr, sizeof(__pyx_k_Empty_shape_tuple_for_cython_arr), 0, 0, 1, 0},
  {&__pyx_n_s_Envelope, __pyx_k_Envelope, sizeof(__pyx_k_Envelope), 0, 0, 1, 1},
  {&__pyx_n_s_EnvelopePair, __pyx_k_EnvelopePair, sizeof(__pyx_k_EnvelopePair), 0, 0, 1, 1},
  {&__pyx_n_s_EnvelopePair___iter, __pyx_k_EnvelopePair___iter, sizeof(__pyx_k_EnvelopePair___iter), 0, 0, 1, 1},
  {&__pyx_n_s_Envelope_reconstructor, __pyx_k_Envelope_reconstructor, sizeof(__pyx_k_Envelope_reconstructor), 0, 0, 1, 1},
  {&__pyx_kp_s_Expected_at_least_d_argument_s_g, __pyx_k_Expected_at_least_d_argument_s_g, sizeof(__pyx_k_Expected_at_least_d_argument_s_g), 0, 0, 1, 0},
  {&__pyx_kp_s_Function_call_with_ambiguous_arg, __pyx_k_Function_call_with_ambiguous_arg, sizeof(__pyx_k_Function_call_with_ambiguous_arg), 0, 0, 1, 0},
  {&__pyx_n_s_INTERVAL_INDEX_SIZE, __pyx_k_INTERVAL_INDEX_SIZE, sizeof(__pyx_k_INTERVAL_INDEX_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_n_s_Index, __pyx_k_Index, sizeof(__pyx_k_Index), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_Index_reconstructor, __pyx_k_Index_reconstructor, sizeof(__pyx_k_Index_reconstructor), 0, 0, 1, 1},
  {&__pyx_kp_s_Indirect_dimensions_not_supporte, __pyx_k_Indirect_dimensions_not_supporte, sizeof(__pyx_k_Indirect_dimensions_not_supporte), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_k_Invalid_mode_expected_c_or_fortr, sizeof(__pyx_k_Invalid_mode_expected_c_or_fortr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_k_Invalid_shape_in_axis_d_d, sizeof(__pyx_k_Invalid_shape_in_axis_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_IonMobilityDeconvolutedPeak, __pyx_k_IonMobilityDeconvolutedPeak, sizeof(__pyx_k_IonMobilityDeconvolutedPeak), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_k_MemoryView_of_r_at_0x_x, sizeof(__pyx_k_MemoryView_of_r_at_0x_x), 0, 0, 1, 0},
  {&__pyx_kp_s_MemoryView_of_r_object, __pyx_k_MemoryView_of_r_object, sizeof(__pyx_k_MemoryView_of_r_object), 0, 0, 1, 0},
  {&__pyx_kp_s_No_matching_signature_found, __pyx_k_No_matching_signature_found, sizeof(__pyx_k_No_matching_signature_found), 0, 0, 1, 0},
  {&__pyx_n_b_O, __pyx_k_O, sizeof(__pyx_k_O), 0, 0, 0, 1},
  {&__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_k_Out_of_bounds_on_buffer_access_a, sizeof(__pyx_k_Out_of_bounds_on_buffer_access_a), 0, 0, 1, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_convert_item_to_object, __pyx_k_Unable_to_convert_item_to_object, sizeof(__pyx_k_Unable_to_convert_item_to_object), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_View_MemoryView, __pyx_k_View_MemoryView, sizeof(__pyx_k_View_MemoryView), 0, 0, 1, 1},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_kp_s__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 0, 1, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_a_to_a2_ratio, __pyx_k_a_to_a2_ratio, sizeof(__pyx_k_a_to_a2_ratio), 0, 0, 1, 1},
  {&__pyx_n_s_all_peaks_for, __pyx_k_all_peaks_for, sizeof(__pyx_k_all_peaks_for), 0, 0, 1, 1},
  {&__pyx_n_s_allocate_buffer, __pyx_k_allocate_buffer, sizeof(__pyx_k_allocate_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_area, __pyx_k_area, sizeof(__pyx_k_area), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_attrgetter, __pyx_k_attrgetter, sizeof(__pyx_k_attrgetter), 0, 0, 1, 1},
  {&__pyx_n_s_average_mass, __pyx_k_average_mass, sizeof(__pyx_k_average_mass), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_n_s_center_value, __pyx_k_center_value, sizeof(__pyx_k_center_value), 0, 0, 1, 1},
  {&__pyx_n_s_charge, __pyx_k_charge, sizeof(__pyx_k_charge), 0, 0, 1, 1},
  {&__pyx_n_s_chosen_for_msms, __pyx_k_chosen_for_msms, sizeof(__pyx_k_chosen_for_msms), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clone, __pyx_k_clone, sizeof(__pyx_k_clone), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_kp_s_contiguous_and_direct, __pyx_k_contiguous_and_direct, sizeof(__pyx_k_contiguous_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_contiguous_and_indirect, __pyx_k_contiguous_and_indirect, sizeof(__pyx_k_contiguous_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_convert, __pyx_k_convert, sizeof(__pyx_k_convert), 0, 0, 1, 1},
  {&__pyx_n_s_converted, __pyx_k_converted, sizeof(__pyx_k_converted), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_current_envelope, __pyx_k_current_envelope, sizeof(__pyx_k_current_envelope), 0, 0, 1, 1},
  {&__pyx_kp_s_d_d, __pyx_k_d_d, sizeof(__pyx_k_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_decode_envelopes, __pyx_k_decode_envelopes, sizeof(__pyx_k_decode_envelopes), 0, 0, 1, 1},
  {&__pyx_n_s_defaults, __pyx_k_defaults, sizeof(__pyx_k_defaults), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_n_s_drift_time, __pyx_k_drift_time, sizeof(__pyx_k_drift_time), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dtype_is_object, __pyx_k_dtype_is_object, sizeof(__pyx_k_dtype_is_object), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_envelope, __pyx_k_envelope, sizeof(__pyx_k_envelope), 0, 0, 1, 1},
  {&__pyx_n_s_envelope_list, __pyx_k_envelope_list, sizeof(__pyx_k_envelope_list), 0, 0, 1, 1},
  {&__pyx_n_s_envelope_obj, __pyx_k_envelope_obj, sizeof(__pyx_k_envelope_obj), 0, 0, 1, 1},
  {&__pyx_n_s_eq, __pyx_k_eq, sizeof(__pyx_k_eq), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_error_tolerance, __pyx_k_error_tolerance, sizeof(__pyx_k_error_tolerance), 0, 0, 1, 1},
  {&__pyx_n_s_fit, __pyx_k_fit, sizeof(__pyx_k_fit), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 0, 1, 1},
  {&__pyx_n_u_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 1, 0, 1},
  {&__pyx_n_s_from_peak, __pyx_k_from_peak, sizeof(__pyx_k_from_peak), 0, 0, 1, 1},
  {&__pyx_n_s_full_width_at_half_max, __pyx_k_full_width_at_half_max, sizeof(__pyx_k_full_width_at_half_max), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getitem___locals_genexpr, __pyx_k_getitem___locals_genexpr, sizeof(__pyx_k_getitem___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_k_got_differing_extents_in_dimensi, sizeof(__pyx_k_got_differing_extents_in_dimensi), 0, 0, 1, 0},
  {&__pyx_n_s_has_peak, __pyx_k_has_peak, sizeof(__pyx_k_has_peak), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_indexed, __pyx_k_indexed, sizeof(__pyx_k_indexed), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_init___locals_lambda, __pyx_k_init___locals_lambda, sizeof(__pyx_k_init___locals_lambda), 0, 0, 1, 1},
  {&__pyx_n_s_intensity, __pyx_k_intensity, sizeof(__pyx_k_intensity), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_kp_s_itemsize_0_for_cython_array, __pyx_k_itemsize_0_for_cython_array, sizeof(__pyx_k_itemsize_0_for_cython_array), 0, 0, 1, 0},
  {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_m1, __pyx_k_m1, sizeof(__pyx_k_m1), 0, 0, 1, 1},
  {&__pyx_n_s_m2, __pyx_k_m2, sizeof(__pyx_k_m2), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_s_memview, __pyx_k_memview, sizeof(__pyx_k_memview), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_most_abundant_mass, __pyx_k_most_abundant_mass, sizeof(__pyx_k_most_abundant_mass), 0, 0, 1, 1},
  {&__pyx_n_s_ms_deisotope__c_peak_set, __pyx_k_ms_deisotope__c_peak_set, sizeof(__pyx_k_ms_deisotope__c_peak_set), 0, 0, 1, 1},
  {&__pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_k_ms_deisotope__c_peak_set_pyx, sizeof(__pyx_k_ms_deisotope__c_peak_set_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_mz, __pyx_k_mz, sizeof(__pyx_k_mz), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_neutral_mass, __pyx_k_neutral_mass, sizeof(__pyx_k_neutral_mass), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_operator, __pyx_k_operator, sizeof(__pyx_k_operator), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_pairs, __pyx_k_pairs, sizeof(__pyx_k_pairs), 0, 0, 1, 1},
  {&__pyx_n_s_peak, __pyx_k_peak, sizeof(__pyx_k_peak), 0, 0, 1, 1},
  {&__pyx_n_s_peaks, __pyx_k_peaks, sizeof(__pyx_k_peaks), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_getbuffer, __pyx_k_pyx_getbuffer, sizeof(__pyx_k_pyx_getbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Enum, __pyx_k_pyx_unpickle_Enum, sizeof(__pyx_k_pyx_unpickle_Enum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reindex, __pyx_k_reindex, sizeof(__pyx_k_reindex), 0, 0, 1, 1},
  {&__pyx_n_s_release_buffers, __pyx_k_release_buffers, sizeof(__pyx_k_release_buffers), 0, 0, 1, 1},
  {&__pyx_n_s_repr___locals_genexpr, __pyx_k_repr___locals_genexpr, sizeof(__pyx_k_repr___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_kp_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 0},
  {&__pyx_n_s_score, __pyx_k_score, sizeof(__pyx_k_score), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self___class_____name___a_to_a2, __pyx_k_self___class_____name___a_to_a2, sizeof(__pyx_k_self___class_____name___a_to_a2), 0, 0, 1, 0},
  {&__pyx_kp_s_self___class_____name___a_to_a2_2, __pyx_k_self___class_____name___a_to_a2_2, sizeof(__pyx_k_self___class_____name___a_to_a2_2), 0, 0, 1, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_signal_to_noise, __pyx_k_signal_to_noise, sizeof(__pyx_k_signal_to_noise), 0, 0, 1, 1},
  {&__pyx_n_s_signatures, __pyx_k_signatures, sizeof(__pyx_k_signatures), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_solution, __pyx_k_solution, sizeof(__pyx_k_solution), 0, 0, 1, 1},
  {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_strided_and_direct, __pyx_k_strided_and_direct, sizeof(__pyx_k_strided_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_direct_or_indirect, __pyx_k_strided_and_direct_or_indirect, sizeof(__pyx_k_strided_and_direct_or_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_indirect, __pyx_k_strided_and_indirect, sizeof(__pyx_k_strided_and_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_tolerance, __pyx_k_tolerance, sizeof(__pyx_k_tolerance), 0, 0, 1, 1},
  {&__pyx_kp_s_unable_to_allocate_array_data, __pyx_k_unable_to_allocate_array_data, sizeof(__pyx_k_unable_to_allocate_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_unable_to_allocate_shape_and_str, __pyx_k_unable_to_allocate_shape_and_str, sizeof(__pyx_k_unable_to_allocate_shape_and_str), 0, 0, 1, 0},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_mz, __pyx_k_use_mz, sizeof(__pyx_k_use_mz), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 51, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 190, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_n_s_map); if (!__pyx_builtin_map) __PYX_ERR(0, 251, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 459, __pyx_L1_error)
  __pyx_builtin_sorted = __Pyx_GetBuiltinName(__pyx_n_s_sorted); if (!__pyx_builtin_sorted) __PYX_ERR(0, 572, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 607, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 884, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(3, 133, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(3, 148, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(3, 151, __pyx_L1_error)
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_n_s_Ellipsis); if (!__pyx_builtin_Ellipsis) __PYX_ERR(3, 404, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) __PYX_ERR(3, 613, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__5);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__5);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__6);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "ms_deisotope/_c/peak_set.pyx":1157
 * 
 *     def __setstate__(self, d):
 *         if d.get("indexed", False):             # <<<<<<<<<<<<<<
 *             self.reindex()
 * 
 */

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__7 = PyTuple_Pack(2, __pyx_n_s_indexed, Py_False); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 1157, __pyx_L1_error)

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__7);

#line 1157 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(2, 884, __pyx_L1_error)

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_tuple__8);

#line 884 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(2, 890, __pyx_L1_error)

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GOTREF(__pyx_tuple__9);

#line 890 "../../../../../private/var/folders/24/8k48jl6d249_n_qfxwsl6xvm0000gn/T/pip-build-env-2tukze1j/overlay/lib/python3.9/site-packages/numpy/__init__.pxd"
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "View.MemoryView":133
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */

#line 133 "View.MemoryView"
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_Empty_shape_tuple_for_cython_arr); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(3, 133, __pyx_L1_error)

#line 133 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__10);

#line 133 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "View.MemoryView":136
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */

#line 136 "View.MemoryView"
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_itemsize_0_for_cython_array); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(3, 136, __pyx_L1_error)

#line 136 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__11);

#line 136 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "View.MemoryView":148
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 148 "View.MemoryView"
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_shape_and_str); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(3, 148, __pyx_L1_error)

#line 148 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__12);

#line 148 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "View.MemoryView":176
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */

#line 176 "View.MemoryView"
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_array_data); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(3, 176, __pyx_L1_error)

#line 176 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__13);

#line 176 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "View.MemoryView":192
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */

#line 192 "View.MemoryView"
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Can_only_create_a_buffer_that_is); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(3, 192, __pyx_L1_error)

#line 192 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__14);

#line 192 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__15);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__16);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "View.MemoryView":418
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError("Cannot assign to read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 */

#line 418 "View.MemoryView"
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Cannot_assign_to_read_only_memor); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(3, 418, __pyx_L1_error)

#line 418 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__17);

#line 418 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "View.MemoryView":495
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */

#line 495 "View.MemoryView"
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_convert_item_to_object); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(3, 495, __pyx_L1_error)

#line 495 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__18);

#line 495 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "View.MemoryView":520
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError("Cannot create writable memory view from read-only memoryview")             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
 */

#line 520 "View.MemoryView"
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_Cannot_create_writable_memory_vi); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(3, 520, __pyx_L1_error)

#line 520 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__19);

#line 520 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "View.MemoryView":570
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */

#line 570 "View.MemoryView"
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_Buffer_view_does_not_expose_stri); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(3, 570, __pyx_L1_error)

#line 570 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__20);

#line 570 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "View.MemoryView":577
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */

#line 577 "View.MemoryView"
  __pyx_tuple__21 = PyTuple_New(1); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(3, 577, __pyx_L1_error)

#line 577 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__21);

#line 577 "View.MemoryView"
  __Pyx_INCREF(__pyx_int_neg_1);

#line 577 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_int_neg_1);

#line 577 "View.MemoryView"
  PyTuple_SET_ITEM(__pyx_tuple__21, 0, __pyx_int_neg_1);

#line 577 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__22);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__23);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "View.MemoryView":682
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */

#line 682 "View.MemoryView"
  __pyx_slice__24 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__24)) __PYX_ERR(3, 682, __pyx_L1_error)

#line 682 "View.MemoryView"
  __Pyx_GOTREF(__pyx_slice__24);

#line 682 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_slice__24);

  /* "View.MemoryView":703
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 703 "View.MemoryView"
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_Indirect_dimensions_not_supporte); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(3, 703, __pyx_L1_error)

#line 703 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__25);

#line 703 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

#line 2 "(tree fragment)"
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(3, 2, __pyx_L1_error)

#line 2 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__26);

#line 2 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */

#line 4 "(tree fragment)"
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__27);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__27);

#line 4 "(tree fragment)"
  __pyx_tuple__28 = PyTuple_Pack(3, __pyx_int_184977713, __pyx_int_136983863, __pyx_int_112105877); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(3, 4, __pyx_L1_error)

#line 4 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__28);

#line 4 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "ms_deisotope/_c/peak_set.pyx":41
 * cdef double epsilon = 1e-3
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 * mz_getter = operator.attrgetter('mz')
 * 
 */

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_neutral_mass); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 41, __pyx_L1_error)

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__29);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "ms_deisotope/_c/peak_set.pyx":42
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 * mz_getter = operator.attrgetter('mz')             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_n_s_mz); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__30);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 1095, __pyx_L1_error)

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__31);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__31);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_n_s_convert, 1095, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 1095, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_tuple__33 = PyTuple_Pack(9, __pyx_n_s_array, __pyx_n_s_envelope_list, __pyx_n_s_current_envelope, __pyx_n_s_converted, __pyx_n_s_envelope_obj, __pyx_n_s_i, __pyx_n_s_n, __pyx_n_s_a, __pyx_n_s_b); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_tuple__33);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_tuple__33);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(1, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ms_deisotope__c_peak_set_pyx, __pyx_n_s_decode_envelopes, 1106, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 1106, __pyx_L1_error)

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */

#line 286 "View.MemoryView"
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct_or_indirect); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(3, 286, __pyx_L1_error)

#line 286 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__35);

#line 286 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */

#line 287 "View.MemoryView"
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(3, 287, __pyx_L1_error)

#line 287 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__36);

#line 287 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 288 "View.MemoryView"
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_s_strided_and_indirect); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(3, 288, __pyx_L1_error)

#line 288 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__37);

#line 288 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */

#line 291 "View.MemoryView"
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_direct); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(3, 291, __pyx_L1_error)

#line 291 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__38);

#line 291 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 292 "View.MemoryView"
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_indirect); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(3, 292, __pyx_L1_error)

#line 292 "View.MemoryView"
  __Pyx_GOTREF(__pyx_tuple__39);

#line 292 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_tuple__40 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(3, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_tuple__40);

#line 1 "(tree fragment)"
  __Pyx_GIVEREF(__pyx_tuple__40);

#line 1 "(tree fragment)"
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Enum, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(3, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return 0;

#line 1 "(tree fragment)"
  __pyx_L1_error:;

#line 1 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 1 "(tree fragment)"
  return -1;

#line 1 "(tree fragment)"
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_1eneg_5 = PyFloat_FromDouble(1e-5); if (unlikely(!__pyx_float_1eneg_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_112105877 = PyInt_FromLong(112105877L); if (unlikely(!__pyx_int_112105877)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_136983863 = PyInt_FromLong(136983863L); if (unlikely(!__pyx_int_136983863)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_184977713 = PyInt_FromLong(184977713L); if (unlikely(!__pyx_int_184977713)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter = Py_None; Py_INCREF(Py_None);
  __pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter = Py_None; Py_INCREF(Py_None);
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_INTERVAL_INDEX_SIZE, (void *)&__pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("_binary_search_interval", (void (*)(void))__pyx_f_12ms_deisotope_2_c_8peak_set__binary_search_interval, "int (double *, double, double, size_t, size_t *, size_t *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index = &__pyx_vtable_12ms_deisotope_2_c_8peak_set__Index;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set__Index._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set__Index *(*)(size_t, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_6_Index__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set__Index.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Index, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set__Index) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set__Index = &__pyx_type_12ms_deisotope_2_c_8peak_set__Index;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_EnvelopePair._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*)(double, double))__pyx_f_12ms_deisotope_2_c_8peak_set_12EnvelopePair__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_EnvelopePair, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_EnvelopePair = &__pyx_type_12ms_deisotope_2_c_8peak_set_EnvelopePair;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.get_size = (size_t (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_get_size;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.getitem = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_EnvelopePair *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_getitem;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope.clone = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_Envelope._create = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *(*)(PyObject *))__pyx_f_12ms_deisotope_2_c_8peak_set_8Envelope__create;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Envelope, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_Envelope = &__pyx_type_12ms_deisotope_2_c_8peak_set_Envelope;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakBase", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakBase*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.__pyx_base = *__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakBase;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__eq;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak._create_simple = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(double, double, int, double, double, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_Envelope *))__pyx_f_12ms_deisotope_2_c_8peak_set_16DeconvolutedPeak__create_simple;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_base = __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakBase;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeak, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.__pyx_base._eq = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_27IonMobilityDeconvolutedPeak__eq;
  __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IonMobilityDeconvolutedPeak, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak) < 0) __PYX_ERR(0, 455, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak = &__pyx_type_12ms_deisotope_2_c_8peak_set_IonMobilityDeconvolutedPeak;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.__pyx_base.__pyx_base.clone = (struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *(*)(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakBase *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_24DeconvolutedPeakSolution_clone;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSolution, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.reindex = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_reindex;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.clone = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_clone;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.copy = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_copy;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet._has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.all_peaks_for = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet._get_nearest_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, double *))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__get_nearest_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.get_size = (size_t (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_get_size;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.getitem = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getitem;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.getslice = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, size_t, size_t))__pyx_f_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_getslice;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSet, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet) < 0) __PYX_ERR(0, 544, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = &__pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base = *__pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base.reindex = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, int __pyx_skip_dispatch))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_reindex;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base._has_peak = (struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeak *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet__has_peak *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__has_peak;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.__pyx_base.all_peaks_for = (PyObject *(*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_12ms_deisotope_2_c_8peak_set_19DeconvolutedPeakSet_all_peaks_for *__pyx_optional_args))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed_all_peaks_for;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed._build_index_arrays = (void (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__build_index_arrays;
  __pyx_vtable_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed._interval_for = (int (*)(struct __pyx_obj_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed *, double, double, size_t *, size_t *))__pyx_f_12ms_deisotope_2_c_8peak_set_26DeconvolutedPeakSetIndexed__interval_for;
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_base = __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSet;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed.tp_dict, __pyx_vtabptr_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DeconvolutedPeakSetIndexed, (PyObject *)&__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed) < 0) __PYX_ERR(0, 1139, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed = &__pyx_type_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSetIndexed;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 143, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct____iter__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__) < 0) __PYX_ERR(0, 259, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_1___repr__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr) < 0) __PYX_ERR(0, 260, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_2_genexpr;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__) < 0) __PYX_ERR(0, 529, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_3___iter__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__) < 0) __PYX_ERR(0, 599, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__ = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_4___getitem__;
  if (PyType_Ready(&__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr) < 0) __PYX_ERR(0, 601, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_dictoffset && __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr = &__pyx_type_12ms_deisotope_2_c_8peak_set___pyx_scope_struct_5_genexpr;
  __pyx_vtabptr_array = &__pyx_vtable_array;
  __pyx_vtable_array.get_memview = (PyObject *(*)(struct __pyx_array_obj *))__pyx_array_get_memview;
  if (PyType_Ready(&__pyx_type___pyx_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_array.tp_print = 0;
  #endif
  if (__Pyx_SetVtable(__pyx_type___pyx_array.tp_dict, __pyx_vtabptr_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_array) < 0) __PYX_ERR(3, 105, __pyx_L1_error)
  __pyx_array_type = &__pyx_type___pyx_array;
  if (PyType_Ready(&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(3, 279, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_MemviewEnum.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_MemviewEnum.tp_dictoffset && __pyx_type___pyx_MemviewEnum.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_MemviewEnum.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_MemviewEnum) < 0) __PYX_ERR(3, 279, __pyx_L1_error)
  __pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  if (PyType_Ready(&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryview.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryview.tp_dictoffset && __pyx_type___pyx_memoryview.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryview.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryview.tp_dict, __pyx_vtabptr_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryview) < 0) __PYX_ERR(3, 330, __pyx_L1_error)
  __pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_type___pyx_memoryviewslice.tp_base = __pyx_memoryview_type;
  if (PyType_Ready(&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type___pyx_memoryviewslice.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type___pyx_memoryviewslice.tp_dictoffset && __pyx_type___pyx_memoryviewslice.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type___pyx_memoryviewslice.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryviewslice.tp_dict, __pyx_vtabptr__memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type___pyx_memoryviewslice) < 0) __PYX_ERR(3, 965, __pyx_L1_error)
  __pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.composition"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7brainpy_2_c_11composition_PyComposition = __Pyx_ImportType(__pyx_t_1, "brainpy._c.composition", "PyComposition", sizeof(struct __pyx_obj_7brainpy_2_c_11composition_PyComposition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7brainpy_2_c_11composition_PyComposition) __PYX_ERR(4, 123, __pyx_L1_error)
  __pyx_vtabptr_7brainpy_2_c_11composition_PyComposition = (struct __pyx_vtabstruct_7brainpy_2_c_11composition_PyComposition*)__Pyx_GetVtable(__pyx_ptype_7brainpy_2_c_11composition_PyComposition->tp_dict); if (unlikely(!__pyx_vtabptr_7brainpy_2_c_11composition_PyComposition)) __PYX_ERR(4, 123, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.isotopic_distribution"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = __Pyx_ImportType(__pyx_t_1, "brainpy._c.isotopic_distribution", "TheoreticalPeak", sizeof(struct __pyx_obj_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak) __PYX_ERR(5, 63, __pyx_L1_error)
  __pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak = (struct __pyx_vtabstruct_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak*)__Pyx_GetVtable(__pyx_ptype_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak->tp_dict); if (unlikely(!__pyx_vtabptr_7brainpy_2_c_21isotopic_distribution_TheoreticalPeak)) __PYX_ERR(5, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "FittedPeak", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_FittedPeak), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak) __PYX_ERR(6, 12, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_FittedPeak*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_FittedPeak->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_FittedPeak)) __PYX_ERR(6, 12, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSet", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSet), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet) __PYX_ERR(6, 29, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSet*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSet->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSet)) __PYX_ERR(6, 29, __pyx_L1_error)
  __pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_set", "PeakSetIndexed", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed) __PYX_ERR(6, 83, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_8peak_set_PeakSetIndexed)) __PYX_ERR(6, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 199, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 222, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 226, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_index"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex = __Pyx_ImportType(__pyx_t_1, "ms_peak_picker._c.peak_index", "PeakIndex", sizeof(struct __pyx_obj_14ms_peak_picker_2_c_10peak_index_PeakIndex), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex) __PYX_ERR(8, 5, __pyx_L1_error)
  __pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex = (struct __pyx_vtabstruct_14ms_peak_picker_2_c_10peak_index_PeakIndex*)__Pyx_GetVtable(__pyx_ptype_14ms_peak_picker_2_c_10peak_index_PeakIndex->tp_dict); if (unlikely(!__pyx_vtabptr_14ms_peak_picker_2_c_10peak_index_PeakIndex)) __PYX_ERR(8, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "Averagine", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_Averagine), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine) __PYX_ERR(9, 12, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_Averagine*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_Averagine->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_Averagine)) __PYX_ERR(9, 12, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "TheoreticalIsotopicPattern", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern) __PYX_ERR(9, 23, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_TheoreticalIsotopicPattern)) __PYX_ERR(9, 23, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.averagine", "AveragineCache", sizeof(struct __pyx_obj_12ms_deisotope_2_c_9averagine_AveragineCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache) __PYX_ERR(9, 79, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache = (struct __pyx_vtabstruct_12ms_deisotope_2_c_9averagine_AveragineCache*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_9averagine_AveragineCache->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_9averagine_AveragineCache)) __PYX_ERR(9, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.deconvoluter_base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "DeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase) __PYX_ERR(10, 25, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_DeconvoluterBase)) __PYX_ERR(10, 25, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "AveragineDeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase) __PYX_ERR(10, 57, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_AveragineDeconvoluterBase)) __PYX_ERR(10, 57, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "MultiAveragineDeconvoluterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase) __PYX_ERR(10, 69, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_MultiAveragineDeconvoluterBase)) __PYX_ERR(10, 69, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.deconvoluter_base", "ChargeIterator", sizeof(struct __pyx_obj_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator) __PYX_ERR(10, 91, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator = (struct __pyx_vtabstruct_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_17deconvoluter_base_ChargeIterator)) __PYX_ERR(10, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.scoring"); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "IsotopicFitRecord", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitRecord), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitRecord*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitRecord->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitRecord)) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "FitSelectorBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FitSelectorBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FitSelectorBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_FitSelectorBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FitSelectorBase)) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "IsotopicFitterBase", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_IsotopicFitterBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase) __PYX_ERR(11, 50, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_IsotopicFitterBase*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_IsotopicFitterBase->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_IsotopicFitterBase)) __PYX_ERR(11, 50, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "LeastSquaresFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_LeastSquaresFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter) __PYX_ERR(11, 62, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_LeastSquaresFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_LeastSquaresFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_LeastSquaresFitter)) __PYX_ERR(11, 62, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "GTestFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_GTestFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter) __PYX_ERR(11, 66, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_GTestFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_GTestFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_GTestFitter)) __PYX_ERR(11, 66, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "ScaledGTestFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledGTestFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter) __PYX_ERR(11, 69, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledGTestFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledGTestFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledGTestFitter)) __PYX_ERR(11, 69, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "MSDeconVFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_MSDeconVFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter) __PYX_ERR(11, 72, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_MSDeconVFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_MSDeconVFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_MSDeconVFitter)) __PYX_ERR(11, 72, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "PenalizedMSDeconVFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter) __PYX_ERR(11, 78, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_PenalizedMSDeconVFitter)) __PYX_ERR(11, 78, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "FunctionScorer", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_FunctionScorer), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer) __PYX_ERR(11, 84, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_FunctionScorer*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_FunctionScorer->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_FunctionScorer)) __PYX_ERR(11, 84, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "InterferenceDetection", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_InterferenceDetection), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection) __PYX_ERR(11, 89, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_InterferenceDetection*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_InterferenceDetection->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_InterferenceDetection)) __PYX_ERR(11, 89, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "DistinctPatternFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_DistinctPatternFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter) __PYX_ERR(11, 95, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DistinctPatternFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_DistinctPatternFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DistinctPatternFitter)) __PYX_ERR(11, 95, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "ScaledPenalizedMSDeconvFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter) __PYX_ERR(11, 102, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_ScaledPenalizedMSDeconvFitter)) __PYX_ERR(11, 102, __pyx_L1_error)
  __pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter = __Pyx_ImportType(__pyx_t_1, "ms_deisotope._c.scoring", "DotProductFitter", sizeof(struct __pyx_obj_12ms_deisotope_2_c_7scoring_DotProductFitter), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter) __PYX_ERR(11, 112, __pyx_L1_error)
  __pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter = (struct __pyx_vtabstruct_12ms_deisotope_2_c_7scoring_DotProductFitter*)__Pyx_GetVtable(__pyx_ptype_12ms_deisotope_2_c_7scoring_DotProductFitter->tp_dict); if (unlikely(!__pyx_vtabptr_12ms_deisotope_2_c_7scoring_DotProductFitter)) __PYX_ERR(11, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.composition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "nist_mass", (void **)&__pyx_vp_7brainpy_2_c_11composition_nist_mass, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PROTON", (void **)&__pyx_vp_7brainpy_2_c_11composition_PROTON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_ElementTable", (void **)&__pyx_vp_7brainpy_2_c_11composition__ElementTable, "struct __pyx_t_7brainpy_2_c_11composition_ElementHashTable *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("brainpy._c.isotopic_constants"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "DEFAULT_ISOTOPIC_CONSTANTS_SIZE", (void **)&__pyx_vp_7brainpy_2_c_18isotopic_constants_DEFAULT_ISOTOPIC_CONSTANTS_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_peak_picker._c.peak_set"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INTERVAL_INDEX_SIZE", (void **)&__pyx_vp_14ms_peak_picker_2_c_8peak_set_INTERVAL_INDEX_SIZE, "size_t") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "PROTON", (void **)&__pyx_vp_12ms_deisotope_2_c_9averagine_PROTON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.scoring"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INFINITY", (void **)&__pyx_vp_12ms_deisotope_2_c_7scoring_INFINITY, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ms_deisotope._c.averagine"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "mass_charge_ratio", (void (**)(void))&__pyx_f_12ms_deisotope_2_c_9averagine_mass_charge_ratio, "double (double, int, struct __pyx_opt_args_12ms_deisotope_2_c_9averagine_mass_charge_ratio *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initpeak_set(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initpeak_set(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_peak_set(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_peak_set(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_peak_set(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  static PyThread_type_lock __pyx_t_5[8];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'peak_set' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_peak_set(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("peak_set", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ms_deisotope___c__peak_set) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ms_deisotope._c.peak_set")) {
      if (unlikely(PyDict_SetItemString(modules, "ms_deisotope._c.peak_set", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ms_deisotope/_c/peak_set.pyx":3
 * # cython: embedsignature=True
 * 
 * import operator             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */

#line 3 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_operator, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)

#line 3 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 3 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_operator, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)

#line 3 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":27
 * cimport numpy as np
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 27 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_2 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 27, __pyx_L1_error)

  /* "ms_deisotope/_c/peak_set.pyx":39
 *     object mz_getter
 * 
 * cdef double epsilon = 1e-3             # <<<<<<<<<<<<<<
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 */

#line 39 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_12ms_deisotope_2_c_8peak_set_epsilon = 1e-3;

  /* "ms_deisotope/_c/peak_set.pyx":41
 * cdef double epsilon = 1e-3
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")             # <<<<<<<<<<<<<<
 * mz_getter = operator.attrgetter('mz')
 * 
 */

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_operator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 41, __pyx_L1_error)

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGOTREF(__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF_SET(__pyx_v_12ms_deisotope_2_c_8peak_set_neutral_mass_getter, __pyx_t_1);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 41 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":42
 * 
 * neutral_mass_getter = operator.attrgetter("neutral_mass")
 * mz_getter = operator.attrgetter('mz')             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_operator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_attrgetter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_XGOTREF(__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF_SET(__pyx_v_12ms_deisotope_2_c_8peak_set_mz_getter, __pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_1);

#line 42 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":535
 * 
 *     @classmethod
 *     def from_peak(cls, peak, solution):             # <<<<<<<<<<<<<<
 *         inst = cls(
 *             solution, peak.fit, peak.neutral_mass, peak.intensity,
 */

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution, __pyx_n_s_from_peak); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)

#line 535 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

  /* "ms_deisotope/_c/peak_set.pyx":534
 *         yield self.fit
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_peak(cls, peak, solution):
 *         inst = cls(
 */

#line 534 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_Method_ClassMethod(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 534, __pyx_L1_error)

#line 534 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 534 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 534 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem((PyObject *)__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution->tp_dict, __pyx_n_s_from_peak, __pyx_t_3) < 0) __PYX_ERR(0, 535, __pyx_L1_error)

#line 534 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

#line 534 "ms_deisotope/_c/peak_set.pyx"
  PyType_Modified(__pyx_ptype_12ms_deisotope_2_c_8peak_set_DeconvolutedPeakSolution);

  /* "ms_deisotope/_c/peak_set.pyx":809
 * 
 * cdef double INF
 * INF = float('inf')             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 809 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_4 = __Pyx_PyObject_AsDouble(__pyx_n_s_inf); if (unlikely(__pyx_t_4 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 809, __pyx_L1_error)

#line 809 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_12ms_deisotope_2_c_8peak_set_INF = __pyx_t_4;

  /* "ms_deisotope/_c/peak_set.pyx":1095
 * 
 * 
 * def convert(self):             # <<<<<<<<<<<<<<
 *     return DeconvolutedPeak(
 *         self.neutral_mass, self.intensity, self.charge,
 */

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_13convert, NULL, __pyx_n_s_ms_deisotope__c_peak_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1095, __pyx_L1_error)

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convert, __pyx_t_3) < 0) __PYX_ERR(0, 1095, __pyx_L1_error)

#line 1095 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1106
 * @cython.boundscheck(False)
 * @cython.wraparound(False)
 * def decode_envelopes(np.ndarray[cython.floating, ndim=1, mode='c'] array):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list envelope_list, current_envelope
 */

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_3);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_0__pyx_mdef_12ms_deisotope_2_c_8peak_set_17decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_float, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_fuse_1__pyx_mdef_12ms_deisotope_2_c_8peak_set_19decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_double, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __pyx_FusedFunction_New(&__pyx_mdef_12ms_deisotope_2_c_8peak_set_15decode_envelopes, 0, __pyx_n_s_decode_envelopes, NULL, __pyx_n_s_ms_deisotope__c_peak_set, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_empty_tuple);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  ((__pyx_FusedFunctionObject *) __pyx_t_1)->__signatures__ = __pyx_t_3;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GIVEREF(__pyx_t_3);

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_3 = 0;

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_decode_envelopes, __pyx_t_1) < 0) __PYX_ERR(0, 1106, __pyx_L1_error)

#line 1106 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1137
 * 
 * 
 * INTERVAL_INDEX_SIZE = 0             # <<<<<<<<<<<<<<
 * 
 * cdef class DeconvolutedPeakSetIndexed(DeconvolutedPeakSet):
 */

#line 1137 "ms_deisotope/_c/peak_set.pyx"
  __pyx_v_12ms_deisotope_2_c_8peak_set_INTERVAL_INDEX_SIZE = 0;

  /* "ms_deisotope/_c/peak_set.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * 
 * import operator
 */

#line 1 "ms_deisotope/_c/peak_set.pyx"
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)

#line 1 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "ms_deisotope/_c/peak_set.pyx"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)

#line 1 "ms_deisotope/_c/peak_set.pyx"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":209
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
 */

#line 209 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_array_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 209, __pyx_L1_error)

#line 209 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 209 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_array_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 209, __pyx_L1_error)

#line 209 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 209 "View.MemoryView"
  PyType_Modified(__pyx_array_type);

  /* "View.MemoryView":286
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */

#line 286 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 286, __pyx_L1_error)

#line 286 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 286 "View.MemoryView"
  __Pyx_XGOTREF(generic);

#line 286 "View.MemoryView"
  __Pyx_DECREF_SET(generic, __pyx_t_1);

#line 286 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 286 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":287
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */

#line 287 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 287, __pyx_L1_error)

#line 287 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 287 "View.MemoryView"
  __Pyx_XGOTREF(strided);

#line 287 "View.MemoryView"
  __Pyx_DECREF_SET(strided, __pyx_t_1);

#line 287 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 287 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":288
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 288 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 288, __pyx_L1_error)

#line 288 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 288 "View.MemoryView"
  __Pyx_XGOTREF(indirect);

#line 288 "View.MemoryView"
  __Pyx_DECREF_SET(indirect, __pyx_t_1);

#line 288 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 288 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":291
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */

#line 291 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 291, __pyx_L1_error)

#line 291 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 291 "View.MemoryView"
  __Pyx_XGOTREF(contiguous);

#line 291 "View.MemoryView"
  __Pyx_DECREF_SET(contiguous, __pyx_t_1);

#line 291 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 291 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":292
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 292 "View.MemoryView"
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 292, __pyx_L1_error)

#line 292 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 292 "View.MemoryView"
  __Pyx_XGOTREF(indirect_contiguous);

#line 292 "View.MemoryView"
  __Pyx_DECREF_SET(indirect_contiguous, __pyx_t_1);

#line 292 "View.MemoryView"
  __Pyx_GIVEREF(__pyx_t_1);

#line 292 "View.MemoryView"
  __pyx_t_1 = 0;

  /* "View.MemoryView":316
 * 
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0             # <<<<<<<<<<<<<<
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [
 *     PyThread_allocate_lock(),
 */

#line 316 "View.MemoryView"
  __pyx_memoryview_thread_locks_used = 0;

  /* "View.MemoryView":317
 * DEF THREAD_LOCKS_PREALLOCATED = 8
 * cdef int __pyx_memoryview_thread_locks_used = 0
 * cdef PyThread_type_lock[THREAD_LOCKS_PREALLOCATED] __pyx_memoryview_thread_locks = [             # <<<<<<<<<<<<<<
 *     PyThread_allocate_lock(),
 *     PyThread_allocate_lock(),
 */

#line 317 "View.MemoryView"
  __pyx_t_5[0] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[1] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[2] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[3] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[4] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[5] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[6] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  __pyx_t_5[7] = PyThread_allocate_lock();

#line 317 "View.MemoryView"
  memcpy(&(__pyx_memoryview_thread_locks[0]), __pyx_t_5, sizeof(__pyx_memoryview_thread_locks[0]) * (8));

  /* "View.MemoryView":549
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 549 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 549, __pyx_L1_error)

#line 549 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 549 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_memoryview_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 549, __pyx_L1_error)

#line 549 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 549 "View.MemoryView"
  PyType_Modified(__pyx_memoryview_type);

  /* "View.MemoryView":995
 *         return self.from_object
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */

#line 995 "View.MemoryView"
  __pyx_t_1 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), ((char *)"getbuffer(obj, view, flags)")); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 995, __pyx_L1_error)

#line 995 "View.MemoryView"
  __Pyx_GOTREF(__pyx_t_1);

#line 995 "View.MemoryView"
  if (PyDict_SetItem((PyObject *)__pyx_memoryviewslice_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_1) < 0) __PYX_ERR(3, 995, __pyx_L1_error)

#line 995 "View.MemoryView"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

#line 995 "View.MemoryView"
  PyType_Modified(__pyx_memoryviewslice_type);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

#line 1 "(tree fragment)"
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum, NULL, __pyx_n_s_View_MemoryView); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_GOTREF(__pyx_t_1);

#line 1 "(tree fragment)"
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Enum, __pyx_t_1) < 0) __PYX_ERR(3, 1, __pyx_L1_error)

#line 1 "(tree fragment)"
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  /*--- Wrapped vars code ---*/

#line 11 "(tree fragment)"


#line 11 "(tree fragment)"
  goto __pyx_L0;

#line 11 "(tree fragment)"
  __pyx_L1_error:;

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_1);

#line 11 "(tree fragment)"
  __Pyx_XDECREF(__pyx_t_3);

#line 11 "(tree fragment)"
  if (__pyx_m) {

#line 11 "(tree fragment)"
    if (__pyx_d) {

#line 11 "(tree fragment)"
      __Pyx_AddTraceback("init ms_deisotope._c.peak_set", __pyx_clineno, __pyx_lineno, __pyx_filename);

#line 11 "(tree fragment)"
    }

#line 11 "(tree fragment)"
    Py_CLEAR(__pyx_m);

#line 11 "(tree fragment)"
  } else if (!PyErr_Occurred()) {

#line 11 "(tree fragment)"
    PyErr_SetString(PyExc_ImportError, "init ms_deisotope._c.peak_set");

#line 11 "(tree fragment)"
  }

#line 11 "(tree fragment)"
  __pyx_L0:;

#line 11 "(tree fragment)"
  __Pyx_RefNannyFinishContext();

#line 11 "(tree fragment)"
  #if CYTHON_PEP489_MULTI_PHASE_INIT

#line 11 "(tree fragment)"
  return (__pyx_m != NULL) ? 0 : -1;

#line 11 "(tree fragment)"
  #elif PY_MAJOR_VERSION >= 3

#line 11 "(tree fragment)"
  return __pyx_m;

#line 11 "(tree fragment)"
  #else

#line 11 "(tree fragment)"
  return;

#line 11 "(tree fragment)"
  #endif

#line 11 "(tree fragment)"
}

#line 1 "cython_utility"

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* None */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* StringJoin */
#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* py_abs */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* pyobject_as_double */
static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    PyObject* float_value;
#if !CYTHON_USE_TYPE_SLOTS
    float_value = PyNumber_Float(obj);  if ((0)) goto bad;
#else
    PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
    if (likely(nb) && likely(nb->nb_float)) {
        float_value = nb->nb_float(obj);
        if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
            PyErr_Format(PyExc_TypeError,
                "__float__ returned non-float (type %.200s)",
                Py_TYPE(float_value)->tp_name);
            Py_DECREF(float_value);
            goto bad;
        }
    } else if (PyUnicode_CheckExact(obj) || PyBytes_CheckExact(obj)) {
#if PY_MAJOR_VERSION >= 3
        float_value = PyFloat_FromString(obj);
#else
        float_value = PyFloat_FromString(obj, 0);
#endif
    } else {
        PyObject* args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        PyTuple_SET_ITEM(args, 0, obj);
        float_value = PyObject_Call((PyObject*)&PyFloat_Type, args, 0);
        PyTuple_SET_ITEM(args, 0, 0);
        Py_DECREF(args);
    }
#endif
    if (likely(float_value)) {
        double value = PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
bad:
    return (double)-1;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", Py_TYPE(c)->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* BytesEquals */
  static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
  static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* DivInt[Py_ssize_t] */
  static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* GetAttr */
  static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* decode_c_string */
  static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* GetAttr3 */
  static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* SwapException */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* Import */
  static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* None */
  static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* DivInt[long] */
  static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* ImportFrom */
  static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
  static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
  static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
  static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
  static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
  #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
  static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
  static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObjectGetAttrStrNoError */
  static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* ClassMethod */
  static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYSTON || CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    #if PY_MAJOR_VERSION == 2
    static PyTypeObject *methoddescr_type = NULL;
    if (methoddescr_type == NULL) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (!meth) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    #else
    PyTypeObject *methoddescr_type = &PyMethodDescr_Type;
    #endif
    if (__Pyx_TypeCheck(method, methoddescr_type))
#endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else {
        return PyClassMethod_New(method);
    }
}

/* GetNameInClass */
  static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    PyObject *result;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* FusedFunction */
  static PyObject *
__pyx_FusedFunction_New(PyMethodDef *ml, int flags,
                        PyObject *qualname, PyObject *closure,
                        PyObject *module, PyObject *globals,
                        PyObject *code)
{
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_FusedFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        __pyx_FusedFunctionObject *fusedfunc = (__pyx_FusedFunctionObject *) op;
        fusedfunc->__signatures__ = NULL;
        fusedfunc->type = NULL;
        fusedfunc->self = NULL;
        PyObject_GC_Track(op);
    }
    return op;
}
static void
__pyx_FusedFunction_dealloc(__pyx_FusedFunctionObject *self)
{
    PyObject_GC_UnTrack(self);
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    __Pyx__CyFunction_dealloc((__pyx_CyFunctionObject *) self);
}
static int
__pyx_FusedFunction_traverse(__pyx_FusedFunctionObject *self,
                             visitproc visit,
                             void *arg)
{
    Py_VISIT(self->self);
    Py_VISIT(self->type);
    Py_VISIT(self->__signatures__);
    return __Pyx_CyFunction_traverse((__pyx_CyFunctionObject *) self, visit, arg);
}
static int
__pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self)
{
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    return __Pyx_CyFunction_clear((__pyx_CyFunctionObject *) self);
}
static PyObject *
__pyx_FusedFunction_descr_get(PyObject *self, PyObject *obj, PyObject *type)
{
    __pyx_FusedFunctionObject *func, *meth;
    func = (__pyx_FusedFunctionObject *) self;
    if (func->self || func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(self);
        return self;
    }
    if (obj == Py_None)
        obj = NULL;
    meth = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_New(
                    ((PyCFunctionObject *) func)->m_ml,
                    ((__pyx_CyFunctionObject *) func)->flags,
                    ((__pyx_CyFunctionObject *) func)->func_qualname,
                    ((__pyx_CyFunctionObject *) func)->func_closure,
                    ((PyCFunctionObject *) func)->m_module,
                    ((__pyx_CyFunctionObject *) func)->func_globals,
                    ((__pyx_CyFunctionObject *) func)->func_code);
    if (!meth)
        return NULL;
    if (func->func.defaults) {
        PyObject **pydefaults;
        int i;
        if (!__Pyx_CyFunction_InitDefaults((PyObject*)meth,
                                      func->func.defaults_size,
                                      func->func.defaults_pyobjects)) {
            Py_XDECREF((PyObject*)meth);
            return NULL;
        }
        memcpy(meth->func.defaults, func->func.defaults, func->func.defaults_size);
        pydefaults = __Pyx_CyFunction_Defaults(PyObject *, meth);
        for (i = 0; i < meth->func.defaults_pyobjects; i++)
            Py_XINCREF(pydefaults[i]);
    }
    Py_XINCREF(func->func.func_classobj);
    meth->func.func_classobj = func->func.func_classobj;
    Py_XINCREF(func->__signatures__);
    meth->__signatures__ = func->__signatures__;
    Py_XINCREF(type);
    meth->type = type;
    Py_XINCREF(func->func.defaults_tuple);
    meth->func.defaults_tuple = func->func.defaults_tuple;
    if (func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD)
        obj = type;
    Py_XINCREF(obj);
    meth->self = obj;
    return (PyObject *) meth;
}
static PyObject *
_obj_to_str(PyObject *obj)
{
    if (PyType_Check(obj))
        return PyObject_GetAttr(obj, __pyx_n_s_name_2);
    else
        return PyObject_Str(obj);
}
static PyObject *
__pyx_FusedFunction_getitem(__pyx_FusedFunctionObject *self, PyObject *idx)
{
    PyObject *signature = NULL;
    PyObject *unbound_result_func;
    PyObject *result_func = NULL;
    if (self->__signatures__ == NULL) {
        PyErr_SetString(PyExc_TypeError, "Function is not fused");
        return NULL;
    }
    if (PyTuple_Check(idx)) {
        PyObject *list = PyList_New(0);
        Py_ssize_t n = PyTuple_GET_SIZE(idx);
        PyObject *sep = NULL;
        int i;
        if (unlikely(!list))
            return NULL;
        for (i = 0; i < n; i++) {
            int ret;
            PyObject *string;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(idx, i);
#else
            PyObject *item = PySequence_ITEM(idx, i);  if (unlikely(!item)) goto __pyx_err;
#endif
            string = _obj_to_str(item);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
            Py_DECREF(item);
#endif
            if (unlikely(!string)) goto __pyx_err;
            ret = PyList_Append(list, string);
            Py_DECREF(string);
            if (unlikely(ret < 0)) goto __pyx_err;
        }
        sep = PyUnicode_FromString("|");
        if (likely(sep))
            signature = PyUnicode_Join(sep, list);
__pyx_err:
;
        Py_DECREF(list);
        Py_XDECREF(sep);
    } else {
        signature = _obj_to_str(idx);
    }
    if (!signature)
        return NULL;
    unbound_result_func = PyObject_GetItem(self->__signatures__, signature);
    if (unbound_result_func) {
        if (self->self || self->type) {
            __pyx_FusedFunctionObject *unbound = (__pyx_FusedFunctionObject *) unbound_result_func;
            Py_CLEAR(unbound->func.func_classobj);
            Py_XINCREF(self->func.func_classobj);
            unbound->func.func_classobj = self->func.func_classobj;
            result_func = __pyx_FusedFunction_descr_get(unbound_result_func,
                                                        self->self, self->type);
        } else {
            result_func = unbound_result_func;
            Py_INCREF(result_func);
        }
    }
    Py_DECREF(signature);
    Py_XDECREF(unbound_result_func);
    return result_func;
}
static PyObject *
__pyx_FusedFunction_callfunction(PyObject *func, PyObject *args, PyObject *kw)
{
     __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    int static_specialized = (cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD &&
                              !((__pyx_FusedFunctionObject *) func)->__signatures__);
    if (cyfunc->flags & __Pyx_CYFUNCTION_CCLASS && !static_specialized) {
        return __Pyx_CyFunction_CallAsMethod(func, args, kw);
    } else {
        return __Pyx_CyFunction_Call(func, args, kw);
    }
}
static PyObject *
__pyx_FusedFunction_call(PyObject *func, PyObject *args, PyObject *kw)
{
    __pyx_FusedFunctionObject *binding_func = (__pyx_FusedFunctionObject *) func;
    Py_ssize_t argc = PyTuple_GET_SIZE(args);
    PyObject *new_args = NULL;
    __pyx_FusedFunctionObject *new_func = NULL;
    PyObject *result = NULL;
    PyObject *self = NULL;
    int is_staticmethod = binding_func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD;
    int is_classmethod = binding_func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD;
    if (binding_func->self) {
        Py_ssize_t i;
        new_args = PyTuple_New(argc + 1);
        if (!new_args)
            return NULL;
        self = binding_func->self;
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_INCREF(self);
#endif
        Py_INCREF(self);
        PyTuple_SET_ITEM(new_args, 0, self);
        for (i = 0; i < argc; i++) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            PyObject *item = PyTuple_GET_ITEM(args, i);
            Py_INCREF(item);
#else
            PyObject *item = PySequence_ITEM(args, i);  if (unlikely(!item)) goto bad;
#endif
            PyTuple_SET_ITEM(new_args, i + 1, item);
        }
        args = new_args;
    } else if (binding_func->type) {
        if (argc < 1) {
            PyErr_SetString(PyExc_TypeError, "Need at least one argument, 0 given.");
            return NULL;
        }
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        self = PyTuple_GET_ITEM(args, 0);
#else
        self = PySequence_ITEM(args, 0);  if (unlikely(!self)) return NULL;
#endif
    }
    if (self && !is_classmethod && !is_staticmethod) {
        int is_instance = PyObject_IsInstance(self, binding_func->type);
        if (unlikely(!is_instance)) {
            PyErr_Format(PyExc_TypeError,
                         "First argument should be of type %.200s, got %.200s.",
                         ((PyTypeObject *) binding_func->type)->tp_name,
                         Py_TYPE(self)->tp_name);
            goto bad;
        } else if (unlikely(is_instance == -1)) {
            goto bad;
        }
    }
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
    self = NULL;
#endif
    if (binding_func->__signatures__) {
        PyObject *tup;
        if (is_staticmethod && binding_func->func.flags & __Pyx_CYFUNCTION_CCLASS) {
            tup = PyTuple_Pack(3, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __Pyx_CyFunction_CallMethod(
                func, binding_func->__signatures__, tup, NULL);
        } else {
            tup = PyTuple_Pack(4, binding_func->__signatures__, args,
                               kw == NULL ? Py_None : kw,
                               binding_func->func.defaults_tuple);
            if (unlikely(!tup)) goto bad;
            new_func = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_callfunction(func, tup, NULL);
        }
        Py_DECREF(tup);
        if (unlikely(!new_func))
            goto bad;
        Py_XINCREF(binding_func->func.func_classobj);
        Py_CLEAR(new_func->func.func_classobj);
        new_func->func.func_classobj = binding_func->func.func_classobj;
        func = (PyObject *) new_func;
    }
    result = __pyx_FusedFunction_callfunction(func, args, kw);
bad:
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
    Py_XDECREF(self);
#endif
    Py_XDECREF(new_args);
    Py_XDECREF((PyObject *) new_func);
    return result;
}
static PyMemberDef __pyx_FusedFunction_members[] = {
    {(char *) "__signatures__",
     T_OBJECT,
     offsetof(__pyx_FusedFunctionObject, __signatures__),
     READONLY,
     0},
    {0, 0, 0, 0, 0},
};
static PyMappingMethods __pyx_FusedFunction_mapping_methods = {
    0,
    (binaryfunc) __pyx_FusedFunction_getitem,
    0,
};
static PyTypeObject __pyx_FusedFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "fused_cython_function",
    sizeof(__pyx_FusedFunctionObject),
    0,
    (destructor) __pyx_FusedFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    0,
    0,
    0,
    &__pyx_FusedFunction_mapping_methods,
    0,
    (ternaryfunc) __pyx_FusedFunction_call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __pyx_FusedFunction_traverse,
    (inquiry) __pyx_FusedFunction_clear,
    0,
    0,
    0,
    0,
    0,
    __pyx_FusedFunction_members,
    __pyx_CyFunction_getsets,
    &__pyx_CyFunctionType_type,
    0,
    __pyx_FusedFunction_descr_get,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_FusedFunction_init(void) {
    __pyx_FusedFunctionType_type.tp_base = __pyx_CyFunctionType;
    __pyx_FusedFunctionType = __Pyx_FetchCommonType(&__pyx_FusedFunctionType_type);
    if (__pyx_FusedFunctionType == NULL) {
        return -1;
    }
    return 0;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_array_type)) return __pyx_array_getbuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_memoryview_type)) return __pyx_memoryview_getbuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


  /* MemviewSliceIsContig */
  static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs.memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs.suboffsets[index] >= 0 || mvs.strides[index] != itemsize)
            return 0;
        itemsize *= mvs.shape[index];
    }
    return 1;
}

/* OverlappingSlices */
  static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

/* Capsule */
  static CYTHON_INLINE PyObject *
__pyx_capsule_create(void *p, CYTHON_UNUSED const char *sig)
{
    PyObject *cobj;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, NULL);
#else
    cobj = PyCObject_FromVoidPtr(p, NULL);
#endif
    return cobj;
}

/* TypeInfoCompare */
  static int
__pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b)
{
    int i;
    if (!a || !b)
        return 0;
    if (a == b)
        return 1;
    if (a->size != b->size || a->typegroup != b->typegroup ||
            a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
        if (a->typegroup == 'H' || b->typegroup == 'H') {
            return a->size == b->size;
        } else {
            return 0;
        }
    }
    if (a->ndim) {
        for (i = 0; i < a->ndim; i++)
            if (a->arraysize[i] != b->arraysize[i])
                return 0;
    }
    if (a->typegroup == 'S') {
        if (a->flags != b->flags)
            return 0;
        if (a->fields || b->fields) {
            if (!(a->fields && b->fields))
                return 0;
            for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                __Pyx_StructField *field_a = a->fields + i;
                __Pyx_StructField *field_b = b->fields + i;
                if (field_a->offset != field_b->offset ||
                    !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                    return 0;
            }
            return !a->fields[i].type && !b->fields[i].type;
        }
    }
    return 1;
}

/* MemviewSliceValidateAndInit */
  static int
__pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
{
    if (buf->shape[dim] <= 1)
        return 1;
    if (buf->strides) {
        if (spec & __Pyx_MEMVIEW_CONTIG) {
            if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                if (unlikely(buf->strides[dim] != sizeof(void *))) {
                    PyErr_Format(PyExc_ValueError,
                                 "Buffer is not indirectly contiguous "
                                 "in dimension %d.", dim);
                    goto fail;
                }
            } else if (unlikely(buf->strides[dim] != buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
        if (spec & __Pyx_MEMVIEW_FOLLOW) {
            Py_ssize_t stride = buf->strides[dim];
            if (stride < 0)
                stride = -stride;
            if (unlikely(stride < buf->itemsize)) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
    } else {
        if (unlikely(spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1)) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not contiguous in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(spec & (__Pyx_MEMVIEW_PTR))) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not indirect in "
                         "dimension %d", dim);
            goto fail;
        } else if (unlikely(buf->suboffsets)) {
            PyErr_SetString(PyExc_ValueError,
                            "Buffer exposes suboffsets but no strides");
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec)
{
    if (spec & __Pyx_MEMVIEW_DIRECT) {
        if (unlikely(buf->suboffsets && buf->suboffsets[dim] >= 0)) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer not compatible with direct access "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    if (spec & __Pyx_MEMVIEW_PTR) {
        if (unlikely(!buf->suboffsets || (buf->suboffsets[dim] < 0))) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer is not indirectly accessible "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
{
    int i;
    if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = 0; i < ndim; i++) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not fortran contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = ndim - 1; i >- 1; i--) {
            if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not C contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    }
    return 1;
fail:
    return 0;
}
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj)
{
    struct __pyx_memoryview_obj *memview, *new_memview;
    __Pyx_RefNannyDeclarations
    Py_buffer *buf;
    int i, spec = 0, retval = -1;
    __Pyx_BufFmt_Context ctx;
    int from_memoryview = __pyx_memoryview_check(original_obj);
    __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
    if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                            original_obj)->typeinfo)) {
        memview = (struct __pyx_memoryview_obj *) original_obj;
        new_memview = NULL;
    } else {
        memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                            original_obj, buf_flags, 0, dtype);
        new_memview = memview;
        if (unlikely(!memview))
            goto fail;
    }
    buf = &memview->view;
    if (unlikely(buf->ndim != ndim)) {
        PyErr_Format(PyExc_ValueError,
                "Buffer has wrong number of dimensions (expected %d, got %d)",
                ndim, buf->ndim);
        goto fail;
    }
    if (new_memview) {
        __Pyx_BufFmt_Init(&ctx, stack, dtype);
        if (unlikely(!__Pyx_BufFmt_CheckString(&ctx, buf->format))) goto fail;
    }
    if (unlikely((unsigned) buf->itemsize != dtype->size)) {
        PyErr_Format(PyExc_ValueError,
                     "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                     "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                     buf->itemsize,
                     (buf->itemsize > 1) ? "s" : "",
                     dtype->name,
                     dtype->size,
                     (dtype->size > 1) ? "s" : "");
        goto fail;
    }
    if (buf->len > 0) {
        for (i = 0; i < ndim; i++) {
            spec = axes_specs[i];
            if (unlikely(!__pyx_check_strides(buf, i, ndim, spec)))
                goto fail;
            if (unlikely(!__pyx_check_suboffsets(buf, i, ndim, spec)))
                goto fail;
        }
        if (unlikely(buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag)))
            goto fail;
    }
    if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                         new_memview != NULL) == -1)) {
        goto fail;
    }
    retval = 0;
    goto no_fail;
fail:
    Py_XDECREF(new_memview);
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *obj, int writable_flag) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS_RO | writable_flag, 1,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static PyObject* __pyx_convert__to_py_struct____pyx_t_12ms_deisotope_2_c_8peak_set_index_cell(struct __pyx_t_12ms_deisotope_2_c_8peak_set_index_cell s) {
    PyObject* res;
    PyObject* member;
    res = __Pyx_PyDict_NewPresized(3); if (unlikely(!res)) return NULL;
    member = PyFloat_FromDouble(s.center_value); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_center_value, member) < 0)) goto bad;
    Py_DECREF(member);
    member = __Pyx_PyInt_FromSize_t(s.start); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_start, member) < 0)) goto bad;
    Py_DECREF(member);
    member = __Pyx_PyInt_FromSize_t(s.end); if (unlikely(!member)) goto bad;
    if (unlikely(PyDict_SetItem(res, __pyx_n_s_end, member) < 0)) goto bad;
    Py_DECREF(member);
    return res;
    bad:
    Py_XDECREF(member);
    Py_DECREF(res);
    return NULL;
  }
  /* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
  #if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
  #if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* MemviewSliceCopyTemplate */
  static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object)
{
    __Pyx_RefNannyDeclarations
    int i;
    __Pyx_memviewslice new_mvs = { 0, 0, { 0 }, { 0 }, { 0 } };
    struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
    Py_buffer *buf = &from_memview->view;
    PyObject *shape_tuple = NULL;
    PyObject *temp_int = NULL;
    struct __pyx_array_obj *array_obj = NULL;
    struct __pyx_memoryview_obj *memview_obj = NULL;
    __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
    for (i = 0; i < ndim; i++) {
        if (unlikely(from_mvs->suboffsets[i] >= 0)) {
            PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                           "indirect dimensions (axis %d)", i);
            goto fail;
        }
    }
    shape_tuple = PyTuple_New(ndim);
    if (unlikely(!shape_tuple)) {
        goto fail;
    }
    __Pyx_GOTREF(shape_tuple);
    for(i = 0; i < ndim; i++) {
        temp_int = PyInt_FromSsize_t(from_mvs->shape[i]);
        if(unlikely(!temp_int)) {
            goto fail;
        } else {
            PyTuple_SET_ITEM(shape_tuple, i, temp_int);
            temp_int = NULL;
        }
    }
    array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, (char *) mode, NULL);
    if (unlikely(!array_obj)) {
        goto fail;
    }
    __Pyx_GOTREF(array_obj);
    memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                    (PyObject *) array_obj, contig_flag,
                                    dtype_is_object,
                                    from_mvs->memview->typeinfo);
    if (unlikely(!memview_obj))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
        goto fail;
    if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                dtype_is_object) < 0))
        goto fail;
    goto no_fail;
fail:
    __Pyx_XDECREF(new_mvs.memview);
    new_mvs.memview = NULL;
    new_mvs.data = NULL;
no_fail:
    __Pyx_XDECREF(shape_tuple);
    __Pyx_XDECREF(temp_int);
    __Pyx_XDECREF(array_obj);
    __Pyx_RefNannyFinishContext();
    return new_mvs;
}

/* MemviewSliceInit */
  static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (unlikely(memviewslice->memview || memviewslice->data)) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF(memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}
#ifndef Py_NO_RETURN
#define Py_NO_RETURN
#endif
static void __pyx_fatalerror(const char *fmt, ...) Py_NO_RETURN {
    va_list vargs;
    char msg[200];
#ifdef HAVE_STDARG_PROTOTYPES
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    va_end(vargs);
    Py_FatalError(msg);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    int first_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None))
        return;
    if (unlikely(__pyx_get_slice_count(memview) < 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    first_time = __pyx_add_acquisition_count(memview) == 0;
    if (unlikely(first_time)) {
        if (have_gil) {
            Py_INCREF((PyObject *) memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_INCREF((PyObject *) memview);
            PyGILState_Release(_gilstate);
        }
    }
}
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    int last_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None)) {
        memslice->memview = NULL;
        return;
    }
    if (unlikely(__pyx_get_slice_count(memview) <= 0))
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    last_time = __pyx_sub_acquisition_count(memview) == 1;
    memslice->data = NULL;
    if (unlikely(last_time)) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        memslice->memview = NULL;
    }
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint32(npy_uint32 value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_uint32 neg_one = (npy_uint32) -1, const_zero = (npy_uint32) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_uint32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_uint32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_uint32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_uint32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_uint32),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* BytesContains */
  static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character) {
    const Py_ssize_t length = PyBytes_GET_SIZE(bytes);
    char* char_start = PyBytes_AS_STRING(bytes);
    return memchr(char_start, (unsigned char)character, (size_t)length) != NULL;
}

/* ImportNumPyArray */
  static PyObject* __Pyx__ImportNumPyArray(void) {
    PyObject *numpy_module, *ndarray_object = NULL;
    numpy_module = __Pyx_Import(__pyx_n_s_numpy, NULL, 0);
    if (likely(numpy_module)) {
        ndarray_object = PyObject_GetAttrString(numpy_module, "ndarray");
        Py_DECREF(numpy_module);
    }
    if (unlikely(!ndarray_object)) {
        PyErr_Clear();
    }
    if (unlikely(!ndarray_object || !PyObject_TypeCheck(ndarray_object, &PyType_Type))) {
        Py_XDECREF(ndarray_object);
        Py_INCREF(Py_None);
        ndarray_object = Py_None;
    }
    return ndarray_object;
}
static CYTHON_INLINE PyObject* __Pyx_ImportNumPyArrayTypeIfAvailable(void) {
    if (unlikely(!__pyx_numpy_ndarray)) {
        __pyx_numpy_ndarray = __Pyx__ImportNumPyArray();
    }
    Py_INCREF(__pyx_numpy_ndarray);
    return __pyx_numpy_ndarray;
}

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const char neg_one = (char) -1, const_zero = (char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, digits[0])
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) ((((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) ((((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) ((((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

/* PyObjectCallMethod1 */
  static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
  #include <structmember.h>
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
  static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
  #ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
  static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrExport */
  static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* FunctionExport */
  static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
  #ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
  #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
